# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2015 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

import database
from Utils              import *


class CEventTypeDescription(object):
    cache = {}

    def __init__(self, db, eventTypeId):
        tableEventType   = db.table('EventType')
        tableFinance = db.table('rbFinance')
        table = tableEventType.leftJoin(tableFinance, tableFinance['id'].eq(tableEventType['finance_id']))
        eventType = db.getRecordEx(table, 'EventType.code as code, period, singleInPeriod, isLong, dateInput, form, minDuration, maxDuration, rbFinance.code as financeCode, showTime', tableEventType['id'].eq(eventTypeId))
        if eventType:
            self.code = forceString(eventType.value('code'))
            self.period = max(0, forceInt(eventType.value('period')))
            self.singleInPeriod = min(5, max(0, forceInt(eventType.value('singleInPeriod'))))
            self.isLong = forceBool(eventType.value('isLong'))
            self.dateInput = forceInt(eventType.value('dateInput'))
            self.isUrgent = self.code == '22'
            self.isDeath  = self.code == '15'
            self.form = forceString(eventType.value('form'))
            self.financeCode = forceInt(eventType.value('financeCode'))
            self.duration = (forceInt(eventType.value('minDuration')), forceInt(eventType.value('maxDuration')))
            self.showTime = forceBool(eventType.value('showTime'))
        else:
            self.period = None
            self.singleInPeriod = None
            self.isLong = None
            self.dateInput = None
            self.isUrgent = None
            self.isDeath = None
            self.form = ''
            self.financeCode = 0
            self.duration = (0, 0)
            self.showTime = False


    @staticmethod
    def get(eventTypeId):
        result = CEventTypeDescription.cache.get(eventTypeId, None)
        if not result:
            if not CEventTypeDescription.cache:
                QObject.connect(QtGui.qApp, QtCore.SIGNAL('dbConnectionChanged(bool)'), CEventTypeDescription.onConnectionChanged)
            result = CEventTypeDescription(eventTypeId)
            CEventTypeDescription.cache[eventTypeId] = result
        return result


    @staticmethod
    def onConnectionChanged(value):
        if value:
            CEventTypeDescription.cache = {}
            QObject.disconnect(QtGui.qApp, QtCore.SIGNAL('dbConnectionChanged(bool)'), CEventTypeDescription.onConnectionChanged)


def isEventDeath(eventTypeId):
    description = CEventTypeDescription.get(eventTypeId)
    return description.isDeath


def isEventPeriodic(eventTypeId):
    description = CEventTypeDescription.get(eventTypeId)
    return description.period or description.singleInPeriod


def isEventLong(eventTypeId):
    description = CEventTypeDescription.get(eventTypeId)
    return description.isLong


def getEventDateInput(eventTypeId):
    description = CEventTypeDescription.get(eventTypeId)
    return description.dateInput


def getEventDateInput(eventTypeId):
    description = CEventTypeDescription.get(eventTypeId)
    return description.dateInput


def isEventUrgent(eventTypeId):
    description = CEventTypeDescription.get(eventTypeId)
    return description.isUrgent


def getEventTypeForm(eventTypeId):
    description = CEventTypeDescription.get(eventTypeId)
    return description.form


def getEventFinanceCode(eventTypeId):
    description = CEventTypeDescription.get(eventTypeId)
    return description.financeCode


def getEventDuration(eventTypeId):
    description = CEventTypeDescription.get(eventTypeId)
    return description.duration


def getEventShowTime(eventTypeId):
    description = CEventTypeDescription.get(eventTypeId)
    return description.showTime


def checkEventPosibilityLetal(db, clientId, eventTypeId, eventDate):
    table = db.table('ClientAttach')
    cond=[table['client_id'].eq(clientId), table['attachType_id'].eq(8), table['begDate'].lt(eventDate)]
    if db.getRecordEx(table, '*', where=cond):
        table = db.table('EventType')
        cond=[table['id'].eq(eventTypeId), table['purpose_id'].eq(6)]
        if not db.getRecordEx(table, '*', where=cond):
            return False
    return True


def checkEventPosibility(db, clientId, eventTypeId, personId, eventDate):
    u'''
        Проверка возможности создания события
    '''
    if not checkEventPosibilityLetal(db, clientId, eventTypeId, eventDate):
        return False,  None
    description = CEventTypeDescription.get(eventTypeId)
    if description.form == '030':
        posible, eventId = checkF030EventPosibility(clientId, eventTypeId, personId, eventDate)
    elif description.period or description.singleInPeriod:
        posible, eventId = checkPeriodicEventPosibility(clientId, eventTypeId, eventDate, description.period, description.singleInPeriod)
    else:
        posible, eventId = checkNonPeriodicEventPosibility(clientId, eventTypeId, personId, eventDate)
    return posible, eventId


def checkNonPeriodicEventPosibility(db, clientId, eventTypeId, personId, eventDate):
    # Проверка возможности создания непериодического события
    table = db.table('Event')
    cond = [table['client_id'].eq(clientId),
            table['eventType_id'].eq(eventTypeId),
            table['execPerson_id'].eq(personId),
            table['execDate'].eq(eventDate),
            ]
    idList = db.getIdList(table, where=cond, order='id', limit=1)
    if idList:
        return False, idList[0]
    else:
        return True, None


def checkPeriodicEventPosibility(db, clientId, eventTypeId, eventDate, period, singleInPeriod):
    # Проверка возможности создания периодического события
    table = db.table('Event')
    cond = [table['client_id'].eq(clientId), table['eventType_id'].eq(eventTypeId) ]
    if period != 0:
        lowDate  = eventDate.addMonths(-period)
        highDate = eventDate.addMonths(+period)
        cond.append(table['setDate'].between(lowDate,  highDate))
    if singleInPeriod:
        if singleInPeriod == 1:
            lowDate  = firstWeekDay(eventDate)
            highDate = lastWeekDay(eventDate)
        elif singleInPeriod == 2:
            lowDate  = firstMonthDay(eventDate)
            highDate = lastMonthDay(eventDate)
        elif singleInPeriod == 3:
            lowDate  = firstQuarterDay(eventDate)
            highDate = lastQuarterDay(eventDate)
        elif singleInPeriod == 4:
            lowDate  = firstHalfYearDay(eventDate)
            highDate = lastHalfYearDay(eventDate)
        elif singleInPeriod == 5:
            lowDate  = firstYearDay(eventDate)
            highDate = lastYearDay(eventDate)
        else:
            lowDate  = eventDate
            highDate = eventDate
        cond.append(table['setDate'].between(lowDate,  highDate))
    idList = db.getIdList(table, where=cond, order='id')
    if idList:
        return False, idList[0]
    else:
        return True, None


def getEventPeriod(period, singleInPeriod):
    # вычисление периода события по интервалу и периоду однократности
    if singleInPeriod == 2:
        periodBySingle = 1
    elif singleInPeriod == 3:
        periodBySingle = 3
    elif singleInPeriod == 4:
        periodBySingle = 6
    elif singleInPeriod == 5:
        periodBySingle = 12
    else:
        periodBySingle = 0
    return max(period, periodBySingle)
