# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

u"""Утилиты формирования счёта"""
import re
import json

from collections                     import namedtuple
from PyQt4                           import QtGui
from PyQt4.QtCore                    import QDate, QVariant

from library.AgeSelector             import checkAgeSelector
from library.Calendar                import wpSevenDays
from library.database                import CTableRecordCache
from library.Utils                   import ( smartDict,
                                              calcAgeInYears,
                                              calcAgeTuple,
                                              firstMonthDay,
                                              firstYearDay,
                                              forceBool,
                                              forceDate,
                                              forceDateTime,
                                              forceDouble,
                                              forceInt,
                                              forceRef,
                                              forceString,
                                              lastMonthDay,
                                              lastYearDay,
                                              toVariant,
                                            )

from Accounting.Tariff               import CTariff
from Accounting.Utils                import ( CCoefficientTypeDescr,
                                              getNextAccountNumber,
                                              getRefuseTypeId,
                                              setActionPayStatus,
                                              setEventPayStatus,
                                              setEventVisitsPayStatus,
                                              setVisitPayStatus,
                                              setEventCsgPayStatus,
                                              unpackExposeDiscipline,
                                              updateDocsPayStatus,
                                            )

from Events.ActionServiceType        import CActionServiceType
from Events.ActionStatus             import CActionStatus
from Events.MapActionTypeToServiceIdList import CMapActionTypeIdToServiceIdList
from Events.Utils                    import ( CPayStatus,
                                              CFinanceType,
                                              getEventDuration,
                                              getEventServiceId,
                                              getEventAidTypeRegionalCode,
                                              getEvenMesServiceMask,
                                              getEventDiagnosis
                                            )

__all__ = [ 'CAccountDetails',
            'CAccountPool',
            'CAccountBuilder',
            'getEventVisitsCircumstances',
            'getEventActionsCircumstances',
            'filterUniqueCircumstances',
            'getMesAmount',
            'evalPriceForKrasnodarA13',
            'evalPriceForMurmansk2015Hospital',
          ]


class CAccountDetails:
    def __init__(self, contractDescr, orgId, orgStructureId, settleDate, reexpose):
        self.contractDescr = contractDescr
        self.orgId = orgId
        self.orgStructureId = orgStructureId
        self.settleDate = settleDate
        self.reexpose = reexpose
        self.id, self.record = self.createAccountRecord()
        self.totalAmount = 0
        self.totalUet = 0
        self.totalSum = 0


    def createAccountRecord(self):
        db = QtGui.qApp.db
        table = db.table('Account')
        record = table.newRecord()
        record.setValue('contract_id', toVariant(self.contractDescr.id))
        record.setValue('orgStructure_id',  toVariant(self.orgStructureId))
        record.setValue('payer_id',    toVariant(self.contractDescr.payerId))
        record.setValue('settleDate',  toVariant(self.settleDate))
#        record.setValue('number',      '')
        record.setValue('date',        toVariant(QDate.currentDate()))
        accountId = db.insertRecord(table,  record)
        record.setValue('id',          toVariant(accountId))
        return accountId, record


    def update(self, settleDate, number):
        record = self.record
        record.setValue('settleDate',  toVariant(settleDate))
        record.setValue('number',      toVariant(number))
        record.setValue('amount',      toVariant(self.totalAmount))
        record.setValue('uet',         toVariant(self.totalUet))
        record.setValue('sum',         toVariant(self.totalSum))
        QtGui.qApp.db.updateRecord('Account', record)


class CAccountPool:
    # порядок полей важен для упрощения сортировки
    CKey = namedtuple('CKey',
                      (
                        'date',
                        'insurer',
                        'batch',
                        'client',
                        'event',
                      )
                     )


    def __init__(self, contractDescr, orgId, orgStructureId, settleDate, reexposeInSeparateAccount):
        self.contractDescr = contractDescr
        self.orgId = orgId
        self.orgStructureId = orgStructureId
        self.settleDate = settleDate
        self.reexposeInSeparateAccount = reexposeInSeparateAccount
        self.mapKeyExToDetails = {}
        self.mapEventIdToExternalIdAndSeqNumber = {}
        self.mapInsurerIdToKey = {}
        self.baseAccountNumber = None

        self.exposeByBatch, self.exposeByEvent, self.exposeByMonth, self.exposeByClient, self.exposeByInsurer = unpackExposeDiscipline(self.contractDescr.exposeDiscipline)

        if self.contractDescr.financeType == CFinanceType.cash and self.exposeByEvent:
            self.getTransformedEventId = self._getExternalIdAndSeqNumber
            self.getAccountNumber      = self._getAccountNumberWithoutBaseNumber
        else:
            self.getTransformedEventId = self._getEventIdAsIs
            self.getAccountNumber      = self._getTrivialAccountNumber

        if self.exposeByMonth:
            self.getSettleDate = self._getLastMonthDayAsSettleDate
        else:
            self.getSettleDate = self._getTrivialSettleDate


    def getBaseAccountNumber(self):
        if self.baseAccountNumber is None:
            self.baseAccountNumber = getNextAccountNumber(self.contractDescr.number)
        return self.baseAccountNumber


    def createAccount(self, reexpose):
        return CAccountDetails(self.contractDescr, self.orgId, self.orgStructureId, self.settleDate, reexpose)


    def _getAccount(self, key, reexpose):
        reexpose = bool(self.reexposeInSeparateAccount and reexpose)
        result = self.mapKeyExToDetails.get((key, reexpose), None)
        if result is None:
            result = self.createAccount(reexpose)
            self.mapKeyExToDetails[(key, reexpose)] = result
        return result


    def _getEventIdAsIs(self, eventId):
        return eventId


    # это для случая if self.contractDescr.financeType == CFinanceType.cash and self.exposeByEvent:
    def _getExternalIdAndSeqNumber(self, eventId):
        if not eventId:
            return None

        # кеширование приводит к тому, что мы не видим новые счета,
        # таким образом происходит группировка услуг одного события в один счёт
        result = self.mapEventIdToExternalIdAndSeqNumber.get(eventId, None)
        if result is None:
            db = QtGui.qApp.db
            tableEvent = db.table('Event')
            result = forceString(db.translate(tableEvent, tableEvent['id'], eventId, tableEvent['externalId']))
            if not result:
                result = '{%d}' % eventId

            tableAccount     = db.table('Account')
            tableAccountItem = db.table('Account_Item')
            cnt = db.getDistinctCount(tableAccountItem.innerJoin(tableAccount,
                                                                 tableAccount['id'].eq(tableAccountItem['master_id'])
                                                                ),
                                      tableAccount['id'].name(),
                                      [ tableAccountItem['deleted'].eq(0),
                                        tableAccountItem['event_id'].eq(eventId),
                                        tableAccount['deleted'].eq(0),
                                        tableAccount['contract_id'].eq(self.contractDescr.id),
                                        tableAccount['settleDate'].eq(self.settleDate),
                                      ]
                                     )
            if cnt != 0:
                result += '/%d' % cnt
            self.mapEventIdToExternalIdAndSeqNumber[eventId] = result
        return result


    def getInsurerKeyPart(self, clientId, date):
        from Orgs.Utils import getOrganisationShortName
        from Registry.Utils import getClientCompulsoryPolicy, getClientVoluntaryPolicy

        def getInsurerKeyPartByInsurerId(insurerId):
            result = self.mapInsurerIdToKey.get(insurerId, None)
            if result is None:
                if insurerId:
                    tmpInsurerId = insurerId
                    db = QtGui.qApp.db
                    table = db.table('Organisation')
                    if self.exposeByInsurer == 1:
                        while True:
                            headId = forceRef(db.translate(table, 'id', tmpInsurerId, 'head_id'))
                            if headId:
                                tmpInsurerId = headId
                            else:
                                break
                    result = forceString(db.translate(table, 'id', tmpInsurerId, 'infisCode'))
                    if not result:
                        result = getOrganisationShortName(tmpInsurerId)
                    if not result:
                        result = '{%d}' % tmpInsurerId
                else:
                    result = u'CK не указана'
                self.mapInsurerIdToKey[insurerId] = result
            return result

        if clientId:
            if self.contractDescr.financeType == CFinanceType.VMI:
                record = getClientVoluntaryPolicy(clientId)
            else:
                record = getClientCompulsoryPolicy(clientId)
            if record:
                insurerId = forceRef(record.value('insurer_id'))
                return getInsurerKeyPartByInsurerId(insurerId)
        return u'нет полиса'


    def _getTrivialSettleDate(self, key):
        return self.settleDate


    def _getLastMonthDayAsSettleDate(self, key):
        return min(self.settleDate, lastMonthDay(QDate(key.date)))


    def getKey(self, clientId, date, eventId, batch):
        key = CAccountPool.CKey(
                date     = firstMonthDay(date).toPyDate()         if self.exposeByMonth   else None,
                insurer  = self.getInsurerKeyPart(clientId, date) if self.exposeByInsurer else None,
                batch    = batch                                  if self.exposeByBatch   else None,
                client   = clientId                               if self.exposeByClient  else None,
                event    = self.getTransformedEventId(eventId)    if self.exposeByEvent   else None,
              )
        return key


    def getAccount(self, clientId, date, eventId, batch, reexpose=False):
        return self._getAccount(self.getKey(clientId, date, eventId, batch), reexpose)


    def addAccountIfEmpty(self, reexpose=False):
        if not self.mapKeyExToDetails:
            self.getAccount(None, self.settleDate, None, '', False)
            if reexpose and self.reexposeInSeparateAccount:
                self.getAccount(None, self.settleDate, None, '', True)


    def getDetailsList(self):
        return [ details for key, details in sorted(self.mapKeyExToDetails.iteritems()) ]


    def getAccountIdList(self):
        return [details.id for details in self.getDetailsList()]


    def updateDetails(self):
        for (key, reexpose), details in sorted(self.mapKeyExToDetails.iteritems()):
            details.update(self.getSettleDate(key), self.getAccountNumber(key, reexpose))


    def _getTrivialAccountNumber(self, key, reexpose):
        return self.formatNumber(self.getBaseAccountNumber(), key, reexpose)


    def _getAccountNumberWithoutBaseNumber(self, key, reexpose):
        if any(key):
            return self.formatNumber('', key, reexpose)
        else:
            return self._getTrivialAccountNumber(key, reexpose)


    def formatNumber(self, baseAccountNumber, key, reexpose):
        parts = []
        if baseAccountNumber:
            parts.append(baseAccountNumber)
        if self.exposeByInsurer and key.insurer:
            parts.append(unicode(key.insurer))
        if self.exposeByBatch and key.batch:
            parts.append(unicode(key.batch))
        if self.exposeByClient and key.client:
            parts.append(unicode(key.client))
        if self.exposeByEvent and key.event:
            parts.append(unicode(key.event))
        if reexpose:
            parts.append(u'П')
        return '/'.join(parts)


class CAccountBuilder(CMapActionTypeIdToServiceIdList):
    def __init__(self):
        CMapActionTypeIdToServiceIdList.__init__(self)
        self.mapMesIdToDescr = {}
        self.mapCsgCodeToDescr = {}
        self.mapEventIdToMKB = {}
        self.semifinishedMesRefuseTypeId = None
        self.clientRecordCache = CTableRecordCache(QtGui.qApp.db, 'Client', ('sex', 'birthDate'))


    def resetBuilder(self):
        self.mapMesIdToDescr.clear()
        self.mapCsgCodeToDescr.clear()
        self.mapEventIdToMKB.clear()
        self.semifinishedMesRefuseTypeId = None
        self.clientRecordCache.invalidate()


    def exposeByEvents(self, progressDialog, contractDescr, accountFactory, eventIdList, checkMes):
        for eventId in eventIdList:
            if progressDialog:
                progressDialog.step()
                QtGui.qApp.processEvents()
            self.exposeEvent(contractDescr, accountFactory, eventId)
            self.exposeEventAsCoupleVisit(contractDescr, accountFactory, eventId)
            self.exposeEventByHospitalBedDay(contractDescr, accountFactory, eventId)
            self.exposeVisitsByMes(contractDescr, accountFactory, eventId, checkMes)
            self.exposeEventByMes(contractDescr,  accountFactory, eventId)


    def exposeByVisitsByActionServices(self, progressDialog, contractDescr, accountFactory, mapServiceIdToVisitIdList):
        for serviceId, visitIdList in mapServiceIdToVisitIdList.items():
            for visitId in visitIdList:
                if progressDialog:
                    progressDialog.step()
                    QtGui.qApp.processEvents()
                self.exposeVisitByActionService(contractDescr, accountFactory, serviceId, visitId)


    def exposeByVisits(self, progressDialog, contractDescr, accountFactory, visitIdList):
        for visitId in visitIdList:
            if progressDialog:
                progressDialog.step()
                QtGui.qApp.processEvents()
            self.exposeVisit(contractDescr, accountFactory, visitId)


    def exposeByActions(self, progressDialog, contractDescr, accountFactory, actionIdList, date=None):
        for actionId in actionIdList:
            if progressDialog:
                progressDialog.step()
                QtGui.qApp.processEvents()
            self.exposeAction(contractDescr, accountFactory, actionId, date)


    def exposeByHospitalBedActionProperties(self, progressDialog, contractDescr, accountFactory, actionPropertyIdList):
        for actionPropertyId in actionPropertyIdList:
            if progressDialog:
                progressDialog.step()
                QtGui.qApp.processEvents()
            self.exposeHospitalBedActionProperty(contractDescr, accountFactory, actionPropertyId)


    def exposeVisitsByEventAndServicePairs(self, progressDialog, contractDescr, accountFactory, eventIdServiceIdPairList):
        for eventId, serviceId in eventIdServiceIdPairList:
            if progressDialog:
                progressDialog.step()
                QtGui.qApp.processEvents()
            self.exposeVisitsByEventAndServicePair(contractDescr, accountFactory, eventId, serviceId)


    def exposeByCsgs(self, progressDialog, contractDescr, accountFactory, csgIdList):
        for csgId in csgIdList:
            if progressDialog:
                progressDialog.step()
                QtGui.qApp.processEvents()
            self.exposeCsg(contractDescr, accountFactory, csgId)


    def exposeEvent(self, contractDescr, accountFactory, eventId):
        db = QtGui.qApp.db
        event = db.getRecord('Event'\
                             ' LEFT JOIN Person ON Person.id = Event.execPerson_id'\
                             ' LEFT JOIN rbMesSpecification ON rbMesSpecification.id = Event.mesSpecification_id',
                             'Event.eventType_id, Event.result_id, Event.client_id, Event.execDate, Person.tariffCategory_id, rbMesSpecification.level',
                             eventId)
        eventTypeId  = forceRef(event.value('eventType_id'))
        resultId     = forceRef(event.value('result_id'))
        mesLevel     = forceInt(event.value('level'))
        eventEndDate = forceDate(event.value('execDate'))
        tariffCategoryId = forceRef(event.value('tariffCategory_id'))
        tariffList = contractDescr.tariffByEventType.get(eventTypeId, None)
        serviceId = getEventServiceId(eventTypeId)
        if tariffList:
            for tariff in tariffList:
                if ( self.isTariffApplicable(tariff, eventId, resultId, mesLevel, tariffCategoryId, eventEndDate) and
                     ( tariff.serviceId is None or serviceId is None or tariff.serviceId == serviceId )
                   ):
                    price  = tariff.price
                    amount = 1.0
                    sum    = round(price*amount, 2)
                    clientId = forceRef(event.value('client_id'))
                    account = accountFactory(clientId, eventEndDate, eventId, tariff.batch)
                    tableAccountItem = db.table('Account_Item')
                    accountItem = tableAccountItem.newRecord()
                    accountItem.setValue('master_id',   toVariant(account.id))
                    accountItem.setValue('serviceDate', toVariant(eventEndDate))
                    accountItem.setValue('event_id',    toVariant(eventId))
                    accountItem.setValue('price',       toVariant(price))
                    accountItem.setValue('unit_id',     toVariant(tariff.unitId))
                    accountItem.setValue('amount',      toVariant(amount))
                    accountItem.setValue('sum',         toVariant(sum))
                    accountItem.setValue('tariff_id',   toVariant(tariff.id))
                    accountItem.setValue('service_id',  toVariant(tariff.serviceId or serviceId))
                    accountItem.setValue('VAT',         toVariant(tariff.vat))
                    db.insertRecord(tableAccountItem, accountItem)
                    account.totalAmount += amount
                    account.totalSum    += sum
                    setEventPayStatus(eventId, contractDescr.payStatusMask, CPayStatus.exposedBits)
                    return


    def exposeEventAsCoupleVisit(self, contractDescr, accountFactory, eventId):
        db = QtGui.qApp.db
        event = db.getRecord('Event'\
                             ' LEFT JOIN Person ON Person.id = Event.execPerson_id'\
                             ' LEFT JOIN rbMesSpecification ON rbMesSpecification.id = Event.mesSpecification_id',
                             'Event.eventType_id, Event.result_id, Event.client_id, Event.execDate, Person.tariffCategory_id, rbMesSpecification.level',
                             eventId)
        eventTypeId  = forceRef(event.value('eventType_id'))
        resultId     = forceRef(event.value('result_id'))
        mesLevel     = forceInt(event.value('level'))
        eventEndDate = forceDate(event.value('execDate'))
        tariffCategoryId = forceRef(event.value('tariffCategory_id'))
        tariffList = contractDescr.tariffByCoupleVisitEventType.get(eventTypeId, None)
        if tariffList:
            for tariff in tariffList:
                if self.isTariffApplicable(tariff, eventId, resultId, mesLevel, tariffCategoryId, eventEndDate):
                    serviceId = tariff.serviceId
                    if not serviceId:
                        continue
                    count = db.getCount('Visit', '1', 'event_id=%d AND service_id=%d AND deleted=0'%(eventId, serviceId))
                    if count == 0:
                        continue
                    amount = float(count)
                    amount, price, sum = tariff.evalAmountPriceSum(amount)
                    clientId = forceRef(event.value('client_id'))
                    account = accountFactory(clientId, eventEndDate, eventId, tariff.batch)
                    tableAccountItem = db.table('Account_Item')
                    accountItem = tableAccountItem.newRecord()
                    accountItem.setValue('master_id',   toVariant(account.id))
                    accountItem.setValue('serviceDate', toVariant(eventEndDate))
                    accountItem.setValue('event_id',    toVariant(eventId))
                    accountItem.setValue('price',       toVariant(price))
                    accountItem.setValue('unit_id',     toVariant(tariff.unitId))
                    accountItem.setValue('amount',      toVariant(amount))
                    accountItem.setValue('sum',         toVariant(sum))
                    accountItem.setValue('tariff_id',   toVariant(tariff.id))
                    accountItem.setValue('service_id',  toVariant(serviceId))
                    accountItem.setValue('VAT',         toVariant(tariff.vat))
                    db.insertRecord(tableAccountItem, accountItem)
                    account.totalAmount += amount
                    account.totalSum    += sum
                    setEventPayStatus(eventId, contractDescr.payStatusMask, CPayStatus.exposedBits)
                    return


    def exposeEventByHospitalBedDay(self, contractDescr, accountFactory, eventId):
        db = QtGui.qApp.db
        event = db.getRecord('Event'\
                             ' LEFT JOIN Person ON Person.id = Event.execPerson_id'\
                             ' LEFT JOIN rbMesSpecification ON rbMesSpecification.id = Event.mesSpecification_id',
                             'Event.eventType_id, Event.result_id, Event.client_id, Event.setDate, Event.execDate, Person.tariffCategory_id, rbMesSpecification.level',
                             eventId)
        resultId     = forceRef(event.value('result_id'))
        mesLevel     = forceInt(event.value('level'))
        eventTypeId  = forceRef(event.value('eventType_id'))
        eventBegDate = forceDate(event.value('setDate'))
        eventEndDate = forceDate(event.value('execDate'))
        tariffCategoryId = forceRef(event.value('tariffCategory_id'))
        if eventBegDate and eventEndDate and eventBegDate<=eventEndDate:
            tariffList = contractDescr.tariffByHospitalBedDay.get(eventTypeId, None)
            if tariffList:
                for tariff in tariffList:
                    if self.isTariffApplicable(tariff, eventId, resultId, mesLevel, tariffCategoryId, eventEndDate):
                        count = eventBegDate.daysTo(eventEndDate)+1
                        amount = float(count)
                        amount, price, sum = tariff.evalAmountPriceSum(amount)
                        clientId = forceRef(event.value('client_id'))
                        account = accountFactory(clientId, eventEndDate, eventId, tariff.batch)
                        tableAccountItem = db.table('Account_Item')
                        accountItem = tableAccountItem.newRecord()
                        accountItem.setValue('master_id',   toVariant(account.id))
                        accountItem.setValue('serviceDate', toVariant(eventEndDate))
                        accountItem.setValue('event_id',    toVariant(eventId))
                        accountItem.setValue('price',       toVariant(price))
                        accountItem.setValue('unit_id',     toVariant(tariff.unitId))
                        accountItem.setValue('amount',      toVariant(amount))
                        accountItem.setValue('sum',         toVariant(sum))
                        accountItem.setValue('tariff_id',   toVariant(tariff.id))
                        accountItem.setValue('service_id',  toVariant(tariff.serviceId))
                        accountItem.setValue('VAT',         toVariant(tariff.vat))
                        db.insertRecord(tableAccountItem, accountItem)
                        account.totalAmount += amount
                        account.totalSum    += sum
                        setEventPayStatus(eventId, contractDescr.payStatusMask, CPayStatus.exposedBits)
                        return


    def exposeVisitsByMes(self, contractDescr, accountFactory, eventId, checkMes):
        db = QtGui.qApp.db
        event = db.getRecord('Event'\
                             ' LEFT JOIN Person ON Person.id = Event.execPerson_id'\
                             ' LEFT JOIN rbMesSpecification ON rbMesSpecification.id = Event.mesSpecification_id',
                             'Event.eventType_id, Event.result_id, Event.client_id, Event.execDate, Event.MES_id, Person.tariffCategory_id, rbMesSpecification.level',
                             eventId)
        eventTypeId  = forceRef(event.value('eventType_id'))
        resultId     = forceRef(event.value('result_id'))
        mesLevel     = forceInt(event.value('level'))
        eventEndDate = forceDate(event.value('execDate'))
        mesId = forceRef(event.value('MES_id'))
        serviceId = self.getMesService(mesId)
        tariffCategoryId = forceRef(event.value('tariffCategory_id'))
        tariffList = contractDescr.tariffVisitsByMES.get((eventTypeId, serviceId), None)
        if tariffList and mesId:
            for tariff in tariffList:
                if self.isTariffApplicable(tariff, eventId, resultId, mesLevel, tariffCategoryId, eventEndDate):
                    price  = tariff.price
                    amount = float(getMesAmount(eventId, mesId))
                    amount, price, sum = tariff.evalAmountPriceSum(amount)
                    sum    = round(price*amount, 2)
                    clientId = forceRef(event.value('client_id'))
                    account = accountFactory(clientId, eventEndDate, eventId, tariff.batch)
                    tableAccountItem = db.table('Account_Item')
                    accountItem = tableAccountItem.newRecord()
                    accountItem.setValue('master_id',   toVariant(account.id))
                    accountItem.setValue('serviceDate', toVariant(eventEndDate))
                    accountItem.setValue('event_id',    toVariant(eventId))
                    accountItem.setValue('price',       toVariant(price))
                    accountItem.setValue('unit_id',     toVariant(tariff.unitId))
                    accountItem.setValue('amount',      toVariant(amount))
                    accountItem.setValue('sum',         toVariant(sum))
                    accountItem.setValue('tariff_id',   toVariant(tariff.id))
                    accountItem.setValue('service_id',  toVariant(serviceId))
                    accountItem.setValue('VAT',         toVariant(tariff.vat))
                    if checkMes:
                        norm = self.getMesNorm(mesId)
                        if amount < norm:
                            self.rejectAccountItemBySemifinishedMes(accountItem, contractDescr.financeId)
                    db.insertRecord(tableAccountItem, accountItem)
                    account.totalAmount += amount
                    account.totalSum    += sum
                    setEventPayStatus(eventId, contractDescr.payStatusMask, CPayStatus.exposedBits)
                    setEventVisitsPayStatus(eventId, contractDescr.payStatusMask, CPayStatus.exposedBits)
                    return


    def exposeEventByMes(self, contractDescr, accountFactory, eventId):
        db = QtGui.qApp.db
        event = db.getRecord('Event'\
                             ' LEFT JOIN Person ON Person.id = Event.execPerson_id'
                             ' LEFT JOIN rbMesSpecification ON rbMesSpecification.id=Event.mesSpecification_id',
                             'Event.eventType_id, Event.result_id, Event.client_id, Event.setDate, Event.execDate, Event.MES_id, Person.tariffCategory_id, rbMesSpecification.level',
                             eventId)

        eventTypeId  = forceRef(event.value('eventType_id'))
        resultId     = forceRef(event.value('result_id'))
        mesLevel     = forceInt(event.value('level'))
        eventBegDate = forceDate(event.value('setDate'))
        eventEndDate = forceDate(event.value('execDate'))
        mesId = forceRef(event.value('MES_id'))
        serviceId = self.getMesService(mesId)
        tariffCategoryId = forceRef(event.value('tariffCategory_id'))
        tariffList = contractDescr.tariffEventByMES.get((eventTypeId, serviceId), None)
        if tariffList and mesId:
            for tariff in tariffList:
                if self.isTariffApplicable(tariff, eventId, resultId, mesLevel, tariffCategoryId, eventEndDate):
                    clientId = forceRef(event.value('client_id'))
                    price  = tariff.price
                    usedCoefficients = None
                    mesDescr = self.getMesDescr(mesId)
                    duration = getEventDuration(eventBegDate, eventEndDate, wpSevenDays, eventTypeId)

                    if tariff.tariffType == CTariff.ttEventByMESLen:
                        amount, price, sum = tariff.evalAmountPriceSum(duration)
                        coefficient, usedCoefficients = self.getCoefficientAndReport(contractDescr, mesDescr, duration, eventEndDate, eventId, None)
                        sum = round(sum*coefficient, 2)
                    elif tariff.tariffType == CTariff.ttEventByMESLevel:
                        amount = float(duration)/mesDescr.avgDuration if mesDescr.avgDuration else 1
                        if mesLevel == 2: # выполнен полностью:
                            amount = 1
#                        elif mesLevel == 1: # выполнен частично
#                            amount = min(1, amount)
#                        elif mesLevel == 0: # прерван
#                            amount = min(1, amount)
                        else: # fallback
                            amount = min(1, amount)
                        amount, price, sum = tariff.evalAmountPriceSum(amount)
                        coefficient, usedCoefficients = self.getCoefficientAndReport(contractDescr, mesDescr, duration, eventEndDate, eventId, None)
                        sum = round(sum*coefficient, 2)
                    elif tariff.tariffType == CTariff.ttKrasnodarA13:
                        amount = 1.0
                        price  = evalPriceForKrasnodarA13(contractDescr, tariff, clientId, eventId, eventTypeId, eventBegDate, eventEndDate)
                        sum    = round(price, 2)
                    elif tariff.tariffType == CTariff.ttMurmansk2015Hospital:
                        eventBegDateTime = forceDateTime(event.value('setDate'))
                        eventEndDateTime = forceDateTime(event.value('execDate'))
                        shortHospitalisation = eventBegDateTime.addDays(1) > eventEndDateTime
                        level = forceInt(event.value('level'))
                        amount = 1.0
                        price = evalPriceForMurmansk2015Hospital(contractDescr, tariff, eventId, eventTypeId, eventBegDate, eventEndDate,  mesId, shortHospitalisation, level, mesDescr = mesDescr)
                        sum    = round(price, 2)
                    else:
                        amount, price, sum = tariff.evalAmountPriceSum(1)
                        coefficient, usedCoefficients = self.getCoefficientAndReport(contractDescr, mesDescr, duration, eventEndDate, eventId, None)
                        sum = round(sum*coefficient, 2)

                    account = accountFactory(clientId, eventEndDate, eventId, tariff.batch)
                    tableAccountItem = db.table('Account_Item')
                    accountItem = tableAccountItem.newRecord()
                    accountItem.setValue('master_id',   toVariant(account.id))
                    accountItem.setValue('serviceDate', toVariant(eventEndDate))
                    accountItem.setValue('event_id',    toVariant(eventId))
                    accountItem.setValue('price',       toVariant(price))
                    accountItem.setValue('unit_id',     toVariant(tariff.unitId))
                    accountItem.setValue('amount',      toVariant(amount))
                    accountItem.setValue('sum',         toVariant(sum))
                    accountItem.setValue('tariff_id',   toVariant(tariff.id))
                    accountItem.setValue('service_id',  toVariant(serviceId))
                    accountItem.setValue('VAT',         toVariant(tariff.vat))
                    accountItem.setValue('usedCoefficients', toVariant(usedCoefficients))
                    db.insertRecord(tableAccountItem, accountItem)
                    account.totalAmount += amount
                    account.totalSum    += sum
                    setEventPayStatus(eventId, contractDescr.payStatusMask, CPayStatus.exposedBits)
                    return


    def exposeVisitByActionService(self, contractDescr, accountFactory, serviceId, visitId):
        db = QtGui.qApp.db
        visit = db.getRecord('Visit'\
                             ' INNER JOIN Event ON Event.id=Visit.event_id'\
                             ' LEFT JOIN Person ON Person.id = Visit.person_id'\
                             ' LEFT JOIN rbMesSpecification ON rbMesSpecification.id = Event.mesSpecification_id',
                             'Event.result_id, Event.eventType_id, Event.client_id, Visit.id, Visit.event_id, Visit.date, Person.tariffCategory_id, rbMesSpecification.level',
                             visitId)
        tariffList = contractDescr.tariffVisitByActionService.get(serviceId, None)
        if tariffList:
            resultId  = forceRef(visit.value('result_id'))
            mesLevel  = forceInt(visit.value('level'))
            eventId   = forceRef(visit.value('event_id'))
            visitDate = forceDate(visit.value('date'))
            tariffCategoryId = forceRef(visit.value('tariffCategory_id'))
            for tariff in tariffList:
                if self.isTariffApplicable(tariff, eventId, resultId, mesLevel, tariffCategoryId, visitDate):
                    price  = tariff.price
                    amount = 1.0
                    sum    = round(price*amount, 2)
                    clientId = forceRef(visit.value('client_id'))
                    account = accountFactory(clientId, visitDate, eventId, tariff.batch)
                    tableAccountItem = db.table('Account_Item')
                    accountItem = tableAccountItem.newRecord()
                    accountItem.setValue('master_id',  toVariant(account.id))
                    accountItem.setValue('serviceDate',toVariant(visitDate))
                    accountItem.setValue('event_id',   visit.value('event_id'))
                    accountItem.setValue('visit_id',   toVariant(visitId))
                    accountItem.setValue('price',      toVariant(price))
                    accountItem.setValue('unit_id',    toVariant(tariff.unitId))
                    accountItem.setValue('amount',     toVariant(amount))
                    accountItem.setValue('sum',        toVariant(sum))
                    accountItem.setValue('tariff_id',  toVariant(tariff.id))
                    accountItem.setValue('service_id', toVariant(serviceId))
                    accountItem.setValue('VAT',        toVariant(tariff.vat))
                    db.insertRecord(tableAccountItem, accountItem)
                    account.totalAmount += amount
                    account.totalSum    += sum
                    setVisitPayStatus(visitId, contractDescr.payStatusMask, CPayStatus.exposedBits)
                    return


    def exposeVisit(self, contractDescr, accountFactory, visitId):
        db = QtGui.qApp.db
        visit = db.getRecord('Visit'\
                             ' INNER JOIN Event ON Event.id = Visit.event_id'\
                             ' LEFT JOIN Person ON Person.id = Visit.person_id'\
                             ' LEFT JOIN rbMesSpecification ON rbMesSpecification.id = Event.mesSpecification_id',
                             'Event.result_id, Event.eventType_id, Event.client_id, Visit.id, Visit.event_id, Visit.date, Visit.service_id, Person.tariffCategory_id, rbMesSpecification.level',
                             visitId)
        serviceId = forceRef(visit.value('service_id'))
        tariffList = contractDescr.tariffByVisitService.get(serviceId, None)
        if tariffList:
            resultId  = forceRef(visit.value('result_id'))
            mesLevel  = forceInt(visit.value('level'))
            eventId   = forceRef(visit.value('event_id'))
            visitDate = forceDate(visit.value('date'))
            tariffCategoryId = forceRef(visit.value('tariffCategory_id'))
            for tariff in tariffList:
                if self.isTariffApplicable(tariff, eventId, resultId, mesLevel, tariffCategoryId, visitDate):
                    price  = tariff.price
                    amount = 1.0
                    sum    = round(price*amount, 2)
                    clientId = forceRef(visit.value('client_id'))
                    account = accountFactory(clientId, visitDate, eventId, tariff.batch)
                    tableAccountItem = db.table('Account_Item')
                    accountItem = tableAccountItem.newRecord()
                    accountItem.setValue('master_id',  toVariant(account.id))
                    accountItem.setValue('serviceDate',toVariant(visitDate))
                    accountItem.setValue('event_id',   visit.value('event_id'))
                    accountItem.setValue('visit_id',   toVariant(visitId))
                    accountItem.setValue('price',      toVariant(price))
                    accountItem.setValue('unit_id',    toVariant(tariff.unitId))
                    accountItem.setValue('amount',     toVariant(amount))
                    accountItem.setValue('sum',        toVariant(sum))
                    accountItem.setValue('tariff_id',  toVariant(tariff.id))
                    accountItem.setValue('service_id', toVariant(serviceId))
                    accountItem.setValue('VAT',        toVariant(tariff.vat))
                    db.insertRecord(tableAccountItem, accountItem)
                    account.totalAmount += amount
                    account.totalSum    += sum
                    setVisitPayStatus(visitId, contractDescr.payStatusMask, CPayStatus.exposedBits)
                    return


    def exposeAction(self, contractDescr, accountFactory, actionId, date):
        db = QtGui.qApp.db
        action = db.getRecord('vAction'\
                              ' INNER JOIN Event ON Event.id = vAction.event_id'\
                              ' LEFT JOIN Person ON Person.id = vAction.person_id'\
                              ' LEFT JOIN rbMesSpecification ON rbMesSpecification.id = Event.mesSpecification_id',
                              'Event.result_id, Event.eventType_id, Event.client_id, vAction.id, vAction.actionType_id, vAction.event_id, vAction.exposeDate, vAction.amount, vAction.MKB, Person.tariffCategory_id, rbMesSpecification.level',
                              actionId)
        serviceIdList = self.getActionTypeServiceIdList(forceRef(action.value('actionType_id')), contractDescr.financeId)
        origAmount = forceDouble(action.value('amount'))
        for serviceId in serviceIdList:
            tariffList = contractDescr.tariffByActionService.get(serviceId, None)
            if tariffList:
                resultId   = forceRef(action.value('result_id'))
                mesLevel   = forceInt(action.value('level'))
                eventId    = forceRef(action.value('event_id'))
                exposeDate = forceDate(action.value('exposeDate')) or date
                MKB = forceString(action.value('MKB'))
                tariffCategoryId = forceRef(action.value('tariffCategory_id'))
                for tariff in tariffList:
                    if self.isTariffApplicable(tariff, eventId, resultId, mesLevel, tariffCategoryId, exposeDate, MKB):
                        amount, price, sum = tariff.evalAmountPriceSum(origAmount)
                        uet = amount*tariff.uet if tariff.tariffType == CTariff.ttActionUET else 0
                        clientId = forceRef(action.value('client_id'))
                        account = accountFactory(clientId, exposeDate, eventId, tariff.batch)
                        tableAccountItem = db.table('Account_Item')
                        accountItem = tableAccountItem.newRecord()
                        accountItem.setValue('master_id',  toVariant(account.id))
                        accountItem.setValue('serviceDate',toVariant(exposeDate))
                        accountItem.setValue('event_id',   action.value('event_id'))
                        accountItem.setValue('action_id',  toVariant(actionId))
                        accountItem.setValue('price',      toVariant(price))
                        accountItem.setValue('unit_id',    toVariant(tariff.unitId))
                        accountItem.setValue('amount',     toVariant(amount))
                        accountItem.setValue('uet',        toVariant(uet))
                        accountItem.setValue('sum',        toVariant(sum))
                        accountItem.setValue('tariff_id',  toVariant(tariff.id))
                        accountItem.setValue('service_id', toVariant(serviceId))
                        accountItem.setValue('VAT',        toVariant(tariff.vat))
                        db.insertRecord(tableAccountItem, accountItem)
                        account.totalAmount += amount
                        account.totalUet    += uet
                        account.totalSum    += sum
                        setActionPayStatus(actionId, contractDescr.payStatusMask, CPayStatus.exposedBits)
                        break


    def exposeHospitalBedActionProperty(self, contractDescr, accountFactory, actionPropertyId):
        db = QtGui.qApp.db
        record = db.getRecord('ActionProperty'\
                              ' INNER JOIN Action ON Action.id = ActionProperty.action_id'\
                              ' LEFT JOIN Person ON Person.id = Action.person_id'\
                              ' LEFT JOIN ActionProperty_HospitalBed ON ActionProperty_HospitalBed.id = ActionProperty.id'\
                              ' LEFT JOIN OrgStructure_HospitalBed   ON OrgStructure_HospitalBed.id = ActionProperty_HospitalBed.value'\
                              ' LEFT JOIN rbHospitalBedProfile       ON rbHospitalBedProfile.id = OrgStructure_HospitalBed.profile_id'\
                              ' LEFT JOIN Event ON Event.id = Action.event_id'\
                              ' LEFT JOIN rbMesSpecification ON rbMesSpecification.id = Event.mesSpecification_id',
                              'Event.eventType_id, Event.result_id, Event.client_id, ActionProperty.action_id, Action.event_id, Action.amount, Action.MKB, Person.tariffCategory_id, rbHospitalBedProfile.service_id, rbMesSpecification.level',
                              actionPropertyId)
        eventTypeId = forceRef(record.value('eventType_id'))
        resultId    = forceRef(record.value('result_id'))
        mesLevel    = forceInt(record.value('level'))

        tariffList  = contractDescr.tariffByHospitalBedService.get(eventTypeId, None)
        if not tariffList:
            tariffList = contractDescr.tariffByHospitalBedService.get(None, None)
        if tariffList:
            endDate = forceDate(record.value('endDate'))
            MKB = forceString(record.value('MKB'))
            tariffCategoryId = forceRef(record.value('tariffCategory_id'))
            actionId     = forceRef(record.value('action_id'))
            eventId     = forceRef(record.value('event_id'))
            serviceId   = forceRef(record.value('service_id'))
            amount = forceDouble(record.value('amount'))
            for tariff in tariffList:
                if tariff.serviceId == serviceId and self.isTariffApplicable(tariff, eventId, resultId, mesLevel, tariffCategoryId, endDate, MKB):
                    amount, price, sum = tariff.evalAmountPriceSum(amount)
                    uet = amount*tariff.uet if tariff.tariffType == CTariff.ttActionUET else 0
                    clientId = forceRef(record.value('client_id'))
                    account = accountFactory(clientId, endDate, eventId, tariff.batch)
                    tableAccountItem = db.table('Account_Item')
                    accountItem = tableAccountItem.newRecord()
                    accountItem.setValue('master_id',   toVariant(account.id))
                    accountItem.setValue('serviceDate', toVariant(endDate))
                    accountItem.setValue('event_id',    record.value('event_id'))
                    accountItem.setValue('action_id',   toVariant(actionId))
                    accountItem.setValue('price',       toVariant(price))
                    accountItem.setValue('unit_id',     toVariant(tariff.unitId))
                    accountItem.setValue('amount',      toVariant(amount))
                    accountItem.setValue('uet',         toVariant(uet))
                    accountItem.setValue('sum',         toVariant(sum))
                    accountItem.setValue('tariff_id',   toVariant(tariff.id))
                    accountItem.setValue('service_id',  toVariant(serviceId))
                    accountItem.setValue('VAT',         toVariant(tariff.vat))
                    db.insertRecord(tableAccountItem, accountItem)
                    account.totalAmount += amount
                    account.totalUet    += uet
                    account.totalSum    += sum
                    setActionPayStatus(actionId, contractDescr.payStatusMask, CPayStatus.exposedBits)
                    return


    def exposeVisitsByEventAndServicePair(self, contractDescr, accountFactory, eventId, serviceId):
        db = QtGui.qApp.db
        event = db.getRecord('Event'\
                             ' LEFT JOIN rbMesSpecification ON rbMesSpecification.id = Event.mesSpecification_id',
                             'eventType_id, result_id, client_id, rbMesSpecification.level',
                             eventId)
        eventTypeId = forceRef(event.value('eventType_id'))
        resultId    = forceRef(event.value('result_id'))
        mesLevel    = forceInt(event.value('level'))
        clientId    = forceRef(event.value('client_id'))

        visits = db.getRecordList('Visit'\
                                  ' INNER JOIN Person ON Person.id = Visit.person_id'\
                                  ' INNER JOIN Event ON Event.id = Visit.event_id',
                                  'Visit.id, Visit.date, Person.tariffCategory_id',
                                  'Visit.deleted=0 AND Visit.event_id=%d AND Visit.service_id=%d AND (Visit.payStatus&%s)=0' % (eventId, serviceId, contractDescr.payStatusMask),
                                  'Visit.date'
                                 )
        if not visits:
            return
        tariffList = contractDescr.tariffCoupleVisits.get((eventTypeId, serviceId), []) + \
                     contractDescr.tariffCoupleVisits.get((eventTypeId, None), []) + \
                     contractDescr.tariffCoupleVisits.get((None, serviceId), []) + \
                     contractDescr.tariffCoupleVisits.get((None, None), [])
        tariffList.sort(key=lambda item:item.id)
        if tariffList:
            #firstVisitDate = forceDate(visits[0].value('date'))
            lastVisitDate  = forceDate(visits[-1].value('date'))
            tariffCategoryId = forceRef(visits[-1].value('tariffCategory_id')) #?!
            lastVisitId = forceRef(visits[-1].value('id')) #?!
            for tariff in tariffList:
                if self.isTariffApplicable(tariff, eventId, resultId, mesLevel, tariffCategoryId, lastVisitDate):
                    amount, price, sum = tariff.evalAmountPriceSum(len(visits))
                    account = accountFactory(clientId, lastVisitDate, eventId, tariff.batch)
                    tableAccountItem = db.table('Account_Item')
                    accountItem = tableAccountItem.newRecord()
                    accountItem.setValue('master_id',   toVariant(account.id))
                    accountItem.setValue('serviceDate', toVariant(lastVisitDate))
                    accountItem.setValue('event_id',    toVariant(eventId))
                    accountItem.setValue('visit_id',    toVariant(lastVisitId))
                    accountItem.setValue('price',       toVariant(price))
                    accountItem.setValue('unit_id',     toVariant(tariff.unitId))
                    accountItem.setValue('amount',      toVariant(amount))
                    accountItem.setValue('sum',         toVariant(sum))
                    accountItem.setValue('tariff_id',   toVariant(tariff.id))
                    accountItem.setValue('service_id',  toVariant(serviceId))
                    accountItem.setValue('VAT',         toVariant(tariff.vat))
                    db.insertRecord(tableAccountItem, accountItem)
                    account.totalAmount += amount
                    account.totalSum    += sum
                    for visit in visits:
                        setVisitPayStatus(forceRef(visit.value('id')), contractDescr.payStatusMask, CPayStatus.exposedBits)
                    return


    def exposeCsg(self, contractDescr, accountFactory, csgId):
        db = QtGui.qApp.db
        csg = db.getRecordEx('Event_CSG'\
                             ' INNER JOIN Event ON Event.id = Event_CSG.master_id'\
                             ' INNER JOIN rbService ON rbService.code = Event_CSG.CSGCode'\
                             ' LEFT JOIN Action ON Action.eventCSG_id=Event_CSG.id AND Action.deleted=0' \
                             ' LEFT JOIN Person ON Person.id = IFNULL(Action.person_id, Event.execPerson_id)'\
                             ' LEFT JOIN rbMesSpecification ON rbMesSpecification.id = Event.mesSpecification_id',
                             'Event.result_id, Event.eventType_id, Event.client_id, Event_CSG.CSGCode, Event_CSG.begDate, Event_CSG.endDate, '\
                             'Person.tariffCategory_id, rbMesSpecification.level, rbService.id AS service_id, '\
                             'Event.id AS event_id',
                             db.table('Event_CSG')['id'].eq(csgId),
                             'Action.id')
        serviceId = forceRef(csg.value('service_id'))
        eventTypeId = forceRef(csg.value('eventType_id'))
        tariffList = contractDescr.tariffByCSG.get((eventTypeId, serviceId), None)
        if tariffList:
            resultId  = forceRef(csg.value('result_id'))
            mesLevel  = forceInt(csg.value('level'))
            eventId   = forceRef(csg.value('event_id'))
            csgCode   = forceRef(csg.value('CSGCode'))
            csgBegDate= forceDate(csg.value('begDate'))
            csgEndDate= forceDate(csg.value('endDate'))
            tariffCategoryId = forceRef(csg.value('tariffCategory_id'))
            for tariff in tariffList:
                if self.isTariffApplicable(tariff, eventId, resultId, mesLevel, tariffCategoryId, csgEndDate):
                    price  = tariff.price
                    amount = 1.0
                    sum    = round(price*amount, 2)
                    duration = getEventDuration(csgBegDate, csgEndDate, wpSevenDays, eventTypeId)
                    csgDescr = self.getCsgDescr(csgCode)
                    coefficient, usedCoefficients = self.getCoefficientAndReport(contractDescr, csgDescr, duration, csgEndDate, eventId, csgId)
                    sum = round(sum*coefficient, 2)
                    clientId = forceRef(csg.value('client_id'))
                    account = accountFactory(clientId, csgEndDate, eventId, tariff.batch)
                    tableAccountItem = db.table('Account_Item')
                    accountItem = tableAccountItem.newRecord()
                    accountItem.setValue('master_id',  toVariant(account.id))
                    accountItem.setValue('serviceDate',toVariant(csgEndDate))
                    accountItem.setValue('event_id',   csg.value('event_id'))
                    accountItem.setValue('eventCSG_id', csgId)
                    accountItem.setValue('price',      toVariant(price))
                    accountItem.setValue('unit_id',    toVariant(tariff.unitId))
                    accountItem.setValue('amount',     toVariant(amount))
                    accountItem.setValue('sum',        toVariant(sum))
                    accountItem.setValue('tariff_id',  toVariant(tariff.id))
                    accountItem.setValue('service_id', toVariant(serviceId))
                    accountItem.setValue('VAT',        toVariant(tariff.vat))
                    accountItem.setValue('usedCoefficients', toVariant(usedCoefficients))
                    db.insertRecord(tableAccountItem, accountItem)
                    account.totalAmount += amount
                    account.totalSum    += sum
                    setEventCsgPayStatus(csgId, contractDescr.payStatusMask, CPayStatus.exposedBits)
                    return


    def getSemifinishedMesRefuseTypeId(self, financeId):
        return getRefuseTypeId(u'МЭС не выполнен полностью', True, financeId, True)


    def rejectAccountItemBySemifinishedMes(self, accountItem, financeId):
        if self.semifinishedMesRefuseTypeId is None:
            self.semifinishedMesRefuseTypeId = self.getSemifinishedMesRefuseTypeId(financeId)
        accountItem.setValue('date', QVariant(QDate.currentDate()))
        accountItem.setValue('refuseType_id', toVariant(self.semifinishedMesRefuseTypeId))
        accountItem.setValue('number', QVariant(u'Проверка МЭС'))


    def isTariffApplicable(self, tariff, eventId, resultId, mesLevel, tariffCategoryId, date, actualMKB=''):
        if tariff.tariffCategoryId and tariff.tariffCategoryId != tariffCategoryId:
            return False
        if not tariff.dateInRange(date):
            return False
        if tariff.resultId and tariff.resultId != resultId:
            return False
        if tariff.mesStatus and ( (tariff.mesStatus == 2) != (mesLevel == 2) ):
            return False

        sex = tariff.sex
        ageSelector = tariff.ageSelector
        eventTypeId = tariff.eventTypeId
        if sex or ageSelector or eventTypeId:
            eventRecord = self.getEventRecord(eventId)
            if eventRecord:
                if eventTypeId and eventTypeId != forceRef(eventRecord.value('eventType_id')):
                    return False
                if not date:
                    date = forceDate(eventRecord.value('execDate'))
                if sex or ageSelector:
                    clientId = forceRef(eventRecord.value('client_id'))
                    clientRecord = self.getClientRecord(clientId) if clientId else None
                    if clientRecord:
                        clientSex = forceInt(clientRecord.value('sex'))
                        if sex and sex != clientSex:
                            return False
                        if ageSelector:
                            if tariff.controlPeriod == 1:
                                date = firstYearDay(date)
                            elif tariff.controlPeriod == 2:
                                date = lastYearDay(date)
                            else:
                                pass

                            clientBirthDate = forceDate(clientRecord.value('birthDate'))
                            clientAge = calcAgeTuple(clientBirthDate, date)
                            if not clientAge:
                                clientAge = (0, 0, 0, 0)
                            if not checkAgeSelector(ageSelector, clientAge):
                                return False
                    else:
                        return False
            else:
                return False
        if tariff.MKB:
            if actualMKB:
                MKB = actualMKB
            else:
                eventMKB = self.mapEventIdToMKB.get(eventId, None)
                if eventMKB is None:
                    eventMKB = getEventDiagnosis(eventId)
                    self.mapEventIdToMKB[eventId] = eventMKB
                MKB = eventMKB
            if not tariff.matchMKB(MKB):
                return False
        return True



    def reexpose(self, progressDialog, contractDescr, accountFactory, idList):
        db = QtGui.qApp.db
        qvNull = QVariant()
        tableAccountItem = db.table('Account_Item')
        mapTariffIdToBatch = {}
        for oldId in idList:
            progressDialog.step()
            oldAccountItem = db.getRecord(tableAccountItem, '*', oldId)
            self.updateDocsPayStatus(oldAccountItem, contractDescr, CPayStatus.exposedBits)
            qvDate    = oldAccountItem.value('serviceDate')
            qvEventId = oldAccountItem.value('event_id')
            clientId  = forceRef(db.translate('Event', 'id', qvEventId, 'client_id'))
            tariffId  = forceRef(oldAccountItem.value('tariff_id'))
            batch = mapTariffIdToBatch.get(tariffId)
            if batch is None:
                batch = forceString(db.translate('Contract_Tariff', 'id', tariffId, 'batch'))
                mapTariffIdToBatch[tariffId] = batch
            account = accountFactory(clientId,
                                     forceDate(qvDate),
                                     forceRef(qvEventId),
                                     batch,
                                     1)
            newAccountItem = type(oldAccountItem)(oldAccountItem) # это я вывернулся чтобы не писать QtSql.QSqlRecord(oldAccountItem)
            newAccountItem.setValue('id',  qvNull) # или setNull достаточно?
            newAccountItem.setValue('master_id',  toVariant(account.id))
            newAccountItem.setValue('date', qvNull)
            newAccountItem.setValue('number', qvNull)
            newAccountItem.setValue('refuseType_id', qvNull)
            newAccountItem.setValue('reexposeItem_id', qvNull)
            newId = db.insertRecord(tableAccountItem, newAccountItem)
            oldAccountItem.setValue('reexposeItem_id', toVariant(newId))
            db.updateRecord(tableAccountItem, oldAccountItem)
            account.totalAmount += forceDouble(oldAccountItem.value('amount'))
            account.totalUet    += forceDouble(oldAccountItem.value('uet'))
            account.totalSum    += forceDouble(oldAccountItem.value('sum'))


    def updateDocsPayStatus(self,  accountItem, contractDescr, bits):
        updateDocsPayStatus(accountItem, contractDescr.payStatusMask, bits)


    def getEventRecord(self, eventId):
        # we need eventType_id, client_id, execDate
        return QtGui.qApp.db.getRecord('Event', ['eventType_id', 'client_id', 'execDate'], eventId)


    def getClientRecord(self, clientId):
        return self.clientRecordCache.get(clientId)


    def getMesDescr(self, mesId):
        descr = self.mapMesIdToDescr.get(mesId)
        if descr is None:
            db = QtGui.qApp.db
            descr = smartDict()
            record = db.getRecord('mes.MES',
                                  ('code',
                                   'minDuration',
                                   'maxDuration',
                                   'avgDuration',
                                   'ksg_id',
                                   'KSGNorm',
                                  ),
                                  mesId)
            if not record:
                record = db.dummyRecord()
            descr.code = forceString(record.value('code'))
            descr.serviceId = forceRef(db.translate('rbService', 'code', descr.code, 'id'))
            descr.minDuration = forceInt(record.value('minDuration'))
            descr.maxDuration = forceInt(record.value('maxDuration'))
            descr.avgDuration = forceDouble(record.value('avgDuration'))
            descr.ksgNorm     = forceInt(record.value('KSGNorm'))
            ksgId = forceRef(record.value('ksg_id'))
            record = db.getRecord('mes.MES_ksg',
                                  ('code',
                                   'type',
                                   'vk',
                                   'managementFactor',
                                  ),
                                  ksgId)
            if not record:
                record = db.dummyRecord()
            descr.ksgCode = forceString(record.value('code'))
            descr.ksgType = forceInt(record.value('type'))
            descr.ksgVk   = forceDouble(record.value('vk'))
            descr.ksgManagementFactor   = forceDouble(record.value('managementFactor'))
            self.mapMesIdToDescr[mesId] = descr
        return descr


    def getCsgDescr(self, csgCode):
        descr = self.mapCsgCodeToDescr.get(csgCode)
        if descr is None:
            db = QtGui.qApp.db
            descr = smartDict()
            table = db.table('mes.CSG')
            record = db.getRecordEx(table,
                                  ('minDuration',
                                   'maxDuration',
                                   'avgDuration',
                                  ),
                                  table['code'].eq(csgCode))
            if not record:
                record = db.dummyRecord()
            descr.code = csgCode
            descr.minDuration = forceInt(record.value('minDuration'))
            descr.maxDuration = forceInt(record.value('maxDuration'))
            descr.avgDuration = forceDouble(record.value('avgDuration'))
            self.mapCsgCodeToDescr[csgCode] = descr
        return descr


    def getMesCode(self, mesId):
        return self.getMesDescr(mesId).code


    def getMesService(self, mesId):
        return self.getMesDescr(mesId).serviceId


    def getMesNorm(self, mesId):
        return self.getMesDescr(mesId).ksgNorm


    def getMesAvgDuration(self, mesId):
        return self.getMesDescr(mesId).avgDuration


    def isEventLeavedWithDeath(self, eventId):
        db = QtGui.qApp.db
        table = db.table('vActionLeaved')

        record = db.getRecordEx(table,
                                'id',
                                [ table['event_id'].eq(eventId),
                                  table['deleted'].eq(0),
                                  db.joinOr( [ table['propOutcome'].like(u'%умер%'),
                                               table['propOutcome'].like(u'%смерть%')
                                             ]
                                           )
                                ]
                               )
        return bool(record)


    def getCoefficientAndReport(self, contractDescr, mesOrCsgDescr, duration, date, eventId, eventCsgId=None):
        mapGroupCodeToCoefficientTypes = {}
        eventDescr = CCoefficientTypeDescr.getEventPartDescr(date, eventId, eventCsgId)
        for coefficientType in contractDescr.coefficients.coefficientTypes:
            if coefficientType.applicable(eventDescr):
                groupCodes = coefficientType.usageGroupCodes
                for groupCode in groupCodes:
                    mapGroupCodeToCoefficientTypes.setdefault(groupCode, []).append(coefficientType)

        report = {}
        overallVal = 1.0
        for groupCode, coefficientTypes in mapGroupCodeToCoefficientTypes.iteritems():
            group = contractDescr.coefficients[groupCode]
            values = []
            groupReport = {}
            for coefficientType in coefficientTypes:
                value = group[coefficientType.code][date]
                value = coefficientType.algorithm(value, duration, mesOrCsgDescr.minDuration, mesOrCsgDescr.maxDuration, mesOrCsgDescr.avgDuration)
                values.append(value)
                groupReport[coefficientType.code] = value
            groupOp = group.groupOp
            if groupOp == 0: # сумма
                groupVal = sum(values)
            elif groupOp == 1: # произведение
                groupVal = reduce(lambda r,e: r*e, values, 1.0)
            elif groupOp == 2: # первый подходящий
#                groupVal = reduce(lambda r, e: r if r else e, [], 0.0)
                groupVal = values[0] if values else 0.0
            elif groupOp == 3: # max
                groupVal = max(values) if values else 0.0
            else:
                groupVal = 0.0
            groupVal = round( min(groupVal, group.maxLimit), 2)
            groupReport['__all__'] = groupVal
            report[groupCode] = groupReport
            overallVal *= groupVal

        return round(overallVal, 2), json.dumps(report, ensure_ascii=False) if report else None



# =================================================================================

CCircumstance = namedtuple('CCircumstance',
                           (
                            'date',
                            'personId',
                            'specialityId',
                            'specialityCode',
                            'prvsTypeId',
                            'prvsGroup',
                            'important',
                            'additionalServiceCode',
                            'originatedFromAction',
                            'servMade',
                            'incompl',
                            'orgId',
                           )
                          )


def getEventVisitsCircumstances(availabilityCounters, eventId, mesId):
    # получить сведения о визитах, выполенных в рамках выполенния МЭС
    # для случая когда визит оформен как Visit
    # возвращает список кортежей
    #    date,
    #    personId,
    #    specialityId,
    #    specialityCode,
    #    prvsTypeId,
    #    prvsGroup,
    #    important,
    #    additionalServiceCode,
    #    originatedFromAction,
    #    servMade,
    #    orgId,
    result = []
    db = QtGui.qApp.db
    currentOrgId = QtGui.qApp.currentOrgId()
    stmt = u'''
        SELECT DATE(Visit.date)   AS date,
        Visit.person_id           AS person_id,
        Person.speciality_id      AS speciality_id,
        Person.org_id             AS org_id,
        rbSpeciality.regionalCode AS specialityCode,
        mmv.visitType_id          AS prvsTypeIdRequired,
        mVT.id                    AS prvsTypeIdPresent,
        mVT.code NOT LIKE '%%доп' AS important,
        mmv.groupCode             AS prvsGroup,
        mmv.averageQnt            AS averageQnt,
        mmv.additionalServiceCode AS additionalServiceCode,
        Visit.id                  AS visit_id,
        (rbVisitType.code != mVT.code) AS visitTypeErr,
        DATE(Visit.date) < DATE(Event.setDate) AS outOfEvent
        FROM Visit
        INNER JOIN Event  ON Event.id = Visit.event_id
        INNER JOIN Client ON Client.id = Event.client_id
        LEFT JOIN Person ON Person.id  = Visit.person_id
        LEFT JOIN rbSpeciality ON rbSpeciality.id = Person.speciality_id
        LEFT JOIN rbVisitType  ON rbVisitType.id = Visit.visitType_id
        LEFT JOIN mes.mrbSpeciality AS mS   ON mS.regionalCode = rbSpeciality.regionalCode
        LEFT JOIN mes.MES_visit     AS mmv  ON mmv.speciality_id = mS.id AND mmv.deleted = 0
        LEFT JOIN mes.mrbVisitType  AS mVT  ON mVT.id = mmv.visitType_id
        WHERE Visit.deleted = 0
        AND Visit.event_id = %d
        AND mmv.master_id = %d
        AND isSexAndAgeSuitable(Client.sex,
                                Client.birthDate,
                                mmv.sex,
                                -- где ты, mmv.age :(
                                CONCAT( IF(mmv.begAgeUnit BETWEEN 1 AND 4,
                                           CONCAT( IF(mmv.minimumAge='','0',mmv.minimumAge),
                                                   SUBSTR('ДНМГ', mmv.begAgeUnit, 1)
                                                 ),
                                           ''
                                          ),
                                        '-',
                                        IF(mmv.endAgeUnit BETWEEN 1 AND 4,
                                           CONCAT( IF(mmv.maximumAge='','0',mmv.maximumAge),
                                                   SUBSTR('ДНМГ', mmv.endAgeUnit, 1)
                                                 ),
                                           ''
                                          )
                                      ),
                                CASE  mmv.controlPeriod
                                    WHEN 1 -- Конец текущего года
                                        THEN CONCAT( YEAR(Event.execDate),
                                                     '-12-31'
                                                   )
                                    WHEN 2 -- Конец предыдущего года
                                        THEN CONCAT( YEAR(Event.execDate)-1,
                                                     '-12-31'
                                                   )
                                    ELSE -- 0 and fallback
                                        Event.execDate
                                END
                               )
        ORDER BY visitTypeErr, (mVT.code LIKE '%%доп'), mmv.groupCode, Visit.date
    ''' % (eventId, mesId)
    query = db.query(stmt)
    countedVisits = set()
    incompl = 5 # услуга оказана в полном объеме
    while query.next():
        record = query.record()
        visitId = forceRef(record.value('visit_id'))
        if visitId not in countedVisits:
            date = forceDate(record.value('date'))
            personId = forceRef(record.value('person_id'))
            specialityId = forceRef(record.value('speciality_id'))
            orgId = forceRef(record.value('org_id'))
            specialityCode = forceInt(record.value('specialityCode'))
            prvsTypeId = forceInt(record.value('prvsTypeIdRequired'))
            prvsGroup = forceInt(record.value('prvsGroup')) or 1
            important  = forceBool(record.value('important'))
            additionalServiceCode = forceInt(record.value('additionalServiceCode'))
            averageQnt = forceDouble(record.value('averageQnt'))
            outOfEvent = forceBool(record.value('outOfEvent'))
            availabilityKey = (prvsGroup, additionalServiceCode)
            available = availabilityCounters.get(availabilityKey, averageQnt)
            if available > 0:
                availabilityCounters[availabilityKey] = available-1
                if outOfEvent:
                    servMade = 5
                elif orgId != currentOrgId:
                    servMade = 3
#                    elif available <= 0.9999:
#                        servMade = 6
                else:
                    servMade = 1
                result.append(CCircumstance(date, personId, specialityId, specialityCode, prvsTypeId, prvsGroup, important, additionalServiceCode, False, servMade, incompl, orgId))
                countedVisits.add(visitId)
    return result


def getEventActionsCircumstances(availabilityCounters, eventId, mesId, onlyVisitLike):
    # получить сведения о визитах, выполенных в рамках выполенния МЭС
    # для случая когда визит оформен как Action (осмотр)
    # возвращает список кортежей
    #    date,
    #    personId,
    #    specialityId,
    #    specialityCode,
    #    prvsTypeId,
    #    prvsGroup,
    #    important
    #    additionalServiceCode,
    #    originatedFromAction,
    #    servMade,
    #    orgId
    result = []
    db = QtGui.qApp.db
    currentOrgId = QtGui.qApp.currentOrgId()
    stmt = u'''
        SELECT
        Action.id                 AS action_id,
        Action.status             AS actionStatus,
        Action.org_id             AS org_id,
        DATE(COALESCE(Action.endDate, Action.directionDate, Event.execDate)) AS date,
        Person.id                 AS person_id,
        Person.speciality_id      AS speciality_id,
        IFNULL(NULLIF(mS.regionalCode, ''), rbSpeciality.regionalCode) AS specialityCode,
        mmv.visitType_id          AS prvsTypeIdRequired,
        mVT.id                    AS prvsTypeIdPresent,
        mmv.groupCode             AS prvsGroup,
        mVT.code NOT LIKE '%%доп' AS important,
        mmv.averageQnt            AS averageQnt,
        mmv.additionalServiceCode AS additionalServiceCode,
        mmv.serviceCode           AS serviceCode,
        (IFNULL(rbSpeciality.regionalCode,'') != mS.regionalCode) AS specialityErr,
        IF(mmv.visitType_id=4, 0, 1) AS visitTypeErr,
        DATE(Action.endDate) < DATE(Event.setDate) AS outOfEvent
        FROM Action
        INNER JOIN Event ON Event.id = Action.event_id
        INNER JOIN Client ON Client.id = Event.client_id
        INNER JOIN ActionType ON ActionType.id = Action.actionType_id
        INNER JOIN rbService ON rbService.id = ActionType.nomenclativeService_id
        INNER JOIN mes.MES_visit    AS mmv  ON rbService.code LIKE CONCAT(mmv.serviceCode,'%%') AND mmv.deleted=0
        INNER JOIN mes.mrbVisitType AS mVT  ON mVT.id = mmv.visitType_id
        LEFT JOIN mes.mrbSpeciality AS mS   ON mS.id = mmv.speciality_id
        LEFT JOIN Person ON Person.id  = IFNULL(Action.person_id, Event.execPerson_id)
        LEFT JOIN rbSpeciality ON rbSpeciality.id = Person.speciality_id
        WHERE Action.deleted = 0
        AND Action.event_id = %(eventId)d
        AND (%(serviceTypeCond)s)
        AND mmv.master_id = %(mesId)d
        AND isSexAndAgeSuitable(Client.sex,
                                Client.birthDate,
                                mmv.sex,
                                -- где ты, mmv.age :(
                                CONCAT( IF(mmv.begAgeUnit BETWEEN 1 AND 4,
                                           CONCAT( IF(mmv.minimumAge='','0',mmv.minimumAge),
                                                   SUBSTR('ДНМГ', mmv.begAgeUnit, 1)
                                                 ),
                                           ''
                                          ),
                                        '-',
                                        IF(mmv.endAgeUnit BETWEEN 1 AND 4,
                                           CONCAT( IF(mmv.maximumAge='','0',mmv.maximumAge),
                                                   SUBSTR('ДНМГ', mmv.endAgeUnit, 1)
                                                 ),
                                           ''
                                          )
                                      ),
                                CASE  mmv.controlPeriod
                                    WHEN 1 -- Конец текущего года
                                        THEN CONCAT( YEAR(Event.execDate),
                                                     '-12-31'
                                                   )
                                    WHEN 2 -- Конец предыдущего года
                                        THEN CONCAT( YEAR(Event.execDate)-1,
                                                     '-12-31'
                                                   )
                                    ELSE -- 0 and fallback
                                        Event.execDate
                                END
                               )
        ORDER BY specialityErr, visitTypeErr, (mVT.code LIKE '%%доп'), mmv.groupCode, Action.endDate
    ''' % {'eventId':eventId,
           'mesId':mesId,
           'serviceTypeCond': (u'ActionType.serviceType in (%d,%d)' % (CActionServiceType.initialInspection, CActionServiceType.reinspection)
                               if onlyVisitLike
                               else '1')
          }

    query = db.query(stmt)
#    countedActions = set()
    while query.next():
        record = query.record()
        date = forceDate(record.value('date'))
        personId = forceRef(record.value('person_id'))
        specialityId = forceRef(record.value('speciality_id'))
        specialityCode = forceInt(record.value('specialityCode'))
        prvsTypeId = forceInt(record.value('prvsTypeIdRequired'))
        prvsGroup = forceInt(record.value('prvsGroup')) or 1
        important = forceBool(record.value('important'))
#        serviceCode = forceString(record.value('serviceCode'))
        additionalServiceCode = forceInt(record.value('additionalServiceCode'))
        averageQnt = forceDouble(record.value('averageQnt'))
        outOfEvent = forceBool(record.value('outOfEvent'))
        actionStatus = forceInt(record.value('actionStatus')) # 0-Начато, 1-Ожидание, 2-Закончено, 3-Отменено, 4-Без результата, 5-Назначено, 6-Отказ
        orgId = forceRef(record.value('org_id')) or currentOrgId

        availabilityKey = (prvsGroup, additionalServiceCode)
        available = availabilityCounters.get(availabilityKey, averageQnt)
        incompl = 5 # услуга оказана в полном объеме

        if available > 0:
            availabilityCounters[availabilityKey] = available-1
            if actionStatus in (CActionStatus.finished, CActionStatus.withoutResult): # закончено, без результата
                if outOfEvent:
                    servMade = 5
                    incompl = 4 # ранее проведённые услуги в пределах установленных сроков
                elif orgId != currentOrgId:
                    servMade = 3
#                elif available <= 0.9999:
#                    servMade = 6
                else:
                    servMade = 1
            elif actionStatus == CActionStatus.refused: # отказ пациента
                servMade = 4
                incompl = 1 # документированный отказ больного
            elif actionStatus == CActionStatus.canceled: # отменено
                servMade = 4
                incompl = 3 # прочие причины (умер, переведён в другое отделение и пр.)
            else:
                if orgId != currentOrgId:
                    servMade = 3
                else:
                    servMade = 4
            result.append(CCircumstance(date, personId, specialityId, specialityCode, prvsTypeId, prvsGroup, important, additionalServiceCode, True, servMade, incompl, orgId))
#            countedActions.add((actionId, additionalServiceCode))
    return result


def getInappropriateCircumstances(availabilityCounters, eventId, mesId):
    # получить сведения о позициях (визитах), которые включены в МЭС но НЕ подходят по полу или возрасту
    # для случая когда визит оформен как Action (осмотр)
    # возвращает список кортежей
    #    date,
    #    personId,
    #    specialityId,
    #    specialityCode,
    #    prvsTypeId,
    #    prvsGroup,
    #    important
    #    additionalServiceCode,
    #    originatedFromAction,
    #    servMade,
    #    orgId
    result = []
    db = QtGui.qApp.db
    currentOrgId = QtGui.qApp.currentOrgId()

    stmt = u'''
        SELECT
            Person.id                 AS person_id,
            Person.speciality_id      AS speciality_id,
            Person.org_id             AS org_id,
            IFNULL(NULLIF(mS.regionalCode, ''), rbSpeciality.regionalCode) AS specialityCode,
            mmv.visitType_id          AS prvsTypeIdRequired,
            mVT.id                    AS prvsTypeIdPresent,
            mVT.code NOT LIKE '%%доп' AS important,
            mmv.groupCode             AS prvsGroup,
            mmv.averageQnt            AS averageQnt,
            mmv.additionalServiceCode AS additionalServiceCode,
            DATE(Event.execDate)      AS date
        FROM Event
        INNER JOIN Client ON Client.id = Event.client_id
        INNER JOIN Person ON Person.id = Event.execPerson_id
        LEFT JOIN rbSpeciality ON rbSpeciality.id = Person.speciality_id,

        mes.MES_visit AS mmv
        INNER JOIN mes.mrbVisitType AS mVT  ON mVT.id = mmv.visitType_id
        LEFT JOIN mes.mrbSpeciality AS mS   ON mS.id = mmv.speciality_id
        WHERE Event.id = %(eventId)d
          AND mmv.master_id = %(mesId)d
          AND mmv.deleted = 0
          AND NOT isSexAndAgeSuitable(Client.sex,
                                      Client.birthDate,
                                      mmv.sex,
                                      -- где ты, mmv.age :(
                                      CONCAT( IF(mmv.begAgeUnit BETWEEN 1 AND 4,
                                                 CONCAT( IF(mmv.minimumAge='','0',mmv.minimumAge),
                                                         SUBSTR('ДНМГ', mmv.begAgeUnit, 1)
                                                       ),
                                                 ''
                                                ),
                                              '-',
                                              IF(mmv.endAgeUnit BETWEEN 1 AND 4,
                                                 CONCAT( IF(mmv.maximumAge='','0',mmv.maximumAge),
                                                         SUBSTR('ДНМГ', mmv.endAgeUnit, 1)
                                                       ),
                                                 ''
                                                )
                                            ),
                                      CASE  mmv.controlPeriod
                                          WHEN 1 -- Конец текущего года
                                              THEN CONCAT( YEAR(Event.execDate),
                                                           '-12-31'
                                                         )
                                          WHEN 2 -- Конец предыдущего года
                                              THEN CONCAT( YEAR(Event.execDate)-1,
                                                           '-12-31'
                                                         )
                                          ELSE -- 0 and fallback
                                              Event.execDate
                                      END
                                     )
        ORDER BY mmv.groupCode
    ''' % {'eventId':eventId,
           'mesId':mesId,
          }

    query = db.query(stmt)
#    countedActions = set()
    while query.next():
        record = query.record()
        date = forceDate(record.value('date'))
        personId = forceRef(record.value('person_id'))
        specialityId = forceRef(record.value('speciality_id'))
        specialityCode = forceInt(record.value('specialityCode'))
        prvsTypeId = forceInt(record.value('prvsTypeIdRequired'))
        prvsGroup = forceInt(record.value('prvsGroup')) or 1
        important = forceBool(record.value('important'))
        additionalServiceCode = forceInt(record.value('additionalServiceCode'))
        averageQnt = forceDouble(record.value('averageQnt'))
        orgId = forceRef(record.value('org_id')) or currentOrgId

        availabilityKey = (prvsGroup, additionalServiceCode)
        available = availabilityCounters.get(availabilityKey, averageQnt)
        incompl = 5 # услуга оказана в полном объеме

        if available > 0:
            servMade = 6 # как прошено
            incompl  = 3 # прочие причины (умер, переведён в другое отделение и пр.)
            availabilityCounters[availabilityKey] = available-1
            result.append(CCircumstance(date, personId, specialityId, specialityCode, prvsTypeId, prvsGroup, important, additionalServiceCode, True, servMade, incompl, orgId))
    return result


def filterUniqueCircumstances(circumstances, distinctByGroup=False):
    circumstances.sort(key=lambda circumstance: (circumstance.date,
                                                 circumstance.personId,
                                                 circumstance.prvsTypeId,
                                                 circumstance.prvsGroup,
                                                 circumstance.additionalServiceCode,
                                                 circumstance.servMade,
                                                 circumstance.originatedFromAction,
                                                 -circumstance.incompl # сначала - бОльшие значения
                                                ))
    result = []
    producedKeys = []
    for circumstance in circumstances:
        if distinctByGroup:
            key = circumstance.date, circumstance.personId, circumstance.prvsTypeId, circumstance.prvsGroup, circumstance.additionalServiceCode
        else:
            key = circumstance.date, circumstance.personId, circumstance.additionalServiceCode
        if key not in producedKeys:
            producedKeys.append(key)
            result.append(circumstance)
    return result


def getMesAmount(eventId, mesId):
    availabilityCounters = {}
    circumstances  = getEventVisitsCircumstances(availabilityCounters, eventId, mesId)
    circumstances += getEventActionsCircumstances(availabilityCounters, eventId, mesId, True)
    circumstances = filterUniqueCircumstances(circumstances)

    result = 0
    for circumstance in circumstances:
        if circumstance.important and circumstance.servMade == 1:
            result += 1
    return result


def evalPriceForKrasnodarA13(contractDescr,
                             tariff,
                             clientId,
                             eventId,
                             eventTypeId,
                             eventBegDate,
                             eventEndDate):

    def getActionCount(eventId, serviceMasks):
        db = QtGui.qApp.db
        tableAction = db.table('Action')
        tableActionType = db.table('ActionType')
        tableRBService = db.table('rbService')
        table = tableAction.innerJoin(tableActionType, tableActionType['id'].eq(tableAction['actionType_id']))
        table = table.innerJoin(tableRBService, tableRBService['id'].eq(tableActionType['nomenclativeService_id']))
        cond = [ tableAction['deleted'].eq(0),
                 tableAction['status'].eq(2),
                 tableAction['event_id'].eq(eventId),
                 db.joinOr([ tableRBService['code'].like(mask) for mask in serviceMasks ])
               ]
        result = db.getCount(table, where=cond)
        return result

    # 1. Определяем типы коэффициентов для тарификации, значения которых будут заданы в Договоре:
    # ДетиДо4 (1.03)
    # Д3безОВ (0.2)
    # Д3ОВ (0.8)
    # Длит (1.2)
    # ДлитОВ (1.3)
    # ОВ (1.1)

    # 2. Алгоритм тарификации:
    # Тарифицируем Событие по коду МЭС.
    # Базовый тариф (БТ) берем в соответствии с фактической длительностью События
    # (цена - длительность меньше "Второй тариф с",
    # Сумма Второго тарифа - длительность меньше "Третий тариф с",
    # Сумма Третьего тарифа - длительность больше "Третий тариф с")

    price  = tariff.price
    minDuration = tariff.frags[-2][0] if len(tariff.frags)>=3 else 3
    maxDuration = tariff.frags[-3][0] if len(tariff.frags)>=3 else 35
    duration = getEventDuration(eventBegDate, eventEndDate, wpSevenDays, eventTypeId)
    mesServiceMask = getEvenMesServiceMask(eventTypeId)
    mesServiceMask = ''.join(re.findall(r'[\w.,;]+', mesServiceMask, re.UNICODE))
    leftMaskSet, rightMaskSet = (mesServiceMask.split(';')+['']*2)[:2]

    # 2.1 Если длительность меньше "Второй тариф с",
    # и в Событии нет выполненных Действий с номенклатурным кодом соответствующим маске,
    # определенной в Типе События (левая часть маски до символа ";"),
    # то Тариф=БТ*Д3безОВ.
    # Если таковые Действия есть, то Тариф=БТ*Д3ОВ.
    # В противном случаи Тариф=БТ
    if duration<minDuration:
        if getActionCount(eventId, [mask + '%' for mask in leftMaskSet.split(',')]) == 0:
            price *= contractDescr.coefficients[0][u'Д3безОВ'][eventEndDate]
        else:
            price *= contractDescr.coefficients[0][u'Д3ОВ'][eventEndDate]

    # 2.2 Если региональный код Тип мед.помощи События не равны 11 или 12 - расчет закончен.
    if getEventAidTypeRegionalCode(eventTypeId) not in ('11', '12'):
        return price

    # В противном случаи последовательно корректируем Тариф:
    # 2.3 если возраст пациента на дату начала Событие меньше 5 лет (до 4 включительно),
    # то Тариф=Тариф*ДетиДо4
    clientRecord = QtGui.qApp.db.getRecord('Client', ['sex', 'birthDate'], clientId) if clientId else None
    if clientRecord:
        clientBirthDate = forceDate(clientRecord.value('birthDate'))
        age = calcAgeInYears(clientBirthDate, eventEndDate)
        if age<5:
            price *= contractDescr.coefficients[0][u'ДетиДо4'][eventEndDate]

    # 2.4 Если длительность меньше "Тариф три с"
    # и в Событии нет выполненных Действий с номенклатурным кодом соответствующим списку,
    # определенном в Типе События (правая часть маски после символа ";" /строгое соответствие коду/),
    # то расчет закончен
    # Если таковые Действия есть, то Тариф=Тариф*ОВ и расчет закончен
    if duration<maxDuration:
        if getActionCount(eventId, rightMaskSet.split(',')) == 0:
            return price
        else:
            price *= contractDescr.coefficients[0][u'ОВ'][eventEndDate]
            return price

    # 2.5 Если длительность больше или равно "Тариф три с"
    # и в Событии нет выполненных Действий с номенклатурным кодом соответствующим списку,
    # определенном в Типе События (правая часть маски после символа ";" /строгое соответствие коду/),
    # то Тариф=Тариф*Длит расчет закончен
    # Если таковые Действия есть, то Тариф=Тариф*ДлитОВ и расчет закончен
    if duration>=maxDuration:
        if getActionCount(eventId, rightMaskSet.split(',')) == 0:
            price *= contractDescr.coefficients[0][u'Длит'][eventEndDate]
            return price
        else:
            price *= contractDescr.coefficients[0][u'ДлитОВ'][eventEndDate]
            return price
    return price


def evalPriceForMurmansk2015Hospital(contractDescr,
                                     tariff,
                                     eventId,
                                     eventTypeId,
                                     eventBegDate,
                                     eventEndDate,
                                     mesId,
                                     shortHospitalisation,
                                     level,  # 0:прерван,1:частично,2:полностью
                                     mesDescr = None
                                    ):


    def getMesDescr(mesId):
        db = QtGui.qApp.db
        descr = smartDict()
        record = db.getRecord('mes.MES',
                              ('code',
                               'minDuration',
                               'maxDuration',
                               'avgDuration',
                               'ksg_id',
                               'KSGNorm',
                              ),
                              mesId)
        if not record:
            record = db.dummyRecord()
        descr.code = forceString(record.value('code'))
        descr.serviceId = forceRef(db.translate('rbService', 'code', descr.code, 'id'))
        descr.minDuration = forceInt(record.value('minDuration'))
        descr.maxDuration = forceInt(record.value('maxDuration'))
        descr.avgDuration = forceDouble(record.value('avgDuration'))
        descr.ksgNorm     = forceInt(record.value('KSGNorm'))
        ksgId = forceRef(record.value('ksg_id'))
        record = db.getRecord('mes.MES_ksg',
                              ('code',
                               'type',
                               'vk',
                               'managementFactor',
                              ),
                              ksgId)
        if not record:
            record = db.dummyRecord()
        descr.ksgCode = forceString(record.value('code'))
        descr.ksgType = forceInt(record.value('type'))
        descr.ksgVk   = forceDouble(record.value('vk'))
        descr.ksgManagementFactor   = forceDouble(record.value('managementFactor'))
        return descr

    def isEventLeavedWithDeath(eventId):
        db = QtGui.qApp.db
        table = db.table('vActionLeaved')

        record = db.getRecordEx(table,
                                'id',
                                [ table['event_id'].eq(eventId),
                                  table['deleted'].eq(0),
                                  db.joinOr( [ table['propOutcome'].like(u'%умер%'),
                                               table['propOutcome'].like(u'%смерть%')
                                             ]
                                           )
                                ]
                               )
        return bool(record)

    # 0006322: Мурманск. Новые правила тарификации Стационаров на 2015 г.
    #- Стоимость одного случая лечения (применяется по-умолчанию для всех случаев)
    #  рассчитывается по формуле:
    #
    #  SКСГсл = БС × Кзатр × Купр × КурСтац
    #
    #  где БС - базовая ставка (Цена тарифа в договоре),
    #      Кзатр - весовой коэффициент (поле vk в MES_ksg),
    #      Купр - управленческий коэффициент (поле managementFactor в MES_ksg),
    #      КурСтац - коэффициент Стационара (поле regionalTariffRegulationFactor в Contract)

    duration = getEventDuration(eventBegDate, eventEndDate, wpSevenDays, eventTypeId)

    if not mesDescr:
        mesDescr = getMesDescr(mesId)

    price = tariff.price * mesDescr.ksgVk * mesDescr.ksgManagementFactor * contractDescr.regionalTariffRegulationFactor

    #- Если длительность события < minDuration МЭС (кроме
    #  событий со свойством действия Исход госпитализации - умер
    #  и МЭС, имеющих КСГ с типом 3 Хир.)
    #
    #- Если длительность события < avgDuration МЭС
    #
    #- Если длительность события меньше суток и свойство действия Исход госпитализации - умер
    #  Тогда расчет по формуле:
    #
    #  SКСГсл = БС × Кзатр × Купр × КурСтац × (Длф / Длср)
    #  где Длф - фактическая длительность события, Длср - avgDuration МЭС

    isDeath = isEventLeavedWithDeath(eventId)
    if (    duration < mesDescr.minDuration and not ( isDeath or mesDescr.ksgType == 3 )
         or (level == 0 and duration < mesDescr.avgDuration)
         or shortHospitalisation and isDeath
       ):
        price *= max(1, duration)/mesDescr.avgDuration if mesDescr.avgDuration>0 else 0.0
    #
    #- Если длительность события > avgDuration МЭС в 2 раза.
    #  Тогда расчет по формуле:
    #
    #  SКСГсл = БС × Кзатр × Купр × КурСтац × (1 + (Длф – Длср) / Длср × 0,35)
    #
    elif ( duration > mesDescr.avgDuration * 2
         ):
        if mesDescr.avgDuration>0:
            price *= (1.0 + (duration-mesDescr.avgDuration)/mesDescr.avgDuration * 0.35)
        else:
            price = 0.0

    #Также необходимо сделать поле в Договоре "Поправочный коэффициент".
    #Итоговая стоимость случая лечения будет рассчитываться так:
    #
    #  SКСГсл = SКСГсл × Кпопр. Если Кпопр=0, тогда Итоговая стоимость случая лечения = SКСГсл.

    return price
