# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QEvent, QModelIndex, QObject, QSize, SIGNAL

from library.AgeSelector import checkAgeSelector, parseAgeSelector
from library.TreeModel import CDBTreeItem, CTreeItemWithId, CTreeModel
from library.Utils import forceInt, forceRef, forceString


class CActionTemplateTreeItem(CDBTreeItem):
    def __init__(self, parent, name, id, model, actionId, filter):
        CDBTreeItem.__init__(self, parent, name, id, model)
        CTreeItemWithId.__init__(self, parent, name, id)
        self._actionId = actionId
        self._filter = filter


    def flags(self):
        if self._actionId or not self._filter.removeEmptyNodes:
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable
        else:
            return Qt.ItemIsEnabled


    def isEmpty(self):
        if self._actionId is not None:
            return False
        for child in self.items():
            if not child.isEmpty():
                return False
        return True


    def loadChildren(self):
        result = []
        db = QtGui.qApp.db
        table = db.table('ActionTemplate')

        tableAction = db.table('Action')
        cond = [table['group_id'].eq(self._id),
                table['deleted'].eq(0)]
        if self._filter.actionTypeIdList:
            cond.append(db.joinOr([tableAction['actionType_id'].inlist(self._filter.actionTypeIdList), table['action_id'].isNull()]))
        if self._filter.personId:
            cond.append(db.joinOr([table['owner_id'].eq(self._filter.personId), table['owner_id'].isNull()]))
        else:
            cond.append(table['owner_id'].isNull())
        if self._filter.specialityId:
            cond.append(db.joinOr([table['speciality_id'].eq(self._filter.specialityId), table['speciality_id'].isNull()]))
        else:
            cond.append(table['speciality_id'].isNull())
        if self._filter.clientSex:
            cond.append(table['sex'].inlist([self._filter.clientSex, 0]))

        query = db.query(db.selectStmt(table.leftJoin(tableAction, tableAction['id'].eq(table['action_id'])),
                         [table['id'].name(),
                          table['name'].name(),
                          table['action_id'].name(),
                          table['age'].name()], where=cond, order=table['name'].name()))
        while query.next():
            record = query.record()
            age = forceString(record.value('age'))
            if age and self._filter.clientAge:
                if not checkAgeSelector(parseAgeSelector(age), self._filter.clientAge):
                    continue
            id   = forceInt(record.value('id'))
            name = forceString(record.value('name'))
            actionId = forceRef(record.value('action_id'))
            child = CActionTemplateTreeItem(self, name, id, self.model, actionId, self._filter)
            if not self._filter.removeEmptyNodes or not child.isEmpty():
                result.append(child)
        return result



class CActionTemplateRootTreeItem(CActionTemplateTreeItem):
    def __init__(self, model, filter):
        CActionTemplateTreeItem.__init__(self, None, u'-', None, model, None, filter)



class CActionTemplateModel(CTreeModel):
    class CFilter:
        def __init__(self, actionTypeId, personId, specialityId, clientSex, clientAge, removeEmptyNodes):
            db = QtGui.qApp.db
            if actionTypeId:
                tableActionTypeTestator = db.table('ActionType_Testator')
                actionTypeIdList = db.getIdList(tableActionTypeTestator,
                                                idCol = 'testator_id',
                                                where = tableActionTypeTestator['master_id'].eq(actionTypeId)
                                               )
                actionTypeIdList.append(actionTypeId)
                actionTypeIdList = db.getTheseAndParents('ActionType', 'group_id', actionTypeIdList)
            else:
                actionTypeIdList = []
            self.actionTypeIdList = actionTypeIdList
            self.personId = personId
            self.specialityId = specialityId
            self.clientSex = clientSex
            self.clientAge = clientAge
            self.removeEmptyNodes = removeEmptyNodes

    def __init__(self, parent, actionTypeId, personId, specialityId, clientSex, clientAge, removeEmptyNodes=True):
        CTreeModel.__init__(self, parent, CActionTemplateRootTreeItem(self, CActionTemplateModel.CFilter(actionTypeId, personId, specialityId, clientSex, clientAge, removeEmptyNodes)))
        self.rootItemVisible = False


    def setFilter(self, actionTypeId, personId, specialityId, clientSex, clientAge, removeEmptyNodes=True):
        self.getRootItem()._filter = CActionTemplateModel.CFilter(actionTypeId, personId, specialityId, clientSex, clientAge, removeEmptyNodes)
        self.update()


    def filter(self):
        return self.getRootItem()._filter


    def update(self):
        self.getRootItem().removeChildren()
        self.reset()


    def isEmpty(self):
        return self.getRootItem().isEmpty()



#    def headerData(self, section, orientation, role):
#        if role == Qt.DisplayRole:
#            return QVariant(u'Шаблоны свойств')
#        return QVariant()


class CActionTemplateTreeView(QtGui.QTreeView):
    def __init__(self, parent):
        QtGui.QTreeView.__init__(self, parent)
        self.header().hide()
        self.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.setMinimumHeight(300)
        self.setMinimumWidth(600)
        self.connect(self, SIGNAL('expanded(QModelIndex)'), self.onExpanded)
#        self.connect(self, SIGNAL('collapsed(QModelIndex)'), self.onCollapsed)
        self.searchString = ''
        self.searchParent = None


    def setModel(self, model):
        QtGui.QTreeView.setModel(self, model)
        self.expandAll()


    def setRootIndex(self, index):
        pass


    def setRealRootIndex(self, index):
        QtGui.QTreeView.setRootIndex(self, index)
#        self.expandAll()


    def onExpanded(self, index):
        self.scrollTo(index, QtGui.QAbstractItemView.PositionAtTop)


    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Left or event.key() == Qt.Key_Minus:
            current = self.currentIndex()
            if self.isExpanded(current) and self.model().rowCount(current):
                self.collapse(current)
            else:
                self.setCurrentIndex(current.parent())
                current = self.currentIndex()
                self.collapse(current)
                self.scrollTo(current, QtGui.QAbstractItemView.PositionAtTop)
            event.accept()
            return
        if event.key() == Qt.Key_Enter or event.key() == Qt.Key_Return:
            event.accept()
            self.emit(SIGNAL('doubleClicked(QModelIndex)'), self.currentIndex())
            return
        return QtGui.QTreeView.keyPressEvent(self, event)



class CActionTemplatePopup(QtGui.QFrame):
    __pyqtSignals__ = ('templateSelected(int)',
                       'closePopup()',
                      )

    def __init__(self, parent = None):
        QtGui.QFrame.__init__(self, parent, Qt.Popup)
        self.setFrameShape(QtGui.QFrame.StyledPanel)
        self.setAttribute(Qt.WA_WindowPropagation)
        self.treeView=CActionTemplateTreeView(self)
        self.widgetLayout = QtGui.QVBoxLayout(self)
        self.widgetLayout.setMargin(0)
        self.widgetLayout.setSpacing(0)
        self.widgetLayout.addWidget(self.treeView)
        self.connect(self.treeView, SIGNAL('doubleClicked(QModelIndex)'), self.on_doubleClicked)
        self.setLayout(self.widgetLayout)
        self.treeView.setFocus()


    def on_doubleClicked(self, index):
        item = index.internalPointer()
        self.emit(SIGNAL('templateSelected(int)'), item.id())
        self.close()


    def mousePressEvent(self, event):
        p = self.parentWidget()
        if p!=None:
            rect = p.rect()
            rect.moveTo(p.mapToGlobal(rect.topLeft()))
            if (rect.contains(event.globalPos()) or self.rect().contains(event.pos())):
                self.setAttribute(Qt.WA_NoMouseReplay)
        QtGui.QFrame.mousePressEvent(self, event)


    def mouseReleaseEvent(self, event):
#        self.emit(SIGNAL('resetButton()'))
        self.parent().mouseReleaseEvent(event)
        pass


#    def event(self, event):
#        if event.type()==QEvent.KeyPress:
#            if event.key()==Qt.Key_Escape:
#                self.dateChanged = False
#        return QFrame.event(self, event)

    def hideEvent(self, event):
        self.emit(SIGNAL('closePopup()'))


class CActionTemplateChooseButton(QtGui.QPushButton):
    __pyqtSignals__ = ('templateSelected(int)',
                      )

    def __init__(self,  parent = None):
        QtGui.QPushButton.__init__(self, parent)
        self.popup = None
#        self.setMenu(QtGui.QMenu(self))
#        self.setMenu(CActionTemplateChooseMenu(self))
        self.connect(self, SIGNAL('pressed()'), self._popupPressed)
        self._model = None


    def setModel(self, model):
        self._model = model
        self.setEnabled( not self._model.isEmpty() )


    def initStyleOption(self, option):
        QtGui.QPushButton.initStyleOption(self, option)
        option.features |= QtGui.QStyleOptionButton.HasMenu

    def sizeHint(self):
        self.ensurePolished()
        w = 0
        h = 0
        opt = QtGui.QStyleOptionButton()
        self.initStyleOption(opt)

        if not opt.icon.isNull():
            ih = opt.iconSize.height()
            iw = opt.iconSize.width() + 4
            w += iw
            h = max(h, ih)

        if opt.features & QtGui.QStyleOptionButton.HasMenu:
            w += self.style().pixelMetric(QtGui.QStyle.PM_MenuButtonIndicator, opt, self)

        empty = opt.text.isEmpty()
        fm = self.fontMetrics()
        sz = fm.size(Qt.TextShowMnemonic, "XXXX" if empty else opt.text)
        if not empty or not w:
            w += sz.width()
        if not empty or not h:
            h = max(h, sz.height())
        result = self.style().sizeFromContents(QtGui.QStyle.CT_PushButton, opt, QSize(w, h), self).expandedTo(QtGui.qApp.globalStrut())
        return result


    def paintEvent(self, paintEvent):
        p = QtGui.QStylePainter(self)
        option = QtGui.QStyleOptionButton()
        self.initStyleOption(option)
        p.drawControl(QtGui.QStyle.CE_PushButton, option)


    def _popupPressed(self):
        self.setDown(True)
        if not self.popup:
            self.popup = CActionTemplatePopup(self)
#            m = CActionTemplateModel(self, None, None, None, 0, None, False)
        self.popup.treeView.setModel(self._model)
        pos = self.rect().bottomLeft()
        pos = self.mapToGlobal(pos)
        size=self.popup.sizeHint()
        screen = QtGui.qApp.desktop().availableGeometry(pos)
        pos.setX( max(min(pos.x(), screen.right()-size.width()), screen.left()) )
        pos.setY( max(min(pos.y(), screen.bottom()-size.height()), screen.top()) )
        self.popup.move(pos)
        self.connect(self.popup, SIGNAL('templateSelected(int)'), self.on_templateSelected)
        self.connect(self.popup, SIGNAL('closePopup()'), self.on_closePopup)
        self.popup.show()

    def on_templateSelected(self, templateId):
        self.emit(SIGNAL('templateSelected(int)'), templateId)


    def on_closePopup(self):
        self.setDown(False)


# ##############################################################################


class CActionTemplateComboBox(QtGui.QComboBox):
    def __init__(self, parent):
        QtGui.QComboBox.__init__(self, parent)
        self.SizeAdjustPolicy(QtGui.QComboBox.AdjustToMinimumContentsLength)
        self.setModelColumn(0)
        self._model = CActionTemplateModel(self, None, None, None, 0, None, False)
        self.setModel(self._model)
        self._popupView = CActionTemplateTreeView(self)
        self._popupView.setObjectName('popupView')
        self._popupView.setModel(self._model)
        self.setView(self._popupView)
        self._popupView.installEventFilter(self)
        self._popupView.viewport().installEventFilter(self)
        QObject.connect(self._popupView, SIGNAL('activated(QModelIndex)'), self.on_ItemActivated)
        QObject.connect(self._popupView, SIGNAL('entered(QModelIndex)'), self.setCurrentIndex)
        self.preferredWidth = 0

    def setPreferredWidth(self, preferredWidth):
        self.preferredWidth = preferredWidth


    def setFilter(self, actionTypeId, personId, specialityId, clientSex, clientAge, removeEmptyNodes=True):
        self._model.setFilter(actionTypeId, personId, specialityId, clientSex, clientAge, removeEmptyNodes)


    def updateModel(self):
        self._model.update()


    def setValue(self, id):
        index = self._model.findItemId(id)
        self.setCurrentIndex(index)


    def value(self):
        modelIndex = self._model.index(self.currentIndex(), 0, self.rootModelIndex())
        if modelIndex.isValid():
            return self._model.itemId(modelIndex)
        return None


    def on_ItemActivated(self, index):
        if index.isValid():
            if (int(index.flags()) & Qt.ItemIsSelectable) != 0:
                self.hidePopup()
                self.emit(SIGNAL('itemSelected(QModelIndex)'), index)
            else:
#                print self.__popupView.isExpanded(index)
                self._popupView.setExpanded(index, not self._popupView.isExpanded(index))


    def showPopup(self):
#        self.__searchString = ''
#        modelIndex = self._model.index(self.currentIndex(), 0, self.rootModelIndex())

        self._popupView.setRealRootIndex(QModelIndex())
        self._popupView.expandAll()
        preferredWidth = self._popupView.sizeHint().width()
        preferredWidth = max(self.preferredWidth if self.preferredWidth else 0, self._popupView.sizeHint().width())
        if preferredWidth:
            if self.width() < preferredWidth:
                self._popupView.setFixedWidth(preferredWidth)
        QtGui.QComboBox.showPopup(self)
        scrollBar = self._popupView.horizontalScrollBar()
        scrollBar.setValue(0)


    def setCurrentIndex(self, index):
        if not index:
            index = QModelIndex()
        if index:
            self.setRootModelIndex(index.parent())
            QtGui.QComboBox.setCurrentIndex(self, index.row())

#        self.emit(SIGNAL('codeSelected(QString)'), self._model.code(index))


    def eventFilter(self, obj, event):
        if event.type() == QEvent.KeyPress: # and obj == self.__popupView:
            if event.key() in (Qt.Key_Enter, Qt.Key_Return, Qt.Key_Select):
                index = self._popupView.currentIndex()
                if index.isValid() and (int(index.flags()) & Qt.ItemIsSelectable) != 0:
                    self.hidePopup()
                    self.setCurrentIndex(index)
                    self.emit(SIGNAL('itemSelected(QModelIndex)'), index)
                return True
#            print 'default for ',event.key()
            return False
        if event.type() == QEvent.MouseButtonRelease and obj == self._popupView.viewport():
            return True
#        return QtGui.QComboBox.eventFilter(self, obj, event)
        return False


# ##############################################################################


class CActionTemplateCache(object):
    def __init__(self, eventEditor, personEditor):
        self.eventEditor = eventEditor
        #personEditor должен иметь два метода:
        # value - возвращает id врача
        # currentPersonSpecialityId - возвращает id специальности текущего врача
        self.personEditor = personEditor
        self.data = {}


    def reset(self, actionTypeId=None):
        if actionTypeId:
            del self.data[actionTypeId]
        else:
            self.data = {}


    def getModel(self, actionTypeId):
        personId, specialityId = self.__getPersonAndSpecialityId()
        actionLevel = self.data.setdefault(actionTypeId, {})
        result = actionLevel.get(personId, None)
        if result is None:
            result = self.__createModel(actionTypeId, personId, specialityId)
            actionLevel[personId] = result
        return result


    def __getPersonAndSpecialityId(self):
        if QtGui.qApp.userSpecialityId:
            return QtGui.qApp.userId, QtGui.qApp.userSpecialityId
        if self.personEditor.currentPersonSpecialityId():
            return self.personEditor.value(), self.personEditor.currentPersonSpecialityId()
        return self.eventEditor.personId, self.eventEditor.personSpecialityId


    def __createModel(self, actionTypeId, personId, specialityId):
        result = CActionTemplateModel(self.eventEditor,
                                      actionTypeId,
                                      personId,
                                      specialityId,
                                      self.eventEditor.clientSex,
                                      self.eventEditor.clientAge)
        return result
