# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt, QAbstractTableModel, QModelIndex, QVariant, SIGNAL, QDate

from library.InDocTable import CRBInDocTableCol
from library.Utils import (
    forceDate, forceDouble, forceInt, forceRef, forceString, toVariant, getDentitionActionTypeId,
    forceDateTime
)
from Events.Utils import getActionTypeIdListByClass, getActionTypeIdListByFlatCode

from Events.Action import CAction, CActionType, CActionTypeCache, getActionDefaultAmountEx, getActionDuration, CActionStatus


__all__ = [
    'CActionsModel',
    'getActionDefaultAmountEx',
    'CGroupActionsProxyModel'
]


class CActionsModel(QAbstractTableModel):

    __pyqtSignals__ = ('amountChanged(int)',
                      )
    def __init__(self, parent, actionTypeClass=None):
        QAbstractTableModel.__init__(self, parent)
        self.actionTypeClass = None
        self.actionTypeIdList = []
        self.disabledActionTypeIdList = []
        self.col = CRBInDocTableCol(u'',  'actionType_id', 10, 'ActionType', addNone=True, preferredWidth=300)
        self._items = [] # each item is pair (record, action)
        self._loadedActionIdListWithEndDate = []
        self.eventEditor = None
        if actionTypeClass is not None:
            self.setActionTypeClass(actionTypeClass)
        self.idxFieldName = 'idx' # :( для обеспечения возможности перемещения строк.
        self.table = QtGui.qApp.db.table('Action')
        self.readOnly = False


    def setReadOnly(self, value):
        self.readOnly = value


    def setActionTypeClass(self, actionTypeClass):
        self.actionTypeClass = actionTypeClass
        self.actionTypeIdList = getActionTypeIdListByClass(actionTypeClass)
        self.col.filter = 'class=%d'%actionTypeClass

    def items(self):
        return self._items


    def updatePersonId(self, oldPersonId, newPersonId):
        pass
#        def replacePerson(record, field):
#            if forceRef(record.value(field)) == oldPersonId:
#                record.setValue(field, toVariant(newPersonId))
#
#        if oldPersonId and newPersonId and oldPersonId != newPersonId:
#            for item in self._items:
#                record = item[0]
##                replacePerson(record, 'setPerson_id')
#                replacePerson(record, 'person_id')


    def emitAmountChanged(self, row):
        self.emit(SIGNAL('amountChanged(int)'), row)


    def updateActionsAmount(self): # по изменению в событии
        for row, item in enumerate(self._items):
            record, action = item
            actionTypeId = forceRef(record.value('actionType_id'))
            if actionTypeId:
                actionType = CActionTypeCache.getById(actionTypeId)
                if actionType.amountEvaluation in (CActionType.eventVisitCount,
                                                   CActionType.eventDurationWithFiveDayWorking,
                                                   CActionType.eventDurationWithSixDayWorking,
                                                   CActionType.eventDurationWithSevenDayWorking):
                    prevValue = forceDouble(record.value('amount'))
                    value = float(self.getDefaultAmountEx(actionType, record, action))
                    if prevValue != value:
                        record.setValue('amount', toVariant(value))
                        self.emitRowsChanged(row, row)
                        self.emitAmountChanged(row)


    def updateActionAmount(self, row): # по изменению в самом действии
        record, action = self._items[row]
        actionTypeId = forceRef(record.value('actionType_id'))
        if actionTypeId:
            actionType = CActionTypeCache.getById(actionTypeId)
            if actionType.amountEvaluation in (CActionType.eventVisitCount,
                                               CActionType.eventDurationWithFiveDayWorking,
                                               CActionType.eventDurationWithSixDayWorking,
                                               CActionType.eventDurationWithSevenDayWorking,
                                               CActionType.actionDurationWithFiveDayWorking,
                                               CActionType.actionDurationWithSixDayWorking,
                                               CActionType.actionDurationWithSevenDayWorking,
                                               CActionType.actionFilledPropsCount,
                                               CActionType.actionAssignedPropsCount):
                prevValue = forceDouble(record.value('amount'))
                value = float(self.getDefaultAmountEx(actionType, record, action))
                if prevValue != value:
                    record.setValue('amount', toVariant(value))
                    self.emitRowsChanged(row, row)
                    self.emitAmountChanged(row)


    def disableActionType(self, actionTypeId):
        self.disabledActionTypeIdList.append(actionTypeId)


    def getActionDuration(self, record, weekProfile):
        return getActionDuration(self.eventEditor.eventTypeId, record, weekProfile)


    def getDefaultAmountEx(self, actionType, record, action):
        return getActionDefaultAmountEx(self.eventEditor, actionType, record, action)


    def getDefaultAmount(self, actionTypeId, record, action):
        if actionTypeId:
            actionType = CActionTypeCache.getById(actionTypeId)
            result = self.getDefaultAmountEx(actionType, record, action)
        else:
            result = 0
        return result


    def columnCount(self, index=None):
        return 1


    def rowCount(self, index=None):
        return len(self._items)+1


    def flags(self, index):
        if self.readOnly:
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable
        row = index.row()
#        if self.isLocked(row):
#            return Qt.ItemIsEnabled | Qt.ItemIsSelectable
        if row<len(self._items):
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable
        if self.isExposed(row):
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable
        return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable


    def headerData(self, section, orientation, role = Qt.DisplayRole):
        if orientation == Qt.Horizontal:
            if role == Qt.DisplayRole:
                return QVariant(u'Наименование')
        return QVariant()


    def data(self, index, role=Qt.DisplayRole):
        row = index.row()
        if row<len(self._items):
            if role == Qt.EditRole:
                record = self._items[row][0]
                return record.value('actionType_id')
            if role == Qt.DisplayRole:
                record = self._items[row][0]
                outName = forceString(record.value('specifiedName'))
                actionTypeId = forceRef(record.value('actionType_id'))
                if actionTypeId:
                    actionType = CActionTypeCache.getById(actionTypeId)
                    if actionType:
                        outName = actionType.name + ' '+outName if outName else actionType.name
                return QVariant(outName)
            if role == Qt.StatusTipRole or role == Qt.ToolTipRole:
                record = self._items[row][0]
#                specifiedName = forceString(record.value('specifiedName'))
                actionTypeId = forceRef(record.value('actionType_id'))
                actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None
                return QVariant(actionType.code + ': ' + actionType.name if actionType else '')
            if role == Qt.ForegroundRole:
                record, action = self._items[row]
                if action and action.getType().isRequiredCoordination and record.isNull('coordDate'):
                    return QVariant(QtGui.QColor(255, 0, 0))
        return QVariant()


    def setData(self, index, value, role=Qt.EditRole, presetAction=None): # presetAction - это ошибка!
        if role == Qt.EditRole:
            row = index.row()
            actionTypeId = forceRef(value)
            if actionTypeId and not ( self.checkMaxOccursLimit(actionTypeId) and
                                      self.checkMovingNoLeaved(actionTypeId) and
                                      self.checkMovingAfterReceived(actionTypeId) and
                                      self.checkLeavedAfterMoving(actionTypeId) and
                                      self.checkLeavedAfterMovingDate(actionTypeId) ):
                return False
            if row == len(self._items): # Это ошибка!
                if actionTypeId is None:
                    return False
                self.addRow(presetAction=presetAction)
            if not presetAction:
                record = self._items[row][0]
                action = self.getFilledAction(record, actionTypeId)
            else:
                record = presetAction.getRecord()
                action = presetAction
            self._items[row] = (record, action)
            if not presetAction:
                record.setValue('amount', toVariant(self.getDefaultAmount(actionTypeId, record, action)))
            self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index, index)
            self.emitItemsCountChanged()
            actionsSummaryRowRow = self.eventEditor.translate2ActionsSummaryRow(self, row)
            if actionsSummaryRowRow is not None:
                self.eventEditor.onActionChanged(actionsSummaryRowRow)
            return True


    def appendOuterAction(self, action):
        item = (action.getRecord(), action)
        self._items.append(item)
        index = QModelIndex()
        cnt = len(self._items)
        self.beginInsertRows(index, cnt, cnt)
        self.insertRows(cnt, 1, index)
        self.endInsertRows()
        self.emitItemsCountChanged()



    def emitItemsCountChanged(self):
        self.emit(SIGNAL('itemsCountChanged()'))


    def addRow(self, actionTypeId=None, amount=None, financeId=None, contractId=None, presetAction=None):
        if not presetAction:
            record = QtGui.qApp.db.table('Action').newRecord()
            action = self.getFilledAction(record, actionTypeId, amount, financeId, contractId)
            item = (record, action)
        else:
            item = (presetAction.getRecord(), presetAction)
        self._items.append(item)
        index = QModelIndex()
        cnt = len(self._items)
        self.beginInsertRows(index, cnt, cnt)
        self.insertRows(cnt, 1, index)
        self.endInsertRows()
        self.emitItemsCountChanged()
        return cnt-1


    def getFilledAction(self, record, actionTypeId, amount=None, financeId=None, contractId=None, **kwargs):
        return CAction.getFilledAction(self.eventEditor, record, actionTypeId, amount, financeId, contractId)


    def delVisit(self, record):
        eventEditor = self.eventEditor
        if not hasattr(eventEditor, 'tblVisits'):
            return
        visitRecord = None
        visitId = forceRef(record.value('visit_id'))
        visitList = eventEditor.modelActionsSummary.visitList
        visitRecord = visitList.get(record)
        visitsModel = eventEditor.modelVisits
        visitList   = visitsModel.items()
        visitRow = None

        if visitRecord:
            visitRow = visitList.index(visitRecord)

        if visitId and not visitRow:
            for i, record in enumerate(visitList):
                id = forceInt(record.value('id'))
                if id == visitId:
                    visitRow = i
        if visitRow is not None:
            visitsModel.removeRows(visitRow, 1)


    def removeRows(self, row, count, parentIndex = QModelIndex()):
        if 0<=row and row+count<=len(self._items):
            for i in xrange(count):
                if self.isLocked(row+i):
                    return False
                record = self._items[row][0]
                actionTypeId = forceRef(record.value('actionType_id'))
                if actionTypeId and not (self.checkReceivedDeleted(actionTypeId) and
                                         self.checkMovingDeleted(actionTypeId, row)):
                    return False
                self.delVisit(record)
            self.beginRemoveRows(parentIndex, row, row+count-1)
            del self._items[row:row+count]
            self.endRemoveRows()
            return True
        else:
            return False

    def loadItems(self, eventId):
        self._items = []
        db = QtGui.qApp.db
        table = db.table('Action')
        cond  = [table['deleted'].eq(0)]
        if eventId:
            cond.append(table['event_id'].eq(eventId))
        if self.actionTypeIdList:
            cond.append(table['actionType_id'].inlist(self.actionTypeIdList))
        actionTypeMDIdList = getActionTypeIdListByFlatCode(u'medicalDiagnosis%')
        if actionTypeMDIdList:
            cond.append(table['actionType_id'].notInlist(actionTypeMDIdList))
        actionTypeTSIdList = getActionTypeIdListByFlatCode(u'temperatureSheet%')
        if actionTypeTSIdList:
            cond.append(table['actionType_id'].notInlist(actionTypeTSIdList))
        actionTypeProtocolIdList = getActionTypeIdListByFlatCode(u'protocol%')
        if actionTypeProtocolIdList:
            cond.append(table['actionType_id'].notInlist(actionTypeProtocolIdList))
        for record in db.getRecordList(table, '*', cond, 'idx, id'):
            self._items.append((record, CAction(record=record)))
            if forceDate(record.value('endDate')):
                self._loadedActionIdListWithEndDate.append(forceRef(record.value('id')))
        self.reset()


    def loadedActionIdListWithEndDate(self):
        return self._loadedActionIdListWithEndDate


    def saveItems(self, eventId):
        db = QtGui.qApp.db
        table = db.table('Action')
        idList = []
        dentActionTypeId, parodentActionTypeId = getDentitionActionTypeId()
        actionTypeDentIdList = []
        if dentActionTypeId:
            actionTypeDentIdList.append(dentActionTypeId)
        if parodentActionTypeId:
            actionTypeDentIdList.append(parodentActionTypeId)
        for record, action in self._items:
            if action:
                self.eventEditor.getMKBValueForActionDuringSaving(record, action)
                if hasattr(self.eventEditor, 'modelActionsSummary'):
                    visitList = self.eventEditor.modelActionsSummary.visitList
                    visitRecord = visitList.get(record)
                    if visitRecord:
                        id = forceRef(visitRecord.value('id'))
                        if id:
                            record.setValue('visit_id', id)
                if not actionTypeDentIdList or forceRef(record.value('actionType_id')) not in actionTypeDentIdList:
                    id = action.save(eventId, len(idList))
                    idList.append(id)
        cond  = [ table['event_id'].eq(eventId), 'NOT ('+table['id'].inlist(idList)+')' ]
        if self.actionTypeIdList:
            cond.append(table['actionType_id'].inlist(self.actionTypeIdList))
        actionTypeMDIdList = getActionTypeIdListByFlatCode(u'medicalDiagnosis%')
        if actionTypeMDIdList:
            cond.append(table['actionType_id'].notInlist(actionTypeMDIdList))
        actionTypeTSIdList = getActionTypeIdListByFlatCode(u'temperatureSheet%')
        if actionTypeTSIdList:
            cond.append(table['actionType_id'].notInlist(actionTypeTSIdList))
        actionTypeProtocolIdList = getActionTypeIdListByFlatCode(u'protocol%')
        if actionTypeProtocolIdList:
            cond.append(table['actionType_id'].notInlist(actionTypeProtocolIdList))
        if actionTypeDentIdList:
            cond.append(table['actionType_id'].notInlist(actionTypeDentIdList))
        db.deleteRecord(table, cond)


    def isLocked(self, row):
        if 0 <= row < len(self._items):
            action = self._items[row][1]
            if action:
                return action.isLocked()
        return False


    def isLockedOrExposed(self, row):
        if 0 <= row < len(self._items):
            recod, action = self._items[row]
            return forceInt(recod.value('payStatus'))!=0 or (action and action.isLocked())
        return False


    def isExposed(self, row):
        if 0 <= row < len(self._items):
            record, action = self._items[row]
            if action:
                return action.isExposed()
        return False


    def actionTypeId(self, row):
        if 0 <= row < len(self._items):
            return forceInt(self._items[row][0].value('actionType_id'))
        else:
            return None


    def payStatus(self, row):
        if 0 <= row < len(self._items):
            return forceInt(self._items[row][0].value('payStatus'))
        else:
            return 0


#    def changeActionType(self, row, newActionTypeId):
#        self._items[row].setValue('actionType_id', QVariant(newActionTypeId))
#        self.emitCellChanged(row, 0)


    def removeRowEx(self, row):
        self.removeRows(row, 1)


    def upRow(self, row):
        if 0 < row < len(self._items):
            self._items[row-1], self._items[row] = self._items[row], self._items[row-1]
            self.emitRowsChanged(row-1, row)
            return True
        else:
            return False


    def downRow(self, row):
        if 0 <= row < len(self._items)-1:
            self._items[row+1], self._items[row] = self._items[row], self._items[row+1]
            self.emitRowsChanged(row, row+1)
            return True
        else:
            return False


    def checkMaxOccursLimit(self, actionTypeId):
        actionType = CActionTypeCache.getById(actionTypeId)
        count = 0
        for record, action in self._items:
            if action and action.getType() == actionType:
                count += 1
        result = actionType.checkMaxOccursLimit(count, True)
        return result


    def checkReceivedDeleted(self, actionTypeId):
        result = True
        actionType = CActionTypeCache.getById(actionTypeId)
        if u'received' in actionType.flatCode.lower():
            for record, action in self._items:
                if action:
                   actionTypeItem = action.getType()
                   if actionTypeItem and (u'moving' in actionTypeItem.flatCode.lower()):
                       return actionType.checkReceivedMovingLeaved(u'Нельзя удалить действие "Поступление" если есть "Движение"')
                   if actionTypeItem and (u'leaved' in actionTypeItem.flatCode.lower()):
                        return actionType.checkReceivedMovingLeaved(u'Нельзя удалить действие "Поступление" если есть "Выписка"')
        return result


    def checkMovingDeleted(self, actionTypeId, rowCurrent):
        result = True
        actionType = CActionTypeCache.getById(actionTypeId)
        if u'moving' in actionType.flatCode.lower():
            for row, item in enumerate(self.items()):
                record, action = item
                if action:
                   actionTypeItem = action.getType()
                   if actionTypeItem and (u'leaved' in actionTypeItem.flatCode.lower()):
                        return actionType.checkReceivedMovingLeaved(u'Нельзя удалить действие "Движение" если есть "Выписка"')
                   elif row > rowCurrent and actionTypeItem and (u'moving' in actionTypeItem.flatCode.lower()):
                       return actionType.checkReceivedMovingLeaved(u'Нельзя удалить действие "Движение" если после него есть "Движение"')
        return result


    def checkMovingNoLeaved(self, actionTypeId):
        result = True
        actionType = CActionTypeCache.getById(actionTypeId)
        if u'moving' in actionType.flatCode.lower():
            for record, action in self._items:
                if action:
                   actionTypeItem = action.getType()
                   if actionTypeItem and (u'leaved' in actionTypeItem.flatCode.lower()):
                       return actionType.checkReceivedMovingLeaved(u'Действие "Движение" не должно применяться после действия "Выписка"')
                   elif actionTypeItem and (u'moving' in actionTypeItem.flatCode.lower()):
                       if not forceDate(record.value('endDate')):
                          return actionType.checkReceivedMovingLeaved(u'Действие "Движение" не может появится при наличии не законченного "Движение"')
        return result


    def checkMovingAfterReceived(self, actionTypeId):
        actionType = CActionTypeCache.getById(actionTypeId)
        if u'moving' in actionType.flatCode.lower():
            for record, action in self._items:
                if action:
                   actionTypeItem = action.getType()
                   if actionTypeItem and (u'received' in actionTypeItem.flatCode.lower()):
                       if not forceDate(record.value('endDate')):
                           return actionType.checkReceivedMovingLeaved(u'Действие "Движение" не может появится при наличии не законченного действия "Поступление"')
                       return True
            return actionType.checkReceivedMovingLeaved(u'Действие "Движение" не должно применяться пока нет действия действия "Поступление"')
        return True


    def checkLeavedAfterMoving(self, actionTypeId):
        actionType = CActionTypeCache.getById(actionTypeId)
        if u'leaved' in actionType.flatCode.lower():
            for record, action in self._items:
                if action:
                   actionTypeItem = action.getType()
                   if actionTypeItem and (u'moving' in actionTypeItem.flatCode.lower()):
                      return True
            return actionType.checkReceivedMovingLeaved(u'Действие "Выписка" не должно применяться пока нет действия "Движение"')
        return True


    def checkLeavedAfterMovingDate(self, actionTypeId):
        actionType = CActionTypeCache.getById(actionTypeId)
        if u'leaved' in actionType.flatCode.lower():
            for record, action in self._items:
                if action:
                   actionTypeItem = action.getType()
                   if actionTypeItem and (u'moving' in actionTypeItem.flatCode.lower()):
                      if not forceDate(record.value('endDate')):
                         return actionType.checkReceivedMovingLeaved(u'Действие "Выписка" не может появится при наличии не законченного действия "Движение"')
        return True


    def emitRowsChanged(self, row1, row2):
        index1 = self.index(row1, 0)
        index2 = self.index(row2, self.columnCount())
        self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index1, index2)


_emptyValue = object()
_nomenclatureIndex = 1

class CProxyGroup(object):
    def __init__(self, model):
        self._model = model
        self._items = []
        self._mapItem2Row = {}
        self._mapRow2Item = {}
        self._mapProxyRow2ModelRow = {}
        self._requireEP = False
        self._expanded = False
        self._headModelRow = None
        self._actionTypeId = None

    def checkOnlyActual(self, onlyActual):
        if onlyActual:
            return bool([i for i in self._items if forceInt(i[0].value('status')) != CActionStatus.finished])
        return True

    @property
    def items(self):
        return self._items

    @property
    def nomenclatureId(self):
        if not self._requireEP:
            return None

        headItem = self.headItem
        _, action = headItem

        return self._getActionNomenclatureId(action)

    def _getActionNomenclatureId(self, action):
        actionType = action.getType()

        for propertyType in actionType.getPropertiesById().values():
            if propertyType.inActionsSelectionTable:
                if propertyType.inActionsSelectionTable == _nomenclatureIndex:
                    property = action.getPropertyById(propertyType.id)
                    return property.getValue()

        return None

    @property
    def model(self):
        return self._model

    def increaseItemsIdx(self, count):
        for record, _ in self._items:
            idx = forceInt(record.value('idx'))
            if idx+count >= 0:
                record.setValue('idx', toVariant(idx+count))
            else:
                record.setValue('idx', toVariant(self.idx))
            self.idx = forceInt(record.value('idx'))

    def decreaseItemsIdx(self, count):
        for record, _ in self._items:
            idx = forceInt(record.value('idx'))
            if idx-count >= 0:
                record.setValue('idx', toVariant(idx-count))
            else:
                record.setValue('idx', toVariant(self.idx))
            self.idx = forceInt(record.value('idx'))

    def canDownInGroup(self, proxyRow):
        return self._expanded

    def downProxyRow(self, proxyRow):
        return False

    def canUpInGroup(self, proxyRow):
        return self._expanded

    def upProxyRow(self, proxyRow):
        return False

    @property
    def actionTypeId(self):
        return self._actionTypeId

    def getHeadItem(self):
        return self._items[0]

    def getItem(self, proxyRow):
        return self._mapRow2Item[self._mapProxyRow2ModelRow[proxyRow]]

    def resetMapping(self, model):
        self._mapRow2Item.clear()
        self._mapItem2Row.clear()
        self._sortItems()

        modelItems = model.items()
        for item in self._items:
            modelRow = modelItems.index(item)
            self._mapItem2Row[item] = modelRow
            self._mapRow2Item[modelRow] = item

        self._mapProxyRow2ModelRow.clear()
        self._headModelRow = None

    @property
    def requireEP(self):
        return self._requireEP

    def canBeGrouped(self):
        return self._requireEP and len(self._items) > 1

    def mapRows(self, proxyRow, item):
        modelRow = self._mapItem2Row[item]
        self._mapProxyRow2ModelRow[proxyRow] = modelRow

    @property
    def expanded(self):
        return self._expanded

    def _sortItems(self):
        self._items.sort(key=lambda i: forceDateTime(i[0].value('begDate')), reverse=self.expanded)
        self._headModelRow = None

    def __contains__(self, item):
        return item in self._items

    def __iter__(self):
        return iter(self._items)

    def __len__(self):
        return len(self._items) if self._expanded else 1 if len(self._items) else 0

    def itemsCount(self):
        return len(self._items)

    def _addItem(self, modelRow, item):
        self._items.append(item)
        self._mapItem2Row[item] = modelRow
        self._mapRow2Item[modelRow] = item
        self._sortItems()

    @property
    def firstItem(self):
        return self._items[-1] if self.expanded else self._items[0]

    def inMonth(self, date):
        item = self.firstItem
        record, action = item
        begDate = forceDate(record.value('begDate'))
        return QDate(date.year(), date.month(), 1) <= begDate <= QDate(date.year(), date.month(), date.daysInMonth())

    def inPeriod(self, begDate, endDate):
        item = self.firstItem
        record, action = item
        date = forceDate(record.value('begDate'))
        return (not begDate or begDate <= date) and (not endDate or date <= endDate)

    def setExpanded(self, value):
        if self._requireEP:
            self._expanded = value
            self._sortItems()

    def addItem(self, modelRow, item):
        record, action = item
        if not self._actionTypeId:
            self._actionTypeId = action.getType().id

        executionPlan = action.getExecutionPlan()
        if not self._items:
            self._addItem(modelRow, item)
            self._requireEP = bool(executionPlan)
            return True

        if not executionPlan:
            return False

        if self._getActionNomenclatureId(action) != self.nomenclatureId:
            return False

        headItem = self.headItem
        gRecord, gAction = headItem
        gPlannedEndDate = forceDateTime(gRecord.value('plannedEndDate'))
        gDirectionDate = forceDateTime(gRecord.value('directionDate'))

        plannedEndDate = forceDateTime(record.value('plannedEndDate'))
        directionDate = forceDateTime(record.value('directionDate'))

        if plannedEndDate == gPlannedEndDate and directionDate == gDirectionDate:
            self._addItem(modelRow, item)
            return True

        return False


    def getModelRow(self, proxyRow, model):
        if self._expanded:
            return self._mapProxyRow2ModelRow[proxyRow]

        if self._headModelRow is None:
            self._headModelRow = model.items().index(self._items[0])

        return self._headModelRow

    def getModelRows(self):
        return self._mapProxyRow2ModelRow.values()

    @property
    def headItem(self):
        return self._items[0]

    def isHeadItem(self, proxyRow, model):
        if self._headModelRow is None:
            self._headModelRow = model.items().index(self._items[0])

        if not self._expanded:
            return True

        modelRow = self.getModelRow(proxyRow, model)

        return self._headModelRow == modelRow

    def deleteProxyRow(self, proxyRow, model):
        modelRow = self.getModelRow(proxyRow, model)
        item = self._mapRow2Item[modelRow]
        self._items.remove(item)
        del self._mapItem2Row[item]
        del self._mapRow2Item[modelRow]
        del self._mapProxyRow2ModelRow[proxyRow]
        self._headModelRow = None

    @property
    def proxyRows(self):
        return self._mapProxyRow2ModelRow.keys()


class CGroups(object):
    def __init__(self, model):
        self._mapProxyRow2Group = {}
        self._groups = []
        self.idx = None
        self._model = model

    def upGroup(self, proxyRow, group):
        if group not in self._groups:
            return False

        index = self._groups.index(group)
        if index == 0:
            return False

        upperGroup = self._groups[index-1]
        upperGroup.increaseItemsIdx(group.itemsCount())
        group.idx = upperGroup.idx
        group.decreaseItemsIdx(upperGroup.itemsCount())

        self._groups[index], self._groups[index - 1] = upperGroup, group

        self._resetModelNewItemsOrder()

        return True

    def getGroupProxyRow(self, group):
        result = 0
        for targetGroup in self._groups:
            if group is targetGroup:
                return result
            result += len(group)
        return -1

    def downGroup(self, proxyRow, group):
        if group not in self._groups:
            return False

        index = self._groups.index(group)
        if index == len(self._groups) - 1:
            return False

        downerGroup = self._groups[index + 1]
        downerGroup.decreaseItemsIdx(group.itemsCount())
        group.idx = downerGroup.idx

        group.increaseItemsIdx(downerGroup.itemsCount())

        self._groups[index], self._groups[index + 1] = downerGroup, group

        self._resetModelNewItemsOrder()

        return True

    def _resetModelNewItemsOrder(self):
        model = self._model.model()

        items = {}
        for group in self._groups:
            for item in group:
                record = item[0]
                items.setdefault(forceInt(record.value('idx')), []).append(item)

        newItemsList = []
        for idx in sorted(items.keys()):
            newItemsList.extend(items[idx])

        model.items()[:] = newItemsList

    def clear(self):
        self._mapProxyRow2Group.clear()
        self._groups = []

    @property
    def groupsIterator(self):
        return iter(self._groups)

    def __iter__(self):
        return iter(self._model.model().items())

    def __len__(self):
        return self.rowsCount()

    def __getitem__(self, proxyRow):
        if proxyRow < 0:
            modelRowsCount = self._model.rowCount() - 1
            proxyRow += modelRowsCount
        return self.getItem(proxyRow)

    def delete(self, group):
        for proxyRow in group.proxyRows:
            group.deleteProxyRow(proxyRow, self._model.model())
            del self._mapProxyRow2Group[proxyRow]
        self._groups.remove(group)

    def addGroup(self):
        group = CProxyGroup(self._model)
        self._groups.append(group)
        return group

    def mapGroup(self, proxyRow, group):
        self._mapProxyRow2Group[proxyRow] = group

    def getItem(self, proxyRow):
        return self._mapProxyRow2Group[proxyRow].getItem(proxyRow)

    def rowsCount(self):
        return sum([len(g) for g in self._groups])


class CGroupActionsProxyModel(QtGui.QProxyModel):
    __groupingAllowed__ = True

    __pyqtSignals__ = (
        'amountChanged(int)',
        'itemsCountChanged()'
    )

    def __init__(self, parent):
        actionModel = CActionsModel(parent)

        QtGui.QProxyModel.__init__(self, parent)
        QtGui.QProxyModel.setModel(self, actionModel)

        self._parent = parent
        self._actionModel = actionModel

        self._groups = CGroups(self)
        self._mapProxyRow2Group = {}
        self._mapModelRow2ProxyRow = {}

        boldFont = QtGui.QFont()
        boldFont.setWeight(QtGui.QFont.Bold)

        self._qBoldFont = QVariant(boldFont)

        self._groupItemsShift = u' ' * 3

        self.connect(self._actionModel, SIGNAL('dataChanged(QModelIndex, QModelIndex)'), self._emitDataChanged)
        self.connect(self._actionModel, SIGNAL('amountChanged(int)'), self._emitAmountChanged)
        self.connect(self._actionModel, SIGNAL('itemsCountChanged()'), self._emitItemsCountChanged)

    @property
    def eventEditor(self):
        return self._actionModel.eventEditor

    @eventEditor.setter
    def eventEditor(self, eventEditor):
        self._actionModel.eventEditor = eventEditor

    def _emitDataChanged(self, index1, index2):
        mr1, mr2 = index1.row(), index2.row()
        if mr1 not in self._mapModelRow2ProxyRow or mr2 not in self._mapModelRow2ProxyRow:
            return
        r1, r2 = self._mapModelRow2ProxyRow[mr1], self._mapModelRow2ProxyRow[mr2]
        i1, i2 = self.index(r1, 0), self.index(r2, 0)
        self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), i1, i2)

    def _emitAmountChanged(self, row):
        row = self._mapModelRow2ProxyRow[row]
        self.emit(SIGNAL('amountChanged(int)'), row)

    def _emitItemsCountChanged(self):
        self.emit(SIGNAL('itemsCountChanged()'))

    def setModel(self, model):
        # Это довольно специфическое решение. Запретим задавать цедевую модель. Она задается в конструктуре.
        raise AttributeError()

    def __getattr__(self,attributeName):
        return getattr(self._actionModel, attributeName)

    def setData(self, index, value, *args, **kwargs):
        proxyRow = index.row()

        if proxyRow not in self._mapProxyRow2Group and proxyRow == len(self._groups):
            modelRow = len(self._actionModel.items())

        elif proxyRow not in self._mapProxyRow2Group:
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable
        else:
            group = self._mapProxyRow2Group[proxyRow]
            modelRow = group.getModelRow(proxyRow, self._actionModel)

        index = self._actionModel.index(modelRow, 0)

        self._actionModel.blockSignals(True)

        result = self._actionModel.setData(index, value, *args, **kwargs)

        self._actionModel.blockSignals(False)

        if result:
            self._addNewItem()
            index = self.index(proxyRow, 0)
            self.emit(SIGNAL('rowsInserted(QModelIndex, int, int)'), index, proxyRow, proxyRow+1)
            self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index, index)
            self.emitRowIndexActivated(proxyRow)

        return result


    def loadItems(self, eventId):
        self._actionModel.loadItems(eventId)
        self._group()

    def _addGroup(self, actionTypeId, modelRow, item, mapActionTypeGroups=None):
        group = self._groups.addGroup()

        group.addItem(modelRow, item)

        if mapActionTypeGroups is not None:
            mapActionTypeGroups.setdefault(actionTypeId, []).append(group)

        return group


    def _group(self):
        actionTypeGroups = {}

        for modelRow, item in enumerate(self._actionModel.items()):
            record, action = item

            actionTypeId = action.getType().id

            if actionTypeId not in actionTypeGroups:
                self._addGroup(actionTypeId, modelRow, item, actionTypeGroups)
                continue

            added = False

            for group in actionTypeGroups[actionTypeId]:
                if group.addItem(modelRow, item):
                    added = True
                    break

            if added:
                continue

            self._addGroup(actionTypeId, modelRow, item, actionTypeGroups)

        self._remapRows()

    def _remapRows(self):
        self._mapProxyRow2Group.clear()
        self._mapModelRow2ProxyRow.clear()

        modelItems = self._actionModel.items()

        proxyRow = 0
        # for group in self._groups.groupsIterator:
        #     group.resetMapping(self._actionModel)
        #     if group.expanded:
        #         for item in group:
        #             group.mapRows(proxyRow, item)
        #             self._mapProxyRow2Group[proxyRow] = group
        #             self._groups.mapGroup(proxyRow, group)
        #             self._mapModelRow2ProxyRow[modelItems.index(item)] = proxyRow
        #             proxyRow += 1
        #
        #     else:
        #         for item in group:
        #             group.mapRows(proxyRow, item)
        #
        #         self._mapProxyRow2Group[proxyRow] = group
        #
        #         self._mapModelRow2ProxyRow[modelItems.index(group.getHeadItem())] = proxyRow
        #
        #         self._groups.mapGroup(proxyRow, group)
        #         proxyRow += 1
        for group in self._groups.groupsIterator:
            group.resetMapping(self._actionModel)
            for item in group:
                group.mapRows(proxyRow, item)
                self._mapProxyRow2Group[proxyRow] = group
                self._groups.mapGroup(proxyRow, group)
                self._mapModelRow2ProxyRow[modelItems.index(item)] = proxyRow
                if group.expanded:
                    proxyRow += 1
            if not group.expanded:
                proxyRow += 1

    def data(self, index, role=Qt.DisplayRole):
        row = index.row()
        if row not in self._mapProxyRow2Group:
            return QVariant()

        group = self._mapProxyRow2Group[row]
        modelRow = group.getModelRow(row, self._actionModel)

        modelIndex = self._actionModel.index(modelRow, 0)

        isHeadItem = group.isHeadItem(row, self._actionModel)

        if role == Qt.DisplayRole and not isHeadItem:
            value = forceString(self._actionModel.data(modelIndex, role))
            return QVariant(self._groupItemsShift+value)

        elif role == Qt.FontRole:
            if isHeadItem and group.canBeGrouped():
                return self._qBoldFont

        return self._actionModel.data(modelIndex, role)

    def index(self, row, column, parent=QtCore.QModelIndex()):
        return QtGui.QProxyModel.index(self, row, column, parent)

    def columnCount(self, index=None):
        return 1

    def rowCount(self, index=None):
        count = 1
        for group in self._groups.groupsIterator:
            count += len(group) if group.expanded else 1
        return count

    def canRowBeGrouped(self, proxyRow):
        if proxyRow not in self._mapProxyRow2Group:
            return False

        group = self._mapProxyRow2Group[proxyRow]
        return group.canBeGrouped()


    def removeRows(self, row, count, parentIndex = QModelIndex()):
        if not (0 <=row and row+count <= self.rowCount()):
            return False

        for proxyRow in xrange(row, row+count):
            self._removeRow(proxyRow)

        return True

    def afterRowsDeleting(self):
        self._emitItemsCountChanged()

    def _removeRow(self, proxyRow):
        group = self._mapProxyRow2Group[proxyRow]

        modelRow = group.getModelRow(proxyRow, self._actionModel)
        removed = self._actionModel.removeRow(modelRow)
        if removed:
            del self._mapModelRow2ProxyRow[modelRow]
            group.deleteProxyRow(proxyRow, self._actionModel)
            if not len(group):
                self._groups.delete(group)
            del self._mapProxyRow2Group[proxyRow]
            self._remapRows()


    def touchGrouping(self, proxyRow):
        group = self._mapProxyRow2Group[proxyRow]
        group.setExpanded(not group.expanded)
        self._resetData()

    def getExpandedByRow(self, proxyRow):
        group = self._mapProxyRow2Group[proxyRow]
        return group.expanded

    @property
    def _items(self):
        return self._groups

    def items(self):
        return self._groups

    def _resetData(self, remap=True):
        self.beginResetModel()
        if remap:
            self._remapRows()
        self.reset()
        self.endResetModel()

    def _addNewItem(self):
        newItem = self._actionModel.items()[-1]
        modelRow = len(self._actionModel.items()) - 1
        newRecord, newAction = newItem
        if not newAction:
            return

        begDateTime = forceDateTime(newRecord.value('begDate'))
        if not begDateTime or not begDateTime.isValid():
            newRecord.setValue('begDate', toVariant(QtCore.QDateTime.currentDateTime()))

        actionTypeId = newItem[1].getType().id

        added = False

        group = None
        for group in self._groups.groupsIterator:
            if group.actionTypeId != actionTypeId:
                continue

            if group.addItem(modelRow, newItem):
                added = True
                break

        if not added:
            group = self._addGroup(actionTypeId, modelRow, newItem)

        self._resetData()

        return group, newItem

    def addRow(self, *args, **kwargs):
        result = self._actionModel.addRow(*args, **kwargs)
        group, _ = self._addNewItem()
        self.emitRowIndexActivated(self._groups.getGroupProxyRow(group))
        return result

    def emitRowIndexActivated(self, row):
        self.emit(SIGNAL("rowIndexActivated(int)"), row)

    def flags(self, index):
        proxyRow = index.row()

        if proxyRow not in self._mapProxyRow2Group and proxyRow == len(self._groups):
            modelRow = len(self._actionModel.items())

        elif proxyRow not in self._mapProxyRow2Group:
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable
        else:
            group = self._mapProxyRow2Group[proxyRow]
            modelRow = group.getModelRow(proxyRow, self._actionModel)

        return self._actionModel.flags(self._actionModel.index(modelRow, 0))

    def upRow(self, proxyRow):
        if not proxyRow in self._mapProxyRow2Group:
            result = False

        else:
            group = self._mapProxyRow2Group[proxyRow]

            if group.canUpInGroup(proxyRow):
                result = group.upProxyRow(proxyRow)

            else:
                result = self._groups.upGroup(proxyRow, group)

        if result:
            self._remapRows()
            self.emitAllChanged()

        return result


    def downRow(self, proxyRow):
        if not proxyRow in self._mapProxyRow2Group:
            result = False

        else:
            group = self._mapProxyRow2Group[proxyRow]

            if group.canDownInGroup(proxyRow):
                result = group.downProxyRow(proxyRow)

            else:
                result = self._groups.downGroup(proxyRow, group)

        if result:
            self._remapRows()
            self.emitAllChanged()

        return result

    def emitAllChanged(self):
        index1 = self.index(0, 0)
        index2 = self.index(self.rowCount(), self.columnCount())
        self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index1, index2)

    def updateActionAmount(self, proxyRow):
        if proxyRow not in self._mapProxyRow2Group:
            return

        group = self._mapProxyRow2Group[proxyRow]
        modelRow = group.getModelRow(proxyRow, self._actionModel)
        self._actionModel.updateActionAmount(modelRow)

    def getFilledAction(self, newRecord, *args, **kwargs):
        plannedEndDate = directionDate = None
        saveDirectionDate = kwargs.pop('saveDirectionDates', False) and newRecord
        if saveDirectionDate:
            directionDate = forceDateTime(newRecord.value('directionDate'))
            plannedEndDate = forceDateTime(newRecord.value('plannedEndDate'))

        action = self._actionModel.getFilledAction(newRecord, *args, **kwargs)

        if saveDirectionDate:
            action.getRecord().setValue('directionDate', directionDate)
            action.getRecord().setValue('plannedEndDate', plannedEndDate)

        return action
