# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, QDate, QVariant, SIGNAL, QString

from library.TableModel import CTableModel, CTextCol
from library.ICDInDocTableCol   import CICDExInDocTableCol
from library.InDocTable         import (
    CRecordListModel, CBoolInDocTableCol, CDateInDocTableCol, CDateTimeInDocTableCol, CFloatInDocTableCol, CInDocTableCol,
    CInDocTableView, CIntInDocTableCol, CLocItemDelegate, CRBInDocTableCol)
from library.Utils              import forceDate, forceDateTime, forceDouble, forceInt, forceRef, forceString, toVariant

from Events.Action              import CAction, CActionType, CActionTypeCache, getActionDefaultContractId
from Events.ActionProperty      import CToothActionPropertyValueType, CNomenclatureActionPropertyValueType, CStringActionPropertyValueType
from Events.ExecutionPlanDialog import CGetExecutionPlan
from Events.Utils               import getEventActionContract
from Orgs.OrgComboBox           import CContractComboBox


def _getNomenclatureValues(nomenclatureId, cache):
    if nomenclatureId not in cache:
        db = QtGui.qApp.db
        nomenclatureRecord = db.getRecord('rbNomenclature', 'unit_id, dosageValue', nomenclatureId)
        dosageUnitId = forceRef(nomenclatureRecord.value('unit_id'))
        result = {'dosageValue': forceDouble(nomenclatureRecord.value('dosageValue'))}
        if dosageUnitId:
            unitRecord = db.getRecord('rbUnit', 'code, name', dosageUnitId)
            name = ' | '.join([forceString(unitRecord.value('name')), forceString(unitRecord.value('code'))])
        else:
            name = ''
        result['name'] = name
        cache[nomenclatureId] = result

    return cache[nomenclatureId]


class CCheckedActionsModel(CRecordListModel):
    class CLocEnableCol(CBoolInDocTableCol):
        def __init__(self, selector):
            CBoolInDocTableCol.__init__(self, u'Включить', 'checked', 10)
            self.selector = selector

        def toCheckState(self, val, record):
            return CBoolInDocTableCol.toCheckState(self, val, record)


    class CContractInDocTableCol(CInDocTableCol):
        def __init__(self, model):
            CInDocTableCol.__init__(self, u'Договор', 'contract_id', 20)
            self.model = model


        def toString(self, val, record):
            contractId = forceRef(val)
            if contractId:
                names = ['number', 'date', 'resolution']
                record = QtGui.qApp.db.getRecord('Contract', names, contractId)
                str = ' '.join([forceString(record.value(name)) for name in names])
            else:
                str = u'не задано'
            return QVariant(str)


        def createEditor(self, parent):
            eventEditor = self.model.parentWidget.eventEditor
            editor = CContractComboBox(parent)
            editor.setOrgId(eventEditor.orgId)
            editor.setClientInfo(eventEditor.clientId,
                                 eventEditor.clientSex,
                                 eventEditor.clientAge,
                                 eventEditor.clientWorkOrgId,
                                 eventEditor.clientPolicyInfoList)
            return editor

        def getEditorData(self, editor):
            return QVariant(editor.value())


        def setEditorData(self, editor, value, record):
            eventEditor = self.model.parentWidget.eventEditor
            financeId = forceRef(record.value('finance_id')) or eventEditor.eventFinanceId
            editor.setFinanceId(financeId)
            editor.setActionTypeId(forceRef(record.value('actionType_id')))
            contractId = forceRef(value)
            if contractId is None:
                if financeId == eventEditor.eventFinanceId:
                    contractId = eventEditor.contractId
            editor.setValue(contractId)
            editor.setBegDate(forceDate(record.value('directionDate')) or QDate.currentDate())
            editor.setEndDate(forceDate(record.value('endDate')) or QDate.currentDate())


    class CDosagePropertyTableCol(CInDocTableCol):
        def __init__(self, model):
            CInDocTableCol.__init__(self, u'Doses', 'doses', 10)
            self._nomenclatureCache = model._nomenclatureCache
            self._model = model

        def toString(self, index, val, record):
            propertyValue = None

            row = index.row()
            column = index.column()
            actionTypeId = forceRef(record.value('actionType_id'))
            values  = self._model._mapActionTypeIdToPropertyValues.get(actionTypeId, None)
            if values:
                fieldName  = self._model._cols[column].fieldName()
                cellValues = values.get(fieldName, None)
                if cellValues:
                    action = self._model._idRowToAction[(actionTypeId, row)]
                    propertyType = cellValues['propertyType']
                    property = action.getPropertyById(propertyType.id)
                    propertyValue = property.getText()

            if not propertyValue:
                return QVariant()

            nomenclatureId = None
            cellValues = values.get('recipe', None)
            if cellValues:
                action = self._model._idRowToAction[(actionTypeId, row)]
                propertyType = cellValues['propertyType']
                if isinstance(propertyType.valueType, CNomenclatureActionPropertyValueType):
                    property = action.getPropertyById(propertyType.id)
                    nomenclatureId = property.getValue()

            if not nomenclatureId:
                return QVariant('{0:.2f}'.format(propertyValue))

            values = _getNomenclatureValues(nomenclatureId, self._nomenclatureCache)
            name = values.get('name')

            return QVariant(
                u'{0:.2f} {1}'.format(propertyValue, name)
            )


    class CLocDateTimeInDocTableCol(CDateTimeInDocTableCol):

        def createEditor(self, parent):
            editor = QtGui.QDateTimeEdit(parent)
            editor.setDisplayFormat('dd.MM.yyyy HH:mm')
#            editor.setCalendarPopup(True)
            return editor

        def getEditorData(self, editor):
            return toVariant(editor.dateTime())

        def setEditorData(self, editor, value, record):
            editor.setDateTime(forceDateTime(value))

        def toString(self, val, record):
            if not val.isNull():
                return QVariant(forceString(val))
            return QVariant()



    def __init__(self, parent, existsActionsModel, actionTypesModel=None, includeTooth=False):
        CRecordListModel.__init__(self, parent)

        self._nomenclatureCache = {}

        self.addExtCol(CCheckedActionsModel.CLocEnableCol(parent),
                       QVariant.Bool)
        self.addExtCol(CRBInDocTableCol(u'Действие', 'actionType_id', 15, 'ActionType', showFields=2).setReadOnly(),
                       QVariant.Int)
        self.addExtCol(CCheckedActionsModel.CLocDateTimeInDocTableCol(u'Назначить', 'directionDate', 10),
                       QVariant.DateTime)
        self.addExtCol(CCheckedActionsModel.CLocDateTimeInDocTableCol(u'Начать', 'begDate', 10),
                       QVariant.DateTime)
        self.addExtCol(CICDExInDocTableCol(u'МКБ', 'MKB', 7),
                       QVariant.String)
        self.addExtCol(CFloatInDocTableCol(u'Количество', 'amount', 10, precision=2),
                       QVariant.Double)
        if includeTooth:
            self.addExtCol(CInDocTableCol(u'Зуб', 'tooth', 10), QVariant.String)
        self.addExtCol(CIntInDocTableCol(u'Длительность', 'duration', 10),
                       QVariant.Int)
        self.addExtCol(CIntInDocTableCol(u'Интервал', 'periodicity', 10),
                       QVariant.Int)
        self.addExtCol(CIntInDocTableCol(u'Кратность', 'aliquoticity', 10),
                       QVariant.Int)
        self.addExtCol(CDateInDocTableCol(u'План', 'plannedEndDate', 10),
                       QVariant.Date)
        self.addExtCol(CRBInDocTableCol(u'Тип финансирования', 'finance_id', 10, 'rbFinance', showFields=2),
                       QVariant.Int)
        self.addExtCol(CCheckedActionsModel.CContractInDocTableCol(self),
                       QVariant.Int)
        self.addExtCol(CFloatInDocTableCol(u'Сумма', 'price', 10, precision=2),
                       QVariant.Double)
        self.addExtCol(CInDocTableCol(u'Recipe', 'recipe', 10),
                       QVariant.String)
        self.addExtCol(CCheckedActionsModel.CDosagePropertyTableCol(self),
                       QVariant.String)
        self.addExtCol(CInDocTableCol(u'Signa', 'signa', 10),
                       QVariant.String)

        self._existsActionsModel = existsActionsModel
        self._actionTypesModel   = actionTypesModel
        self.clientId = None
        self.parentWidget = parent
        self._table            = QtGui.qApp.db.table('Action')
        self._dbFieldNamesList = [field.fieldName.replace('`', '') for field in self._table.fields]
        self.prices = []
        self._mapPropertyTypeCellsActivity = {}
        self._propertyColsNames = ['recipe', 'doses', 'signa']
        if includeTooth:
            self._propertyColsNames.append('tooth')
        self._propertyColsIndexes = [self.getColIndex(name) for name in self._propertyColsNames]
        self._mapActionTypeIdToPropertyValues = {}
        self._rowToAction = {}
        self._idRowToAction = {}
        self._idToRows = {}

        boldFont = QtGui.QFont()
        boldFont.setWeight(QtGui.QFont.Bold)
        self._qBoldFont = QVariant(boldFont)

    def setClientId(self, clientId):
        self.clientId = clientId

    def cellReadOnly(self, index):
        column = index.column()
        if column == self.getColIndex('plannedEndDate'):
            return not self.isRowPlanEndDateEdited(index.row())
        elif column in self._propertyColsIndexes:
            return not self._mapPropertyTypeCellsActivity.get((index.row(), column), False)
        return False

    def headerData(self, section, orientation, role = Qt.DisplayRole):
        if orientation == Qt.Horizontal:
            if role == Qt.ToolTipRole:
                if section == self.getColIndex('duration'):
                    return QVariant(u'Длительность курса лечения в днях.')
                elif section == self.getColIndex('periodicity'):
                    result = '\n'.join(
                                       [
                                        u'0 - каждый день,',
                                        u'1 - через 1 день,',
                                        u'2 - через 2 дня,',
                                        u'3 - через 3 дня,',
                                        u'и т.д.'
                                       ]
                                      )
                    return QVariant(result)
                elif section == self.getColIndex('aliquoticity'):
                    return QVariant(u'Сколько раз в сутки.')
        return CRecordListModel.headerData(self, section, orientation, role)

    def isRowPlanEndDateEdited(self, row):
        return not (self.isRowDpedBegDatePlusAmount(row) or self.isRowDpedBegDatePlusDuration(row))

    def getDefaultPlannedEndDate(self, actionTypeId):
        return CActionTypeCache.getById(actionTypeId).defaultPlannedEndDate

    def isDpedBegDatePlusAmount(self, actionTypeId):
        return self.getDefaultPlannedEndDate(actionTypeId) == CActionType.dpedBegDatePlusAmount

    def isDpedBegDatePlusDuration(self, actionTypeId):
        return self.getDefaultPlannedEndDate(actionTypeId) == CActionType.dpedBegDatePlusDuration

    def isRowDpedBegDatePlusAmount(self, row):
        actionTypeId = forceRef(self.items()[row].value('actionType_id'))
        return self.isDpedBegDatePlusAmount(actionTypeId)

    def isRowDpedBegDatePlusDuration(self, row):
        actionTypeId = forceRef(self.items()[row].value('actionType_id'))
        return self.isDpedBegDatePlusDuration(actionTypeId)

    def regenerate(self):
        count = len(self.items())
        self.prices = [0.0]*count
        self.updatePricesAndSums(0, count-1)


#    def getActionRecord(self, actionTypeId):
#        row = self._idToRow[actionTypeId]
#        item = self._items[row]
#        return self.getClearActionRecord(item)

    def getPropertiesValues(self, actionTypeId):
        result = {}
        values = self._mapActionTypeIdToPropertyValues.get(actionTypeId, None)
        if values:
            for value in values.values():
                if value['value'].isValid():
                    result[value['propertyType'].id] = value['value']
        return result

    def getContractId(self):
        return self.parentWidget.getContractId()

    def getPrice(self, record):
        actionTypeId = forceRef(record.value('actionType_id'))
        financeId    = forceRef(record.value('finance_id'))
        contractId   = forceRef(record.value('contract_id'))
        return self.getPriceEx(actionTypeId, contractId, financeId)

    def getPriceEx(self, actionTypeId, contractId, financeId):
        return self.parentWidget.getPrice(actionTypeId, contractId, financeId)

    def getClearActionRecord(self, item):
        record = self._table.newRecord()
        for iCol in xrange(record.count()):
            fieldName = record.fieldName(iCol)
            record.setValue(fieldName, item.value(fieldName))
        return record

    def insertSameActionByRow(self, row):
        if row >= 0:
            actionTypeId = self._rowToAction[row].getType().id
            self.parentWidget.insertActionIntoCheckedModel(actionTypeId)


    def add(self, actionTypeId, amount=None, financeId=None, contractId=None):
        row = len(self._items)
        record = self.getEmptyRecord(self.getPropertyTypeCellsSettings(actionTypeId, row))
        action = CAction.getFilledAction(self.parentWidget.eventEditor, record, actionTypeId, initPresetValues=True)
        record.setValue('checked', QVariant(Qt.Checked))
        record.setValue('price', QVariant(self.getPrice(record)))
        if amount:
            record.setValue('amount', QVariant(amount))
        if financeId:
            record.setValue('finance_id', QVariant(financeId))
        if contractId:
            record.setValue('contract_id', QVariant(contractId))
        self.insertRecord(row, record)

        rows = self._idToRows.setdefault(actionTypeId, [])
        rows.append(row)

#        action = CAction(record=record)
        self._idRowToAction[(actionTypeId, row)] = action
        self._rowToAction[row] = action

        self.prices.append(0.0)
        if not self.isRowPlanEndDateEdited(row):
            self.updatePlannedEndDate(row)
        self.emitPricesAndSumsUpdated()

    def getSelectedAction(self, actionTypeId):
        result = []
        rows = self._idToRows[actionTypeId]
        for row in rows:
            action = self._idRowToAction[(actionTypeId, row)]
            action._record = self.getClearActionRecord(action._record)
            result.append(action)
        return result

    def getPropertyTypeCellsSettings(self, actionTypeId, row):
        cellSettings = {}
        actionType = CActionTypeCache.getById(actionTypeId)
        toothEists = False
        for propertyType in actionType.getPropertiesById().values():
            if propertyType.inActionsSelectionTable:
                column = self._propertyColsIndexes[propertyType.inActionsSelectionTable-1]
                result = self._mapPropertyTypeCellsActivity.get((row, column), None)
                if result is None:
                    self._cols[column].setValueType(propertyType.valueType.variantType)
                    self._mapPropertyTypeCellsActivity[(row, column)] = True
                    cellName = self._propertyColsNames[propertyType.inActionsSelectionTable-1]
                    cellSettings[cellName] = True
                    values = self._mapActionTypeIdToPropertyValues.get(actionTypeId, None)
                    if values is None:
                        values = {cellName: {'propertyType': propertyType, 'value':QVariant()}}
                        self._mapActionTypeIdToPropertyValues[actionTypeId] = values
                    else:
                        values[cellName] = {'propertyType': propertyType, 'value':QVariant()}

            if isinstance(propertyType.valueType, CToothActionPropertyValueType) and not toothEists: # wtf
                column = self.getColIndex(u'tooth')
                result = self._mapPropertyTypeCellsActivity.get((row, column), None)
                if result is None:
                    self._cols[column].setValueType(propertyType.valueType.variantType)
                    self._mapPropertyTypeCellsActivity[(row, column)] = True
                    cellName = u'tooth'
                    cellSettings[cellName] = True
                    values = self._mapActionTypeIdToPropertyValues.get(actionTypeId, None)
                    if values is None:
                        values = {cellName: {'propertyType': propertyType, 'value':QVariant()}}
                        self._mapActionTypeIdToPropertyValues[actionTypeId] = values
                    else:
                        values[cellName] = {'propertyType': propertyType, 'value':QVariant()}
                toothEists = True
        return cellSettings


    def removeAll(self):
        self.removeRows(0, len(self._items))
        self._idToRows.clear()
        self._rowToAction.clear()
        self._idRowToAction.clear()
        self._items = []
        self.prices = []
        self.emitPricesAndSumsUpdated()


    def remove(self, actionTypeId):
        rows = self._idToRows[actionTypeId]
        deletedRows = []
        for row in rows:
            rowShift = 0
            for deletedRow in deletedRows:
                rowShift = rowShift+1 if row > deletedRow else rowShift
            self.removeRows(row-rowShift, 1)
            deletedRows.append(row)
            action = self._rowToAction.pop(row)
            jobTicketId = action.findFireableJobTicketId()
            if jobTicketId:
                QtGui.qApp.jobTicketReserveHolder.delJobTicketReservation(jobTicketId)
        self._idRowToAction.clear()
        self._idToRows.clear()

        tmpRowToAction = {}
        tmpPrices = []
        for newRow, (oldRow, action) in enumerate(self._rowToAction.items()):
            actionTypeId = action.getType().id
            tmpRowToAction[newRow] = action
            self._idRowToAction[(actionTypeId, newRow)] = action
            rows = self._idToRows.setdefault(actionTypeId, [])
            rows.append(newRow)

            tmpPrices.append(self.prices[oldRow])

        self.prices = tmpPrices
        self._rowToAction = tmpRowToAction

        self.emitPricesAndSumsUpdated()



    def getEmptyRecord(self, propertyTypeCellsSettings={}):
        record = self._table.newRecord()
        for col in self._cols:
            if col.fieldName() not in self._dbFieldNamesList:
                record.append(QtSql.QSqlField(col.fieldName(), col.valueType()))
        for name in self._propertyColsNames:
            isActive = propertyTypeCellsSettings.get(name, False)
            if not isActive:
                record.setValue(name, QVariant(u'----'))
        return record


    def hasExistsInSelected(self):
        for id in self._idToRows.keys():
            if self._existsActionsModel.hasActionTypeId(id):
                return True
        return False


    def flags(self, index):
        column = index.column()
        if column == self.getColIndex('MKB'):
            action = self._rowToAction[index.row()]
            actionType = action.getType()
            if actionType.defaultMKB == CActionType.dmkbNotUsed:
                return Qt.ItemIsSelectable # | Qt.ItemIsEnabled
        return CRecordListModel.flags(self, index)


    def data(self, index, role=Qt.DisplayRole):
        column = index.column()

        if column == self.getColIndex('doses') and role in (Qt.StatusTipRole, Qt.DisplayRole):
            row = index.row()
            col = self._cols[column]
            record = self._items[row]
            return col.toString(index, record.value(col.fieldName()), record)

        if role == Qt.DisplayRole:
            if column == self.getColIndex('actionType_id'):
                row = index.row()
                record = self._items[row]
                outName = forceString(record.value('specifiedName'))
                actionTypeId = forceRef(record.value('actionType_id'))
                if actionTypeId:
                    actionType = CActionTypeCache.getById(actionTypeId)
                    if actionType:
                        actionTypeName = u' | '.join([actionType.code, actionType.name])
                        outName = actionTypeName + ' ' +outName if outName else actionTypeName
                    return QVariant(outName)

            elif column in self._propertyColsIndexes:
                row                 = index.row()
                actionTypeId        = forceRef(self._items[row].value('actionType_id'))
                values              = self._mapActionTypeIdToPropertyValues.get(actionTypeId, None)
                if values:
                    fieldName           = self._cols[column].fieldName()
                    cellValues          = values.get(fieldName, None)
                    if cellValues:
                        action              = self._idRowToAction[(actionTypeId, row)]
                        propertyType        = cellValues['propertyType']
                        property            = action.getPropertyById(propertyType.id)
                        return toVariant(property.getText())

        elif role == Qt.FontRole:
            row = index.row()
            record = self._items[row]
            actionTypeId = forceRef(record.value('actionType_id'))
            if self._existsActionsModel.hasActionTypeId(actionTypeId):
                return self._qBoldFont
        elif role == Qt.BackgroundRole and column == self.getColIndex('checked'):
            row = index.row()
            record = self._items[row]
            actionTypeId = forceRef(record.value('actionType_id'))
            return self._actionTypesModel.getColor(actionTypeId)
        return CRecordListModel.data(self, index, role)


    def setSelected(self, actionTypeId, row, value):
        if len(self._idToRows[actionTypeId]) == 1:
            self.parentWidget.setSelected(actionTypeId, value, resetMainModel=True)
        else:
            self.removeRows(row, 1)
            del self._rowToAction[row]
            self._idRowToAction.clear()
            self._idToRows.clear()
            tmpRowToAction = {}
            tmpPrices = []
            for newRow, (oldRow, action) in enumerate(self._rowToAction.items()):
                actionTypeId = action.getType().id
                tmpRowToAction[newRow] = action
                self._idRowToAction[(actionTypeId, newRow)] = action
                rows = self._idToRows.setdefault(actionTypeId, [])
                rows.append(newRow)
                tmpPrices.append(self.prices[oldRow])

            self.prices = tmpPrices
            self._rowToAction = tmpRowToAction

            self.emitPricesAndSumsUpdated()


    def setData(self, index, value, role=Qt.EditRole):
        column = index.column()
        row = index.row()
        record = self._items[row]
        if role == Qt.CheckStateRole and column == 0:
            actionTypeId = forceRef(record.value('actionType_id'))
            self.setSelected(actionTypeId, row, forceInt(value) == Qt.Checked)
            return False

        result = CRecordListModel.setData(self, index, value, role)

        col = self.cols()[column]
        fieldName = col.fieldName()

        if fieldName == 'finance_id':
            self.initContract(row)
            self.updatePricesAndSums(row, row)
        elif fieldName == 'contract_id':
            self.updatePricesAndSums(row, row)
        elif fieldName == 'amount':
            amount = forceDouble(value)
            actionTypeId = forceRef(record.value('actionType_id'))
            personId = forceRef(record.value('person_id'))
            financeId = forceRef(record.value('finance_id'))
            contractId = forceRef(record.value('contract_id'))
            eventEditor = self.parentWidget.eventEditor
            uet = amount*eventEditor.getUet(actionTypeId, personId, financeId, contractId)
            record.setValue('uet', toVariant(uet))
            self.updatePricesAndSums(row, row)
            if self.isRowDpedBegDatePlusAmount(row):
                self.updatePlannedEndDate(row)
        elif fieldName == 'duration':
            self.updatePlannedEndDate(row)
            self.updateExecutionPlanByRecord(row)
        elif fieldName == 'periodicity':
            self.updateExecutionPlanByRecord(row)
        elif fieldName == 'aliquoticity':
            self.updateExecutionPlanByRecord(row)
        elif fieldName == 'begDate':
            actionType = CActionTypeCache.getById(forceRef(record.value('actionType_id')))
            if actionType.defaultEndDate == CActionType.dedSyncActionBegDate:
                record.setValue('endDate', toVariant(record.value('begDate')))
            self.updatePlannedEndDate(row)
            self.updateExecutionPlanByRecord(row)
        elif fieldName == 'recipe':
            actionTypeId        = forceRef(record.value('actionType_id'))
            action              = self._idRowToAction[(actionTypeId, row)]
            values              = self._mapActionTypeIdToPropertyValues[actionTypeId]
            propertyType        = values[fieldName]['propertyType']
            property            = action.getPropertyById(propertyType.id)
            property.preApplyDependents(action)

            if isinstance(propertyType.valueType, CNomenclatureActionPropertyValueType):
                (nomenclatureId, financeId) = value
                if nomenclatureId:
                    if financeId:
                        self.setData(self.index(row, self.getColIndex('finance_id')), toVariant(financeId))
                    nomenclatureValues = _getNomenclatureValues(nomenclatureId, self._nomenclatureCache)
                    dosageValue = nomenclatureValues.get('dosageValue')
                    dosesColumnIndex = self.getColIndex('doses')
                    dosesIndex = self.index(row, dosesColumnIndex)
                    dosesPropertyType = values['doses']['propertyType']
                    dosesProperty = action.getPropertyById(dosesPropertyType.id)
                    dosesProperty.setValue(dosageValue)
                    self.setData(dosesIndex, toVariant(dosageValue))

                    usingTypes = self.getNomenclatureUsingTypes(nomenclatureId)
                    if usingTypes:
                        signaValues = values.get('signa')
                        if signaValues:
                            signaPropertyType = signaValues['propertyType']
                            if isinstance(signaPropertyType.valueType, CStringActionPropertyValueType):
                                signaProperty = action.getPropertyById(signaPropertyType.id)
                                signaProperty.setValue(usingTypes[0])
                                signaColumnIndex = self.getColIndex('signa')
                                signaIndex = self.index(row, signaColumnIndex)
                                self.setData(signaIndex, toVariant(usingTypes[0]))

            property.applyDependents(action)

        elif fieldName == 'doses':
            actionTypeId = forceRef(record.value('actionType_id'))
            action = self._idRowToAction[(actionTypeId, row)]
            values = self._mapActionTypeIdToPropertyValues[actionTypeId]
            propertyType = values['recipe']['propertyType']
            if isinstance(propertyType.valueType, CNomenclatureActionPropertyValueType):
                property = action.getPropertyById(propertyType.id)
                nomenclatureId = property.getValue()
                if nomenclatureId:
                    action.updateNomenclatureDosageValue(nomenclatureId, forceDouble(value))
                    self.updateExecutionPlanByRecord(row)


        return result

    def updateExecutionPlanByRecord(self, row):
        actionTypeId = forceRef(self._items[row].value('actionType_id'))
        action = self._idRowToAction[(actionTypeId, row)]
        action.updateExecutionPlanByRecord()


    def updatePlannedEndDate(self, row):
        if self.isRowDpedBegDatePlusAmount(row):
            item = self.items()[row]
            begDate = forceDate(item.value('begDate'))
            amountValue = forceInt(item.value('amount'))
            date = begDate.addDays(amountValue-1) if begDate and amountValue else QDate()
            item.setValue('plannedEndDate', toVariant(date))
            self.emitValueChanged(row, 'plannedEndDate')
        else:
            item = self.items()[row]
            begDate = forceDate(item.value('begDate'))
            durationValue = forceInt(item.value('duration'))
            date = begDate.addDays(durationValue-1) if begDate and durationValue else QDate()
            item.setValue('plannedEndDate', toVariant(date))
            self.emitValueChanged(row, 'plannedEndDate')

    def initContract(self, row):
        item = self.items()[row]
        actionTypeId = forceRef(item.value('actionType_id'))
        financeId = forceRef(item.value('finance_id'))
        if financeId:
            eventEditor = self.parentWidget.eventEditor
            if financeId == eventEditor.eventFinanceId and getEventActionContract(eventEditor.eventTypeId):
                contractId = eventEditor.contractId
            else:
                begDate = forceDate(item.value('directionDate'))
                endDate = forceDate(item.value('endDate'))
                contractId = getActionDefaultContractId(self.parentWidget.eventEditor,
                                                        actionTypeId,
                                                        financeId,
                                                        begDate,
                                                        endDate,
                                                        None)
        else:
            contractId = None
        item.setValue('contract_id', toVariant(contractId))
        self.emitValueChanged(row, 'contract_id')

    def getTotalSum(self):
        result = 0
        for item in self._items:
            result += forceDouble(item.value('price'))
        return result

    def updatePricesAndSums(self, top, bottom):
        sumChanged = False
        for i in xrange(top, bottom+1):
            sumChanged = self.updatePriceAndSum(i, self.items()[i]) or sumChanged
        self.emitPricesAndSumsUpdated()

    def emitPricesAndSumsUpdated(self):
        self.emit(SIGNAL('pricesAndSumsUpdated()'))


    def updatePriceAndSum(self, row, item):
        actionTypeId = forceRef(item.value('actionType_id'))
        eventEditor = self.parentWidget.eventEditor
        if actionTypeId:
            contractId = forceRef(item.value('contract_id'))
            if contractId:
                financeId = forceRef(QtGui.qApp.db.translate('Contract', 'id', contractId, 'finance_id'))
            else:
                contractId = eventEditor.contractId
                financeId = eventEditor.eventFinanceId
            price = self.getPriceEx(actionTypeId, contractId, financeId) * forceDouble(item.value('amount'))
        else:
            price = 0.0
        if self.prices[row] != price:
            self.prices[row] = price
            item.setValue('price', price)
            self.emitValueChanged(row, 'price')


    def createEditor(self, index, parent):
        column = index.column()
        if column in self._propertyColsIndexes:
            return self.createPropertyEditor(index, parent)
        else:
            return CRecordListModel.createEditor(self, index, parent)

    def createPropertyEditor(self, index, parent):
        row          = index.row()
        column       = index.column()
        actionTypeId = forceRef(self._items[row].value('actionType_id'))
        values       = self._mapActionTypeIdToPropertyValues[actionTypeId]
        fieldName    = self._cols[column].fieldName()
        cellValues   = values[fieldName]
        propertyType = cellValues['propertyType']
        eventEditor  = self.parentWidget.eventEditor
        # actionType = CActionTypeCache.getById(actionTypeId)
        action = self._idRowToAction[(actionTypeId, row)]
        # action = CAction(actionType)
        if column == self.getColIndex(u'recipe'):
            editor       = propertyType.createEditor(action, parent, self.clientId, eventEditor.eventTypeId,  eventEditor)
        else:
            editor       = propertyType.createEditor(action, parent, self.clientId, eventEditor.eventTypeId)

        if propertyType.inActionsSelectionTable == 3:  # signa
            if isinstance(propertyType.valueType, CStringActionPropertyValueType):
                recipeCellValues = values.get('recipe')
                if not recipeCellValues:
                    return editor

                recipePropertyType = recipeCellValues['propertyType']
                if not isinstance(recipePropertyType.valueType, CNomenclatureActionPropertyValueType):
                    return editor

                recipeProperty = action.getPropertyById(recipePropertyType.id)
                nomenclatureId = recipeProperty.getValue()
                if not nomenclatureId:
                    return editor

                usingTypes = self.getNomenclatureUsingTypes(nomenclatureId)
                if not usingTypes:
                    return editor

                if not isinstance(editor, QtGui.QComboBox):
                    editorClass = propertyType.valueType.CComboBoxPropEditor
                    editor = editorClass(
                        action, propertyType.valueType.domain, parent, self.clientId, eventEditor.eventTypeId
                    )

                editor.addItems(usingTypes)

        return editor

    def getNomenclatureUsingTypes(self, nomenclatureId):
        if not nomenclatureId:
            return []

        db = QtGui.qApp.db
        table = db.table('rbNomenclature_UsingType')
        recordList = db.getRecordList(
            table, 'usingType', where=table['master_id'].eq(nomenclatureId), order=table['idx'].name()
        )
        return [forceString(r.value('usingType')) for r in recordList]

    def setEditorData(self, index, editor, value, record):
        column = index.column()
        if column in self._propertyColsIndexes:
            return self.setPropertyEditorData(index, editor, value, record)
        else:
            return CRecordListModel.setEditorData(self, column, editor, value, record)

    def setPropertyEditorData(self, index, editor, value, record):
        editor.setValue(value)


    def getEditorData(self, index, editor):
        column = index.column()
        if column in self._propertyColsIndexes:
            return self.getPropertyEditorData(index, editor)
        else:
            return CRecordListModel.getEditorData(self, column, editor)


    def getPropertyEditorData(self, index, editor):
        value               = editor.value()
        row                 = index.row()
        column              = index.column()
        actionTypeId        = forceRef(self._items[row].value('actionType_id'))
        values              = self._mapActionTypeIdToPropertyValues[actionTypeId]
        fieldName           = self._cols[column].fieldName()
        cellValues          = values[fieldName]
        cellValues['value'] = toVariant(value)
        action              = self._idRowToAction[(actionTypeId, row)]
        propertyType        = cellValues['propertyType']
        property            = action.getPropertyById(propertyType.id)
        value = propertyType.convertQVariantToPyValue(value) if type(value) == QVariant else value
        property.setValue(value)
        if property.isActionNameSpecifier():
            action.updateSpecifiedName()
            self.emitValueChanged(row, 'actionType_id')
        if column == self.getColIndex(u'recipe'):
            return (value,  editor._popup._financeId if hasattr(editor, '_popup') else None)
        return value





# ###########################################################

class CCheckedActionsItemDelegate(CLocItemDelegate):
    def createEditor(self, parent, option, index):
        column = index.column()
        editor = index.model().createEditor(index, parent)
        self.connect(editor, SIGNAL('commit()'), self.emitCommitData)
        self.connect(editor, SIGNAL('editingFinished()'), self.commitAndCloseEditor)
        self.editor   = editor
        self.row      = index.row()
        self.rowcount = index.model().rowCount(None)
        self.column   = column
        return editor


    def setEditorData(self, editor, index):
        if editor is not None:
            row    = index.row()
            model  = index.model()
            if row < len(model.items()):
                record = model.items()[row]
            else:
                record = model.getEmptyRecord()
            model.setEditorData(index, editor, model.data(index, Qt.EditRole), record)


    def setModelData(self, editor, model, index):
        if editor is not None:
            model.setData(index, index.model().getEditorData(index, editor))


    def updateEditorGeometry(self, editor, option, index):
        def checkOnTextEdit(widget):
            if isinstance(widget, QtGui.QTextEdit):
                return True
            for child in widget.children():
                if checkOnTextEdit(child):
                    return True
            return False
        CLocItemDelegate.updateEditorGeometry(self, editor, option, index)
        if checkOnTextEdit(editor):
            editor.resize(editor.width(), 8*editor.height())


class CCheckedActionsTableView(CInDocTableView):
    def __init__(self, parent):
        CInDocTableView.__init__(self, parent)
        self.setItemDelegate(CCheckedActionsItemDelegate(self))
        self.__actGetExecutionPlan = None
        self.__actInsertSameAction = None


    def addGetExecutionPlan(self):
        if self._popupMenu is None:
            self.createPopupMenu()
        self.__actGetExecutionPlan = QtGui.QAction(u'План выполнения назначений', self)
        self.__actGetExecutionPlan.setObjectName('actGetExecutionPlan')
        self.__actGetExecutionPlan.setShortcut(Qt.Key_F2)
        self.addAction(self.__actGetExecutionPlan)
        self._popupMenu.addAction(self.__actGetExecutionPlan)
        self.connect(self.__actGetExecutionPlan, SIGNAL('triggered()'), self.getExecutionPlan)


    def addInsertSameAction(self):
        if self._popupMenu is None:
            self.createPopupMenu()
        self.__actInsertSameAction = QtGui.QAction(u'Добавить действие подобного типа', self)
        self.__actInsertSameAction.setShortcut(Qt.Key_F9)
        self.addAction(self.__actInsertSameAction)
        self._popupMenu.addAction(self.__actInsertSameAction)
        self.connect(self.__actInsertSameAction, SIGNAL('triggered()'), self.on_insertSameAction)


    def on_insertSameAction(self):
        currentIndex = self.currentIndex()
        self.model().insertSameActionByRow(currentIndex.row())


    def getExecutionPlan(self):
        currentRow = self.currentIndex().row()
        model = self.model()
        items = model.items()
        if 0 <= currentRow < len(items):
            actionTypeId = forceRef(items[currentRow].value('actionType_id'))
            action = model._idRowToAction[(actionTypeId, currentRow)] if actionTypeId else None
            record = action.getRecord() if action else None
            if record and action:
                dialog = CGetExecutionPlan(self, record, action.getExecutionPlan())
                dialog.exec_()
                begDate = forceDateTime(record.value('begDate'))
                duration = forceInt(record.value('duration'))
                record.setValue('plannedEndDate', toVariant(begDate.addDays(duration-1)))
                action.setExecutionPlan(dialog.model.executionPlan)


    def on_popupMenu_aboutToShow(self):
        row = self.currentIndex().row()
        rowCount = len(self.model().items())
        if self.__actGetExecutionPlan:
            self.__actGetExecutionPlan.setEnabled(0<=row<rowCount)
        if self.__actInsertSameAction:
            self.__actInsertSameAction.setEnabled(0<=row<rowCount)
