# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sip

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QDate, QDateTime, QModelIndex, QString, QVariant, SIGNAL

from library.ICDInDocTableCol   import CICDExInDocTableCol
from library.InDocTable         import CInDocTableModel, CBoolInDocTableCol, CDateInDocTableCol, CEnumInDocTableCol, CFloatInDocTableCol, CInDocTableCol, CRBInDocTableCol
from library.Utils              import forceBool, forceDate, forceDateTime, forceDouble, forceInt, forceRef, forceString, toVariant

from Events.Action              import CActionTypeCache
from Events.ActionStatus        import CActionStatus
from Events.ActionTypeComboBox  import CActionTypeTableCol
from Events.ContractTariffCache import CContractTariffCache
from Events.Utils               import CFinanceType, payStatusText, getEventActionContract
from Orgs.OrgComboBox           import CContractComboBox, CContractDbModel, COrgInDocTableColEx
from Orgs.PersonComboBoxEx      import CPersonWithOrgListComboBoxEx
from Users.Rights               import urEditAfterInvoicingEvent


class CActionPerson(CRBInDocTableCol):
    def __init__(self, title, fieldName, width, tableName, **params):
        CRBInDocTableCol.__init__(self, title, fieldName, width, tableName, **params)
        self._parent = params.get('parent', None)
        self._useFilter = params.get('useFilter', None)
        self.filter = None

    def setIndex(self, index):
        if self._useFilter:
            self.filter = None
            filter = []
            model = index.model()
            if len(model._items) == 0:
                return
            record = model._items[index.row()]
            actionTypeId = forceInt(record.value('actionType_id'))
            actionType = CActionTypeCache.getById(actionTypeId)
            orgStructureList = actionType.getPFOrgStructureRecordList()
            specialityList = actionType.getPFSpecialityRecordList()
            tablePerson = QtGui.qApp.db.table('vrbPersonWithSpeciality')
            orgStructureIdList = [forceInt(orgStructureRecord.value('orgStructure_id')) for orgStructureRecord in orgStructureList]
            specialityIdList   = [forceInt(specialityRecord.value('speciality_id'))     for specialityRecord in specialityList]
            if orgStructureIdList:
                filter.append(tablePerson['orgStructure_id'].inlist(orgStructureIdList))
            if specialityIdList:
                filter.append(tablePerson['speciality_id'].inlist(specialityIdList))
            self.filter = QtGui.qApp.db.joinAnd(filter)


    def createEditor(self, parent):
        editor = CPersonWithOrgListComboBoxEx(parent)
        if self.filter:
            editor.setFilter(self.filter)
        editor.setBegDate(self._parent.getSetDateTime().date())
        editor.setPreferredWidth(self.preferredWidth)
        return editor


class CActionsSummaryModel(CInDocTableModel):
    class CUetCol(CFloatInDocTableCol):
        def __init__(self, model):
#            CFloatInDocTableCol.__init__(self, u'УЕТ',  'amount', 6, precision=2)
            CFloatInDocTableCol.__init__(self, u'УЕТ',  'uet', 6, precision=2)
            self.model = model
            self.setReadOnly()


        def getUet(self, value, record):
            actionTypeId = forceRef(record.value('actionType_id'))
            personId = forceRef(record.value('person_id'))
            financeId = forceRef(record.value('finance_id'))
            contractId = forceRef(record.value('contract_id'))
#            result = forceDouble(value)*self.model.eventEditor.getUet(actionTypeId, personId, financeId, contractId)
            result = forceDouble(record.value('amount'))*self.model.eventEditor.getUet(actionTypeId, personId, financeId, contractId)
            return result


        def toString(self, value, record):
            if value.isNull():
                return value
            if not record:
                return QVariant()

            v = self.getUet(value, record)
            s = QString()
            if self.precision is None:
                s.setNum(v)
            else:
                s.setNum(v, 'f', self.precision)
            return toVariant(s)


    class CPayStatusCol(CInDocTableCol):
        def __init__(self):
            CInDocTableCol.__init__(self, u'Оплата', 'payStatus', 10)
            self.setReadOnly()


        def toString(self, value, record):
            payStatus = forceInt(value)
            return toVariant(payStatusText(payStatus))


    # модель сводки действий
    __pyqtSignals__ = ('currentRowMovedTo(int)',
                       'amountChanged()',
                      )

    def __init__(self, parent, editable=False):
        CInDocTableModel.__init__(self, 'Action', 'id', 'event_id', parent)
        self.addCol(CActionTypeTableCol(u'Тип',           'actionType_id', 15, None, classesVisible=True)) #0
        self.addCol(CICDExInDocTableCol(u'МКБ',           'MKB', 10)) #1
        self.addCol(CBoolInDocTableCol(u'Срочный',        'isUrgent', 10)) #2
        self.addCol(CDateInDocTableCol(u'Назначено',      'directionDate', 15, canBeEmpty=True)) #3
        self.addCol(CDateInDocTableCol(u'Начато',         'begDate',       15, canBeEmpty=True)) #4
        self.addCol(CDateInDocTableCol(u'Окончено',       'endDate',       15, canBeEmpty=True)) #5
        self.addCol(CEnumInDocTableCol(u'Состояние',      'status',        10, CActionStatus.names)) #6
        self.addCol(CActionPerson(u'Назначил',            'setPerson_id',  20, 'vrbPersonWithSpecialityAndOrgStr', parent=parent)) #7
        self.addCol(CActionPerson(u'Выполнил',            'person_id',  20, 'vrbPersonWithSpecialityAndOrgStr', parent=parent, useFilter=True)) #8
        self.addCol(CActionPerson(u'Ассистент',           'assistant_id',  20, 'vrbPersonWithSpecialityAndOrgStr', parent=parent)) #9
        self.addCol(CInDocTableCol(u'Каб',                'office',                                6)) # 10
        self.addCol(CFloatInDocTableCol(u'Кол-во',        'amount',                                6, precision=2)) # 11
        self.colUet = self.addCol(CActionsSummaryModel.CUetCol(self)) #12
        self.addCol(COrgInDocTableColEx(u'Место выполнения', 'org_id', 30)) #13
        self.addCol(CInDocTableCol(u'Примечания',         'note',                                 40)) # 14

        self.models = []
        self.itemIndex = []
        self.editable = editable
        self.setEnableAppendLine(self.editable)
        self.eventEditor = parent
        self.connect(self.eventEditor, SIGNAL('updateActionsPriceAndUet()'), self.updateActionsPriceAndUet)

        self.visitList = {}
        self.readOnly = False


    def setReadOnly(self, value):
        self.readOnly = value


    def addModel(self, model):
        self.connect(model, SIGNAL('dataChanged(QModelIndex, QModelIndex)'), self.onDataChanged)
        self.connect(model, SIGNAL('modelReset'), self.onModelReset)
        self.connect(model, SIGNAL('rowsInserted(QModelIndex, int, int)'), self.onRowInserted)
        self.connect(model, SIGNAL('rowsRemoved(QModelIndex, int, int)'), self.onRowRemoved)
        self.connect(model, SIGNAL('amountChanged(int)'), self.onAmountChanged)
        self.models.append(model)

    def translate2ActionsSummaryRow(self, model, modelRow):
        try:
            iModel = self.models.index(model)
            return self.itemIndex.index((iModel, modelRow))
        except:
            return None

    def regenerate(self):
        items = []
        itemIndex = []
        for i, model in enumerate(self.models):
            for j, (record, action) in enumerate(model.items()):
                items.append(record)
                itemIndex.append((i, j))
        self.itemIndex = itemIndex
        self.setItems(items)


    def onDataChanged(self, topLeft, bottomRight):
        iModel = self.models.index(topLeft.model())
        top = topLeft.row()
        bottom = bottomRight.row()
        try:
            newTop = self.itemIndex.index((iModel, top))
        except:
            newTop = 0
        try:
            newBottom = self.itemIndex.index((iModel, bottom))
        except:
            newBottom = len(self.itemIndex)-1
        self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), self.index(newTop, 0), self.index(newBottom, len(self.cols())-1))
        #self.regenerate()


    def onModelReset(self):
        self.regenerate()


    def onRowInserted(self, parent, start, end):
        self.regenerate()


    def onRowRemoved(self, parent, start, end):
        self.regenerate()


    def onAmountChanged(self, row):
        self.emit(SIGNAL('amountChanged()'))


    def isLocked(self, row):
        iModel, iAction = self.itemIndex[row]
        actionsModel = self.models[iModel]
        return actionsModel.isLocked(iAction)


    def isLockedOrExposed(self, row):
        iModel, iAction = self.itemIndex[row]
        actionsModel = self.models[iModel]
        return actionsModel.isLockedOrExposed(iAction)


    def flags(self, index = QModelIndex()):
        if self.readOnly:
            return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        if self.editable:
            row = index.row()
            column = index.column()
            items = self.items()
            if 0<=row<len(items):
                iModel, iAction = self.itemIndex[row]
                actionsModel = self.models[iModel]
                if actionsModel.isLocked(iAction) or column==0 or forceInt(items[row].value('payStatus')): # actionType
                    return Qt.ItemIsSelectable | Qt.ItemIsEnabled
#            if column == 9: # assistant
            if column == self.getColIndex('assistant_id'):
                row = index.row()
                record = items[row] if 0<=row<len(items) else None
                actionTypeId = forceRef(record.value('actionType_id')) if record else None
                if actionTypeId:
                    actionType = CActionTypeCache.getById(actionTypeId)
                    if actionType.hasAssistant:
                        Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsEditable
                    else:
                        return Qt.ItemIsSelectable
#            if column == 11: # amount
            if column == self.getColIndex('amount'):
                items = self.items()
                record = items[row] if 0<=row<len(items) else None
                actionTypeId = forceRef(record.value('actionType_id')) if record else None
                if actionTypeId:
                    actionType = CActionTypeCache.getById(actionTypeId)
                    if actionType.amountEvaluation == 0:
                        return Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsEditable
                    else:
                        return Qt.ItemIsSelectable
#            if column == 12: # uet
            if column == self.getColIndex('uet'):
                return Qt.ItemIsSelectable
            return CInDocTableModel.flags(self, index)
        else:
            return Qt.ItemIsSelectable | Qt.ItemIsEnabled


    def canChangeAction(self, payStatus, actionId, eventId):
        if not payStatus and actionId and eventId:
            db = QtGui.qApp.db
            table = db.table('Action')
            cond = [ table['event_id'].eq(eventId),
                     table['id'].eq(actionId),
                     table['payStatus'].ne(0),
                     table['deleted'].eq(0)
                   ]
            record = db.getRecordEx(table, [table['payStatus']], where=cond)
            payStatus = forceInt(record.value('payStatus')) if record else 0
        if payStatus and not QtGui.qApp.userHasRight(urEditAfterInvoicingEvent):
            return False
        return True


    def setData(self, index, value, role=Qt.EditRole, presetAction=None):
        column = index.column()
        row = index.row()
        payStatus = forceRef(self.items()[row].value('payStatus')) if 0 <= row < len(self.items()) else 0
        actionId = forceRef(self.items()[row].value('id')) if 0 <= row < len(self.items()) else None
        eventId = forceRef(self.items()[row].value('event_id')) if 0 <= row < len(self.items()) else None
        if not self.canChangeAction(payStatus, actionId, eventId):
            return False
        items = self.items()
        cols = self.cols()
        col = cols[column]
        fieldName = col.fieldName()
        if role == Qt.EditRole:
            if row == self.rowCount()-1:
                if value.isNull() or fieldName != 'actionType_id':
                    return False
                actionTypeId = forceRef(value)
                actionTypeClass = CActionTypeCache.getById(actionTypeId).class_ if actionTypeId else None
                for iModel, model in enumerate(self.models):
                    if model.actionTypeClass == actionTypeClass:
                        model.addRow(forceRef(value), presetAction=presetAction)
                        self.regenerate()
                        i = self.itemIndex.index((iModel, model.rowCount()-2))
                        self.emit(SIGNAL('currentRowMovedTo(int)'), i)
                        items = self.items()
                        record = items[row]
                        self.eventEditor.onActionChanged(row)
                        return True
                return False
            iModel, iAction = self.itemIndex[row]
            actionsModel = self.models[iModel]
            if actionsModel.isLocked(iAction):
                return False
            record = items[row]
            oldValue = record.value(fieldName)
            if oldValue != value:
                if fieldName == 'actionType_id':
                    actionTypeId = forceRef(value)
                    oldActionTypeId = forceRef(oldValue)
                    newClass = CActionTypeCache.getById(actionTypeId).class_ if actionTypeId else None
                    oldClass = CActionTypeCache.getById(oldActionTypeId).class_ if oldActionTypeId else None
                    if newClass == oldClass:
                        actionsModel.setData(actionsModel.index(iAction, 0), value, Qt.EditRole)
                    else:
                        actionsModel.removeRow(iAction)
                        for iModel, model in enumerate(self.models):
                            if model.actionTypeClass == newClass:
                                model.addRow(forceRef(value))
                                self.regenerate()
                                i = self.itemIndex.index((iModel, model.rowCount()-2))
                                self.emit(SIGNAL('currentRowMovedTo(int)'), i)
                                return True
                else:
                    if fieldName == 'endDate':
                        date = forceDate(value)
                        record.setValue(fieldName, value)
                        self.eventEditor.onActionChanged(row)
                        status = forceInt(record.value('status'))
                        newStatus = status
                        if date:
                            newStatus = CActionStatus.finished
                        else:
                            if status in (CActionStatus.finished, CActionStatus.withoutResult, CActionStatus.appointed):
                                newStatus = CActionStatus.canceled
                        if status != newStatus:
                            record.setValue('status', toVariant(newStatus))
                            self.emitValueChanged(row, 'status')
                    elif fieldName == 'status':
                        record.setValue(fieldName, value)
                        status = forceInt(value)
                        date = forceDate(record.value('endDate'))
                        newDate = date
                        if status in (CActionStatus.finished, CActionStatus.withoutResult, CActionStatus.appointed) and not date:
                            newDate = QDateTime.currentDateTime()
                        else:
                            newDate = QDateTime()
                        if newDate != date:
                            record.setValue('endDate', toVariant(newDate))
                            self.emitValueChanged(row, 'endDate')
                        if status in (CActionStatus.finished, CActionStatus.withoutResult):
                            self.eventEditor.onActionChanged(row)
                    elif fieldName == 'person_id':
                        record.setValue(fieldName, value)
                        self.eventEditor.onActionChanged(row)
                    else:
                        record.setValue(fieldName, value)
                    self.emitCellChanged(row, column)
                    index = actionsModel.index(iAction, 0)
                    actionsModel.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index, index)
                    actionsModel.updateActionAmount(iAction)
            return True
        elif role == Qt.CheckStateRole:
            if row == self.rowCount()-1:
                return False
            iModel, iAction = self.itemIndex[row]
            actionsModel = self.models[iModel]
            if actionsModel.isLocked(iAction):
                return False
            record = items[row]
            oldValue = record.value(fieldName)
            if oldValue != value:
                record.setValue(fieldName, value)
                self.emitCellChanged(row, column)
                index = actionsModel.index(iAction, 0)
                actionsModel.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index, index)
                actionsModel.updateActionAmount(iAction)
            return True
        return False


    def setBegDate(self, row, date):
        self.setData(self.index(row, 3), toVariant(date), Qt.EditRole)
        self.setData(self.index(row, 4), toVariant(date), Qt.EditRole)


    def calcTotalUet(self):
        result = 0.0
        for item in self.items():
            result += self.colUet.getUet(item.value('amount'), item)
        return result


    def updateActionsPriceAndUet(self):
        self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), self.index(0, 9), self.index(len(self.items()), 9))



class CAccActionsSummary(CActionsSummaryModel):
    __pyqtSignals__ = ('sumChanged()',
                      )

    class CFinanceInDocTableCol(CRBInDocTableCol):
        def __init__(self):
            CRBInDocTableCol.__init__(self, u'Тип финансирования', 'finance_id', 10, 'rbFinance', addNone=True, preferredWidth=100)
            self.eventEditor = None


#        def toString(self, val, record):
#            if val.isNull():
#                val = toVariant(self.eventEditor.eventFinanceId)
#            return CRBInDocTableCol.toString(self, val, record)


#        def toStatusTip(self, val, record):
#            if val.isNull():
#                val = toVariant(self.eventEditor.eventFinanceId)
#            return CRBInDocTableCol.toStatusTip(self, val, record)


#        def setEditorData(self, editor, val, record):
#            if val.isNull():
#                id = self.eventEditor.eventFinanceId
#            else:
#                id = forceRef(val)
#            editor.setValue(id)


#        def getEditorData(self, editor):
#            val = editor.value()
#            if val == self.eventEditor.eventFinanceId:
#                val = None
#            return toVariant(val)


    class CContractInDocTableCol(CInDocTableCol):
        def __init__(self):
            CInDocTableCol.__init__(self, u'Договор', 'contract_id', 20)
            self.eventEditor = None


        def toString(self, val, record):
            contractId = forceRef(val)
#            if contractId is None:
#                financeId = forceRef(record.value('finance_id')) or self.eventEditor.eventFinanceId
#                if financeId == self.eventEditor.eventFinanceId:
#                    contractId = self.eventEditor.contractId
#
            if contractId:
                names = ['number', 'date', 'resolution']
                record = QtGui.qApp.db.getRecord('Contract', names, contractId)
                str = ' '.join([forceString(record.value(name)) for name in names])
            else:
                str = u'не задано'
            return QVariant(str)


        def getFirstContractId(self, actionTypeId, financeId, begDate, endDate):
            model = CContractDbModel(None)
            try:
                model.setOrgId(self.eventEditor.orgId)
                model.setEventTypeId(self.eventEditor.eventTypeId)
                model.setClientInfo(self.eventEditor.clientId,
                                     self.eventEditor.clientSex,
                                     self.eventEditor.clientAge,
                                     self.eventEditor.clientWorkOrgId,
                                     self.eventEditor.clientPolicyInfoList)
                model.setFinanceId(financeId)
                model.setActionTypeId(actionTypeId)
                model.setBegDate(begDate or QDate.currentDate())
                model.setEndDate(endDate or QDate.currentDate())
                model.initDbData()
                return model.getId(0)
            finally:
                sip.delete(model)


        def createEditor(self, parent):
            editor = CContractComboBox(parent, addNone=True)
            editor.setOrgId(self.eventEditor.orgId)
            editor.setClientInfo(self.eventEditor.clientId,
                                 self.eventEditor.clientSex,
                                 self.eventEditor.clientAge,
                                 self.eventEditor.clientWorkOrgId,
                                 self.eventEditor.clientPolicyInfoList)
            return editor


        def setEditorData(self, editor, value, record):
            financeId = forceRef(record.value('finance_id')) or self.eventEditor.eventFinanceId
            editor.setFinanceId(financeId)
            editor.setActionTypeId(forceRef(record.value('actionType_id')))
            contractId = forceRef(value)
            if contractId is None:
                if financeId == self.eventEditor.eventFinanceId:
                    contractId = self.eventEditor.contractId
            editor.setBegDate(forceDate(record.value('directionDate')) or QDate.currentDate())
            editor.setEndDate(forceDate(record.value('endDate')) or QDate.currentDate())
            editor.setValue(contractId)


        def getEditorData(self, editor):
            contractId = editor.value()
#            if contractId == self.eventEditor.contractId:
#                contractId = None
            return toVariant(contractId)


    def __init__(self, parent, editable=False):
        CActionsSummaryModel.__init__(self, parent, editable)
        for col in self.cols():
            col.setReadOnly(True)
        self.addExtCol(CBoolInDocTableCol( u'Считать',    'account', 10), QVariant.Int)#14
        self.addExtCol(CFloatInDocTableCol(u'Цена',  'price', 6, precision=2), QVariant.Double).setReadOnly(True)#15
        self.addExtCol(CFloatInDocTableCol(u'Сумма', 'sum',   6, precision=2), QVariant.Double).setReadOnly(True)#16
        self.financeCol  = self.addCol(CAccActionsSummary.CFinanceInDocTableCol()) #17
        self.contractCol = self.addCol(CAccActionsSummary.CContractInDocTableCol()) #18
        self.addCol(CActionsSummaryModel.CPayStatusCol()) #19
        self.prices = []
        self.sums = []
        self.accSum = []
        self.setEnableAppendLine(False)


    def setEventEditor(self, eventEditor):
        self.eventEditor = eventEditor
        self.financeCol.eventEditor = eventEditor
        self.contractCol.eventEditor = eventEditor


    def regenerate(self):
        CActionsSummaryModel.regenerate(self)
        count = len(self.items())
        self.prices = [0.0]*count
        self.sums = [0.0]*count
        self.accSum = [0.0]*count
        self.setCountFlag()
        self.updatePricesAndSums(0, count-1)


    def data(self, index, role=Qt.DisplayRole):
        if role == Qt.DisplayRole or role == Qt.StatusTipRole:
            column = index.column()
            row = index.row()
            if 0<=row<len(self.items()):
                col = self.cols()[column]
                fieldName = col.fieldName()
                if fieldName == 'price':
                    return QVariant(self.prices[row])
                elif fieldName == 'sum':
                    return QVariant(self.sums[row])
        return CActionsSummaryModel.data(self, index, role)


    def setData(self, index, value, role=Qt.EditRole):
        column = index.column()
        row = index.row()
        payStatus = forceRef(self.items()[row].value('payStatus')) if 0 <= row < len(self.items()) else 0
        actionId = forceRef(self.items()[row].value('id')) if 0 <= row < len(self.items()) else None
        eventId = forceRef(self.items()[row].value('event_id')) if 0 <= row < len(self.items()) else None
        if not self.canChangeAction(payStatus, actionId, eventId):
            return False
        items = self.items()
        cols = self.cols()
        col = cols[column]
        fieldName = col.fieldName()
        if role == Qt.EditRole:
            iModel, iAction = self.itemIndex[row]
            actionsModel = self.models[iModel]
            if actionsModel.isLocked(iAction):
                return False
            record = items[row]
            oldValue = record.value(fieldName)
            if oldValue != value:
                if fieldName == 'note':
                    record.setValue(fieldName, value)
                    self.emitCellChanged(row, column)
                if fieldName == 'finance_id':
                    record.setValue(fieldName, value)
                    self.emitCellChanged(row, column)
                    self.initContract(row)
                    self.updatePricesAndSums(row, row)
                elif fieldName == 'contract_id':
                    record.setValue(fieldName, value)
                    self.emitCellChanged(row, column)
                    self.updatePricesAndSums(row, row)
                elif fieldName == 'amount':
                    record.setValue(fieldName, value)
                    self.emitCellChanged(row, column)
                    self.updatePricesAndSums(row, row)
                else:
                    return False
            return True
        if role == Qt.CheckStateRole:
#            if column == 14: # 'account'
            if column == self.getColIndex('account'):
                val = forceInt(items[row].value('account'))
                items[row].setValue('account',  QVariant(forceInt(not val)))
                self.emitCellChanged(row, column)
                self.updatePricesAndSums(0, len(self.items())-1)
            return True
        return False


    def initContract(self, row):
        item = self.items()[row]
        actionTypeId = forceRef(item.value('actionType_id'))
        financeId = forceRef(item.value('finance_id'))
        if financeId:
            if financeId == self.eventEditor.eventFinanceId and getEventActionContract(self.eventEditor.eventTypeId):
                contractId = self.eventEditor.contractId
            else:
                begDate = forceDate(item.value('directionDate'))
                endDate = forceDate(item.value('endDate'))
                contractId = self.contractCol.getFirstContractId(actionTypeId, financeId, begDate, endDate)
        else:
            contractId = None
        item.setValue('contract_id', toVariant(contractId))
        self.emitValueChanged(row, 'contract_id')


    def onDataChanged(self, topLeft, bottomRight):
        CActionsSummaryModel.onDataChanged(self, topLeft, bottomRight)
        self.updatePricesAndSums(topLeft.row(), bottomRight.row())


    def onAmountChanged(self, row):
        CActionsSummaryModel.onAmountChanged(self, row)
        iModel = self.models.index(self.sender())
        try:
            row = self.itemIndex.index((iModel, row))
        except:
            row = -1

        if row>=0:
            self.updatePricesAndSums(row, row)
        else:
            self.updatePricesAndSums(0, len(self.items())-1)


    def setContractId(self, contractId):
        if getEventActionContract(self.eventEditor.eventTypeId):
            financeId = self.eventEditor.eventFinanceId
            for row, item in enumerate(self.items()):
                if forceRef(item.value('finance_id')) != financeId:
                    item.setValue('contract_id', contractId)
                    item.setValue('finance_id', financeId)
#                self.emitFieldChanged(row, 'contract_id')
                self.emitCellChanged(row, self.getColIndex('contract_id'))
        self.updatePricesAndSums(0, len(self.items())-1)


    def getTariffMap(self, contractId):
        tariffDescr = self.eventEditor.contractTariffCache.getTariffDescr(contractId, self.eventEditor)
        return tariffDescr.actionTariffMap


    def setCountFlag(self):
        cashFinanceId = CFinanceType.getId(CFinanceType.cash)
        eventFinanceId = self.eventEditor.eventFinanceId
        for item in self.items():
            itemFinanceId = forceRef(item.value('finance_id'))
            if itemFinanceId is None:
                itemFinanceId = eventFinanceId
            item.setValue('account', toVariant(itemFinanceId == cashFinanceId))


    def updatePricesAndSums(self, top, bottom):
        sumChanged = False
        for i in xrange(top, bottom+1):
            sumChanged = self.updatePriceAndSum(i, self.items()[i]) or sumChanged
        if sumChanged:
            self.emitSumChanged()


    def updatePriceAndSum(self, row, item):
        result = False
        actionTypeId = forceRef(item.value('actionType_id'))
        if actionTypeId:
            personId = forceRef(item.value('person_id'))
            tariffCategoryId = self.eventEditor.getPersonTariffCategoryId(personId)
            contractId = forceRef(item.value('contract_id'))
            if contractId:
                financeId = forceRef(QtGui.qApp.db.translate('Contract', 'id', contractId, 'finance_id'))
            else:
                contractId = self.eventEditor.contractId
                financeId = self.eventEditor.eventFinanceId
            serviceIdList = self.eventEditor.getActionTypeServiceIdList(actionTypeId, financeId)
            price = CContractTariffCache.getPrice(self.getTariffMap(contractId), serviceIdList, tariffCategoryId)
        else:
            price = 0.0
        if self.prices[row] != price:
            self.prices[row] = price
            self.emitValueChanged(row, 15)
        sum = price*forceDouble(item.value('amount'))
        if sum != self.sums[row]:
            result = True
            self.sums[row] = sum
            self.emitCellChanged(row, 16)
        if forceBool(item.value('account')):
            self.accSum[row] = sum
            result = True
        else:
            self.accSum[row] = 0
            result = True
        return result


    def emitSumChanged(self):
        self.emit(SIGNAL('sumChanged()'))


    def sum(self):
        return sum(self.accSum)


class CFxxxActionsSummaryModel(CActionsSummaryModel):
    def setData(self, index, value, role=Qt.EditRole, presetAction=None):
        result = CActionsSummaryModel.setData(self, index, value, role, presetAction)
        if result:
            column = index.column()
            if column == self.getColIndex('endDate'): # end date
                newDateTime = forceDateTime(value)
                eventSetDateTime = self.eventEditor.eventSetDateTime
                newDate = newDateTime.date() if isinstance(newDateTime, QDateTime) else newDateTime
                eventSetDate = eventSetDateTime.date() if isinstance(eventSetDateTime, QDateTime) else eventSetDateTime
                if not newDate:
                    pass
                elif newDate < eventSetDate:
                    self.setBegDate(index.row(), QDate())
                elif newDate >= eventSetDate:
                    row = index.row()
                    if not forceDate(self.data(self.index(row, 3), Qt.EditRole)):
                        self.setData(self.index(row, 3), toVariant(eventSetDateTime), Qt.EditRole)
                    if not forceDate(self.data(self.index(row, 4), Qt.EditRole)):
                        self.setData(self.index(row, 4), toVariant(eventSetDateTime), Qt.EditRole)
        return result



    def setBegDate(self, row, date):
        execDate = forceDate(self.data(self.index(row, 4), Qt.EditRole))
        if not execDate:
            CActionsSummaryModel.setBegDate(self, row, date)
        else:
            CActionsSummaryModel.setBegDate(self, row, date if date<=execDate else QDate())
