# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sip

from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, QEvent, SIGNAL

from library.Utils                   import forceRef, toVariant

from library.InDocTable              import CInDocTableView

from Events.ActionTypeComboBox       import CActionTypeComboBox
from Events.ActionEditDialog         import CActionEditDialog
from Registry.ClientRecordProperties import CRecordProperties


class CActionsTableView(CInDocTableView):
    def __init__(self, parent):
        CInDocTableView.__init__(self, parent)

        self.delegate = CActionTypeItemDelegate(self)
        self.setItemDelegate(self.delegate)
        self.setTabKeyNavigation(False)
        self._actOpenInEditor = None

        self._parent = None

    def setModel(self, model):
        if getattr(model, '__groupingAllowed__', False):
            self._addTouchGroupingAction()
            self.connect(model, SIGNAL("rowIndexActivated(int)"), self._on_groupRowActivated)
        return CInDocTableView.setModel(self, model)

    def _on_groupRowActivated(self, row):
        if row < 0:
            return

        index = self.model().index(row, 0)
        self.setCurrentIndex(index)

    def setParentWidget(self, parent):
        self._parent = parent

    def _addTouchGroupingAction(self):
        if self._popupMenu is None:
            self.createPopupMenu()
        self.connect(self._popupMenu, SIGNAL('aboutToShow()'), self.on_actTouchGroupingShow)
        self._actTouchGrouping = QtGui.QAction(u'Группировать элементы', self)
        self._popupMenu.addAction(self._actTouchGrouping)
        self.connect(self._actTouchGrouping, SIGNAL('triggered()'), self.on_touchGrouping)
        self.addAction(self._actTouchGrouping)

    def on_actTouchGroupingShow(self):
        row = self.currentIndex().row()
        model = self.model()
        self._actTouchGrouping.setEnabled(model.canRowBeGrouped(row))
        expanded = model.getExpandedByRow(row)
        if expanded:
            self._actTouchGrouping.setText(u'Группировать элементы')
        else:
            self._actTouchGrouping.setText(u'Раскрыть элементы')

    def on_touchGrouping(self):
        row = self.currentIndex().row()
        model = self.model()
        model.touchGrouping(row)

    def addOpenInEditor(self):
        if self._popupMenu is None:
            self.createPopupMenu()
        self.connect(self._popupMenu, SIGNAL('aboutToShow()'), self.on_actOpenInEditorShow)
        self._actOpenInEditor = QtGui.QAction(u'Открыть в редакторе', self)
        self._actOpenInEditor.setShortcuts(['F2'])
        self._popupMenu.addAction(self._actOpenInEditor)
        self.connect(self._actOpenInEditor, SIGNAL('triggered()'), self.on_openInEditor)
        self.addAction(self._actOpenInEditor)


    def on_deleteRows(self):
        CInDocTableView.on_deleteRows(self)
        if getattr(self.model(), '__groupingAllowed__', False):
            self.model().afterRowsDeleting()
        self.emitDelRows()


    def emitDelRows(self):
        self.emit(SIGNAL('delRows()'))


    def on_actOpenInEditorShow(self):
        if self._actOpenInEditor:
            self._actOpenInEditor.setEnabled(bool(self.model().items()))


    def on_openInEditor(self):
        index = self.currentIndex()
        if index.isValid():
            row = index.row()
            items = self.model().items()
            if 0 <= row < len(items):
                oldRecord, oldAction = items[row]
                dialog = CActionEditDialog(self)
                dialog.save = lambda: True
                dialog.setForceClientId(self._parent.clientId())
                dialog.setRecord(QtSql.QSqlRecord(oldRecord))
                dialog.setReduced(True)
                self.copyAction(oldAction, dialog.action)

                if dialog.exec_():
                    record = dialog.getRecord()
                    items[row] = (record, dialog.action)
                    self._parent.onActionCurrentChanged()

                sip.delete(dialog)


    def copyAction(self, sourceAction, targetAction):
        targetAction._propertiesByName.clear()
        targetAction._propertiesById.clear()
        targetAction._executionPlan.clear()
        targetAction._properties = []

        for sourceProperty in sourceAction.getProperties():
            sourcePropertyTypeId = sourceProperty.type().id
            targetProperty = targetAction.getPropertyById(sourcePropertyTypeId)
            targetProperty.copy(sourceProperty)

    def showRecordProperties(self):
        currentRow = self.currentIndex().row()
        items = self.model().items()
        if currentRow < len(items):
            itemId = forceRef(items[currentRow][0].value('id'))
        else:
            return
        table = self.model().table
        CRecordProperties(self, table, itemId).exec_()


class CActionTypeItemDelegate(QtGui.QItemDelegate):
    def __init__(self, parent):
        QtGui.QItemDelegate.__init__(self, parent)


    def createEditor(self, parent, option, index):
        editor = CActionTypeComboBox(parent)
        editor.setClass(index.model().actionTypeClass)
        editor.setDisabledActionTypeIdList(index.model().disabledActionTypeIdList)
        preferredWidth = self.parent().model().col.preferredWidth
        editor.setPreferredWidth(preferredWidth)
        dialog = index.model().eventEditor
        editor.setFilter(dialog.clientSex, dialog.clientAge)
        return editor


    def setEditorData(self, editor, index):
        model = index.model()
        editor.setValue(forceRef(model.data(index, Qt.EditRole)))


    def setModelData(self, editor, model, index):
        model.setData(index, toVariant(editor.value()))


    def emitCommitDataAndClose(self, editor, hint=QtGui.QAbstractItemDelegate.NoHint):
        self.emit(SIGNAL('commitData(QWidget *)'), editor)
        self.emit(SIGNAL('closeEditor(QWidget *,QAbstractItemDelegate::EndEditHint)'), editor, hint)


    def eventFilter(self, editor, event):
        if event.type() == QEvent.KeyPress:
            if event.key() == Qt.Key_Tab:
                if editor is not None:
                    self.emitCommitDataAndClose(editor)
                self.parent().focusNextChild()
                return True
            elif event.key() == Qt.Key_Backtab:
                if editor is not None:
                    self.emitCommitDataAndClose(editor)
                self.parent().focusPreviousChild()
                return True
            elif event.key() == Qt.Key_Return:
                if editor is not None:
                    self.emitCommitDataAndClose(editor)
#                event.accept()
                return True
        return QtGui.QItemDelegate.eventFilter(self, editor, event)
