# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
import json
import datetime

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import SIGNAL

from library.Utils import forceTime, forceDouble, toVariant


class CItem(object):
    def __init__(self, date, time, dosage, editable, idx=0):
        self._date = date
        self._time = time
        self._dosage = dosage
        self._record = None
        self._editable = editable
        self._idx = idx

    @property
    def idx(self):
        return self._idx

    @property
    def time(self):
        return self._time

    @property
    def dosage(self):
        return self._dosage

    @property
    def record(self):
        return self._record

    def setTime(self, time):
        self._time = time

    def setRecord(self, record):
        self._record = record

    def setDoses(self, dosage):
        self._dosage = dosage

    @property
    def editable(self):
        return self._editable


TIME_INDEX = 0
DOSES_INDEX = 1


class CNomenclatureExpenseDayModel(QtCore.QAbstractTableModel):
    def __init__(self, parent, dosageUnitName=''):
        QtCore.QAbstractTableModel.__init__(self, parent)
        self._items = []
        self._canAddItems = False
        self._zeroTime = QtCore.QTime(0, 0)
        self._dosageUnitName = dosageUnitName

    def load(self, data, notEditableData):
        self._canAddItems = bool(data)
        for d in data:
            self._items.append(CItem(None, d['time'], d['dosage'], True, d.get('idx', 0)))
        for d in notEditableData:
            self._items.append(CItem(None, d['time'], d['dosage'], False, d.get('idx', 0)))
            self._items[-1].setRecord(d['record'])

        self._items.sort(key=lambda i: (i.time, i.idx))
        self.reset()

    def removeRows(self, row, count, parentIndex = QtCore.QModelIndex()):
        if 0<=row and row+count<=len(self._items):
            self.beginRemoveRows(parentIndex, row, row+count-1)
            del self._items[row:row+count]
            self.endRemoveRows()
            return True
        else:
            return False

    def rowCount(self, index=None):
        return len(self._items) + 1 if self._canAddItems else len(self._items)

    def columnCount(self, index=None):
        return 2

    def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):
        if role != QtCore.Qt.DisplayRole or orientation == QtCore.Qt.Vertical:
            return QtCore.QAbstractTableModel.headerData(self, section, orientation, role)
        return u'Время' if section == TIME_INDEX else u'Дозировка'

    def flags(self, index):
        row = index.row()
        result = QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled
        if row == len(self._items):
            if self._canAddItems:
                result |= QtCore.Qt.ItemIsEditable
        else:
            item = self._items[row]
            if item.editable:
                result |= QtCore.Qt.ItemIsEditable
        return result

    def data(self, index, role=QtCore.Qt.DisplayRole):
        row = index.row()
        if not (0 <= row < len(self._items)):
            return QtCore.QVariant()
        if role != QtCore.Qt.DisplayRole:
            return QtCore.QVariant()

        column = index.column()
        if column == TIME_INDEX:
            time = self._items[row].time
            if not time.isValid():
                return QtCore.QVariant(self._zeroTime)
            return toVariant(self._items[row].time)
        elif column == DOSES_INDEX:
            if self._dosageUnitName:
                value = '%s %s' % (str(self._items[row].dosage), self._dosageUnitName)
            else:
                value = str(self._items[row].dosage)
            return toVariant(value)
        return QtCore.QVariant()

    def createEditor(self, index, parent):
        column = index.column()
        if column == TIME_INDEX:
            return QtGui.QTimeEdit(parent)
        elif column == DOSES_INDEX:
            editor = QtGui.QSpinBox(parent)
            editor.setMaximum(100000)
            editor.setMinimum(0)
            return editor
        return None

    def setEditorData(self, index, editor):
        row = index.row()
        item = self._items[row] if 0 <= row < len(self._items) else None
        column = index.column()
        if column == TIME_INDEX:
            editor.setTime(item.time if item else QtCore.QTime())
        elif column == DOSES_INDEX:
            editor.setValue(item.dosage if item else 0)

    def getEditorData(self, index, editor):
        column = index.column()
        if column == TIME_INDEX:
            return editor.time()
        elif column == DOSES_INDEX:
            return editor.value()
        else:
            return

    def setData(self, index, value, role=QtCore.Qt.EditRole):
        if role != QtCore.Qt.EditRole:
            return False

        row = index.row()
        if row == len(self._items):
            if not getattr(self, '_canAddLastRow', True):
                return False
            self.beginInsertRows(QtCore.QModelIndex(), row, row)
            if self._items:
                dosage = self._items[-1].dosage
            else:
                dosage = 0
            self._items.append(CItem(None, QtCore.QTime(0, 0), dosage, True))
            self.endInsertRows()

        item = self._items[row]
        column = index.column()
        if column == TIME_INDEX:
            item.setTime(forceTime(value))
        elif column == DOSES_INDEX:
            item.setDoses(forceDouble(value))
        else:
            return False

        self.emitAllChanged()

        return True

    def items(self):
        return self._items

    def emitAllChanged(self):
        index1 = self.index(0, 0)
        index2 = self.index(self.rowCount(), self.columnCount())
        self.emit(QtCore.SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index1, index2)