# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
import datetime

import copy

from PyQt4 import QtGui, QtCore, QtSql
from PyQt4.QtCore import Qt

from Events.ActionStatus import CActionStatus
from Events.Action import CActionTypeCache, CAction, CActionType

from library.Utils import forceDate, forceRef, forceString, forceDouble, forceInt, toVariant, forceTime, forceDateTime


BEG_DATE_INDEX = 0
NOMENCLATURE_INDEX = 1
PLAN_END_DATE = 2
DOSES_INDEX = 3
SIGNA_INDEX = 4
DURATION_INDEX = 5
ALIQUOTICITY_INDEX = 6
PERIODICITY_INDEX = 7


DONE_COLOR = QtGui.QColor(Qt.green)
NOT_DONE_COLOR = QtGui.QColor(Qt.yellow)
OVERDUE_COLOR = QtGui.QColor(Qt.red)

_nomenclatureInTableIndex = 1
_dosageInTableIndex = 2
_signaInTableIndex = 3

_mapInActionsSelectionTable2Index = {
    _nomenclatureInTableIndex: NOMENCLATURE_INDEX,
    _dosageInTableIndex: DOSES_INDEX,
    _signaInTableIndex: SIGNA_INDEX
}

_mapColumnIndex2InActionsSelectionTable = {}
for k, v in _mapInActionsSelectionTable2Index.items():
    _mapColumnIndex2InActionsSelectionTable[v] = k

_mapColumnIndex2ValueConverter = {
    DOSES_INDEX: forceDouble,
    SIGNA_INDEX: forceString,
    NOMENCLATURE_INDEX: forceRef
}

_mapColumnIndex2FieldName = {
    DURATION_INDEX: 'duration',
    ALIQUOTICITY_INDEX: 'aliquoticity',
    PERIODICITY_INDEX: 'periodicity'
}


class CGroup(object):
    def __init__(self, item, nomenclatureId, editable=True, items=None, canBeAdded=False, model=None):
        self._editable = editable
        self._items = items[:] if items else [item]
        self._items.sort(key=lambda i: forceDateTime(i[0].value('begDate')))
        self._item = self._items[0]
        self._actionTypeId = self._item[1].getType().id
        self._nomenclatureId = nomenclatureId
        self._class = self._item[1].getType().class_

        self._canBeAdded = canBeAdded
        self._model = model
        self._itemsToDelete = []
        if self.isNewGroup:
            self._originRecords = {}
            for item in self._items:
                self._originRecords[item] = self._copyRecord(item[0])
            self._orginRecord = self._copyRecord(self._item[0])
            self._nomenclature = self._model.cellSettings.getGroupNomenclature(self)
            self._signa = self._model.cellSettings.getGroupSigna(self)
            self._dosage = self._model.cellSettings.getGroupDoses(self)
        else:
            self._originRecords = None
            self._orginRecord = None
            self._nomenclature = None
            self._signa = None
            self._dosage = None

        self._defaultShcedule = self._analyzeShcedule()

        self._orginExecetedPlanes = {}
        for item in self._items:
            # self._orginExecetedPlanes[item] = copy.deepcopy(item[1].getExecutionPlan())
            self._orginExecetedPlanes[item] = item[1].getExecutionPlan().copy()

    def checkOnlyActual(self, onlyActual):
        if onlyActual:
            return bool([i for i in self._items if forceInt(i[0].value('status')) != CActionStatus.finished])
        return True

    def updateNomenclatureDosageValue(self, dosage):
        for item in self._items:
            record, action = item
            if forceInt(record.value('status')) == CActionStatus.finished:
                continue
            action.updateNomenclatureDosageValue(self._nomenclatureId, dosage)

    def initNomenclatureReservation(self, clientId):
        for item in self._items:
            record, action = item
            if forceInt(record.value('status')) == CActionStatus.finished:
                continue
            action.initNomenclatureReservation(clientId, self._calcReservationDosage())

    def _calcReservationDosage(self):
        return self.headItem[1].getExecutionPlan().calcDosage()

    def prepareToSave(self):
        for item in self._items:
            record, action = item
            if forceInt(record.value('status')) == CActionStatus.finished:
                continue
            ep = action.getExecutionPlan()
            mdk = min(ep.keys())
            mtk = min(ep[mdk].keys())
            epr = ep[mdk][mtk]
            dosage = forceDouble(epr.value('dosage'))
            pt = self._model.cellSettings.getGroupDosesPT(self)
            action.getPropertyById(pt.id).setValue(dosage)
            action.updateNomenclatureDosageValue(self._nomenclatureId, dosage)


    def minExecutionPlanDateTime(self):
        dts = [i[1].minExecutionPlanDateTime() for i in self._items if i[1].minExecutionPlanDateTime()]
        if not dts:
            return None
        return min(dts)

    def canSetNewDuration(self, value):
        item = self.headItem
        record, action = item

        duration = forceInt(record.value('duration'))

        diff = value - duration
        mdt = self.minExecutionPlanDateTime()
        if not mdt:
            return True
        plannedEndDate = forceDateTime(record.value('plannedEndDate'))
        newPlannedEndDate = plannedEndDate.addDays(diff)
        return newPlannedEndDate >= mdt

    def updateDuration(self, value):
        if value < 1:
            return False

        if not self.canSetNewDuration(value):
            return False

        item = self.headItem
        record, action = item

        plannedEndDate = forceDateTime(record.value('plannedEndDate'))
        duration = forceInt(record.value('duration'))

        diff = value - duration
        newPlannedEndDate = plannedEndDate.addDays(diff)
        maxEPDate = forceDateTime(record.value('directionDate'))
        lastItem = None
        for item in self._items:
            minEPDateTime = forceDateTime(item[1].minExecutionPlanDateTime())
            if not minEPDateTime:
                lastItem = item

            if minEPDateTime > maxEPDate:
                maxEPDate = minEPDateTime
                lastItem = item

            item[0].setValue('plannedEndDate', toVariant(newPlannedEndDate))
            item[0].setValue('duration', toVariant(value))

        if lastItem is None:
            lastItem = self._items[-1]
        lastItem[1].addExecutionPlanByRecord()
        return True

    @property
    def defaultShcedule(self):
        return self._defaultShcedule

    def _analyzeShcedule(self):
        defaultExecutionPlanCount = self._item[1].defaultExecutionPlanCount()
        count = 0
        for _, action in self._items:
            count += 1
        lastItem = self._items[-1]
        record, action = lastItem
        if forceRef(record.value('id')) and forceInt(record.value('statue')) != CActionStatus.started:
            return count == defaultExecutionPlanCount
        count -= 1
        ep = action.getExecutionPlan()
        for dk in ep.keys():
            count += len(ep[dk])
        return count == defaultExecutionPlanCount

    def copyDataToRestore(self, group):
        self._orginRecord = group._originRecord
        self._nomenclature = group._nomenclature
        self._signa = group._signa
        self._dosage = group._dosage
        self._orginRecord = group._originRecord
        self._orginExecetedPlanes = group._orginExecetedPlanes

    def deleteDays(self, year, month, day):
        date = datetime.date(year, month, day)
        for item in self._items:
            record, action = item

            if forceRef(record.value('id')) or forceInt(record.value('statue')) != CActionStatus.started:
                continue

            executionPlan = action.getExecutionPlan()
            executionPlan.pop(date, None)

    def setDayNomenclatureExpenseData(self, year, month, days, data):
        if not isinstance(days, list):
            days = [days]

        table = QtGui.qApp.db.table('Action_ExecutionPlan')
        for day in days:
            date = datetime.date(year, month, day)
            allDone, itemsEpCounts, allItems = self._anayzeItemsPlans(date)
            if allDone:
                return

            if not itemsEpCounts:
                allItems.sort(key=lambda i: i[0])
                lastItem = allItems[-1][1]
                action = lastItem[1]
                ep = action.getExecutionPlan()
                minDate = min(ep.keys())
                if minDate < date and date in ep:
                    timesData = {}
                    for d in data:
                        time = d['time']
                        dosage = d['dosage']

                        idx = d['idx']

                        record = table.newRecord()
                        record.setValue('dosage', toVariant(dosage))
                        record.setValue('execDate', toVariant(QtCore.QDateTime(forceDate(date), time)))

                        record.setValue('idx', toVariant(idx))
                        timesData[time] = record

                    ep[date] = timesData

            else:
                action = itemsEpCounts[0][1][1]
                ep = action.getExecutionPlan()
                timesData = {}
                for d in data:
                    idx = d['idx']
                    record = d['record']
                    if record:
                        record.setValue('idx', toVariant(idx))
                        timesData[forceDateTime(record.value('execDate')).time()] = record
                        continue

                    dosage = d['dosage']
                    time = d['time']

                    record = table.newRecord()
                    record.setValue('idx', toVariant(idx))
                    record.setValue('dosage', toVariant(dosage))
                    record.setValue('execDate', toVariant(QtCore.QDateTime(forceDate(date), time)))
                    timesData[time] = record

                ep[date] = timesData

        self._defaultShcedule = self._analyzeShcedule()

    def _anayzeItemsPlans(self, date):
        allDone = False
        itemsEpCounts = []
        allItems = []
        for item in self._items:
            ep = item[1].getExecutionPlan()
            epCounts = sum([len(ep[k]) for k in ep.keys()])
            allItems.append((epCounts, item))
            minDate = min(ep.keys())
            if minDate != date:
                if minDate > date:
                    allDone = True
                continue
            itemsEpCounts.append((epCounts, item))
        return allDone, itemsEpCounts, allItems

    def _getDayData(self, date, allDone, itemsEpCounts, allItems):
        if not itemsEpCounts:
            allItems.sort(key=lambda i: i[0])
            lastItem = allItems[-1][1]
            action = lastItem[1]
            ep = action.getExecutionPlan()
            minDate = min(ep.keys())
            if minDate < date and date in ep:
                return self.formatEPToDayData(action, ep[date], True), []
        else:
            itemsEpCounts.sort(key=lambda i: i[0])
            if allDone:
                data = []
                for _, item in itemsEpCounts:
                    action = item[1]
                    data.append(self.formatEPToDayData(action, action.getExecutionPlan()[date], False))
                return [], data
            else:
                notEditable = []
                for _, item in itemsEpCounts[1:]:
                    action = item[1]
                    notEditable.append(self.formatEPToDayData(action, action.getExecutionPlan()[date], False))

                action = itemsEpCounts[0][1][1]
                ep = action.getExecutionPlan()
                data = self.formatEPToDayData(action, ep[date], True)
                return data, notEditable


    def formatEPToDayData(self, action, times, all):
        if all:
            result = []
            for time in times.keys():
                record = times[time]
                dosage = forceDouble(record.value('dosage'))
                if not dosage:
                    pt = self._model.cellSettings.getGroupDosesPT(self)
                    dosage = action.getPropertyById(pt.id).getValue()
                if not dosage:
                    dosage = self._model.cellSettings.getNomenclatureDoses(self._nomenclatureId)
                result.append({'time': forceTime(time), 'dosage': dosage, 'record': record, 'idx': forceInt(record.value('idx'))})
            return result
        else:
            minTime = sorted(times.keys())[0]
            record = times[minTime]
            dosage = forceDouble(record.value('dosage'))
            if not dosage:
                pt = self._model.cellSettings.getGroupDosesPT(self)
                dosage = action.getPropertyById(pt.id).getValue()
            if not dosage:
                dosage = self._model.cellSettings.getNomenclatureDoses(self._nomenclatureId)

            return {'time': forceTime(minTime), 'dosage': dosage, 'record': record, 'idx': forceInt(record.value('idx'))}

    def getDayData(self, year, month, day):
        date = datetime.date(year, month, day)
        allDone, itemsEpCounts, allItems = self._anayzeItemsPlans(date)
        return self._getDayData(date, allDone, itemsEpCounts, allItems)

    def setNomenclatureId(self, nomenclatureId):
        self._nomenclatureId = nomenclatureId

    def _copyRecord(self, record):
        newRecord = QtSql.QSqlRecord()
        for i in range(record.count()):
            value = record.value(i)
            fieldName = record.fieldName(i)
            newRecord.append(QtSql.QSqlField(fieldName, value.type()))
            newRecord.setValue(fieldName, value)
        return newRecord

    def restoreRecord(self):
        for item, executionPlan in self._orginExecetedPlanes.items():
            if item in self._item:
                item[1].setExecutionPlan(executionPlan)

        if not self._originRecords:
            return

        for item in self._items:
            if item not in self._originRecords:
                continue

            record, action = item
            originRecord = self._originRecords[item]
            for i in range(originRecord.count()):
                record.setValue(originRecord.fieldName(i), originRecord.value(i))

            if self._nomenclature:
                self._nomenclatureId = self._nomenclature
                propertyType = self._model.cellSettings.getGroupNomenclaturePT(self)
                property = action.getPropertyById(propertyType.id)
                property.setValue(self._nomenclature)

            if self._signa:
                propertyType = self._model.cellSettings.getGroupSignaPT(self)
                property = action.getPropertyById(propertyType.id)
                property.setValue(self._signa)

            if self._dosage:
                propertyType = self._model.cellSettings.getGroupDosesPT(self)
                property = action.getPropertyById(propertyType.id)
                property.setValue(self._dosage)

    @property
    def firstItem(self):
        return self._items[0]


    def inMonth(self, date):
        item = self.firstItem
        record, action = item
        begDate = forceDate(record.value('begDate'))
        return QtCore.QDate(date.year(), date.month(), 1) <= begDate <= QtCore.QDate(date.year(), date.month(), date.daysInMonth())

    def inPeriod(self, begDate, endDate):
        item = self.firstItem
        record, action = item
        date = forceDate(record.value('begDate'))
        return (not begDate or begDate <= date) and (not endDate or date <= endDate)

    @classmethod
    def fromExternalGroup(cls, group, model):
        headItem = group.headItem
        nomenclatureId = group.nomenclatureId
        items = group.items
        result = cls(headItem, nomenclatureId, False, items, canBeAdded=False, model=model)
        if isinstance(group, CGroup):
            result.copyDataToRestore(group)
        return result

    @property
    def isNewGroup(self):
        return (
            len(self._items) == 1
            and not forceRef(self.headItem[0].value('id'))
            and forceInt(self.headItem[0].value('status')) == CActionStatus.started
        )

    @property
    def canBeAdded(self):
        return self._canBeAdded

    @property
    def editable(self):
        return self._editable or self.isNewGroup

    @property
    def class_(self):
        return self._class

    def hasPlannedDate(self, date, day):
        date = datetime.date(date.year(), date.month(), day)
        headItem = self.headItem
        _, action = headItem
        executedPlan = action.getExecutionPlan()
        return executedPlan and date in executedPlan.keys()

    def plannedOnDate(self, date):
        items = self.searchByDate(date)
        if items:
            return True

        for item in self._items:
            if date in item[1].getExecutionPlan():
                return True

        return False


    def searchByDate(self, date):
        result = []
        for i in self._items:
            epDate = i[1].minExecutionPlanDate()
            if epDate:
                if forceDate(epDate) == date:
                    result.append(i)
            else:
                if forceDate(i[0].value('begDate')) == date:
                    result.append(i)
        return result

    @property
    def actionTypeId(self):
        return self._actionTypeId

    @property
    def nomenclatureId(self):
        return self._nomenclatureId

    @property
    def items(self):
        return self._items

    @property
    def headItem(self):
        return self._item


class CGroups(object):
    def __init__(self, model):
        self._groups = []
        self._model = model

    def clear(self):
        self._groups = []

    def __len__(self):
        return len(self._groups)

    def copyGroupFrom(self, group):
        if not isinstance(group, CGroup):
            group = CGroup.fromExternalGroup(group, self._model)
        self._groups.append(group)

    def canAddGroup(self, actionTypeId, nomenclatureId):
        for g in self._groups:
            if g.actionTypeId == actionTypeId and g.nomenclatureId == nomenclatureId:
                return False
        return True

    def addNewGroup(self, action, nomenclatureId):
        item = (action.getRecord(), action)
        group =  CGroup(item, nomenclatureId, canBeAdded=True, model=self._model)
        self._groups.append(group)
        return group

    def groupCanEdited(self, group):
        return group.editable

    def canEditBegDate(self, group):
        return group.editable

    def __getitem__(self, index):
        return self._groups[index]

    def groupsToAdd(self):
        return [g for g in self._groups if g.canBeAdded]

    def discardChanges(self):
        for g in self._groups:
            g.restoreRecord()


class Col(object):
    def __init__(self, key, width):
        self.key = key
        self.width = width


class CellSettings(object):
    def __init__(self, model):
        self._model = model
        self._data = {}
        self._nomenclatureData = {}

    def _getNomenclatureUsingTypes(self, nomenclatureId):
        if not nomenclatureId:
            return []


        db = QtGui.qApp.db
        table = db.table('rbNomenclature_UsingType')
        recordList = db.getRecordList(
            table, 'usingType', where=table['master_id'].eq(nomenclatureId), order=table['idx'].name()
        )

        return [forceString(r.value('usingType')) for r in recordList]

    def getNomenclatureUsingTypes(self, nomenclatureId):
        data = self._getNomenclatureValues(nomenclatureId)

        return data.get('usingTypes')

    def getNomenclatureDoses(self, nomenclatureId):
        values = self._getNomenclatureValues(nomenclatureId)
        return values.get('dosageValue', 0)

    def getNomenclatureDosageText(self, nomenclatureId):
        values = self._getNomenclatureValues(nomenclatureId)
        if not values:
            return ''
        return '%s %s' % (values.get('dosageValue', 0), values.get('name'))

    def getNomenclatureDosageUnitName(self, nomenclatureId):
        values = self._getNomenclatureValues(nomenclatureId)
        if not values:
            return ''
        return values['name']

    def _getNomenclatureValues(self, nomenclatureId):
        if not nomenclatureId:
            return {
                'name': u'',
                'usingTypes': [],
                'dosageValue': 0.0,
                'unitId': None
            }
        if nomenclatureId not in self._nomenclatureData:
            db = QtGui.qApp.db
            nomenclatureRecord = db.getRecord('rbNomenclature', 'unit_id, dosageValue', nomenclatureId)
            if not nomenclatureRecord:
                self._nomenclatureData[nomenclatureId] = {
                    'name': u'',
                    'usingTypes': [],
                    'dosageValue': 0.0,
                    'unitId': None
                }
                return self._nomenclatureData[nomenclatureId]

            dosageUnitId = forceRef(nomenclatureRecord.value('unit_id'))

            result = {
                'dosageValue': forceDouble(nomenclatureRecord.value('dosageValue')),
                'unitId': dosageUnitId
            }

            if dosageUnitId:
                unitRecord = db.getRecord('rbUnit', 'code, name', dosageUnitId)
                # name = ' | '.join([forceString(unitRecord.value('name')), forceString(unitRecord.value('code'))])
                name = forceString(unitRecord.value('code')) if unitRecord else u''
            else:
                name = ''
            result['name'] = name
            result['usingTypes'] = self._getNomenclatureUsingTypes(nomenclatureId)
            self._nomenclatureData[nomenclatureId] = result

        return self._nomenclatureData[nomenclatureId]

    def get(self, group):
        return self._get(group)

    def clear(self):
        self._data.clear()

    def _get(self, group):
        actionTypeId = group.actionTypeId
        if not actionTypeId in self._data:
            self._createSettings(group)
        return self._data[actionTypeId]

    def getByRowColumn(self, row, column):
        group = self._model.items()[row]
        return self._get(group).get(column)

    def checkNomenclature(self, group, nomenclatureId):
        if not nomenclatureId:
            return True

        propertyType = self.getGroupNomenclaturePT(group)
        if not propertyType:
            return False

        return self.getGroupNomenclature(group) == nomenclatureId

    def hasGroupSigna(self, group):
        return bool(self.getGroupSignaPT(group))

    def hasGroupDoses(self, group):
        return bool(self.getGroupDosesPT(group))

    def _getPropertyValue(self, group, pt):
        if not pt:
            return None
        action = group.headItem[1]
        return action.getPropertyById(pt.id).getValue()

    def getGroupNomenclature(self, group):
        pt = self.getGroupNomenclaturePT(group)
        return self._getPropertyValue(group, pt)

    def getGroupSigna(self, group):
        pt = self.getGroupSignaPT(group)
        return self._getPropertyValue(group, pt)

    def getGroupDoses(self, group):
        pt = self.getGroupDosesPT(group)
        return self._getPropertyValue(group, pt)

    def getGroupNomenclaturePT(self, group):
        return self._get(group).get(NOMENCLATURE_INDEX)

    def getGroupDosesPT(self, group):
        return self._get(group).get(DOSES_INDEX)

    def getGroupSignaPT(self, group):
        return self._get(group).get(SIGNA_INDEX)

    def _createSettings(self, group):
        action = group.headItem[1]
        actionType = action.getType()

        self._data[actionType.id] = {}

        for propertyType in actionType.getPropertiesById().values():
            if propertyType.inActionsSelectionTable:
                column = _mapInActionsSelectionTable2Index[propertyType.inActionsSelectionTable]
                self._data[actionType.id][column] = propertyType


class CNomenclatureExpenseModel(QtCore.QAbstractTableModel):
    STATIC_HEADERS = {
        BEG_DATE_INDEX: (u'Дата начала', 10),
        PLAN_END_DATE: (u'План', 10),
        NOMENCLATURE_INDEX: (u'ЛС', 10),
        DOSES_INDEX: (u'Доза', 3),
        SIGNA_INDEX: (u'СП', 3),
        DURATION_INDEX: (u'Д', 3),
        ALIQUOTICITY_INDEX: (u'К', 3),
        PERIODICITY_INDEX: (u'И', 3)
    }

    def cols(self):
        cols = []
        key = 0
        for key in sorted(self.STATIC_HEADERS.keys()):
            cols.append(Col(key, self.STATIC_HEADERS[key][1]))

        for monthKey in range(31):
            cols.append(Col(key+monthKey+1, 3))

        return cols

    def __init__(self, parent=None, date=None):
        QtCore.QAbstractTableModel.__init__(self, parent)
        self._groups = CGroups(self)
        self._date = date or QtCore.QDate.currentDate()
        self._cellsSettings = CellSettings(self)
        self._actionTypeId = None
        self._class = None
        self._nomenclatureId = None
        self._originGroups = None
        self._newGroups = []
        self._onlyActual = False
        self._eventEditor = None
        self._begDate = None
        self._endDate = None
        self._considerPeriod = False

    def getDosageUnitName(self, row):
        group = self._groups[row]
        return self.cellSettings.getNomenclatureDosageUnitName(group.nomenclatureId)

    def considerPeriod(self, v):
        self._considerPeriod = v
        self.setGroups(self._originGroups)
        self.reset()

    def setBegDate(self, date):
        self._begDate = date
        self.setGroups(self._originGroups)
        self.reset()

    def setEndDate(self, date):
        self._endDate = date
        self.setGroups(self._originGroups)
        self.reset()

    @property
    def lastMainIndex(self):
        return PERIODICITY_INDEX

    @property
    def days(self):
        return self._date.daysInMonth()

    def setEventEditor(self, eventEditor):
        self._eventEditor = eventEditor
        self.reset()

    def setNomenclatureId(self, nomenclatureId):
        if self._nomenclatureId != nomenclatureId:
            self._nomenclatureId = nomenclatureId

            self.setGroups(self._originGroups)
            self.reset()

    def setActionTypeId(self, actionTypeId):
        if self._actionTypeId != actionTypeId:
            if actionTypeId:
                self._class = CActionTypeCache.getById(actionTypeId).class_
            else:
                self._class = None
            self._actionTypeId = actionTypeId
            self.setGroups(self._originGroups)
            self.reset()

    def setGroups(self, groups):
        self._originGroups = list(groups)
        self._groups.clear()
        self._cellsSettings.clear()
        if not groups:
            return

        for group in (groups + self._newGroups):
            if (self._actionTypeId
                    and group.actionTypeId == self._actionTypeId
                    and self._cellsSettings.checkNomenclature(group, self._nomenclatureId)
                    and group.inMonth(self._date)
                    and (self._considerPeriod or group.inPeriod(self._begDate, self._endDate))
                    and group.checkOnlyActual(self._onlyActual)):
                self._groups.copyGroupFrom(group)

    def setDate(self, date):
        self._date = date
        self.reset()

    def setOnlyActual(self, value):
        if self._onlyActual != value:
            self._onlyActual = value
            self.setGroups(self._originGroups)
            self.reset()

    def items(self):
        return self._groups

    def rowCount(self, index=None):
        return len(self._groups) + 1 if self._eventEditor else len(self._groups)

    def columnCount(self, index=None):
        return len(self.STATIC_HEADERS) + self._date.daysInMonth()

    def headerData(self, columnIndex, orientation, role=Qt.DisplayRole):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            if columnIndex in self.STATIC_HEADERS:
                return QtCore.QVariant(self.STATIC_HEADERS[columnIndex][0])
            else:
                return QtCore.QVariant(columnIndex - PERIODICITY_INDEX)

        return QtCore.QVariant()

    def _canEditIndex(self, index):
        row = index.row()
        column = index.column()
        if not self._actionTypeId:
            return False

        if row == len(self._groups):
            return column == NOMENCLATURE_INDEX

        if column == BEG_DATE_INDEX:
            return self._groups.canEditBegDate(self._groups[row])

        if column == NOMENCLATURE_INDEX:
            return self._groups.groupCanEdited(self._groups[row])

        elif column in (ALIQUOTICITY_INDEX, PERIODICITY_INDEX):
            return self._groups.groupCanEdited(self._groups[row]) and self._groups[row].defaultShcedule

        elif column == DOSES_INDEX and self._groups.groupCanEdited(self._groups[row]):
            return self._cellsSettings.hasGroupDoses(self._groups[row])

        elif column == SIGNA_INDEX and self._groups.groupCanEdited(self._groups[row]):
            return self._cellsSettings.hasGroupSigna(self._groups[row])

        elif column  == DURATION_INDEX:
            return self._groups.groupCanEdited(self._groups[row])
        else:
            return False

    def flags(self, index=QtCore.QModelIndex()):
        result = Qt.ItemIsSelectable | Qt.ItemIsEnabled
        if self._canEditIndex(index):
            result |= Qt.ItemIsEditable
        return result

    @staticmethod
    def _columnToDay(column):
        return column-PERIODICITY_INDEX

    def getGroupDayDataToEdit(self, row, column):
        if not (0 <= row < len(self._groups)):
            return None
        group = self._groups[row]
        return group.getDayData(self._date.year(), self._date.month(), self._columnToDay(column))

    def setGroupDayNomenclatureExpenseData(self, row, column, data):
        if not (0 <= row < len(self._groups)):
            return
        group = self._groups[row]
        group.setDayNomenclatureExpenseData(self._date.year(), self._date.month(), self._columnToDay(column), data)

    def deleteDays(self, row, columns):
        if not (0 <= row < len(self._groups)):
            return

        group = self._groups[row]
        for column in columns:
            group.deleteDays(self._date.year(), self._date.month(), self._columnToDay(column))

    def setDepsByNomenclature(self, nomenclatureId, action, group, updateEp=False):
        if nomenclatureId:
            nomenclatureValues = self.cellSettings._getNomenclatureValues(nomenclatureId)
            dosageValue = nomenclatureValues.get('dosageValue')
            pt = self.cellSettings.getGroupDosesPT(group)
            action.getPropertyById(pt.id).setValue(dosageValue)

            usingTypes = self.cellSettings.getNomenclatureUsingTypes(nomenclatureId)
            if usingTypes:
                pt = self.cellSettings.getGroupSignaPT(group)
                if pt:
                    signaProperty = action.getPropertyById(pt.id)
                    signaProperty.setValue(usingTypes[0])

            action.updateExecutionPlanByRecord(forceDuration=True)

            ep = action.getExecutionPlan()
            for dk in ep.keys():
                for tk in ep[dk].keys():
                    ep[dk][tk].setValue('dosage', toVariant(dosageValue))

    def setData(self, index, value, role=Qt.EditRole):
        if not self._canEditIndex(index):
            return False

        row = index.row()
        column = index.column()

        if column == NOMENCLATURE_INDEX and row == len(self._groups):
            nomenclatureId = forceRef(value)
            if not nomenclatureId:
                return False

            self.beginInsertRows(QtCore.QModelIndex(), row, row)
            action = self.getNewAction(self._actionTypeId, nomenclatureId)
            record = action.getRecord()
            record.setValue('duration', toVariant(1))
            record.setValue('aliquoticity', toVariant(1))
            record.setValue('periodicity', toVariant(0))
            newGroup = self._groups.addNewGroup(action, nomenclatureId)
            self._newGroups.append(newGroup)
            self.updatePlannedEndDate(action)
            self.setDepsByNomenclature(newGroup.nomenclatureId, newGroup.headItem[1], newGroup, updateEp=True)

            property = action.getPropertyById(self._cellsSettings.getGroupNomenclaturePT(newGroup).id)
            property.applyDependents(action)

            if property.isActionNameSpecifier():
                action.updateSpecifiedName()

            self.endInsertRows()
            self.emitRowChanged(row)
            return True

        elif column in (DOSES_INDEX, SIGNA_INDEX, NOMENCLATURE_INDEX):
            group = self._groups[row]
            if column == DOSES_INDEX:
                propertyType = self._cellsSettings.getGroupDosesPT(group)
            elif column == SIGNA_INDEX:
                propertyType = self._cellsSettings.getGroupSignaPT(group)
            else:
                propertyType = self._cellsSettings.getGroupNomenclaturePT(group)

            if not propertyType:
                return False

            action = group.headItem[1]
            valueConverter = _mapColumnIndex2ValueConverter[column]
            property = action.getPropertyById(propertyType.id)

            property.preApplyDependents(action)

            value = valueConverter(value)
            property.setValue(value)
            if column == NOMENCLATURE_INDEX:
                group.setNomenclatureId(value)
            elif column == DOSES_INDEX:
                group.updateNomenclatureDosageValue(forceDouble(value))
            self._cellsSettings.clear()
            property.applyDependents(action)
            if property.isActionNameSpecifier():
                action.updateSpecifiedName()

            return True
        elif column in (ALIQUOTICITY_INDEX, PERIODICITY_INDEX):
            group = self._groups[row]
            fieldValue = _mapColumnIndex2FieldName[column]
            action = group.headItem[1]
            action.getRecord().setValue(fieldValue, value)
            if column == DURATION_INDEX:
                self.updatePlannedEndDate(action)
            action.updateExecutionPlanByRecord()
            self.emitRowChanged(row)
            return True
        elif column == DURATION_INDEX:
            group = self._groups[row]
            value = forceInt(value)
            if value < 1:
                return False

            v = forceInt(group.headItem[0].value('duration'))
            if  v == value:
                return True

            result = group.updateDuration(value)
            if result:
                self.emitRowChanged(row)
            return result
        elif column == BEG_DATE_INDEX:
            group = self._groups[row]
            record, action = group.headItem
            record.setValue('begDate', value)
            self.updatePlannedEndDate(action)
            action.updateExecutionPlanByRecord()
            self.emitRowChanged(row)
            return True

        return False

    def data(self, index, role=Qt.DisplayRole):
        row = index.row()
        if not (0 <= row < len(self._groups)):
            return QtCore.QVariant()

        group = self._groups[row]

        column = index.column()

        if role == Qt.DisplayRole and column in self.STATIC_HEADERS:
            item = group.headItem
            return self._formatGroupData(row, column, item)

        elif role == Qt.BackgroundRole and column not in self.STATIC_HEADERS:
            return self._formatDateValue(group, self._date, self._columnToDay(column))

        return QtCore.QVariant()

    @property
    def cellSettings(self):
        return self._cellsSettings

    def _formatGroupData(self, row, column, item):
        record, action = item
        if column == BEG_DATE_INDEX:
            return toVariant(forceDate(record.value('begDate')))
        elif column == PLAN_END_DATE:
            return record.value('plannedEndDate')
        elif column == DOSES_INDEX:
            nomenclatureId = self._groups[row].nomenclatureId
            group = self._groups[row]
            pt = self.cellSettings.getGroupDosesPT(group)
            if pt:
                property = action.getPropertyById(pt.id)
                dosage = property.getText() or 0
                values = self.cellSettings._getNomenclatureValues(nomenclatureId)
                if not values:
                    return toVariant(str(dosage))
                return toVariant('%s %s' % (dosage, values.get('name')))

            return toVariant('')
        elif column in (NOMENCLATURE_INDEX, SIGNA_INDEX):
            propertyType = self._cellsSettings.getByRowColumn(row, column)
            if propertyType:
                property = action.getPropertyById(propertyType.id)
                return QtCore.QVariant(property.getText())
            else:
                QtCore.QVariant(u'---')
            return QtCore.QVariant()
        elif column == DURATION_INDEX:
            return record.value('duration')
        elif column == ALIQUOTICITY_INDEX:
            return record.value('aliquoticity')
        elif column == PERIODICITY_INDEX:
            return record.value('periodicity')
        else:
            raise RuntimeError("Invalid column index")

    def _formatDateValue(self, group, date, day):
        dateToCheck = QtCore.QDate(date.year(), date.month(), day)
        items = group.searchByDate(dateToCheck)
        if items:
            closed = [i for i in items if i[0].value('status') == CActionStatus.finished]
            if len(closed) == len(items):
                overdue = [i for i in items if
                           forceDate(i[0].value('endDate')) and forceDate(i[0].value('endDate')) > dateToCheck]
                if overdue:
                    return QtCore.QVariant(OVERDUE_COLOR)
                return QtCore.QVariant(DONE_COLOR)
            return QtCore.QVariant(NOT_DONE_COLOR)

        else:
            if group.hasPlannedDate(date, day):
                return QtCore.QVariant(NOT_DONE_COLOR)

    def existsByIndex(self, index):
        row = index.row()
        if not (0 <= row < len(self._groups)):
            return False

        column = index.column()

        if column in self.STATIC_HEADERS:
            return False

        dateToCheck = QtCore.QDate(self._date.year(), self._date.month(), self._columnToDay(column))

        group = self._groups[row]

        return bool(group.plannedOnDate(dateToCheck))

    def existsDoneByIndex(self, index):
        row = index.row()
        if not (0 <= row < len(self._groups)):
            return False

        column = index.column()

        if column in self.STATIC_HEADERS:
            return False

        dateToCheck = QtCore.QDate(self._date.year(), self._date.month(), self._columnToDay(column))

        group = self._groups[row]

        items = group.searchByDate(dateToCheck)
        if not items:
            return False

        closed = [i for i in items if i[0].value('status') == CActionStatus.finished]
        return bool(closed)

    def getActionByRow(self, row):
        return self._groups[row].headItem[1]

    def createEditor(self, index, parent):
        if not self._canEditIndex(index):
            return None

        row = index.row()
        column = index.column()
        if row == len(self._groups):
            action = self.getNewAction(self._actionTypeId)
            actionType = CActionTypeCache.getById(self._actionTypeId)
            for propertyType in actionType.getPropertiesById().values():
                if propertyType.inActionsSelectionTable:
                    if propertyType.inActionsSelectionTable == _nomenclatureInTableIndex:
                        return propertyType.createEditor(action, parent, None, None)
            return None

        if column in (DURATION_INDEX, ALIQUOTICITY_INDEX, PERIODICITY_INDEX):
            return QtGui.QSpinBox(parent)

        if column == BEG_DATE_INDEX:
            return QtGui.QDateEdit(parent)

        elif column in (DOSES_INDEX, NOMENCLATURE_INDEX, SIGNA_INDEX):
            group = self._groups[row]
            propertyType = self._cellsSettings.get(group).get(column)
            if not propertyType:
                return None
            return propertyType.createEditor(group.headItem[1], parent, None, None)
        return None

    def setEditorData(self, index, editor):
        if not self._canEditIndex(index):
            return

        row = index.row()
        if row == len(self._groups):
            return

        column = index.column()

        group = self._groups[row]
        self._setEditorData(editor, group, column)

    def _setEditorData(self, editor, group, column):
        record, action = group.headItem

        if column in (DURATION_INDEX, ALIQUOTICITY_INDEX, PERIODICITY_INDEX):
            fieldName = _mapColumnIndex2FieldName[column]
            return editor.setValue(forceDouble(record.value(fieldName)))
        elif column == BEG_DATE_INDEX:
            dt = forceDateTime(record.value('begDate'))
            editor.setDateTime(dt)
        elif column in (DOSES_INDEX, SIGNA_INDEX, NOMENCLATURE_INDEX):
            pt = self._cellsSettings.get(group).get(column)
            if not pt:
                return None
            value = action.getPropertyById(pt.id).getValue()
            if column in (DOSES_INDEX, NOMENCLATURE_INDEX):
                editor.setValue(value or 0)
            else:
                editor.setText(value or '')
        return

    def getEditorData(self, index, editor):
        if not self._canEditIndex(index):
            return QtCore.QVariant()
        if index.column() == SIGNA_INDEX:
            return QtCore.QVariant(editor.text())
        if index.column() == BEG_DATE_INDEX:
            return QtCore.QVariant(editor.dateTime())
        return QtCore.QVariant(editor.value())

    def getDateByColumn(self, column):
        return QtCore.QDate(self._date.year(), self._date.month(), self._columnToDay(column))

    def emitRowChanged(self, row):
        index1 = self.index(row, 0)
        index2 = self.index(row, self.columnCount() - 1)
        self.emit(QtCore.SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index1, index2)

    def emitAllChanged(self):
        index1 = self.index(0, 0)
        index2 = self.index(self.rowCount(), self.columnCount() -1)
        self.emit(QtCore.SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index1, index2)

    def getNewAction(self, actionTypeId, nomenclatureId=None, amount=None, financeId=None, contractId=None):
        record = QtGui.qApp.db.table('Action').newRecord()
        action = CAction.getFilledAction(self._eventEditor, record, actionTypeId, initPresetValues=True)
        if nomenclatureId:
            actionType = action.getType()
            for propertyType in actionType.getPropertiesById().values():
                if propertyType.inActionsSelectionTable:
                    if propertyType.inActionsSelectionTable == _nomenclatureInTableIndex:
                        property = action.getPropertyById(propertyType.id)
                        property.setValue(nomenclatureId)

        if amount:
            record.setValue('amount', QtCore.QVariant(amount))
        elif not forceInt(record.value('amount')):
            record.setValue('amount', QtCore.QVariant(1))
        if financeId:
            record.setValue('finance_id', QtCore.QVariant(financeId))
        if contractId:
            record.setValue('contract_id', QtCore.QVariant(contractId))

        return action

    def isActionDpedBegDatePlusAmount(self, action):
        actionType = action.getType()
        return actionType.defaultPlannedEndDate == CActionType.dpedBegDatePlusDuration

    def updatePlannedEndDate(self, action):
        record = action.getRecord()
        if self.isActionDpedBegDatePlusAmount(action):
            begDate = forceDate(record.value('begDate'))
            amountValue = forceInt(record.value('amount'))
            date = begDate.addDays(amountValue-1) if begDate and amountValue else QtCore.QDate()
            record.setValue('plannedEndDate', toVariant(date))
        else:
            begDate = forceDate(record.value('begDate'))
            durationValue = forceInt(record.value('duration'))
            date = begDate.addDays(durationValue-1) if begDate and durationValue else QtCore.QDate()
            record.setValue('plannedEndDate', toVariant(date))
