# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
from PyQt4 import QtGui
from PyQt4.QtCore import QDate

from Accounting.Tariff import CTariff
from library.PrintInfo import CDateInfo, CRBInfo, CInfo
from library.Utils     import forceString, forceBool, forceRef, forceDouble, forceInt, forceDate, calcAgeTuple
from library.AgeSelector import parseAgeSelector, checkAgeSelector
from RefBooks.RBMedicalAidKind import CMedicalAidKindInfo
from RefBooks.RBMedicalAidProfile import CMedicalAidProfileInfo
from RefBooks.RBMedicalAidType import CMedicalAidTypeInfo


class CServiceGroupInfo(CRBInfo):
    tableName = 'rbServiceGroup'

    def _initByRecord(self, record):
        self._regionalCode = forceString(record.value('regionalCode'))


    def _initByNull(self):
        self._regionalCode = None

    regionalCode  = property(lambda self: self.load()._regionalCode)


class CServiceInfo(CRBInfo):
    tableName = 'rbService'

    def _initByRecord(self, record):
        self._groupId = forceRef(record.value('group_id'))
        self._eisLegacy = forceBool(record.value('eisLegacy'))
        self._license = forceBool(record.value('license'))
        self._infis = forceString(record.value('infis'))
        self._begDate = CDateInfo(record.value('begDate'))
        self._endDate = CDateInfo(record.value('endDate'))
        self._medicalAidProfile = self.getInstance(CMedicalAidProfileInfo, forceInt(record.value('medicalAidProfile_id')))
        self._medicalAidKind = self.getInstance(CMedicalAidKindInfo, forceInt(record.value('medicalAidKind_id')))
        self._medicalAidType = self.getInstance(CMedicalAidTypeInfo, forceInt(record.value('medicalAidType_id')))
        


    def _initByNull(self):
        self._groupId = None
        self._eisLegacy = False
        self._eisLegacy = False
        self._license = False
        self._infis = ''
        self._begDate = CDateInfo()
        self._endDate = CDateInfo()
        self._medicalAidProfile = self.getInstance(CMedicalAidProfileInfo, None)
        self._medicalAidKind = self.getInstance(CMedicalAidKindInfo, None)
        self._medicalAidType = self.getInstance(CMedicalAidTypeInfo, None)

    def prepareTariff(self, contractId, clientId = None):
        self.tariff = CTariffInfo(self.context,  self.id)
        self.tariff.setContractId(contractId)
        if clientId:
            self.tariff.setClientId(clientId)

    group       = property(lambda self: self.getInstance(CServiceGroupInfo, self.load()._groupId))
    eisLegacy   = property(lambda self: self.load()._eisLegacy)
    license     = property(lambda self: self.load()._license)
    infis       = property(lambda self: self.load()._infis)
    begDate     = property(lambda self: self.load()._begDate)
    endDate     = property(lambda self: self.load()._endDate)
    medicalAidProfile = property(lambda self: self.load()._medicalAidProfile)
    medicalAidKind = property(lambda self: self.load()._medicalAidKind)
    medicalAidType = property(lambda self: self.load()._medicalAidType)


# WTF? что это? почему это здесь?
class CTariffInfo(CInfo):
    def __init__(self, context, serviceId):
        CInfo.__init__(self, context)
        self._serviceId = serviceId
        self._masterId = None
        self._price = None
        self._maxAmount = 0
        self._contractDescr = None
        self._clientId = None
        self._clientBirthDate = None


    def setContractId(self, contractId):
        self._masterId = contractId

    def setClientId(self, clientId):
        self._clientId = clientId


    def _load(self):
        if self._masterId and self._serviceId:
            db = QtGui.qApp.db
            tableContractTariff = db.table('Contract_Tariff')
            cond = [tableContractTariff['deleted'].eq(0),
                    tableContractTariff['service_id'].eq(self._serviceId),
                    tableContractTariff['master_id'].eq(self._masterId)
                   ]

            recordList = db.getRecordList(tableContractTariff, '*', cond)
            tariffRecord = None
            for record in recordList:
                age = forceString(record.value('age'))
                ageSelector = None
                if age and self._clientId:
                    ageSelector = parseAgeSelector(age)
                    if not self._clientBirthDate:
                        self._clientBirthDate = forceDate(db.translate('Client', 'id', self._clientId, 'birthDate'))
                    clientAge = calcAgeTuple(self._clientBirthDate, QDate.currentDate())
                    if not clientAge:
                        clientAge = (0, 0, 0, 0)
                    if checkAgeSelector(ageSelector, clientAge):
                        tariffRecord = record
                        break
                else:
                    tariffRecord = record
                    break
            if tariffRecord:
                self._price            = forceDouble(tariffRecord.value('price'))
                frags = [(0.0, 0.0, self._price)]
                if forceDouble(tariffRecord.value('frag1Start')):
                    frags.append(( forceDouble(tariffRecord.value('frag1Start')),
                                   forceDouble(tariffRecord.value('frag1Sum')),
                                   forceDouble(tariffRecord.value('frag1Price')),
                                ))
                if forceDouble(tariffRecord.value('frag2Start')):
                    frags.append(( forceDouble(tariffRecord.value('frag2Start')),
                                   forceDouble(tariffRecord.value('frag2Sum')),
                                   forceDouble(tariffRecord.value('frag2Price'))/2,
                                ))
                frags.reverse()
                self._frags            = frags
                self._tariffType = forceInt(tariffRecord.value('tariffType'))
                self._maxAmount = forceDouble(tariffRecord.value('amount'))
                self._tariff = CTariff(tariffRecord, 2)
                return True
        return False


    def getPrice(self, amount, eventInfo = None):
        from Accounting.Utils import getContractDescr
        from Accounting.AccountBuilder import evalPriceForKrasnodarA13, evalPriceForMurmansk2015Hospital
        self.load()
        if self._maxAmount and amount > self._maxAmount:
            amount = self._maxAmount
        if self._price:
            if self._tariffType == CTariff.ttEventByMESLen:
                sum = 0
                for fragStart, fragSum, fragPrice in self._frags:
                    if amount>=fragStart:
                        sum = fragSum + (amount-fragStart)*fragPrice
                        break
                price = sum/amount if amount else self._price
                return sum
            elif self._tariffType == CTariff.ttVisitsByMES: #визиты по МЭС
                return self._price
            elif  self._tariffType == CTariff.ttKrasnodarA13 and eventInfo is not None and eventInfo._loaded:
                if not self._contractDescr:
                    self._contractDescr = getContractDescr(self._masterId)
                amount = 1.0
                clientId = eventInfo._clientId
                eventId = eventInfo.id
                eventTypeId = eventInfo.getEventTypeId()
                eventBegDate = eventInfo._setDate.date
                eventEndDate = eventInfo._execDate.date
                price  = evalPriceForKrasnodarA13(self._contractDescr, self._tariff, clientId, eventId, eventTypeId, eventBegDate, eventEndDate)
                sum    = round(price, 2)
                return sum
            elif self._tariffType == CTariff.ttMurmansk2015Hospital and eventInfo is not None and eventInfo._loaded:
                if not self._contractDescr:
                    self._contractDescr = getContractDescr(self._masterId)
                amount = 1.0
                clientId = eventInfo._clientId
                eventId = eventInfo.id
                eventTypeId = eventInfo.getEventTypeId()
                eventBegDate = eventInfo._setDate.date
                eventEndDate = eventInfo._execDate.date
                shortHospitalisation = eventBegDate.addDays(1) > eventEndDate
                level = eventInfo.mesSpecification.level
                mesId = eventInfo.mes.id
                price  = evalPriceForMurmansk2015Hospital(self._contractDescr, self._tariff, eventId, eventTypeId, eventBegDate, eventEndDate, mesId,
                                     shortHospitalisation,
                                     level)
                sum    = round(price, 2)
                return sum
            else:
                return self._price * amount
        return 0
