# -*- coding: utf-8 -*-
#pylint: disable=R0921
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""Абстрактные классы экспорта реестров счетов, экспорта в XML,
    примесевый класс экспорта данных с кэшированием."""

import os
import shutil

from PyQt4 import QtGui
from PyQt4.QtCore import (Qt, QDate, QDir, QTime, QXmlStreamWriter,
    pyqtSignature, SIGNAL, QVariant)

from library.Utils import (anyToUnicode, forceDate, forceDouble,
    forceRef, forceString, forceStringEx, toVariant, trim, forceInt)

from Registry.Utils import getAttachRecord
from Events.Action import CAction
from Exchange.Utils import getClientRepresentativeInfo
from Exchange.Ui_ExportPage2 import Ui_ExportPage2

# ******************************************************************************

class CExportHelperMixin(object):
    u"""Класс часто используемых функций экспорта"""
    sexMap = {1: u'М', 2: u'Ж'}

    def __init__(self):
        self._representativeInfoCache = {}
        self._regionCenterNameCache = {}
        self._regionNameCache = {}
        self._okatoCache = {}
        self._specialityRegionalCodeCache = {}
        self._specialityFederalCodeCache = {}
        self._personSpecialityFederalCodeCache = {}
        self._personRegionalCodeCache = {}
        self._personFederalCodeCache = {}
        self._serviceCodeCache = {}
        self._serviceInfisCache = {}
        self._specialityCache = {}
        self._medicalAidProfileFederalCodeCache = {}
        self._medicalAidProfileRegionalCodeCache = {}
        self._actionPropertyCache = {}
        self._medicalAidTypeRegionalCodeCache = {}
        self._serviceToMedicalAidTypeIdCache = {}
        self._hospitalBedProfileRegionalCodeCache = {}
        self._hospitalBedProfileTfomsCodeCache = {}
        self._hospitalBedProfileCodeCache = {}
        self._hospitalBedProfileId = {}
        self._personOrgStructureInfisCache = {}
        self._actionPropertyTypeCache = {}
        self._personOrgStructureInfisDepTypeCache = {}
        self._personOrgStructureIdCache = {}
        self._personNameCache = {}
        self._orgInfisCodeCache = {}
        self._orgStructureInfisCodeCache = {}
        self._orgStructureTfomsCodeCache = {}
        self._orgStructureInfisDepTypeCodeCache = {}
        self._actionPropertyValueCache = {}
        self._orgStructureInfisInternalCodeCache = {}
        self._clientAttachCache = {}
        self._serviceToMedicalAidProfileIdCache = {}
        self._orgCodesCache = {}
        self._toothNumberCache = {}
        self._db = QtGui.qApp.db


    def getTableFieldById(self, _id, tableName, fieldName, cacheDict,
                          idFieldName='id'):
        u"""Получает поле из таблицы по имени и идентификатору.
            Результат кэшируется."""
        result = cacheDict.get(_id, -1)

        if result == -1:
            result = self._db.translate(tableName, idFieldName, _id, fieldName)
            cacheDict[_id] = result

        return result


    def getClientRepresentativeInfo(self, clientId):
        u"""Возвращает информацию о представите клиента."""

        key = clientId
        result = self._representativeInfoCache.get(key)

        if not result:
            result = getClientRepresentativeInfo(clientId)
            self._representativeInfoCache[key] = result

        return result


    def getAttachRecord(self, clientId, isTemporary):
        u"""Возвращает словарь с иноформацией о прикреплении пациента
        или None, если прикрепление не найдено"""

        key = (clientId, isTemporary)
        result = self._clientAttachCache.get(key)

        if not result:
            result = getAttachRecord(clientId, isTemporary)
            self._clientAttachCache[key] = result

        return result


    def getRegionCenterName(self, code):
        u"""Возвращает название регионального центра."""

        result = self._regionCenterNameCache.get(code)

        if not result:
            result = ''

            stmt = """SELECT `NAME` FROM kladr.KLADR
            WHERE `CODE` LIKE '%s%%' AND `STATUS` IN ('2','3')
            """ % code[:2]

            query = self._db.query(stmt)

            while query.next():
                record = query.record()

                if record:
                    result = forceString(record.value(0))

            self._regionCenterNameCache[code] = result

        return result


    def getRegionName(self, code):
        u""" Возвращает название района. Области отфильтровываются."""

        result = self._regionNameCache.get(code)

        if not result:
            if code != '':
                regionCode = code[:5].ljust(13, '0')
                result = (forceString(self._db.translate('kladr.KLADR', 'CODE',
                    regionCode, 'NAME')) if regionCode[2:5] != '000' else
                        self.getRegionCenterName(code))
            else:
                result = ''

            self._regionNameCache[code] = result

        return result


    def getRegionOKATO(self, subRegionOKATO):
        u"""Определяем ОКАТО региона по ОКАТО района через КЛАДР."""
        result = self._okatoCache.get(subRegionOKATO, -1)

        if result == -1:
            result = ''
            kladrCode = forceString(self._db.translate(
                'kladr.KLADR', 'OCATD', subRegionOKATO, 'CODE'))

            if kladrCode:
                result = forceString(self._db.translate(
                    'kladr.KLADR', 'CODE', kladrCode[:2].ljust(13, '0'),
                    'OCATD')).ljust(5, '0') if kladrCode else ''

            self._okatoCache[subRegionOKATO] = result

        return result


    def getServiceMedicalAidTypeId(self, serviceId):
        u"""Получает идентификатор типа медицинской помощи по
        идентификатору услуги."""
        return forceRef(self.getTableFieldById(serviceId, 'rbService',
                    'medicalAidType_id', self._serviceToMedicalAidTypeIdCache))


    def getServiceMedicalAidProfileId(self, serviceId):
        u"""Получает идентификатор профиля медицинской помощи по
        идентификатору услуги."""
        return forceRef(self.getTableFieldById(serviceId, 'rbService',
                    'medicalAidProfile_id', self._serviceToMedicalAidProfileIdCache))


    def getMedicalAidTypeRegionalCode(self, _id):
        u"""Возвращает региональный код типа медицинской помощи
        по идентификатору типа."""
        return forceString(self.getTableFieldById(_id, 'rbMedicalAidType',
                        'regionalCode', self._medicalAidTypeRegionalCodeCache))


    def getHospitalBedProfileId(self, _id):
        u"""Получает по id койки id ее профиля"""
        return forceRef(self.getTableFieldById(_id, 'OrgStructure_HospitalBed',
                        'profile_id', self._hospitalBedProfileId))


    def getHospitalBedProfileRegionalCode(self, _id):
        u"""Получает по id профиля койки его региональный код"""
        return forceString(self.getTableFieldById(_id, 'rbHospitalBedProfile',
                    'regionalCode', self._hospitalBedProfileRegionalCodeCache))


    def getHospitalBedProfileTfomsCode(self, _id):
        u"""Получает по id профиля койки его региональный код"""
        return forceString(self.getTableFieldById(_id, 'rbHospitalBedProfile',
                    'tfomsCode', self._hospitalBedProfileTfomsCodeCache))
                    

    def getHospitalBedProfileCode(self, _id):
        u"""Получает по id профиля койки его код"""
        return forceString(self.getTableFieldById(_id, 'rbHospitalBedProfile',
                        'code', self._hospitalBedProfileCodeCache))


    def getPersonSpecialityId(self, personId):
        u"""Определяем id специальности врача по его id"""
        return forceRef(self.getTableFieldById(personId, 'Person',
                        'speciality_id', self._specialityCache))


    def getPersonName(self, _id):
        u"""Получает ФИО сотрудника по его id"""
        return forceString(self.getTableFieldById(_id, 'vrbPerson',
                        'name', self._personNameCache))


    def getOrgInfisCode(self, _id):
        u"""Получает инфис код организации по его id"""
        return forceString(self.getTableFieldById(_id, 'Organisation',
                        'infisCode', self._orgInfisCodeCache))


    def getOrgStructureInfisCode(self, _id):
        u"""Получает инфис код отделения по его id"""
        return forceString(self.getTableFieldById(_id, 'OrgStructure',
                        'infisCode', self._orgStructureInfisCodeCache))


    def getOrgStructureTfomsCode(self, _id):
        u"""Получает инфис код отделения по его id"""
        return forceString(self.getTableFieldById(_id, 'OrgStructure',
                        'tfomsCode', self._orgStructureTfomsCodeCache))


    def getOrgStructureInfisInternalCode(self, _id):
        u"""Получает инфис код отделения по его id"""
        return forceString(self.getTableFieldById(_id, 'OrgStructure',
                'infisInternalCode', self._orgStructureInfisInternalCodeCache))


    def getOrgStructureInfisDepTypeCode(self, _id):
        u"""Получает инфис код отделения по его id"""
        return forceString(self.getTableFieldById(_id, 'OrgStructure',
                'infisDepTypeCode', self._orgStructureInfisDepTypeCodeCache))


    def getPersonSpecialityRegionalCode(self, personId):
        u"""Определяем региональный код специальности врача по его id"""
        result = self._specialityRegionalCodeCache.get(personId, -1)

        if result == -1:
            result = None
            specialityId = self.getPersonSpecialityId(personId)

            if specialityId:
                result = forceString(self._db.translate('rbSpeciality', 'id',
                                                specialityId, 'regionalCode'))

            self._specialityRegionalCodeCache[personId] = result

        return result


    def getSpecialityFederalCode(self, _id):
        u"""Получает федеральный код специальности по его id"""
        return forceString(self.getTableFieldById(_id, 'rbSpeciality',
                'federalCode', self._specialityFederalCodeCache))


    def getPersonSpecialityFederalCode(self, personId):
        u"""Определяем федеральный код специальности врача по его id"""
        result = self._personSpecialityFederalCodeCache.get(personId, -1)

        if result == -1:
            result = None
            specialityId = self.getPersonSpecialityId(personId)

            if specialityId:
                result = forceString(self._db.translate('rbSpeciality', 'id',
                                                specialityId, 'federalCode'))

            self._personSpecialityFederalCodeCache[personId] = result

        return result


    def getPersonRegionalCode(self, personId):
        u"""Определяем региональный код врача по его id"""
        return forceString(self.getTableFieldById(personId, 'Person',
                'regionalCode', self._personRegionalCodeCache))


    def getPersonFederalCode(self, personId):
        u"""Определяем федеральный код врача по его id"""
        return forceString(self.getTableFieldById(personId, 'Person',
                'federalCode', self._personFederalCodeCache))


    def getServiceCode(self, _id):
        u"""Определяем код профиля оплаты по его id"""
        result = self._serviceCodeCache.get(_id, -1)

        if result == -1:
            result = forceString(self._db.translate('rbService', 'id', _id,
                'code'))
            self._serviceCodeCache[_id] = result

        return result


    def getServiceInfis(self, _id):
        u"""Определяем инфис код профиля оплаты по его id"""
        result = self._serviceInfisCache.get(_id, -1)

        if result == -1:
            result = forceString(self._db.translate('rbService', 'id', _id,
                'infis'))
            self._serviceInfisCache[_id] = result

        return result


    def getMedicalAidProfileFederalCode(self, _id):
        u"""Определяем федеральный код профиля мед. помощи по его id"""
        result = self._medicalAidProfileFederalCodeCache.get(_id, -1)

        if result == -1:
            result = forceString(self._db.translate('rbMedicalAidProfile',
                'id', _id, 'federalCode'))
            self._medicalAidProfileFederalCodeCache[_id] = result

        return result


    def getMedicalAidProfileRegionalCode(self, _id):
        u"""Определяем региональный код профиля мед. помощи по его id"""
        result = self._medicalAidProfileRegionalCodeCache.get(_id, -1)

        if result == -1:
            result = forceString(self._db.translate('rbMedicalAidProfile',
                'id', _id, 'regionalCode'))
            self._medicalAidProfileRegionalCodeCache[_id] = result

        return result


    def getActionPropertyText(self, eventId, actionTypeId, name):
        u"""Возвращает строку, содержащую текствое свойство действия
        по идентификатору события, типа действия и имени свойства"""
        key = (eventId, actionTypeId, name)
        result = self._actionPropertyCache.get(key, -1)

        if result == -1:
            result = ''

            if actionTypeId and eventId and name:
                table = self._db.table('Action')
                record = self._db.getRecordEx(table, '*', [
                    table['event_id'].eq(eventId),
                    table['actionType_id'].eq(actionTypeId)])

                if record:
                    action = CAction(record=record)
                    prop = action.getProperty(name)

                    if prop:
                        val = prop.getTextScalar()
                        result = val if val else ''

            self._actionPropertyCache[key] = result

        return result


    def getActionPropertyValue(self, eventId, actionTypeId, name):
        u"""Возвращает значение свойства действия по идентификатору
            события, типа действия и имени свойства"""
        key = (eventId, actionTypeId, name)
        result = self._actionPropertyValueCache.get(key, -1)

        if result == -1:
            result = None

            if actionTypeId and eventId and name:
                table = self._db.table('Action')
                record = self._db.getRecordEx(table, '*', [
                    table['event_id'].eq(eventId),
                    table['actionType_id'].eq(actionTypeId)])

                if record:
                    action = CAction(record=record)
                    prop = action.getProperty(name)

                    if prop:
                        result = prop.getValueScalar()

            self._actionPropertyValueCache[key] = result

        return result


    def getActionPropertyTypeIdByName(self, actionTypeId, name):
        u"""Получаем id типа свойства действия по id типа действия
        и имени свойства"""

        key = (actionTypeId, name)
        result = self._actionPropertyTypeCache.get(key, -1)

        if result == -1:
            result = None
            tableActionPropertyType = self._db.table('ActionPropertyType')
            cond = []

            cond.append(tableActionPropertyType['name'].eq(name))
            cond.append(tableActionPropertyType['deleted'].eq(0))
            cond.append(tableActionPropertyType['actionType_id'].eq(
                                                                actionTypeId))

            record = self._db.getRecordEx(tableActionPropertyType, 'id', cond)

            if record:
                result = forceRef(record.value(0))

            self._actionPropertyTypeCache[key] = result

        return result


    def getPersonOrgStructreId(self, personId):
        u"""Возвращет id подразделения сотрудника"""
        result = self._personOrgStructureIdCache.get(personId, -1)

        if result == -1:
            result = forceRef(self._db.translate(
                'Person', 'id', personId, 'orgStructure_id'))
            self._personOrgStructureIdCache[personId] = result

        return result


    def getPersonOrgStructreInfis(self, personId):
        u"""Возвращет код подразделения сотрудника"""
        result = self._personOrgStructureInfisCache.get(personId, -1)

        if result == -1:
            result = None
            orgStructureId = self.getPersonOrgStructreId(personId)

            if orgStructureId:
                result = self.getOrgStructureInfisCode(orgStructureId)

            self._personOrgStructureInfisCache[personId] = result

        return result


    def getPersonOrgStructreInfisDepType(self, personId):
        u"""Возвращет код типа подразделения сотрудника"""
        result = self._personOrgStructureInfisDepTypeCache.get(personId, -1)

        if result == -1:
            result = None
            orgStructureId = self.getPersonOrgStructreId(personId)

            if orgStructureId:
                result = self.getOrgStructureInfisDepTypeCode(orgStructureId)

            self._personOrgStructureInfisCache[personId] = result

        return result


    def _getAttachOrgCodes(self, orgId):
        u"""Возвращает словарь с тремя кодами: инфис, смо, тфомс"""
        result = self._orgCodesCache.get(orgId)

        if not result:
            record = self.db.getRecord('Organisation',
                'infisCode,smoCode,tfomsExtCode', orgId)
            result = {
                'attachOrgCode': forceString(record.value('infisCode')),
                'attachOrgSmoCode': forceString(record.value('smoCode')),
                'attachOrgTfomsExtCode': forceString(record.value(
                    'tfomsExtCode'))
            } if record else {}
            self._orgCodesCache[orgId] = result

        return result


    def toothNumber(self, actionId):
        u"""Возвращает номер зуба по идентификатору действия"""

        result = self._toothNumberCache.get(actionId, -1)

        if result == -1:
            result = ''
            action = CAction.getActionById(actionId)
            if action:
                try:
                    result = action[u'№ зуба']

                    if not result:
                        result = ''

                except KeyError:
                    self.logWarning(u'Для действия id=%d отсутствует свойство'
                        u' "№ зуба"' % actionId)
            self._toothNumberCache[actionId] = result
        return result

# ******************************************************************************

class LoggerMixin(object):
    u'Примесевый класс для вывода сообщений в журнал QTextbrowser logBrowser'

    def log(self, _str, forceLog=False):
        if not hasattr(self, 'logBrowser'):
            return

        if hasattr(self, 'chkVerboseLog'):
            verboseLog = self.chkVerboseLog.isChecked()
        else:
            verboseLog = True

        if verboseLog or forceLog:
            timeStr = forceString(QTime.currentTime().toString('hh:mm:ss'))
            self.logBrowser.append('[%s] %s' % (timeStr, _str))
            self.logBrowser.update()


    def logWarning(self, _str):
        self.log(u'<font color=orange><b>Предупреждение</b></font>: %s' % _str,
            forceLog=True)


    def logError(self, _str):
        self.log(u'<font color=red><b>ОШИБКА</b></font>: %s' % _str,
            forceLog=True)

# ******************************************************************************

class CAbstractExportWizard(QtGui.QWizard):
    u"""Родительский класс мастера экспорта счетов."""

    def __init__(self, parent=None, title=''):
        QtGui.QWizard.__init__(self, parent)
        self.setWizardStyle(QtGui.QWizard.ModernStyle)
        self.setWindowTitle(title)
        self.dbfFileName = ''
        self.tmpDir = ''
        self.db = QtGui.qApp.db
        self.info = {}
        self.accountId = None


    def setAccountId(self, accountId):
        u"""Устанавливает счета для экспорта по его идентификатору"""
        if hasattr(self, 'page1'):
            self.accountId = accountId

            self.info = self.getAccountInfo(accountId)
            date = self.info.get('accDate', QDate())
            number = self.info.get('accNumber', '')

            strNumber = number if trim(number) else u'б/н'
            strDate = forceString(date) if date else u'б/д'
            self.page1.setTitle(u'Экспорт данных реестра по счёту'
                u' №%s от %s' % (strNumber, strDate))

        if hasattr(self, 'page2'):
            self.page2.setTitle(u'Укажите директорий для сохранения '
                u'обменных файлов "*.dbf"')


    def setAccountExposeDate(self):
        u"""Меняет дату выставления счета на текущую"""
        accountRecord = self.db.table('Account').newRecord(['id', 'exposeDate'])
        accountRecord.setValue('id', toVariant(self.accountId))
        accountRecord.setValue('exposeDate', toVariant(QDate.currentDate()))
        self.db.updateRecord('Account', accountRecord)


    def getTmpDir(self):
        u"""Возвращает строку с путем к временному каталогу"""
        return self.getTmpDirEx('EXPORT')


    def getTmpDirEx(self, prefix):
        u"""Возвращает строку с путем к временному каталогу с учетом префикса"""
        if not self.tmpDir:
            self.tmpDir = QtGui.qApp.getTmpDir(prefix)
        return self.tmpDir


    def getFullDbfFileName(self):
        u"""Возвращает полный путь к DBF файлу"""
        return os.path.join(self.getTmpDir(), self.dbfFileName + '.dbf')


    def setAccountItemsIdList(self, accountItemIdList):
        u"""Устанавливает список элементов реестра счета"""
        self.page1.setAccountItemsIdList(accountItemIdList)


    def cleanup(self):
        u"""Удаляет временный каталог при его наличии"""
        if self.tmpDir:
            QtGui.qApp.removeTmpDir(self.tmpDir)
            self.tmpDir = ''


    def exec_(self):
        u"""Запускает диалог мастера экспорта, по окончании работы
        вызывает метод cleanup()"""
        QtGui.QWizard.exec_(self)
        self.cleanup()


    def getAccountInfo(self, accountId):
        u"""Возвращает словарь с данными о счете по его идентификатору"""
        result = {}
        record = self.db.getRecord('Account', 'id, date, number, exposeDate, '
            'contract_id, payer_id, settleDate, personalDate, note, sum, '
            'orgStructure_id, amount', accountId)

        if record:
            result['accDate'] = forceDate(record.value('date'))
            result['exposeDate'] = forceDate(record.value('exposeDate'))
            result['accNumber'] = forceString(record.value('number'))
            result['contractId'] = forceRef(record.value('contract_id'))
            result['payerId'] = forceRef(record.value('payer_id'))
            result['settleDate'] = forceDate(record.value('settleDate'))
            result['personalDate'] = forceDate(record.value('personalDate'))
            result['note'] = forceString(record.value('note'))
            result['accSum'] = forceDouble(record.value('sum'))
            result['accId'] = forceRef(record.value('id'))
            result['accOrgStructureId'] = forceRef(record.value('orgStructure_id'))
            result['accAmount'] = forceInt(record.value('amount'))

        return result

# ******************************************************************************

class CAbstractExportPage1Common(QtGui.QWizardPage, LoggerMixin):
    u"""Абстрактная страница 1 мастера экспорта счетов.
        Для работы необходимо реализовать следующие функции:
            setExportMode(flag)
            createDbf()
            createQuery()
            process()
        Имя для индикатора прогресса: progressBar
        Имя журнала ошибок: logBrowser
        Имя checkbox'a включающего подробный журнал ошибок: chkVerboseLog
        """
    def __init__(self, parent):
        QtGui.QWizardPage.__init__(self, parent)
        LoggerMixin.__init__(self)
        self.setTitle(u'Экспорт данных реестра по счёту')
        self.setSubTitle(u'для выполнения шага нажмите кнопку "экспорт"')
        self.setupUi(self)

        if hasattr(self, 'progressBar'):
            self.progressBar.setMinimum(0)
            self.progressBar.setMaximum(1)
            self.progressBar.setText('')

        self.setExportMode(False)
        self.aborted = False
        self.done = False
        self.idList = []
        self._parent = parent
        self.connect(parent, SIGNAL('rejected()'), self.abort)
        self.db = QtGui.qApp.db
        self.tableAccountItem = self.db.table('Account_Item')
        self._processParams = {}
        self._processFuncList = []

        if hasattr(self, 'lblRevision'):
            try:
                from s11main import gLastChangedRev
                self.lblRevision.setText(u'рев. %s' % gLastChangedRev)
            except:
                pass


    def setExportMode(self, flag):
        """Выключаем элементы управления, если flag == True,
        иначе -- включаем. Абстрактный метод."""
        raise NotImplementedError


    def validatePage(self):
        return self.done


    def setAccountItemsIdList(self, accountItemIdList):
        self.idList = accountItemIdList


    def prepareToExport(self):
        self.done = False
        self.aborted = False
        self.emit(SIGNAL('completeChanged()'))
        self.setExportMode(True)
        dest = self.createExportDest()

        try:
            from s11main import (gTitle, gVersion,
                 gLastChangedRev, gLastChangedDate)
            self.log(u'<font color=blue>%s, вер. %s (рев. <b>%s</b> от '
                u'<b>%s</b>)</font>' % (gTitle, gVersion, gLastChangedRev,
                    gLastChangedDate))
        except:
            pass

        self.log(u'Запрос в БД...')
        if hasattr(self, 'progressBar'):
            self.progressBar.reset()
            self.progressBar.setMaximum(1)
            self.progressBar.setValue(0)
            self.progressBar.setText(u'Запрос в БД...')

        QtGui.qApp.processEvents()
        query = QtGui.qApp.callWithWaitCursor(self, self.createQuery)

        if hasattr(self, 'progressBar'):
            querySize = 0

            if isinstance(query, list) or isinstance(query, tuple):
                for item in query:
                    if callable(item):
                        continue

                    querySize += item.size()
            else:
                querySize = query.size()

            self.progressBar.setMaximum(max(querySize, 1))
            self.progressBar.reset()
            self.progressBar.setValue(0)

        return dest, query


    def export(self):
        self._processParams = {}
        (result, _) = QtGui.qApp.call(self, self.exportInt)
        self.setExportMode(False)

        if self.aborted or not result:
            if hasattr(self, 'progressBar'):
                self.progressBar.setText(u'прервано')
            self.log(u'Прервано')
        else:
            if hasattr(self, 'progressBar'):
                self.progressBar.setText(u'готово')
            self.log(u'Успешно завершено')
            self.done = True
            self.emit(SIGNAL('completeChanged()'))


    def _processQuery(self, dest, query, func, time, startPos):
        while query.next():
            QtGui.qApp.processEvents()

            if self.aborted:
                break

            if hasattr(self, 'progressBar'):
                self.progressBar.step()

            func(dest, query.record(), self._processParams)

            if hasattr(self, 'progressBar') and hasattr(self, 'lblElapsed'):
                elapsed = time.elapsed()

                if elapsed != 0 and (self.progressBar.value()-startPos > 100):
                    startPos = self.progressBar.value()
                    oldSpeed = 100*1000/elapsed
                    newSpeed = (oldSpeed+100*1000/elapsed)/2
                    if newSpeed != 0:
                        finishTime = QTime.currentTime().addSecs(
                             self.progressBar.maximum()/newSpeed)
                        self.lblElapsed.setText(u'Текущая операция: %03.f '
                            u'зап/с, окончание в %s' % (
                              newSpeed, finishTime.toString('hh:mm:ss')))
                        time.restart()


    def preprocessQuery(self, query, params):
        pass


    def postExport(self):
        pass


    def preExport(self):
        pass


    def exportInt(self):
        dest, query = self.prepareToExport()
        time = QTime()
        startPos = 0
        self.log(u'Обработка результатов...')
        self.preprocessQuery(query, self._processParams)

        if (hasattr(self, 'lblElapsed') and hasattr(self, 'progressBar')
                and (self.progressBar.maximum() > 100)):
            self.lblElapsed.setText(
                u'Текущая операция: ??? зап/с, окончание в ??:??:??')
            time.start()

        self.preExport()

        if self.idList:
            if ((isinstance(query, list) or isinstance(query, tuple)) and
                    self._processFuncList != []):
                for (q, f) in zip(query, self._processFuncList):
                    if callable(q):
                        q = q()
                    self._processQuery(dest, q, f, time, startPos)
            else:
                self._processQuery(dest, query, self.process, time, startPos)

        else:
            self.log(u'Нечего выгружать.')

            if hasattr(self, 'progressBar'):
                self.progressBar.step()

        self.postExport()

        if isinstance(dest, (list, tuple)):
            for item in dest:
                if item is not None:
                    item.close()
        else:
            dest.close()


    def setProcessParams(self, params):
        self._processParams = params


    def processParams(self):
        return self._processParams


    def setProcessFuncList(self, funcList):
        self._processFuncList = list(funcList)


    def process(self, dest, record, params):
        raise NotImplementedError


    def prepareStmt(self, params):
        u'Возращает tuple из строк select, таблицы, where, orderBy'
        raise NotImplementedError


    def createQuery(self):
        (select, tables, where, orderBy) = self.prepareStmt(self._processParams)
        stmt = (u'SELECT {select} FROM {table}\n'
            u'WHERE {where} ORDER BY {orderBy}').format(
            select=select, table=tables, where=where, orderBy=orderBy)
        return self.db.query(stmt)


    def createExportDest(self):
        raise NotImplementedError


    def isComplete(self):
        return self.done


    def abort(self):
        self.aborted = True


    def getTmpDir(self):
        return self._parent.getTmpDir()


    def accountInfo(self):
        return self._parent.info


    def accountId(self):
        return self._parent.accountId


    @pyqtSignature('')
    def on_btnExport_clicked(self):
        self.export()


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        self.abort()


    def getEventSum(self):
        u"""Возвращает общую стоимость услуг за событие"""

        stmt = """SELECT Account_Item.event_id,
            SUM(Account_Item.`sum`) AS totalSum
        FROM Account_Item
        LEFT JOIN rbPayRefuseType ON rbPayRefuseType.id = Account_Item.refuseType_id
        WHERE Account_Item.reexposeItem_id IS NULL
            AND (Account_Item.date IS NULL
                 OR (Account_Item.date IS NOT NULL AND rbPayRefuseType.rerun != 0)
                )
            AND %s
        GROUP BY Account_Item.event_id;""" \
            % self.tableAccountItem['id'].inlist(self.idList)

        query = self.db.query(stmt)
        result = {}

        while query.next():
            record = query.record()
            eventId = forceRef(record.value(0))
            result[eventId] = forceDouble(record.value(1))

        return result


    def getEventCount(self):
        u"""Возвращает строкой количество событий в счете."""

        result = u''
        stmt = """SELECT COUNT(DISTINCT Account_Item.event_id)
        FROM Account_Item
        WHERE %s""" % self.tableAccountItem['id'].inlist(self.idList)

        query = self.db.query(stmt)

        if query.first():
            record = query.record()
            result = forceString(record.value(0))

        return result

# ******************************************************************************

class CAbstractExportPage1(CAbstractExportPage1Common):
    def __init__(self, parent):
        CAbstractExportPage1Common.__init__(self, parent)


    def createExportDest(self):
        return self.createDbf()


    def createDbf(self):
        raise NotImplementedError

# ******************************************************************************

class CAbstractExportPage1Xml(CAbstractExportPage1Common):
    def __init__(self, parent,  xmlWriter=None):
        CAbstractExportPage1Common.__init__(self, parent)

        self._xmlWriter = xmlWriter


    def setXmlWriter(self, xmlWriter):
        self._xmlWriter = xmlWriter


    def xmlWriter(self):
        return self._xmlWriter


    def createExportDest(self):
        return self.xmlWriter()


    def process(self, dest, record, params):
        if isinstance(dest, (tuple, set, list)):
            for i in dest:
                i.writeRecord(record, params)

                if hasattr(i, 'hasError') and i.hasError():
                    raise IOError(u'Ошибка записи данных')
        else:
            dest.writeRecord(record, params)

            if hasattr(dest, 'hasError') and dest.hasError():
                raise IOError(u'Ошибка записи данных')


    def postExport(self):
        if isinstance(self._xmlWriter, (tuple, set, list)):
            for i in self._xmlWriter:
                i.writeFooter(self._processParams)
        else:
            self._xmlWriter.writeFooter(self._processParams)


    def preExport(self):
        if isinstance(self._xmlWriter, (tuple, set, list)):
            for i in self._xmlWriter:
                i.writeStartDocument()
                i.writeHeader(self._processParams)
        else:
            self._xmlWriter.writeStartDocument()
            self._xmlWriter.writeHeader(self._processParams)

# ******************************************************************************

class CAbstractExportPage2(QtGui.QWizardPage, Ui_ExportPage2):
    def __init__(self, parent, prefix=''):
        QtGui.QWizardPage.__init__(self, parent)
        self._parent = parent
        self.setupUi(self)
        self.setTitle(u'Экспорт данных реестра по счёту')
        self.setSubTitle(u'для выполнения шага нажмите кнопку "финиш"')

        self.pathIsValid = True
        homePath = QDir.toNativeSeparators(QDir.homePath())
        self.prefix = prefix
        self.exportDir = forceString(QtGui.qApp.preferences.appPrefs.get('%sExportDir' % prefix, homePath))
        self.edtDir.setText(self.exportDir)


    def isComplete(self):
        return self.pathIsValid


    def saveExportResults(self):
        u"""Абстрактный метод, используется для сохранения результатов экспорта
        в указанный пользователем каталог. Должен вернуть true в случае успеха,
        иначе false."""
        raise NotImplementedError


    def moveFiles(self, fileList):
        u"""Переносит файлы из списка в каталог, выбранный пользователем"""

        for src in fileList:
            srcFullName = os.path.join(forceStringEx(self.getTmpDir()),
                                                    os.path.basename(src))
            dst = os.path.join(forceStringEx(self.edtDir.text()),
                                                    os.path.basename(src))
            success, _ = QtGui.qApp.call(self, shutil.move,
                                              (srcFullName, dst))

            if not success:
                break

        return success


    def validatePage(self):
        success = self.saveExportResults()

        if success:
            QtGui.qApp.preferences.appPrefs[
                '%sExportDir' % self.prefix] = toVariant(self.edtDir.text())
            self.wizard().setAccountExposeDate()
        return success


    def getTmpDir(self):
        return self._parent.getTmpDir()


    @pyqtSignature('QString')
    def on_edtDir_textChanged(self):
        path = forceStringEx(self.edtDir.text())
        pathIsValid = os.path.isdir(path)
        if self.pathIsValid != pathIsValid:
            self.pathIsValid = pathIsValid
            self.emit(SIGNAL('completeChanged()'))


    @pyqtSignature('')
    def on_btnSelectDir_clicked(self):
        path = QtGui.QFileDialog.getExistingDirectory(self,
                u'Выберите директорий для сохранения файла выгрузки',
                 forceStringEx(self.edtDir.text()),
                 QtGui.QFileDialog.ShowDirsOnly)
        if forceString(path):
            self.edtDir.setText(QDir.toNativeSeparators(path))

# ******************************************************************************

class CAbstractExportXmlStreamWriter(QXmlStreamWriter):
    u"""Абстрактный класс для экспорта в формате XML"""
    def __init__(self, parent):
        QXmlStreamWriter.__init__(self)
        self._parent = parent
        self.setAutoFormatting(True)
        self.aborted = False


    def abort(self):
        self.aborted = True


    def writeHeader(self, params):
        u"""Запись заголовка xml файла, абстрактный метод"""
        raise NotImplementedError


    def writeRecord(self, record, params):
        u"""Пишет sql запись в файл, абстрактный метод"""
        raise NotImplementedError


    def writeFooter(self, params):
        raise NotImplementedError


    def writeElement(self, elementName, value=None):
        self.writeTextElement(elementName, '' if not value else value)


    def writerFuncEx(self, prefix, rec, field, dateFields=None):
        val = rec.value('%s_%s' % (prefix, field))

        if dateFields and field in dateFields:
            self.writeElement(field, forceDate(val).toString(Qt.ISODate))
        elif isinstance(val, (tuple, list, set)):
            for i in val:
                self.writeElement(field, forceString(i))
        else:
            self.writeElement(field, forceString(val))


    def writeGroup(self, groupName, fieldList, record, subGroup=None,
                closeGroup=True, namePrefix=None, dateFields=tuple(),
                openGroup=True):

        fieldName = '%s_%s' % (groupName, fieldList[0])

        if isinstance(record.value(fieldName), (tuple, list)):
            items = []
            keys = []
            for f in fieldList:
                name = '%s_%s' % (groupName, f)
                items.append(record.value(name))
                keys.append(name)

            for group in zip(*items):
                r = ExtendedRecord(params=dict(zip(keys, group)))
                self.writeGroup(groupName, fieldList, r, subGroup, True,
                    namePrefix, dateFields, True)

            return

        if openGroup:
            self.writeStartElement(groupName)

        if namePrefix:
            groupName = '%s_%s' % (namePrefix, groupName)

        exportedSubgroups = set()

        # если поле совпадает с именем подгруппы, выгружаем ее
        for field in fieldList:
            if subGroup and field in subGroup.keys():
                val = subGroup[field]
                self.writeGroup(field, val.get('fields', tuple()), record,
                    val.get('subGroup'), True, val.get('prefix'),
                    val.get('dateFields'))
                exportedSubgroups.add(field)
            else:
                self.writerFuncEx(groupName, record, field, dateFields)

        # выгружаем оставшиемя подгруппы
        if subGroup:
            for (name, val) in subGroup.iteritems():
                if name not in exportedSubgroups:
                    self.writeGroup(name, val.get('fields', tuple()), record,
                        val.get('subGroup'), True, val.get('prefix'),
                        val.get('dateFields'))

        if closeGroup:
            self.writeEndElement()


    def writeFile(self, device, query, progressBar=None):
        try:
            self.setDevice(device)

            if progressBar:
                progressBar.setMaximum(max(query.size(), 1))
                progressBar.reset()
                progressBar.setValue(0)

            self.writeStartDocument()
            self.writeHeader()
            self.aborted = False

            while query.next():
                self.writeRecord(query.record())

                if progressBar:
                    progressBar.step()

                if self.aborted:
                    return False

                QtGui.qApp.processEvents()

            self.writeFooter()
            self.writeEndDocument()

        except IOError, exception:
            QtGui.qApp.logCurrentException()
            msg = ''

            if hasattr(exception, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (
                   exception.filename, exception.errno,
                   anyToUnicode(exception.strerror))
            else:
                msg = u'[Errno %s] %s' % (exception.errno,
                                          anyToUnicode(exception.strerror))

            QtGui.QMessageBox.critical(self._parent,
                u'Произошла ошибка ввода-вывода', msg,
                QtGui.QMessageBox.Close)
            return False

        except Exception, exception:
            QtGui.qApp.logCurrentException()
            QtGui.QMessageBox.critical(self._parent,
                u'Произошла ошибка', unicode(exception),
                QtGui.QMessageBox.Close)
            return False

        return True


    def close(self):
        u"""Закрывает устройство при его наличии"""
        if self.device():
            self.device().close()

# ******************************************************************************

class ExtendedRecord(object):
    u"""Возвращает значения и QSqlRecord record, если они есть,
    иначе из указанного словаря params"""
    def __init__(self, record=None, params=None, debug=False):
        self._record = record
        self._params = params
        self._debug = debug

    def setRecord(self, record):
        u"""Установить новое значение поля record"""
        self._record = record


    def setParams(self, params):
        u"""Установить новое значение поля params"""
        self._params = params


    def value(self, name):
        u"""Возвращает поле по имени. Либо из записи, либо из словаря"""
        result = QVariant()

        if self._record and self._record.contains(name):
            result = self._record.value(name)
        elif self._params:
            if self._debug and not self._params.has_key(name):
                print('ExtendedRecord: unknown key `%s`' % name)

            result = self._params.get(name, QVariant())
        else:
            if self._debug:
                print('ExtendedRecord: unknown key `%s`' % name)

        return result


    def contains(self, field):
        result = False

        if self._record:
            result = self._record.contains(field)

        if self._params:
            result = result or self._params.has_key(field)

        return result


    def setValue(self, name, val):
        u"""Сохраняет новое значение для поля"""
        if self._record and self._record.contains(name):
            self._record.setValue(name, val)
        elif self._params:
            self._params[name] = val
