# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2016-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""Экспорт счетов для Пскова"""

import json
import os

from PyQt4.QtCore import Qt, QFile, QIODevice, QTextStream

from library.AmountToWords import amountToWords
from library.Utils import (forceDouble, forceInt, forceRef, forceString,
        forceBool, toVariant)

from Events.Action import CAction
from Exchange.Export import ExtendedRecord, CExportHelperMixin
from Exchange.Order79Export import (COrder79ExportWizard, COrder79ExportPage1,
        COrder79ExportPage2, COrder79PersonalDataWriter,
        COrder79XmlStreamWriter)
from Exchange.Utils import compressFileInZip

from Exchange.Ui_ExportR60NativeAmbulancePage1 import Ui_ExportPage1

DEBUG = False

def exportR60Ambulance(widget, accountId, accountItemIdList, accountIdList):
    u"""Создает диалог экспорта реестра счета"""

    wizard = CExportWizard(widget)
    wizard.setAccountId(accountId)
    wizard.setAccountItemsIdList(accountItemIdList)
    wizard.setAccountIdList(accountIdList)
    wizard.exec_()


def getXmlBaseFileName(_db, info, packetNumber, addPostfix=True):
    u"""Возвращает имя файла для записи данных"""

    def getPrefix(org):
        u"""Возвращает префиксы по коду инфис и признаку страховой"""
        result = ''

        if not org['isMedical'] and not org['isInsurer']:
            result = 'T'
        else:
            result = 'S' if org['isInsurer'] else 'M'

        return result

    tableContract = _db.table('Contract')
    payer = {'id': info['payerId']}
    recipient = {'id': forceRef(_db.translate(tableContract, 'id',
                                info['contractId'], 'recipient_id'))}

    for org in (payer, recipient):
        record = _db.getRecord('Organisation',
            'infisCode, isInsurer, isMedical', org['id'])

        if record:
            org['code'] = forceString(record.value(0))
            org['isInsurer'] = forceBool(record.value(1))
            org['isMedical'] = forceBool(record.value(2))

    p_i = getPrefix(recipient)
    p_p = getPrefix(payer)

    postfix = u'%02d%02d%d' % (info['settleDate'].year() %100,
        info['settleDate'].month(), packetNumber) if addPostfix else u''
    result = u'%s%s%s%s_%s.xml' % (
            p_i, recipient['code'], p_p, payer['code'], postfix)
    return result

# ******************************************************************************

class CExportWizard(COrder79ExportWizard):
    u"""Мастер экспорта Псковской области"""

    def __init__(self, parent=None):
        title = u'Мастер экспорта реестра услуг для Псковской области'
        prefix = 'R60NativeAmbulance'
        COrder79ExportWizard.__init__(self, title, prefix, CExportPage1,
                                      CExportPage2, parent)

        self.accountIdList = None
        self.__xmlBaseFileName = None


    def setAccountIdList(self, accountIdList):
        u"""Запоминает список идентификаторов счетов"""
        self.accountIdList = accountIdList


    def getZipFileName(self):
        u"""Возвращает имя архива"""
        return u'%s.zip' % self.getXmlFileName()[:-4]


    def _getXmlBaseFileName(self, addPostfix=True):
        u"""Возвращает имя файла для записи данных"""
        result = self.__xmlBaseFileName

        if not result:
            result = getXmlBaseFileName(self.db, self.info,
                    self.page1.edtPacketNumber.value(), addPostfix)
            self.__xmlBaseFileName = result

        return result


    def getTxtFileName(self):
        u"""Возвращает имя файла счет-фактуры"""
        return u'sf%s.txt'% self._getXmlBaseFileName(True)


    def getFullTxtFileName(self):
        u"""Возвращает полное имя архива данных.
        Добавляется путь к временному каталогу"""
        return os.path.join(self.getTmpDir(), self.getTxtFileName())

# ******************************************************************************

class CExportPage1(COrder79ExportPage1, Ui_ExportPage1):
    u"""Первая страница мастера экспорта"""

    mapMesLevelToInvoice = {2: 0, 1: 1, 0: 1}
    ambulanceCodes = (26, 27, 29, 30) # Поликлиника
    emergencyCodes = (31, 24) # СМП
    urgentCodes = (39, 41) # Неотложная
    stomatologyCodes = (9, )

    def __init__(self, parent, prefix):
        xmlWriter = (XmlStreamWriter(self),
                     PersonalDataWriter(self))
        COrder79ExportPage1.__init__(self, parent, prefix, xmlWriter)
        self.cmbRegistryType.setVisible(False)
        self.lblRegistryType.setVisible(False)

        self._contractAttributeCache = {}
        self._contractAttributeTypeCache = {}


    def preprocessQuery(self, query, params):
        ambulanceIncomplete = not self.chkAmbulancePlanComplete.isChecked()
        emergencyIncomplete = not self.chkEmergencyPlanComplete.isChecked()
        params['ambulanceIncomplete'] = ambulanceIncomplete
        params['emergencyIncomplete'] = emergencyIncomplete

        if ambulanceIncomplete:
            params['SUM_POD'] = 0

        if emergencyIncomplete:
            params['SUM_SMP'] = 0

        params['accSum'] = 0

        while query.next():
            record = query.record()
            idsp = forceInt(record.value('Z_SL_IDSP'))
            val = forceDouble(record.value('sum'))

            if idsp in self.emergencyCodes + self.ambulanceCodes:
                if emergencyIncomplete and idsp in self.emergencyCodes:
                    params['SUM_SMP'] += val
                elif ambulanceIncomplete and idsp in self.ambulanceCodes:
                    params['SUM_POD'] += val
            else:
                params['accSum'] += val

        query.exec_()


    def nschet(self, registryType, params):
        settleDate = params['settleDate']

        return u'{p}H{year:02d}{month:02d}{registryNumber:d}'.format(
            p=params['lpuMiacCode'][-3:],
            year=settleDate.year() % 100,
            month=settleDate.month(),
            registryNumber=self.edtPacketNumber.value()
        )


    def getStatistics(self):
        u'Заполняет значения для SUM_POD, SUM_SMP'
        result = (0, 0)

        stmt = """SELECT
            SUM(IF(rbMedicalAidUnit.federalCode IN ('26', '27'),
                Account_Item.sum, 0)),
            SUM(IF(rbMedicalAidUnit.federalCode NOT IN ('31', '26', '27'),
                Account_Item.sum, 0))
        FROM Account_Item
        LEFT JOIN rbMedicalAidUnit ON rbMedicalAidUnit.id = Account_Item.unit_id
        WHERE {0}""".format(self.tableAccountItem['id'].inlist(self.idList))

        query = self.db.query(stmt)


        if query.first():
            record = query.record()
            ambulance = forceDouble(record.value(0))
            other = forceDouble(record.value(1))
            result = (ambulance, other)

        return result


    def getCompleteEventSum(self):
        u"""Возвращает общую стоимость услуг за событие"""

        stmt = """SELECT getLastEventId(Account_Item.event_id) AS lastEventId,
            SUM(Account_Item.`sum`) AS totalSum
        FROM Account_Item
        LEFT JOIN rbPayRefuseType ON rbPayRefuseType.id = Account_Item.refuseType_id
        WHERE Account_Item.reexposeItem_id IS NULL
            AND (Account_Item.date IS NULL
                 OR (Account_Item.date IS NOT NULL AND rbPayRefuseType.rerun != 0)
                )
            AND %s
        GROUP BY lastEventId;""" \
            % self.tableAccountItem['id'].inlist(self.idList)

        query = self.db.query(stmt)
        result = {}

        while query.next():
            record = query.record()
            eventId = forceRef(record.value(0))
            result[eventId] = forceDouble(record.value(1))

        return result


    def getCompleteEventCount(self):
        u"""Возвращает строкой количество событий в счете."""

        result = u''
        stmt = """SELECT COUNT(DISTINCT getLastEventId(Account_Item.event_id))
        FROM Account_Item
        WHERE %s""" % self.tableAccountItem['id'].inlist(self.idList)

        query = self.db.query(stmt)

        if query.first():
            record = query.record()
            result = forceString(record.value(0))

        return result


    def getBedDaysCount(self):
        days = {}
        completeDays = {}
        stmt = """SELECT  Action.event_id,
                getLastEventId(Action.event_id),
                DATEDIFF(MAX(Action.endDate), MIN(Action.begDate)),
                rbMedicalAidType.code
            FROM Account_Item
            LEFT JOIN Event ON Account_Item.event_id = Event.id
            LEFT JOIN Action ON Action.event_id = Event.id AND
                      Action.deleted = 0 AND
                      Action.actionType_id = (
                            SELECT AT.id
                            FROM ActionType AT
                            WHERE AT.flatCode ='moving'
                      )
            LEFT JOIN EventType ON EventType.id = Event.eventType_id
            LEFT JOIN rbMedicalAidType ON
                EventType.medicalAidType_id = rbMedicalAidType.id
            WHERE {0}
            GROUP BY Action.event_id""".format(
                self.tableAccountItem['id'].inlist(self.idList))

        query = self.db.query(stmt)

        while query.next():
            record = query.record()
            eventId = forceRef(record.value(0))
            lastEventId = forceRef(record.value(1))
            val = forceInt(record.value(2))
            medicalAidTypeCode = forceInt(record.value(3))

            if medicalAidTypeCode == 7:
                val += 1

            if val <= 0:
                val = 1

            days[eventId] = val

            if completeDays.has_key(lastEventId):
                completeDays[lastEventId] += val
            else:
                completeDays[lastEventId] = val

        return (days, completeDays)


    def getMedicalAidType(self):
        stmt = """SELECT Event.id, rbMedicalAidType.code
        FROM Account_Item
        LEFT JOIN Event ON Account_Item.event_id = Event.id
        LEFT JOIN EventType ON EventType.id = Event.eventType_id
        LEFT JOIN rbMedicalAidType ON
            EventType.medicalAidType_id = rbMedicalAidType.id
        WHERE {0}
        GROUP BY Account_Item.event_id""".format(
            self.tableAccountItem['id'].inlist(self.idList))

        query = self.db.query(stmt)
        result = {}

        while query.next():
            record = query.record()
            eventId = forceRef(record.value(0))
            code = forceInt(record.value(1))
            result[eventId] = code

        return result


    def prepareStmt(self, params):
        select = u"""Account_Item.event_id AS eventId,
            Event.client_id AS clientId,
            Event.order AS eventOrder,
            MesService.infis AS mesServiceInfis,
            MesAction.begDate AS mesActionBegDate,
            MesAction.endDate AS mesActionEndDate,
            Account_Item.`sum` AS `sum`,
            Account_Item.uet,
            Account_Item.amount,
            Account_Item.usedCoefficients,
            LastEvent.id AS lastEventId,
            EventType.id AS eventTypeId,
            HospitalAction.id AS hospActionId,
            Diagnostic.dispanser_id AS dispanserId,

            rbService.name AS invoiceName, -- поля для печатной формы
            rbService.code AS invoiceCode,
            1 AS invoiceAmount,
            Account_Item.price AS invoicePrice,
            HospitalAction.amount AS invoiceHospitalAmount,
            VisitCount.amount AS invoiceVisitCount,

            LeavedOrgStruct.tfomsCode AS invoiceOrgStructCode,
            LeavedOrgStruct.name AS invoiceOrgStructName,
            AnotherMesService.infis AS invoiceServiceCode,
            IFNULL(CONCAT(InvoiceMesService.code, ' ', InvoiceMesService.name),
                CONCAT(MesMKB.DiagId, ' ', MesMKB.DiagName)) AS invoiceServiceName,
            rbMesSpecification.level AS invoiceMesLevel,

            Event.client_id AS PACIENT_ID_PAC,
            rbPolicyKind.regionalCode AS PACIENT_VPOLIS,
            IF(rbPolicyKind.regionalCode = '1', ClientPolicy.serial, '') AS PACIENT_SPOLIS,
            ClientPolicy.number AS PACIENT_NPOLIS,
            Insurer.miacCode AS PACIENT_SMO,
            IF(Insurer.miacCode IS NULL OR Insurer.miacCode = '',
                        Insurer.OGRN,'') AS PACIENT_SMO_OGRN,
            IF(Insurer.miacCode IS NULL OR Insurer.miacCode = '',
                        Insurer.OKATO,'') AS PACIENT_SMO_OK,
            IF((Insurer.miacCode IS NULL OR Insurer.miacCode = '')
                    AND (Insurer.OGRN IS NULL OR Insurer.OGRN = ''),
                    Insurer.shortName, '') AS PACIENT_SMO_NAM,
            Client.birthWeight AS PACIENT_VNOV_D,
            0 AS PACIENT_NOVOR,
            IF(MseAction.id IS NOT NULL, 1, '') AS PACIENT_MSE,

            getLastEventId(Account_Item.event_id) AS Z_SL_IDCASE,
            rbEventTypePurpose.regionalCode AS Z_SL_USL_OK,
            IFNULL(OrgStructure_Identification.value, rbMedicalAidKind.regionalCode) AS Z_SL_VIDPOM,
            0 AS Z_SL_FOR_POM,
            PersonOrganisation.miacCode AS Z_SL_LPU,
            IFNULL(NextEvent.setDate, Event.setDate) AS Z_SL_DATE_Z_1,
            IFNULL(LastEvent.execDate, Event.execDate) AS Z_SL_DATE_Z_2,
            0 AS Z_SL_KD_Z,
            EventResult.federalCode AS Z_SL_RSLT,
            rbDiagnosticResult.federalCode AS Z_SL_ISHOD,
            IF(NextEvent.id IS NOT NULL, 1, '') AS Z_SL_VB_P,
            rbMedicalAidUnit.federalCode AS Z_SL_IDSP,
            IF(rbMedicalAidUnit.federalCode IN ('31', '35') AND rbEventProfile.regionalCode = '5',
                0, IF(rbEventProfile.regionalCode = '6', UetInfo.`sum`, Account_Item.`sum`)) AS Z_SL_SUMV,
            Relegate.miacCode AS Z_SL_NPR_MO,
            IFNULL(Event.srcDate, Event.setDate) AS Z_SL_NPR_DATE,

            Account_Item.event_id AS SL_SL_ID,
            PersonOrgStructure.infisCode AS SL_LPU_1,
            IFNULL(LeavedOrgStruct.tfomsCode,
                IFNULL(EventType_Identification.value, PersonOrgStructure.tfomsCode)) AS SL_PODR,
            IF(rbService_Profile.medicalAidProfile_id IS NOT NULL,
                ServiceProfileMedicalAidProfile.code,
                    ServiceMedicalAidProfile.code) AS SL_PROFIL,
            IF(age(Client.birthDate,Event.execDate) < 18, 1, 0) AS SL_DET,
            rbService_Identification.value AS SL_P_CEL,
            IF(rbEventProfile.regionalCode IN ('1', '3'), Event.externalId, Client.id) AS SL_NHISTORY,
            IF(EventType.form = '110', Event.execDate, Event.setDate) AS SL_DATE_1,
            Event.execDate AS SL_DATE_2,
            DS0.MKB AS SL_DS0,
            Diagnosis.MKB AS SL_DS1,
            DS9.MKB AS SL_DS2,
            DS3.MKB AS SL_DS3,
            mes.MES_ksg.code AS SL_CODE_MES1,
            EventResult.federalCode AS SL_RSLT,
            Person.federalCode AS SL_IDDOKT,
            IF(rbMedicalAidUnit.federalCode IN ('9', '24', '31'), Account_Item.price,'') AS SL_TARIF,
            IF(rbMedicalAidUnit.federalCode IN ('31', '35') AND rbEventProfile.regionalCode = '5',
                0, IF(rbEventProfile.regionalCode = '6', UetInfo.`sum`, Account_Item.`sum`)) AS SL_SUM_M,
            IF(rbEventProfile.regionalCode IN ('1', '3'),
                HospitalAction.amount + IF(rbEventProfile.regionalCode = '3' AND HospitalAction.cnt > 1, 1, 0),
                IF(rbEventProfile.regionalCode = '6', UetInfo.uet, Account_Item.amount)) AS SL_ED_COL,
            rbSpeciality.federalCode AS SL_PRVS,

            mes.MES_ksg.code AS KSG_KPG_N_KSG,
            mes.MES_ksg.vk AS KSG_KPG_KOEF_Z,
            IF(mes.MES_ksg.vk IS NOT NULL, Account_Item.price/mes.MES_ksg.vk, 0) AS KSG_KPG_KOEF_U,

            PersonOrganisation.miacCode AS USL_LPU,
            PersonOrgStructure.infisCode AS USL_LPU_1,
            IFNULL(LeavedOrgStruct.tfomsCode,
                IFNULL(EventType_Identification.value, PersonOrgStructure.tfomsCode)) AS USL_PODR,
            IF(rbService_Profile.medicalAidProfile_id IS NOT NULL,
                ServiceProfileMedicalAidProfile.code,
                    ServiceMedicalAidProfile.code) AS USL_PROFIL,
            IF(SUBSTR(rbService.code, 1, 3) = 'A16', rbService.code, '') AS USL_VID_VME,
            IF(age(Client.birthDate,Event.execDate) < 18, 1, 0) AS USL_DET,
            Event.setDate AS USL_DATE_IN,
            Event.execDate AS USL_DATE_OUT,
            Diagnosis.MKB AS USL_DS,
            IF(Account_Item.visit_id IS NOT NULL,
                VisitService.infis, rbService.infis) AS USL_CODE_USL,
            IF(rbEventProfile.regionalCode IN ('1', '3'),
                HospitalAction.amount + IF(rbEventProfile.regionalCode = '3' AND HospitalAction.cnt > 1, 1, 0),
                Account_Item.amount) AS USL_KOL_USL,
            IF(rbMedicalAidUnit.federalCode IN ('9', '24', '31'), Account_Item.price,'') AS USL_TARIF,
            IF(rbMedicalAidUnit.federalCode IN ('31', '35') AND rbEventProfile.regionalCode = '5',
                0, Account_Item.`sum`) AS USL_SUMV_USL,
            rbSpeciality.federalCode AS USL_PRVS,
            Person.federalCode AS USL_IDDOKT,
            IF(rbEventProfile.regionalCode = '2', VisitPerson.federalCode,
                Person.federalCode) AS USL_CODE_MD,

            Event.client_id AS PERS_ID_PAC,
            UPPER(Client.lastName) AS PERS_FAM,
            UPPER(Client.firstName) AS PERS_IM,
            UPPER(Client.patrName) AS PERS_OT,
            Client.sex AS PERS_W,
            Client.birthDate AS PERS_DR,
            UPPER(Client.birthPlace) AS PERS_MR,
            rbDocumentType.regionalCode AS PERS_DOCTYPE,
            IF(rbDocumentType.code = '003',
                REPLACE(TRIM(ClientDocument.serial),' ', '-'),
                TRIM(ClientDocument.serial)) AS PERS_DOCSER,
            TRIM(ClientDocument.number) AS PERS_DOCNUM,
            RegKLADR.OCATD AS PERS_OKATOG,
            LocKLADR.OCATD AS PERS_OKATOP,

            rbEventProfile.regionalCode AS eventProfileRegionalCode,
            DeliveredBy.value AS delivered"""
        tables = u"""Account_Item
            LEFT JOIN Event ON Event.id  = Account_Item.event_id
            LEFT JOIN Client ON Client.id = Event.client_id
            LEFT JOIN ClientDocument ON ClientDocument.client_id = Client.id AND
                      ClientDocument.id = (SELECT MAX(CD.id)
                                         FROM   ClientDocument AS CD
                                         LEFT JOIN rbDocumentType AS rbDT ON rbDT.id = CD.documentType_id
                                         LEFT JOIN rbDocumentTypeGroup AS rbDTG ON rbDTG.id = rbDT.group_id
                                         WHERE  rbDTG.code = '1' AND CD.client_id = Client.id AND CD.deleted=0)
            LEFT JOIN rbDocumentType ON rbDocumentType.id = ClientDocument.documentType_id
            LEFT JOIN ClientAddress ClientRegAddress ON ClientRegAddress.id  = getClientRegAddressId(Client.id)
            LEFT JOIN ClientAddress ClientLocAddress ON ClientLocAddress.id = getClientLocAddressId(Client.id)
            LEFT JOIN Address   RegAddress ON RegAddress.id = ClientRegAddress.address_id
            LEFT JOIN AddressHouse RegAddressHouse ON RegAddressHouse.id = RegAddress.house_id
            LEFT JOIN kladr.KLADR AS RegKLADR ON RegKLADR.CODE = RegAddressHouse.KLADRCode
            LEFT JOIN Address   LocAddress ON LocAddress.id = ClientLocAddress.address_id
            LEFT JOIN AddressHouse LocAddressHouse ON LocAddressHouse.id = LocAddress.house_id
            LEFT JOIN kladr.KLADR AS LocKLADR ON LocKLADR.CODE = LocAddressHouse.KLADRCode
            LEFT JOIN Diagnostic ON Diagnostic.id = (
                SELECT MAX(dc.id)
                    FROM Diagnostic dc
                    WHERE dc.event_id = Account_Item.event_id
                    AND dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code in ('1', '2'))
                    AND dc.deleted = 0
            )
            LEFT JOIN Diagnosis ON Diagnosis.id = Diagnostic.diagnosis_id AND Diagnosis.deleted = 0
            LEFT JOIN Diagnosis AS DS3 ON DS3.id = (
                  SELECT MAX(ds.id)
                    FROM Diagnosis ds
                    INNER JOIN Diagnostic dc ON (ds.id = dc.diagnosis_id AND ds.deleted = 0 AND dc.deleted = 0)
                    WHERE dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code = '3')
                    AND dc.event_id = Account_Item.event_id
            )
            LEFT JOIN Diagnosis AS DS9 ON DS9.id = (
                  SELECT MAX(ds.id)
                    FROM Diagnosis ds
                    INNER JOIN Diagnostic dc ON (ds.id = dc.diagnosis_id AND ds.deleted = 0 AND dc.deleted = 0)
                    WHERE dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code = '9')
                    AND dc.event_id = Account_Item.event_id
            )
            LEFT JOIN Diagnosis AS DS0 ON DS0.id = (
                  SELECT MAX(ds.id)
                    FROM Diagnosis ds
                    INNER JOIN Diagnostic dc ON (ds.id = dc.diagnosis_id AND ds.deleted = 0 AND dc.deleted = 0)
                    WHERE dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code = '7')
                    AND dc.event_id = Account_Item.event_id
            )
            LEFT JOIN rbDiagnosticResult ON rbDiagnosticResult.id=Diagnostic.result_id
            LEFT JOIN rbResult AS EventResult ON EventResult.id=Event.result_id
            LEFT JOIN rbPayRefuseType ON rbPayRefuseType.id = Account_Item.refuseType_id
            LEFT JOIN Person ON Person.id = Event.execPerson_id
            LEFT JOIN rbSpeciality ON Person.speciality_id = rbSpeciality.id
            LEFT JOIN EventType ON EventType.id = Event.eventType_id
            LEFT JOIN rbService ON rbService.id = Account_Item.service_id
            LEFT JOIN rbService_Profile ON rbService_Profile.id = (SELECT MAX(id) FROM rbService_Profile rs
                WHERE rs.master_id = rbService.id AND rs.speciality_id = Person.speciality_id)
            LEFT JOIN rbMedicalAidKind ON rbMedicalAidKind.id = IFNULL(rbService_Profile.medicalAidKind_id,
                IFNULL(rbService.medicalAidKind_id,EventType.medicalAidKind_id))
            LEFT JOIN ClientPolicy ON ClientPolicy.id = getClientPolicyIdForDate(Client.`id`, 1, Event.execDate)
            LEFT JOIN rbPolicyKind ON rbPolicyKind.id = ClientPolicy.policyKind_id
            LEFT JOIN Organisation AS Insurer ON Insurer.id = ClientPolicy.insurer_id
            LEFT JOIN rbEventTypePurpose ON rbEventTypePurpose.id = EventType.purpose_id
            LEFT JOIN mes.MES ON Event.MES_id = mes.MES.id
            LEFT JOIN mes.MES_ksg ON mes.MES_ksg.id = mes.MES.ksg_id
            LEFT JOIN rbMedicalAidProfile AS ServiceProfileMedicalAidProfile ON ServiceProfileMedicalAidProfile.id = rbService_Profile.medicalAidProfile_id
            LEFT JOIN rbMedicalAidProfile AS ServiceMedicalAidProfile ON ServiceMedicalAidProfile.id = rbService.medicalAidProfile_id
            LEFT JOIN rbMedicalAidUnit ON rbMedicalAidUnit.id = Account_Item.unit_id
            LEFT JOIN Organisation AS PersonOrganisation ON PersonOrganisation.id = Person.org_id
            LEFT JOIN OrgStructure AS PersonOrgStructure ON PersonOrgStructure.id = Person.orgStructure_id
            LEFT JOIN rbEventProfile ON rbEventProfile.id = EventType.eventProfile_id
            LEFT JOIN (
                    SELECT A.event_id, SUM(A.amount) AS amount, COUNT(DISTINCT A.id) AS cnt, MAX(A.id) AS id
                    FROM Action A
                    WHERE A.deleted = 0 AND
                              A.actionType_id IN (
                                    SELECT AT.id
                                    FROM ActionType AT
                                    WHERE AT.flatCode ='moving'
                              )
                    GROUP BY A.event_id
                ) AS HospitalAction ON HospitalAction.event_id = Account_Item.event_id
            LEFT JOIN Visit ON Account_Item.visit_id = Visit.id
            LEFT JOIN Action ON Account_Item.action_id = Action.id
            LEFT JOIN Person AS VisitPerson ON VisitPerson.id = IFNULL(Visit.person_id, Action.person_id)
            LEFT JOIN rbService AS VisitService ON VisitService.id = Visit.service_id
            LEFT JOIN Action AS MesAction ON MesAction.id = (
                    SELECT MAX(A.id)
                    FROM mes.MES_service MS
                    LEFT JOIN mes.mrbService AS S ON S.id = MS.service_id
                        AND S.deleted = 0
                    LEFT JOIN ActionType AS AType ON S.code = AType.code
                        AND AType.deleted = 0
                    LEFT JOIN Action AS A ON A.actionType_id = AType.id
                        AND A.deleted = 0
                    WHERE MS.deleted = 0 AND MS.master_id = Event.MES_id
                        AND A.event_id = Event.id
                )
            LEFT JOIN rbService AS MesService ON MesService.id = (
                    SELECT AType.nomenclativeService_id
                    FROM ActionType AS AType
                    WHERE AType.id = MesAction.actionType_id
                )
            LEFT JOIN ActionProperty_String AS DeliveredBy ON DeliveredBy.id = (
                SELECT APS.id
                FROM Action A
                INNER JOIN ActionType AS AT ON AT.id = A.actionType_id
                INNER JOIN ActionPropertyType AS APT ON APT.actionType_id = AT.id
                INNER JOIN ActionProperty AS AP ON AP.action_id = A.id AND AP.type_id = APT.id
                INNER JOIN ActionProperty_String AS APS ON APS.id = AP.id
                WHERE A.deleted = 0 AND A.event_id = Account_Item.event_id
                    AND AP.deleted = 0
                    AND AT.`flatCode` = 'received'
                    AND APT.`name` = 'Кем доставлен'
                ORDER BY A.begDate DESC
                LIMIT 0, 1
            )
            LEFT JOIN (
                SELECT Account_Item.event_id,
                    SUM(Account_Item.uet) AS uet,
                    SUM(Account_Item.`sum`) AS `sum`
                FROM Account_Item
                GROUP BY Account_Item.event_id
            ) AS UetInfo ON UetInfo.event_id = Account_Item.event_id
            LEFT JOIN EventType_Identification ON  EventType_Identification.master_id = EventType.id
                AND EventType_Identification.system_id = (
                SELECT MAX(id) FROM rbAccountingSystem
                WHERE rbAccountingSystem.code = 'НеотлПом'
            )
            LEFT JOIN OrgStructure AS LeavedOrgStruct ON LeavedOrgStruct.id = (
                SELECT OS.id
                FROM Action A
                INNER JOIN ActionType AS AT ON AT.id = A.actionType_id
                INNER JOIN ActionPropertyType AS APT ON APT.actionType_id = AT.id
                INNER JOIN ActionProperty AS AP ON AP.action_id = A.id AND AP.type_id = APT.id
                INNER JOIN ActionProperty_OrgStructure AS APO ON APO.id = AP.id
                INNER JOIN OrgStructure AS OS ON OS.id = APO.value
                WHERE A.deleted = 0 AND A.event_id = Account_Item.event_id
                    AND AP.deleted = 0
                    AND AT.`flatCode` = 'leaved'
                    AND APT.`typeName` = 'OrgStructure'
                ORDER BY A.begDate DESC
                LIMIT 0, 1
            )
            LEFT JOIN OrgStructure_Identification
                ON OrgStructure_Identification.master_id = IFNULL(LeavedOrgStruct.id, PersonOrgStructure.id)
                AND OrgStructure_Identification.deleted = 0
                AND OrgStructure_Identification.system_id = (
                SELECT MAX(id) FROM rbAccountingSystem
                WHERE rbAccountingSystem.code = 'ВидМП'
            )
            LEFT JOIN mes.mrbService AS InvoiceMesService ON InvoiceMesService.id = (
                SELECT mes.mrbService.id
                FROM mes.MES_service MS
                LEFT JOIN mes.mrbService ON mes.mrbService.id = MS.service_id
                    AND mes.mrbService.deleted = 0
                WHERE MS.deleted = 0 AND MS.master_id = Event.MES_id
                LIMIT 0, 1
            )
            LEFT JOIN MKB AS MesMKB ON MesMKB.DiagId = (
                SELECT MM.mkb
                FROM mes.MES_mkb MM
                WHERE MM.master_id = Event.MES_id AND MM.deleted = 0
                LIMIT 0, 1
            )
            LEFT JOIN rbService AS AnotherMesService ON AnotherMesService.id = (
                SELECT S.id
                FROM rbService S
                WHERE mes.MES.code = S.code AND mes.MES.deleted = 0
                LIMIT 0, 1
            )
            LEFT JOIN rbMesSpecification ON Event.mesSpecification_id = rbMesSpecification.id
            LEFT JOIN (
                SELECT V.event_id, V.service_id, COUNT(*) AS amount
                FROM Visit V
                WHERE V.deleted = 0
                GROUP BY V.event_id, V.service_id
            ) AS VisitCount ON VisitCount.event_id = Event.id
                AND VisitCount.service_id = Account_Item.service_id
            LEFT JOIN Action AS MseAction ON MseAction.id = (
                SELECT MAX(A.id)
                FROM Action A
                WHERE A.event_id = Event.id AND
                          A.deleted = 0 AND
                          A.actionType_id = (
                                SELECT AT.id
                                FROM ActionType AT
                                WHERE AT.flatCode ='DirectionMSE'
                          )
            )
            LEFT JOIN Event AS NextEvent ON NextEvent.prevEvent_id = Event.id
            LEFT JOIN Event AS LastEvent ON LastEvent.id = getLastEventId(Event.id)
            LEFT JOIN Organisation AS Relegate ON Relegate.id = Event.relegateOrg_id
            LEFT JOIN rbService_Identification ON rbService_Identification.id = (
                SELECT MAX(SId.id)
                FROM rbService_Identification SId
                WHERE SId.master_id = rbService.id
                AND SId.system_id IN (SELECT id FROM rbAccountingSystem WHERE code = 'tfomsPCEL')
                AND SId.deleted = 0
            )
            """
        where = u"""Account_Item.reexposeItem_id IS NULL
            AND (Account_Item.date IS NULL
                 OR (Account_Item.date IS NOT NULL AND rbPayRefuseType.rerun != 0)
                )
            AND %s""" % self.tableAccountItem['id'].inlist(self.idList)
        orderBy = u'Client.id, LastEvent.id, Event.id'

        return (select, tables, where, orderBy)


    def contractAttributeTypeId(self, code):
        u"""Получает идентификатор типа параметра договора по его коду"""
        result = self._contractAttributeTypeCache.get(code, -1)

        if result == -1:
            result = forceRef(self.db.translate(
                    'rbContractAttributeType', 'code', code, 'id'))
            self._contractAttributeTypeCache[code] = result

        return result


    def contractAttribute(self, attributeId):
        u"""Получает значение параметра договора по идентификатору типа"""
        result = self._contractAttributeCache.get(attributeId, -1)

        if result == -1:
            result = None
            table = self.db.table('Contract_Attribute')
            params = self._parent.info
            cond = [table['master_id'].eq(params['contractId']),
                table['deleted'].eq(0),
                table['attributeType_id'].eq(attributeId)]

            record = self.db.getRecordEx(table, 'value', cond)

            if record:
                result = forceString(record.value(0))

            self._contractAttributeCache[attributeId] = result

        return result


    def contractAttributeByType(self, code):
        u"""Возвращает параметр договора по коду типа"""
        return self.contractAttribute(self.contractAttributeTypeId(code))


    def setExportMode(self, flag):
        COrder79ExportPage1.setExportMode(self, flag)
        self.chkAmbulancePlanComplete.setEnabled(not flag)
        self.chkEmergencyPlanComplete.setEnabled(not flag)
        self.chkReexposed.setEnabled(not flag)


    def process(self, dest, record, params):
        COrder79ExportPage1.process(self, dest, record, params)
        self.processInvoice(record, params)


    def exportInt(self):
        self.invoiceFile = QFile(self.wizard().getFullTxtFileName())
        self.invoiceFile.open(QIODevice.WriteOnly | QIODevice.Text)
        self.invoiceStream = QTextStream(self.invoiceFile)
        self.invoiceStream.setCodec('UTF8')

        params = self.processParams()
        params['recNum'] = 1
        params['isReexposed'] = '1' if self.chkReexposed.isChecked() else '0'
        params['completeEventSum'] = self.getCompleteEventSum()
        params['medicalAidType'] = self.getMedicalAidType()
        params['bedDays'],  params['completeBedDays'] = self.getBedDaysCount()
        params['ambulance'], params['other'] = self.getStatistics()
        self.setProcessParams(params)

        COrder79ExportPage1.exportInt(self)


    def processInvoice(self, record, params):
        u"""Пишет данные для счет-фактуры в словарь"""

        def addToInvoice(code, name, amount, inSum, price, uet):
            if params.setdefault('invoice', {}).setdefault(
                    tableNum, {}).has_key(code):
                (_name, _amount, _sum, _price, _uet) = params[
                    'invoice'][tableNum][code]
                params['invoice'][tableNum][code] = (
                    _name, _amount + amount, _sum, _price, _uet + uet)
            else:
                params['invoice'][tableNum][code] = (
                    name, amount, inSum, price, uet)

            if params.setdefault('ambInfo', {}).setdefault(
                    tableNum, {}).has_key('totalAmount'):
                params['ambInfo'][tableNum]['totalAmount'] += amount
            else:
                params['ambInfo'][tableNum]['totalAmount'] = amount

        invoiceCode = forceString(record.value('invoiceCode'))
        invoiceName = forceString(record.value('invoiceName'))
        invoiceAmount = forceDouble(record.value('invoiceAmount'))
        invoiceSum = forceDouble(record.value('sum'))
        invoiceUet = forceDouble(record.value('uet'))
        invoicePrice = forceDouble(record.value('invoicePrice'))
        idsp = forceInt(record.value('Z_SL_IDSP'))
        clientId = forceRef(record.value('clientId'))
        params.setdefault('clients', set()).add(clientId)
        isHospital = forceInt(record.value('Z_SL_USL_OK')) in (1, 2)

        if idsp in self.ambulanceCodes:
            tableNum = 5 if invoiceCode in ('101025',
                '101125', '101034', '101134', '101099') else 1
        elif idsp in self.emergencyCodes:
            tableNum = 2
        elif idsp in self.urgentCodes:
            tableNum = 3
        elif idsp in self.stomatologyCodes:
            tableNum = 4
        else:
            tableNum = 0


        if tableNum in (1, 5):
            addToInvoice(invoiceCode[2:] if tableNum == 1 else invoiceCode, invoiceName, invoiceAmount,
                invoiceSum, invoicePrice, invoiceUet)

            if invoiceCode[3] == '1':
                addToInvoice(u'{beg}3{end}'.format(beg=invoiceCode[:2] if tableNum == 5 else '',
                    end=invoiceCode[3:]),
                    u'Первичн. прием в рамках услуги {0}'.format(invoiceName),
                    invoiceAmount, 0, 0, 0)
                addToInvoice(u'{beg}4{end}'.format(beg=invoiceCode[:2] if tableNum == 5 else '',
                    end=invoiceCode[3:]),
                    u'Повторн. прием в рамках услуги {0}'.format(invoiceName),
                    forceInt(record.value('invoiceVisitCount')) - 2*invoiceAmount,
                    0, 0, 0)
        else:
            addToInvoice(invoiceCode, invoiceName, invoiceAmount, invoiceSum,
                invoicePrice, invoiceUet)


        for (key, val) in (('totalSum', invoiceSum),
                           ('totalUet', invoiceUet),
                           ('uetSum', invoiceSum if invoiceUet else 0)):
            if params.setdefault('ambInfo', {}).setdefault(
                    tableNum, {}).has_key(key):
                params['ambInfo'][tableNum][key] += val
            else:
                params['ambInfo'][tableNum][key] = val

        params['ambInfo'][tableNum].setdefault('hospClientCount' if isHospital
            else 'clientCount', set()).add(clientId)

        self.processHospitalInvoice(record, params)
        self.processTotal(record, params)


    def processHospitalInvoice(self, record, params):
        u"""Еще одна СФ для Стационара и ДС"""

        eventProfile = forceInt(record.value('eventProfileRegionalCode'))

        if eventProfile not in (1, 3): # Стационар, ДС
            return

        invoiceOrgStructCode = forceString(record.value('invoiceOrgStructCode'))

        if not invoiceOrgStructCode:
            return

        invoiceServiceCode = forceString(record.value('invoiceServiceCode'))
        level = forceInt(record.value('invoiceMesLevel'))
        _sum = forceDouble(record.value('sum'))
        amount = forceDouble(record.value('amount'))
        uet = forceDouble(record.value('uet'))
        hospAmount = forceDouble(record.value('invoiceHospitalAmount'))
        key = (invoiceServiceCode, level, _sum)

        if params.setdefault('hospInvoice', {}
                ).setdefault(invoiceOrgStructCode, {}).has_key(key):
            params['hospInvoice'][invoiceOrgStructCode][key] += amount
        else:
            params['hospInvoice'][invoiceOrgStructCode][key] = amount

        if params.setdefault('hospInvoiceInfo', {}).setdefault(
                invoiceOrgStructCode, {}).has_key('amount'):
            params['hospInvoiceInfo'][invoiceOrgStructCode]['amount'] += amount
        else:
            params['hospInvoiceInfo'][invoiceOrgStructCode]['amount'] = amount

        if params['hospInvoiceInfo'][invoiceOrgStructCode].has_key('bedDays'):
            params['hospInvoiceInfo'][invoiceOrgStructCode][
                'bedDays'] += hospAmount
        else:
            params['hospInvoiceInfo'][invoiceOrgStructCode][
                'bedDays'] = hospAmount

        for (key, val) in (('totalSum', _sum),
                           ('totalUet', uet),
                           ('uetSum', _sum if uet > 0 else 0)):
            if params['hospInvoiceInfo'][invoiceOrgStructCode].has_key(key):
                params['hospInvoiceInfo'][invoiceOrgStructCode][key] += val
            else:
                params['hospInvoiceInfo'][invoiceOrgStructCode][key] = val

        clientId = forceRef(record.value('clientId'))
        params['hospInvoiceInfo'][invoiceOrgStructCode].setdefault(
            'clients', set()).add(clientId)

        params.setdefault('orgStructMap', {})[invoiceOrgStructCode
            ] = forceString(record.value('invoiceOrgStructName'))
        params.setdefault('serviceMap', {})[invoiceServiceCode
            ] = forceString(record.value('invoiceServiceName'))


    def processTotal(self, record, params):
        u"""Собирает данные для сводки по условиям оказания помощи."""

        def initData(name, params):
            params['total'].setdefault(name, {'clients': set(),
                'ambClients': set(), 'hospClients': set(), 'amount':0, 'sum': 0,
                'uet': 0, 'mesAmount': 0, 'mesSum': 0, 'bedDays': 0,
                'totalSum': 0,  'uetSum': 0, 'totalUet': 0,
            })

        idsp = forceInt(record.value('Z_SL_IDSP'))
        params.setdefault('total', {})
        clientId = forceRef(record.value('clientId'))
        name = None
        isHospital = False

        if idsp in self.ambulanceCodes: #Поликлиника
            name = u'Поликлиника'
        elif idsp in self.emergencyCodes: # СМП
            name = u'СМП'
        elif idsp in self.urgentCodes: # Неотложная помощь
            name = u'Неотложная помощь'
        elif idsp in self.stomatologyCodes:
            name = u'Стоматология'
        else:
            profile = forceInt(record.value('eventProfileRegionalCode'))
            isHospital = True
            if profile == 1: # Стационар
                name = u'Стационар'
            elif profile == 3: # ДС
                name = u'Дневной стационар'

        if name:
            initData(name, params)
            _sum = forceDouble(record.value('sum'))
            amount = forceDouble(record.value('invoiceAmount'))
            uet = forceDouble(record.value('uet'))
            params['total'][name]['totalUet'] += uet
            params['total'][name]['totalSum'] += _sum
            params['total'][name]['clients'].add(clientId)
            params['total'][name]['hospClients' if isHospital else 'ambClients'
                ].add(clientId)

            if uet:
                params['total'][name]['uetSum'] += _sum

            if isHospital:
                params['total'][name]['mesAmount'] += amount
                params['total'][name]['mesSum'] += _sum
                params['total'][name]['bedDays'] += forceDouble(
                    record.value('invoiceHospitalAmount'))
            else:
                params['total'][name]['amount'] += amount
                params['total'][name]['sum'] += _sum


    def postExport(self):
        self.writeInvoice(self._processParams)
        self.invoiceFile.close()
        COrder79ExportPage1.postExport(self)


    def writeInvoice(self, params):
        orgStructName = [
            u'0001 Поликлиническое отделение для взрослых (Поликлиническое отделение для взрослых)',
            u'0241 Скорая помощь (Скорая помощь)',
            u'0731 Приемное отделение (Хирургические услуги)',
            u'0351 Стоматологические услуги для взрослых (Стоматологические для взрослых)',
            u'1001 Поликлиническое отделение для детей (Поликлиническое отделение для детей)'
        ]

        fmtStr = (u'{invoiceCode: <6.6}│{invoiceName: <60.60}│{level: <3.3}│'
            u'{price: <11.11}│{invoiceAmount:9.2f}│{sum:11.2f}\n')
        splitter = (u'──────┼───────────────────────────────────────────────'
            u'─────────────┼───┼───────────┼─────────┼───────────\n')

        settleDate = params['settleDate']
        output = self.invoiceStream

        tableHeader = u"""\nОтделение: {orgStructName}
Количество пролеченных пациентов {totalClientCount:10d} на сумму {totalSum:11.2f}
             из них амбулаторных {ambClients:10d} на сумму {ambSum:11.2f}
                    стационарных {hospClients:10d} на сумму {hospSum:11.2f}
                                      Средства ФФОМС        0.00
                                               Итого {total:11.2f}
Выполнено услуг (УЕТ)       {uetCount:15.2f} на сумму {uetSum:11.2f}
Выполнено услуг (операций)  {servCount:15.2f} на сумму {servSum:11.2f}
Выполнено МЭСов             {mesCount:15.2f} на сумму {mesSum:11.2f}
Всего реализовано койко-дней {bedDays:15.2f} на сумму {bedDaysSum:11.2f}
──────┬────────────────────────────────────────────────────────────┬───┬───────────┬─────────┬───────────
 Код  │                        Наименование                        │Тип│ Стоимость │ Кол-во  │   Итого
услуги│                                                            │МЭС│ стандарта │         │
(МЭС) │                                                            │   │           │         │
(К/Д) │                                                            │   │           │         │
(П/П) │                                                            │   │           │         │
"""

        output << u"""
Медицинское учреждение ГБУЗ "Островская мб"

СЧЕТ-ФАКТУРА № {invoiceNumber} от {accountDate}
за оказанные медицинские услуги по стандартам оказания мед.помощи в рамках региональной программы модернизации здравоохранения
пациентам своей территории
за {settleDate}
--------------------------------------------------
Основная информация\n1. РАБОТА ПРОФИЛЬНЫХ ОТДЕЛЕНИЙ\n""".format(
        accountDate=params['accDate'].toString('dd.MM.yyyy'),
        settleDate=settleDate.toString('MMMM yyyy'),
        invoiceNumber=params['NSCHET'])

        for i in range(1, 6):
            if not params.get('invoice', {}).has_key(i):
                continue

            invoice = params['invoice'][i]
            info = params['ambInfo'][i]
            clientCount = len(info.get('clientCount', set()))
            totalClientCount = len(
                info.get('hospClientCount', set())) + clientCount
            totalSum = info['totalSum'] if clientCount else 0
            _sum = params['emergency'] if clientCount else 0
            totalAmount = info['totalAmount']

            if i == 1:
                totalSum = params['ambulance']
            if i == 2:
                totalSum = params['emergency']
            if i == 5:
                totalSum = 0

            output << tableHeader.format(orgStructName=orgStructName[i-1],
                totalClientCount=totalClientCount, ambClients=clientCount,
                totalSum=totalSum, ambSum=totalSum, total=totalSum,
                servCount=totalAmount, servSum=totalSum,
                mesCount=0, mesSum=0, bedDays=0, bedDaysSum=0,
                hospClients=0, hospSum=0, uetCount=info['totalUet'],
                uetSum=info['uetSum'])

            output << splitter

            for code in sorted(invoice):
                (name, amount, _sum, price, _) = invoice[code]

                if i in (3, 4):
                    priceStr = '%11.2f' % price
                else:
                    priceStr = ''
                    price = 0

                output << fmtStr.format(invoiceCode=code, invoiceName=name,
                    invoiceAmount=amount, sum=price*amount, price=priceStr, level='')

            if i == 1: #подушевой только для таблиц 1 и 2
                output << fmtStr.format(invoiceCode='201001',
                    invoiceName=u'Подушевой норматив по амбулат.полик.пом.',
                    invoiceAmount=1, sum=params['ambulance'], level='',
                    price=('%11.2f' % params['ambulance']))
                totalSum = params['ambulance']
            elif i == 2:
                output << fmtStr.format(invoiceCode='201002',
                    invoiceName=u'Подушевой норматив по скорой помощи',
                    invoiceAmount=1, sum=params['emergency'], level='',
                    price=('%11.2f' % params['emergency']))
                totalSum = params['emergency']

            output << splitter
            output << fmtStr.format(invoiceCode=u'ИТОГО', invoiceName='',
                invoiceAmount=totalAmount, price='', level='',
                sum=totalSum)

        # СФ для ДС и Стационара
        if params.has_key('hospInvoice'):
            for code, services in params['hospInvoice'].iteritems():
                info = params['hospInvoiceInfo'][code]
                output << tableHeader.format(
                    orgStructName=('%s %s' % (code,
                        params['orgStructMap'].get(code, ''))),
                        totalClientCount=len(info['clients']),
                        totalSum=info['totalSum'], ambClients=0, ambSum=0,
                        hospClients=len(info['clients']), hospSum=info['totalSum'] ,
                        total=info['totalSum'], servCount=0, servSum=0,
                        mesCount=info['amount'], mesSum=info['totalSum'],
                        bedDays=info['bedDays'], bedDaysSum=info['totalSum'],
                        uetCount=info['totalUet'], uetSum=info['uetSum']
                    )
                output << splitter
                totalAmount = 0

                for key, amount in services.iteritems():
                    (serviceCode, level, _sum) = key
                    name = ('C' if serviceCode and serviceCode[0] == u'C' else
                        params['serviceMap'].get(serviceCode, ''))
                    output << fmtStr.format(invoiceCode=serviceCode,
                        invoiceName=name,
                        invoiceAmount=amount, price=('%11.2f' % _sum),
                        sum=(_sum*amount),
                        level=('%3d' % self.mapMesLevelToInvoice.get(level, 1)))
                    totalAmount += amount

                output << splitter
                output << fmtStr.format(invoiceCode=u'ИТОГО',invoiceName='',
                    invoiceAmount=totalAmount, price='', sum=info['totalSum'],
                        level='')
        # Сводная по отделениям
            resultTableHeader = u"""\n2. СВОДНАЯ СПРАВКА ПО ПРОФИЛЬНЫМ ОТДЕЛЕНИЯМ
Всего обратившихся пациентов {totalClientCount:10d}
+-------------------+------+------+------+---------+-----------+---------+------+-----------+------+-----------+
|Наименование       | Пролечено пациентов|       Выполнено услуг         |    Реализовано МЭСов    | Стоимость |
|отделения          |                    |                               |                         |           |
|                   +------+------+------+---------+-----------+---------+------+-----------+------+           |
|                   |Всего | Амб. | Стац.| Кол-во  | Стоимость |   УЕТ   |Кол-во| Стоимость | К/д  |           |
"""
            splitter = (u'+-------------------+------+------+------+---------'
             u'+-----------+---------+------+-----------+------+-----------+\n')
            output << resultTableHeader.format(
                totalClientCount=len(params['clients']))
            fmtStr = (u'|{name: <19.19}|{clients:6d}|{ambClients:6d}|'
                u'{hospClients:6d}|{amount:9.2f}|{sum:11.2f}|{uet:9.2f}|'
                u'{mesAmount:6.0f}|{mesSum:11.2f}|{bedDays:6.0f}|{totalSum:11.2f}|\n')

            totalAmount = 0
            totalClients = 0
            totalDays = 0
            totalSum = 0
            totalUet = 0

            if params['ambInfo'].has_key(1) and params['ambInfo'].has_key(2):
                params['ambInfo'][1]['totalSum'] = params['ambulance']
                params['ambInfo'][1]['totalAmount'] += params['ambInfo'][5][
                    'totalAmount']
                params['ambInfo'][1]['clientCount'].update(params['ambInfo'][5][
                    'clientCount'])

            if params['ambInfo'].has_key(2): #СМП
                params['ambInfo'][2]['totalSum'] = params['emergency']

            if params['total'].has_key(u'СМП'):
                params['total'][u'СМП']['sum'] = params['emergency']
                params['total'][u'СМП']['totalSum'] = params['emergency']


            if params['total'].has_key(u'Поликлиника'):
                params['total'][u'Поликлиника']['sum'] = params['ambulance']
                params['total'][u'Поликлиника']['totalSum'] = params['ambulance']

            for i in range(1, 5):
                if not params['invoice'].has_key(i):
                    continue

                info = params['ambInfo'][i]
                ambClients = len(info.get('clientCount', set()))
                hospClients = len(
                    info.get('hospClientCount', set()))

                output << splitter
                output << fmtStr.format(
                    name=orgStructName[i-1][5:],
                    clients=(ambClients + hospClients),
                    ambClients=ambClients, hospClients=hospClients,
                    amount=info['totalAmount'], sum=info['totalSum'], uet=info['totalUet'],
                    mesAmount=0,  mesSum=0, bedDays=0, totalSum=info['totalSum'])

                totalSum += info['totalSum']
                totalClients += ambClients + hospClients
                totalAmount += info['totalAmount']


            for code, services in params['hospInvoice'].iteritems():
                info = params['hospInvoiceInfo'][code]
                output << splitter
                clients = len(info['clients'])
                output << fmtStr.format(
                    name=params['orgStructMap'].get(code, ''),
                    clients=clients, ambClients=0, hospClients=clients,
                    amount=0, sum=0, uet=info['totalUet'],
                    mesAmount=info['amount'], mesSum=info['totalSum'],
                    bedDays=info['bedDays'], totalSum=info['totalSum'])
                totalSum += info['totalSum']
                totalClients += clients
                totalAmount += info['amount']
                totalDays += info['bedDays']
                totalUet += info['totalUet']

            output << splitter
            output << fmtStr.format(
                name=u'Итого',
                clients=totalClients, ambClients=0, hospClients=totalClients,
                amount=0, sum=0, uet=totalUet,
                mesAmount=totalAmount, mesSum=totalSum,
                bedDays=totalDays, totalSum=totalSum)

            output << splitter
            output << (u'|В том числе:       |      |      |      |         '
             u'|           |         |      |           |      |           |\n')

            for name, val in params['total'].iteritems():
                for key in ('clients', 'ambClients', 'hospClients'):
                    val[key] = len(val[key])

                output << fmtStr.format(name=name, **val)

            output << splitter
            output << u"""\nВСЕГО ПРЕДСТАВЛЕНО К ОПЛАТЕ: {totalSum:.2f}
      ({words})\n
Руководитель мед.учреждения ____________________ Богачева И М\n
Гл.бухгалтер мед.учреждения ____________________ Дербилова Р Л
""".format(totalSum=params['totalSum'], words=amountToWords(params['totalSum']))




# ******************************************************************************

class CExportPage2(COrder79ExportPage2):
    u"""Первая страница мастера экспорта"""
    def __init__(self, parent, prefix):
        COrder79ExportPage2.__init__(self, parent, prefix)


    def saveExportResults(self):
        fileList = (self._parent.getFullXmlFileName(),
                    self._parent.getPersonalDataFullXmlFileName(),
                    self._parent.getFullTxtFileName())
        zipFileName = self._parent.getFullZipFileName()

        return (compressFileInZip(fileList, zipFileName) and
                        self.moveFiles([zipFileName]))

# ******************************************************************************

class XmlStreamWriter(COrder79XmlStreamWriter, CExportHelperMixin):
    u"""Осуществляет запись данных экспорта в XML"""

    clientFields = ('ID_PAC', 'VPOLIS', 'SPOLIS', 'NPOLIS', 'SMO',
                    'SMO_OGRN', 'SMO_OK', 'SMO_NAM', 'INV', 'MSE', 'NOVOR')

    completeEventDateFields1 = ('DATE_Z_1', 'DATE_Z_2')
    completeEventDateFields2 = ('NPR_DATE',  )
    completeEventDateFields = completeEventDateFields1 + completeEventDateFields2
    completeEventFields1 = (('IDCASE', 'USL_OK', 'VIDPOM', 'FOR_POM', 'NPR_MO')
                            + completeEventDateFields2 +
                            ('LPU',  )
                           + completeEventDateFields1 +
                           ('KD_Z', 'VNOV_M', 'RSLT', 'ISHOD', 'OS_SLUCH',
                            'VB_P'))
    completeEventFields2 = ('IDSP', 'SUMV')
    completeEventFields = completeEventFields1 + completeEventFields2

    eventDateFields = ('DATE_1', 'DATE_2')

    eventFields = ('SL_ID', 'LPU_1', 'PODR', 'PROFIL', 'PROFIL_K', 'DET',
                   'P_CEL', 'NHISTORY', 'P_PER') +  eventDateFields + ('KD',
                   'DS0', 'DS1', 'DS2', 'DS3', 'DN', 'CODE_MES1', 'KSG_KPG',  'REAB', 'PRVS',
                   'VERS_SPEC', 'IDDOKT', 'ED_COL', 'TARIF', 'SUM_M')

    ksgSubGroup = {
        'SL_KOEF' : {'fields': ('IDSL', 'Z_SL')}
    }

    eventSubGroup = {
        'KSG_KPG': { 'fields': ('N_KSG', 'VER_KSG', 'KSG_PG',  'N_KPG',  'KOEF_Z',  'KOEF_UP',
                                                 'BZTSZ', 'KOEF_D',  'KOEF_U', 'DKK1', 'DKK2',  'SL_K',  'IT_SL',
                                                 'SL_KOEF'),
                               'subGroup': ksgSubGroup }
    }

    serviceDateFields = ('DATE_IN', 'DATE_OUT')

    serviceFields = (('IDSERV', 'LPU', 'LPU_1', 'PODR', 'PROFIL', 'VID_VME',
            'DET') + serviceDateFields + ('DS', 'CODE_USL', 'KOL_USL', 'TARIF',
            'SUMV_USL', 'PRVS', 'CODE_MD', 'NPL', 'COMENTU'))

    mapEventOrderToForPom = {1: '3', 2: '1', 3: '1', 4: '3', 5: '3', 6: '2'}
    mapEventOrderToPPer = {5: '4', 7: '3'}

    def __init__(self, parent):
        COrder79XmlStreamWriter.__init__(self, parent)
        CExportHelperMixin.__init__(self)

        self._coefficientCache = {}
        self._reasonCache = {}
        self._dispanserCache = {}


    def writeHeader(self, params):
        u"""Запись заголовка xml файла"""
        settleDate = params['settleDate']
        population = self._parent.contractAttributeByType(u'населениесмп')
        population = forceDouble(population) if population else 0.0
        emergency = self._parent.contractAttributeByType(u'смп')
        emergency = forceDouble(emergency) if emergency else 0.0

        payerMiacCode = forceString(self._parent.db.translate(
                'Organisation', 'id', params['payerId'], 'miacCode'))
        payerIsInsurer = forceBool(self._parent.db.translate(
                'Organisation', 'id', params['payerId'], 'isInsurer'))
        financeId = forceRef(self._parent.db.translate(
                'Contract', 'id', params['contractId'], 'finance_id'))
        financeCode = forceInt(self._parent.db.translate(
                'rbFinance', 'id', financeId, 'code'))
        self.writeStartElement('ZL_LIST')

        self.writeStartElement('ZGLV')
        self.writeTextElement('VERSION', '3.0')
        self.writeTextElement('DATA', settleDate.toString(Qt.ISODate))
        self.writeTextElement('FILENAME', params['shortFileName'][:-4])
        self.writeTextElement('SD_Z', forceString(
            self._parent.getCompleteEventCount()))
        self.writeEndElement() # ZGLV

        self.writeStartElement('SCHET')
        self.writeTextElement('CODE', '%d' % params['accId'])
        self.writeTextElement('CODE_MO', params['lpuMiacCode'])
        self.writeTextElement('YEAR', forceString(settleDate.year()))
        self.writeTextElement('MONTH', forceString(settleDate.month()))
        self.writeTextElement('NSCHET', params['NSCHET'])
        self.writeTextElement('DSCHET', settleDate.toString(Qt.ISODate))

        if payerMiacCode:
            self.writeTextElement('PLAT', payerMiacCode)

        params['totalSum'] = emergency*population + params['ambulance'] + params['other']
        self.writeTextElement('SUMMAV', forceString(params['totalSum']))

        if financeCode == 2 and payerIsInsurer:
            self.writeTextElement('SUM_POD', forceString(params['ambulance']))
            self.writeTextElement('SUM_SMP', forceString(emergency*population))

        params['emergency'] = emergency*population
        self.writeEndElement() # SCHET


    def writeRecord(self, record, params):
        clientId = forceRef(record.value('PACIENT_ID_PAC'))
        lastEventId = forceRef(record.value('lastEventId'))

        if (clientId != params.setdefault('lastClientId') or
            lastEventId != params.setdefault('lastComleteEventId')):
            if params['lastClientId']:
                self.writeEndElement() # SL
                self.writeGroup('Z_SL', self.completeEventFields2,
                                params['lastRecord'],
                                closeGroup=True, openGroup=False)
                self.writeEndElement() # ZAP

            self.writeStartElement('ZAP')
            self.writeTextElement('N_ZAP', forceString(params['recNum']))
            self.writeTextElement('PR_NOV', params['isReexposed'])
            self.writeClientInfo(record, params)

            params['lastClientId'] = clientId
            params['lastEventId'] = None
            params['lastComleteEventId'] = None
            params['recNum'] += 1

        self.writeCompleteEvent(record, params)
        self.writeEventInfo(record, params)
        self.writeService(record, params)


    def writeCompleteEvent(self, record, params):
        lastEventId = forceRef(record.value('lastEventId'))

        if lastEventId == params.get('lastComleteEventId'):
            return

        if params['lastEventId']:
            self.writeEndElement()
            self.writeGroup('Z_SL', self.completeEventFields2,
                            params['lastRecord'],
                            closeGroup=True,
                            openGroup=False)

        eventOrder = forceInt(record.value('eventOrder'))
        record.setValue('Z_SL_FOR_POM', self.mapEventOrderToForPom.get(
            eventOrder, ''))

        params['idsp'] = forceInt(record.value('Z_SL_IDSP'))

        if params['emergencyIncomplete'] and params['idsp'] == 31:
            record.setValue('Z_SL_SUMV', toVariant(0))
        else:
            record.setValue('Z_SL_SUMV', toVariant(
                params['completeEventSum'].get(lastEventId)))

        if record.isNull('Z_SL_NPR_MO'):
            record.setValue('Z_SL_NPR_MO',  toVariant(params['lpuMiacCode']))

        record.setValue('Z_SL_KD_Z', toVariant(params['completeBedDays'].get(lastEventId)))
        self.writeGroup('Z_SL', self.completeEventFields1, record,
                        closeGroup=False,
                        dateFields=self.completeEventDateFields)
        params['lastComleteEventId'] = forceRef(record.value('lastEventId'))
        params['lastEventId'] = None
        params['lastRecord'] = record


    def prepareCoefficients(self, coefficients):
        u"""Готовит блок SL_KOEF"""
        result = {}

        for _, i in coefficients.iteritems():
            for key, val in i.iteritems():
                if key == u'__all__':
                    continue

                code = self.getCoefficientRegionalCode(key)

                if code:
                    result.setdefault('SL_KOEF_IDSL', []).append(code[:4])
                    result.setdefault('SL_KOEF_Z_SL', []).append('%1.5f' % val)

        return result


    def writeEventInfo(self, record, params):
        u"""Пишет информацию о событии"""
        eventId = forceRef(record.value('eventId'))

        if eventId == params.setdefault('lastEventId'):
            return

        patrName = forceString(record.value('PERS_OT'))
        params['eventProfileRegionalCode'] = forceInt(record.value(
            'eventProfileRegionalCode'))
        eventOrder = forceInt(record.value('eventOrder'))
        medicalAidTypeCode = params['medicalAidType'].get(eventId, 0)

        local_params = {
            'SL_OS_SLUCH': '2' if (not patrName or
                                      patrName.upper() == u'НЕТ') else '',
            'SL_VERS_SPEC': 'V021',
            'SL_KD': params['bedDays'].get(eventId)
        }
        params['isHospital'] = medicalAidTypeCode in (1, 2, 3, 7)

        if record.isNull('SL_P_CEL'):
            eventTypeId = forceRef(record.value('eventTypeId'))
            record.setValue('SL_P_CEL', toVariant(self.getReason(eventTypeId)))

        local_params['SL_DN'] = self.getDispanserCode(forceRef(
            record.value('dispanserId')))

        if params['isHospital']:
            local_params['SL_EXTR'] = (forceString(
                eventOrder) if eventOrder in (1, 2)  else '')
            local_params['SL_P_PER'] = self.mapEventOrderToPPer.get(
                eventOrder)

            hospActionId = forceRef(record.value('hospActionId'))
            if hospActionId:
                action = CAction.getActionById(hospActionId)
                bedCode = self.getHospitalBedProfileTfomsCode(
                    self.getHospitalBedProfileId(forceRef(action[u'койка'])))
                local_params['SL_PROFIL_K'] = bedCode

            if not local_params['SL_P_PER']:
                delivered = forceString(record.value('delivered')) == u'СМП'
                local_params['SL_P_PER'] = '2' if delivered else '1'
        elif params['eventProfileRegionalCode'] == 6: #стоматология
            record.setValue('SL_TARIF', toVariant(
                self._parent.contractAttributeByType(u'ует')))

        local_params.update(params)
        params['USL_IDSERV'] = 0

        if params['lastEventId']:
            self.writeEndElement() # SL

        subGroup = dict(self.eventSubGroup)
        if params['isHospital']:
            local_params['KSG_KPG_VER_KSG'] = forceString(params['settleDate'].year())
            local_params['KSG_KPG_KSG_PG'] = '0'
            local_params['KSG_KPG_KOEF_D'] = '1'
            local_params['KSG_KPG_BZTSZ'] = self._parent.contractAttributeByType(
                u'КС' if medicalAidTypeCode in (1, 2, 3) else u'ДС')
            local_params['KSG_KPG_KOEF_UP']  = 1
            local_params['KSG_KPG_SL_K'] = 0

            usedCoefficients = forceString(record.value('usedCoefficients'))

            if usedCoefficients:
                coefficients = json.loads(usedCoefficients)

                for _, i in coefficients.iteritems():
                    if i.has_key('DS_uk'):
                        local_params['KSG_KPG_KOEF_UP'] = i.get('DS_uk')

                    if i.has_key('__all__'):
                        local_params['KSG_KPG_SL_K'] = (
                            1 if i['__all__'] > 1 else 0)

                        if i['__all__'] > 1:
                            local_params['KSG_KPG_IT_SL'] = i['__all__']

                if local_params.has_key('KSG_KPG_IT_SL'):
                    local_params.update(self.prepareCoefficients(
                        coefficients))
            else:
                subGroup['KSG_KPG']['subGroup'] = None

            coef = forceDouble(local_params['KSG_KPG_BZTSZ'])

            if coef != 0:
                record.setValue('KSG_KPG_KOEF_U', u'%.2f' % (forceDouble(
                    record.value('KSG_KPG_KOEF_U')) / coef / local_params['KSG_KPG_KOEF_UP']))
            else:
                record.setNull('KSG_KPG_KOEF_U')

        _record = ExtendedRecord(record, local_params, DEBUG)
        self.writeGroup('SL', self.eventFields, _record,
            subGroup if params['isHospital'] else None,
            closeGroup=False, dateFields=self.eventDateFields)
        params['lastEventId'] = eventId


    def exportVisits(self, eventId, params):
        u"""Экспортирует данные для визитов с 0 стоимостью,
            при тарификации по посещениям"""
        stmt = u"""SELECT
            PersonOrganisation.miacCode AS USL_LPU,
            ExecPersonOrgStructure.infisCode AS USL_LPU_1,
            PersonOrgStructure.tfomsCode AS USL_PODR,
            IF(rbService_Profile.medicalAidProfile_id IS NOT NULL,
                ServiceProfileMedicalAidProfile.code,
                    ServiceMedicalAidProfile.code) AS USL_PROFIL,
            IF(SUBSTR(rbService.code, 1, 3) = 'A16', rbService.code, '') AS USL_VID_VME,
            IF(age(Client.birthDate,Event.execDate) < 18, 1, 0) AS USL_DET,
            Visit.date AS USL_DATE_IN,
            Visit.date AS USL_DATE_OUT,
            Diagnosis.MKB AS USL_DS,
            IF(rbService.infis IS NULL OR rbService.infis = '',
                rbService.code, rbService.infis) AS USL_CODE_USL,
            1 AS USL_KOL_USL,
            IF('{unit}' IN ('9', '24', '31'),0,'') AS USL_TARIF,
            0 AS USL_SUMV_USL,
            rbSpeciality.federalCode AS USL_PRVS,
            Person.federalCode AS USL_IDDOKT,
            Person.federalCode AS USL_CODE_MD
        FROM Visit
        LEFT JOIN Event ON Event.id = Visit.event_id
        LEFT JOIN Diagnostic ON ( Diagnostic.event_id = Event.id
                                  AND Diagnostic.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code in ('1', '2'))
                                  AND Diagnostic.deleted = 0 )
        LEFT JOIN Diagnosis ON Diagnosis.id = Diagnostic.diagnosis_id AND Diagnosis.deleted = 0
        LEFT JOIN Person ON Person.id = Visit.person_id
        LEFT JOIN Organisation AS PersonOrganisation ON PersonOrganisation.id = Person.org_id
        LEFT JOIN OrgStructure AS PersonOrgStructure ON PersonOrgStructure.id = Person.orgStructure_id
        LEFT JOIN Person AS ExecPerson ON ExecPerson.id = Event.execPerson_id
        LEFT JOIN OrgStructure AS ExecPersonOrgStructure ON ExecPerson.orgStructure_id = ExecPersonOrgStructure.id
        LEFT JOIN rbSpeciality ON Person.speciality_id = rbSpeciality.id
        LEFT JOIN rbService ON Visit.service_id = rbService.id
        LEFT JOIN Client ON Client.id = Event.client_id
        LEFT JOIN rbService_Profile ON rbService_Profile.id = (SELECT MAX(id) FROM rbService_Profile rs
            WHERE rs.master_id = rbService.id AND rs.speciality_id = Person.speciality_id)
        LEFT JOIN rbMedicalAidProfile AS ServiceProfileMedicalAidProfile ON ServiceProfileMedicalAidProfile.id = rbService_Profile.medicalAidProfile_id
        LEFT JOIN rbMedicalAidProfile AS ServiceMedicalAidProfile ON ServiceMedicalAidProfile.id = rbService.medicalAidProfile_id
        LEFT JOIN EventType ON EventType.id = Event.eventType_id
        LEFT JOIN rbEventProfile ON rbEventProfile.id = EventType.eventProfile_id
        WHERE Visit.event_id = {eventId}
            AND Visit.deleted = '0'
            AND rbEventProfile.regionalCode != '6' -- исключаем стоматологию
        ORDER BY Visit.date""".format(
            eventId=eventId, unit=params['medicalAidUnit'])

        query = self._parent.db.query(stmt)
        result = False

        if query.size() > 1:
            curPos = 1
            querySize = query.size()

            while query.next():
                record = query.record()
                params['isLastVisit'] = curPos == querySize

                self.writeService(record, params)
                curPos += 1

            result = True

        return result


    def writeService(self, record, params):
        u"""Выгружает информацию об услугах"""

        eventId = forceRef(record.value('eventId'))
        isVisitExported = False

        idsp = params['idsp']
        if params['emergencyIncomplete'] and idsp == 31:
            record.setValue('USL_SUMV_USL', toVariant(0))

        if (eventId and
             eventId not in params.setdefault('exportedEvents', set())):
            params['tarif'] = record.value('SL_TARIF')
            params['sum'] = record.value('Z_SL_SUMV')
            params['amount'] = record.value('SL_ED_COL')
            params['medicalAidUnit'] = forceString(record.value('Z_SL_IDSP'))
            isVisitExported = self.exportVisits(eventId, params)
            params['exportedEvents'].add(eventId)

        if not isVisitExported and (not params['isHospital'] or params.get('force')):
            params['USL_IDSERV'] += 1

            if params.get('isLastVisit'):
                record.setValue('USL_TARIF', params['tarif'])
                record.setValue('USL_SUMV_USL', params['sum'])

                if params['isHospital']:
                    record.setValue('USL_KOL_USL', params['amount'])

                params['isLastVisit'] = False

            if (params['eventProfileRegionalCode'] in (1, 3)
                    and not params.get('isMesServiceExported')):
                usedCoefficients = forceString(record.value('usedCoefficients'))
                if usedCoefficients:
                    coefficientList = json.loads(usedCoefficients)
                    for _, list in coefficientList.iteritems():
                        for _, value in list.iteritems():
                            if value > 1.0:
                                params['USL_COMENTU'] = value
                                break

                        if params.has_key('USL_COMENTU'):
                            break

            _record = ExtendedRecord(record, params, DEBUG)
            self.writeGroup('USL', self.serviceFields, _record,
                    dateFields=self.serviceDateFields)

            if params.has_key('USL_COMENTU'):
                params.pop('USL_COMENTU')

        # выгрузка номенклатурных услуг
        mesService = forceString(record.value('mesServiceInfis'))

        if (params['eventProfileRegionalCode'] in (1, 3)
                and  mesService
                and  not params.setdefault('isMesServiceExported', False)):
            params['isMesServiceExported'] = True
            record.setValue('USL_CODE_USL', toVariant(mesService))
            record.setValue('USL_SUMV_USL', toVariant(0))
            record.setValue('USL_TARIF', toVariant(0))
            record.setValue('USL_DATE_IN', record.value('mesActionBegDate'))
            record.setValue('USL_DATE_OUT', record.value('mesActionEndDate'))
            record.setValue('USL_KOL_USL', record.value('amount'))
            params['force'] = True

            self.writeService(record, params)
            params['isMesServiceExported'] = False
            params['force'] = False


    def writeClientInfo(self, record, params):
        u"""Пишет информацию о пациенте"""

        _record = ExtendedRecord(record, params, DEBUG)
        self.writeGroup('PACIENT', self.clientFields, _record)


    def writeFooter(self, params):
        u"""Закрывает теги в конце файла"""
        if params.get('lastEventId'):
            self.writeEndElement() # SL
            self.writeGroup('Z_SL', self.completeEventFields2,
                            params['lastRecord'],
                            closeGroup=True,
                            openGroup=False)
            self.writeEndElement() # ZAP

        self.writeEndElement() # ZL_LIST


    def getCoefficientRegionalCode(self, code):
        u'Возвращает региональный код коэффициента по его имени'
        result = self._coefficientCache.get(code, -1)

        if result == -1:
            result = forceString(self._parent.db.translate(
                     'rbContractCoefficientType', 'code', code, 'regionalCode'))
            self._coefficientCache[code] = result

        return result


    def getReason(self, eventTypeId):
        result = self._reasonCache.get(eventTypeId, -1)

        if result == -1:
            stmt = """SELECT value
            FROM EventType_Identification
            WHERE EventType_Identification.master_id = {eventTypeId}
                AND EventType_Identification.system_id = (
                SELECT MAX(id) FROM rbAccountingSystem
                WHERE rbAccountingSystem.code = 'tfomsPCEL'
            )""".format(eventTypeId=eventTypeId)

            query = self._parent.db.query(stmt)
            result = ''

            if query.first():
                record = query.record()
                result = forceString(record.value(0))
                self._reasonCache[eventTypeId] = result

        return result


    def getDispanserCode(self, dispanserId):
        result = self._dispanserCache.get(dispanserId, -1)

        if result == -1:
            result = ''

            if dispanserId:
                stmt = u"""SELECT
                    CASE
                        WHEN rbDispanser.observed = 1 AND rbDispanser.name LIKE '%%взят%%' THEN 2
                        WHEN rbDispanser.observed = 1 AND rbDispanser.name NOT LIKE '%%взят%%' THEN 1
                        WHEN rbDispanser.observed = 0 AND rbDispanser.name LIKE '%%выздор%%' THEN 4
                        WHEN rbDispanser.observed = 0 AND rbDispanser.name NOT LIKE '%%выздор%%' THEN 6
                        ELSE NULL
                    END AS dispanserObserved
                FROM rbDispanser
                WHERE rbDispanser.id = {0}""".format(dispanserId)
                query = self._parent.db.query(stmt)

                if query and query.first():
                    record = query.record()
                    result = forceString(record.value(0))
                    self._dispanserCache[dispanserId] = result

        return result


# ******************************************************************************

class PersonalDataWriter(COrder79PersonalDataWriter):
    u"""Осуществляет запись данных экспорта в XML"""

    clientFields = (('ID_PAC', 'FAM', 'IM', 'OT', 'W',) +
                    COrder79PersonalDataWriter.clientDateFields + (
                    'DOST', 'TEL', 'FAM_P', 'IM_P', 'OT_P', 'W_P', 'MR',
                    'DOCTYPE', 'DOCSER', 'DOCNUM', 'SNILS', 'OKATOG', 'OKATOP'))

    def __init__(self, parent):
        COrder79PersonalDataWriter.__init__(self, parent)


    def writeClientInfo(self, record, params):
        u"""Пишет информацию о пациенте"""

        if record.contains('PERS_TEL'):
            phoneNumber = forceString(record.value('PERS_TEL'))
            record.setValue('PERS_TEL', toVariant(
                    filter(lambda ch: ch not in '+()- ', phoneNumber)[-10:]))

        COrder79PersonalDataWriter.writeClientInfo(self, record, params)

# ******************************************************************************

if __name__ == '__main__':
    #pylint: disable=wrong-import-position,ungrouped-imports
    from Exchange.Utils import testAccountExport
    #pylint: enable=wrong-import-position,ungrouped-imports
    testAccountExport(exportR60Ambulance, 1149)
