# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
##
## Форма 043: стоматология и другие зубные дела
##
#############################################################################

import sip

from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, QDate, QDateTime, QModelIndex, QObject, QTime, QVariant, SIGNAL, pyqtSignature

from library.Attach.AttachAction        import getAttachAction
from library.crbcombobox                import CRBComboBox
from library.ICDInDocTableCol           import CICDExInDocTableCol
from library.ICDMorphologyInDocTableCol import CMKBMorphologyCol
from library.InDocTable                 import CMKBListInDocTableModel, CBoolInDocTableCol, CDateTimeForEventInDocTableCol, CInDocTableCol, CRBInDocTableCol
from library.interchange                import getDateEditValue, getDatetimeEditValue, getRBComboBoxValue, setDateEditValue, setDatetimeEditValue, setRBComboBoxValue
from library.PrintTemplates             import customizePrintButton, getPrintButton
from library.TNMS.TNMSComboBox          import CTNMSCol
from library.Utils                      import copyFields, forceBool, forceDate, forceDateTime, forceDouble, forceInt, forceRef, forceString, formatNum, toVariant, variantEq, getDentitionActionTypeId

from Events.Action                      import initActionProperties
from Events.ActionInfo                  import CActionInfoProxyList
from Events.ActionsSummaryModel         import CFxxxActionsSummaryModel
from Events.DiagnosisType               import CDiagnosisTypeCol
from Events.EventEditDialog             import CEventEditDialog, CDiseaseCharacter, CDiseaseStage, CDiseasePhases
from Events.EventInfo                   import CDiagnosticInfoProxyList, CVisitInfoProxyList
from Events.EventVisitsModel            import CDentitionVisitsModel
from Events.TeethEventInfo              import CTeethEventInfo
from Events.Utils                       import checkDiagnosis, checkIsHandleDiagnosisIsChecked, getAvailableCharacterIdByMKB, getDiagnosisId2, getDiagnosticResultId, getEventAddVisit, getEventDurationRange, getEventIsPrimary, getEventMesRequired, getEventResultId, getEventSetPerson, getEventShowTime, getEventShowVisitTime, hasEventVisitAssistant, setAskedClassValueForDiagnosisManualSwitch, CTableSummaryActionsMenuMixin, getEventCSGRequired
from F043.DentitionTable                import CClientDentitionHistoryModel, CDentitionModel, CParodentiumModel
from F043.PreF043Dialog                 import CPreF043Dialog, CPreF043DagnosticAndActionPresets
from Orgs.PersonComboBoxEx              import CPersonFindInDocTableCol
from RefBooks.Tables                    import rbDispanser, rbHealthGroup, rbTraumaType
from Users.Rights                       import urAccessF043planner, urAdmin, urEditEndDateEvent, urRegTabWriteRegistry, urRegTabReadRegistry

from F043.Ui_F043                       import Ui_F043Dialog


INSPECTION_DENTITION_TAB_INDEX = 0
INSPECTION_DENTITION_ADDITIONAL_TAB_INDEX = 1
RESULT_DENTITION_TAB_INDEX = 2


class CF043Dialog(CEventEditDialog, Ui_F043Dialog, CTableSummaryActionsMenuMixin):
    defaultEventResultId = None
    defaultDiagnosticResultId = None

    def __init__(self, parent):
        CEventEditDialog.__init__(self, parent)
        self.dentitionActionTypeId = forceRef(
                                    QtGui.qApp.db.translate('ActionType', 'flatCode', 'dentitionInspection', 'id'))
        self.parodentActionTypeId = forceRef(
                                    QtGui.qApp.db.translate('ActionType', 'flatCode', 'parodentInsp', 'id'))
        self.mapSpecialityIdToDiagFilter = {}
        self.addModels('Visits', CDentitionVisitsModel(self))
        self.addModels('Diagnostics', CF043FinalDiagnosticsModel(self))
        self.addModels('ActionsSummary', CFxxxActionsSummaryModel(self, True))
        self.addModels('ClientDentitionHistory', CClientDentitionHistoryModel(self))
        self.addModels('Dentition', CDentitionModel(self, self.modelClientDentitionHistory))
        self.addModels('Parodentium', CParodentiumModel(self, self.modelClientDentitionHistory))
        self.modelDentition.setIsExistsDentitionAction(bool(self.dentitionActionTypeId))
        self.modelParodentium.setIsExistsDentitionAction(bool(self.parodentActionTypeId))

        self.addObject('actEditClient', QtGui.QAction(u'Открыть регистрационную карточку', self))
        self.addObject('actShowAttachedToClientFiles', getAttachAction('Client_FileAttach',  self))
        self.setupDiagnosticsMenu()
        self.addObject('btnPrint', getPrintButton(self, ''))
        self.addObject('btnTemperatureList', QtGui.QPushButton(u'Температурный лист', self))
        self.addObject('btnPrintMedicalDiagnosis', getPrintButton(self, '', u'Врачебный диагноз'))
        self.createSaveAndCreateAccountButton()

        self.setupUi(self)

        self.setupSaveAndCreateAccountButton()

        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setWindowTitleEx(u'Стоматология Ф.043')

        self.setMedicalDiagnosisContext()
        self.tabToken.setFocusProxy(self.tblDiagnostics)
        self.tabMes.setEventEditor(self)
        self.tabTempInvalidAndAegrotat.setCurrentIndex(1 if QtGui.qApp.tempInvalidDoctype() == '2' else 0)
        self.grpTempInvalid.setEventEditor(self)
        self.grpTempInvalid.setType(0, '1')
        self.grpAegrotat.setEventEditor(self)
        self.grpAegrotat.setType(0, '2')
        self.grpDisability.setEventEditor(self)
        self.grpDisability.setType(1)
        self.grpVitalRestriction.setEventEditor(self)
        self.grpVitalRestriction.setType(2)

        self.tabStatus.setEventEditor(self)
        self.tabMedicalDiagnosis.setEventEditor(self)
        self.tabDiagnostic.setEventEditor(self)
        self.tabCure.setEventEditor(self)
        self.tabMisc.setEventEditor(self)
        self.tabCash.setEventEditor(self)
        self.tabStatus.setActionTypeClass(0)
        self.tabDiagnostic.setActionTypeClass(1)
        self.tabCure.setActionTypeClass(2)
        self.tabMisc.setActionTypeClass(3)

        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)
        self.buttonBox.addButton(self.btnTemperatureList, QtGui.QDialogButtonBox.ActionRole)
        self.buttonBox.addButton(self.btnPrintMedicalDiagnosis, QtGui.QDialogButtonBox.ActionRole)
        self.setupActionSummarySlots()

        self.tblVisits.setModel(self.modelVisits)
        self.tblVisits.setSelectionModel(self.selectionModelVisits)
        self.tblDiagnostics.setModel(self.modelDiagnostics)
        self.setModels(self.tblClientDentitionHistory,
                       self.modelClientDentitionHistory,
                       self.selectionModelClientDentitionHistory)
        self.setModels(self.tblDentition,
                       self.modelDentition,
                       self.selectionModelDentition)
        self.setModels(self.tblParodentium,
                       self.modelParodentium,
                       self.selectionModelParodentium)

        self.tblDentition.addInspectionActions()
        self.tblParodentium.addParodentActions()
        self.tblActions.setModel(self.modelActionsSummary)
        self.modelActionsSummary.addModel(self.tabStatus.modelAPActions)
        self.modelActionsSummary.addModel(self.tabDiagnostic.modelAPActions)
        self.modelActionsSummary.addModel(self.tabCure.modelAPActions)
        self.modelActionsSummary.addModel(self.tabMisc.modelAPActions)
        self.tabCash.addActionModel(self.tabStatus.modelAPActions)
        self.tabCash.addActionModel(self.tabDiagnostic.modelAPActions)
        self.tabCash.addActionModel(self.tabCure.modelAPActions)
        self.tabCash.addActionModel(self.tabMisc.modelAPActions)
# popup menus
        self.txtClientInfoBrowser.actions.append(self.actEditClient)
        self.txtClientInfoBrowser.actions.append(self.actShowAttachedToClientFiles)
        self.actEditClient.setEnabled(QtGui.qApp.userHasAnyRight([urAdmin, urRegTabWriteRegistry, urRegTabReadRegistry]))
        self.tblVisits.setEventEditor(self)
        self.tblVisits.addPopupDelRow()
        self.setupVisitsIsExposedPopupMenu()
        self.tblDiagnostics.setPopupMenu(self.mnuDiagnostics)
        CTableSummaryActionsMenuMixin.__init__(self)

        self.setupDirtyCather()
        self.setIsDirty(False)
        self.blankMovingIdList = []
        self.isResultDentitionLoadedByInspection = False
        self.isNewDentitionActionInitialized = False
        self.clientId = None
        self.prolongateEvent = False
        self.prevEventId = None
        self.tabNotes.setEventEditor(self)
        self.edtDentitionObjectively.getThesaurus(1)
        self.edtDentitionMucosa.getThesaurus(2)
        self.tblActions.enableColsHide()
        self.tblActions.enableColsMove()

        self.postSetupUi()

        self._applyUETCountConnection()
        if hasattr(self, 'edtEndDate'):
            self.edtEndDate.setEnabled(QtGui.qApp.userHasRight(urEditEndDateEvent))
        if hasattr(self, 'edtEndTime'):
            self.edtEndTime.setEnabled(QtGui.qApp.userHasRight(urEditEndDateEvent))
        self.actionDentitionList = {}
        self.actionParodentiumList = {}
        self.btnPrintMedicalDiagnosis.setVisible(False)


    def _applyUETCountConnection(self):
        for actionsTab in self.getActionsTabsList():
            self.connect(actionsTab.modelAPActions, SIGNAL('itemsCountChanged()'), self.on_actionItemsCountChanged)
            self.connect(actionsTab, SIGNAL('actionUetChanged()'), self.on_actionUetChanged)


    def getActionsTabsList(self):
        result = []
        if hasattr(self, 'tabStatus'):
            result.append(self.tabStatus)
        if hasattr(self, 'tabDiagnostic'):
            result.append(self.tabDiagnostic)
        if hasattr(self, 'tabCure'):
            result.append(self.tabCure)
        if hasattr(self, 'tabMisc'):
            result.append(self.tabMisc)
        return result


    def on_actionUetChanged(self):
        self._recountCommonUet()


    def on_actionItemsCountChanged(self):
        self._recountCommonUet()


    def getServiceActionCode(self):
        return None


    def _recountCommonUet(self):
        result = 0.0
        actionItemsList = self.getActionsModelsItemsList()
        for (record, action) in actionItemsList:
            result += forceDouble(record.value('uet'))
        self.edtCommonUet.setValue(result)


    def destroy(self):
        self.tblVisits.setModel(None)
        self.tblDiagnostics.setModel(None)
        self.tblClientDentitionHistory.setModel(None)
        self.tblDentition.setModel(None)
        self.tblActions.setModel(None)
        self.grpTempInvalid.destroy()
        self.grpAegrotat.destroy()
        self.grpDisability.destroy()
        self.grpVitalRestriction.destroy()
        self.tabStatus.destroy()
        self.tabDiagnostic.destroy()
        self.tabCure.destroy()
        self.tabMisc.destroy()
        self.tabCash.destroy()
        self.tabMes.destroy()
        del self.modelVisits
        del self.modelDiagnostics
        del self.modelDentition
        del self.modelClientDentitionHistory
        self.tabAmbCard.destroy()


    def getModelFinalDiagnostics(self):
        return self.modelDiagnostics


#    def currentClientId(self): # for AmbCard mixin
#        return self.clientId


    def onActionChanged(self, actionsSummaryRow):
        self.addVisitByActionSummaryRow(actionsSummaryRow, checkActionPersonIdIsEventPerson=True)
        self.closeEventByAction(actionsSummaryRow)


    @pyqtSignature('int')
    def on_modelActionsSummary_currentRowMovedTo(self, row):
        self.tblActions.setCurrentIndex(self.modelActionsSummary.index(row, 0))


    @pyqtSignature('QModelIndex')
    def on_tblActions_doubleClicked(self, index):
        row = index.row()
        if 0 <= row < len(self.modelActionsSummary.itemIndex):
            page, row = self.modelActionsSummary.itemIndex[row]
            if page in [0, 1, 2, 3]:
                self.tabWidget.setCurrentIndex(page+3)
                tbl = [self.tabStatus.tblAPActions, self.tabDiagnostic.tblAPActions, self.tabCure.tblAPActions, self.tabMisc.tblAPActions][page]
                tbl.setCurrentIndex(tbl.model().index(row, 0))


    @pyqtSignature('')
    def on_btnTemperatureList_clicked(self):
        self.getTemperatureList(self.eventSetDateTime)


    def setupDiagnosticsMenu(self):
        self.addObject('mnuDiagnostics', QtGui.QMenu(self))
        self.addObject('actDiagnosticsRemove', QtGui.QAction(u'Удалить запись', self))
        self.mnuDiagnostics.addAction(self.actDiagnosticsRemove)


    def getFinalDiagnosisId(self):
        diagnostics = self.modelDiagnostics.items()
        return forceRef(diagnostics[0].value('diagnosis_id')) if diagnostics else None


    def getFinalDiagnosisMKB(self):
        diagnostics = self.modelDiagnostics.items()
        if diagnostics:
            MKB   = forceString(diagnostics[0].value('MKB'))
            MKBEx = forceString(diagnostics[0].value('MKBEx'))
            return MKB, MKBEx
        else:
            return '', ''


    def _prepare(self, contractId, clientId, eventTypeId, orgId, personId, eventSetDatetime, eventDatetime,
                 weekProfile, numDays, presetDiagnostics, presetActions, disabledActions, externalId,
                 assistantId, curatorId, movingActionTypeId = None, valueProperties = [], relegateOrgId = None,
                 relegatePersonId=None, diagnos = None, financeId = None, protocolQuoteId = None,
                 actionByNewEvent = [], eventOrder = 1, typeQueue = -1, relegateInfo=[], plannedEndDate = None):
        def getPrimary(clientId, eventTypeId, personId):
            db = QtGui.qApp.db
            tableEvent = db.table('Event')
            tablePerson = db.table('Person')
            tableP1 = tablePerson.alias('p1')
            tableP2 = tablePerson.alias('p2')
            table = tableEvent.leftJoin(tableP1, tableP1['id'].eq(tableEvent['execPerson_id']))
            table = table.leftJoin(tableP2, tableP1['speciality_id'].eq(tableP2['speciality_id']))
            cond = [tableEvent['deleted'].eq(0),
                    tableEvent['client_id'].eq(clientId),
                    tableEvent['eventType_id'].eq(eventTypeId),
                    tableP2['id'].eq(personId),
                    ]
            record = db.getRecordEx(table, tableEvent['nextEventDate'].name(), cond, order=tableEvent['execDate'].name()+' DESC')
            return not(record and not record.value('nextEventDate').isNull())

        def prepVisit(date, personId):
            sceneId = None
            if typeQueue is not None and typeQueue > -1:
                db = QtGui.qApp.db
                tableRBScene = db.table('rbScene')
                recScene = db.getRecordEx(tableRBScene, 'id', [tableRBScene['appointmentType'].eq(typeQueue + 1)], 'rbScene.code')
                if recScene:
                    sceneId = forceRef(recScene.value('id'))
            visit = self.modelVisits.getEmptyRecord(sceneId=sceneId, personId=personId)
            visit.setValue('date', toVariant(date))
            return visit

        self.eventSetDateTime = eventSetDatetime if isinstance(eventSetDatetime, QDateTime) else QDateTime(eventSetDatetime)
        self.eventDate = eventDatetime.date() if isinstance(eventDatetime, QDateTime) else eventDatetime
        self.setOrgId(orgId if orgId else QtGui.qApp.currentOrgId())
        self.setEventTypeId(eventTypeId)
        self.setClientId(clientId)
        self.prolongateEvent = True if actionByNewEvent else False
        self.tabNotes.setNotesEx(externalId, assistantId, curatorId, relegateOrgId, relegatePersonId, clientId, relegateInfo)
        self.setExternalId(externalId)
        self.cmbPerson.setValue(personId)
        setPerson = getEventSetPerson(self.eventTypeId)
        if setPerson == 0:
            self.setPerson = personId
        elif setPerson == 1:
            self.setPerson = QtGui.qApp.userId
        self.edtBegDate.setDate(self.eventSetDateTime.date())
        self.edtEndDate.setDate(self.eventDate)
        self.edtBegTime.setTime(eventSetDatetime.time() if isinstance(eventSetDatetime, QDateTime) else QTime())
        self.edtEndTime.setTime(eventDatetime.time() if isinstance(eventDatetime, QDateTime) else QTime())
        self.edtNextDate.setDate(QDate())
#        self.chkPrimary.setChecked(getPrimary(clientId, eventTypeId, personId))
        self.cmbOrder.setCurrentIndex(eventOrder)
        self.cmbContract.setCurrentIndex(0)
        #self.cmbResult.setCurrentIndex(0)
        self.cmbPrimary.setCurrentIndex(getEventIsPrimary(eventTypeId))
        self.initFocus()

        visitTypeId = presetDiagnostics[0][3] if presetDiagnostics else None
        self.modelVisits.setDefaultVisitTypeId(visitTypeId)
        visits = []
        date = QDate()
        if self.eventDate:
            date = self.eventDate
            resultId = self.cmbResult.value()
            if not resultId:
                if self.cmbResult.model().rowCount() > 1:
                    self.cmbResult.setValue(self.cmbResult.model().getId(1))
        elif self.eventSetDateTime and self.eventSetDateTime.date():
              date = self.eventSetDateTime.date()
        else:
             date = QDate.currentDate()
        if getEventAddVisit(eventTypeId):
            showTime = getEventShowTime(eventTypeId)
            showVisitTime = getEventShowVisitTime(eventTypeId)
            if showVisitTime:
                if not showTime:
                    if self.eventDate:
                        date = QDateTime(self.eventDate, QTime.currentTime())
                    elif self.eventSetDateTime and self.eventSetDateTime.date():
                        date = QDateTime(self.eventSetDateTime.date(), QTime.currentTime())
                    else:
                        date = QDateTime.currentDateTime()
                else:
                    if self.eventDate:
                        date = eventDatetime
                    elif self.eventSetDateTime and self.eventSetDateTime.date():
                        date = self.eventSetDateTime
                    else:
                        date = QDateTime.currentDateTime()
            else:
                if self.eventDate:
                    date = self.eventDate
                elif self.eventSetDateTime and self.eventSetDateTime.date():
                    date = self.eventSetDateTime.date()
                else:
                    date = QDate.currentDate()
            visits.append(prepVisit(date, personId))
        if not len(visits):
            QtGui.QMessageBox.warning( self,
                                             u'Внимание!',
                                             u'Для события по форме 043 требуется обязательное указание параметра "Добавлять визит при создании" на вкладке "Визиты" в справочнике "Типы событий"',
                                             QtGui.QMessageBox.Ok,
                                             QtGui.QMessageBox.Ok)
            return False
        self.modelVisits.setItems(visits)
        self.updateVisitsInfo()
        self.modelDentition.setClientId(self.clientId)
        self.modelParodentium.setClientId(self.clientId)
        self.prepareDentition(actionTypeId=None, date=forceDateTime(visits[0].value('date')), personId=forceRef(visits[0].value('person_id')))
        self.prepareParodentium(actionTypeId=None, date=forceDateTime(visits[0].value('date')), personId=forceRef(visits[0].value('person_id')))
        if presetDiagnostics:
            resultId = None
            if self.cmbResult.model().rowCount() > 1:
                resultId = getDiagnosticResultId(self.cmbResult.model().getId(1))
            for MKB, dispanserId, healthGroupId, visitTypeId in presetDiagnostics:
                item = self.modelDiagnostics.getEmptyRecord()
                item.setValue('MKB', toVariant(MKB))
                item.setValue('dispanser_id',   toVariant(dispanserId))
                item.setValue('healthGroup_id', toVariant(healthGroupId))
                item.setValue('result_id', toVariant(resultId))
                self.modelDiagnostics.items().append(item)
            self.modelDiagnostics.reset()
        self.prepareActions(contractId, presetActions, disabledActions, movingActionTypeId, valueProperties, diagnos, financeId, protocolQuoteId, actionByNewEvent, plannedEndDate)
        self.grpTempInvalid.pickupTempInvalid()
        self.grpAegrotat.pickupTempInvalid()
        self.grpDisability.pickupTempInvalid()
        self.grpVitalRestriction.pickupTempInvalid()
        self.setIsDirty(False)
        self.tabNotes.setEventEditor(self)
        self.setFilterResult(self.eventSetDateTime.date())
        return self.checkEventCreationRestriction() and self.checkDeposit()


    def prepareActions(self, contractId, presetActions, disabledActions, actionTypeIdValue, valueProperties, diagnos, financeId,
                       protocolQuoteId, actionByNewEvent, plannedEndDate):
        def addActionType(actionTypeId, amount, financeId, contractId, idListActionType, idListActionTypeIPH, actionFinance, idListActionTypeMoving, plannedEndDate):
            db = QtGui.qApp.db
            tableOrgStructure = db.table('OrgStructure')
            for model in (self.tabStatus.modelAPActions,
                          self.tabDiagnostic.modelAPActions,
                          self.tabCure.modelAPActions,
                          self.tabMisc.modelAPActions):
                if actionTypeId in model.actionTypeIdList:
                    if actionTypeId in idListActionType and not actionByNewEvent:
                        model.addRow(actionTypeId=actionTypeId, amount=amount, financeId=financeId, contractId=contractId)
                        record, action = model.items()[-1]
                        if plannedEndDate:
                            record.setValue('directionDate', QVariant(plannedEndDate))
                        if u'Приемное отделение' in action._actionType._propertiesByName:
                            curOrgStructureId = QtGui.qApp.currentOrgStructureId()
                            if curOrgStructureId:
                                recOS = db.getRecordEx(tableOrgStructure,
                                               [tableOrgStructure['id']],
                                               [tableOrgStructure['deleted'].eq(0),
                                                tableOrgStructure['id'].eq(curOrgStructureId),
                                                tableOrgStructure['type'].eq(4)])
                                if recOS:
                                    curOSId = forceRef(recOS.value('id'))
                                    action[u'Приемное отделение'] = curOSId if curOSId else None
                        if valueProperties and len(valueProperties) > 0 and valueProperties[0]:
                            action[u'Направлен в отделение'] = valueProperties[0]
                        if protocolQuoteId:
                            action[u'Квота'] = protocolQuoteId
                        if actionFinance == 0:
                            record.setValue('finance_id', toVariant(financeId))
                    elif actionTypeId in idListActionTypeIPH:
                        model.addRow(actionTypeId=actionTypeId, amount=amount, financeId=financeId, contractId=contractId)
                        record, action = model.items()[-1]
                        if diagnos:
                            record, action = model.items()[-1]
                            action[u'Диагноз'] = diagnos
                    #[self.eventActionFinance, self.receivedFinanceId, orgStructureTransfer, orgStructurePresence, oldBegDate, movingQuoting, personId]
                    elif actionByNewEvent and actionTypeId in idListActionTypeMoving:
                        model.addRow(actionTypeId=actionTypeId, amount=amount, financeId=financeId, contractId=contractId)
                        record, action = model.items()[-1]
                        if actionByNewEvent[0] == 0:
                            record.setValue('finance_id', toVariant(actionByNewEvent[1]))
                        action[u'Отделение пребывания'] = actionByNewEvent[2]
                        if actionByNewEvent[3]:
                            action[u'Переведен из отделения'] = actionByNewEvent[3]
                        if actionByNewEvent[4]:
                            record.setValue('begDate', toVariant(actionByNewEvent[4]))
                        else:
                            record.setValue('begDate', toVariant(QDateTime.currentDateTime()))
                        if u'Квота' in action._actionType._propertiesByName and actionByNewEvent[5]:
                            action[u'Квота'] = actionByNewEvent[5]
                        if actionByNewEvent[6]:
                            record.setValue('person_id', toVariant(actionByNewEvent[6]))
                    elif (actionByNewEvent and actionTypeId not in idListActionType) or not actionByNewEvent:
                        model.addRow(actionTypeId=actionTypeId, amount=amount, financeId=financeId, contractId=contractId)
                        record, action = model.items()[-1]

        def disableActionType(actionTypeId):
            for model in (self.tabStatus.modelAPActions,
                          self.tabDiagnostic.modelAPActions,
                          self.tabCure.modelAPActions,
                          self.tabMisc.modelAPActions):
                if actionTypeId in model.actionTypeIdList:
                    model.disableActionType(actionTypeId)
                    break

        if disabledActions:
            for actionTypeId in disabledActions:
                disableActionType(actionTypeId)
        if presetActions:
            db = QtGui.qApp.db
            tableActionType = db.table('ActionType')
            tableEventType = db.table('EventType')
            idListActionType = db.getIdList(tableActionType, [tableActionType['id']], [tableActionType['flatCode'].like(u'received%'), tableActionType['deleted'].eq(0)])
            idListActionTypeIPH = db.getIdList(tableActionType, [tableActionType['id']], [tableActionType['flatCode'].like(u'inspectPigeonHole%'), tableActionType['deleted'].eq(0)])
            idListActionTypeMoving = db.getIdList(tableActionType, [tableActionType['id']], [tableActionType['flatCode'].like(u'moving%'), tableActionType['deleted'].eq(0)])
            eventTypeId = self.getEventTypeId()
            actionFinance = None
            if eventTypeId:
                recordET = db.getRecordEx(tableEventType, [tableEventType['actionFinance']], [tableEventType['deleted'].eq(0), tableEventType['id'].eq(eventTypeId)])
                actionFinance = forceInt(recordET.value('actionFinance')) if recordET else None
            if actionByNewEvent:
                actionTypeMoving = False
                for actionTypeId, amount, cash in presetActions:
                    if actionTypeId in idListActionTypeMoving:
                        actionTypeMoving = True
                        break
                if not actionTypeMoving and idListActionTypeMoving:
                    presetActions.append((idListActionTypeMoving[0], 1.0, False))
            for actionTypeId, amount, cash in presetActions:
                addActionType(actionTypeId, amount, financeId if cash else None, contractId if cash else None, idListActionType, idListActionTypeIPH, actionFinance, idListActionTypeMoving, plannedEndDate)


    def prepare(self, clientId, eventTypeId, orgId, personId, eventSetDatetime, eventDatetime, weekProfile,
                numDays, externalId, assistantId, curatorId, flagHospitalization = False,
                movingActionTypeId = None, valueProperties = [], tissueTypeId=None, selectPreviousActions=False,
                relegateOrgId = None, relegatePersonId=None, diagnos = None, financeId = None,
                protocolQuoteId = None, actionByNewEvent = [], order = 1,
                actionListToNewEvent = [], typeQueue = -1, docNum=None, relegateInfo=[], plannedEndDate = None,
                mapJournalInfoTransfer = []):
        self.setPersonId(personId)
        eventDate = eventDatetime.date() if isinstance(eventDatetime, QDateTime) else eventDatetime
        if not eventDate and eventSetDatetime:
            eventDate = eventSetDatetime.date() if isinstance(eventSetDatetime, QDateTime) else eventSetDatetime
        else:
            eventDate = QDate.currentDate()
        if QtGui.qApp.userHasAnyRight([urAccessF043planner, urAdmin]):
            dlg = CPreF043Dialog(self, self.contractTariffCache)
            try:
                dlg.setBegDateEvent(eventSetDatetime.date() if isinstance(eventSetDatetime, QDateTime) else eventSetDatetime)
                dlg.prepare(clientId, eventTypeId, eventDate, self.personId, self.personSpecialityId, self.personTariffCategoryId, flagHospitalization, movingActionTypeId, tissueTypeId)
                if dlg.diagnosticsTableIsNotEmpty() or dlg.actionsTableIsNotEmpty():
                    if not dlg.exec_():
                        return False
                result = self._prepare(dlg.contractId, clientId, eventTypeId, orgId, personId, eventSetDatetime,
                                       eventDatetime, weekProfile, numDays, dlg.diagnostics(), dlg.actions(),
                                       dlg.disabledActionTypeIdList, externalId, assistantId, curatorId,
                                       movingActionTypeId, valueProperties, relegateOrgId, relegatePersonId,
                                       diagnos, financeId, protocolQuoteId, actionByNewEvent, order,
                                       typeQueue, relegateInfo, plannedEndDate)
                return result
            finally:
                dlg.destroy()
                sip.delete(dlg)
                del dlg
        else:
            presets = CPreF043DagnosticAndActionPresets(clientId, eventTypeId, eventDate, self.personSpecialityId, flagHospitalization, movingActionTypeId)
            presets.setBegDateEvent(eventSetDatetime.date() if isinstance(eventSetDatetime, QDateTime) else eventSetDatetime)
            return self._prepare(None, clientId, eventTypeId, orgId, personId, eventSetDatetime, eventDatetime, weekProfile, numDays,
                                 presets.unconditionalDiagnosticList, presets.unconditionalActionList, presets.disabledActionTypeIdList,
                                 externalId, assistantId, curatorId, None, [], relegateOrgId, relegatePersonId,
                                 diagnos, financeId, protocolQuoteId, actionByNewEvent, order, typeQueue, relegateInfo, plannedEndDate)


    def addActionTypeDentition(self, date, personId, eventId, dentType):
        db = QtGui.qApp.db
#        tableActionType = db.table('ActionType')
        tablePerson     = db.table('vrbPersonWithSpeciality')
        actionTypeId = None
        newAction = None
        newRecord = None
        dentActionTypeId, parodentActionTypeId = getDentitionActionTypeId()
        if not dentType:
            actionTypeId = parodentActionTypeId
        else:
            actionTypeId = dentActionTypeId
        for model in (self.tabStatus.modelAPActions,
                      self.tabDiagnostic.modelAPActions,
                      self.tabCure.modelAPActions,
                      self.tabMisc.modelAPActions):
            if actionTypeId in model.actionTypeIdList:
                model.addRow(actionTypeId=actionTypeId)
                newRecord, newAction = model.items()[-1]
                initActionProperties(newAction)
                newRecord.setValue('directionDate', QVariant(date))
                newRecord.setValue('begDate', QVariant(date))
                newRecord.setValue('endDate', QVariant(date))
                newRecord.setValue('person_id', QVariant(personId))
                newRecord.setValue('event_id', QVariant(eventId))
                recordAction = newAction.getRecord()
                recordAction.setValue('directionDate', QVariant(date))
                recordAction.setValue('begDate', QVariant(date))
                recordAction.setValue('endDate', QVariant(date))
                recordAction.setValue('person_id', QVariant(personId))
                recordAction.setValue('event_id', QVariant(eventId))
                break
        actionEventId = forceRef(recordAction.value('event_id')) if recordAction else None
        isCurrentDentitionAction = (eventId != actionEventId)
        actionId = forceRef(newRecord.value('id')) if newRecord else None
        personName = forceString(db.translate(tablePerson, 'id', personId, 'name'))
        isChecked = 0
        isLoaded = False
        if not dentType:
            if len(self.modelClientDentitionHistory._resultDentitionHistoryItems) > 0:
                self.modelClientDentitionHistory._resultDentitionHistoryItems.sort(key=lambda x: forceDateTime(x[0].value('begDate')) if forceRef(x[0].value('event_id')) == eventId and forceDateTime(x[0].value('begDate')) <= date else None)
                self.modelClientDentitionHistory._resultDentitionHistoryItems.reverse()
            self.modelClientDentitionHistory.addNewParodentiumForVisitItem(newRecord, newAction, actionId, eventId, isChecked, personName)
            penultimateItemAction = self.modelParodentium.getPenultimateItemAction()
            if penultimateItemAction:
                self.modelParodentium.copyParodent(action=penultimateItemAction)
                isLoaded = True
            isInit = True
            if not isLoaded or (isInit and not self.modelParodentium.teethIsSet()):
                self.modelParodentium.setDefaults(self.clientAge[3]>8)
            self.actionParodentiumList[(date.toPyDateTime(), personId, eventId)] = (newRecord, newAction)
            self.modelParodentium.loadAction(newRecord, newAction, isCurrentDentitionAction)
            if len(self.modelClientDentitionHistory._resultDentitionHistoryItems) > 0:
                self.modelClientDentitionHistory._resultDentitionHistoryItems.sort(key=lambda x: forceDateTime(x[0].value('begDate')))
                self.modelClientDentitionHistory._resultDentitionHistoryItems.reverse()
            if len(self.modelClientDentitionHistory._dentitionHistoryItems) > 0:
                self.modelClientDentitionHistory._dentitionHistoryItems.sort(key=lambda x: forceDateTime(x[0].value('begDate')))
                self.modelClientDentitionHistory._dentitionHistoryItems.reverse()
            self.modelClientDentitionHistory.setCurrentDentitionItems(newRecord, newAction, eventId)
        else:
            if len(self.modelClientDentitionHistory._dentitionHistoryItems) > 0:
                self.modelClientDentitionHistory._dentitionHistoryItems.sort(key=lambda x: forceDateTime(x[0].value('begDate')) if forceRef(x[0].value('event_id')) == eventId and forceDateTime(x[0].value('begDate')) <= date else None)
                self.modelClientDentitionHistory._dentitionHistoryItems.reverse()
            self.modelClientDentitionHistory.addNewDentitionForVisitItem(newRecord, newAction, actionId, eventId, isChecked, personName)
            penultimateItemAction = self.modelDentition.getPenultimateItemAction()
            if penultimateItemAction:
                self.modelDentition.copyInspection(skippingNames=[u'Статус', u'Санация'],
                                                   action=penultimateItemAction)
                isLoaded = True
            self.tblClientDentitionHistory.setCurrentIndex(self.modelClientDentitionHistory.index(0, 0))
            self.isNewDentitionActionInitialized = True
            if not isLoaded or (self.isNewDentitionActionInitialized and not self.modelDentition.teethIsSet()):
                self.modelDentition.setDefaults(self.clientAge[3]>8)
            self.actionDentitionList[(date.toPyDateTime(), personId, eventId)] = (newRecord, newAction)
            self.modelClientDentitionHistory.setCurrentDentitionItems(newRecord, newAction, eventId)
        self.modelClientDentitionHistory.reset()
        return newRecord, newAction


    def setEventTypeId(self, eventTypeId):
        CEventEditDialog.setEventTypeId(self, eventTypeId, u'Ф.043')
        showTime = getEventShowTime(eventTypeId)
        showVisitTime = getEventShowVisitTime(self.eventTypeId)
        if showVisitTime:
            self.modelVisits._cols[self.modelVisits.getColIndex('date')] = CDateTimeForEventInDocTableCol(u'Дата', 'date', 20)
        self.getShowButtonAccount()
        self.edtBegTime.setVisible(showTime)
        self.edtEndTime.setVisible(showTime)
        self.cmbResult.setTable('rbResult', True, 'eventPurpose_id=\'%d\'' % self.eventPurposeId)
        self.cmbResult.setValue(CF043Dialog.defaultEventResultId)
        cols = self.modelDiagnostics.cols()
        resultCol = cols[len(cols)-1]
        resultCol.filter = 'eventPurpose_id=\'%d\'' % self.eventPurposeId
        self.cmbContract.setEventTypeId(eventTypeId)
        self.setVisitAssistantVisible(self.tblVisits, hasEventVisitAssistant(eventTypeId))
        customizePrintButton(self.btnPrint, self.eventContext if self.eventContext else 'F043')


    def setOrgId(self, orgId):
        self.orgId = orgId
        self.cmbContract.setOrgId(orgId)
        self.cmbPerson.setOrgId(orgId)


    def setRecord(self, record):
        CEventEditDialog.setRecord(self, record)
        setDatetimeEditValue(self.edtBegDate, self.edtBegTime, record, 'setDate')
        setRBComboBoxValue(self.cmbPerson,      record, 'setPerson_id')
        setDatetimeEditValue(self.edtEndDate, self.edtEndTime, record, 'execDate')
        setRBComboBoxValue(self.cmbPerson,      record, 'execPerson_id')
        setRBComboBoxValue(self.cmbResult,      record, 'result_id')
        setDateEditValue(self.edtNextDate,      record, 'nextEventDate')
        self.cmbPrimary.setCurrentIndex(forceInt(record.value('isPrimary'))-1)
        self.setExternalId(forceString(record.value('externalId')))
        self.cmbOrder.setCurrentIndex(forceInt(record.value('order'))-1)
        self.setPersonId(self.cmbPerson.value())
        setRBComboBoxValue(self.cmbContract, record, 'contract_id')
        self.setPerson = forceRef(record.value('setPerson_id'))
        self._updateNoteByPrevEventId()
        self.tabNotes.setNotes(record)
        self.tabNotes.setEventEditor(self)
        self.loadVisits()
        self.loadDiagnostics()
        self.tabMedicalDiagnosis.load(self.itemId())
        self.updateMesMKB()
        self.tabMes.setRecord(record)
        self.loadActions()
        self.modelDentition.setClientId(self.clientId)
        self.modelParodentium.setClientId(self.clientId)
        self.setIsDirty(False)

        self.grpTempInvalid.pickupTempInvalid()
        self.grpAegrotat.pickupTempInvalid()
        self.grpDisability.pickupTempInvalid()
        self.grpVitalRestriction.pickupTempInvalid()
        self.loadDentition()
        self.loadParodentium()
        self._recountCommonUet()
        self.tabCash.load(self.itemId())
        self.initFocus()
        self.setIsDirty(False)
        self.blankMovingIdList = []
        if self.tabNotes.isEventClosed() or self.isHBDialog:
            self.protectClosedEvent()


    def getDiagnosisTypeId(self, dt):
        return forceRef(QtGui.qApp.db.translate('rbDiagnosisType', 'code', '2' if dt else '9', 'id'))


    def loadVisits(self):
        self.modelVisits.loadItems(self.itemId())
        self.updateVisitsInfo()


    def loadDiagnostics(self):
        db = QtGui.qApp.db
        table = db.table('Diagnostic')
#        tablePerson = db.table('Person')
        isDiagnosisManualSwitch = self.modelDiagnostics.manualSwitchDiagnosis()
        rawItems = db.getRecordList(table, '*', [table['deleted'].eq(0), table['event_id'].eq(self.itemId())], 'id')
        items = []
        for record in rawItems:
#            specialityId = forceRef(record.value('speciality_id'))
            diagnosisId     = record.value('diagnosis_id')
            MKB             = db.translate('Diagnosis', 'id', forceRef(diagnosisId), 'MKB')
            MKBEx           = db.translate('Diagnosis', 'id', forceRef(diagnosisId), 'MKBEx')
            morphologyMKB   = db.translate('Diagnosis', 'id', forceRef(diagnosisId), 'morphologyMKB')
            setDate         = forceDate(record.value('setDate'))
            newRecord = self.modelDiagnostics.getEmptyRecord()
            copyFields(newRecord, record)
            newRecord.setValue('MKB',           MKB)
            newRecord.setValue('MKBEx',         MKBEx)
            newRecord.setValue('morphologyMKB', morphologyMKB)

            if isDiagnosisManualSwitch:
                isCheckedHandleDiagnosis = checkIsHandleDiagnosisIsChecked(setDate,
                                                                           self.clientId,
                                                                           diagnosisId)
                newRecord.setValue('handleDiagnosis', QVariant(isCheckedHandleDiagnosis))

            items.append(newRecord)
        self.modelDiagnostics.setItems(items)


    def loadActions(self):
        eventId = self.itemId()
        self.tabStatus.loadActions(eventId)
        self.tabDiagnostic.loadActions(eventId)
        self.tabCure.loadActions(eventId)
        self.tabMisc.loadActions(eventId)
        self.modelActionsSummary.regenerate()
        self.tabCash.modelAccActions.regenerate()


    def loadDentition(self):
        model = self.modelClientDentitionHistory
        setDate = self.eventSetDateTime.date()
        currentEventId = self.itemId()
        model.loadClientDentitionHistory(self.clientId, setDate)
        self.modelDentition.setAdultDefaults(self.clientAge[3]>8)
        for tab in (self.tabStatus,
                    self.tabDiagnostic,
                    self.tabCure,
                    self.tabMisc):
            modelAPActions = tab.tblAPActions.model()
            for record, action in modelAPActions.items():
                actionType = action.getType()
                if actionType.flatCode == u'dentitionInspection':
                    eventId = forceRef(record.value('event_id'))
                    if currentEventId and currentEventId == eventId:
                        date = forceDateTime(record.value('begDate'))
                        personId = forceRef(record.value('person_id'))
                        self.actionDentitionList[(date.toPyDateTime(), personId, eventId)] = (record, action)
                        model.setCurrentDentitionItems(record, action, currentEventId)
        if len(self.modelClientDentitionHistory._dentitionHistoryItems) > 0:
            self.modelClientDentitionHistory._dentitionHistoryItems.sort(key=lambda x: forceDateTime(x[0].value('begDate')))
            self.modelClientDentitionHistory._dentitionHistoryItems.reverse()
        record, action, id, eventId, isChecked, personName = self.modelClientDentitionHistory.getItemByRow(0)
        self.modelClientDentitionHistory.setCurrentDentitionItem(record, action)
        currentEventId = self.itemId()
        actionEventId = forceRef(record.value('event_id')) if record else None
        self.modelDentition.setIsCurrentDentitionAction(currentEventId == actionEventId)


    def loadParodentium(self):
        self.modelParodentium.setAdultDefaults(self.clientAge[3]>8)
        for tab in (self.tabStatus,
                    self.tabDiagnostic,
                    self.tabCure,
                    self.tabMisc):
            modelAPActions = tab.tblAPActions.model()
            for record, action in modelAPActions.items():
                actionType = action.getType()
                if actionType.flatCode == u'parodentInsp':
                    actionEventId = forceRef(record.value('event_id'))
                    date = forceDateTime(record.value('begDate'))
                    personId = forceRef(record.value('person_id'))
                    eventId = forceRef(record.value('event_id'))
                    self.actionParodentiumList[(date.toPyDateTime(), personId, eventId)] = (record, action)
        if len(self.modelClientDentitionHistory._resultDentitionHistoryItems) > 0:
            self.modelClientDentitionHistory._resultDentitionHistoryItems.sort(key=lambda x: forceDateTime(x[0].value('begDate')))
            self.modelClientDentitionHistory._resultDentitionHistoryItems.reverse()
        record, action, id, eventId, isChecked, personName = self.modelClientDentitionHistory.getParadentiumItem(0)
        self.modelClientDentitionHistory.setCurrentParadentiumItem(record, action)
        currentEventId = self.itemId()
        actionEventId = forceRef(record.value('event_id')) if record else None
        self.modelParodentium.setIsCurrentParadentiumAction(currentEventId == actionEventId)
        self.tblVisits.setCurrentIndex(self.modelVisits.index(len(self.modelVisits.items())-1, 0))
        self.tblClientDentitionHistory.setCurrentIndex(self.modelClientDentitionHistory.index(0, 0))


    def loadDentitionAdditional(self, record, action, isCurrentDentitionAction):
        #дополнительные свойства для Dentition
        for widget in (self.edtDentitionObjectively,
                       self.cmbDentitionBite,
                       self.cmbDentitionApparat,
                       self.cmbDentitionProtezes,
                       self.cmbDentitionOrtodontCure,
                       self.edtDentitionMucosa,
                       self.edtDentitionNote,
                       self.cmbDentitionSanitation):
            widget.setEnabled(isCurrentDentitionAction)
        if action:
            if u'Объективность' in action._actionType._propertiesByName:
                dentitionObjectivelyText = action.getProperty(u'Объективность').getText()
                dentitionObjectivelyText = dentitionObjectivelyText if dentitionObjectivelyText else u''
                self.edtDentitionObjectively.setValue(dentitionObjectivelyText)

            if u'Прикус' in action._actionType._propertiesByName:
                self.cmbDentitionBite.blockSignals(True)
                dentitionBiteProperty = action.getProperty(u'Прикус')
                self.cmbDentitionBite.setDomain(dentitionBiteProperty.type().valueDomain)
                dentitionBiteText = dentitionBiteProperty.getText() or ''
                self.cmbDentitionBite.setEditText(dentitionBiteText)
                self.cmbDentitionBite.blockSignals(False)

            if u'Аппарат' in action._actionType._propertiesByName:
                self.cmbDentitionApparat.blockSignals(True)
                dentitionApparatProperty = action.getProperty(u'Аппарат')
                self.cmbDentitionApparat.setDomain(dentitionApparatProperty.type().valueDomain)
                dentitionApparatText = dentitionApparatProperty.getText() or ''
                self.cmbDentitionApparat.setEditText(dentitionApparatText)
                self.cmbDentitionApparat.blockSignals(False)

            if u'Протезы' in action._actionType._propertiesByName:
                self.cmbDentitionProtezes.blockSignals(True)
                dentitionProtezesProperty = action.getProperty(u'Протезы')
                self.cmbDentitionProtezes.setDomain(dentitionProtezesProperty.type().valueDomain)
                dentitionProtezesText = dentitionProtezesProperty.getText() or ''
                self.cmbDentitionProtezes.setEditText(dentitionProtezesText)
                self.cmbDentitionProtezes.blockSignals(False)

            if u'Ортодонтическое лечение' in action._actionType._propertiesByName:
                self.cmbDentitionOrtodontCure.blockSignals(True)
                dentitionOrtodontCureProperty = action.getProperty(u'Ортодонтическое лечение')
                self.cmbDentitionOrtodontCure.setDomain(dentitionOrtodontCureProperty.type().valueDomain)
                dentitionOrtodontCureText = dentitionOrtodontCureProperty.getText() or ''
                self.cmbDentitionOrtodontCure.setEditText(dentitionOrtodontCureText)
                self.cmbDentitionOrtodontCure.blockSignals(False)

            if u'Санация' in action._actionType._propertiesByName:
                self.cmbDentitionSanitation.blockSignals(True)
                dentitionSanitationProperty = action.getProperty(u'Санация')
                self.cmbDentitionSanitation.setDomain(dentitionSanitationProperty.type().valueDomain)
                dentitionSanitationText = dentitionSanitationProperty.getText() or ''
                self.cmbDentitionSanitation.setValue(dentitionSanitationText)
                self.cmbDentitionSanitation.blockSignals(False)

            if u'Слизистая' in action._actionType._propertiesByName:
                dentitionMucosaText = action.getProperty(u'Слизистая').getText() or ''
                self.edtDentitionMucosa.setValue(dentitionMucosaText)

            if u'Примечание' in action._actionType._propertiesByName:
                dentitionNoteText = action.getProperty(u'Примечание').getText() or ''
                self.edtDentitionNote.setPlainText(dentitionNoteText)


    def updateActionProperties(self, oldAction, newAction):
        newActionType = newAction.getType()
        actionTypeList = newActionType.getPropertiesById().items()
        for actionPropertyType in actionTypeList:
            actionPropertyType = actionPropertyType[1]
            actionPropertyTypeName = actionPropertyType.name
            actionPropertyTypeValue = newAction.getProperty(actionPropertyTypeName).getText()
            oldAction.getProperty(actionPropertyTypeName).setValue(
                                actionPropertyType.convertQVariantToPyValue(actionPropertyTypeValue))
        return oldAction


    def prepareDentition(self, actionTypeId = None, date = None, personId = None):
        db = QtGui.qApp.db
        tablePerson = db.table('vrbPersonWithSpeciality')
        eventId = self.itemId()
        if not actionTypeId:
            actionTypeIdList = db.getIdList('ActionType', 'id', '`flatCode`=\'dentitionInspection\' AND `deleted`=0')
            if actionTypeIdList:
                actionTypeId = actionTypeIdList[0]
        setDate = self.eventSetDateTime.date()
        self.modelClientDentitionHistory.loadClientDentitionHistory(self.clientId, setDate)
        isLoaded = False
        if actionTypeId:
#            actionType = CActionTypeCache.getById(actionTypeId)
            for actionsModel in (self.tabStatus.modelAPActions,
                                 self.tabDiagnostic.modelAPActions,
                                 self.tabCure.modelAPActions,
                                 self.tabMisc.modelAPActions):
                if actionTypeId in actionsModel.actionTypeIdList:
                    actionsModel.addRow(actionTypeId=actionTypeId)
                    newRecord, newAction = actionsModel.items()[-1]
                    initActionProperties(newAction)
                    newRecord.setValue('directionDate', QVariant(date))
                    newRecord.setValue('begDate', QVariant(date))
                    newRecord.setValue('endDate', QVariant(date))
                    newRecord.setValue('person_id', QVariant(personId))
                    newRecord.setValue('event_id', QVariant(eventId))
                    recordAction = newAction.getRecord()
                    recordAction.setValue('directionDate', QVariant(date))
                    recordAction.setValue('begDate', QVariant(date))
                    recordAction.setValue('endDate', QVariant(date))
                    recordAction.setValue('person_id', QVariant(personId))
                    recordAction.setValue('event_id', QVariant(eventId))
                    if not self.modelDentition.getClientId():
                        return
                    actionId = forceRef(newRecord.value('id')) if newRecord else None
                    personName = forceString(db.translate(tablePerson, 'id', personId, 'name'))
                    isChecked = 0
                    self.modelClientDentitionHistory.addNewDentitionForVisitItem(newRecord, newAction, actionId, eventId, isChecked, personName)
                    penultimateItemAction = self.modelDentition.getPenultimateItemAction()
                    if penultimateItemAction:
                        self.modelDentition.copyInspection(skippingNames=[u'Статус', u'Санация'],
                                                           action=penultimateItemAction)
                        isLoaded = True
                    self.tblClientDentitionHistory.setCurrentIndex(self.modelClientDentitionHistory.index(0, 0))
                    self.modelClientDentitionHistory.setCurrentDentitionItems(newRecord, newAction, eventId)
                    self.modelClientDentitionHistory.reset()
                    self.isNewDentitionActionInitialized = True
                    self.actionDentitionList[(date.toPyDateTime(), personId, self.itemId())] = (newRecord, newAction)
            if not isLoaded or (self.isNewDentitionActionInitialized and not self.modelDentition.teethIsSet()):
                self.modelDentition.setDefaults(self.clientAge[3]>8)


    def prepareParodentium(self, actionTypeId = None, date = None, personId = None):
        db = QtGui.qApp.db
        tablePerson = db.table('vrbPersonWithSpeciality')
        eventId = self.itemId()
        if not actionTypeId:
            actionTypeIdList = db.getIdList('ActionType', 'id', '`flatCode`=\'parodentInsp\' AND `deleted`=0')
            if actionTypeIdList:
                actionTypeId = actionTypeIdList[0]

        isLoaded = False
        isInit   = False
        if actionTypeId:
            for actionsModel in (self.tabStatus.modelAPActions,
                                 self.tabDiagnostic.modelAPActions,
                                 self.tabCure.modelAPActions,
                                 self.tabMisc.modelAPActions):
                if actionTypeId in actionsModel.actionTypeIdList:
                    actionsModel.addRow(actionTypeId=actionTypeId)
                    newRecord, newAction = actionsModel.items()[-1]
                    initActionProperties(newAction)
                    newRecord.setValue('directionDate', QVariant(date))
                    newRecord.setValue('begDate', QVariant(date))
                    newRecord.setValue('endDate', QVariant(date))
                    newRecord.setValue('person_id', QVariant(personId))
                    newRecord.setValue('event_id', QVariant(eventId))
                    recordAction = newAction.getRecord()
                    recordAction.setValue('directionDate', QVariant(date))
                    recordAction.setValue('begDate', QVariant(date))
                    recordAction.setValue('endDate', QVariant(date))
                    recordAction.setValue('person_id', QVariant(personId))
                    recordAction.setValue('event_id', QVariant(eventId))
                    self.modelParodentium.clientId = self.clientId
                    if not self.modelParodentium.clientId:
                        return
                    actionId = forceRef(newRecord.value('id')) if newRecord else None
                    personName = forceString(db.translate(tablePerson, 'id', personId, 'name'))
                    isChecked = 0
                    self.modelClientDentitionHistory.addNewParodentiumForVisitItem(newRecord, newAction, actionId, eventId, isChecked, personName)
                    self.modelParodentium.loadAction(newRecord, newAction, False)
                    penultimateItemAction = self.modelParodentium.getPenultimateItemAction()
                    if penultimateItemAction:
                        self.modelParodentium.copyParodent(action=penultimateItemAction)
                        isLoaded = True
                    isInit = True
                    self.actionParodentiumList[(date.toPyDateTime(), personId, eventId)] = (newRecord, newAction)
            if not isLoaded or (isInit and not self.modelParodentium.teethIsSet()):
                self.modelParodentium.setDefaults(self.clientAge[3]>8)
        self.tblClientDentitionHistory.setCurrentIndex(self.modelClientDentitionHistory.index(0, 0))



    def updateMesMKB(self):
        MKB, MKBEx = self.getFinalDiagnosisMKB()
        self.tabMes.setMKB(MKB)
        self.tabMes.setMKBEx(MKBEx)


    @pyqtSignature('QDate')
    def on_edtBegDate_dateChanged(self, date):
        self.cmbContract.setDate(self.getDateForContract())
        self.tabMes.setEventBegDate(date)
        self.setFilterResult(date)
        self.setPersonDate(date)


    @pyqtSignature('QDate')
    def on_edtEndDate_dateChanged(self, date):
        self.cmbContract.setDate(self.getDateForContract())
        if getEventShowTime(self.eventTypeId):
            time = QTime.currentTime() if date else QTime()
            self.edtEndTime.setTime(time)


    @pyqtSignature('')
    def on_modelDiagnostics_diagnosisChanged(self):
        self.updateMesMKB()


    @pyqtSignature('')
    def on_edtDentitionObjectively_editingFinished(self):
        txt = forceString(self.edtDentitionObjectively.value())
        current = self.tblClientDentitionHistory.currentIndex()
        record, action, id, eventId, isChecked, personName = self.modelClientDentitionHistory.getItem(current)
        if action:
            action[u'Объективность'] = txt


    @pyqtSignature('QString')
    def on_cmbDentitionBite_editTextChanged(self, txt):
        current = self.tblClientDentitionHistory.currentIndex()
        record, action, id, eventId, isChecked, personName = self.modelClientDentitionHistory.getItem(current)
        if action:
            action[u'Прикус'] = unicode(txt)


    @pyqtSignature('QString')
    def on_cmbDentitionApparat_editTextChanged(self, txt):
        current = self.tblClientDentitionHistory.currentIndex()
        record, action, id, eventId, isChecked, personName = self.modelClientDentitionHistory.getItem(current)
        if action:
            action[u'Аппарат'] = unicode(txt)


    @pyqtSignature('QString')
    def on_cmbDentitionProtezes_editTextChanged(self, txt):
        current = self.tblClientDentitionHistory.currentIndex()
        record, action, id, eventId, isChecked, personName = self.modelClientDentitionHistory.getItem(current)
        if action:
            action[u'Протезы'] = unicode(txt)


    @pyqtSignature('QString')
    def on_cmbDentitionOrtodontCure_editTextChanged(self, txt):
        current = self.tblClientDentitionHistory.currentIndex()
        record, action, id, eventId, isChecked, personName = self.modelClientDentitionHistory.getItem(current)
        if action:
            action[u'Ортодонтическое лечение'] = unicode(txt)


    @pyqtSignature('QString')
    def on_cmbDentitionSanitation_editTextChanged(self, txt):
        current = self.tblClientDentitionHistory.currentIndex()
        record, action, id, eventId, isChecked, personName = self.modelClientDentitionHistory.getItem(current)
        if action:
            action[u'Санация'] = unicode(txt)


    @pyqtSignature('')
    def on_edtDentitionMucosa_editingFinished(self):
        txt = forceString(self.edtDentitionMucosa.value())
        current = self.tblClientDentitionHistory.currentIndex()
        record, action, id, eventId, isChecked, personName = self.modelClientDentitionHistory.getItem(current)
        if action:
            action[u'Слизистая'] = txt


    @pyqtSignature('')
    def on_edtDentitionNote_textChanged(self):
        txt = unicode(self.edtDentitionNote.toPlainText())
        current = self.tblClientDentitionHistory.currentIndex()
        record, action, id, eventId, isChecked, personName = self.modelClientDentitionHistory.getItem(current)
        if action:
            action[u'Примечание'] = txt


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelVisits_currentChanged(self, current, previous):
        if current.isValid():
            row = current.row()
            items = self.modelVisits.items()
            if row >= 0 and row < len(items):
                item = items[row]
                date = forceDateTime(item.value('date'))
                personId = forceRef(item.value('person_id'))
                eventId = forceRef(item.value('event_id'))
                if not eventId:
                    eventId = self.itemId()
                recordDentition, actionDentition = self.actionDentitionList.get((date.toPyDateTime(), personId, eventId), (None, None))
                actionRecordDentition = actionDentition.getRecord() if actionDentition else None
                actionEventId = forceRef(actionRecordDentition.value('event_id')) if actionRecordDentition else None
                isCurrentDentitionAction = (eventId == actionEventId)
                self.modelDentition.loadAction(actionRecordDentition, actionDentition, isCurrentDentitionAction)
                self.loadDentitionAdditional(actionRecordDentition, actionDentition, isCurrentDentitionAction)
                recordParodentium, actionParodentium = self.actionParodentiumList.get((date.toPyDateTime(), personId, eventId), (None, None))
                actionRecordParodentium = actionParodentium.getRecord() if actionParodentium else None
                self.modelParodentium.loadAction(actionRecordParodentium, actionParodentium, isCurrentDentitionAction)
                self.modelClientDentitionHistory.setCurrentParadentiumItem(actionRecordParodentium, actionParodentium)
                currentRow = self.modelClientDentitionHistory.setCurrentDentitionItem(actionRecordDentition, actionDentition)
                if currentRow is not None:
                    self.tblClientDentitionHistory.setCurrentIndex(self.modelClientDentitionHistory.index(currentRow, 0))


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelClientDentitionHistory_currentChanged(self, current, previous):
        if current.isValid():
            currentRow = current.row()
            self.modelClientDentitionHistory.setCurrentHistoryRow(currentRow)
            record, action, id, eventId, isChecked, personName = self.modelClientDentitionHistory.getItem(current)
            isCurrentDentitionAction = self.modelClientDentitionHistory.isCurrentDentitionAction(currentRow) and (self.itemId() == eventId)
            self.modelDentition.loadAction(record, action, isCurrentDentitionAction)
            self.loadDentitionAdditional(record, action, isCurrentDentitionAction)
            paradentiumItem = self.modelClientDentitionHistory.getParadentiumItem(currentRow)
            self.modelParodentium.loadAction(paradentiumItem[0], paradentiumItem[1], isCurrentDentitionAction)


    def initFocus(self):
        if self.cmbContract.count() != 1:
            self.cmbContract.setFocus(Qt.OtherFocusReason)
        else:
            self.tblDiagnostics.setFocus(Qt.OtherFocusReason)

    # этот код повторяется 4 раза!
    def updateVisitsInfo(self):
        items = self.modelVisits.items()
        self.lblVisitsCountValue.setText(str(len(items)))
        minDate = maxDate = None
        for item in items:
            date = forceDate(item.value('date'))
            if date:
                if minDate:
                    minDate = min(minDate, date)
                    maxDate = max(maxDate, date)
                else:
                    minDate = maxDate = date
        if minDate:
            days = minDate.daysTo(maxDate)+1
        else:
            days = 0
        self.lblVisitsDurationValue.setText(str(days))


    def isPrimary(self):
        return self.cmbPrimary.currentIndex() == 1


    def getRecord(self):
        record = CEventEditDialog.getRecord(self)
        showTime = getEventShowTime(self.eventTypeId)
        self.getShowButtonAccount()
        CF043Dialog.defaultEventResultId = self.cmbResult.value()
        getRBComboBoxValue(self.cmbContract,    record, 'contract_id')
        getDatetimeEditValue(self.edtBegDate, self.edtBegTime, record, 'setDate', showTime)
        record.setValue('setPerson_id', self.setPerson)
        getDatetimeEditValue(self.edtEndDate, self.edtEndTime, record, 'execDate', showTime)
        getRBComboBoxValue(self.cmbPerson,      record, 'execPerson_id')
        getRBComboBoxValue(self.cmbResult,      record, 'result_id')
        getDateEditValue(self.edtNextDate,      record, 'nextEventDate')
        record.setValue('isPrimary', toVariant(self.cmbPrimary.currentIndex()+1))
        record.setValue('order',  toVariant(self.cmbOrder.currentIndex()+1))
        if self.prolongateEvent:
            record.setValue('order', toVariant(5))
        self.tabMes.getRecord(record)
        self.tabNotes.getNotes(record, self.eventTypeId)
        # Это хак: удаляем payStatus из записи
        result = type(record)(record) # copy record
        result.remove(result.indexOf('payStatus'))
        return result


    def saveInternals(self, eventId):
        self.saveVisits(eventId)
        self.saveDiagnostics(eventId)
        self.tabMedicalDiagnosis.save(eventId)
        self.tabMes.save(eventId)
        setAskedClassValueForDiagnosisManualSwitch(None)
        self.saveActions(eventId)
        self.tabCash.save(eventId)
        self.saveBlankUsers(self.blankMovingIdList)
        self.tabNotes.saveAttachedFiles(eventId)


    def saveVisits(self, eventId):
        self.modelVisits.saveItems(eventId)


    def updateDiagnosisTypes(self):
        items = self.modelDiagnostics.items()
        isFirst = True
        for item in items:
            diagnosisTypeId = self.getDiagnosisTypeId(isFirst)
            item.setValue('diagnosisType_id', toVariant(diagnosisTypeId))
            isFirst = False


    def saveDiagnostics(self, eventId):
        items = self.modelDiagnostics.items()
        isDiagnosisManualSwitch = self.modelDiagnostics.manualSwitchDiagnosis()
        isFirst = True
        endDate = self.edtEndDate.date()
        endDateVariant = toVariant(endDate)
        MKBDiagnosisIdPairList = []
        prevId = 0
        for row, item in enumerate(items):
            MKB   = forceString(item.value('MKB'))
            MKBEx = forceString(item.value('MKBEx'))
            TNMS  = forceString(item.value('TNMS'))
            morphologyMKB = forceString(item.value('morphologyMKB'))
            personId = forceRef(item.value('person_id'))
            specialityId = forceRef(QtGui.qApp.db.translate('Person', 'id', personId, 'speciality_id'))
            diagnosisTypeId = self.getDiagnosisTypeId(isFirst)
            diagnosticTypeId = forceRef(item.value('diagnosisType_id'))
            if not diagnosticTypeId:
                item.setValue('diagnosisType_id', toVariant(diagnosisTypeId))
            item.setValue('speciality_id', toVariant(specialityId))
            item.setValue('setDate', endDateVariant )
            item.setValue('endDate', endDateVariant )
            diagnosisId = forceRef(item.value('diagnosis_id'))
            characterId = forceRef(item.value('character_id'))

            diagnosisId, characterId = getDiagnosisId2(
                    endDate,
                    self.personId,
                    self.clientId,
                    diagnosisTypeId,
                    MKB,
                    MKBEx,
                    forceRef(item.value('character_id')),
                    forceRef(item.value('dispanser_id')),
                    forceRef(item.value('traumaType_id')),
                    diagnosisId,
                    forceRef(item.value('id')),
                    isDiagnosisManualSwitch,
                    forceBool(item.value('handleDiagnosis')),
                    TNMS=TNMS,
                    morphologyMKB=morphologyMKB
                    )
            item.setValue('diagnosis_id', toVariant(diagnosisId))
            item.setValue('TNMS', toVariant(TNMS))
            item.setValue('character_id', toVariant(characterId))
            itemId = forceInt(item.value('id'))
            if prevId > itemId:
                item.setValue('id', QVariant())
                prevId=0
            else:
               prevId = itemId
            isFirst = False
            MKBDiagnosisIdPairList.append((MKB, diagnosisId))
        self.modelDiagnostics.saveItems(eventId)
        self.modifyDiagnosises(MKBDiagnosisIdPairList)


    def saveActions(self, eventId):
        self.tabStatus.saveActions(eventId)
        self.tabDiagnostic.saveActions(eventId)
        self.tabCure.saveActions(eventId)
        self.tabMisc.saveActions(eventId)


    def checkDataEntered(self):
        result = CEventEditDialog.checkDataEntered(self)
        self.blankMovingIdList = []
        mesRequired = getEventMesRequired(self.eventTypeId)
        csgRequired  = getEventCSGRequired(self.eventTypeId)

        showTime = getEventShowTime(self.eventTypeId)
        if showTime:
            begDate = QDateTime(self.edtBegDate.date(), self.edtBegTime.time())
            endDate = QDateTime(self.edtEndDate.date(), self.edtEndTime.time())
        else:
            begDate = self.edtBegDate.date()
            endDate = self.edtEndDate.date()
        begDateCheck = self.edtBegDate.date()
        endDateCheck = self.edtEndDate.date()
        nextDate = self.edtNextDate.date()
        result = result and (self.orgId != QtGui.qApp.currentOrgId() or self.cmbContract.value() or self.checkInputMessage(u'договор', False, self.cmbContract))
        result = result and (self.cmbPerson.value()   or self.checkInputMessage(u'врача',        False, self.cmbPerson))
        result = result and (not begDateCheck.isNull() or self.checkInputMessage(u'дату назначения', False, self.edtBegDate))
        if endDateCheck:
            result = result and self.checkActionDataEntered(begDate, QDateTime(), endDate, self.tabToken, self.edtBegDate, None, self.edtEndDate)
            result = result and self.checkEventDate(begDate, endDate, nextDate, self.tabToken, self.edtNextDate,self.edtEndDate, True)
            minDuration,  maxDuration = getEventDurationRange(self.eventTypeId)
            if minDuration<=maxDuration:
                result = result and (begDateCheck.daysTo(endDateCheck)+1>=minDuration or self.checkValueMessage(u'Длительность должна быть не менее %s'%formatNum(minDuration, (u'дня', u'дней', u'дней')), False, self.edtEndDate))
                result = result and (maxDuration==0 or begDateCheck.daysTo(endDateCheck)+1<=maxDuration or self.checkValueMessage(u'Длительность должна быть не более %s'%formatNum(maxDuration, (u'дня', u'дней', u'дней')), False, self.edtEndDate))
            result = result and (len(self.modelDiagnostics.items())>0 or self.checkInputMessage(u'диагноз', False, self.tblDiagnostics))
            result = result and (self.cmbResult.value()  or self.checkInputMessage(u'результат',   False, self.cmbResult))
            result = result and self.checkDiagnosticsDataEntered()
            result = result and self.checkExecDateForVisit(endDateCheck)
            if mesRequired:
                result = result and self.tabMes.checkMesAndSpecification()
                result = result and (self.tabMes.chechMesDuration() or self.checkValueMessage(u'Длительность события должна попадать в интервал минимальной и максимальной длительности по требованию к МЭС', True, self.edtBegDate))
            if csgRequired:
                result = result and self.tabMes.checkCsg()
#                result = result and self.checkInspectionsMKBForMes(self.tblInspections, self.tabMes.cmbMes.value())
        result = result and self.checkActionsDateEnteredActuality(begDate, endDate, [self.tabStatus, self.tabDiagnostic, self.tabCure, self.tabMisc])
        result = result and self.checkActionsDataEntered(begDate, endDate)
        result = result and self.checkDeposit(True)
        result = result and (len(self.modelVisits.items())>0 or self.checkInputMessage(u'посещение', False, self.tblVisits))
        #result = result and self.checkVisitsDataEntered(begDate.date() if isinstance(begDate, QDateTime) else begDate, endDate.date() if isinstance(endDate, QDateTime) else endDate)
        result = result and self.checkVisitsDataEntered(begDate, endDate)
        result = result and self.tabCash.checkDataLocalContract()
        result = result and self.checkSerialNumberEntered()
        result = result and self.checkTabNotesEventExternalId()
        self.valueForAllActionEndDate = None
        if self.edtEndDate.date():
            result = result and self.checkAndUpdateExpertise(self.edtEndDate.date(), self.cmbPerson.value())
        return result


    def checkDiagnosticsDataEntered(self):
        result = True
        result = result and self.checkDiagnostics(self.modelDiagnostics, self.cmbPerson.value())
        return result


    def checkDiagnostics(self, model, finalPersonId):
        for row, record in enumerate(self.modelDiagnostics.items()):
            if not self.checkDiagnosticDataEntered(row, record):
                return False
        return True


    def checkDiagnosticDataEntered(self, row, record):
        result = True
        if result:
            MKB = forceString(record.value('MKB'))
            MKBEx = forceString(record.value('MKBEx'))
            result = MKB or self.checkInputMessage(u'диагноз', False, self.tblDiagnostics, row, record.indexOf('MKB'))
            result = result and self.checkActualMKB(self.tblDiagnostics, self.edtBegDate.date(), MKB, record, row)
            if result:
                char = MKB[:1]
                traumaTypeId = forceRef(record.value('traumaType_id'))
                if char in 'ST':
                    if not traumaTypeId:
                        result = self.checkValueMessage(u'Необходимо указать тип травмы', False, self.tblDiagnostics, row, record.indexOf('traumaType_id'))
                    if result:
                        result = MKBEx or self.checkInputMessage(u'Дополнительный диагноз', True if QtGui.qApp.controlMKBExForTraumaType()==0 else False, self.tblDiagnostics, row, record.indexOf('MKBEx'))
                        if result:
                            charEx = MKBEx[:1]
                            if charEx not in 'VWXY':
                                result = self.checkValueMessage(u'Доп.МКБ не соотвествует Доп.МКБ при травме', True, self.tblDiagnostics, row, record.indexOf('MKBEx'))
                if char not in 'ST' and traumaTypeId:
                    result = self.checkValueMessage(u'Необходимо удалить тип травмы', False, self.tblDiagnostics, row, record.indexOf('traumaType_id'))
                result = self.checkRequiresFillingDispanser(result, self.tblDiagnostics, record, row, MKB)
            if not forceRef(record.value('result_id')):
                result = result and self.checkValueMessage(u'Необходимо указать результат', False, self.tblDiagnostics, row, record.indexOf('result_id'))
            result = result and self.checkPeriodResultHealthGroup(record, row, self.tblDiagnostics)
        return result


    def getDiagFilter(self):
        specialityId = self.personSpecialityId
        result = self.mapSpecialityIdToDiagFilter.get(specialityId, None)
        if result is None:
            result = QtGui.qApp.db.translate('rbSpeciality', 'id', specialityId, 'mkbFilter')
            if result is None:
                result = ''
            else:
                result = forceString(result)
            self.mapSpecialityIdToDiagFilter[specialityId] = forceString(result)
        return result


    def checkDiagnosis(self, MKB):
        diagFilter = self.getDiagFilter()
        return checkDiagnosis(self, MKB, diagFilter, self.clientId, self.clientSex, self.clientAge, self.edtBegDate.date())


    def getEventInfo(self, context):
        result = CEventEditDialog.getEventInfo(self, context, CTeethEventInfo)
        # инициализация свойств
        result._isPrimary = self.isPrimary()
        # инициализация таблиц
        result._actions = CActionInfoProxyList(context,
                [self.tabStatus.modelAPActions, self.tabDiagnostic.modelAPActions, self.tabCure.modelAPActions, self.tabMisc.modelAPActions],
                result)
        result.initActions() # инициализируем _action_stomat и _action_parodent
        result._diagnosises = CDiagnosticInfoProxyList(context, [self.modelDiagnostics])
        result._visits = CVisitInfoProxyList(context, self.modelVisits)
        return result


    @pyqtSignature('int')
    def on_dentitionTabWidget_currentChanged(self, index):
        model = self.modelClientDentitionHistory
        if index == INSPECTION_DENTITION_TAB_INDEX:
            model.setShownHistoryType(model.dentitionIsShown)
        elif index == RESULT_DENTITION_TAB_INDEX:
            model.setShownHistoryType(model.paradentiumIsShown)


    @pyqtSignature('int')
    def on_tabWidget_currentChanged(self, index):
        widget = self.tabWidget.widget(index)
        if widget is not None:
            focusProxy = widget.focusProxy()
            if focusProxy:
                focusProxy.setFocus(Qt.OtherFocusReason)
        self.btnPrintMedicalDiagnosisEnabled(index)
        if widget == self.tabMes and self.eventTypeId:
            self.tabMes.setMESServiceTemplate(self.eventTypeId)
        if widget == self.tabAmbCard: # amb card page
            self.tabAmbCard.resetWidgets()
        if widget == self.tabToken:
            self._recountCommonUet()


    def getTempInvalidInfo(self, context):
        return self.grpTempInvalid.getTempInvalidInfo(context)


    @pyqtSignature('')
    def on_mnuDiagnostics_aboutToShow(self):
        canRemove = False
        currentRow = self.tblDiagnostics.currentIndex().row()
        if currentRow>=0:
#            canRemove = self.modelDiagnostics.payStatus(currentRow) == 0
            canRemove = True
        self.actDiagnosticsRemove.setEnabled(canRemove)


    @pyqtSignature('')
    def on_actDiagnosticsRemove_triggered(self):
        currentRow = self.tblDiagnostics.currentIndex().row()
        self.modelDiagnostics.removeRowEx(currentRow)
        self.updateDiagnosisTypes()


    @pyqtSignature('')
    def on_modelDiagnostics_resultChanged(self):
        CF043Dialog.defaultDiagnosticResultId = self.modelDiagnostics.resultId()
        defaultResultId = getEventResultId(CF043Dialog.defaultDiagnosticResultId, self.eventPurposeId)
        if defaultResultId:
            self.cmbResult.setValue(defaultResultId)


    @pyqtSignature('int')
    def on_cmbPerson_currentIndexChanged(self):
        self.setPersonId(self.cmbPerson.value())


# #####################################################################

class CF043BaseDiagnosticsModel(CMKBListInDocTableModel):
    __pyqtSignals__ = ('typeOrPersonChanged()',
                       'diagnosisChanged()',
                       'resultChanged()',
                      )
    MKB_allowed_morphology = ['C', 'D']

    def __init__(self, parent, finishDiagnosisTypeCode, baseDiagnosisTypeCode, accompDiagnosisTypeCode):
        CMKBListInDocTableModel.__init__(self, 'Diagnostic', 'id', 'event_id', parent)
        self.isManualSwitchDiagnosis = QtGui.qApp.defaultIsManualSwitchDiagnosis()
        self.isMKBMorphology = QtGui.qApp.defaultMorphologyMKBIsVisible()
        self.characterIdForHandleDiagnosis = None
        self.columnHandleDiagnosis = None
        self._parent = parent
        self.diagnosisTypeCol = CDiagnosisTypeCol( u'Тип', 'diagnosisType_id', 2, [finishDiagnosisTypeCode, baseDiagnosisTypeCode, accompDiagnosisTypeCode], smartMode=False)
        self.addCol(self.diagnosisTypeCol)
        self.addCol(CPersonFindInDocTableCol(u'Врач', 'person_id',  20, 'vrbPersonWithSpecialityAndOrgStr', parent=parent))
        self.addExtCol(CICDExInDocTableCol(u'МКБ',         'MKB',   7), QVariant.String)
        self.addExtCol(CICDExInDocTableCol(u'Доп.МКБ',     'MKBEx', 7), QVariant.String)
        if QtGui.qApp.isTNMSVisible():
            self.addCol(CTNMSCol(u'TNM-Ст', 'TNMS',  10))
        if self.isMKBMorphology:
            self.addExtCol(CMKBMorphologyCol(u'Морф.', 'morphologyMKB', 10, 'MKB_Morphology', filter='`group` IS NOT NULL'), QVariant.String)
        self.addCol(CDiseaseCharacter(     u'Хар',         'character_id',   7, showFields=CRBComboBox.showCode, preferredWidth=150)).setToolTip(u'Характер')
        if self.isManualSwitchDiagnosis:
            self.addExtCol(CBoolInDocTableCol( u'П',   'handleDiagnosis', 10), QVariant.Int)
            self.characterIdForHandleDiagnosis = forceRef(QtGui.qApp.db.translate('rbDiseaseCharacter', 'code', '1', 'id'))
            self.columnHandleDiagnosis = self._mapFieldNameToCol.get('handleDiagnosis')
        self.addCol(CDiseasePhases(        u'Фаза',        'phase_id',       7, showFields=CRBComboBox.showCode, preferredWidth=150)).setToolTip(u'Фаза')
        self.addCol(CDiseaseStage(         u'Ст',          'stage_id',       7, showFields=CRBComboBox.showCode, preferredWidth=150)).setToolTip(u'Стадия')
        self.addCol(CRBInDocTableCol(    u'ДН',            'dispanser_id',   7, rbDispanser, showFields=CRBComboBox.showCode, preferredWidth=150)).setToolTip(u'Диспансерное наблюдение')
#        self.addCol(CRBLikeEnumInDocTableCol(u'Госп',      'hospital',       7, CHospitalInfo.names, showFields=CRBComboBox.showCode, preferredWidth=150)).setToolTip(u'Потребность в госпитализации')
        self.addCol(CRBInDocTableCol(    u'Травма',        'traumaType_id', 10, rbTraumaType, addNone=True, showFields=CRBComboBox.showName, preferredWidth=150))
        self.addCol(CRBInDocTableCol(    u'ГрЗд',          'healthGroup_id', 7, rbHealthGroup, addNone=True, showFields=CRBComboBox.showCode, preferredWidth=150)).setToolTip(u'Группа здоровья')
        self.addCol(CInDocTableCol(u'Описание',     'freeInput', 15))
        self.setFilter(self.table['diagnosisType_id'].inlist([id for id in self.diagnosisTypeCol.ids if id]))
        self.readOnly = False
        self.getDispanserIdLists()


    def getDispanserIdLists(self):
        db = QtGui.qApp.db
        self.observedDispanserIdList = db.getDistinctIdList('rbDispanser', 'id', ['observed = 1'])
        recordIdList = db.getDistinctIdList('rbDispanser', 'id', ['code = 2'])
        self.takenDispanserId = recordIdList[0] if len(recordIdList) > 0 else None
        self.takenDispanserIdList = db.getDistinctIdList('rbDispanser', 'id', ['code = 2 OR code = 6'])
        consistDispanserIdList = db.getDistinctIdList('rbDispanser', 'id', ['code = 1'])
        self.consistDispanserId = consistDispanserIdList[0] if len(consistDispanserIdList) > 0 else None


    def setReadOnly(self, value):
        self.readOnly = value


    def manualSwitchDiagnosis(self):
        return self.isManualSwitchDiagnosis


    def flags(self, index=QModelIndex()):
        if self.readOnly:
            return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        result = CMKBListInDocTableModel.flags(self, index)
        row = index.row()
        if row < len(self._items):
            column = index.column()
            if self.isManualSwitchDiagnosis and index.isValid():
                if column == self.columnHandleDiagnosis:
                    characterId = forceRef(self.items()[row].value('character_id'))
                    if characterId != self.characterIdForHandleDiagnosis:
                        result = (result & ~Qt.ItemIsUserCheckable)
#                        return result
            if self.isMKBMorphology and index.isValid():
                if column == self._mapFieldNameToCol.get('morphologyMKB'):
                    mkb = forceString(self.items()[row].value('MKB'))
                    if not (bool(mkb) and mkb[0] in CF043BaseDiagnosticsModel.MKB_allowed_morphology):
                        result = (result & ~Qt.ItemIsEditable)
        return result


    def getEmptyRecord(self):
        eventEditor = QObject.parent(self)
        result = CMKBListInDocTableModel.getEmptyRecord(self)
        result.append(QtSql.QSqlField('diagnosis_id',     QVariant.Int))
        result.append(QtSql.QSqlField('speciality_id',    QVariant.Int))
        result.append(QtSql.QSqlField('setDate',          QVariant.DateTime))
        result.append(QtSql.QSqlField('endDate',          QVariant.DateTime))
        result.setValue('person_id',     toVariant(eventEditor.getSuggestedPersonId()))
        if self.items():
            result.setValue('diagnosisType_id',  toVariant(self.diagnosisTypeCol.ids[1] if QtGui.qApp.diagnosisTypeAfterFinalForm043() else self.diagnosisTypeCol.ids[2]))
        else:
            result.setValue('diagnosisType_id',  toVariant(self.diagnosisTypeCol.ids[0] if self.diagnosisTypeCol.ids[0] else self.diagnosisTypeCol.ids[1]))
            result.setValue('result_id',  toVariant(CF043Dialog.defaultDiagnosticResultId))
        return result


    def setData(self, index, value, role=Qt.EditRole):
        column = index.column()
        row = index.row()
        if not variantEq(self.data(index, role), value):
            eventEditor = QObject.parent(self)
            if column == 0: # тип диагноза
                result = CMKBListInDocTableModel.setData(self, index, value, role)
                if result:
                    self.updateDiagnosisType(set([row]))
                    self.emitTypeOrPersonChanged()
                return result
            elif column == 1: # врач
                personId = forceRef(value)
                if not eventEditor.checkClientAttendanceEE(personId):
                    return False
                result = CMKBListInDocTableModel.setData(self, index, value, role)
                if result:
                    self.updateDiagnosisType(set())
                    #self.emitTypeOrPersonChanged()
                return result
            elif column == 2: # код МКБ
                newMKB = forceString(value)
                if not newMKB:
                    specifiedMKB = ''
                    specifiedMKBEx = ''
                    specifiedCharacterId = None
                    specifiedTraumaTypeId = None
                    specifiedDispanserId = None
                    specifiedRequiresFillingDispanser = 0
                    specifiedProlongMKB = False
                else:
                    acceptable, specifiedMKB, specifiedMKBEx, specifiedCharacterId, specifiedTraumaTypeId, specifiedDispanserId, specifiedRequiresFillingDispanser, specifiedProlongMKB = eventEditor.specifyDiagnosis(newMKB)
                    if not acceptable:
                        return False
                value = toVariant(specifiedMKB)
                result = CMKBListInDocTableModel.setData(self, index, value, role)
                if result:
                    if specifiedRequiresFillingDispanser == 2:
                        self.updateDispanserByMKB(row, specifiedDispanserId, specifiedProlongMKB)
                    self.updateCharacterByMKB(row, specifiedMKB, specifiedCharacterId)
                    self.updateTraumaType(row, specifiedMKB, specifiedTraumaTypeId)
                    self.emitTypeOrPersonChanged()
                return result
            if column == 3: # доп. код МКБ
                newMKB = forceString(value)
                if not newMKB:
                    pass
                else:
                    acceptable = eventEditor.checkDiagnosis(newMKB)
                    if not acceptable:
                        return False
                value = toVariant(newMKB)
                result = CMKBListInDocTableModel.setData(self, index, value, role)
#                if result:
#                    self.updateCharacterByMKB(row, specifiedMKB)
                self.emit(SIGNAL('diagnosisChanged()'))
                return result
            return CMKBListInDocTableModel.setData(self, index, value, role)
        else:
            return True


    def removeRowEx(self, row):
        self.removeRows(row, 1)


    def updateDiagnosisType(self, fixedRowSet):
        mapPersonIdToRow = {}
        diagnosisTypeIds = []
        for row, item in enumerate(self.items()):
            personId = forceRef(item.value('person_id'))
            rows = mapPersonIdToRow.setdefault(personId, [])
            rows.append(row)
            diagnosisTypeId = forceRef(item.value('diagnosisType_id'))
            diagnosisTypeIds.append(diagnosisTypeId)
        for personId, rows in mapPersonIdToRow.iteritems():
            if self.diagnosisTypeCol.ids[0] and personId == self._parent.personId:
                firstDiagnosisId = self.diagnosisTypeCol.ids[0]
            else:
                firstDiagnosisId = self.diagnosisTypeCol.ids[1]
            baseDiagnosisId = self.diagnosisTypeCol.ids[1] if QtGui.qApp.diagnosisTypeAfterFinalForm043() else self.diagnosisTypeCol.ids[2]
            usedDiagnosisTypeIds = [diagnosisTypeIds[row] for row in fixedRowSet.intersection(set(rows))]
            freeRows = set(rows).difference(fixedRowSet)
            if firstDiagnosisId != baseDiagnosisId and self.diagnosisTypeCol.ids[0] in usedDiagnosisTypeIds and freeRows:
                for row in rows:
                    if row in freeRows and diagnosisTypeIds[row] == firstDiagnosisId:
                        self.items()[row].setValue('diagnosisType_id', toVariant(baseDiagnosisId))
                        self.emitCellChanged(row, item.indexOf('diagnosisType_id'))
                        diagnosisTypeIds[row] = forceRef(self.items()[row].value('diagnosisType_id'))
            if firstDiagnosisId == baseDiagnosisId and self.diagnosisTypeCol.ids[0] in usedDiagnosisTypeIds:
                for row in fixedRowSet.intersection(set(rows)):
                    usedDiagnosisTypeId = diagnosisTypeIds[row]
                    if self.diagnosisTypeCol.ids[0] == usedDiagnosisTypeId:
                        self.items()[row].setValue('diagnosisType_id', toVariant(baseDiagnosisId))
                        self.emitCellChanged(row, item.indexOf('diagnosisType_id'))
                        diagnosisTypeIds[row] = forceRef(self.items()[row].value('diagnosisType_id'))


    def updateDispanserByMKB(self, row, specifiedDispanserId, specifiedProlongMKB):
        item = self.items()[row]
        if specifiedProlongMKB and specifiedDispanserId and specifiedDispanserId in self.observedDispanserIdList:
            dispanserId = specifiedDispanserId if specifiedDispanserId not in self.takenDispanserIdList else self.consistDispanserId
            item.setValue('dispanser_id', toVariant(dispanserId))
            self.emitCellChanged(row, item.indexOf('dispanser_id'))
        elif not specifiedProlongMKB:
            item.setValue('dispanser_id', toVariant(self.takenDispanserId))
            self.emitCellChanged(row, item.indexOf('dispanser_id'))


    def updateCharacterByMKB(self, row, MKB, specifiedCharacterId):
        characterIdList = getAvailableCharacterIdByMKB(MKB)
        item = self.items()[row]
        if specifiedCharacterId in characterIdList:
            characterId = specifiedCharacterId
        else:
            characterId = forceRef(item.value('character_id'))
            if (characterId in characterIdList) or (characterId is None and not characterIdList):
                return
            if characterIdList:
                characterId = characterIdList[0]
            else:
                characterId = None
        item.setValue('character_id', toVariant(characterId))
        self.emitCellChanged(row, item.indexOf('character_id'))


    def updateTraumaType(self, row, MKB, specifiedTraumaTypeId):
        item = self.items()[row]
        prevTraumaTypeId = forceRef(item.value('traumaType_id'))
        if specifiedTraumaTypeId:
            traumaTypeId = specifiedTraumaTypeId
        else:
            traumaTypeId = prevTraumaTypeId
        if traumaTypeId != prevTraumaTypeId:
            item.setValue('traumaType_id', toVariant(traumaTypeId))
            self.emitCellChanged(row, item.indexOf('traumaType_id'))


    def getPersonsWithSignificantDiagnosisType(self):
        result = []
        significantDiagnosisTypeIdList = [self.diagnosisTypeCol.ids[0], self.diagnosisTypeCol.ids[1]]
        for item in self.items():
            diagnosisTypeId = forceRef(item.value('diagnosisType_id'))
            if diagnosisTypeId and diagnosisTypeId in significantDiagnosisTypeIdList:
                personId = forceRef(item.value('person_id'))
                if personId and personId not in result:
                    result.append(personId)
        return result


    def emitTypeOrPersonChanged(self):
        self.emit(SIGNAL('typeOrPersonChanged()'))


class CF043FinalDiagnosticsModel(CF043BaseDiagnosticsModel):
    __pyqtSignals__ = ('resultChanged()',
                      )

    def __init__(self, parent):
        CF043BaseDiagnosticsModel.__init__(self, parent, '1', '2', '9')
        self.addCol(CRBInDocTableCol(    u'Результат',     'result_id',     10, 'rbDiagnosticResult', showFields=CRBComboBox.showNameAndCode, preferredWidth=350))
        self.mapMKBToServiceId = {}


    def getCloseOrMainDiagnosisTypeIdList(self):
        return [self.diagnosisTypeCol.codeToId(code) for code in ('1', '2')]


    def setData(self, index, value, role=Qt.EditRole):
        result = CF043BaseDiagnosticsModel.setData(self, index, value, role)
        if result:
            column = index.column()
            if column == 0 or column == self._mapFieldNameToCol.get('result_id'): # тип диагноза и результат
                row = index.row()
                item = self.items()[row]
                diagnosisTypeId = forceRef(item.value('diagnosisType_id'))
                if diagnosisTypeId == self.diagnosisTypeCol.ids[0]:
                    self.emitResultChanged()
        return result


    def resultId(self):
        finalDiagnosisId = self.diagnosisTypeCol.ids[0]
        items = self.items()
        for item in items:
            diagnosisTypeId = forceRef(item.value('diagnosisType_id'))
            if diagnosisTypeId == finalDiagnosisId:
                return forceRef(item.value('result_id'))
        return None


    def diagnosisServiceId(self):
        items = self.items()
        if items:
            code = forceString(items[0].value('MKB'))
            if code in self.mapMKBToServiceId:
                return self.mapMKBToServiceId[code]
            else:
                serviceId = forceRef(QtGui.qApp.db.translate('MKB', 'DiagID', code, 'service_id'))
                self.mapMKBToServiceId[code] = serviceId
                return serviceId
        else:
            return None


    def emitResultChanged(self):
        self.emit(SIGNAL('resultChanged()'))
