# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""Управление оповещениями пациентов"""

from urllib2 import HTTPError

from PyQt4 import QtGui
from PyQt4.QtCore import QDateTime

from library.PrintInfo import CInfoContext
from library.PrintTemplates import readUrl, compileAndExecTemplate, loadJson
from library.SendMailDialog import sendMailInt
from library.Utils import forceRef, forceString, forceInt, toVariant

from Exchange.Utils import tbl
from Orgs.PersonInfo import CPersonInfo
from Orgs.Utils import COrgInfo, COrgStructureInfo, CActivityInfo
from Registry.Utils import CClientInfo


from Notifications.NotificationRule import (tblNotificationRule,
                                            CNotificationRule)

tblNotificationLog = 'Notification_Log'
kindCodeSMS = '1'
kindCodeEmail = '2'
kindCodeViber = '3'
strSuccess = u'Успех'
strUnknownError = u'Неизвестный код ошибки'

class CNotificationManager(object):
    u"""Реализация управления оповещениями"""

    mapSmsErrorCode = {
        1 : u'Внутренняя ошибка.',
        2 : u'Ошибка парсинга.',
        3 : u'Ошибка авторизации.',
        4 : u'невалидное значение (режима вывода) mode',
        5 : u'невалидное значение (формата вывода) output',
        10 : u'Ошибка номера абонента',
        11 : u'Ошибка имени отправителя',
        12 : u'Ошибка текста сообщения',
        14 : u'Ошибка пользовательского messageId',
        15 : u'Недостаточно средств на балансе',
        16 : u'Ошибка отправки (не настроена маршрутизация)',
        18 : u'Сообщение с указанным userMessageId уже было',
        19 : u'Номер находится в «Чёрном списке»',
        20 : u'Превышено допустимое кол-во абонентов в запросе на множественную'
             u' отправку',
        25 : u'Неверный формат smsId',
        26 : u'Неверный формат userMessageId',
        27 : u'Не передано ни одного Id сообщения',
        28 : u'Нельзя передавать и smsId и userMessageId одновременно',
        29 : u'Сообщение с указанным Id не найдено',
        30 : u'Ошибка очередности каналов отправки order',
        34 : u'Ошибка таймаута («времени жизни») messengerModeTTL на Viber',
        35 : u'Неверный набор параметров для контента (текст, картинка, кнопка'
             u')',
        36 : u'Некорректный URL для картинки',
        37 : u'Некорректный URL для кнопки',
        38 : u'Некорректный маршрут',
        40 : u'Ошибка таймаута («времени жизни») smsTTL в SMS',
        44 : u'Не задан параметр options для поля типа enum',
        45 : u'Не знадано название поля',
        46 : u'Неверный тип поля',
        47 : u'Поле типа `link` должно называться `link`',
        48 : u'Поле типа `link` должно иметь значение по умолчанию (оно будет '
             u'подставлено в название кнопки при отправке через Viber)',
        50 : u'Такого шаблона не существует',
        51 : u'У вас нет доступа к данному шаблону',
        52 : u'Ошибка при проверке значений полей',
        53 : u'Не передано обязательное значение',
        54 : u'Неверный формат даты',
        55 : u'В базе не заданы возможные значения для поля типа enum',
        56 : u'Неверное значение для данного типа поля',
        57 : u'Неверное значение для поля типа link (невалидная ссылка)',
        59 : u'У данного пользователя нет ни одного шаблона',
        60 : u'Неверный формат id базы данных',
        61 : u'Имя базы не задано или содержит недопустимые символы',
        62 : u'Не передано ни одного подписчика для импорта',
        63 : u'Нет поля в данной базе абонентов',
    }


    def __init__(self):
        if hasattr(QtGui.qApp, 'db'):
            self._db = QtGui.qApp.db
            self._tblNotificationLog = tbl(tblNotificationLog)
            self._tblNotificationRule = tbl(tblNotificationRule)
            self._tblClientContact = tbl('ClientContact')
        prefs = QtGui.qApp.preferences.appPrefs if hasattr(QtGui.qApp,
                                                           'preferences') else {}
        self.__smsUser = forceString(prefs.get('smsLogin', 'test'))
        self.__smsPassword = forceString(prefs.get('smsPassword', 'test'))
        self.__smsSender = forceString(prefs.get('smsSender', 'SAMSON'))
        self.__viberSender = forceString(prefs.get('viberSender', 'SAMSON'))
        self.__apiKey = forceString(prefs.get('apiKey'))


    def notifyClient(self, clientInfoList, ruleId, schedule, msg=''):
        u"""Создает оповещения по списку пациентов по заданному правилу"""
        record = self._db.getRecord(self._tblNotificationRule, '*', ruleId)

        if record and clientInfoList:
            rule = CNotificationRule(record)

            for (clientId, scheduleItem) in clientInfoList:
                text = parseNotificationTemplate(rule.template,
                                                 clientId, (scheduleItem,
                                                            schedule), msg=msg)

                if not text:
                    break

                for (_, kindId, contactTypeId) in rule.notificationKindList:
                    addrList = self.getClientContact(clientId, contactTypeId)

                    for addr in addrList:
                        self.createNotificationLogRecord(
                            ruleId, kindId, text, addr, clientId)


    def getClientContact(self, clientId, contactTypeId):
        u"""Возвращает список контактов пациента по типу, или пустой список"""
        result = []

        cond = [
            self._tblClientContact['client_id'].eq(clientId),
            self._tblClientContact['contactType_id'].eq(contactTypeId),
            self._tblClientContact['deleted'].eq(0)
        ]

        recordList = self._db.getRecordList(self._tblClientContact, 'contact',
                                            cond)

        for record in recordList:
            result.append(forceString(record.value(0)))

        return result


    def createNotificationLogRecord(self, ruleId, kindId, text, addr, clientId):
        u"""Создает запись в журнале оповещений, возвращает ее идентификатор"""
        record = self._tblNotificationLog.newRecord()
        record.setValue('createPerson_id', toVariant(QtGui.qApp.userId))
        record.setValue('createDatetime',
                        toVariant(QDateTime.currentDateTime()))
        record.setValue('rule_id', toVariant(ruleId))
        record.setValue('kind_id', toVariant(kindId))
        record.setValue('addr', toVariant(addr))
        record.setValue('text', toVariant(text))
        record.setValue('recipient', toVariant(clientId))
        record.setValue('sendDatetime', toVariant(QDateTime.currentDateTime()))
        return self._db.insertRecord(self._tblNotificationLog, record)


    def logNotification(self, logId, result, errorStr):
        u"""Записывает результат оповещения в журнал"""
        record = self._db.getRecord(self._tblNotificationLog, '*', logId)
        record.setValue('lastErrorText', toVariant(
            strSuccess if result else errorStr))
        self._db.updateRecord(self._tblNotificationLog, record)


    def sendNotification(self, notificationKindCode, text, addr, externalId):
        u"""Отправляет оповещение"""
        result = False
        errorStr = ''

        if notificationKindCode == kindCodeSMS:
            (result, errorStr) = self._sendSMSNotification(
                text, addr, externalId)
        elif notificationKindCode == kindCodeEmail:
            (result, errorStr) = sendEmailNotification(text, addr)
        elif notificationKindCode == kindCodeViber:
            (result, errorStr) = self._sendViberNotification(text, addr,
                                                             externalId)
        else:
            errorStr = (u'Неподдерживаемый вид оповещения %s' %
                        notificationKindCode)

        return result, errorStr


    def _sendViberNotification(self, text, addr, externalId):
        result = False
        errorStr = ''
        url = 'https://api.hub.imobis.ru/v3/message/send'
        headers = ({'Authorization' : 'Token {0}'.format(self.__apiKey)}
                   if self.__apiKey else {})
        phone = filter(lambda ch: ch not in '+()-', addr)
        postData = [{
            'custom_id' : externalId,
            'channel': 'viber',
            'phone': phone,
            'sender': self.__viberSender,
            'text': text,
            'ttl': 30
        }, {
            'custom_id' : externalId,
            'channel': 'sms',
            'phone': phone,
            'sender': self.__smsSender,
            'text': text
        }]

        try:
            rval = readUrl(url, postData=postData, jsonData=True,
                           headers=headers)

            if rval.get('result', '') == 'ok':
                result = True
            else:
                errorStr = rval.get('error', {}).get('desc', strUnknownError)

        except HTTPError, error:
            eDict = loadJson(error.read())
            if isinstance(eDict, dict):
                errorStr = eDict.get('error', {}).get('desc', strUnknownError)
            else:
                errorStr = strUnknownError
        except IOError, error:
            QtGui.qApp.logCurrentException()
            errorStr = u'Произошла ошибка %s' % unicode(error)

        return (result, errorStr)



    def _sendSMSNotification(self, text, addr, externalId):
        u"""Отправляет СМС оповещение через Imobis"""
        result = False
        errorStr = ''
        url = 'http://gate.imobis.ru/send'

        getData = {
            'user': self.__smsUser,
            'password': self.__smsPassword,
            'messageId': forceString(externalId),
            'phone':  filter(lambda ch: ch not in '+()-', addr),
            'sender': self.__smsSender,
            'text': text,
            'smsTTL' : 300,
        }

        try:
            rval = forceInt(readUrl(url, getData))

            if rval > 0:
                result = True
            else:
                errorStr = self.mapSmsErrorCode.get(-rval, u'Неизвестный код '
                                                    u'ошибки %d' %  rval)

        except IOError, error:
            QtGui.qApp.logCurrentException()
            errorStr = u'Произошла ошибка %s' % unicode(error)

        return (result, errorStr)


    def _getSMSConfirmation(self, externalId):
        u"""Получает код подтверждения для SMS"""
        result = False
        url = 'http://gate.imobis.ru/callback'

        getData = {
            'user': self.__smsUser,
            'password': self.__smsPassword,
            'userMessageId': externalId,
            'mode' : 'brief',
        }

        try:
            rval = forceString(readUrl(url, getData))

            if rval == 'DELIVRD':
                result = True

        except IOError:
            QtGui.qApp.logCurrentException()

        return result


    def _getSuccessfullLogRecords(self, ruleId, kindId):
        u"""Выдает список успешно отправленных уведомлений"""
        cond = [
            self._tblNotificationLog['rule_id'].eq(ruleId),
            self._tblNotificationLog['kind_id'].eq(kindId),
            self._tblNotificationLog['lastErrorText'].eq(strSuccess),
            self._tblNotificationLog['confirmationDatetime'].isNull(),
        ]

        return self._db.getIdList(self._tblNotificationLog, where=cond)


    def processNotificationConfirmation(self, rule):
        u"""Обработка подтверждений отправки, реализовано для SMS"""
        for (kindCode, kindId, _) in rule.notificationKindList:
            if kindCode == kindCodeSMS:
                idList = self._getSuccessfullLogRecords(rule.id, kindId)

                for _id in idList:
                    if self._getSMSConfirmation(_id):
                        self._writeConfirmation(_id)


    def _writeConfirmation(self, logId):
        u"""Устанавливает время подтверждения отправки"""
        record = self._db.getRecord(self._tblNotificationLog,
                                    'id, confirmationDatetime', logId)
        record.setValue('confirmationDatetime', toVariant(
            QDateTime.currentDateTime()))
        self._db.updateRecord(self._tblNotificationLog, record)



def sendEmailNotification(text, addr):
    u"""Отправляет оповещение по электронной почте"""
    return sendMailInt(addr, u'Оповещение', text, [])


def parseNotificationTemplate(template, clientId, scheduleItem=None, msg=''):
    u"""Обрабатывает шаблон уведомления"""
    result = None

    try:
        context = CInfoContext()
        client = context.getInstance(CClientInfo, clientId)

        data = {
            'client': client,
            'message': msg,
            'currentOrganisation': context.getInstance(COrgInfo,
                                                       QtGui.qApp.currentOrgId()),
            'currentOrgStructure': context.getInstance(COrgStructureInfo,
                                                       QtGui.qApp.currentOrgStructureId()),
        }

        if isinstance(scheduleItem, tuple):
            (item, schedule) = scheduleItem

            if schedule:
                personId = schedule.personId
                person = context.getInstance(CPersonInfo, personId)
                data['office'] = schedule.office
                data['person'] = person
                activityId = schedule.activityId

                if activityId:
                    activity = context.getInstance(CActivityInfo, activityId)
                    data['activity'] = activity

            data['time'] = forceString(item.time) if item else '-'
        elif scheduleItem:
            personId = forceRef(scheduleItem.value('person_id'))
            person = context.getInstance(CPersonInfo, personId)
            data['office'] = forceString(scheduleItem.value('office'))
            data['person'] = person
            data['time'] = forceString(scheduleItem.value('time'))

        (result, _) = compileAndExecTemplate(template, data)
        # убираем escape-символы html:
        doc = QtGui.QTextDocument()
        doc.setHtml(result)
        result = forceString(doc.toPlainText())

    except Exception:
        QtGui.qApp.logCurrentException()

    return result

if __name__ == '__main__':
    import urllib2
    handler= urllib2.HTTPSHandler(debuglevel=1)
    opener = urllib2.build_opener(handler)
    urllib2.install_opener(opener)
    manager = CNotificationManager()
    (_result, erroText) = manager.sendNotification(kindCodeViber, 'test',
                                                  '+79000000000', 1)
    print u'result={0}, {1}'.format(_result, erroText)
