# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import re

import sip
from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, QVariant, pyqtSignature, SIGNAL

from library.IdentificationModel import CIdentificationModel, checkIdentification
from library.InDocTable import CInDocTableModel, CBoolInDocTableCol, CInDocTableCol, CRBInDocTableCol
from library.interchange import getCheckBoxValue, getComboBoxValue, getLineEditValue, getRBComboBoxValue, getTextEditValue, setCheckBoxValue, setComboBoxValue, setLineEditValue, setRBComboBoxValue, setTextEditValue
from library.ItemsListDialog import CItemEditorBaseDialog
from library.ItemsListDialog import CItemsListDialog
from library.TableModel import CTextCol
from library.Utils import addDotsEx, forceRef, forceString, forceStringEx, toVariant

from RefBooks.Tables import rbNet, rbOKFS, rbOKPF

from Banks import CBanksList
from Utils import fixOKVED, getShortBankName, parseOKVEDList


from Ui_OrgEditor import Ui_OrgEditorDialog
from Ui_OrgFilterDialog   import Ui_OrgFilterDialog


def selectOrganisation(parent, orgId, showListFirst, filter=None, hospitalBedProfileId=None):
    if showListFirst:
        try:
            dialog = COrgsList(parent)
            dialog.filter = filter
            dialog.setItemId(orgId)
            if dialog.exec_():
                return dialog.currentItemId()
        finally:
            sip.delete(dialog)
    else:
        filterDialog = COrgFilterDialog(parent)
        filterDialog.setProps({'hospitalBedProfileId':hospitalBedProfileId})
        try:
            if filterDialog.exec_():
                dialog = COrgsList(parent, True)
                dialog.filter = filter
                try:
                    dialog.props = filterDialog.props()
                    dialog.renewListAndSetTo(orgId)
                    if dialog.model.rowCount() == 1:
                        return dialog.currentItemId()
                    else:
                        if dialog.exec_():
                            return dialog.currentItemId()
                finally:
                    sip.delete(dialog)
        finally:
            sip.delete(filterDialog)
    return None


class COrgsList(CItemsListDialog):
    def __init__(self, parent, forSelect=False):
        CItemsListDialog.__init__(self, parent, [
            CTextCol(u'Краткое наименование',    ['shortName'], 30),
            CTextCol(u'ИНН',                     ['INN'],       12),
            CTextCol(u'ОГРН',                    ['OGRN'],      10),
            CTextCol(u'Код ТФОМС',               ['infisCode'], 10),
            CTextCol(u'Код ФФОМС',               ['smoCode'],   10),
            CTextCol(u'Код ОКАТО',               ['OKATO'],     10),
            CTextCol(u'Полное наименование',     ['fullName'],  30),
            ],
            'Organisation',
            ['shortName', 'INN', 'OGRN'],
            forSelect=forSelect,
            filterClass=COrgFilterDialog
            )
        self.setWindowTitleEx(u'Организации')
        self.filter = None
        self.tblItems.addPopupRecordProperies()


    def getItemEditor(self):
        return COrgEditor(self)


    def select(self, props):
        table = self.model.table()
        cond = []
        name = props.get('name', '')
        if name:
            nameFilter = []
            dotedName = addDotsEx(name)
            nameFilter.append(table['shortName'].like(dotedName))
            nameFilter.append(table['fullName'].like(dotedName))
            nameFilter.append(table['title'].like(dotedName))
            cond.append(QtGui.qApp.db.joinOr(nameFilter))

        inn = props.get('INN', '')
        if inn:
            cond.append(table['INN'].eq(inn))
        ogrn = props.get('OGRN', '')
        if ogrn:
            cond.append(table['OGRN'].eq(ogrn))
        okato = props.get('OKATO', '')
        if okato:
            cond.append(table['OKATO'].eq(okato))
        tfomsCode = props.get('tfomsCode', '')
        if tfomsCode:
            cond.append(table['infisCode'].eq(tfomsCode))
        okved = props.get('OKVED', '')
        if okved:
            cond.append(table['OKVED'].like(okved))
        isInsurer = props.get('isInsurer', 0)
        if isInsurer:
            cond.append(table['isInsurer'].eq(isInsurer-1))
        isActive = props.get('isActive', 0)
        if isActive:
            cond.append(table['isActive'].eq(isActive-1))
        hospitalBedProfileId = props.get('hospitalBedProfileId')
        if hospitalBedProfileId:
            # это можно, и наверное, нужно переделать на LEFT JOIN
            # но на первый взгляд и так прокатит
            cond.append('EXISTS (SELECT 1 FROM Organisation_HospitalBedProfile WHERE master_id=Organisation.id and hospitalBedProfile_id=%d)' % hospitalBedProfileId)
        isMedical = props.get('isMedical', 0)
        if isMedical:
            cond.append(table['isMedical'].eq(isMedical))
        if self.filter:
            cond.append(self.filter)
        return QtGui.qApp.db.getIdList(table.name(), 'id', cond, self.order)


    def updateOrgsList(self, itemId):
        self.filter = None
        idList = self.select(self.props)
        self.model.setIdList(idList, itemId)
        if idList:
            self.tblItems.selectRow(0)
        self.setCurrentItemId(itemId)
        self.label.setText(u'всего: %d' % len(idList))
        self.btnSelect.setFocus(Qt.OtherFocusReason)
        self.props = {}


    @pyqtSignature('')
    def on_btnSelect_clicked(self):
        self.renewListAndSetTo(self.currentItemId())
        self.accept()


    @pyqtSignature('')
    def on_btnNew_clicked(self):
        dialog = self.getItemEditor()
        name = self.props.get('name', '')
        if name:
            words = forceStringEx(name).split(u'...')
            if len(words) == 1 and (u'...' not in words[0].lower()):
                name = words[0]
                dialog.edtFullName.setText(name)
                dialog.edtShortName.setText(name)
                dialog.edtTitle.setText(name)
        try:
            if dialog.exec_():
                itemId = dialog.itemId()
                self.updateOrgsList(itemId)
        finally:
            sip.delete(dialog)


    @pyqtSignature('')
    def on_btnEdit_clicked(self):
        itemId = self.currentItemId()
        if itemId:
            dialog = self.getItemEditor()
            try:
                dialog.load(itemId)
                if dialog.exec_():
                    self.updateOrgsList(itemId)
            finally:
                sip.delete(dialog)
        else:
            self.on_btnNew_clicked()

#
# ##########################################################################
#

class COrgFilterDialog(QtGui.QDialog, Ui_OrgFilterDialog):
    def __init__(self,  parent):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.edtName.setFocus(Qt.ShortcutFocusReason)
        self.cmbHospitalBedProfile.setTable('rbHospitalBedProfile', addNone=True)
        self.cmbActive.setCurrentIndex(2)
#        self.cmbHospitalBedProfile.setValue(None)


    def setProps(self,  props):
        self.edtName.setText(props.get('name', ''))
        self.edtINN.setText(props.get('INN',   ''))
        self.edtOGRN.setText(props.get('OGRN', ''))
        self.edtOKATO.setText(props.get('OKATO', ''))
        self.edtTfomsCode.setText(props.get('tfomsCode', ''))
        self.edtOKVED.setText(props.get('OKVED', ''))
        self.cmbIsInsurer.setCurrentIndex(props.get('isInsurer', 0))
        self.cmbHospitalBedProfile.setValue(props.get('hospitalBedProfileId'))
        self.cmbIsMedical.setCurrentIndex(props.get('isMedical', 0))
        self.cmbActive.setCurrentIndex(props.get('isActive', 2))


    def props(self):
        result = {}
        result['name'] = forceStringEx(self.edtName.text())
        result['INN'] = forceStringEx(self.edtINN.text())
        result['OGRN'] = forceStringEx(self.edtOGRN.text())
        result['OKATO'] = forceStringEx(self.edtOKATO.text())
        result['tfomsCode'] = forceStringEx(self.edtTfomsCode.text())
        result['OKVED'] = forceStringEx(self.edtOKVED.text())
        result['isInsurer'] = self.cmbIsInsurer.currentIndex()
        result['hospitalBedProfileId'] = self.cmbHospitalBedProfile.value()
        result['isMedical'] = self.cmbIsMedical.currentIndex()
        result['isActive'] = self.cmbActive.currentIndex()
        return result

#
# ##########################################################################
#

class COrgEditor(CItemEditorBaseDialog, Ui_OrgEditorDialog):
    def __init__(self,  parent):
        CItemEditorBaseDialog.__init__(self, parent, 'Organisation')
        self.addModels('OrganisationAccounts', COrganisationAccountsModel(self))
        self.addModels('PolicySerials', CPolicySerialsModel(self))
        self.addModels('HospitalBedProfiles', CHospitalBedProfilesModel(self))
        self.addModels('Identification', CIdentificationModel(self, 'Organisation_Identification', 'Organisation'))

        self.setupUi(self)
        self.tblOrganisationAccounts.addPopupDelRow()
        self.tabWidget.setTabEnabled(2, False)
        self.setWindowTitleEx(u'Организация')

        self.setModels(self.tblOrganisationAccounts, self.modelOrganisationAccounts, self.selectionModelOrganisationAccounts)
        self.setModels(self.tblPolicySerials,        self.modelPolicySerials,        self.selectionModelPolicySerials)
        self.setModels(self.tblHospitalBedProfiles,  self.modelHospitalBedProfiles,  self.selectionModelHospitalBedProfiles)
        self.setModels(self.tblIdentification,       self.modelIdentification,       self.selectionModelIdentification)

        self.tblPolicySerials.addPopupDelRow()
        self.tblHospitalBedProfiles.addPopupDelRow()
        self.tblIdentification.addPopupDelRow()

        self.cmbNet.setTable(rbNet, True)
        self.cmbOKPF.setTable(rbOKPF, True)
        self.cmbOKFS.setTable(rbOKFS, True)
        self.cmbArea.setAreaSelectable(True)
        self.setupDirtyCather()


    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        setLineEditValue(self.edtFullName,          record, 'fullName')
        setLineEditValue(self.edtShortName,         record, 'shortName')
        setLineEditValue(self.edtTitle,             record, 'title')
        setLineEditValue(self.edtAddress,           record, 'address')
        setRBComboBoxValue(self.cmbHead,            record, 'head_id')

        setLineEditValue(self.edtOGRN,              record, 'OGRN')
        setLineEditValue(self.edtINN,               record, 'INN')
        setLineEditValue(self.edtKPP,               record, 'KPP')
        setLineEditValue(self.edtFSS,               record, 'FSS')
        setLineEditValue(self.edtOKVED,             record, 'OKVED')
        setLineEditValue(self.edtOKATO,             record, 'OKATO')
        setRBComboBoxValue(self.cmbOKPF,            record, 'OKPF_id')
        setRBComboBoxValue(self.cmbOKFS,            record, 'OKFS_id')
        setLineEditValue(self.edtOKPO,              record, 'OKPO')

        setLineEditValue(self.edtChief,             record, 'chief')
        setLineEditValue(self.edtAccountant,        record, 'accountant')
        setLineEditValue(self.edtPhone,             record, 'phone')

        setLineEditValue(self.edtTfomsCode,         record, 'infisCode')
        setLineEditValue(self.edtTfomsExtCode,      record, 'tfomsExtCode')
#        setLineEditValue(self.edtObsoleteInfisCode, record, 'obsoleteInfisCode')
        setLineEditValue(self.edtMiacCode,          record, 'miacCode')
        setLineEditValue(self.edtSmoCode,           record, 'smoCode')
        setLineEditValue(self.edtUsishCode,         record, 'usishCode')
        setComboBoxValue(self.cmbIsMedical,         record, 'isMedical')
        setRBComboBoxValue(self.cmbNet,             record, 'net_id')

        setCheckBoxValue(self.chkIsInsurer,         record, 'isInsurer')
        setCheckBoxValue(self.chkIsActive,       record, 'isActive')
        self.cmbArea.setCode(forceString(record.value('area')))
        setCheckBoxValue(self.chkCompulsoryServiceStop, record, 'compulsoryServiceStop')
        setCheckBoxValue(self.chkVoluntaryServiceStop,  record, 'voluntaryServiceStop')
        setCheckBoxValue(self.chkIsSupplier, record, 'isSupplier')

        setTextEditValue(self.edtNotes,             record, 'notes')

        self.modelOrganisationAccounts.loadItems(self.itemId())
        self.modelPolicySerials.loadItems(self.itemId())
        self.modelHospitalBedProfiles.loadItems(self.itemId())
        self.modelIdentification.loadItems(self.itemId())


    def getRecord(self):
#        obsoleteInfisCode = forceString(self.edtObsoleteInfisCode.text())
#        obsoleteInfisCode = trim(obsoleteInfisCode.replace(',', ' ')).replace(' ', ',')

        record = CItemEditorBaseDialog.getRecord(self)
        getLineEditValue(self.edtFullName,          record, 'fullName')
        getLineEditValue(self.edtShortName,         record, 'shortName')
        getLineEditValue(self.edtTitle,             record, 'title')
        getLineEditValue(self.edtAddress,           record, 'address')
        getRBComboBoxValue(self.cmbHead,            record, 'head_id')

        getLineEditValue(self.edtOGRN,              record, 'OGRN')
        getLineEditValue(self.edtINN,               record, 'INN')
        getLineEditValue(self.edtKPP,               record, 'KPP')
        getLineEditValue(self.edtFSS,               record, 'FSS')
        getLineEditValue(self.edtOKVED,             record, 'OKVED')
        getLineEditValue(self.edtOKATO,             record, 'OKATO')
        getRBComboBoxValue(self.cmbOKPF,            record, 'OKPF_id')
        getRBComboBoxValue(self.cmbOKFS,            record, 'OKFS_id')
        getLineEditValue(self.edtOKPO,              record, 'OKPO')

        getLineEditValue(self.edtChief,             record, 'chief')
        getLineEditValue(self.edtAccountant,        record, 'accountant')
        getLineEditValue(self.edtPhone,             record, 'phone')

        getLineEditValue(self.edtTfomsCode,         record, 'infisCode')
        getLineEditValue(self.edtTfomsExtCode,      record, 'tfomsExtCode')
#        record.setValue('obsoleteInfisCode',        toVariant(obsoleteInfisCode))
        getLineEditValue(self.edtMiacCode,          record, 'miacCode')

        getLineEditValue(self.edtSmoCode,           record, 'smoCode')
        getLineEditValue(self.edtUsishCode,         record, 'usishCode')

        getComboBoxValue(self.cmbIsMedical,         record, 'isMedical')
        getRBComboBoxValue(self.cmbNet,             record, 'net_id')

        getCheckBoxValue(self.chkIsInsurer,         record, 'isInsurer')
        getCheckBoxValue(self.chkIsActive,       record, 'isActive')
        area = self.cmbArea.code() if self.chkIsInsurer.isChecked() else ''
        record.setValue('area', toVariant(area))

        getCheckBoxValue(self.chkCompulsoryServiceStop, record, 'compulsoryServiceStop')
        getCheckBoxValue(self.chkVoluntaryServiceStop,  record, 'voluntaryServiceStop')
        getCheckBoxValue(self.chkIsSupplier, record, 'isSupplier')

        getTextEditValue(self.edtNotes,             record, 'notes')
        return record


    def saveInternals(self, id):
        self.modelOrganisationAccounts.saveItems(id)
        self.modelPolicySerials.saveItems(id)
        self.modelHospitalBedProfiles.saveItems(id)
        self.modelIdentification.saveItems(id)


    def checkDataEntered(self):
        result = True
        message = u'Необходимо указать '
        if not forceStringEx(self.edtFullName.text()):
            res = self.checkValueMessageIgnoreAll(message + u'полное наименование', False, self.edtFullName)
            if res == 0:
                return False
            elif res == 2:
                return True
        if not forceStringEx(self.edtShortName.text()):
            res = self.checkValueMessageIgnoreAll(message + u'краткое наименование', False, self.edtShortName)
            if res == 0:
                return False
            elif res == 2:
                return True
        if not forceStringEx(self.edtOGRN.text()):
            res = self.checkValueMessageIgnoreAll(message + u'ОГРН', True, self.edtOGRN)
            if res == 0:
                return False
            elif res == 2:
                return True
        headOrgId = self.cmbHead.value()
        if not headOrgId:
            INN = forceStringEx(self.edtINN.text())
            if not INN:
                res = self.checkValueMessageIgnoreAll(message + u'ИНН', True, self.edtINN)
                if res == 0:
                    return False
                elif res == 2:
                    return True
            if INN and not self.checkINN(INN):
                res = self.checkValueMessageIgnoreAll(message + u'правильный ИНН', True, self.edtINN)
                if res == 0:
                    return False
                elif res == 2:
                    return True
            if not forceStringEx(self.edtKPP.text()):
                res = self.checkValueMessageIgnoreAll(message + u'КПП', True, self.edtKPP)
                if res == 0:
                    return False
                elif res == 2:
                    return True
            if not self.cmbOKPF.value():
                res = self.checkValueMessageIgnoreAll(message + u'ОКПФ', True, self.cmbOKPF)
                if res == 0:
                    return False
                elif res == 2:
                    return True
            if not self.cmbOKFS.value():
                res = self.checkValueMessageIgnoreAll(message + u'ОКФС', True, self.cmbOKFS)
                if res == 0:
                    return False
                elif res == 2:
                    return True
#        result = result and (forceStringEx(self.edtTitle.text()) or  self.checkValueMessageIgnoreAll(message + u'наименование для печати', False, self.edtTitle))
        if not forceStringEx(self.edtOKVED.text()):
            res =  self.checkValueMessageIgnoreAll(message + u'ОКВЭД', True, self.edtOKVED)
            if res == 0:
                return False
            elif res == 2:
                return True
        result = result and self.checkOKVED()
        if result:
            for row, record in enumerate(self.modelOrganisationAccounts.items()):
                if not self.checkAccountDataEntered(row, record):
                    return False
        result = result and self.checkDup()
        result = result and checkIdentification(self, self.tblIdentification)
        return result


    def checkINN(self, INN):
        def checksum(n, s):
            result = sum( d*int(c) for (d,c) in zip(n,s) ) % 11
            if result == 10:
                result = 0
            return result

        n121 = (7, 2, 4, 10, 3,  5, 9, 4, 6, 8)
        n122 = (3, 7, 2,  4, 10, 3, 5, 9, 4, 6, 8)
        n10  = (2, 4, 10, 3, 5, 9, 4, 6, 8)

        if len(INN) == 10:
            return str(checksum(n10, INN[0:9])) == INN[9]
        elif len(INN) == 12:
            return str(checksum(n121, INN[0:11]))+str(checksum(n122, INN[0:11])) == INN[10:12]
        else:
            return False


    def checkOKVED(self):
        fixedCodes = []
        str = forceString(self.edtOKVED.text())
        list = parseOKVEDList(str)
        for code in list:
            success, fixedCode = fixOKVED(code)
            fixedCodes.append(fixedCode)
        self.edtOKVED.setText(u','.join(fixedCodes))
        return True


    def checkAccountDataEntered(self, row, record):
        result = True
        column = record.indexOf('name')
        name = forceString(record.value(column))
        if result and not re.match(r'^\d{20}$', name):
            result = self.checkValueMessage(u'Номер расчетного счета должен состоять из 20 цифр', True, self.tblOrganisationAccounts, row, column)

        column = record.indexOf('bank_id')
        bankId = forceRef(record.value(column))
        if result and not bankId:
            result = self.checkValueMessage(u'Необходимо указать банк', True, self.tblOrganisationAccounts, row, column)
        return result


    def checkDup(self):
        dupCheckList = ((self.findDupByTfomsCode,  u'по коду ТФОМС'),
                        (self.findDupByINN,  u'по ИНН'),
                        (self.findDupByOGRN, u'по ОГРН'),
                       )
        for method, title in dupCheckList:
            idlist = method()
            if idlist:
                res = QtGui.QMessageBox.question( self,
                                                 u'Внимание!',
                                                 u'Обнаружен "двойник" %s\nИгнорировать?' % title,
                                                 QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                                                 QtGui.QMessageBox.No)
                if res == QtGui.QMessageBox.No:
                    return False
        return True


    def findDup(self, cond):
        db = QtGui.qApp.db
        table = db.table('Organisation')
        id = self.itemId()
        if id:
            cond.append(table['id'].ne(id))
        return db.getIdList(table, where=cond, order='id')


    def findDupByTfomsCode(self):
        tfomsCode  = forceStringEx(self.edtTfomsCode.text())
        if tfomsCode:
            db = QtGui.qApp.db
            table = db.table('Organisation')
            cond = [table['infisCode'].eq(tfomsCode)]
            return self.findDup(cond)
        else:
            return None


    def findDupByINN(self):
        INN  = forceStringEx(self.edtINN.text())
        if not self.cmbHead.value() and INN:
            db = QtGui.qApp.db
            table = db.table('Organisation')
            cond = [table['INN'].eq(INN)]
            return self.findDup(cond)
        else:
            return None


    def findDupByOGRN(self):
        OGRN  = forceStringEx(self.edtOGRN.text())
        if not self.cmbHead.value() and OGRN:
            db = QtGui.qApp.db
            table = db.table('Organisation')
            cond = [table['OGRN'].eq(OGRN)]
            return self.findDup(cond)
        else:
            return None


    def updateInfoByHead(self, orgId):
        if orgId or self.cmbHead.value():
            db = QtGui.qApp.db
            record = db.getRecord(db.table('Organisation'), '*', orgId)
            if not record:
                record = QtSql.QSqlRecord()
            setLineEditValue(self.edtINN,    record, 'INN')
            setLineEditValue(self.edtKPP,    record, 'KPP')
            setLineEditValue(self.edtOGRN,   record, 'OGRN')
            setLineEditValue(self.edtFSS,    record, 'FSS')
            setLineEditValue(self.edtOKATO,  record, 'OKATO')
            setLineEditValue(self.edtOKPO,   record, 'OKPO')
            setRBComboBoxValue(self.cmbOKPF, record, 'OKPF_id')
            setRBComboBoxValue(self.cmbOKFS, record, 'OKFS_id')
        headIsEmpty = not orgId
        self.edtINN.setEnabled(headIsEmpty)
        self.edtKPP.setEnabled(headIsEmpty)
        self.edtOGRN.setEnabled(headIsEmpty)
        self.edtFSS.setEnabled(headIsEmpty)
        self.edtOKATO.setEnabled(headIsEmpty)
        self.edtOKPO.setEnabled(headIsEmpty)
        self.cmbOKPF.setEnabled(headIsEmpty)
        self.cmbOKFS.setEnabled(headIsEmpty)


    @pyqtSignature('')
    def on_btnSelectHeadOrganisation_clicked(self):
        headOrgId = selectOrganisation(self, self.cmbHead.value(), False)
        self.cmbHead.updateModel()
        if headOrgId:
            self.cmbHead.setValue(headOrgId)
#            self.updateInfoByHead(orgId)


    @pyqtSignature('int')
    def on_cmbHead_currentIndexChanged(self, index):
        headOrgId = self.cmbHead.value()
        self.updateInfoByHead(headOrgId)


    @pyqtSignature('int')
    def on_cmbIsMedical_currentIndexChanged(self, index):
        self.cmbNet.setEnabled(bool(index))
        self.lblNet.setEnabled(bool(index))


    @pyqtSignature('bool')
    def on_chkIsInsurer_toggled(self, checked):
        self.tabWidget.setTabEnabled(2, checked)



class COrganisationAccountsModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'Organisation_Account', 'id', 'organisation_id', parent)
        self.addCol(CInDocTableCol(     u'Расчетный счет',  'name',    22, maxLength=20, inputMask='9'*20))
        self.addCol(CInDocTableCol(     u'Лицевой счет',  'personalAccount',    22, maxLength=20))
        self.addCol(CBankInDocTableCol( u'Банк',            'bank_id', 22))
        self.addCol(CInDocTableCol(     u'Наименование в банке',  'bankName', 22))
        self.addCol(CInDocTableCol(     u'Примечания',      'notes',   22))
        self.addCol(CBoolInDocTableCol( u'Нал',             'cash',    4))



class CBankInDocTableCol(CInDocTableCol):
    def __init__(self, title, fieldName, width):
        CInDocTableCol.__init__(self, title, fieldName, width)


    def toString(self, value, record):
        bankId = forceRef(value)
        return QVariant(getShortBankName(bankId))


    def createEditor(self, parent):
        editor = CBankSelectButton(parent)
        return editor


    def setEditorData(self, editor, value, record):
        editor.setBankId(forceRef(value))


    def getEditorData(self, editor):
        return toVariant(editor.bankId)



class CBankSelectButton(QtGui.QPushButton):
    def __init__(self,  parent):
        QtGui.QPushButton.__init__(self, parent)
        self.setFocusPolicy(Qt.StrongFocus)
        self.bankId = None
        self.connect(self, SIGNAL('clicked()'), self.selectBank)


    def setBankId(self,  bankId):
        if bankId != self.bankId:
            self.bankId = bankId
            self.setText(getShortBankName(self.bankId))


    @pyqtSignature('')
    def selectBank(self):
        dlg = CBanksList(self, True)
        dlg.setCurrentItemId(self.bankId)
        if dlg.selectItem():
            self.setBankId(dlg.currentItemId())
        sip.delete(dlg)



class CPolicySerialsModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'Organisation_PolicySerial', 'id', 'organisation_id', parent)
        self.addCol(CRBInDocTableCol(u'Тип', 'policyType_id', 30, 'rbPolicyType'))
        self.addCol(CInDocTableCol(     u'Серия',  'serial',  16, maxLength=8))


class CHospitalBedProfilesModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'Organisation_HospitalBedProfile', 'id', 'master_id', parent)
        self.addCol(CRBInDocTableCol(u'Профиль', 'hospitalBedProfile_id', 30, 'rbHospitalBedProfile'))
