# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui

from library.PrintInfo import CInfo, CInfoList, CDateInfo, CRBInfo
from Events.Service import CServiceInfo
from Utils import COrgInfo, COrgStructureInfo, CActivityInfo
from library.Utils import forceDate, forceInt, forceRef, forceString, formatNameInt, formatShortNameInt


class CPersonInfo(CInfo):
    def __init__(self, context, personId):
        CInfo.__init__(self, context)
        self.personId = personId
        self._code = ''
        self._federalCode = ''
        self._regionalCode = ''
        self._lastName = ''
        self._firstName = ''
        self._patrName = ''
        self._office = ''
        self._office2 = ''
        self._organisation   = self.getInstance(COrgInfo, None)
        self._orgStructure   = self.getInstance(COrgStructureInfo, None)
        self._speciality     = self.getInstance(CSpecialityInfo, None)
        self._post           = self.getInstance(CPostInfo, None)
        self._tariffCategory = self.getInstance(CTariffCategoryInfo, None)
        self._academicDegree = ''
        self._login = ''
        self._orders = None
        self._educations = None

    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('Person', '*', self.personId)
        if record:
            self._code = forceString(record.value('code'))
            self._federalCode = forceString(record.value('federalCode'))
            self._regionalCode = forceString(record.value('regionalCode'))
            self._lastName = forceString(record.value('lastName'))
            self._firstName = forceString(record.value('firstName'))
            self._patrName = forceString(record.value('patrName'))
            self._office = forceString(record.value('office'))
            self._office2 = forceString(record.value('office2'))

            self._organisation = self.getInstance(COrgInfo, forceRef(record.value('org_id')))
            self._orgStructure = self.getInstance(COrgStructureInfo, forceRef(record.value('orgStructure_id')))
            self._speciality = self.getInstance(CSpecialityInfo, forceRef(record.value('speciality_id')))
            self._post = self.getInstance(CPostInfo, forceRef(record.value('post_id')))
            self._tariffCategory = self.getInstance(CTariffCategoryInfo, forceRef(record.value('tariffCategory_id')))
            self._academicDegree = forceString(record.value('academicDegree'))
            self._login = forceString(record.value('login'))
            self._educations = self.getInstance(CEducationInfoList, forceRef(record.value('id')))
            self._activities = self.getInstance(CPersonActivitiesInfoList, forceRef(record.value('id')))
            return True
        else:
            return False


    def getShortName(self):
        self.load()
        return formatShortNameInt(self._lastName, self._firstName, self._patrName)


    def getFullName(self):
        self.load()
        return formatNameInt(self._lastName, self._firstName, self._patrName)


    def getOrders(self):
        self.load()
        if self._orders is None:
            self._orders = self.getInstance(CPersonOrderInfoList, self.personId)
        return self._orders


#    def __unicode__(self):
    def __str__(self):
        self.load()
        result = formatShortNameInt(self._lastName, self._firstName, self._patrName)
        if self._speciality:
            result += ', '+self._speciality.name
        if self._academicDegree:
            result += ', '+self._academicDegree
        return unicode(result)

    code           = property(lambda self: self.load()._code)
    federalCode    = property(lambda self: self.load()._federalCode)
    regionalCode   = property(lambda self: self.load()._regionalCode)
    lastName       = property(lambda self: self.load()._lastName)
    firstName      = property(lambda self: self.load()._firstName)
    patrName       = property(lambda self: self.load()._patrName)
    fullName       = property(getFullName)
    longName       = property(getFullName)
    shortName      = property(getShortName)
    name           = property(getShortName)
    office         = property(lambda self: self.load()._office)
    office2        = property(lambda self: self.load()._office2)

    organisation   = property(lambda self: self.load()._organisation)
    orgStructure   = property(lambda self: self.load()._orgStructure)
    speciality     = property(lambda self: self.load()._speciality)
    post           = property(lambda self: self.load()._post)
    tariffCategory = property(lambda self: self.load()._tariffCategory)
    academicDegree = property(lambda self: self.load()._academicDegree)
    login          = property(lambda self: self.load()._login)
    orders         = property(getOrders)
    educations = property(lambda self: self.load()._educations)
    activities = property(lambda self: self.load()._activities)


class CPersonInfoList(CInfoList):
    def __init__(self, context, tableName, masterId):
        CInfoList.__init__(self, context)
        self.tableName = tableName
        self.masterId = masterId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table(self.tableName)
        if table.hasField('deleted'):
            cond = db.joinAnd([table['master_id'].eq(self.masterId), table['deleted'].eq(0) ])
        else:
            cond = db.joinAnd([table['master_id'].eq(self.masterId),])
        idList = db.getIdList(table, 'person_id', cond, 'person_id')
        self._items = [ self.getInstance(CPersonInfo, id) for id in idList ]


class CPersonActivitiesInfoList(CInfoList):
    def __init__(self, context, personId):
        CInfoList.__init__(self, context)
        self.personId = personId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Person_Activity')
        idList = db.getIdList(table, 'activity_id',
                              db.joinAnd([table['master_id'].eq(self.personId),
                                          table['deleted'].eq(0)
                                         ]),
                              'id')
        self._items = [ self.getInstance(CActivityInfo, id) for id in idList ]


class CPersonOrderInfoList(CInfoList):
    def __init__(self, context, personId):
        CInfoList.__init__(self, context)
        self.personId = personId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Person_Order')
        idList = db.getIdList(table, 'id',
                              db.joinAnd([table['master_id'].eq(self.personId),
                                          table['deleted'].eq(0)
                                         ]),
                              'id')
        self._items = [ self.getInstance(CPersonOrderInfo, id) for id in idList ]


class CPersonOrderInfo(CInfo):
    def __init__(self, context, orderId):
        CInfo.__init__(self, context)
        self.orderId = orderId
        self._date = CDateInfo()
        self._documentDate = CDateInfo()
        self._documentNumber = ''
        self._documentType = None
        self._validFromDate = CDateInfo()
        self._validToDate = CDateInfo()
        self._orgStructure = None
        self._post = None

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Person_Order')
        record = db.getRecord(table, '*', self.orderId)
        if record:
            self._date = CDateInfo(forceDate(record.value('date')))
            self._type = forceInt(record.value('type'))
            self._documentDate = CDateInfo(forceDate(record.value('documentDate')))
            self._documentNumber = forceString(record.value('documentNumber'))
            self._documentType = self.getInstance(CDocumentTypeInfo, forceRef(record.value('documentType_id')))
            self._validFromDate = CDateInfo(forceDate(record.value('validFromDate')))
            self._validToDate = CDateInfo(forceDate(record.value('validToDate')))
            self._orgStructure = self.getInstance(COrgStructureInfo, forceRef(record.value('orgStructure_id')))
            self._post = self.getInstance(CPostInfo, forceRef(record.value('post_id')))
            return True
        return False


    def getTypeAsString(self):
        self.load()
        return (u'Приём на работу', u'Увольнение', u'Назначение на должность', u'Отпуск', u'Учёба', u'Командировка')[self._type]

#    def __unicode__(self):
    def __str__(self):
        self.load()
        return u'%s %s' % (self._date + self.getTypeAsString())


    date = property(lambda self: self.load()._date)
    type = property(lambda self: self.load()._type)
    typeAsString = property(getTypeAsString)
    documentDate = property(lambda self: self.load()._documentDate)
    documentNumber = property(lambda self: self.load()._documentNumber)
    documentType = property(lambda self: self.load()._documentType)
    validFromDate = property(lambda self: self.load()._validFromDate)
    validToDate = property(lambda self: self.load()._validToDate)
    orgStructure = property(lambda self: self.load()._orgStructure)
    post = property(lambda self: self.load()._post)


class CEducationInfoList(CInfoList):
    def __init__(self, context, personId):
        CInfoList.__init__(self, context)
        self.personId = personId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Person_Education')
        idList = db.getIdList(table, 'id',
                              db.joinAnd([table['master_id'].eq(self.personId),
                                          table['deleted'].eq(0)
                                         ]),
                              'id')
        self._items = [ self.getInstance(CEducationInfo, id) for id in idList ]


class CEducationInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self._id = id
        self._documentType = self.getInstance(CDocumentTypeInfo, None)
        self._serial = ''
        self._number = ''
        self._date = CDateInfo()
        self._origin = ''
        self._status = ''
        self._validFromDate = CDateInfo()
        self._validToDate = CDateInfo()
        self._speciality = self.getInstance(CSpecialityInfo, None)

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Person_Education')
        record = db.getRecord(table, '*', self._id)
        if record:
            self._documentType = self.getInstance(CDocumentTypeInfo, forceRef(record.value('documentType_id')))
            self._serial = forceString(record.value('serial'))
            self._number = forceString(record.value('number'))
            self._date = CDateInfo(forceDate(record.value('date')))
            self._origin =  forceString(record.value('origin'))
            self._status = forceString(record.value('status'))
            self._validFromDate = CDateInfo(forceDate(record.value('validFromDate')))
            self._validToDate = CDateInfo(forceDate(record.value('validToDate')))
            self._speciality = self.getInstance(CSpecialityInfo, forceRef(record.value('speciality_id')))
            return True
        return False

    documentType  = property(lambda self: self.load()._documentType)
    serial  = property(lambda self: self.load()._serial)
    number  = property(lambda self: self.load()._number)
    date  = property(lambda self: self.load()._date)
    origin  = property(lambda self: self.load()._origin)
    status  = property(lambda self: self.load()._status)
    validFromDate  = property(lambda self: self.load()._validFromDate)
    validToDate  = property(lambda self: self.load()._validToDate)
    speciality  = property(lambda self: self.load()._speciality)
    

# NB! если CDocumentTypeInfo понадобится не только для врача, но и для пациента,
# то нужно будет создать подходящий модуль (что-нибуть типа Registry.CommonInfo) и перенести его туда.
class CDocumentTypeInfo(CRBInfo):
    tableName = 'rbDocumentType'


class CSpecialityInfo(CInfo):
    def __init__(self, context, specialityId):
        CInfo.__init__(self, context)
        self.specialityId = specialityId
        self._code = ''
        self._name = ''
        self._OKSOName = ''
        self._OKSOCode = ''
        self._service = self.getInstance(CServiceInfo, None)


    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('rbSpeciality', '*', self.specialityId)
        if record:
            self._code = forceString(record.value('code'))
            self._name = forceString(record.value('name'))
            self._OKSOName = forceString(record.value('OKSOName'))
            self._OKSOCode = forceString(record.value('OKSOCode'))
            self._service = self.getInstance(CServiceInfo, forceRef(record.value('service_id')))
            return True
        else:
            return False


    def __str__(self):
        return self.load()._name

    code     = property(lambda self: self.load()._code)
    name     = property(lambda self: self.load()._name)
    OKSOName = property(lambda self: self.load()._OKSOName)
    OKSOCode = property(lambda self: self.load()._OKSOCode)
    service  = property(lambda self: self.load()._service)


class CPostInfo(CInfo):
    def __init__(self, context, postId):
        CInfo.__init__(self, context)
        self.postId = postId
        self._code = ''
        self._name = ''
        self._regionalCode = ''


    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('rbPost', '*', self.postId)
        if record:
            self._code = forceString(record.value('code'))
            self._name = forceString(record.value('name'))
            self._regionalCode = forceString(record.value('regionalCode'))
            return True
        else:
            return False


    def __str__(self):
        return self.load()._name

    code     = property(lambda self: self.load()._code)
    name     = property(lambda self: self.load()._name)
    regionalCode = property(lambda self: self.load()._regionalCode)


class CTariffCategoryInfo(CRBInfo):
    tableName = 'rbTariffCategory'
