# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import re

from PyQt4 import QtGui
#from PyQt4.QtCore import *

from library.DbEntityCache import CDbEntityCache
from library.PrintInfo     import CInfo, CRBInfo
from library.AgeSelector   import parseAgeSelector, checkAgeSelector
from library.Utils         import forceBool, forceInt, forceRef, forceString, forceStringEx, formatNameInt, formatSex, formatShortNameInt, nameCase, trim

from KLADR.KLADRModel      import getKladrTreeModel

__all__ = [ 'CNet',
            'CNetInfo',
            'COKFSInfo',
            'COKPFInfo',
            'COrgInfo',
            'COrgStructure',
            'COrgStructureInfo',
            'CSexAgeConstraint',
            'advisePolicyType',
            'findOrgStructuresByAddress',
            'findOrgStructuresByHouseAndFlat',
            'fixOKVED',
            'getAccountInfo',
            'getActionTypeOrgStructureIdList',
            'getBankFilials',
            'getOKVEDName',
            'getOrganisationCluster',
            'getOrganisationDescendants',
            'getOrganisationInfo',
            'getOrganisationParent',
            'getOrganisationShortName',
            'getOrgStructureActionTypeIdSet',
            'getOrgStructureAddressIdList',
            'getOrgStructureDescendants',
            'getOrgStructureFullName',
            'getOrgStructureName',
            'getOrgStructureNetId',
            'getOrgStructurePersonIdList',
            'getOrgStructures',
            'getPersonChiefs',
            'getPersonInfo',
            'getPersonShortName',
            'getPostName',
            'getShortBankName',
            'getSolitaryOrgStructureId',
            'getSpecialityName',
            'parseOKVEDList',
          ]


def getOrganisationShortName(id):
    result = QtGui.qApp.db.translate('Organisation', 'id', id, 'shortName')
    if result:
        return forceString(result)
    else:
        return ''


def getOrganisationInfo(id):
    if id:
        db = QtGui.qApp.db
        table  = db.table('Organisation')
        record = db.getRecord(table, '*', id)
        if record:
            return dict(id        = id,
                        fullName  = forceString(record.value('fullName')),
                        shortName = forceString(record.value('shortName')),
                        title     = forceString(record.value('title')),
                        INN       = forceString(record.value('INN')),
                        KPP       = forceString(record.value('KPP')),
                        OGRN      = forceString(record.value('OGRN')),
                        OKVED     = forceString(record.value('OKVED'))
                       )
    return {}


def parseOKVEDList(str):
    return [ trim(s) for s in str.replace(',', ';').replace('|', ';').split(';') if s != '' ]


def fixOKVED(code):
    db = QtGui.qApp.db
    table  = db.table('rbOKVED')
    fixedCode = forceString(db.translate(table, 'code', code, 'code'))
    if fixedCode:
        return True, fixedCode
    partialCode = code
    while partialCode:
        fixedCode = forceString(db.translate(table, 'OKVED', partialCode, 'code'))
        if fixedCode:
            return True, fixedCode
        partialCode = partialCode[:-1]
    return False, code


def getOKVEDName(code):
    if code:
        db = QtGui.qApp.db
        table  = db.table('rbOKVED')
        records= db.getRecordList(table, 'name', table['code'].eq(code))
        if records:
            return forceString(records[0].value(0))
        else:
            return u'{%s}' % code
    else:
        return u''


def advisePolicyType(insurerId, serial):
    # подобрать тип полиса по с.к. и серии
    db = QtGui.qApp.db
    table = db.table('Organisation_PolicySerial')
    record = db.getRecordEx(table, 'policyType_id',
                            [table['organisation_id'].eq(insurerId), table['serial'].eq(serial)]
                           )
    if record:
        return forceRef(record.value(0))
    else:
        return None


def getShortBankName(bankId):
    if bankId:
        db = QtGui.qApp.db
        bankRecord = db.getRecord('Bank', 'BIK, name', bankId)
        if bankRecord:
            return forceString(bankRecord.value('BIK'))+ ' '+forceString(bankRecord.value('name'))
        else:
            return '{%d}' % bankId
    return ''


def getBankFilials(BIK):
    # Список филиалов банка, имеющего данный БИК
    db = QtGui.qApp.db
    table  = db.table('Bank')
    records = db.getRecordList(table, 'branchName', table['BIK'].eq(BIK))
    if records:
        return [record.value(0) for record in records]
    else:
        return []


def getAccountInfo(id):
    if id:
        db = QtGui.qApp.db
        table  = db.table('Organisation_Account')
        record = db.getRecord(table, ('name','notes','bank_id'), id)
        if record:
            bankId = forceRef(record.value('bank_id'))
            return dict(id      = id,
                        name    = forceString(record.value('name')),
                        notes   = forceString(record.value('notes')),
                        bank_id = bankId,
                        shortBankName = getShortBankName(bankId)
                       )
    return {}


def getOrganisationParent(orgId):
    # найти самый верхний элемент в иерархии организаций
    db = QtGui.qApp.db
    table = db.table('Organisation')

    while orgId:
        headId = forceRef(db.translate(table, 'id', orgId, 'head_id'))
        if headId:
            orgId = headId
    return orgId


def getOrganisationDescendants(orgId):
    # найти все организации явл. "потомками" данной
    return QtGui.qApp.db.getDescendants('Organisation', 'head_id', orgId)


def getOrganisationCluster(orgId):
    # найти все родственные организации
    return getOrganisationDescendants(getOrganisationParent(orgId))


def getPostName(postId):
    return forceString(QtGui.qApp.db.translate('rbPost', 'id', postId, 'name'))


def getSpecialityName(specialityId):
    return forceString(QtGui.qApp.db.translate('rbSpeciality', 'id', specialityId, 'name'))


def getTariffCategoryName(tariffCategoryId):
    return forceString(QtGui.qApp.db.translate('rbTariffCategory', 'id', tariffCategoryId, 'name'))


def getPersonInfo(personId):
    if personId:
        db = QtGui.qApp.db
        table  = db.table('Person')
        record = db.getRecord(table,
                              ['code',
                               'lastName',
                               'firstName',
                               'patrName',
                               'orgStructure_id',
                               'post_id',
                               'speciality_id',
                               'tariffCategory_id'
                              ],
                              personId)
        if record:
            code           = forceString(record.value('code'))
            lastName       = nameCase(forceString(record.value('lastName')))
            firstName      = nameCase(forceString(record.value('firstName')))
            patrName       = nameCase(forceString(record.value('patrName')))
            orgStructureId = forceRef(record.value('orgStructure_id'))
            postId         = forceRef(record.value('post_id'))
            specialityId   = forceRef(record.value('speciality_id'))
            tariffCategoryId = forceRef(record.value('tariffCategory_id'))
            return dict(id = personId,
                        code      = code,
                        lastName  = lastName,
                        firstName = firstName,
                        patrName  = patrName,
                        fullName  = formatNameInt(lastName, firstName, patrName),
                        shortName = formatShortNameInt(lastName, firstName, patrName),
                        postId    = postId,
                        postName  = getPostName(postId),
                        orgStructureId = orgStructureId,
                        orgStructureName = getOrgStructureName(orgStructureId),
                        specialityId   = specialityId,
                        specialityName = getSpecialityName(specialityId),
                        tariffCategoryId = tariffCategoryId,
                        tariffCategoryName = getTariffCategoryName(tariffCategoryId)
                       )
    return {}


def getPersonShortName(personId):
    if personId:
        db = QtGui.qApp.db
        table  = db.table('Person')
        record = db.getRecord(table, 'lastName,firstName,patrName', personId)
        if record:
            lastName     = nameCase(forceString(record.value('lastName')))
            firstName    = nameCase(forceString(record.value('firstName')))
            patrName     = nameCase(forceString(record.value('patrName')))
            return formatShortNameInt(lastName, firstName, patrName)
    return ''


def getPersonChiefs(personId):
    # Список начальников сотрудника
    result = []
    if personId:
        db = QtGui.qApp.db
        stmt = '''
        SELECT Chief.id
        FROM Person
        INNER JOIN rbPost ON rbPost.id = Person.post_id
        INNER JOIN Person AS Chief ON Chief.orgStructure_id = Chief.orgStructure_id
        INNER JOIN rbPost AS rbChiefPost ON rbChiefPost.id = Chief.post_id
        WHERE Person.id = %d AND SUBSTR(rbChiefPost.code, 1, 1)  in ('1', '2') AND SUBSTR(rbChiefPost.code, 1, 1) <  SUBSTR(rbPost.code, 1, 1)''' % personId
        query = db.query(stmt)
        while query.next():
            result.append( forceRef(query.record().value(0)) )
    return result


# пишу и плачу :(
def getNetIdList(clientSex, clientAge):
    db = QtGui.qApp.db
    recordList = db.getRecordList('rbNet', '*')
    result = []
    for record in recordList:
        net = CSexAgeConstraint()
        net.initByRecord(record)
        if net.applicable(clientSex, clientAge):
            result.append(forceRef(record.value('id')))
    return result


def findOrgStructuresByHouseAndFlat(houseId, flat, orgId, orgStructureId=None, clientSex=None, clientAge=None):
    if houseId:
        db = QtGui.qApp.db
        flatNum = 0
        flatNumFilter = re.compile(r'^(\d+)')
        flatNumPart = flatNumFilter.search(flat)
        if flatNumPart:
            flatNumText = flatNumPart.group(1)
            if flatNumText:
                flatNum = int(flatNumText)

        tableOrgStructure = db.table('OrgStructure')
        tableOrgStructureAddress = db.table('OrgStructure_Address')
        tableOrgAddressHouse = db.table('AddressHouse').alias('OrgAddressHouse')
        tableSrcAddressHouse = db.table('AddressHouse').alias('SrcAddressHouse')
#        tableNet = db.table('rbNet')
        table = tableOrgStructureAddress.innerJoin(tableOrgStructure, tableOrgStructure['id'].eq(tableOrgStructureAddress['master_id']) )
        table = table.innerJoin(tableOrgAddressHouse, tableOrgAddressHouse['id'].eq(tableOrgStructureAddress['house_id']))
        table = table.innerJoin(tableSrcAddressHouse,
                                [
                                  tableSrcAddressHouse['KLADRCode'].eq(tableOrgAddressHouse['KLADRCode']),
                                  tableSrcAddressHouse['KLADRStreetCode'].eq(tableOrgAddressHouse['KLADRStreetCode']),
                                  tableSrcAddressHouse['number'].eq(tableOrgAddressHouse['number']),
                                  tableSrcAddressHouse['corpus'].eq(tableOrgAddressHouse['corpus']),
                                ]
                               )
        cond = [tableOrgStructure['organisation_id'].eq(orgId),
                tableSrcAddressHouse['id'].eq(houseId),
                db.joinOr([tableOrgStructureAddress['firstFlat'].eq(0),
                           tableOrgStructureAddress['firstFlat'].le(flatNum),
                          ]),
                db.joinOr([tableOrgStructureAddress['lastFlat'].eq(0),
                           tableOrgStructureAddress['lastFlat'].ge(flatNum),
                          ]),
               ]
        if clientSex or clientAge:
            # пишу и плачу :(
            # альтернатива - закешировать данные о подразделениях на уровне приложения
            # но я всё ещё не готов :(
            netIdList = getNetIdList(clientSex, clientAge)
            netIdListAsStr = ','.join((str(netId) for netId in netIdList))
            if netIdList:
                cond.append('getOrgStructureNetId(OrgStructure.id) in (%s)' % netIdListAsStr)
            else:
                return []
        if orgStructureId:
            cond.append(tableOrgStructure['id'].inlist(getOrgStructureDescendants(orgStructureId)))
        return db.getIdList(table, tableOrgStructureAddress['master_id'].name(), cond)
    return []


def findOrgStructuresByAddress(addressId, orgId, orgStructureId=None):
    if addressId:
        db = QtGui.qApp.db
        tableAddress  = db.table('Address')
        record = db.getRecord(tableAddress, 'house_id, flat', addressId)
        houseId = record.value('house_id')
        flat = forceStringEx(record.value('flat'))
        return findOrgStructuresByHouseAndFlat(houseId, flat, orgId, orgStructureId)
    return []


def getOrgStructureName(orgStructureId):
    db = QtGui.qApp.db
    table = db.table('OrgStructure')
    return forceString(db.translate(table,'id',orgStructureId,'code'))


def getOrgStructureFullName(orgStructureId):
    db = QtGui.qApp.db
    table = db.table('OrgStructure')
    names = []
    ids   = set()

    while orgStructureId:
        record = db.getRecord(table, 'name, parent_id', orgStructureId)
        if record:
            names.append(forceString(record.value('name')))
            orgStructureId = forceRef(record.value('parent_id'))
        else:
            orgStructureId = None
        if orgStructureId in ids:
            orgStructureId = None
        else:
            ids.add(orgStructureId)
    return '/'.join(reversed(names))


def getOrgStructureNetId(orgStructureId):
    u'''получить id сети заданного подразделения'''
    if orgStructureId:
        db = QtGui.qApp.db
        stmt = 'SELECT getOrgStructureNetId(%d)'%orgStructureId
        query = db.query(stmt)
        if query.next():
            return forceRef(query.record().value(0))
    else:
        return None


def getOrgStructures(orgId):
    u'''получить список id подразделений заданной организации'''
    db = QtGui.qApp.db
    table = db.table('OrgStructure')
    return db.getIdList(table, where=table['organisation_id'].eq(orgId))


def getOrgStructureDescendants(orgStructureId):
    u'''получить список id подразделений, вложенных в данное + само подразделение'''
    return QtGui.qApp.db.getDescendants('OrgStructure', 'parent_id', orgStructureId)


def getOrgStructureListDescendants(orgStructureIdList):
    u'''получить список id подразделений, вложенных в данные + сами подразделения'''
    orgStructureIdSet = set([])
    for orgStructureId in orgStructureIdList:
        orgStructureIdSet |= set(QtGui.qApp.db.getDescendants('OrgStructure', 'parent_id', orgStructureId))
    return list(orgStructureIdSet)


def getOrgStructurePersonIdList(orgStructureId):
    u'''получить список персонала с учётом вложенных подразделений'''
    db = QtGui.qApp.db
    table = db.table('Person')
    orgStructureIdList = getOrgStructureDescendants(orgStructureId)
    return db.getIdList(table, where=table['orgStructure_id'].inlist(orgStructureIdList))


def getOrgStructureAddressIdList(orgStructureId):
    u'''получить список id адресов, которые находятся на территории обслуживания подразделения'''
    if orgStructureId:
        orgStructureIdList = getOrgStructureDescendants(orgStructureId)
    else:
        orgStructureIdList = getOrgStructures(QtGui.qApp.currentOrgId())
    db = QtGui.qApp.db
    tableAddress = db.table('Address')
    tableOrgStructureAddress = db.table('OrgStructure_Address')
    table = tableAddress
    table = table.leftJoin(tableOrgStructureAddress, tableOrgStructureAddress['house_id'].eq(tableAddress['house_id']))
    cond = db.joinOr([
                        tableOrgStructureAddress['lastFlat'].eq(0),
                        db.joinAnd([
                                    tableOrgStructureAddress['firstFlat'].le(tableAddress['flat']),
                                    tableOrgStructureAddress['lastFlat'].ge(tableAddress['flat']),
                                  ])
                    ])
    cond = [tableOrgStructureAddress['master_id'].inlist(orgStructureIdList), cond]
    return db.getIdList(table, tableAddress['id'].name(), where=cond)


def getSolitaryOrgStructureId(orgStructureId):
    u'''получить id обособленного подразделения, в которое входит данное подразделение'''
    db = QtGui.qApp.db
    table = db.table('OrgStructure')
    skipableAddress = None
    while orgStructureId:
        record = db.getRecord(table, 'address, parent_id', orgStructureId)
        parentId = forceRef(record.value('parent_id'))
        address  = forceStringEx(record.value('address'))
        if skipableAddress is None:
            skipableAddress = address
        if parentId is None or address != skipableAddress:
            return orgStructureId
        orgStructureId = parentId
    return None


def getOrgStructureActionTypeIdSet(orgStructureId):
        idSet = set()
        db = QtGui.qApp.db
        tableOSAT = db.table('OrgStructure_ActionType')
        tableActionType = db.table('ActionType')
        table = tableOSAT.innerJoin(tableActionType, tableActionType['id'].eq(tableOSAT['actionType_id']))
        tableOrgStructure = db.table('OrgStructure')
        while orgStructureId:
            cond = [tableOSAT['master_id'].eq(orgStructureId),
                    tableOSAT['actionType_id'].isNotNull(),
                    tableActionType['deleted'].eq(0),
                    tableActionType['showInForm'].ne(0),
                   ]
            idList = db.getIdList(table,
                           tableOSAT['actionType_id'].name(),
                           cond
                           )
            idSet |= set(idList)
            record = db.getRecord(tableOrgStructure, ['parent_id', 'inheritActionTypes'], orgStructureId)
            orgStructureId = forceRef(record.value(0)) if forceBool(record.value(1)) else None
        return idSet


def getActionTypeOrgStructureIdList(actionTypeId, includeInheritance=False):
        db = QtGui.qApp.db
        table = db.table('OrgStructure_ActionType')
        tableOrgStructure = db.table('OrgStructure')

        resultIdList = db.getDistinctIdList(table.leftJoin(tableOrgStructure, tableOrgStructure['id'].eq(table['master_id'])),
                                            table['master_id'],
                                            [ table['actionType_id'].eq(actionTypeId),
                                              tableOrgStructure['deleted'].eq(0)
                                            ]
                                           )
        if includeInheritance:
            resultIdSet = set(resultIdList)
            idSet = set(resultIdSet)
            while idSet:
                idList = db.getIdList(tableOrgStructure,
                                      'id',
                                      [ tableOrgStructure['deleted'].eq(0),
                                        tableOrgStructure['master_id'].inlist(idSet),
                                        tableOrgStructure['inheritActionTypes'].eq(1),
                                      ]
                                     )
                idSet = set(idList)-resultIdSet
                resultIdList += list(idSet)
                resultIdSet |= idSet
        return resultIdList



class CSexAgeConstraint(object):
    def __init__(self):
        self.age = None
        self.sex = None


    def constrain(self):
        return bool(self.age or self.sex)


    def initBySimpleType(self, sex, age):
        self.sex = sex
        self.age = parseAgeSelector(age) if age else None


    def initByRecord(self, record):
        self.initBySimpleType(forceInt(record.value('sex')), forceStringEx(record.value('age')))


    def applicable(self, clientSex, clientAge):
        if self.sex and clientSex != self.sex:
            return False
        if self.age and not checkAgeSelector(self.age, clientAge):
            return False
        return True


class CNet(CSexAgeConstraint):
    def __init__(self, id):
        CSexAgeConstraint.__init__(self)
        db = QtGui.qApp.db
        table = db.table('rbNet')
        record = db.getRecord(table, '*', id)
        if record:
            self.initByRecord(record)
            self.code = forceString(record.value('code'))
            self.name = forceString(record.value('name'))
        else:
            self.code = None
            self.name = None


# ####################################################

class COKFSInfo(CRBInfo):
    tableName = 'rbOKFS'


    def _initByRecord(self, record):
        self._ownership = forceInt(record.value('ownership'))


    def _initByNull(self):
        self._ownership = None

    ownership = property(lambda self: self.load()._ownership)


class COKPFInfo(CRBInfo):
    tableName = 'rbOKPF'


class CNetInfo(CRBInfo):
    tableName = 'rbNet'


    def _initByRecord(self, record):
        self._sexCode = forceInt(record.value('sex'))
        self._age = forceString(record.value('age'))


    def _initByNull(self):
        self._sexCode = 0
        self._age = ''


    sexCode = property(lambda self: self.load()._sexCode)
    sex = property(lambda self: formatSex(self.load()._sexCode))
    age = property(lambda self: self.load()._age)


class COrgInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id
        self._fullName = ''
        self._shortName = ''
        self._title = ''
        self._net = None
        self._ffoms = ''
        self._tfomsCode = ''
        self._miacCode = ''
        self._OKVED = ''
        self._INN = ''
        self._KPP = ''
        self._OGRN = ''
        self._OKATO = ''
        self._OKPF = None
        self._OKFS = None
        self._OKPO = ''
        self._FSS = ''
        self._region = ''
        self._address = ''
        self._chief = ''
        self._phone = ''
        self._accountant = ''
        self._notes = ''
        self._area = ''
        self._areaRaw = ''
        self._medicalType = None


    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('Organisation', '*', self.id) if self.id else None
        if record:
            self._fullName = forceString(record.value('fullName'))
            self._shortName = forceString(record.value('shortName'))
            self._title = forceString(record.value('title'))
            self._net = self.getInstance(CNetInfo, record.value('net_id'))
            self._ffoms = forceString(record.value('smoCode'))
            self._tfomsCode = forceString(record.value('infisCode'))
            self._miacCode = forceString(record.value('miacCode'))
            self._OKVED = forceString(record.value('OKVED'))
            self._INN = forceString(record.value('INN'))
            self._KPP = forceString(record.value('KPP'))
            self._OGRN = forceString(record.value('OGRN'))
            self._OKATO = forceString(record.value('OKATO'))
#            self._OKPF = forceString(db.translate('rbOKPF', 'id', forceRef(record.value('OKPF_id')), 'name'))
            self._OKPF = self.getInstance(COKPFInfo, forceRef(record.value('OKPF_id')))
            self._OKFS = self.getInstance(COKFSInfo, forceRef(record.value('OKFS_id')))
            self._OKPO = forceString(record.value('OKPO'))
            self._FSS = forceString(record.value('FSS'))
            self._region = forceString(record.value('region'))
            self._address = forceString(record.value('address'))
            self._chief = forceString(record.value('chief'))
            self._phone = forceString(record.value('phone'))
            self._accountant = forceString(record.value('accountant'))
            self._notes = forceString(record.value('notes'))
            self._area = self.getKLADRName(forceString(record.value('area')))
            self._areaRaw = forceString(record.value('area'))
            self._medicalType = forceInt(record.value('isMedical'))
            return True
        else:
            self._OKPF = self.getInstance(COKPFInfo, None)
            self._OKFS = self.getInstance(COKFSInfo, None)
            return False

    def getKLADRName(self, code):
        model = getKladrTreeModel()
        item = model.getRootItem().findCode(code)
        if item and item != model.getRootItem():
            return item._name
        return u''


#    def __unicode__(self):
    def __str__(self):
        return self.load()._shortName

    fullName    = property(lambda self: self.load()._fullName)
    shortName   = property(lambda self: self.load()._shortName)
    title       = property(lambda self: self.load()._title)
    net         = property(lambda self: self.load()._net)
    ffoms       = property(lambda self: self.load()._ffoms)
    tfomsCode   = property(lambda self: self.load()._tfomsCode)
    miacCode    = property(lambda self: self.load()._miacCode)
    OKVED       = property(lambda self: self.load()._OKVED)
    INN         = property(lambda self: self.load()._INN)
    KPP         = property(lambda self: self.load()._KPP)
    OGRN        = property(lambda self: self.load()._OGRN)
    OKATO       = property(lambda self: self.load()._OKATO)
    OKPF        = property(lambda self: self.load()._OKPF)
    OKFS        = property(lambda self: self.load()._OKFS)
    OKPO        = property(lambda self: self.load()._OKPO)
    FSS         = property(lambda self: self.load()._FSS)
    region      = property(lambda self: self.load()._region)
    address     = property(lambda self: self.load()._address)
    chief       = property(lambda self: self.load()._chief)
    phone       = property(lambda self: self.load()._phone)
    accountant  = property(lambda self: self.load()._accountant)
    notes       = property(lambda self: self.load()._notes)
    note        = property(lambda self: self.load()._notes)
    area        = property(lambda self: self.load()._area)
    areaRaw     = property(lambda self: self.load()._areaRaw)
    medicalType = property(lambda self: self.load()._medicalType)

    infisCode   = property(lambda self: self.load()._tfomsCode)


class COrgStructureInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id
        self._name = ''
        self._code = ''
        self._organisation = None
        self._parent = None
        self._net = None
        self._jobs = []
        self._chief = None
        self._headNurse = None
        self._address = None
        self._infisDepTypeCode = None
        self._infisInternalCode  = None
        self._actionTypes = None
        self._note = ''


    def _load(self):
        from PersonInfo import CPersonInfo

        db = QtGui.qApp.db
        record = db.getRecord('OrgStructure', '*', self.id) if self.id else None
        if record:
            self._code = forceString(record.value('code'))
            self._name = forceString(record.value('name'))
            parentId = forceRef(record.value('parent_id'))
            self._parent = self.getInstance(COrgStructureInfo, parentId) if parentId else None
            organisationId = forceRef(record.value('organisation_id'))
            self._organisation = self.getInstance(COrgInfo, organisationId) if organisationId else None
            netId = forceRef(record.value('net_id'))
            self._net = self.getInstance(CNetInfo, netId) if netId else None
            self._chief = self.getInstance(CPersonInfo, forceRef(record.value('chief_id')))
            self._headNurse = self.getInstance(CPersonInfo, forceRef(record.value('headNurse_id')))
            self._infisInternalCode = forceString(record.value('infisInternalCode'))
            self._infisDepTypeCode = forceString(record.value('infisDepTypeCode'))
            self._note = forceString(record.value('note'))
#            self._jobs =  self.getInstance(COrgStructureJobInfoList, self.id)
            self._jobs = None
            address = forceString(record.value('address'))
            if address:
                self._address = address
            return True
        else:
            return False


    def getNet(self):
        self.load()
        if self._net is None:
            if self._parent:
                self._net = self._parent.getNet()
            elif self._organisation:
                self._net = self._organisation.net
            else:
                self._net = self.getInstance(CNetInfo, None)
        return self._address

    def getFullName(self):
#        self.load()
        return getOrgStructureFullName(self.id)


    def getAddress(self):
        self.load()
        if self._address is None:
            if self._parent:
                self._address = self._parent.getAddress()
            elif self._organisation:
                self._address = self._organisation.address
            else:
                self._address = ''
        return self._address


    def getActionTypes(self):
        if self._actionTypes is None:
            from Events.ActionInfo import CActionTypeInfoList
            self._actionTypes = CActionTypeInfoList()
            self._actionTypes.idList = list(getOrgStructureActionTypeIdSet(self.id))
        return self._actionTypes

    def __str__(self):
        return self.getFullName()


    code              = property(lambda self: self.load()._code)
    name              = property(lambda self: self.load()._name)
    organisation      = property(lambda self: self.load()._organisation)
    parent            = property(lambda self: self.load()._parent)
    net               = property(getNet)
    jobs               = property(lambda self: self.load()._jobs)
    chief             = property(lambda self: self.load()._chief)
    headNurse         = property(lambda self: self.load()._headNurse)
    infisInternalCode = property(lambda self: self.load()._infisInternalCode)
    infisDepTypeCode  = property(lambda self: self.load()._infisDepTypeCode)
    fullName          = property(getFullName)
    address           = property(getAddress)
    actionTypes       = property(getActionTypes)
    note              = property(lambda self: self.load()._note)


# #####################################################################


class COrgStructure(CDbEntityCache):
    cache = {}

    ostAmb = 0
    ostHospital = 1
    ostFirstAid = 2
    ostFobileStation = 3
    ostEmergencyDepartment = 4

    def __init__(self, orgStructureId):
        self._orgStructureId = orgStructureId

        db = QtGui.qApp.db

        record = db.getRecord('OrgStructure', 'type', orgStructureId)
        if not record:
            from PyQt4.QtSql import QSqlRecord
            record = QSqlRecord()

        self.type = forceInt(record.value('type'))

    @classmethod
    def purge(cls):
        cls.cache.clear()


    @classmethod
    def get(cls, orgStructureId):
        result = cls.cache.get(orgStructureId, None)
        if result is None:
            cls.connect()
            result = COrgStructure(orgStructureId)
            cls.cache[orgStructureId] = result
        return result



class CActivityInfo(CRBInfo):
    tableName = 'rbActivity'


    def _initByRecord(self, record):
        self._regionalCode = forceString(record.value('regionalCode'))
        self._note         = forceString(record.value('note'))


    def _initByNull(self):
        self._regionalCode = self._note = ''


    regionalCode = property(lambda self: self.load()._regionalCode)
    note = property(lambda self: self.load()._note)
