# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import weakref

from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, QVariant, pyqtSignature, SIGNAL

from library.abstract        import abstract
from library.crbcombobox     import CRBComboBox
from library.DialogBase      import CDialogBase
from library.InDocTable      import CInDocTableModel, CRecordListModel, CInDocTableCol, CRBInDocTableCol, CFloatInDocTableCol
from library.interchange     import (
    getDateEditValue, getLineEditValue, getRBComboBoxValue, getSpinBoxValue, setDateEditValue, setLineEditValue, setSpinBoxValue, setRBComboBoxValue
)
from library.ItemsListDialog import CItemsListDialog, CItemEditorBaseDialog
from library.TableModel      import CRefBookCol, CTextCol
from library.Utils           import forceRef, forceString, forceStringEx, toVariant, forceBool

from RefBooks.Tables         import rbCode, rbName
from Stock.NomenclatureComboBox import CNomenclatureInDocTableCol, getFeaturesAndValues

from RefBooks.Ui_RBNomenclatureEditor import Ui_ItemEditorDialog
from RefBooks.Ui_RBNomenclatureFilter import Ui_ItemFilterDialog


class CRBNomenclatureList(CItemsListDialog):
    def __init__(self, parent, forSelect=False):
        CItemsListDialog.__init__(self, parent, [
            CRefBookCol(u'Тип',   ['type_id'], 'rbNomenclatureType', 20),
            CTextCol(u'Код',          [rbCode], 20),
            CTextCol(u'Региональный код',  ['regionalCode'], 20),
            CTextCol(u'Наименование', [rbName], 40),
            CTextCol(u'Дозировка', ['dosageValue'], 10),
            CRefBookCol(u'Ед. дозировки',['unit_id'], 'rbUnit', 20),
            CRefBookCol(u'Форма выпуска',['lfForm_id'],'rbLfForm',40),
            CTextCol(u'Дата включения',['inDate'],40),
            CTextCol(u'Дата исключения',['exDate'],40),
            CTextCol(u'Комплектность',['completeness'],40),
            ], 'rbNomenclature', [rbCode, rbName,'dosageValue','inDate','exDate'], forSelect,
            filterClass=CRBNomenclatureListFilter
            )
        self.setWindowTitleEx(u'Номенклатура лекарственных средств и изделий медицинского назначения')
        if self.forSelect:
            self.parent = parent


    def preSetupUi(self):
        CItemsListDialog.preSetupUi(self)
        self.addObject('actDuplicate', QtGui.QAction(u'Дублировать', self))
        self.addObject('actDelete',    QtGui.QAction(u'Удалить', self))


    def postSetupUi(self):
        CItemsListDialog.postSetupUi(self)
        self.tblItems.createPopupMenu([self.actDuplicate, '-', self.actDelete])
        self.connect(self.tblItems.popupMenu(), SIGNAL('aboutToShow()'), self.popupMenuAboutToShow)


    def getItemEditor(self):
        return CRBNomenclatureEditor(self)


    def generateFilterByProps(self, props):
#        db = QtGui.qApp.db
        cond = CItemsListDialog.generateFilterByProps(self, props)
        table = self.model.table()
        code = props.get('code', None)
        name = props.get('name', None)
        regionalCode = props.get('regionalCode', None)
        internationalNonproprietaryName = props.get('internationalNonproprietaryName', None)
        ATC = props.get('ATC', None)
        producer = props.get('producer', None)
        classId = props.get('classId', None)
        kindId = props.get('kindId', None)
        typeId = props.get('typeId', None)

        if code:
            cond.append(table['code'].like(code))
        if name:
            cond.append(table['name'].like(name))
        if regionalCode:
            cond.append(table['regionalCode'].like(regionalCode))
        if internationalNonproprietaryName:
            cond.append(table['internationalNonproprietaryName'].like(internationalNonproprietaryName))
        if ATC:
            cond.append(table['ATC'].like(ATC))
        if producer:
            cond.append(table['producer'].like(producer))

        if typeId:
            cond.append(table['type_id'].eq(typeId))
        elif kindId:
            cond.append(table['type_id'].inlist(self.getTypeIdListByKind(kindId)))
        elif classId:
            cond.append(table['type_id'].inlist(self.getTypeIdListByClass(classId)))
        return cond


    def popupMenuAboutToShow(self):
        currentItemId = self.currentItemId()
        self.actDuplicate.setEnabled(bool(currentItemId))
        self.actDelete.setEnabled(bool(currentItemId))


    @pyqtSignature('')
    def on_actDuplicate_triggered(self):
        def duplicateCurrentInternal():
            currentItemId = self.currentItemId()
            if currentItemId:
                db = QtGui.qApp.db
                table = db.table('rbNomenclature')
                db.transaction()
                try:
                    record = db.getRecord(table, '*', currentItemId)
                    record.setValue('code', toVariant(forceString(record.value('code'))+'_1'))
                    record.setNull('id')
                    newItemId = db.insertRecord(table, record)
                    db.commit()
                except:
                    db.rollback()
                    QtGui.qApp.logCurrentException()
                    raise
                self.renewListAndSetTo(newItemId)
        QtGui.qApp.call(self, duplicateCurrentInternal)


    @pyqtSignature('')
    def on_actDelete_triggered(self):
        def deleteCurrentInternal():
            currentItemId = self.currentItemId()
            if currentItemId:
                row = self.tblItems.currentIndex().row()
                db = QtGui.qApp.db
                table = db.table('rbNomenclature')
                db.deleteRecord(table, table['id'].eq(currentItemId))
                self.renewListAndSetTo()
                self.tblItems.setCurrentRow(row)
        QtGui.qApp.call(self, deleteCurrentInternal)


    @staticmethod
    def getKindIdListByClass(classId):
        db = QtGui.qApp.db
        tableKind = db.table('rbNomenclatureKind')
        kindIdList = db.getIdList(tableKind, 'id', tableKind['class_id'].eq(classId))
        return kindIdList


    @staticmethod
    def getTypeIdListByKindList(kindIdList):
        db = QtGui.qApp.db
        tableType = db.table('rbNomenclatureType')
        typeIdList = db.getIdList(tableType, 'id', tableType['kind_id'].inlist(kindIdList))
        return typeIdList


    @staticmethod
    def getTypeIdListByKind(kindId):
        typeIdList = CRBNomenclatureList.getTypeIdListByKindList([kindId])
        return typeIdList


    @staticmethod
    def getTypeIdListByClass(classId):
        kindIdList = CRBNomenclatureList.getKindIdListByClass(classId)
        typeIdList = CRBNomenclatureList.getTypeIdListByKindList(kindIdList)
        return typeIdList


#
# ##########################################################################
#


class CRBNomenclatureListFilter(CDialogBase, Ui_ItemFilterDialog):
    def __init__(self,  parent):
        CDialogBase.__init__(self, parent)
        self.setupUi(self)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setWindowTitleEx(u'Фильтр ЛСиИМН')
        self.cmbClass.setTable('rbNomenclatureClass', True)
        self.cmbKind.setTable('rbNomenclatureKind', True)
        self.cmbType.setTable('rbNomenclatureType', True)


    @pyqtSignature('int')
    def on_cmbClass_currentIndexChanged(self, index):
        self.setKindFilter()
        self.setTypeFilter()


    @pyqtSignature('int')
    def on_cmbKind_currentIndexChanged(self, index):
        self.setTypeFilter()


    @pyqtSignature('QAbstractButton*')
    def on_buttonBox_clicked(self, button):
        buttonCode = self.buttonBox.standardButton(button)
        if buttonCode == self.buttonBox.Reset:
            self.setProps({})


    def setKindFilter(self):
        self.cmbKind.setFilter(self.formKindFilter(self.cmbClass.value()))



    def setTypeFilter(self):
        self.cmbType.setFilter(self.formTypeFilter(self.cmbClass.value(), self.cmbKind.value()))


    def setProps(self, props):
        self.edtCode.setText(props.get('code', ''))
        self.edtName.setText(props.get('name', ''))
        self.edtInternationalNonproprietaryName.setText(props.get('internationalNonproprietaryName', ''))
        self.edtRegionalCode.setText(props.get('regionalCode', ''))
        self.edtATC.setText(props.get('ATC', ''))
        self.edtProducer.setText(props.get('producer', ''))
        self.cmbClass.setValue(props.get('classId', None))
        self.cmbKind.setValue(props.get('kindId', None))
        self.cmbType.setValue(props.get('typeId', None))


    def props(self):
        code = forceStringEx(self.edtCode.text())
        name = forceStringEx(self.edtName.text())
        internationalNonproprietaryName  = forceStringEx(self.edtInternationalNonproprietaryName.text())
        regionalCode = forceStringEx(self.edtRegionalCode.text())
        ATC  = forceStringEx(self.edtATC.text())
        producer = forceStringEx(self.edtProducer.text())
        classId = self.cmbClass.value()
        kindId = self.cmbKind.value()
        typeId = self.cmbType.value()
        result = {}
        if code:
            result['code'] = code
        if name:
            result['name'] = name
        if internationalNonproprietaryName:
            result['internationalNonproprietaryName'] = internationalNonproprietaryName
        if regionalCode:
            result['regionalCode'] = regionalCode
        if ATC:
            result['ATC'] = ATC
        if producer:
            result['producer'] = producer
        if classId:
            result['classId'] = classId
        if kindId:
            result['kindId'] = kindId
        if typeId:
            result['typeId'] = typeId
        return result



    @staticmethod
    def formKindFilter(classId):
        if classId:
            table = QtGui.qApp.db.table('rbNomenclatureKind')
            return table['class_id'].eq(classId)
        return ''


    @staticmethod
    def formTypeFilter(classId, kindId):
        db = QtGui.qApp.db
        table = db.table('rbNomenclatureType')
        if kindId:
            return table['kind_id'].eq(kindId)
        else:
            if classId:
                kindIdList = CRBNomenclatureList.getKindIdListByClass(classId)
                return table['kind_id'].inlist(kindIdList)
        return ''

#
# ##########################################################################
#

class CRBNomenclatureEditor(CItemEditorBaseDialog, Ui_ItemEditorDialog):
    def __init__(self,  parent):
        CItemEditorBaseDialog.__init__(self, parent, 'rbNomenclature')
        self.addModels('Features', CFeaturesModel(self))
        self.addModels('Ingredients', CIngredientsModel(self))
        self.addModels('Analogs', CAnalogsModel(self))
        self.addModels('UnitRatio', CUnitRatioModel(self))
        self.addModels('UsingTypes', CUsingTypesModel(self))
        self.addObject('actFillFeaturesBySiblings', QtGui.QAction(u'Добавить характеристики', self))

        self.setupUi(self)
        self.setWindowTitleEx(u'Лекарственное средство или изделие медицинского назначения')
        self.cmbClass.setTable('rbNomenclatureClass', True)
        self.cmbKind.setTable('rbNomenclatureKind', True)
        self.cmbType.setTable('rbNomenclatureType', True)
        self.cmbUnit.setTable('rbUnit', True)
        self.cmbDefaultClientUnit.setTable('rbUnit', True)
        self.cmbDefaultStockUnit.setTable('rbUnit', True)
        self.cmbLfForm.setTable('rbLfForm', True)
        self.setModels(self.tblFeatures, self.modelFeatures, self.selectionModelFeatures)
        self.tblFeatures.addPopupAction(self.actFillFeaturesBySiblings)
        self.tblFeatures.addPopupSeparator()
        self.tblFeatures.addMoveRow()
        self.tblFeatures.addPopupDuplicateCurrentRow()
        self.tblFeatures.addPopupSelectAllRow()
        self.tblFeatures.addPopupClearSelectionRow()
        self.tblFeatures.addPopupSeparator()
        self.tblFeatures.addPopupDelRow()

        self.tblUnitRatio.addPopupDelRow()

        self.setModels(self.tblIngredients, self.modelIngredients, self.selectionModelIngredients)
        self.tblIngredients.addPopupSelectAllRow()
        self.tblIngredients.addPopupClearSelectionRow()
        self.tblIngredients.addPopupSeparator()
        self.tblIngredients.addPopupDelRow()

        self.modelAnalogs.setEnableAppendLine(True)
        self.setModels(self.tblAnalogs, self.modelAnalogs, self.selectionModelAnalogs)
        self.setModels(self.tblUnitRatio, self.modelUnitRatio, self.selectionModelUnitRatio)
#        self.tblAnalogs.setModel(self.modelAnalogs)
        self.tblAnalogs.addPopupSelectAllRow()
        self.tblAnalogs.addPopupClearSelectionRow()
        self.tblAnalogs.addPopupSeparator()
        self.tblAnalogs.addPopupDelRow()

        self.setModels(self.tblUsingTypes, self.modelUsingTypes, self.selectionModelUsingTypes)
        self.tblUsingTypes.addPopupDelRow()

        self.setupDirtyCather()

    def checkDataEntered(self):
        stockDefaultId = self.cmbDefaultStockUnit.value()
        clientDefaultId = self.cmbDefaultClientUnit.value()

        if not (stockDefaultId or clientDefaultId):
            return self.checkInputMessage(u'расходную и упаковочную единицы по умолчанию', skipable=True,
                                          widget=self.cmbDefaultStockUnit)


        if not clientDefaultId:
            return self.checkInputMessage(u'расходную единицу по умолчанию', skipable=False,
                                          widget=self.cmbDefaultClientUnit)

        items = self.modelUnitRatio.items()

        if not stockDefaultId:
            return self.checkInputMessage(u'упаковочную единицу по умолчанию', skipable=False, widget=self.cmbDefaultStockUnit)

        allUnits = set()
        for item in items:
            targetUnitId = forceRef(item.value('targetUnit_id'))
            sourceUnitId = forceRef(item.value('sourceUnit_id'))
            if targetUnitId:
                allUnits.add(targetUnitId)
            if sourceUnitId:
                allUnits.add(sourceUnitId)
        if clientDefaultId not in allUnits:
            return self.checkInputMessage(u'расходную единицу по умолчанию в таблице отношений единиц учета', skipable=False, widget=self.tblUnitRatio)

        if stockDefaultId not in allUnits:
            return self.checkInputMessage(u'упаковочную единицу по умолчанию в таблице отношений единиц учета', skipable=False, widget=self.tblUnitRatio)
        return True


    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        typeId = forceRef(record.value('type_id'))
        kindId = forceRef(QtGui.qApp.db.translate('rbNomenclatureType', 'id', typeId, 'kind_id'))
        classId = forceRef(QtGui.qApp.db.translate('rbNomenclatureKind', 'id', kindId, 'class_id'))
        unitId = forceRef(record.value('unit_id'))
        lfFormId = forceRef(record.value('lfForm_id'))
        self.cmbClass.setValue(classId)
        self.cmbKind.setValue(kindId)
        self.cmbType.setValue(typeId)
        self.cmbUnit.setValue(unitId)
        self.cmbLfForm.setValue(lfFormId)
        setRBComboBoxValue(self.cmbDefaultClientUnit, record, 'defaultClientUnit_id')
        setRBComboBoxValue(self.cmbDefaultStockUnit, record, 'defaultStockUnit_id')
        setLineEditValue(   self.edtCode,          record, 'code')
        setLineEditValue(   self.edtRegionalCode,  record, 'regionalCode')
        setLineEditValue(   self.edtName,          record, 'name')
        setLineEditValue(self.edtOriginName, record, 'originName')
        setLineEditValue(   self.edtInternationalNonproprietaryName, record, 'internationalNonproprietaryName')
        setLineEditValue(   self.edtATC,           record, 'ATC')
        setLineEditValue(   self.edtProducer,      record, 'producer')
        setSpinBoxValue(    self.spnPackSize,      record, 'packSize')
        setLineEditValue(   self.edtMnnCode,       record, 'mnnCode')
        setLineEditValue(self.edtMnnLatin, record, 'mnnLatin')
        setLineEditValue(   self.edtTrnCode,       record, 'trnCode')
        setLineEditValue(   self.edtDosageValue,   record, 'dosageValue')
        setDateEditValue(   self.edtInDate,        record, 'inDate')
        setDateEditValue(   self.edtExDate,        record, 'exDate')
        setLineEditValue(   self.edtCompleteness,  record, 'completeness')
        self.modelFeatures.loadItems(self.itemId())
        self.modelIngredients.loadItems(self.itemId())
        self.modelAnalogs.loadItems(self.itemId(), forceRef(record.value('analog_id')))
        self.modelUnitRatio.loadItems(self.itemId())
        self.modelUsingTypes.loadItems(self.itemId())
        self.applyUnitWidgets()

    def applyUnitWidgets(self):
        db = QtGui.qApp.db
        table = db.table('StockTrans')
        cond = db.joinOr([table['debNomenclature_id'].eq(self.itemId()), table['creNomenclature_id'].eq(self.itemId())])
        stockTransExists = QtGui.qApp.db.existsStmt(table, cond)
        query = db.query('SELECT %s' % stockTransExists)
        exists = query.first() and forceBool(query.record().value(0))
        self.cmbDefaultStockUnit.setEnabled(not exists)


    def getRecord(self):
        record = CItemEditorBaseDialog.getRecord(self)
        getRBComboBoxValue( self.cmbType,          record, 'type_id')
        getRBComboBoxValue( self.cmbUnit,          record, 'unit_id')
        getRBComboBoxValue(self.cmbDefaultClientUnit, record, 'defaultClientUnit_id')
        getRBComboBoxValue(self.cmbDefaultStockUnit, record, 'defaultStockUnit_id')
        getRBComboBoxValue( self.cmbLfForm,        record, 'lfForm_id')
        getLineEditValue(   self.edtCode,          record, 'code')
        getLineEditValue(   self.edtRegionalCode,  record, 'regionalCode')
        getLineEditValue(   self.edtName,          record, 'name')
        getLineEditValue(self.edtOriginName, record, 'originName')
        getLineEditValue(   self.edtInternationalNonproprietaryName, record, 'internationalNonproprietaryName')
        getLineEditValue(   self.edtATC,           record, 'ATC')
        getLineEditValue(   self.edtProducer,      record, 'producer')
        getSpinBoxValue(    self.spnPackSize,      record, 'packSize')
        getLineEditValue(   self.edtMnnCode,       record, 'mnnCode')
        getLineEditValue(self.edtMnnLatin, record, 'mnnLatin')
        getLineEditValue(   self.edtTrnCode,       record, 'trnCode')
        getLineEditValue(   self.edtDosageValue,   record, 'dosageValue')
        getDateEditValue(   self.edtInDate,       record, 'inDate')
        getDateEditValue(   self.edtExDate,       record, 'exDate')
        getLineEditValue(   self.edtCompleteness, record, 'completeness')
        record.setValue('analog_id', toVariant(self.modelAnalogs.correctAlanogId()))
        return record


    def saveInternals(self, id):
        self.modelFeatures.saveItems(id)
        self.modelIngredients.saveItems(id)
        self.modelAnalogs.saveItems(id)
        self.modelUnitRatio.saveItems(id)
        self.modelUsingTypes.saveItems(id)


    def setKindFilter(self):
        self.cmbKind.setFilter(CRBNomenclatureListFilter.formKindFilter(self.cmbClass.value()))



    def setTypeFilter(self):
        self.cmbType.setFilter(CRBNomenclatureListFilter.formTypeFilter(self.cmbClass.value(), self.cmbKind.value()))


    @pyqtSignature('int')
    def on_cmbClass_currentIndexChanged(self, index):
        self.setKindFilter()
        self.setTypeFilter()
        self.modelAnalogs.setDefaultClassId(self.cmbClass.value())
        self.modelAnalogs.setDefaultKindId(self.cmbKind.value())
        self.modelAnalogs.setDefaultTypeId(self.cmbType.value())


    @pyqtSignature('int')
    def on_cmbKind_currentIndexChanged(self, index):
        self.setTypeFilter()
        self.modelAnalogs.setDefaultKindId(self.cmbKind.value())
        self.modelAnalogs.setDefaultTypeId(self.cmbType.value())


    @pyqtSignature('int')
    def on_cmbType_currentIndexChanged(self, index):
        typeId = self.cmbType.value()
        self.modelAnalogs.setDefaultTypeId(typeId)
        self.actFillFeaturesBySiblings.setEnabled(bool(typeId))


    @pyqtSignature('int')
    def on_tabWidget_currentChanged(self, index):
        if index == 1:
            if len(self.modelFeatures.items())==0:
                self.on_actFillFeaturesBySiblings_triggered()
            else:
                classId = self.cmbClass.value()
                kindId = self.cmbKind.value()
                typeId = self.cmbType.value()
                self.modelFeatures.updateLikelyFeatures(classId, kindId, typeId)


    @pyqtSignature('')
    def on_actFillFeaturesBySiblings_triggered(self):
        classId = self.cmbClass.value()
        kindId = self.cmbKind.value()
        typeId = self.cmbType.value()
        self.modelFeatures.fillBySiblings(classId, kindId, typeId)


class CBaseFeatureInDocTableCol(CInDocTableCol):
    # В базе данных хранится номер,
    # а на экране рисуется комбо-бокс с соотв. значениями
    def __init__(self, title, fieldName, width, model, **params):
        CInDocTableCol.__init__(self, title, fieldName, width, **params)
        self.model = weakref.ref(model)


    def toString(self, val, record):
        return val


    @abstract
    def getValues(self, record):
        pass

    def createEditor(self, parent):
        editor = QtGui.QComboBox(parent)
        editor.setEditable(True)
        return editor


    def setEditorData(self, editor, value, record):
        values = self.getValues(record)
        editor.addItems(values)
#        for item in values:
#            editor.addItem(item)
        index = editor.findText(forceString(value), Qt.MatchFixedString)
        if index>=0:
            editor.setCurrentIndex(index)
        else:
            editor.insertItem(0, forceString(value))
            editor.setCurrentIndex(0)
        editor.setEditText(forceString(value))


    def getEditorData(self, editor):
        return toVariant(editor.currentText())


class CFeatureNameInDocTableCol(CBaseFeatureInDocTableCol):
    def getValues(self, record):
        return self.model().getFeatureNames()


class CFeatureValueInDocTableCol(CBaseFeatureInDocTableCol):
    def getValues(self, record):
        return self.model().getValuesForName(forceString(record.value('name')))


class CFeaturesModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'rbNomenclature_Feature', 'id', 'master_id', parent)
        self.addCol(CFeatureNameInDocTableCol(   u'Наименование',  'name', 20, self)).setSortable(True)
        self.addCol(CFeatureValueInDocTableCol(   u'Значение',  'value', 20, self)).setSortable(True)
        self.typeId = None
        self.kindId = None
        self.classId = None
        self.likelyFeatures = {}


    def updateLikelyFeatures(self, classId, kindId, typeId):
        if (   self.typeId != typeId
            or self.kindId != kindId
            or self.classId != classId ):
            self.likelyFeatures = getFeaturesAndValues(classId, kindId, typeId)
            self.typeId = typeId
            self.kindId = kindId
            self.classId = classId


    def fillBySiblings(self, classId, kindId, typeId):
        self.updateLikelyFeatures(classId, kindId, typeId)
        if self.likelyFeatures:
            actualFeatures = {}
            for item in self.items():
                name   = forceString(item.value('name'))
                value  = forceString(item.value('value'))
                if value:
                    actualFeatures.setdefault(name, set()).add(value)
            for name in actualFeatures:
                actualFeatures.setdefault(name, set())

            featureNames = actualFeatures.keys()
            featureNames.sort()

            items = []
            for name in featureNames:
                values = list(actualFeatures[name])
                values.sort()
                for value in values or ('', ):
                    item = self.getEmptyRecord()
                    item.setValue('name', name)
                    item.setValue('value', value)
                    items.append(item)
            self.setItems(items)


    def getFeatureNames(self):
        result = self.likelyFeatures.keys()
        result.sort()
        return result


    def getValuesForName(self, name):
        return self.likelyFeatures.get(name) or []


class CIngredientsModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'rbNomenclature_Ingredient', 'id', 'master_id', parent)
        self.addCol(CRBInDocTableCol(   u'Ингредиент',  'ingredient_id', 20, 'rbIngredient')).setSortable(True)


class CUnitRatioModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'rbNomenclature_UnitRatio', 'id', 'master_id', parent)
        self.addCol(CRBInDocTableCol(u'Исходная Ед.учета', 'sourceUnit_id', 20, 'rbUnit')).setSortable(True)
        self.addCol(CRBInDocTableCol(u'Целевая Ед.учета', 'targetUnit_id', 20, 'rbUnit')).setSortable(True)
        self.addCol(CFloatInDocTableCol(u'Коэффициент', 'ratio', 20, precision=1)).setSortable(True)
        self.addHiddenCol('auto_calculated')
        self.setFilter('auto_calculated = 0')
        self._autoCalculated = {}

    def saveItems(self, masterId):
        if self._items is not None:
            db = QtGui.qApp.db
            table = self._table
            masterId = toVariant(masterId)
            masterIdFieldName = self._masterIdFieldName
            idFieldName = self._idFieldName
            idList = []
            for idx, record in enumerate(self._items):
                record.setValue(masterIdFieldName, masterId)
                record.setValue('auto_calculated', toVariant(0))
                if self._idxFieldName:
                    record.setValue(self._idxFieldName, toVariant(idx))
                if self._extColsPresent:
                    outRecord = self.removeExtCols(record)
                else:
                    outRecord = record
                id = db.insertOrUpdate(table, outRecord)
                record.setValue(idFieldName, toVariant(id))
                idList.append(id)
                self.saveDependence(idx, id)

            nodesIds = self._calculateNodes()
            idList.extend(nodesIds)

            filter = [table[masterIdFieldName].eq(masterId),
                      'NOT ('+table[idFieldName].inlist(idList)+')']
            if self._filter:
                filter.append(self._filter)
            if table.hasField('deleted'):
                db.markRecordsDeleted(table, filter)
            else:
                db.deleteRecord(table, filter)

    def _calculateNodes(self):
        return []

    def loadItems(self, masterId):
        CInDocTableModel.loadItems(self, masterId)
        # cols = []
        # for col in self._cols:
        #     if not col.external():
        #         cols.append(col.fieldName())
        # cols.append(self._idFieldName)
        # cols.append(self._masterIdFieldName)
        # if self._idxFieldName:
        #     cols.append(self._idxFieldName)
        # for col in self._hiddenCols:
        #     cols.append(col)
        # table = self._table
        # filter = [table[self._masterIdFieldName].eq(masterId),
        #           table['auto_calculated'].eq(1)]
        # if table.hasField('deleted'):
        #     filter.append(table['deleted'].eq(0))
        # if self._idxFieldName:
        #     order = [self._idxFieldName, self._idFieldName]
        # else:
        #     order = [self._idFieldName]
        # auto_calculated_items = db.getRecordList(table, cols, filter, order)
        #







class CAnalogsModel(CRecordListModel):
    def __init__(self, parent):
        CRecordListModel.__init__(self, parent)
        self.addExtCol(CNomenclatureInDocTableCol(u'ЛСиИМН', 'id', 50, showFields = CRBComboBox.showName),
                       QVariant.Int
                      )
        self.masterId = None # в rbNomenclature
        self.analogId = None # в rbNomenclature_analog


    def setDefaultClassId(self, classId):
        self._cols[0].setDefaultClassId(classId)


    def setDefaultKindId(self, kindId):
        self._cols[0].setDefaultClassId(kindId)


    def setDefaultTypeId(self, typeId):
        self._cols[0].setDefaultClassId(typeId)


    def getEmptyRecord(self):
        record = CRecordListModel.getEmptyRecord(self)
        record.append(QtSql.QSqlField('id', QVariant.Int))
        return record


    def getAnalogies(self, nomenclatureIdList, analogId):
        db = QtGui.qApp.db
        table = db.table('rbNomenclature')
        items = db.getRecordList(table,
                                 'id',
                                 [table['analog_id'].eq(analogId), table['id'].notInlist(nomenclatureIdList)],
                                 'name'
                                 )
        return items


    def loadItems(self, masterId, analogId):
        self.masterId = masterId
        self.analogId = analogId
        if analogId:
            items = self.getAnalogies([masterId], analogId)
            self.setItems(items)


    def getIdList(self):
        idlist = []
        for item in self.items():
            id = forceRef(item.value('id'))
            if id:
                idlist.append(id)
        if self.masterId:
            idlist.append(self.masterId)
        return idlist


    def newAnalogId(self):
        db = QtGui.qApp.db
        table = db.table('rbNomenclature_Analog')
        return db.insertRecord(table, table.newRecord())


    def correctAlanogId(self):
        idlist = self.getIdList()
        if idlist:
            idlist.sort()
            db = QtGui.qApp.db
            table = db.table('rbNomenclature')
            usedAnalogIdList = db.getIdList(table,
                                            idCol='DISTINCT analog_id',
                                            where=[table['id'].inlist(idlist), table['analog_id'].isNotNull()] )
            if len(usedAnalogIdList) == 1:
                memberIdList = db.getIdList(table,
                                     'id',
                                     [table['analog_id'].eq(usedAnalogIdList[0]), table['id'].ne(self.masterId)],
                                     'id'
                                     )
                if memberIdList == idlist:
                    self.analogId = usedAnalogIdList[0]
                else:
                    self.analogId = self.newAnalogId()
            else:
                self.analogId = self.newAnalogId()
        else:
            self.analogId = None
        return self.analogId


    def saveItems(self, masterId):
        if self.analogId:
            self.masterId = masterId
            idlist = self.getIdList()
            db = QtGui.qApp.db
            table = db.table('rbNomenclature')
            db.updateRecords(table, table['analog_id'].eq(self.analogId), table['id'].inlist(idlist))


    def setData(self, index, value, role=Qt.EditRole):
        if role==Qt.EditRole:
            id = forceRef(value)
            if not id: # не добавляем пустую строку
                return False
            idlist = self.getIdList()
            if id in idlist: # не добавляем повторы
                return False
            result = CRecordListModel.setData(self, index, value, role)
            if not result:
                return False
            # добавляем аналоги только что указанного
            idlist.append(id)
            db = QtGui.qApp.db
            table = db.table('rbNomenclature')
            newAnalogId = db.translate(table, 'id', value, 'analog_id')
            if newAnalogId:
                row = index.row()+1
                for record in self.getAnalogies(idlist, newAnalogId):
                    self.insertRecord(row, record)
                    row += 1
            return True
        else:
            return CRecordListModel.setData(self, index, value, role)


class CUsingTypesModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'rbNomenclature_UsingType', 'id', 'master_id', parent)
        self.addCol(CInDocTableCol(u'Способ применения',  'usingType', 30)).setSortable(True)
