# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, QDate, pyqtSignature, SIGNAL, pyqtSlot

from library.Attach.AttachFilesTableFlag import CAttachFilesTableFlag
from library.DialogBase           import CConstructHelperMixin
from library.ICDUtils             import getMKBName
from library.PrintInfo            import CInfoContext
from library.PrintTemplates       import applyTemplate, getPrintAction
from library.Utils                import forceDate, forceInt, forceRef, forceString

from Events.Action                import CAction
from Events.ActionInfo            import CActionInfo, CActionTypeCache, CLocActionInfoList
from Events.ActionPropertiesTable import CActionPropertiesTableModel
from Events.ActionStatus          import CActionStatus
from Events.AmbCardJournalDialog  import CAmbCardJournalDialog
from Events.EventInfo             import CEventInfo, CDiagnosisInfoList, CLocEventInfoList
from Events.TempInvalidInfo       import CTempInvalidInfoList
from Events.Utils                 import getActionTypeDescendants, getOrderText, payStatusText, setActionPropertiesColumnVisible
from Orgs.Utils                   import getOrgStructurePersonIdList
from Registry.RegistryTable       import ( CAmbCardDiagnosticsAccompDiagnosticsTableModel,
                                           CAmbCardDiagnosticsActionsTableModel,
                                           CAmbCardDiagnosticsTableModel,
                                           CAmbCardDiagnosticsVisitsTableModel,
                                           CAmbCardStatusActionsTableModel,
                                           CAmbCardVisitTableModel,
                                           CAmbCardAttachedFilesTableModel,
                                         )
from Registry.Utils               import getClientBanner, getClientInfo2
from Reports.ClientDiagnostics         import CClientDiagnostics


def getClientActions(clientId, filter, classCode, order = ['endDate DESC', 'id'], fieldName = None):
    db = QtGui.qApp.db
    tableAction = db.table('Action')
    queryTable = tableAction
    tableEvent = db.table('Event')
    queryTable = queryTable.leftJoin(tableEvent, tableEvent['id'].eq(tableAction['event_id']))
    tableActionType = db.table('ActionType')
    queryTable = queryTable.leftJoin(tableActionType, tableActionType['id'].eq(tableAction['actionType_id']))
    if fieldName in [u'setPerson_id', u'person_id']:
        tableSPWS = db.table('vrbPersonWithSpeciality')
        queryTable = queryTable.leftJoin(tableSPWS, tableSPWS['id'].eq(tableAction[fieldName]))
    cond = [tableAction['deleted'].eq(0),
                tableAction['status'].ne(CActionStatus.withoutResult),
                tableEvent['deleted'].eq(0),
                tableEvent['client_id'].eq(clientId),
                tableActionType['class'].eq(classCode)]
    begDate = filter.get('begDate', None)
    if begDate:
        cond.append(tableAction['endDate'].ge(begDate))
    endDate = filter.get('endDate', None)
    if endDate:
        cond.append(tableAction['endDate'].le(endDate))
    actionGroupId = filter.get('actionGroupId', None)
    if actionGroupId:
        cond.append(tableAction['actionType_id'].inlist(getActionTypeDescendants(actionGroupId, classCode)))
    office = filter.get('office', '')
    if office:
        cond.append(tableAction['office'].like(office))
    orgStructureId = filter.get('orgStructureId', None)
    if orgStructureId:
        cond.append(tableAction['person_id'].inlist(getOrgStructurePersonIdList(orgStructureId)))
    try:
        QtGui.QApplication.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
        return db.getIdList(queryTable,
               tableAction['id'].name(),
               cond,
               order)
    finally:
        QtGui.QApplication.restoreOverrideCursor()


def getClientVisits(clientId, filter):
    begDate        = filter.get('begDate')
    endDate        = filter.get('endDate')
    orgStructureId = filter.get('orgStructureId')
    specialityId   = filter.get('specialityId')
    personId       = filter.get('personId')
    sceneId        = filter.get('sceneId')
    eventTypeId    = filter.get('eventTypeId')
    serviceId      = filter.get('serviceId')

    db = QtGui.qApp.db

    tableVisit  = db.table('Visit')
    tableEvent  = db.table('Event')
    tablePerson = db.table('Person')

    queryTable = tableVisit.leftJoin(tableEvent, tableEvent['id'].eq(tableVisit['event_id']))

    cond = [tableVisit['deleted'].eq(0),
            tableEvent['deleted'].eq(0),
            tableEvent['client_id'].eq(clientId)]

    if begDate:
        cond.append(tableVisit['date'].dateGe(begDate))

    if endDate:
        cond.append(tableVisit['date'].dateLe(endDate))

    if specialityId or orgStructureId:
        queryTable = queryTable.leftJoin(tablePerson, tableVisit['person_id'].eq(tablePerson['id']))
        if specialityId:
            cond.append(tablePerson['speciality_id'].eq(specialityId))
        if orgStructureId:
            cond.append(tablePerson['orgStructure_id'].eq(orgStructureId))

    if personId:
        cond.append(tableVisit['person_id'].eq(personId))

    if sceneId:
        cond.append(tableVisit['scene_id'].eq(sceneId))

    if eventTypeId:
        cond.append(tableEvent['eventType_id'].eq(eventTypeId))

    if serviceId:
        cond.append(tableVisit['service_id'].eq(serviceId))

    return db.getIdList(queryTable, tableVisit['id'].name(), cond)


class CAmbCardMixin(CConstructHelperMixin):
    def __init__(self):
        CConstructHelperMixin.__init__(self)
        self.__ambCardVisitIsInitialised = False
        self.__ambCardFilesIsInitialised = False
        self.__ambCardDiagnosticsFilter = {}
        #self.btnAmbCardJournal.clicked.connect(self.on_btnAmbCardJournal_clicked)


    def preSetupUi(self):
        self.addModels('AmbCardDiagnostics', CAmbCardDiagnosticsTableModel(self))
        self.addModels('AmbCardDiagnosticsVisits', CAmbCardDiagnosticsVisitsTableModel(self))
        self.addModels('AmbCardDiagnosticsAccompDiagnostics', CAmbCardDiagnosticsAccompDiagnosticsTableModel(self))
        self.addModels('AmbCardDiagnosticsActions', CAmbCardDiagnosticsActionsTableModel(self))
        self.addModels('AmbCardDiagnosticsActionProperties', CActionPropertiesTableModel(self))
        self.addModels('AmbCardStatusActions', CAmbCardStatusActionsTableModel(self))
        self.addModels('AmbCardStatusActionProperties', CActionPropertiesTableModel(self))
        self.addModels('AmbCardDiagnosticActions', CAmbCardStatusActionsTableModel(self))
        self.addModels('AmbCardDiagnosticActionProperties', CActionPropertiesTableModel(self))
        self.addModels('AmbCardCureActions', CAmbCardStatusActionsTableModel(self))
        self.addModels('AmbCardCureActionProperties', CActionPropertiesTableModel(self))
        self.addModels('AmbCardMiscActions', CAmbCardStatusActionsTableModel(self))
        self.addModels('AmbCardMiscActionProperties', CActionPropertiesTableModel(self))
        self.addModels('AmbCardVisits', CAmbCardVisitTableModel(self))
        self.addModels('AmbCardFiles', CAmbCardAttachedFilesTableModel(self))

        self.addObject('actDiagnosticsShowPropertyHistory',  QtGui.QAction(u'Показать журнал значения свойства', self))
        self.addObject('actDiagnosticsShowPropertiesHistory',QtGui.QAction(u'Показать журнал значения свойств...', self))
        self.addObject('actStatusShowPropertyHistory',       QtGui.QAction(u'Показать журнал значения свойства', self))
        self.addObject('actStatusShowPropertiesHistory',     QtGui.QAction(u'Показать журнал значения свойств...', self))
        self.addObject('actDiagnosticShowPropertyHistory',   QtGui.QAction(u'Показать журнал значения свойства', self))
        self.addObject('actDiagnosticShowPropertiesHistory', QtGui.QAction(u'Показать журнал значения свойств...', self))
        self.addObject('actCureShowPropertyHistory',         QtGui.QAction(u'Показать журнал значения свойства', self))
        self.addObject('actCureShowPropertiesHistory',       QtGui.QAction(u'Показать журнал значения свойств...', self))
        self.addObject('actMiscShowPropertyHistory',         QtGui.QAction(u'Показать журнал значения свойства', self))
        self.addObject('actMiscShowPropertiesHistory',       QtGui.QAction(u'Показать журнал значения свойств...', self))
        self.addObject('actAmbCardPrintEvents',              QtGui.QAction(u'Напечатать список диагнозов', self))
        self.addObject('actAmbCardPrintCaseHistory',         getPrintAction(self, 'caseHistory', u'Напечатать карту'))
#        self.actAmbCardPrintEvents.setShortcut('F6')
        self.addObject('mnuAmbCardPrintEvents',              QtGui.QMenu(self))
        self.mnuAmbCardPrintEvents.addAction(self.actAmbCardPrintEvents)
        self.connect(self.actAmbCardPrintEvents, SIGNAL('triggered()'), self.on_actAmbCardPrintEvents_triggered)
        self.mnuAmbCardPrintEvents.addAction(self.actAmbCardPrintCaseHistory)
        self.addObject('actAmbCardActionTypeGroupId',        QtGui.QAction(u'Фильтровать по группе Действия', self))
        self.addObject('actAmbCardPrintAction',              getPrintAction(self, None, u'Напечатать по шаблону', False))
        self.addObject('actAmbCardPrintActions',             QtGui.QAction(u'Напечатать список мероприятий', self))
        self.addObject('actAmbCardPrintActionsHistory',      getPrintAction(self, 'actionsHistory', u'Напечатать карту мероприятий', False))
#        self.actAmbCardPrintActions.setShortcut('F6')
        self.addObject('mnuAmbCardPrintActions',             QtGui.QMenu(self))
        self.mnuAmbCardPrintActions.addAction(self.actAmbCardPrintAction)
        self.mnuAmbCardPrintActions.addAction(self.actAmbCardPrintActions)
        self.mnuAmbCardPrintActions.addAction(self.actAmbCardPrintActionsHistory)


    def postSetupUi(self):
        self.setModels(self.tblAmbCardDiagnostics, self.modelAmbCardDiagnostics, self.selectionModelAmbCardDiagnostics)
        self.setModels(self.tblAmbCardDiagnosticsVisits, self.modelAmbCardDiagnosticsVisits, self.selectionModelAmbCardDiagnosticsVisits)
        self.setModels(self.tblAmbCardDiagnosticsAccompDiagnostics, self.modelAmbCardDiagnosticsAccompDiagnostics, self.selectionModelAmbCardDiagnosticsAccompDiagnostics)
        self.setModels(self.tblAmbCardDiagnosticsActions, self.modelAmbCardDiagnosticsActions, self.selectionModelAmbCardDiagnosticsActions)
        self.setModels(self.tblAmbCardDiagnosticsActionProperties, self.modelAmbCardDiagnosticsActionProperties, self.selectionModelAmbCardDiagnosticsActionProperties)
        self.setModels(self.tblAmbCardStatusActions, self.modelAmbCardStatusActions, self.selectionModelAmbCardStatusActions)
        self.setModels(self.tblAmbCardStatusActionProperties, self.modelAmbCardStatusActionProperties, self.selectionModelAmbCardStatusActionProperties)
        self.setModels(self.tblAmbCardDiagnosticActions, self.modelAmbCardDiagnosticActions, self.selectionModelAmbCardDiagnosticActions)
        self.setModels(self.tblAmbCardDiagnosticActionProperties, self.modelAmbCardDiagnosticActionProperties, self.selectionModelAmbCardDiagnosticActionProperties)
        self.setModels(self.tblAmbCardCureActions, self.modelAmbCardCureActions, self.selectionModelAmbCardCureActions)
        self.setModels(self.tblAmbCardCureActionProperties, self.modelAmbCardCureActionProperties, self.selectionModelAmbCardCureActionProperties)
        self.setModels(self.tblAmbCardMiscActions, self.modelAmbCardMiscActions, self.selectionModelAmbCardMiscActions)
        self.setModels(self.tblAmbCardMiscActionProperties, self.modelAmbCardMiscActionProperties, self.selectionModelAmbCardMiscActionProperties)
        self.setModels(self.tblAmbCardVisits, self.modelAmbCardVisits, self.selectionModelAmbCardVisits)
        self.setModels(self.tblAmbCardAttachedFiles, self.modelAmbCardFiles, self.selectionModelAmbCardFiles)

        self.tblAmbCardStatusActions.createPopupMenu([self.actAmbCardActionTypeGroupId])
        self.tblAmbCardDiagnosticActions.createPopupMenu([self.actAmbCardActionTypeGroupId])
        self.tblAmbCardCureActions.createPopupMenu([self.actAmbCardActionTypeGroupId])
        self.tblAmbCardMiscActions.createPopupMenu([self.actAmbCardActionTypeGroupId])

        self.cmbAmbCardDiagnosticsPurpose.setTable('rbEventTypePurpose', True)
        self.cmbAmbCardDiagnosticsSpeciality.setTable('rbSpeciality', True)
        self.cmbHealthGroup.setTable('rbHealthGroup', True)

        self.cmbAmbCardStatusGroup.setClass(0)
        self.cmbAmbCardDiagnosticGroup.setClass(1)
        self.cmbAmbCardCureGroup.setClass(2)
        self.cmbAmbCardMiscGroup.setClass(3)

        self.tabAmbCardContent.setCurrentIndex(0)
        self.btnAmbCardPrint.setMenu(self.mnuAmbCardPrintEvents)

        self.cmbAmbCardVisitSpeciality.setTable('rbSpeciality')
        self.cmbAmbCardVisitScene.setTable('rbScene')
        self.cmbAmbCardVisitEventType.setTable('EventType')
        self.cmbAmbCardVisitService.setTable('rbService')

        self.connect(self.tblAmbCardStatusActions.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setActionsOrderByColumn)
        self.connect(self.tblAmbCardDiagnosticActions.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setActionsOrderByColumn)
        self.connect(self.tblAmbCardCureActions.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setActionsOrderByColumn)
        self.connect(self.tblAmbCardMiscActions.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setActionsOrderByColumn)
        self.connect(self.btnAmbCardJournal,  SIGNAL('clicked()'),  self.on_btnAmbCardJournalClicked)
        self.tblAmbCardStatusActions.enableColsHide()
        self.tblAmbCardDiagnosticActions.enableColsHide()
        self.tblAmbCardCureActions.enableColsHide()
        self.tblAmbCardMiscActions.enableColsHide()
        self.tblAmbCardStatusActions.enableColsMove()
        self.tblAmbCardDiagnosticActions.enableColsMove()
        self.tblAmbCardCureActions.enableColsMove()
        self.tblAmbCardMiscActions.enableColsMove()
        self.tblAmbCardAttachedFiles.setFlags(CAttachFilesTableFlag.canRead)

        self.prepareActionTable(self.tblAmbCardDiagnosticsActionProperties, self.actDiagnosticsShowPropertyHistory, self.actDiagnosticsShowPropertiesHistory)
        self.prepareActionTable(self.tblAmbCardStatusActionProperties, self.actStatusShowPropertyHistory, self.actStatusShowPropertiesHistory)
        self.prepareActionTable(self.tblAmbCardDiagnosticActionProperties, self.actDiagnosticShowPropertyHistory, self.actDiagnosticShowPropertiesHistory)
        self.prepareActionTable(self.tblAmbCardCureActionProperties, self.actCureShowPropertyHistory, self.actCureShowPropertiesHistory)
        self.prepareActionTable(self.tblAmbCardMiscActionProperties, self.actMiscShowPropertyHistory, self.actMiscShowPropertiesHistory)

        self.connect(self.cmdAmbCardDiagnosticsButtonBox, SIGNAL('clicked(QAbstractButton*)'), self.on_cmdAmbCardDiagnosticsButtonBox_clicked)
        self.connect(self.cmdAmbCardStatusButtonBox, SIGNAL('clicked(QAbstractButton*)'), self.on_cmdAmbCardStatusButtonBox_clicked)
        self.connect(self.cmdAmbCardDiagnosticButtonBox, SIGNAL('clicked(QAbstractButton*)'), self.on_cmdAmbCardDiagnosticButtonBox_clicked)
        self.connect(self.cmdAmbCardCureButtonBox, SIGNAL('clicked(QAbstractButton*)'), self.on_cmdAmbCardCureButtonBox_clicked)
        self.connect(self.cmdAmbCardMiscButtonBox, SIGNAL('clicked(QAbstractButton*)'), self.on_cmdAmbCardMiscButtonBox_clicked)
        self.connect(self.cmdAmbCardVisitButtonBox, SIGNAL('clicked(QAbstractButton*)'), self.on_cmdAmbCardVisitButtonBox_clicked)
        


    def resetWidgets(self):
        self.on_cmdAmbCardDiagnosticsButtonBox_reset()
        self.on_cmdAmbCardStatusButtonBox_reset()
        self.on_cmdAmbCardDiagnosticButtonBox_reset()
        self.on_cmdAmbCardCureButtonBox_reset()
        self.on_cmdAmbCardMiscButtonBox_reset()
        self.on_cmdAmbCardVisitButtonBox_reset()

        self.on_cmdAmbCardDiagnosticsButtonBox_apply()
        self.on_cmdAmbCardStatusButtonBox_apply()
        self.on_cmdAmbCardDiagnosticButtonBox_apply()
        self.on_cmdAmbCardCureButtonBox_apply()
        self.on_cmdAmbCardMiscButtonBox_apply()

        self.__ambCardVisitIsInitialised = False
        self.__ambCardFilesIsInitialised = False
        self.__ambCardDiagnosticsFilter = {}
        self.on_tabAmbCardContent_currentChanged(self.tabAmbCardContent.currentIndex())


    @pyqtSignature('')
    def on_tblAmbCardStatusActions_popupMenuAboutToShow(self):
        notEmpty = self.modelAmbCardStatusActions.rowCount() > 0
        self.actAmbCardActionTypeGroupId.setEnabled(notEmpty)


    @pyqtSignature('')
    def on_tblAmbCardDiagnosticActions_popupMenuAboutToShow(self):
        notEmpty = self.modelAmbCardDiagnosticActions.rowCount() > 0
        self.actAmbCardActionTypeGroupId.setEnabled(notEmpty)


    @pyqtSignature('')
    def on_tblAmbCardCureActions_popupMenuAboutToShow(self):
        notEmpty = self.modelAmbCardCureActions.rowCount() > 0
        self.actAmbCardActionTypeGroupId.setEnabled(notEmpty)


    @pyqtSignature('')
    def on_tblAmbCardMiscActions_popupMenuAboutToShow(self):
        notEmpty = self.modelAmbCardMiscActions.rowCount() > 0
        self.actAmbCardActionTypeGroupId.setEnabled(notEmpty)


    @pyqtSignature('')
    def on_actAmbCardActionTypeGroupId_triggered(self):
        groupId = None
        column = None
        table, updateFunction, cmbActionType, filter = self.getCurrentActionsTable()
        index = table.currentIndex()
        if index:
            row = index.row()
            column = index.column()
            record = index.model().getRecordByRow(row) if row >= 0 else None
            actionTypeId = forceRef(record.value('actionType_id')) if record else None
            actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None
            groupId = actionType.groupId if actionType else None
            if not groupId:
               groupId = actionTypeId if actionTypeId else None
        cmbActionType.setValue(groupId)
        self._setActionsOrderByColumn(column)


    def getCurrentActionsTable(self):
        index = self.tabAmbCardContent.currentIndex()
        if index in [1, 2, 3, 4]:
            return [(self.tblAmbCardStatusActions, self.updateAmbCardStatus, self.cmbAmbCardStatusGroup,
                     self.getAmbCardFilter(
                        self.edtAmbCardStatusBegDate,
                        self.edtAmbCardStatusEndDate,
                        self.cmbAmbCardStatusGroup,
                        self.edtAmbCardStatusOffice,
                        self.cmbAmbCardStatusOrgStructure
                        )),
                    (self.tblAmbCardDiagnosticActions, self.updateAmbCardDiagnostic, self.cmbAmbCardDiagnosticGroup,
                     self.getAmbCardFilter(
                        self.edtAmbCardDiagnosticBegDate,
                        self.edtAmbCardDiagnosticEndDate,
                        self.cmbAmbCardDiagnosticGroup,
                        self.edtAmbCardDiagnosticOffice,
                        self.cmbAmbCardDiagnosticOrgStructure
                        )),
                    (self.tblAmbCardCureActions, self.updateAmbCardCure, self.cmbAmbCardCureGroup,
                     self.getAmbCardFilter(
                        self.edtAmbCardCureBegDate,
                        self.edtAmbCardCureEndDate,
                        self.cmbAmbCardCureGroup,
                        self.edtAmbCardCureOffice,
                        self.cmbAmbCardCureOrgStructure
                        )),
                    (self.tblAmbCardMiscActions, self.updateAmbCardMisc, self.cmbAmbCardMiscGroup,
                     self.getAmbCardFilter(
                        self.edtAmbCardMiscBegDate,
                        self.edtAmbCardMiscEndDate,
                        self.cmbAmbCardMiscGroup,
                        self.edtAmbCardMiscOffice,
                        self.cmbAmbCardMiscOrgStructure
                        ))][index-1]


    def _setActionsOrderByColumn(self, column):
        table, updateFunction, cmbActionType, filter = self.getCurrentActionsTable()
        table.setOrder(column)
        fieldName = table.model().cols()[column].fields()[0]
        updateFunction(filter, table.currentItemId(), fieldName)


    def ambCardTableDoubleClicked(self, table, cmbActionType):
        actionTypeId = None
        column = None
        actionId = table.currentItemId()
        if actionId:
            index = table.currentIndex()
            if index:
                row = index.row()
                column = index.column()
                record = index.model().getRecordByRow(row) if row >= 0 else None
                actionTypeId = forceRef(record.value('actionType_id')) if record else None
        cmbActionType.setValue(actionTypeId)
        self._setActionsOrderByColumn(column)


    @pyqtSignature('QModelIndex')
    def on_tblAmbCardStatusActions_doubleClicked(self, index):
        self.ambCardTableDoubleClicked(self.tblAmbCardStatusActions, self.cmbAmbCardStatusGroup)


    @pyqtSignature('QModelIndex')
    def on_tblAmbCardDiagnosticActions_doubleClicked(self, index):
        self.ambCardTableDoubleClicked(self.tblAmbCardDiagnosticActions, self.cmbAmbCardDiagnosticGroup)


    @pyqtSignature('QModelIndex')
    def on_tblAmbCardCureActions_doubleClicked(self, index):
        self.ambCardTableDoubleClicked(self.tblAmbCardCureActions, self.cmbAmbCardCureGroup)


    @pyqtSignature('QModelIndex')
    def on_tblAmbCardMiscActions_doubleClicked(self, index):
        self.ambCardTableDoubleClicked(self.tblAmbCardMiscActions, self.cmbAmbCardMiscGroup)


    def prepareActionTable(self, tbl, *actions):
        tbl.setEditTriggers(QtGui.QAbstractItemView.SelectedClicked | QtGui.QAbstractItemView.DoubleClicked)
        tbl.model().setReadOnly(True)
        tbl.addPopupCopyCell()
        tbl.addPopupSeparator()
        for action in actions:
            if action == '-':
                tbl.addPopupSeparator()
            else:
                tbl.addPopupAction(action)


    def destroy(self):
        self.tblAmbCardDiagnostics.setModel(None)
        self.tblAmbCardDiagnosticsVisits.setModel(None)
        self.tblAmbCardDiagnosticsAccompDiagnostics.setModel(None)
        self.tblAmbCardDiagnosticsActions.setModel(None)
        self.tblAmbCardDiagnosticsActionProperties.setModel(None)
        self.tblAmbCardStatusActions.setModel(None)
        self.tblAmbCardStatusActionProperties.setModel(None)
        self.tblAmbCardDiagnosticActions.setModel(None)
        self.tblAmbCardDiagnosticActionProperties.setModel(None)
        self.tblAmbCardCureActions.setModel(None)
        self.tblAmbCardCureActionProperties.setModel(None)
        self.tblAmbCardMiscActions.setModel(None)
        self.tblAmbCardMiscActionProperties.setModel(None)
        self.tblAmbCardAttachedFiles.setModel(None)
        del self.modelAmbCardDiagnostics
        del self.modelAmbCardDiagnosticsVisits
        del self.modelAmbCardDiagnosticsAccompDiagnostics
        del self.modelAmbCardDiagnosticsActions
        del self.modelAmbCardDiagnosticsActionProperties
        del self.modelAmbCardStatusActions
        del self.modelAmbCardStatusActionProperties
        del self.modelAmbCardDiagnosticActions
        del self.modelAmbCardDiagnosticActionProperties
        del self.modelAmbCardCureActions
        del self.modelAmbCardCureActionProperties
        del self.modelAmbCardMiscActions
        del self.modelAmbCardMiscActionProperties
        del self.modelAmbCardFiles


    def updateAmbCardDiagnostics(self, filter):
        """
            в соответствии с фильтром обновляет список Diagnostics на вкладке AmbCard/Diagnostics.
        """
        self.__ambCardDiagnosticsFilter = filter
        db = QtGui.qApp.db
        tableDiagnostic = db.table('Diagnostic')
        queryTable = tableDiagnostic
        tableEvent = db.table('Event')
        queryTable = queryTable.leftJoin(tableEvent, tableEvent['id'].eq(tableDiagnostic['event_id']))
        cond = [tableDiagnostic['deleted'].eq(0),
                tableEvent['deleted'].eq(0),
                tableEvent['client_id'].eq(self.currentClientId())]
        begDate = filter.get('begDate', None)
        if begDate:
            cond.append(tableDiagnostic['endDate'].ge(begDate))
        endDate = filter.get('endDate', None)
        if endDate:
            cond.append(tableDiagnostic['endDate'].le(endDate))
        eventPurposeId = filter.get('eventPurposeId', None)
        if eventPurposeId:
            tableEventType = db.table('EventType')
            queryTable = queryTable.leftJoin(tableEventType, tableEventType['id'].eq(tableEvent['eventType_id']))
            cond.append(tableEventType['purpose_id'].eq(eventPurposeId))
        specialityId = filter.get('specialityId', None)
        if specialityId:
            cond.append(tableDiagnostic['speciality_id'].eq(specialityId))
        personId = filter.get('personId', None)
        if personId:
            cond.append(tableDiagnostic['person_id'].eq(personId))
        healthGroupId = filter.get('healthGroupId', None)
        if healthGroupId:
            cond.append(tableDiagnostic['healthGroup_id'].eq(healthGroupId))
        tableDiagnosisType = db.table('rbDiagnosisType')
        queryTable = queryTable.leftJoin(tableDiagnosisType, tableDiagnosisType['id'].eq(tableDiagnostic['diagnosisType_id']))
        cond.append(tableDiagnosisType['code'].inlist(['1', '2', '4']))
        try:
            QtGui.QApplication.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
            idList = db.getIdList(queryTable,
                           tableDiagnostic['id'].name(),
                           cond,
                           ['endDate DESC', 'id'])
            self.tblAmbCardDiagnostics.setIdList(idList, None)
        finally:
            QtGui.QApplication.restoreOverrideCursor()


    def focusAmbCardDiagnostics(self):
        self.tblAmbCardDiagnostics.setFocus(Qt.TabFocusReason)


    def updateAmbCardDiagnosticsInfo(self): # обновить таблички внизу AmbCardDiagnostic
        diagnosticId = self.tblAmbCardDiagnostics.currentItemId()
        if diagnosticId:
            eventId = forceRef(QtGui.qApp.db.translate('Diagnostic', 'id', diagnosticId, 'event_id'))
        else:
            eventId = None
        pageIndex = self.tabAmbCardDiagnosticDetails.currentIndex()
        if pageIndex == 0:
            self.updateAmbCardDiagnosticsEvent(eventId, diagnosticId)
        elif pageIndex == 1:
            self.updateAmbCardDiagnosticsVisits(eventId)
        elif pageIndex == 2:
            self.updateAmbCardDiagnosticsAccompDiagnostics(eventId, diagnosticId)
        elif pageIndex == 3:
            self.updateAmbCardDiagnosticsActions(eventId)


    def updateAmbCardDiagnosticsEvent(self, eventId, diagnosticId):
        db = QtGui.qApp.db
        if eventId:
            stmt = 'SELECT Event.externalId AS externalId, '\
                   'EventType.name AS eventTypeName, ' \
                   'Organisation.shortName AS orgName, ' \
                   'P1.name AS setPersonName, Event.setDate, ' \
                   'P2.name AS execPersonName, Event.execDate, ' \
                   'Event.isPrimary, Event.order, Event.nextEventDate, Event.nextEventDate, Event.payStatus, ' \
                   'rbResult.name AS result ' \
                   'FROM Event ' \
                   'LEFT JOIN EventType    ON EventType.id = Event.eventType_id ' \
                   'LEFT JOIN Organisation ON Organisation.id = Event.org_id ' \
                   'LEFT JOIN vrbPersonWithSpeciality AS P1 ON P1.id = Event.setPerson_id ' \
                   'LEFT JOIN vrbPersonWithSpeciality AS P2 ON P2.id = Event.execPerson_id ' \
                   'LEFT JOIN rbResult     ON rbResult.id = Event.result_id ' \
                   'WHERE Event.id=%d' % eventId
            query = db.query(stmt)
            if query.first():
                record=query.record()
            else:
                record=QtSql.QSqlRecord()
        else:
            record=QtSql.QSqlRecord()
        if diagnosticId:
            stmt = 'SELECT createPerson.name AS createName,  modifyPerson.name AS modityName, ' \
                   'Diagnosis.MKB, Diagnosis.MKBEx ' \
                   'FROM '\
                   'Diagnostic '\
                   'LEFT JOIN vrbPersonWithSpeciality AS createPerson ON createPerson.id = Diagnostic.createPerson_id '\
                   'LEFT JOIN vrbPersonWithSpeciality AS modifyPerson ON modifyPerson.id = Diagnostic.modifyPerson_id '\
                   'LEFT JOIN Diagnosis ON Diagnosis.id = Diagnostic.diagnosis_id '\
                   'WHERE Diagnostic.id=%d' % diagnosticId
            query = db.query(stmt)
            if query.first():
                diagnosticRecord=query.record()
            else:
                diagnosticRecord=QtSql.QSqlRecord()
        else:
            diagnosticRecord=QtSql.QSqlRecord()

        self.lblAmbCardEventExtIdValue.setText(forceString(record.value('externalId')))
        self.lblAmbCardEventTypeValue.setText(forceString(record.value('eventTypeName')))
        self.lblAmbCardEventOrgValue.setText(forceString(record.value('orgName')))
        self.lblAmbCardEventSetPersonValue.setText(forceString(record.value('setPersonName')))
        self.lblAmbCardEventSetDateValue.setText(forceString(record.value('setDate')))
        self.lblAmbCardEventExecPersonValue.setText(forceString(record.value('execPersonName')))
        self.lblAmbCardEventExecDateValue.setText(forceString(record.value('execDate')))
        self.lblAmbCardEventOrderValue.setText(getOrderText(forceInt(record.value('order'))))
        self.chkAmbCardEventPrimary.setChecked(forceInt(record.value('isPrimary'))==1)

        self.lblAmbCardEventCreatePersonValue.setText(forceString(diagnosticRecord.value('createName')))
        self.lblAmbCardEventModifyPersonValue.setText(forceString(diagnosticRecord.value('modifyName')))
        MKB = forceString(diagnosticRecord.value('MKB'))
        MKBName = getMKBName(MKB) if MKB else ''
        self.lblAmbCardEventDiagValue.setText(MKB)
        self.lblAmbCardEventDiagName.setText(MKBName)
        MKBEx = forceString(diagnosticRecord.value('MKBEx'))
        MKBExName = getMKBName(MKBEx) if MKBEx else ''
        self.lblAmbCardEventDiagExValue.setText(MKBEx)
        self.lblAmbCardEventDiagExName.setText(MKBExName)

        self.lblAmbCardEventNextDateValue.setText(forceString(forceDate(record.value('nextEventDate'))))
        self.lblAmbCardEventResultValue.setText(forceString(record.value('result')))
        self.lblAmbCardEventPayStatusValue.setText(payStatusText(forceInt(record.value('payStatus'))))



    def updateAmbCardVisitEvent(self, eventId, diagnosticId=None):
        # tabAmbCardVisit

        db = QtGui.qApp.db
        if eventId:
            stmt = 'SELECT Event.externalId AS externalId, '\
                   'EventType.name AS eventTypeName, ' \
                   'Organisation.shortName AS orgName, ' \
                   'P1.name AS setPersonName, Event.setDate, ' \
                   'P2.name AS execPersonName, Event.execDate, ' \
                   'Event.isPrimary, Event.order, Event.nextEventDate, Event.nextEventDate, Event.payStatus, ' \
                   'rbResult.name AS result ' \
                   'FROM Event ' \
                   'LEFT JOIN EventType    ON EventType.id = Event.eventType_id ' \
                   'LEFT JOIN Organisation ON Organisation.id = Event.org_id ' \
                   'LEFT JOIN vrbPersonWithSpeciality AS P1 ON P1.id = Event.setPerson_id ' \
                   'LEFT JOIN vrbPersonWithSpeciality AS P2 ON P2.id = Event.execPerson_id ' \
                   'LEFT JOIN rbResult     ON rbResult.id = Event.result_id ' \
                   'WHERE Event.id=%d' % eventId
            query = db.query(stmt)
            if query.first():
                record=query.record()
            else:
                record=QtSql.QSqlRecord()
        else:
            record=QtSql.QSqlRecord()
        if diagnosticId:
            stmt = 'SELECT createPerson.name AS createName,  modifyPerson.name AS modityName, ' \
                   'Diagnosis.MKB, Diagnosis.MKBEx ' \
                   'FROM '\
                   'Diagnostic '\
                   'LEFT JOIN vrbPersonWithSpeciality AS createPerson ON createPerson.id = Diagnostic.createPerson_id '\
                   'LEFT JOIN vrbPersonWithSpeciality AS modifyPerson ON modifyPerson.id = Diagnostic.modifyPerson_id '\
                   'LEFT JOIN Diagnosis ON Diagnosis.id = Diagnostic.diagnosis_id '\
                   'WHERE Diagnostic.id=%d' % diagnosticId
            query = db.query(stmt)
            if query.first():
                diagnosticRecord=query.record()
            else:
                diagnosticRecord=QtSql.QSqlRecord()
        else:
            diagnosticRecord=QtSql.QSqlRecord()

        self.lblAmbCardEventExtIdValue_2.setText(forceString(record.value('externalId')))
        self.lblAmbCardEventTypeValue_2.setText(forceString(record.value('eventTypeName')))
        self.lblAmbCardEventOrgValue_2.setText(forceString(record.value('orgName')))
        self.lblAmbCardEventSetPersonValue_2.setText(forceString(record.value('setPersonName')))
        self.lblAmbCardEventSetDateValue_2.setText(forceString(record.value('setDate')))
        self.lblAmbCardEventExecPersonValue_2.setText(forceString(record.value('execPersonName')))
        self.lblAmbCardEventExecDateValue_2.setText(forceString(record.value('execDate')))
        self.lblAmbCardEventOrderValue_2.setText(getOrderText(forceInt(record.value('order'))))
        self.chkAmbCardEventPrimary_2.setChecked(forceInt(record.value('isPrimary'))==1)

        self.lblAmbCardEventCreatePersonValue_2.setText(forceString(diagnosticRecord.value('createName')))
        self.lblAmbCardEventModifyPersonValue_2.setText(forceString(diagnosticRecord.value('modifyName')))

        MKB = forceString(diagnosticRecord.value('MKB'))
        MKBName = getMKBName(MKB) if MKB else ''
        self.lblAmbCardEventDiagValue_2.setText(MKB)
        self.lblAmbCardEventDiagName_2.setText(MKBName)

        MKBEx = forceString(diagnosticRecord.value('MKBEx'))
        MKBExName = getMKBName(MKBEx) if MKBEx else ''
        self.lblAmbCardEventDiagExValue_2.setText(MKBEx)
        self.lblAmbCardEventDiagExName_2.setText(MKBExName)

        self.lblAmbCardEventNextDateValue_2.setText(forceString(forceDate(record.value('nextEventDate'))))
        self.lblAmbCardEventResultValue_2.setText(forceString(record.value('result')))
        self.lblAmbCardEventPayStatusValue_2.setText(payStatusText(forceInt(record.value('payStatus'))))


    def updateAmbCardDiagnosticsVisits(self,  eventId):
        if eventId:
            db = QtGui.qApp.db
            table = db.table('Visit')
            try:
                QtGui.QApplication.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
                idList = db.getIdList(table,
                                      table['id'].name(),
                                      [table['event_id'].eq(eventId), table['deleted'].eq(0)],
                                      ['date', 'id'])
            finally:
                QtGui.QApplication.restoreOverrideCursor()
        else:
            idList = []
        self.tblAmbCardDiagnosticsVisits.setIdList(idList, None)


    def updateAmbCardDiagnosticsAccompDiagnostics(self, eventId, diagnosticId):
        if eventId:
            db = QtGui.qApp.db
            table = db.table('Diagnostic')
            specialityId = db.translate(table, 'id', diagnosticId, 'speciality_id')
            tableDiagnosisType = db.table('rbDiagnosisType')
            queryTable = table.leftJoin(tableDiagnosisType, tableDiagnosisType['id'].eq(table['diagnosisType_id']))
            cond = [table['event_id'].eq(eventId),
                    table['deleted'].eq(0),
                    table['speciality_id'].eq(specialityId),
                    'NOT ('+tableDiagnosisType['code'].inlist(['1', '2', '4'])+')']
            try:
                QtGui.QApplication.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
                idList = db.getIdList(queryTable, table['id'].name(), cond, ['endDate', 'id'])
            finally:
                QtGui.QApplication.restoreOverrideCursor()
        else:
            idList = []
        self.tblAmbCardDiagnosticsAccompDiagnostics.setIdList(idList, None)


    def updateAmbCardDiagnosticsActions(self, eventId):
        if eventId:
            db = QtGui.qApp.db
            table = db.table('Action')
            try:
                QtGui.QApplication.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
                idList = db.getIdList(table,
                                      table['id'].name(),
                                      [table['event_id'].eq(eventId),
                                       table['deleted'].eq(0),
                                       table['status'].ne(CActionStatus.withoutResult)
                                      ],
                                      ['endDate DESC', 'id'])
            finally:
                QtGui.QApplication.restoreOverrideCursor()
        else:
            idList = []
        self.tblAmbCardDiagnosticsActions.setIdList(idList, None)


    def resetAmbCardFilter(self, edtBegDate, edtEndDate, cmbGroup, edtOffice, cmbOrgStructure):
        edtBegDate.setDate(QDate())
        edtEndDate.setDate(QDate())
        cmbGroup.setValue(None)
        edtOffice.setText('')
        cmbOrgStructure.setValue(None)


    def getAmbCardFilter(self, edtBegDate, edtEndDate, cmbGroup, edtOffice, cmbOrgStructure):
        filter = {}
        filter['begDate'] = edtBegDate.date()
        filter['endDate'] = edtEndDate.date()
        filter['actionGroupId'] = cmbGroup.value()
        filter['office'] = forceString(edtOffice.text())
        filter['orgStructureId'] = cmbOrgStructure.value()
        return filter


    def selectAmbCardActions(self, filter, classCode, order, fieldName):
        return getClientActions(self.currentClientId(), filter, classCode, order, fieldName)


    def selectAmbCardVisits(self, filter):
        return getClientVisits(self.currentClientId(), filter)



    def updateAmbCardStatus(self, filter, posToId=None, fieldName=None):
        """
            в соответствии с фильтром обновляет список Actions на вкладке AmbCard/Status.
        """
        self.__ambCardStatusFilter = filter
        order = self.tblAmbCardStatusActions.order() if self.tblAmbCardStatusActions.order() else ['endDate DESC', 'id']
        self.tblAmbCardStatusActions.setIdList(self.selectAmbCardActions(filter, 0, order, fieldName), posToId)


    def focusAmbCardStatusActions(self):
        self.tblAmbCardStatusActions.setFocus(Qt.TabFocusReason)


    def updateAmbCardDiagnostic(self, filter, posToId=None, fieldName=None):
        """
            в соответствии с фильтром обновляет список Actions на вкладке AmbCard/Diagnostic.
        """
        self.__ambCardDiagnosticFilter = filter
        order = self.tblAmbCardDiagnosticActions.order() if self.tblAmbCardDiagnosticActions.order() else ['endDate DESC', 'id']
        self.tblAmbCardDiagnosticActions.setIdList(self.selectAmbCardActions(filter, 1, order, fieldName), posToId)


    def focusAmbCardDiagnosticActions(self):
        self.tblAmbCardDiagnosticActions.setFocus(Qt.TabFocusReason)


    def updateAmbCardCure(self, filter, posToId=None, fieldName=None):
        """
            в соответствии с фильтром обновляет список Actions на вкладке AmbCard/Cure.
        """
        self.__ambCardCureFilter = filter
        order = self.tblAmbCardCureActions.order() if self.tblAmbCardCureActions.order() else ['endDate DESC', 'id']
        self.tblAmbCardCureActions.setIdList(self.selectAmbCardActions(filter, 2, order, fieldName), posToId)


    def focusAmbCardCureActions(self):
        self.tblAmbCardCureActions.setFocus(Qt.TabFocusReason)


    def updateAmbCardMisc(self, filter, posToId=None, fieldName=None):
        """
            в соответствии с фильтром обновляет список Actions на вкладке AmbCard/Cure.
        """
        self.__ambCardMiscFilter = filter
        order = self.tblAmbCardMiscActions.order() if self.tblAmbCardMiscActions.order() else ['endDate DESC', 'id']
        self.tblAmbCardMiscActions.setIdList(self.selectAmbCardActions(filter, 3, order, fieldName), posToId)


    def focusAmbCardMiscActions(self):
        self.tblAmbCardMiscActions.setFocus(Qt.TabFocusReason)



    def updateAmbCardVisit(self, filter):
        self._ambCardVisitFilter = filter
        self.tblAmbCardVisits.setIdList(self.selectAmbCardVisits(filter), None)

    #### AmbCard page: Diagnostics page ##############

    @pyqtSignature('int')
    def on_cmbAmbCardDiagnosticsSpeciality_currentIndexChanged(self, index):
        self.cmbAmbCardDiagnosticsPerson.setSpecialityId(self.cmbAmbCardDiagnosticsSpeciality.value())


    @pyqtSignature('QAbstractButton*')
    def on_cmdAmbCardDiagnosticsButtonBox_clicked(self, button):
        buttonCode = self.cmdAmbCardDiagnosticsButtonBox.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_cmdAmbCardDiagnosticsButtonBox_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_cmdAmbCardDiagnosticsButtonBox_reset()


#    @pyqtSignature('')
    def on_cmdAmbCardDiagnosticsButtonBox_reset(self):
        self.edtAmbCardDiagnosticsBegDate.setDate(QDate())
        self.edtAmbCardDiagnosticsEndDate.setDate(QDate())
        self.cmbAmbCardDiagnosticsPurpose.setValue(None)
        self.cmbAmbCardDiagnosticsSpeciality.setValue(None)
        self.cmbAmbCardDiagnosticsPerson.setValue(None)
        self.cmbHealthGroup.setValue(None)


#    @pyqtSignature('')
    def on_cmdAmbCardDiagnosticsButtonBox_apply(self):
        filter = {}
        filter['begDate'] = self.edtAmbCardDiagnosticsBegDate.date()
        filter['endDate'] = self.edtAmbCardDiagnosticsEndDate.date()
        filter['eventPurposeId'] = self.cmbAmbCardDiagnosticsPurpose.value()
        filter['specialityId'] = self.cmbAmbCardDiagnosticsSpeciality.value()
        filter['personId'] = self.cmbAmbCardDiagnosticsPerson.value()
        filter['healthGroupId'] = self.cmbHealthGroup.value()
        self.updateAmbCardDiagnostics(filter)
        self.focusAmbCardDiagnostics()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelAmbCardDiagnostics_currentRowChanged(self, current, previous):
        self.updateAmbCardDiagnosticsInfo()



    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelAmbCardVisits_currentRowChanged(self, current, previous):
        if current and current.isValid():
            row = current.row()
            record = self.modelAmbCardVisits.getRecordByRow(row)
            self.updateAmbCardVisitEvent(forceRef(record.value('event_id')))
        else:
            self.updateAmbCardVisitEvent(None)


    @pyqtSignature('int')
    def on_tabAmbCardDiagnosticDetails_currentChanged(self, index):
        self.updateAmbCardDiagnosticsInfo()


    def updateAmbCardPropertiesTable(self, index, tbl, previous=None):
        if previous:
            tbl.savePreferencesLoc(previous.row())
        row = index.row()
        record = index.model().getRecordByRow(row) if row >= 0 else None
        if record:
            clientId = self.currentClientId()
            action = CAction(record=record)
#            tblProps.model().setAction(action, self.clientSex, self.clientAge)
            tbl.model().setAction2(action, clientId)
            setActionPropertiesColumnVisible(action._actionType, tbl)
            tbl.resizeColumnsToContents()
            tbl.resizeRowsToContents()
            tbl.horizontalHeader().setStretchLastSection(True)
            tbl.loadPreferencesLoc(tbl.preferencesLocal, row)
        else:
            tbl.model().setAction2(None, None)


    def updateAmbCardPrintActionAction(self, index):
        row = index.row()
        record = index.model().getRecordByRow(row) if row >= 0 else None
        actionTypeId = forceRef(record.value('actionType_id')) if record else None
        actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None
        context = actionType.context if actionType else None
        self.actAmbCardPrintAction.setContext(context, False)


    def on_btnAmbCardJournalClicked(self):
        dialog = CAmbCardJournalDialog(self, clientId=self.currentClientId())
        dialog.exec_()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelAmbCardDiagnosticsActions_currentRowChanged(self, current, previous):
        self.updateAmbCardPropertiesTable(current, self.tblAmbCardDiagnosticsActionProperties, previous)
        self.updateAmbCardPrintActionAction(current)


    @pyqtSignature('')
    def on_actDiagnosticsShowPropertyHistory_triggered(self):
        self.tblAmbCardDiagnosticsActionProperties.showHistory()


    @pyqtSignature('')
    def on_actDiagnosticsShowPropertiesHistory_triggered(self):
        self.tblAmbCardDiagnosticsActionProperties.showHistoryEx()


    @pyqtSignature('int')
    def on_tabAmbCardContent_currentChanged(self, index):
        self.btnAmbCardPrint.setMenu(self.mnuAmbCardPrintEvents if index == 0 else self.mnuAmbCardPrintActions)

        if index == self.tabAmbCardContent.indexOf(self.tabAmbCardVisit):
            if not self.__ambCardVisitIsInitialised:
                self.on_cmdAmbCardVisitButtonBox_apply()
                self.__ambCardVisitIsInitialised = True
        elif index == self.tabAmbCardContent.indexOf(self.tabAttachedFiles):
            clientId = self.currentClientId()
            if not self.__ambCardFilesIsInitialised:
                self.modelAmbCardFiles.loadItems(clientId)
                self.tblAmbCardAttachedFiles.resetSortIndicator()
                self.__ambCardFilesIsInitialised = True


    @pyqtSignature('')
    def on_actAmbCardPrintEvents_triggered(self):
        filter = {'clientId':self.currentClientId()}
        if self.__ambCardDiagnosticsFilter:
            filter.update(self.__ambCardDiagnosticsFilter)
        CClientDiagnostics(self).oneShot(filter)


    @pyqtSignature('int')
    def on_actAmbCardPrintCaseHistory_printByTemplate(self, templateId):
        clientId = self.currentClientId()
        clientInfo = getClientInfo2(clientId)
        db = QtGui.qApp.db
        tableEvent = db.table('Event')
        tableDiagnostic = db.table('Diagnostic')
        table = tableEvent.leftJoin(tableDiagnostic, tableDiagnostic['event_id'].eq(tableEvent['id']))
        cond = tableDiagnostic['id'].inlist(self.modelAmbCardDiagnostics.idList())
        idList = db.getDistinctIdList(table, tableEvent['id'].name(), cond, [tableEvent['setDate'].name(), tableEvent['id'].name()])
        events = CLocEventInfoList(clientInfo.context, idList)
        diagnosises = CDiagnosisInfoList(clientInfo.context, clientId)
        getTempInvalidList = lambda begDate=None, endDate=None, types=None: CTempInvalidInfoList._get(clientInfo.context, clientId, begDate, endDate, types)
        data = {'client':clientInfo,
                'events':events,
                'diagnosises':diagnosises,
                'getTempInvalidList': getTempInvalidList,
                'tempInvalids':getTempInvalidList()}
        QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    @pyqtSignature('')
    def on_mnuAmbCardPrintActions_aboutToShow(self):
        index = self.tabAmbCardContent.currentIndex()
        if index:
            table = [ self.tblAmbCardStatusActions,
                      self.tblAmbCardDiagnosticActions,
                      self.tblAmbCardCureActions,
                      self.tblAmbCardMiscActions,
                    ][index-1]
            self.updateAmbCardPrintActionAction(table.currentIndex())


    @pyqtSignature('int')
    def on_actAmbCardPrintAction_printByTemplate(self, templateId):
        index = self.tabAmbCardContent.currentIndex()
        if index:
            table = [ self.tblAmbCardStatusActions,
                      self.tblAmbCardDiagnosticActions,
                      self.tblAmbCardCureActions,
                      self.tblAmbCardMiscActions,
                    ][index-1]
            actionId = table.currentItemId()
            if actionId:
                eventId = forceRef(QtGui.qApp.db.translate('Action', 'id', actionId, 'event_id'))
                context = CInfoContext()
                eventInfo = context.getInstance(CEventInfo, eventId)
                eventActions = eventInfo.actions
                eventActions._idList = [actionId]
                eventActions._items  = [context.getInstance(CActionInfo, actionId)]
                eventActions._loaded = True
                #currentActionIndex = 0
                action = eventInfo.actions[0]
                data = { 'event' : eventInfo,
                         'action': action,
                         'client': eventInfo.client,
                         'actions':eventActions,
                         'currentActionIndex': 0,
                         'tempInvalid': None
                       }
                QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    @pyqtSignature('')
    def on_actAmbCardPrintActions_triggered(self):
        index = self.tabAmbCardContent.currentIndex()
        if index:
            table, title = [ (self.tblAmbCardStatusActions,      u'статус'),
                             (self.tblAmbCardDiagnosticActions,  u'диагностика'),
                             (self.tblAmbCardCureActions,        u'лечение'),
                             (self.tblAmbCardMiscActions,        u'прочие мероприятия'),
                           ][index-1]
            table.setReportHeader(u'Список мероприятий (%s) пациента' % title)
            table.setReportDescription(getClientBanner(self.currentClientId()))
            table.printContent()


    @pyqtSignature('int')
    def on_actAmbCardPrintActionsHistory_printByTemplate(self, templateId):
        index = self.tabAmbCardContent.currentIndex()
        if index:
            clientId = self.currentClientId()
            clientInfo = getClientInfo2(clientId)
            model = [ self.modelAmbCardStatusActions,
                      self.modelAmbCardDiagnosticActions,
                      self.modelAmbCardCureActions,
                      self.modelAmbCardMiscActions,
                    ][index-1]
            idList = model.idList()
            actions = CLocActionInfoList(clientInfo.context, idList, clientInfo.sexCode, clientInfo.ageTuple)
            diagnosises = CDiagnosisInfoList(clientInfo.context, clientId)
            getTempInvalidList = lambda begDate=None, endDate=None, types=None: CTempInvalidInfoList._get(clientInfo.context, clientId, begDate, endDate, types)
            data = {'client':clientInfo,
                    'actions':actions,
                    'diagnosises':diagnosises,
                    'getTempInvalidList': getTempInvalidList,
                    'tempInvalids':getTempInvalidList()}
            QtGui.qApp.call(self, applyTemplate, (self, templateId, data))

    #### AmbCard page: Status page ###################

    @pyqtSignature('QAbstractButton*')
    def on_cmdAmbCardStatusButtonBox_clicked(self, button):
        buttonCode = self.cmdAmbCardStatusButtonBox.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_cmdAmbCardStatusButtonBox_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_cmdAmbCardStatusButtonBox_reset()

#    @pyqtSignature('')
    def on_cmdAmbCardStatusButtonBox_reset(self):
        self.resetAmbCardFilter(self.edtAmbCardStatusBegDate,
                                self.edtAmbCardStatusEndDate,
                                self.cmbAmbCardStatusGroup,
                                self.edtAmbCardStatusOffice,
                                self.cmbAmbCardStatusOrgStructure
                                )


#    @pyqtSignature('')
    def on_cmdAmbCardStatusButtonBox_apply(self):
        filter = self.getAmbCardFilter(
                        self.edtAmbCardStatusBegDate,
                        self.edtAmbCardStatusEndDate,
                        self.cmbAmbCardStatusGroup,
                        self.edtAmbCardStatusOffice,
                        self.cmbAmbCardStatusOrgStructure
                        )
        self.updateAmbCardStatus(filter)
        self.focusAmbCardStatusActions()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelAmbCardStatusActions_currentRowChanged(self, current, previous):
        self.updateAmbCardPropertiesTable(current, self.tblAmbCardStatusActionProperties, previous)
#        self.updateAmbCardPrintActionAction(current)


    @pyqtSignature('')
    def on_actStatusShowPropertyHistory_triggered(self):
        self.tblAmbCardStatusActionProperties.showHistory()


    @pyqtSignature('')
    def on_actStatusShowPropertiesHistory_triggered(self):
        self.tblAmbCardStatusActionProperties.showHistoryEx()

    #### AmbCard page: Diagnostic page ###################

    @pyqtSignature('QAbstractButton*')
    def on_cmdAmbCardDiagnosticButtonBox_clicked(self, button):
        buttonCode = self.cmdAmbCardDiagnosticButtonBox.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_cmdAmbCardDiagnosticButtonBox_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_cmdAmbCardDiagnosticButtonBox_reset()


#    @pyqtSignature('')
    def on_cmdAmbCardDiagnosticButtonBox_reset(self):
        self.resetAmbCardFilter(self.edtAmbCardDiagnosticBegDate,
                                self.edtAmbCardDiagnosticEndDate,
                                self.cmbAmbCardDiagnosticGroup,
                                self.edtAmbCardDiagnosticOffice,
                                self.cmbAmbCardDiagnosticOrgStructure
                                )


#    @pyqtSignature('')
    def on_cmdAmbCardDiagnosticButtonBox_apply(self):
        filter = self.getAmbCardFilter(
                        self.edtAmbCardDiagnosticBegDate,
                        self.edtAmbCardDiagnosticEndDate,
                        self.cmbAmbCardDiagnosticGroup,
                        self.edtAmbCardDiagnosticOffice,
                        self.cmbAmbCardDiagnosticOrgStructure
                        )
        self.updateAmbCardDiagnostic(filter)
        self.focusAmbCardDiagnosticActions()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelAmbCardDiagnosticActions_currentRowChanged(self, current, previous):
        self.updateAmbCardPropertiesTable(current, self.tblAmbCardDiagnosticActionProperties, previous)
#        self.updateAmbCardPrintActionAction(current)


    @pyqtSignature('')
    def on_actDiagnosticShowPropertyHistory_triggered(self):
        self.tblAmbCardDiagnosticActionProperties.showHistory()


    @pyqtSignature('')
    def on_actDiagnosticShowPropertiesHistory_triggered(self):
        self.tblAmbCardDiagnosticActionProperties.showHistoryEx()

    #### AmbCard page: Cure page ###################

    @pyqtSignature('QAbstractButton*')
    def on_cmdAmbCardCureButtonBox_clicked(self, button):
        buttonCode = self.cmdAmbCardCureButtonBox.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_cmdAmbCardCureButtonBox_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_cmdAmbCardCureButtonBox_reset()

#    @pyqtSignature('')
    def on_cmdAmbCardCureButtonBox_reset(self):
        self.resetAmbCardFilter(self.edtAmbCardCureBegDate,
                                self.edtAmbCardCureEndDate,
                                self.cmbAmbCardCureGroup,
                                self.edtAmbCardCureOffice,
                                self.cmbAmbCardCureOrgStructure
                                )


#    @pyqtSignature('')
    def on_cmdAmbCardCureButtonBox_apply(self):
        filter = self.getAmbCardFilter(
                        self.edtAmbCardCureBegDate,
                        self.edtAmbCardCureEndDate,
                        self.cmbAmbCardCureGroup,
                        self.edtAmbCardCureOffice,
                        self.cmbAmbCardCureOrgStructure
                        )
        self.updateAmbCardCure(filter)
        self.focusAmbCardCureActions()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelAmbCardCureActions_currentRowChanged(self, current, previous):
        self.updateAmbCardPropertiesTable(current, self.tblAmbCardCureActionProperties, previous)
#        self.updateAmbCardPrintActionAction(current)


    @pyqtSignature('')
    def on_actCureShowPropertyHistory_triggered(self):
        self.tblAmbCardCureActionProperties.showHistory()


    @pyqtSignature('')
    def on_actCureShowPropertiesHistory_triggered(self):
        self.tblAmbCardCureActionProperties.showHistoryEx()

    ### AmbCard page: Visit page ###################

    @pyqtSignature('QAbstractButton*')
    def on_cmdAmbCardVisitButtonBox_clicked(self, button):
        buttonCode = self.cmdAmbCardVisitButtonBox.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_cmdAmbCardVisitButtonBox_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_cmdAmbCardVisitButtonBox_reset()


    def on_cmdAmbCardVisitButtonBox_reset(self):
        self.edtAmbCardVisitBegDate.setDate(QDate())
        self.edtAmbCardVisitEndDate.setDate(QDate())
        self.cmbAmbCardVisitOrgStructure.setValue(None)
        self.cmbAmbCardVisitSpeciality.setValue(None)
        self.cmbAmbCardVisitPerson.setValue(None)
        self.cmbAmbCardVisitScene.setValue(None)
        self.cmbAmbCardVisitEventType.setValue(None)
        self.cmbAmbCardVisitService.setValue(None)


    def on_cmdAmbCardVisitButtonBox_apply(self):
        filter = {}
        filter['begDate'] = self.edtAmbCardVisitBegDate.date()
        filter['endDate'] = self.edtAmbCardVisitEndDate.date()
        filter['orgStructureId'] = self.cmbAmbCardVisitOrgStructure.value()
        filter['specialityId'] = self.cmbAmbCardVisitSpeciality.value()
        filter['personId'] = self.cmbAmbCardVisitPerson.value()
        filter['sceneId'] = self.cmbAmbCardVisitScene.value()
        filter['eventTypeId'] = self.cmbAmbCardVisitEventType.value()
        filter['serviceId'] = self.cmbAmbCardVisitService.value()

        self.updateAmbCardVisit(filter)
        self.tblAmbCardVisits.setFocus(Qt.TabFocusReason)


    #### AmbCard page: Misc page ###################

    @pyqtSignature('QAbstractButton*')
    def on_cmdAmbCardMiscButtonBox_clicked(self, button):
        buttonCode = self.cmdAmbCardMiscButtonBox.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_cmdAmbCardMiscButtonBox_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_cmdAmbCardMiscButtonBox_reset()


#    @pyqtSignature('')
    def on_cmdAmbCardMiscButtonBox_reset(self):
        self.resetAmbCardFilter(self.edtAmbCardMiscBegDate,
                                self.edtAmbCardMiscEndDate,
                                self.cmbAmbCardMiscGroup,
                                self.edtAmbCardMiscOffice,
                                self.cmbAmbCardMiscOrgStructure
                                )


#    @pyqtSignature('')
    def on_cmdAmbCardMiscButtonBox_apply(self):
        filter = self.getAmbCardFilter(
                        self.edtAmbCardMiscBegDate,
                        self.edtAmbCardMiscEndDate,
                        self.cmbAmbCardMiscGroup,
                        self.edtAmbCardMiscOffice,
                        self.cmbAmbCardMiscOrgStructure
                        )
        self.updateAmbCardMisc(filter)
        self.focusAmbCardMiscActions()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelAmbCardMiscActions_currentRowChanged(self, current, previous):
        self.updateAmbCardPropertiesTable(current, self.tblAmbCardMiscActionProperties, previous)


    @pyqtSignature('')
    def on_actMiscShowPropertyHistory_triggered(self):
        self.tblAmbCardMiscActionProperties.showHistory()


    @pyqtSignature('')
    def on_actMiscShowPropertiesHistory_triggered(self):
        self.tblAmbCardMiscActionProperties.showHistoryEx()
