# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QEventLoop, QModelIndex, QVariant, SIGNAL, QDateTime

from library.crbcombobox  import CRBComboBox
from library.database     import CRecordCache
from library.TableModel   import CTableModel, CBoolCol, CCol, CDateCol, CDesignationCol, CEnumCol, CIntCol, CNameCol, CNumCol, CRefBookCol, CTextCol, CDoubleCol
from library.TableView    import CTableView, CExtendedSelectionTableView
from library.Attach.AttachedFile import CAttachedFilesLoader, CAttachedFilesModel
from library.Utils        import forceDouble, forceInt, forceRef, forceString, formatSex, formatShortNameInt, formatSNILS, toVariant, formatName, forceDate, forceDateTime
from Accounting.Utils     import unpackExposeDiscipline
from Events.Action        import CActionTypeCache, CAction
from Events.ActionStatus  import CActionStatus
from Events.ActionTypeCol import CActionTypeCol
from Events.Utils         import orderTexts, payStatusText
from Registry.Utils       import canRemoveEventWithTissue, canRemoveEventWithJobTicket
from Reports.ReportBase   import CReportBase, createTable
from Users.Rights         import urDeleteNotOwnEvents

codeIsPrimary = [u'-', u'первичный', u'повторный', u'активное посещение', u'перевозка', u'амбулаторно']


class CSNILSCol(CTextCol):
    def format(self, values):
        val = unicode(values[0].toString())
        return QVariant(formatSNILS(val))


class CDiagnosisCol(CTextCol):
    def __init__(self, title, fields, defaultWidth, alignment='l'):
        CTextCol.__init__(self, title, fields, defaultWidth, alignment)


    def format(self, values):
        MKB   = forceString(values[0])
        MKBEx = forceString(values[1])
        outText = MKB+'+'+MKBEx if MKBEx else MKB
        return toVariant(outText)


class CMorphologyMKBCol(CTextCol):
    def __init__(self, title, fields, defaultWidth, alignment='l'):
        CTextCol.__init__(self, title, fields, defaultWidth, alignment)


class CClientEvalCol(CTextCol):
    def __init__(self, title, fields, expr, defaultWidth, alignment='l'):
        CTextCol.__init__(self, title, fields, defaultWidth, alignment)
        self.expr = expr
        self.cache = {}


    def format(self, values):
        clientId = values[0]
        output = self.cache.get(clientId)
        if output is None:
            output = QtGui.qApp.db.translate('Client', 'id', clientId, self.expr)
            self.cache[clientId] = output
        return output


    def invalidateRecordsCache(self):
        self.cache.clear()


#class CClientEvalExCol(CTextCol):
#    def __init__(self, title, fields, rbFields, tableName, rbTableName, defaultWidth, alignment='l'):
#        CTextCol.__init__(self, title, fields, defaultWidth, alignment)
#        self.color = None
#        self.tableName = tableName
#        self.rbTableName = rbTableName
#        self.rbFields = rbFields
#
#
#    def format(self, values):
#        code = ''
#        self.color = None
#        id = values[0]
#        db = QtGui.qApp.db
#        table = db.table(self.tableName)
#        rbTable = db.table(self.rbTableName)
#        record = db.getRecordEx(table.innerJoin(rbTable, table[self.rbFields].eq(rbTable['id'])), [rbTable['code'], rbTable['name'], rbTable['color']], [table['deleted'].eq(0), table['master_id'].eq(id)], '%s.id DESC'%(self.tableName))
#        if record:
#            self.codeName = forceString(record.value('code')) + '-' + forceString(record.value('name'))
#            code = forceString(record.value('code'))
#            self.color = record.value('color')
#        return code
#
#
#    def getCodeNameToolTip(self, values):
#        id = values[0]
#        db = QtGui.qApp.db
#        table = db.table(self.tableName)
#        rbTable = db.table(self.rbTableName)
#        record = db.getRecordEx(table.innerJoin(rbTable, table[self.rbFields].eq(rbTable['id'])), [rbTable['code'], rbTable['name'], rbTable['color']], [table['deleted'].eq(0), table['master_id'].eq(id)], '%s.id DESC'%(self.tableName))
#        if record:
#            codeName = forceString(record.value('code')) + '-' + forceString(record.value('name'))
#            if codeName:
#                return QVariant(codeName)
#        return CCol.invalid
#
#
#    def getBackgroundColor(self, values):
#        val = self.color
#        if val:
#            colorName = forceString(val)
#            if colorName:
#                return QVariant(QtGui.QColor(colorName))
#        return CCol.invalid


class CClientsTableModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)
        self.addColumn(CNameCol(u'Фамилия',    ['lastName'], 15))
        self.addColumn(CNameCol(u'Имя',        ['firstName'], 15))
        self.addColumn(CNameCol(u'Отчество',   ['patrName'], 15))
        self.addColumn(CDateCol(u'Дата рожд.', ['birthDate'], 12, highlightRedDate=False))
        self.addColumn(CEnumCol(u'Пол',        ['sex'], [u'-', u'М', u'Ж'], 4, 'c'))
        self.addColumn(CSNILSCol(u'СНИЛС',     ['SNILS'], 4))

#        self.addColumn(CTextCol(u'Документ',   ['getClientDocument(id)'], 30))
#        self.addColumn(CTextCol(u'Полис',      ['getClientPolicy(id,1)'], 30))
#        self.addColumn(CTextCol(u'Адрес регистрации', ['getClientRegAddress(id)'], 30))
#        self.addColumn(CTextCol(u'Адрес проживания',  ['getClientLocAddress(id)'], 30))
#        self.addColumn(CTextCol(u'Занятость',         ['getClientWork(id)'], 30))
#        self.addColumn(CTextCol(u'Контакты',          ['getClientContacts(id)'], 30))

        self.addColumn(CClientEvalCol(u'Документ',          ['id'], 'getClientDocument(id)',   30))
        self.addColumn(CClientEvalCol(u'Полис',             ['id'], 'getClientPolicy(id,1)',   30))
        self.addColumn(CClientEvalCol(u'Адрес регистрации', ['id'], 'getClientRegAddress(id)', 30))
        self.addColumn(CClientEvalCol(u'Адрес проживания',  ['id'], 'getClientLocAddress(id)', 30))
        self.addColumn(CClientEvalCol(u'Занятость',         ['id'], 'getClientWork(id)',       30))
        self.addColumn(CClientEvalCol(u'Контакты',          ['id'], 'getClientContacts(id)',   30))
        self.addColumn(CClientEvalCol(u'Прикрепление',          ['id'], 'getFormatedClientAttach(id)',   30))
#        self.addColumn(CClientEvalExCol(u'С',               ['id'], 'statusObservationType_id', 'Client_StatusObservation', 'rbStatusObservationClientType', 30)).setToolTip(u'Статус наблюдения пациента')
        self.setTable('Client')


    def data(self, index, role=Qt.DisplayRole):
        if not index.isValid():
            return QVariant()
        column = index.column()
        row    = index.row()
        if role == Qt.DisplayRole: ### or role == Qt.EditRole:
            (col, values) = self.getRecordValues(column, row)
            return col.format(values)
        elif role == Qt.TextAlignmentRole:
           col = self._cols[column]
           return col.alignment()
        elif role == Qt.CheckStateRole:
            (col, values) = self.getRecordValues(column, row)
            return col.checked(values)
        elif role == Qt.ForegroundRole:
            (col, values) = self.getRecordValues(column, row)
            return col.getForegroundColor(values)
        elif role == Qt.BackgroundRole:
            (col, values) = self.getRecordValues(column, row)
            return col.getBackgroundColor(values)
#        elif role == Qt.ToolTipRole:
#            if column == 12 or column == 13:
#                (col, values) = self.getRecordValues(column, row)
#                return col.getCodeNameToolTip(values)
        return QVariant()



class CClientsTableView(CTableView):
    def __init__(self, parent):
        CTableView.__init__(self, parent)


class CEventsTableModel(CTableModel):
    class CLocClientColumn(CCol):
        def __init__(self, title, fields, defaultWidth, clientCache):
            CCol.__init__(self, title, fields, defaultWidth, 'l')
            self.clientCache = clientCache

        def format(self, values):
            val = values[0]
            clientId  = forceRef(val)
            clientRecord = self.clientCache.get(clientId)
            if clientRecord:
                name = formatShortNameInt(forceString(clientRecord.value('lastName')),
                   forceString(clientRecord.value('firstName')),
                   forceString(clientRecord.value('patrName')))
                return toVariant(name)
            return CCol.invalid


    class CLocClientBirthDateColumn(CCol):
        def __init__(self, title, fields, defaultWidth, clientCache, extraFields=[]):
            CCol.__init__(self, title, fields, defaultWidth, 'l')
            self.extraFields = extraFields
            self.clientCache = clientCache

        def format(self, values):
            val = values[0]
            clientId  = forceRef(val)
            clientRecord = self.clientCache.get(clientId)
            if clientRecord:
                return toVariant(forceString(clientRecord.value('birthDate')))
            return CCol.invalid


    class CLocClientSexColumn(CCol):
        def __init__(self, title, fields, defaultWidth, clientCache, extraFields=[]):
            CCol.__init__(self, title, fields, defaultWidth, 'l')
            self.extraFields = extraFields
            self.clientCache = clientCache

        def format(self, values):
            val = values[0]
            clientId  = forceRef(val)
            clientRecord = self.clientCache.get(clientId)
            if clientRecord:
                return toVariant(formatSex(clientRecord.value('sex')))
            return CCol.invalid


    class CLocEventMKBColumn(CCol):
        def __init__(self, title, fields, defaultWidth, extraFields=[], alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.extraFields = extraFields
            self._cache = {}

        def format(self, values):
            eventId = forceRef(values[0])
            result = self._cache.get(eventId, None)
            if result is None:
                db = QtGui.qApp.db
                query = db.query('SELECT getEventDiagnosis(%d)'%eventId)
                if query.next():
                    diagnosisId = forceRef(query.value(0))
                    if diagnosisId:
                        table = db.table('Diagnosis')
                        record = db.getRecordEx(table, 'MKB', table['id'].eq(diagnosisId))
                        if record:
                            MKB   = forceString(record.value('MKB'))
                            outText = MKB
                            result = toVariant(outText)
                            self._cache[eventId] = result
                    else:
                        result = CCol.invalid
                        self._cache[eventId] = result
                else:
                    result = CCol.invalid
                    self._cache[eventId] = result
            return result

        def invalidateRecordsCache(self):
            self._cache = {}


    class CLocEventMKBExColumn(CCol):
        def __init__(self, title, fields, defaultWidth, extraFields=[], alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.extraFields = extraFields
            self._cache = {}

        def format(self, values):
            eventId = forceRef(values[0])
            result = self._cache.get(eventId, None)
            if result is None:
                db = QtGui.qApp.db
                query = db.query('SELECT getEventDiagnosis(%d)'%eventId)
                if query.next():
                    diagnosisId = forceRef(query.value(0))
                    if diagnosisId:
                        table = db.table('Diagnosis')
                        record = db.getRecordEx(table, 'MKBEx', table['id'].eq(diagnosisId))
                        if record:
                            MKBEx = forceString(record.value('MKBEx'))
                            outText = MKBEx
                            result = toVariant(outText)
                            self._cache[eventId] = result
                    else:
                        result = CCol.invalid
                        self._cache[eventId] = result
                else:
                    result = CCol.invalid
                    self._cache[eventId] = result
            return result

        def invalidateRecordsCache(self):
            self._cache = {}


    class CLocEventHealthGroupColumn(CCol):
        def __init__(self, title, fields, defaultWidth, extraFields=[], alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.extraFields = extraFields
            self._cache = {}

        def format(self, values):
            eventId = forceRef(values[0])
            result = self._cache.get(eventId, None)
            if result is None:
                db = QtGui.qApp.db
                query = db.query('SELECT getEventHealthGroupDiagnostic(%d)'%eventId)
                if query.next():
                    healthGroupId = forceRef(query.value(0))
                    if healthGroupId:
                        result = db.translate('rbHealthGroup', 'id', healthGroupId, 'code')
                        self._cache[eventId] = result
                    else:
                        result = CCol.invalid
                        self._cache[eventId] = result
                else:
                    result = CCol.invalid
                    self._cache[eventId] = result
            return result

        def invalidateRecordsCache(self):
            self._cache = {}


    class CLocEventResultColumn(CCol):
        def __init__(self, title, fields, defaultWidth, extraFields=[], alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.extraFields = extraFields
            self._cache = {}

        def format(self, values):
            eventId = forceRef(values[0])
            result = self._cache.get(eventId, None)
            if result is None:
                db = QtGui.qApp.db
                query = db.query('SELECT getEventResultDiagnostic(%d)'%eventId)
                if query.next():
                    resultId = forceRef(query.value(0))
                    if resultId:
                        result = db.translate('rbDiagnosticResult', 'id', resultId, 'name')
                        self._cache[eventId] = result
                    else:
                        result = CCol.invalid
                        self._cache[eventId] = result
                else:
                    result = CCol.invalid
                    self._cache[eventId] = result
            return result

        def invalidateRecordsCache(self):
            self._cache = {}


    def __init__(self, parent, clientCache):
        CTableModel.__init__(self, parent)
        clientCol   = CEventsTableModel.CLocClientColumn( u'Ф.И.О.', ['client_id'], 60, clientCache)
        clientBirthDateCol = CEventsTableModel.CLocClientBirthDateColumn(u'Дата рожд.', ['client_id'], 20, clientCache, ['birthDate'])
        clientSexCol = CEventsTableModel.CLocClientSexColumn(u'Пол', ['client_id'], 5, clientCache, ['sex'])
        self.addColumn(clientCol)
        self.addColumn(CTextCol(u'Код пациента', ['client_id'], 30))
        self.addColumn(clientBirthDateCol)
        self.addColumn(clientSexCol)
        self.addColumn(CDateCol(u'Назначен', ['setDate'],  10))
        self.addColumn(CDateCol(u'Выполнен', ['execDate'], 10))
        self.addColumn(CDateCol(u'След. явка', ['nextEventDate'], 10))
        self.addColumn(CDesignationCol(u'Договор',         ['contract_id'],('vrbContract', 'code'), 20))
        self.addColumn(CRefBookCol(u'Тип',  ['eventType_id'], 'EventType', 15))
        self.addColumn(CRefBookCol(u'МЭС',  ['MES_id'], 'mes.MES', 15, CRBComboBox.showCode))
        self.addColumn(CEventsTableModel.CLocEventMKBColumn(u'MKB', ['id'], 7, ['MKB']))
        self.addColumn(CEventsTableModel.CLocEventMKBExColumn(u'MKBEx', ['id'], 7, ['MKBEx']))
        self.addColumn(CRefBookCol(u'Врач', ['execPerson_id'], 'vrbPersonWithSpecialityAndOrgStr', 15))
        self.addColumn(CEnumCol(u'Первичный', ['isPrimary'], codeIsPrimary, 8))
        self.addColumn(CEnumCol(u'Порядок', ['order'], orderTexts, 8))
        self.addColumn(CRefBookCol(u'Результат события', ['result_id'], 'rbResult', 40))
        self.addColumn(CTextCol(u'Номер документа', ['externalId'], 30))
        self.addColumn(CEventsTableModel.CLocEventHealthGroupColumn(u'Группа здоровья', ['id'], 7, ['healthGroup_id']))
        self.addColumn(CEventsTableModel.CLocEventResultColumn(u'Результат осмотра', ['id'], 7, ['diagnosticResult_id']))
        self.loadField('createPerson_id')
        self.loadField('prevEvent_id')
        self.setTable('Event')
        self.diagnosisIdList = None
        self._mapColumn2Order = {u'client_id'          :u'CONCAT(Client.lastName, Client.firstName, Client.patrName)',
                                 u'birthDate'          :u'Client.birthDate',
                                 u'sex'                :u'Client.sex',
                                 u'setDate'            :u'Event.setDate',
                                 u'execDate'           :u'Event.execDate',
                                 u'nextEventDate'   :u'Event.nextEventDate',
                                 u'contract_id'        :u'vrbContract.code',
                                 u'eventType_id'       :u'EventType.name',
                                 u'MES_id'             :u'mes.MES.code',
                                 u'MKB'                :u'Diagnosis.MKB',
                                 u'MKBEx'              :u'Diagnosis.MKBEx',
                                 u'execPerson_id'      :u'Event.execPerson_id',
                                 u'isPrimary'          :u'Event.isPrimary',
                                 u'order'              :u'Event.order',
                                 u'result_id'          :u'rbResult.name',
                                 u'externalId'         :u'Event.externalId',
                                 u'healthGroup_id'     :u'rbHealthGroup.code',
                                 u'diagnosticResult_id':u'rbDiagnosticResult.name'
                                 }
        self.personalAccountAvailabilityCache = {}
        self.connect(self, SIGNAL('itemsCountChanged(int)'), self.personalAccountRestrictions)



    def getClientIdList(self):
        clientIdList = []
        if self.idList():
            db = QtGui.qApp.db
            tableEvent = db.table('Event')
            cond = [tableEvent['id'].inlist(self._idList)]
            records = db.getRecordList(tableEvent, tableEvent['client_id'], cond)
            for record in records:
                clientId = forceInt(record.value('client_id'))
                if clientId not in clientIdList:
                    clientIdList.append(clientId)
        return clientIdList


    def personalAccountRestrictions(self, count):
        self.personalAccountAvailabilityCache = {}
        if count:
            db = QtGui.qApp.db
            tableEvent = db.table('Event')
            tableContract = db.table('Contract')
            table = tableEvent.innerJoin(tableContract, tableContract['id'].eq(tableEvent['contract_id']))
            cond = [tableEvent['id'].inlist(self._idList)]
            records = db.getRecordList(table, [tableEvent['id'].name(), tableContract['exposeDiscipline'].name()], cond)
            for record in records:
                exposeDiscipline = forceInt(record.value(1))
                eventId = forceInt(record.value(0))
                exposeByBatch, exposeByEvent, exposeByMonth, exposeByClient, exposeByInsurer = unpackExposeDiscipline(exposeDiscipline)
                if exposeByClient or exposeByEvent:
                    self.personalAccountAvailabilityCache[eventId] = True
                else:
                    self.personalAccountAvailabilityCache[eventId] = False


    def getOrder(self, fieldName, column):
        if hasattr(self._cols[column], 'extraFields'):
            if len(self._cols[column].extraFields) > 0:
                fieldName = self._cols[column].extraFields[0]
        return self._mapColumn2Order[fieldName]


    def removeRow(self, row, parent = QModelIndex()):
        if self._idList and 0<=row<len(self._idList):
            itemId = self._idList[row]
            if self.canRemoveEvent(itemId):
                return CTableModel.removeRow(self, row, parent)
        return False


    def canRemoveEvent(self, itemId):
        result = True
        if self.hasActionsWithTissue(itemId):
            result = result and canRemoveEventWithTissue()
        if self.hasActionWithJobTicket(itemId):
            result = result and canRemoveEventWithJobTicket()
        return result


    def canRemoveItem(self, eventId):
        userId = QtGui.qApp.userId
        canRemoveAll = QtGui.qApp.userHasRight(urDeleteNotOwnEvents)
        record = self.getRecordById(eventId)
        canRemoveMine = (userId == forceRef(record.value('createPerson_id')) or userId == forceRef(record.value('execPerson_id')))
        return canRemoveAll or canRemoveMine


    def hasActionWithJobTicket(self, eventId):
        db                           = QtGui.qApp.db
        tableEvent                   = db.table('Event')
        tableAction                  = db.table('Action')
        tableActionProperty          = db.table('ActionProperty')
        tableActionPropertyJobTicket = db.table('ActionProperty_Job_Ticket')
        tableJobTicket               = db.table('Job_Ticket')
        tableJob                     = db.table('Job')
        queryTable = tableEvent.innerJoin(tableAction,
                                          tableAction['event_id'].eq(tableEvent['id']))
        queryTable = queryTable.innerJoin(tableActionProperty,
                                          tableActionProperty['action_id'].eq(tableAction['id']))
        queryTable = queryTable.innerJoin(tableActionPropertyJobTicket,
                                          tableActionPropertyJobTicket['id'].eq(tableActionProperty['id']))
        queryTable = queryTable.innerJoin(tableJobTicket,
                                          tableJobTicket['id'].eq(tableActionPropertyJobTicket['value']))
        queryTable = queryTable.innerJoin(tableJob,
                                          tableJob['id'].eq(tableJobTicket['master_id']))
        cond = [tableAction['event_id'].eq(eventId),
                tableAction['deleted'].eq(0),
                tableActionProperty['deleted'].eq(0),
                tableJob['deleted'].eq(0)]
        return bool(db.getCount(queryTable, countCol='Job.id', where=cond))


    def hasActionsWithTissue(self, itemId):
        db = QtGui.qApp.db
        tableAction = db.table('Action')
        cond = [tableAction['event_id'].eq(itemId),
                tableAction['takenTissueJournal_id'].isNotNull()]
        return bool(db.getCount(tableAction, where=cond))


    def canMakePersonalAccount(self, itemId):
        return self.personalAccountAvailabilityCache.get(itemId, False)


    def beforeRemoveItem(self, itemId):
        db = QtGui.qApp.db
        self.diagnosisIdList = db.getIdList('Diagnostic', idCol='diagnosis_id', where='event_id=%d'%itemId)


    def afterRemoveItem(self, itemId):
        if self.diagnosisIdList:
            db = QtGui.qApp.db

            tableDiagnosis = db.table('Diagnosis')
            stmt = 'DELETE Diagnosis '                                                  \
                   'FROM Diagnosis '                                                    \
                   'LEFT JOIN Diagnostic ON Diagnostic.diagnosis_id = Diagnosis.id '    \
                   'LEFT JOIN TempInvalid ON TempInvalid.diagnosis_id = Diagnosis.id '  \
                   'LEFT JOIN Diagnosis AS D ON D.mod_id = Diagnosis.id '               \
                   'WHERE (%s) '                                                        \
                   'AND Diagnostic.id IS NULL '                                         \
                   'AND TempInvalid.id IS NULL '                                        \
                   'AND D.id IS NULL;' % tableDiagnosis['id'].inlist(self.diagnosisIdList)
            db.query(stmt)
#
#            db.deleteRecord(tableDiagnosis, [tableDiagnosis['id'].inlist(self.diagnosisIdList),
#                                             'NOT EXISTS (SELECT NULL FROM Diagnostic WHERE Diagnostic.diagnosis_id = Diagnosis.id)',
#                                             'NOT EXISTS (SELECT NULL FROM TempInvalid WHERE TempInvalid.diagnosis_id = Diagnosis.id)'
#                                            ])
            self.diagnosisIdList = None
        QtGui.qApp.emitCurrentClientInfoChanged()


class CEventsTableView(CTableView):
    def __init__(self, parent):
        CTableView.__init__(self, parent)
        self._order = None
        self._orderColumn = None
        self._isDesc = True


    def removeEventAfterDirection(self):
        def removeCurrentRowEvent():
            index = self.currentIndex()
            if index.isValid() and self.confirmRemoveRow(self.currentIndex().row()):
                row = self.currentIndex().row()
                self.model().removeRow(row)
                self.setCurrentRow(row)
        row = self.currentIndex().row()
        record = self.model().getRecordByRow(row)
        if record:
            prevEventId = forceRef(record.value('prevEvent_id'))
            if prevEventId:
                db = QtGui.qApp.db
                tableEvent = db.table('Event')
                record = db.getRecordEx(tableEvent, [tableEvent['relegateOrg_id']], [tableEvent['id'].eq(prevEventId), tableEvent['deleted'].eq(0)])
                relegateOrgId = forceRef(record.value('relegateOrg_id')) if record else None
                if relegateOrgId and relegateOrgId != QtGui.qApp.currentOrgId():
                    tableAction = db.table('Action')
                    tableActionType = db.table('ActionType')
                    table = tableAction.innerJoin(tableActionType, tableActionType['id'].eq(tableAction['actionType_id']))
                    cond = [tableAction['event_id'].eq(prevEventId),
                            tableAction['deleted'].eq(0),
                            tableActionType['deleted'].eq(0),
                            db.joinOr([tableActionType['flatCode'].like(u'recoveryDirection%'),
                                       tableActionType['flatCode'].like(u'inspectionDirection%'),
                                       tableActionType['flatCode'].like(u'researchDirection%'),
                                       tableActionType['flatCode'].like(u'consultationDirection%')])
                            ]
                    record = db.getRecordEx(table, u'*', cond, tableAction['id'].name())
                    actionId = forceRef(record.value('id')) if record else None
                    if actionId:
                        res = QtGui.QMessageBox.warning( self,
                                     u'Внимание!',
                                     u'При удалении События, связанного с внешним направлением, направление будет аннулированно. Продолжить удаление?',
                                     QtGui.QMessageBox.Ok|QtGui.QMessageBox.Cancel,
                                     QtGui.QMessageBox.Cancel)
                        if res == QtGui.QMessageBox.Ok:
                            QtGui.qApp.call(self, removeCurrentRowEvent)
                            recordAction = db.getRecordEx(tableAction, u'*', [tableAction['id'].eq(actionId), tableAction['deleted'].eq(0)])
                            personId = QtGui.qApp.userId if (QtGui.qApp.userId and QtGui.qApp.userSpecialityId) else forceRef(recordAction.value('person_id'))
                            tablePWS = db.table('vrbPersonWithSpeciality')
                            recordPWS = db.getRecordEx(tablePWS, [tablePWS['name']], [tablePWS['id'].eq(personId)]) if personId else None
                            personName = forceString(recordPWS.value('name')) if recordPWS else ''
                            recordAction.setValue('status', toVariant(CActionStatus.canceled))
                            recordAction.setValue('note', toVariant(u'Отменить: %s %s'%(QDateTime.currentDateTime().toString('dd-MM-yyyy hh:mm'), personName)))
                            db.transaction()
                            try:
                                db.updateRecord(tableAction, recordAction)
                                db.commit()
                            except:
                                db.rollback()
                        return True
        return False


    def order(self):
        return self._order


    def setOrder(self, column):
        if column is not None:
            self._isDesc = not self._isDesc if self._orderColumn == column else False
            self._order = self.getOrder(column) + (' DESC' if self._isDesc else ' ASC')
            self._orderColumn = column
            self.horizontalHeader().setSortIndicator(column, Qt.DescendingOrder if self._isDesc else Qt.AscendingOrder)
        else:
            self._order = None
            self._orderColumn = None
            self._isDesc = True


    def getOrder(self, column):
        return self.model().getOrder(self.model().cols()[column].fields()[0], column)


    def eventHasContinuation(self, eventId):
        db = QtGui.qApp.db
        tableEvent = db.table('Event')
        record = db.getRecordEx(tableEvent, [tableEvent['id']], [tableEvent['prevEvent_id'].eq(eventId), tableEvent['deleted'].eq(0)])
        return bool(record)


    def eventHasAccountItem(self, eventId):
        db = QtGui.qApp.db
        accountItemId = forceRef(db.translate('Account_Item', 'event_id', eventId, 'id'))
        return bool(accountItemId)


    def removeCurrentRow(self, needConfirm = True):
        def removeCurrentRowInternal(needConfirm = True):
            eventId = self.currentItemId()
            if eventId:
                if self.eventHasContinuation(eventId):
                    QtGui.QMessageBox.warning(self, u'Внимание!', u'Событие имеет продолжение, поэтому не может быть удалено.', QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)
                elif self.eventHasAccountItem(eventId):
                    QtGui.QMessageBox.warning(self, u'Внимание!', u'По событию выставлен счёт, поэтому оно не может быть удалено.', QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)
                elif self.removeEventAfterDirection():
                    pass
                else:
                    row = self.currentRow()
                    if not needConfirm or self.confirmRemoveRow(row):
                        self.model().removeRow(row)
                        self.setCurrentRow(row)
        QtGui.qApp.call(self, removeCurrentRowInternal, (needConfirm, ))


class CEventDiagnosticsTableModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)
#        self.addColumn(CDateCol(u'дата назначения',['setDate'], 10))
        col = self.addColumn(CDateCol(u'Установлен', ['endDate'], 10))
        self.addColumn(CRefBookCol(u'Тип', ['diagnosisType_id'], 'rbDiagnosisType', 6))
        col = self.addColumn(CRefBookCol(u'ГpЗд', ['healthGroup_id'], 'rbHealthGroup', 2, CRBComboBox.showCode))
        col.setToolTip(u'Группа здоровья')
        self.addColumn(CDiagnosisCol(u'Диагноз', ['MKB', 'MKBEx'], 6))
        if QtGui.qApp.defaultMorphologyMKBIsVisible():
            col = self.addColumn(CMorphologyMKBCol(u'Морф.', ['morphologyMKB'], 8))
            col.setToolTip(u'Морфология диагноза МКБ')
        col = self.addColumn(CRefBookCol(u'Хар', ['character_id'], 'rbDiseaseCharacter', 6))
        col.setToolTip(u'Характер')
        col = self.addColumn(CRefBookCol(u'Фаза', ['phase_id'], 'rbDiseasePhases', 6))
        col.setToolTip(u'Фаза')
        col = self.addColumn(CRefBookCol(u'Ст', ['stage_id'], 'rbDiseaseStage', 6))
        col.setToolTip(u'Стадия')
        col = self.addColumn(CRefBookCol(
            u'ДН', ['dispanser_id'], 'rbDispanser', 6, CRBComboBox.showCode))
        col.setToolTip(u'Диспансерное наблюдение')
#        col = self.addColumn(CEnumCol(u'Г',['hospital'], CHospitalInfo.names, 4))
        col = self.addColumn(CTextCol(u'Госп', ['hospital'], 4))
        col.setToolTip(u'Госпитализация')
        col = self.addColumn(CRefBookCol(
            u'Тр', ['traumaType_id'], 'rbTraumaType', 6, CRBComboBox.showCode))
        col.setToolTip(u'Тип травмы')
        self.addColumn(CRefBookCol(u'Врач', ['person_id'], 'vrbPerson', 6))
        self.addColumn(CRefBookCol(u'Специальность', ['speciality_id'], 'rbSpeciality', 6))
        self.addColumn(CRefBookCol(u'Результат', ['result_id'], 'rbDiagnosticResult', 6))
        self.addColumn(CTextCol(u'Примечания', ['notes'], 6))
        self.addColumn(CTextCol(u'Описание', ['freeInput'], 6))

#        self.addColumn(CDateCol(u'дата', ['endDate'], 6))
#        self.addColumn(CEnumCol(u'СК', ['sanatorium'], [u'не нуждается', u'нуждается', u'направлен', u'пролечен'], 4))
        db = QtGui.qApp.db
        tableDiagnostic = db.table('Diagnostic')
        tableDiagnosis  = db.table('Diagnosis')
        queryTable = tableDiagnostic.leftJoin(tableDiagnosis, tableDiagnosis['id'].eq(tableDiagnostic['diagnosis_id']))
#        self.setTable('Diagnostic')
        self.setTable(queryTable)


class CPayStatusColumn(CCol):
    def format(self, values):
        payStatus = forceInt(values[0])
        return toVariant(payStatusText(payStatus))


class CEventActionsTableModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)
        self.addColumn(CDateCol(u'Назначено',      ['directionDate'], 15))
        self.addColumn(CBoolCol(u'Срочно',         ['isUrgent'],      15))
        self.addColumn(CDateCol(u'План',           ['plannedEndDate'],15))
        self.addColumn(CDateCol(u'Начато',         ['begDate'],       15))
        self.addColumn(CDateCol(u'Окончено',       ['endDate'],       15))
        self.addColumn(CActionTypeCol(u'Тип',                         15))
        self.addColumn(CEnumCol(u'Состояние',      ['status'], CActionStatus.names, 4))
        self.addColumn(CRefBookCol(u'Назначил',    ['setPerson_id'], 'vrbPersonWithSpeciality', 20))
        self.addColumn(CRefBookCol(u'Выполнил',    ['person_id'],    'vrbPersonWithSpeciality', 20))
        self.addColumn(CTextCol(u'Каб',            ['office'],                                6))
        self.addColumn(CTextCol(u'Примечания',     ['note'],                                  6))
        self.addColumn(CPayStatusColumn(u'Оплата', ['payStatus'], 20, 'l'))
        self.addColumn(CDoubleCol(u'Количество',      ['amount'], 20))
        self.addColumn(CDoubleCol(u'УЕТ',          ['uet'], 20))
        self.addColumn(CTextCol(u'МКБ',            ['MKB'], 20))
        self.setTable('Action')


class CEventVisitsTableModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)
        self.addColumn(CRefBookCol(u'Место',       ['scene_id'],         'rbScene',    15))
        self.addColumn(CDateCol(u'Дата',           ['date'],                           15))
        self.addColumn(CRefBookCol(u'Тип',         ['visitType_id'],    'rbVisitType', 15))
        self.addColumn(CRefBookCol(u'Услуга',      ['service_id'],      'rbService',   15))
        self.addColumn(CRefBookCol(u'Выполнил',    ['person_id'],     'vrbPersonWithSpecialityAndOrgStr', 20))
        self.addColumn(CEnumCol(u'Первичный',      ['isPrimary'], codeIsPrimary, 4, 'l'))
        self.addColumn(CPayStatusColumn(u'Оплата', ['payStatus'], 20, 'l'))
        self.setTable('Visit')


class CAmbCardDiagnosticsTableModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)
#        self.addColumn(CDateCol(u'дата назначения',['setDate'], 10))
        self.addColumn(CRefBookCol(u'Врач', ['person_id'], 'vrbPerson', 6))
        self.addColumn(CRefBookCol(u'Специальность', ['speciality_id'], 'rbSpeciality', 6))
        col = self.addColumn(CDateCol(u'Установлен', ['endDate'], 10))
        self.addColumn(CRefBookCol(u'Тип', ['diagnosisType_id'], 'rbDiagnosisType', 6))
        col = self.addColumn(CRefBookCol(
            u'ГpЗд', ['healthGroup_id'], 'rbHealthGroup', 2, CRBComboBox.showCode))
        col.setToolTip(u'Группа здоровья')
        self.addColumn(CDiagnosisCol(u'Диагноз', ['MKB', 'MKBEx'], 6))
        col = self.addColumn(CRefBookCol(u'Хар', ['character_id'], 'rbDiseaseCharacter', 6))
        col.setToolTip(u'Характер')
        col = self.addColumn(CRefBookCol(u'Фаза', ['phase_id'], 'rbDiseasePhases', 6))
        col.setToolTip(u'Фаза')
        col = self.addColumn(CRefBookCol(u'Ст', ['stage_id'], 'rbDiseaseStage', 6))
        col.setToolTip(u'Стадия')
        col = self.addColumn(CRefBookCol(
            u'ДН', ['dispanser_id'], 'rbDispanser', 6, CRBComboBox.showCode))
        col.setToolTip(u'Диспансерное наблюдение')
#        col = self.addColumn(CEnumCol(u'Г',['hospital'], CHospitalInfo.names, 4))
        col = self.addColumn(CTextCol(u'Госп', ['hospital'], 4))
        col.setToolTip(u'Госпитализация')
        col = self.addColumn(CRefBookCol(
            u'Тр', ['traumaType_id'], 'rbTraumaType', 6, CRBComboBox.showCode))
        col.setToolTip(u'Тип травмы')
        self.addColumn(CRefBookCol(u'Результат', ['result_id'], 'rbDiagnosticResult', 6))
        self.addColumn(CTextCol(u'Примечания', ['notes'], 6))
        self.addColumn(CTextCol(u'Описание', ['freeInput'], 6))

#        self.addColumn(CDateCol(u'дата', ['endDate'], 6))
#        self.addColumn(CEnumCol(u'СК', ['sanatorium'], [u'не нуждается', u'нуждается', u'направлен', u'пролечен'], 4))
        db = QtGui.qApp.db
        tableDiagnostic = db.table('Diagnostic')
        tableDiagnosis  = db.table('Diagnosis')
        queryTable = tableDiagnostic.leftJoin(tableDiagnosis, tableDiagnosis['id'].eq(tableDiagnostic['diagnosis_id']))
        self.setTable(queryTable)


class CAmbCardDiagnosticsVisitsTableModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)
        self.addColumn(CRefBookCol(u'Место',       ['scene_id'],         'rbScene',    15))
        self.addColumn(CDateCol(u'Дата',           ['date'],                           15))
        self.addColumn(CRefBookCol(u'Тип',         ['visitType_id'],    'rbVisitType', 15))
        self.addColumn(CRefBookCol(u'Услуга',      ['service_id'],      'rbService',   15))
        self.addColumn(CRefBookCol(u'Выполнил',    ['person_id'],     'vrbPersonWithSpeciality', 20))
        self.addColumn(CEnumCol(u'Первичный', ['isPrimary'], codeIsPrimary, 4, 'l'))
        self.addColumn(CDateCol(u'Дата ввода',     ['createDatetime'],                 15))
        self.addColumn(CDateCol(u'Дата изменения', ['modifyDatetime'],                 15))
        self.setTable('Visit')


class CAmbCardDiagnosticsAccompDiagnosticsTableModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)
#        self.addColumn(CDateCol(u'дата назначения',['setDate'], 10))
        self.addColumn(CRefBookCol(u'Врач', ['person_id'], 'vrbPerson', 6))
        self.addColumn(CRefBookCol(u'Специальность', ['speciality_id'], 'rbSpeciality', 6))
        col = self.addColumn(CDateCol(u'Установлен', ['endDate'], 10))
        self.addColumn(CRefBookCol(u'Тип', ['diagnosisType_id'], 'rbDiagnosisType', 6))
        col = self.addColumn(CRefBookCol(
            u'ГpЗд', ['healthGroup_id'], 'rbHealthGroup', 2, CRBComboBox.showCode))
        col.setToolTip(u'Группа здоровья')
        self.addColumn(CDiagnosisCol(u'Диагноз', ['MKB', 'MKBEx'], 6))
        col = self.addColumn(CRefBookCol(u'Хар', ['character_id'], 'rbDiseaseCharacter', 6))
        col.setToolTip(u'Характер')
        col = self.addColumn(CRefBookCol(u'Фаза', ['phase_id'], 'rbDiseasePhases', 6))
        col.setToolTip(u'Фаза')
        col = self.addColumn(CRefBookCol(u'Ст', ['stage_id'], 'rbDiseaseStage', 6))
        col.setToolTip(u'Стадия')
        col = self.addColumn(CRefBookCol(
            u'ДН', ['dispanser_id'], 'rbDispanser', 6, CRBComboBox.showCode))
        col.setToolTip(u'Диспансерное наблюдение')
#        col = self.addColumn(CEnumCol(u'Г',['hospital'], CHospitalInfo.names, 4))
        col = self.addColumn(CTextCol(u'Госп', ['hospital'], 4))
        col.setToolTip(u'Госпитализация')
        col = self.addColumn(CRefBookCol(
            u'Тр', ['traumaType_id'], 'rbTraumaType', 6, CRBComboBox.showCode))
        col.setToolTip(u'Тип травмы')
        self.addColumn(CRefBookCol(u'Результат', ['result_id'], 'rbDiagnosticResult', 6))
        self.addColumn(CTextCol(u'Примечания', ['notes'], 6))

#        self.addColumn(CDateCol(u'дата', ['endDate'], 6))
#        self.addColumn(CEnumCol(u'СК', ['sanatorium'], CHospitalInfo.names, 4))
        db = QtGui.qApp.db
        tableDiagnostic = db.table('Diagnostic')
        tableDiagnosis  = db.table('Diagnosis')
        queryTable = tableDiagnostic.leftJoin(tableDiagnosis, tableDiagnosis['id'].eq(tableDiagnostic['diagnosis_id']))
        self.setTable(queryTable)


class CAmbCardDiagnosticsActionsTableModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)
        self.addColumn(CDateCol(u'Назначено',      ['directionDate'], 15))
        self.addColumn(CActionTypeCol(u'Тип',                         15))
        self.addColumn(CBoolCol(u'Срочно',         ['isUrgent'],      15))
        self.addColumn(CEnumCol(u'Состояние',      ['status'], CActionStatus.names, 4))
        self.addColumn(CDateCol(u'План',           ['plannedEndDate'],15))
        self.addColumn(CDateCol(u'Начато',         ['begDate'],       15))
        self.addColumn(CDateCol(u'Окончено',       ['endDate'],       15))
        self.addColumn(CRefBookCol(u'Назначил',    ['setPerson_id'], 'vrbPersonWithSpeciality', 20))
        self.addColumn(CRefBookCol(u'Выполнил',    ['person_id'],    'vrbPersonWithSpeciality', 20))
        self.addColumn(CTextCol(u'Каб',            ['office'],                                6))
        self.addColumn(CTextCol(u'Примечания',     ['note'],                                  6))
        self.setTable('Action')
        self._mapColumn2Order = {u'directionDate'      :u'Action.directionDate',
                                 u'actionType_id'      :u'ActionType.name',
                                 u'isUrgent'           :u'Action.isUrgent',
                                 u'status'             :u'Action.status',
                                 u'plannedEndDate'     :u'Action.plannedEndDate',
                                 u'begDate'            :u'Action.begDate',
                                 u'endDate'            :u'Action.endDate',
                                 u'setPerson_id'       :u'vrbPersonWithSpeciality.name',
                                 u'person_id'          :u'vrbPersonWithSpeciality.name',
                                 u'office'             :u'Action.office',
                                 u'note'               :u'Action.note'
                                 }


    def getOrder(self, fieldName, column):
        if hasattr(self._cols[column], 'extraFields'):
            if len(self._cols[column].extraFields) > 0:
                fieldName = self._cols[column].extraFields[0]
        return self._mapColumn2Order[fieldName]


class CAmbCardStatusActionsTableModel(CAmbCardDiagnosticsActionsTableModel):
    pass


class CAmbCardVisitTableModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)

        self.addColumn(CRefBookCol(u'Врач',               ['person_id'],    'vrbPersonWithSpeciality', 20))
        self.addColumn(CRefBookCol(u'Ассистент',          ['assistant_id'], 'vrbPersonWithSpeciality', 20))
        self.addColumn(CRefBookCol(u'Место',              ['scene_id'],     'rbScene',                 10))
        self.addColumn(CDateCol(u'Дата',                  ['date'],                                    15))
        self.addColumn(CRefBookCol(u'Тип визита',         ['visitType_id'], 'rbVisitType',             10))
        self.addColumn(CRefBookCol(u'Услуга',             ['service_id'],   'rbService',               20))
        self.addColumn(CRefBookCol(u'Тип финансирования', ['finance_id'],   'rbFinance',               10))
        self.loadField('event_id')
        self.setTable('Visit')


class CAmbCardAttachedFilesTableModel(CAttachedFilesModel):
    def __init__(self, parent):
        CAttachedFilesModel.__init__(self, parent)
        self.setInterface(QtGui.qApp.webDAVInterface)


    def loadItems(self, clientId):
        db = QtGui.qApp.db
        items = CAttachedFilesLoader.loadItems(self.interface, 'Client_FileAttach', clientId)
        tableEvent = db.table('Event')
        tableAction = db.table('Action')
        for eventId in db.getIdList(tableEvent,
                                    'id',
                                    where=['deleted=0',
                                           tableEvent['client_id'].eq(clientId),
                                          ]
                                   ):
            items.extend(CAttachedFilesLoader.loadItems(self.interface, 'Event_FileAttach', eventId))
            for actionId in db.getIdList(tableAction,
                                         'id',
                                         where=['deleted=0',
                                                tableAction['event_id'].eq(eventId),
                                               ]
                                        ):
                items.extend(CAttachedFilesLoader.loadItems(self.interface, 'Action_FileAttach', actionId))

        self.items = items
        self.reset()


    def saveItems(self, masterId):
        pass



# ##################################3

class CRegistryActionsTableView(CTableView):
    def __init__(self, parent):
        CTableView.__init__(self, parent)
        self._order = None
        self._orderColumn = None
        self._isDesc = True


    def order(self):
        return self._order


    def setOrder(self, column):
        if column is not None:
            self._isDesc = not self._isDesc if self._orderColumn == column else False
            self._order = self.getOrder(column) + (' DESC' if self._isDesc else ' ASC')
            self._orderColumn = column
            self.horizontalHeader().setSortIndicator(column, Qt.DescendingOrder if self._isDesc else Qt.AscendingOrder)
        else:
            self._order = None
            self._orderColumn = None
            self._isDesc = True


    def getOrder(self, column):
        return self.model().getOrder(self.model().cols()[column].fields()[0], column)


    def setClientInfoHidden(self, hideClientInfo):
        verticalHeaderView = self.verticalHeader()
        h = self.fontMetrics().height()
        if hideClientInfo:
            verticalHeaderView.setDefaultSectionSize(3*h/2)
            verticalHeaderView.hide()
        else:
            verticalHeaderView.setDefaultSectionSize(3*h)
            verticalHeaderView.show()
#        self.model().reset()


    def contentToHTML(self, showMask=None):
        model = self.model()
        cols = model.cols()
        if showMask is None:
            showMask = [None]*(len(cols)+1)
        _showMask = [ not self.isColumnHidden(iCol) if v is None else v
                      for iCol, v in enumerate(showMask)
                    ]
        QtGui.qApp.startProgressBar(model.rowCount())
        try:
            doc = QtGui.QTextDocument()
            cursor = QtGui.QTextCursor(doc)

            cursor.setCharFormat(CReportBase.ReportTitle)
            header = self.reportHeader()
            if Qt.mightBeRichText(header):
                cursor.insertHtml(header)
            else:
                cursor.insertText(header)
            cursor.insertBlock()
            cursor.setCharFormat(CReportBase.ReportBody)
            description = self.reportDescription()
            if Qt.mightBeRichText(description):
                cursor.insertHtml(description)
            else:
                cursor.insertText(description)
            cursor.insertBlock()

            colWidths  = [ self.columnWidth(i) for i in xrange(len(cols)+1) ]
            colWidths.insert(0,10)
            totalWidth = sum(colWidths)
            tableColumns = []
            for iCol, colWidth in enumerate(colWidths):
                widthInPercents = str(max(1, colWidth*90/totalWidth))+'%'
                if iCol == 0:
                    tableColumns.append((widthInPercents, [u'№'], CReportBase.AlignRight))
                elif iCol == 1:
                    tableColumns.append((widthInPercents, [u'ФИО'], CReportBase.AlignLeft))
                else:
                    if not _showMask[iCol-2]:
                        continue
                    col = cols[iCol-2]
                    colAlingment = Qt.AlignHorizontal_Mask & forceInt(col.alignment())
                    format = QtGui.QTextBlockFormat()
                    format.setAlignment(Qt.AlignmentFlag(colAlingment))
                    tableColumns.append((widthInPercents, [forceString(col.title())], format))

            table = createTable(cursor, tableColumns)
            for iModelRow in xrange(model.rowCount()):
                QtGui.qApp.stepProgressBar()
                QtGui.qApp.processEvents(QEventLoop.ExcludeUserInputEvents)
                iTableRow = table.addRow()
                table.setText(iTableRow, 0, iModelRow+1)
                headerData = model.headerData(iTableRow-1, Qt.Vertical)
                table.setText(iTableRow, 1, forceString(headerData))
                iTableCol = 2
                for iModelCol in xrange(len(cols)):
                    if not _showMask[iModelCol]:
                        continue
                    index = model.createIndex(iModelRow, iModelCol)
                    text = forceString(model.data(index))
                    table.setText(iTableRow, iTableCol, text)
                    iTableCol += 1
            return doc
        finally:
            QtGui.qApp.stopProgressBar()


class CRegistryActionsMCTableView(CExtendedSelectionTableView):
    def __init__(self, parent):
        CExtendedSelectionTableView.__init__(self, parent)
        self._order = None
        self._orderColumn = None
        self._isDesc = True


class CActionsTableModel(CTableModel):
    class CLocClientColumn(CEventsTableModel.CLocClientColumn):
        def __init__(self, title, fields, defaultWidth, clientCache, eventCache):
            CEventsTableModel.CLocClientColumn.__init__(self, title, fields, defaultWidth, clientCache)
            self.eventCache = eventCache


        def format(self, values):
            val = values[0]
            eventId  = forceRef(val)
            eventRecord = self.eventCache.get(eventId)
            if eventRecord:
                clientValues = [eventRecord.value('client_id')]
                clientVal = clientValues[0]
                clientId  = forceRef(clientVal)
                clientRecord = self.clientCache.get(clientId)
                if clientRecord:
                    name  = formatShortNameInt(forceString(clientRecord.value('lastName')),
                                       forceString(clientRecord.value('firstName')),
                                       forceString(clientRecord.value('patrName')))
                    return toVariant(name)
                return CCol.invalid
            return CCol.invalid


    class CLocClientIdentifierColumn(CCol):
        def __init__(self, title, fields, defaultWidth, eventCache):
            CCol.__init__(self, title, fields, defaultWidth, 'l')
            self.eventCache = eventCache
            self.identifiersCache = CRecordCache()


        def getClientIdentifier(self, clientId):
            identifier = self.identifiersCache.get(clientId)
            if identifier is None:
                accountingSystemId = QtGui.qApp.defaultAccountingSystemId()
                if accountingSystemId is None:
                    identifier = clientId
                else:
                    db = QtGui.qApp.db
                    tableClientIdentification = db.table('ClientIdentification')
                    cond = [tableClientIdentification['client_id'].eq(clientId),
                            tableClientIdentification['accountingSystem_id'].eq(accountingSystemId)]
                    record = db.getRecordEx(tableClientIdentification, tableClientIdentification['identifier'], cond)
                    identifier = forceString(record.value(0)) if record else ''
                self.identifiersCache.put(clientId, identifier)
            return identifier


        def format(self, values):
            val = values[0]
            eventId = forceRef(val)
            eventRecord = self.eventCache.get(eventId)
            if eventRecord:
                clientId = forceRef(eventRecord.value('client_id'))
                return toVariant(self.getClientIdentifier(clientId))
            return CCol.invalid


    class CLocClientBirthDateColumn(CEventsTableModel.CLocClientBirthDateColumn):
        def __init__(self, title, fields, defaultWidth, clientCache, eventCache):
            CEventsTableModel.CLocClientBirthDateColumn.__init__(self, title, fields, defaultWidth, clientCache)
            self.eventCache = eventCache

        def format(self, values):
            val = values[0]
            eventId  = forceRef(val)
            eventRecord = self.eventCache.get(eventId)
            if eventRecord:
                return CEventsTableModel.CLocClientBirthDateColumn.format(self, [eventRecord.value('client_id')])
            return CCol.invalid


    class CLocClientSexColumn(CEventsTableModel.CLocClientSexColumn):
        def __init__(self, title, fields, defaultWidth, clientCache, eventCache):
            CEventsTableModel.CLocClientSexColumn.__init__(self, title, fields, defaultWidth, clientCache)
            self.eventCache = eventCache

        def format(self, values):
            val = values[0]
            eventId  = forceRef(val)
            eventRecord = self.eventCache.get(eventId)
            if eventRecord:
                return CEventsTableModel.CLocClientSexColumn.format(self, [eventRecord.value('client_id')])
            return CCol.invalid


    class CLocEventExternalIdColumn(CDesignationCol):
        def __init__(self, title, fields, designationChain, defaultWidth, extraFields=[], alignment='l'):
            CDesignationCol.__init__(self, title, fields, designationChain, defaultWidth, alignment)
            self.extraFields = extraFields


    class CLocDateShowTimeColumn(CDateCol):
        def format(self, values):
            val = values[0]
            actionTypeId  = forceRef(values[1])
            if actionTypeId:
                actionType   = CActionTypeCache.getById(actionTypeId)
                showTime = actionType.showTime
                if showTime:
                    val = val.toDateTime()
                    return QVariant(val.toString('dd.MM.yyyy hh:mm'))
                else:
                    val = val.toDate()
                    return QVariant(val.toString('dd.MM.yyyy'))
            return CCol.invalid


    def __init__(self, parent, clientCache, eventCache):
        CTableModel.__init__(self, parent)
        clientCol   = CActionsTableModel.CLocClientColumn( u'Ф.И.О.', ['event_id'], 60, clientCache, eventCache)
#        clientIdentifierCol = CActionsTableModel.CLocClientIdentifierColumn(u'Идентификатор', ['event_id'], 30, eventCache)
        clientBirthDateCol = CActionsTableModel.CLocClientBirthDateColumn(u'Дата рожд.', ['event_id'], 20, clientCache, eventCache)
        clientSexCol = CActionsTableModel.CLocClientSexColumn(u'Пол', ['event_id'], 5, clientCache, eventCache)
        self.addColumn(clientCol)
        self.addColumn(clientBirthDateCol)
        self.addColumn(clientSexCol)
        self.addColumn(CActionsTableModel.CLocDateShowTimeColumn(u'Назначено',      ['directionDate', 'actionType_id'], 15))
        self.addColumn(CActionTypeCol(u'Тип',                         15))
        self.addColumn(CIntCol(u'Количество',      ['amount'],        6))
        self.addColumn(CTextCol(u'УЕТ',            ['uet'],             6))
        self.addColumn(CBoolCol(u'Срочно',         ['isUrgent'],      15))
        self.addColumn(CEnumCol(u'Состояние',      ['status'], CActionStatus.names, 4))
        self.addColumn(CActionsTableModel.CLocDateShowTimeColumn(u'План',           ['plannedEndDate', 'actionType_id'],15))
        self.addColumn(CActionsTableModel.CLocDateShowTimeColumn(u'Начато',         ['begDate', 'actionType_id'],       15))
        self.addColumn(CActionsTableModel.CLocDateShowTimeColumn(u'Окончено',       ['endDate', 'actionType_id'],       15))
        self.addColumn(CTextCol(u'МКБ',            ['MKB'],       15))
        self.addColumn(CRefBookCol(u'Назначил',    ['setPerson_id'], 'vrbPersonWithSpeciality', 20))
        self.addColumn(CRefBookCol(u'Выполнил',    ['person_id'],    'vrbPersonWithSpeciality', 20))
        self.addColumn(CTextCol(u'Каб',            ['office'],                                6))
        self.addColumn(CTextCol(u'Примечания',     ['note'],                                  6))
        self.addColumn(CActionsTableModel.CLocEventExternalIdColumn(u'Номер документа', ['event_id'],   ('Event', 'externalId'), 20, ['externalId']))
        self.addColumn(CRefBookCol(u'Тип финансирования',  ['finance_id'], 'rbFinance',    10))
#        self.addColumn(CRefBookCol(u'Договор',            ['contract_id'], 'vrbContract', 20))
        self.addColumn(CDesignationCol(u'Договор',         ['contract_id'],('vrbContract', 'code'), 20))
        self.loadField('id')
        self.loadField('event_id')
        self.loadField('amount')
        self.setTable('Action')
        self._mapColumn2Order = {u'event_id'           :u'CONCAT(Client.lastName, Client.firstName, Client.patrName)',
                                 u'birthDate'          :u'Client.birthDate',
                                 u'sex'                :u'Client.sex',
                                 u'directionDate'      :u'Action.directionDate',
                                 u'actionType_id'      :u'Action.actionType_id',
                                 u'amount'             :u'Action.amount',
                                 u'uet'                :u'Action.uet',
                                 u'isUrgent'           :u'Action.isUrgent',
                                 u'status'             :u'Action.status',
                                 u'plannedEndDate'     :u'Action.plannedEndDate',
                                 u'begDate'            :u'Action.begDate',
                                 u'endDate'            :u'Action.endDate',
                                 u'MKB'                :u'Action.MKB',
                                 u'setPerson_id'       :u'vrbSetPersonWithSpeciality.name',
                                 u'person_id'          :u'vrbPersonWithSpeciality.name',
                                 u'office'             :u'Action.office',
                                 u'note'               :u'Action.note',
                                 u'externalId'         :u'Event.externalId',
                                 u'finance_id'         :u'rbFinance.code',
                                 u'contract_id'        :u'vrbContract.code'
                                 }


    def getOrder(self, fieldName, column):
        if hasattr(self._cols[column], 'extraFields'):
            if len(self._cols[column].extraFields) > 0:
                fieldName = self._cols[column].extraFields[0]
        return self._mapColumn2Order[fieldName]


#    def headerData(self, section, orientation, role=Qt.DisplayRole):
##        if orientation == Qt.Vertical:
##            if role == Qt.DisplayRole:
##                id = self._idList[section]
##                self._recordsCache.weakFetch(id, self._idList[max(0, section-self.fetchSize):(section+self.fetchSize)])
##                record = self._recordsCache.get(id)
##                clientValues   = self.clientCol.extractValuesFromRecord(record)
##                clientValue = forceString(self.clientCol.format(clientValues))
##                clientIdentifier = ''
##                clientIdentifierValues = self.clientIdentifierCol.extractValuesFromRecord(record)
##                clientIdentifier = forceString(self.clientIdentifierCol.format(clientIdentifierValues))
##                clientBirthDateValues = self.clientBirthDateCol.extractValuesFromRecord(record)
##                clientBirthDate = forceString(self.clientBirthDateCol.format(clientBirthDateValues))
##                clientSexValues = self.clientSexCol.extractValuesFromRecord(record)
##                clientSex = forceString(self.clientSexCol.format(clientSexValues))
##                clientFIOSex = u', '.join([clientValue, clientSex])
##                birthDateSex = u', '.join([clientIdentifier, clientBirthDate])
##                result =  u'\n'.join([clientFIOSex, birthDateSex])
##                return QVariant(result)
#        return CTableModel.headerData(self, section, orientation, role)


    def getAmountSum(self):
        result = 0
        for id in self.idList():
            record = self.recordCache().get(id)
            result += forceDouble(record.value('amount'))
        return result

    def getUetSum(self):
        result = 0
        for id in self.idList():
            record = self.recordCache().get(id)
            result += forceDouble(record.value('uet'))
        return result


class CMedicalCommissionActionsTableModel(CActionsTableModel):
    class CLocActionPropertiColumn(CCol):
        def __init__(self, title, fields, propertiName, defaultWidth, alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.actionCaches = {}
            self.propertiName = propertiName

        def clearActionCaches(self):
            self.actionCaches.clear()

        def updateActionCaches(self, id):
            if id in self.actionCaches.keys():
                self.actionCaches.pop(id, None)

        def format(self, values):
            actionId = forceRef(values[0])
            if actionId:
                action = self.actionCaches.get(actionId, None)
                if not action:
                    db = QtGui.qApp.db
                    tableAction = db.table('Action')
                    record = db.getRecordEx(tableAction, '*', [tableAction['id'].eq(actionId), tableAction['deleted'].eq(0)])
                    if record:
                        action = CAction(record=record)
                        if action:
                            self.actionCaches[actionId] = action
                if action and self.propertiName in action._actionType._propertiesByName:
                    return QVariant(action[self.propertiName])
            return CCol.invalid

    class CLocActionDateColumn(CCol):
        def __init__(self, title, fields, fieldName, defaultWidth, alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.actionCaches = {}
            self.fieldName = fieldName

        def clearActionCaches(self):
            self.actionCaches.clear()

        def updateActionCaches(self, id):
            if id in self.actionCaches.keys():
                self.actionCaches.pop(id, None)

        def format(self, values):
            actionId = forceRef(values[0])
            actionTypeId = forceRef(values[1])
            if actionId:
                record = self.actionCaches.get(actionId, None)
                if not record:
                    db = QtGui.qApp.db
                    tableAction = db.table('Action')
                    record = db.getRecordEx(tableAction, '*', [tableAction['prevAction_id'].eq(actionId), tableAction['deleted'].eq(0)])
                    if record:
                        self.actionCaches[actionId] = record
                if record and self.fieldName:
                    actionTypeId = forceRef(record.value('actionType_id'))
                    if actionTypeId:
                        actionType   = CActionTypeCache.getById(actionTypeId)
                        showTime = actionType.showTime
                        if showTime:
                            val = forceDateTime(record.value(self.fieldName))
                            return QVariant(val.toString('dd.MM.yyyy hh:mm'))
                        else:
                            val = forceDate(record.value(self.fieldName))
                            return QVariant(val.toString('dd.MM.yyyy'))
                    return CCol.invalid
            return CCol.invalid

    Col_Number = 4
    Col_NumberMC = 5
    Col_Decision = 14

    def __init__(self, parent, clientCache, eventCache):
        CActionsTableModel.__init__(self, parent, clientCache, eventCache)
        self._cols = []
        clientCol   = CActionsTableModel.CLocClientColumn( u'Ф.И.О.', ['event_id'], 60, clientCache, eventCache)
        clientBirthDateCol = CActionsTableModel.CLocClientBirthDateColumn(u'Дата рожд.', ['event_id'], 20, clientCache, eventCache)
        clientSexCol = CActionsTableModel.CLocClientSexColumn(u'Пол', ['event_id'], 5, clientCache, eventCache)
        self.addColumn(clientCol)
        self.addColumn(clientBirthDateCol)
        self.addColumn(clientSexCol)
        self.addColumn(CActionTypeCol(u'Тип', 15, showFields=CRBComboBox.showName))
        self.addColumn(CMedicalCommissionActionsTableModel.CLocActionPropertiColumn(u'Номер',    ['id'], u'Номер',    15))
        self.addColumn(CMedicalCommissionActionsTableModel.CLocActionPropertiColumn(u'Номер ЛН', ['id'], u'Номер ЛН', 15))
        self.addColumn(CEnumCol(                                 u'Состояние', ['status'], CActionStatus.names,    4))
        self.addColumn(CActionsTableModel.CLocDateShowTimeColumn(u'Назначено', ['directionDate', 'actionType_id'], 15))
        self.addColumn(CActionsTableModel.CLocDateShowTimeColumn(u'План',      ['plannedEndDate', 'actionType_id'],15))
        self.addColumn(CActionsTableModel.CLocDateShowTimeColumn(u'Начато',    ['begDate', 'actionType_id'],       15))
        self.addColumn(CActionsTableModel.CLocDateShowTimeColumn(u'Окончено',  ['endDate', 'actionType_id'],       15))
        self.addColumn(CTextCol(   u'МКБ',        ['MKB'],                                     15))
        self.addColumn(CRefBookCol(u'Назначил',   ['setPerson_id'], 'vrbPersonWithSpeciality', 20))
        self.addColumn(CRefBookCol(u'Выполнил',   ['person_id'],    'vrbPersonWithSpeciality', 20))
        self.addColumn(CMedicalCommissionActionsTableModel.CLocActionPropertiColumn(u'Решение', ['id'], u'Решение', 50))
        self.addColumn(CTextCol(   u'Примечания', ['note'],                                    6))
        self.loadField('id')
        self.loadField('event_id')
        self.setTable('Action')
        self._mapColumn2Order = {u'id'                 :u'expertNumberExpertise',
                                 u'event_id'           :u'CONCAT(Client.lastName, Client.firstName, Client.patrName)',
                                 u'birthDate'          :u'Client.birthDate',
                                 u'sex'                :u'Client.sex',
                                 u'directionDate'      :u'Action.directionDate',
                                 u'actionType_id'      :u'Action.actionType_id',
                                 u'status'             :u'Action.status',
                                 u'plannedEndDate'     :u'Action.plannedEndDate',
                                 u'begDate'            :u'Action.begDate',
                                 u'endDate'            :u'Action.endDate',
                                 u'MKB'                :u'Action.MKB',
                                 u'setPerson_id'       :u'vrbSetPersonWithSpeciality.name',
                                 u'person_id'          :u'vrbPersonWithSpeciality.name',
                                 u'note'               :u'Action.note',
                                 u'id'                 :u'expertNumberMC',
                                 u'id'                 :u'expertDecision',
                                 }


    def getOrder(self, fieldName, column):
        if column == CMedicalCommissionActionsTableModel.Col_Number:
            return u'expertNumberExpertise'
        elif column == CMedicalCommissionActionsTableModel.Col_NumberMC:
            return u'expertNumberMC'
        elif column == CMedicalCommissionActionsTableModel.Col_Decision:
            return u'expertDecision'
        if hasattr(self._cols[column], 'extraFields'):
            if len(self._cols[column].extraFields) > 0:
                fieldName = self._cols[column].extraFields[0]
        return self._mapColumn2Order[fieldName]


    def updateColumnsCaches(self, id):
        self._cols[CMedicalCommissionActionsTableModel.Col_Number].updateActionCaches(id)
        self._cols[CMedicalCommissionActionsTableModel.Col_NumberMC].updateActionCaches(id)
        self._cols[CMedicalCommissionActionsTableModel.Col_Decision].updateActionCaches(id)


    def clearColumnsCaches(self):
        self._cols[CMedicalCommissionActionsTableModel.Col_Number].clearActionCaches()
        self._cols[CMedicalCommissionActionsTableModel.Col_NumberMC].clearActionCaches()
        self._cols[CMedicalCommissionActionsTableModel.Col_Decision].clearActionCaches()


    def data(self, index, role=Qt.DisplayRole):
        result = CActionsTableModel.data(self, index, role)
        if role == Qt.ToolTipRole:
            column = index.column()
            if column == CMedicalCommissionActionsTableModel.Col_Decision:
                row = index.row()
                (col, values) = self.getRecordValues(column, row)
                return col.format(values)
        return result


class CProtocolMCActionsTableModel(CMedicalCommissionActionsTableModel):
    Col_DirectionDateMSI  = 16
    Col_PlannedEndDateMSI = 17
    Col_EndDateMSI        = 18

    def __init__(self, parent, clientCache, eventCache):
        CMedicalCommissionActionsTableModel.__init__(self, parent, clientCache, eventCache)
        self.addColumn(CMedicalCommissionActionsTableModel.CLocActionDateColumn(u'Направление на МСЭ', ['id', 'actionType_id'], 'directionDate',  15))
        self.addColumn(CMedicalCommissionActionsTableModel.CLocActionDateColumn(u'Регистрация МСЭ',    ['id', 'actionType_id'], 'plannedEndDate', 15))
        self.addColumn(CMedicalCommissionActionsTableModel.CLocActionDateColumn(u'Выполнение МСЭ',     ['id', 'actionType_id'], 'endDate',        15))
        self.setTable('Action')
        self._mapColumn2Order = {u'id'                 :u'expertNumberExpertise',
                                 u'event_id'           :u'CONCAT(Client.lastName, Client.firstName, Client.patrName)',
                                 u'birthDate'          :u'Client.birthDate',
                                 u'sex'                :u'Client.sex',
                                 u'directionDate'      :u'Action.directionDate',
                                 u'actionType_id'      :u'Action.actionType_id',
                                 u'status'             :u'Action.status',
                                 u'plannedEndDate'     :u'Action.plannedEndDate',
                                 u'begDate'            :u'Action.begDate',
                                 u'endDate'            :u'Action.endDate',
                                 u'MKB'                :u'Action.MKB',
                                 u'setPerson_id'       :u'vrbSetPersonWithSpeciality.name',
                                 u'person_id'          :u'vrbPersonWithSpeciality.name',
                                 u'note'               :u'Action.note',
                                 u'id'                 :u'expertNumberMC',
                                 u'id'                 :u'expertDecision',
                                 u'id'                 :u'directionDateMSI',
                                 u'id'                 :u'plannedEndDateMSI',
                                 u'id'                 :u'endDateMSI',
                                 }


    def getOrder(self, fieldName, column):
        if column == CProtocolMCActionsTableModel.Col_DirectionDateMSI:
            return u'directionDateMSI'
        elif column == CProtocolMCActionsTableModel.Col_PlannedEndDateMSI:
            return u'plannedEndDateMSI'
        elif column == CProtocolMCActionsTableModel.Col_EndDateMSI:
            return u'endDateMSI'
        else:
            return CMedicalCommissionActionsTableModel.getOrder(self, fieldName, column)
        return self._mapColumn2Order[fieldName]


    def updateColumnsCaches(self, id):
        CMedicalCommissionActionsTableModel.updateColumnsCaches(self, id)
        self._cols[CProtocolMCActionsTableModel.Col_DirectionDateMSI].updateActionCaches(id)
        self._cols[CProtocolMCActionsTableModel.Col_PlannedEndDateMSI].updateActionCaches(id)
        self._cols[CProtocolMCActionsTableModel.Col_EndDateMSI].updateActionCaches(id)


    def clearColumnsCaches(self):
        CMedicalCommissionActionsTableModel.clearColumnsCaches(self)
        self._cols[CProtocolMCActionsTableModel.Col_DirectionDateMSI].clearActionCaches()
        self._cols[CProtocolMCActionsTableModel.Col_PlannedEndDateMSI].clearActionCaches()
        self._cols[CProtocolMCActionsTableModel.Col_EndDateMSI].clearActionCaches()


class CMedicalSocialInspectionActionsTableModel(CMedicalCommissionActionsTableModel):
    class CLocDateActionPropertiColumn(CDateCol):
        def __init__(self, title, fields, propertiName, defaultWidth, alignment='l', highlightRedDate=True):
            CDateCol.__init__(self, title, fields, defaultWidth, alignment)
            self.actionCaches = {}
            self.propertiName = propertiName

        def clearActionCaches(self):
            self.actionCaches.clear()

        def updateActionCaches(self, id):
            if id in self.actionCaches.keys():
                self.actionCaches.pop(id, None)

        def format(self, values):
            actionId = forceRef(values[0])
            if actionId:
                action = self.actionCaches.get(actionId, None)
                if not action:
                    db = QtGui.qApp.db
                    tableAction = db.table('Action')
                    record = db.getRecordEx(tableAction, '*', [tableAction['id'].eq(actionId), tableAction['deleted'].eq(0)])
                    if record:
                        action = CAction(record=record)
                        if action:
                            self.actionCaches[actionId] = action
                if action and self.propertiName in action._actionType._propertiesByName:
                    properti = action[self.propertiName]
                    if properti:
                        return QVariant(properti.toString(Qt.LocaleDate))
            return CCol.invalid

    Col_Decision = 16
    Col_DisabilityGroup = 17

    def __init__(self, parent, clientCache, eventCache):
        CMedicalCommissionActionsTableModel.__init__(self, parent, clientCache, eventCache)
        self._cols = []
        clientCol   = CActionsTableModel.CLocClientColumn( u'Ф.И.О.', ['event_id'], 60, clientCache, eventCache)
        clientBirthDateCol = CActionsTableModel.CLocClientBirthDateColumn(u'Дата рожд.', ['event_id'], 20, clientCache, eventCache)
        clientSexCol = CActionsTableModel.CLocClientSexColumn(u'Пол', ['event_id'], 5, clientCache, eventCache)
        self.addColumn(clientCol)
        self.addColumn(clientBirthDateCol)
        self.addColumn(clientSexCol)
        self.addColumn(CActionTypeCol(u'Тип', 15, showFields=CRBComboBox.showName))
        self.addColumn(CMedicalCommissionActionsTableModel.CLocActionPropertiColumn(u'Номер',    ['id'], u'Номер',    15))
        self.addColumn(CMedicalCommissionActionsTableModel.CLocActionPropertiColumn(u'Номер ЛН', ['id'], u'Номер ЛН', 15))
        self.addColumn(CEnumCol(                                 u'Состояние', ['status'], CActionStatus.names,    4))
        self.addColumn(CActionsTableModel.CLocDateShowTimeColumn(u'Назначено', ['directionDate', 'actionType_id'], 15))
        self.addColumn(CActionsTableModel.CLocDateShowTimeColumn(u'План',      ['plannedEndDate', 'actionType_id'],15))
        self.addColumn(CActionsTableModel.CLocDateShowTimeColumn(u'Начато',    ['begDate', 'actionType_id'],       15))
        self.addColumn(CActionsTableModel.CLocDateShowTimeColumn(u'Окончено',  ['endDate', 'actionType_id'],       15))
        self.addColumn(CTextCol(   u'МКБ',        ['MKB'],                                     15))
        self.addColumn(CRefBookCol(u'Назначил',   ['setPerson_id'], 'vrbPersonWithSpeciality', 20))
        self.addColumn(CRefBookCol(u'Выполнил',   ['person_id'],    'vrbPersonWithSpeciality', 20))
        self.addColumn(CMedicalSocialInspectionActionsTableModel.CLocDateActionPropertiColumn(u'Рег-ция', ['id'], u'Дата регистрации МСЭ',         50).setToolTip(u'Дата регистрации документов в бюро МСЭ'))
        self.addColumn(CMedicalSocialInspectionActionsTableModel.CLocDateActionPropertiColumn(u'МСЭ',     ['id'], u'Дата освидетельствования МСЭ', 50).setToolTip(u'Дата освидетельствования в бюро МСЭ'))
        self.addColumn(CMedicalCommissionActionsTableModel.CLocActionPropertiColumn(u'Результат', ['id'], u'Результат', 50))
        self.addColumn(CMedicalCommissionActionsTableModel.CLocActionPropertiColumn(u'Группа',    ['id'], u'Группа инвалидности', 50))
        self.addColumn(CTextCol(   u'Примечания', ['note'],                                    6))
        self.setTable('Action')


    def data(self, index, role=Qt.DisplayRole):
        result = CActionsTableModel.data(self, index, role)
        if role == Qt.ToolTipRole:
            column = index.column()
            if column in [CMedicalSocialInspectionActionsTableModel.Col_Decision, CMedicalSocialInspectionActionsTableModel.Col_DisabilityGroup]:
                row = index.row()
                (col, values) = self.getRecordValues(column, row)
                return col.format(values)
        return result


class CExpertTempInvalidTableModel(CTableModel):
    class CLocClientColumn(CEventsTableModel.CLocClientColumn):
        def __init__(self, title, fields, defaultWidth, clientCache):
            CEventsTableModel.CLocClientColumn.__init__(self, title, fields, defaultWidth, clientCache)

    class CLocClientBirthDateColumn(CEventsTableModel.CLocClientBirthDateColumn):
        def __init__(self, title, fields, defaultWidth, clientCache):
            CEventsTableModel.CLocClientBirthDateColumn.__init__(self, title, fields, defaultWidth, clientCache)

    class CLocClientSexColumn(CEventsTableModel.CLocClientSexColumn):
        def __init__(self, title, fields, defaultWidth, clientCache):
            CEventsTableModel.CLocClientSexColumn.__init__(self, title, fields, defaultWidth, clientCache)

    class CPrevCol(CCol):
        def __init__(self, title, fields, defaultWidth):
            CCol.__init__(self, title, fields, defaultWidth, 'l')
            self.recordCache = None

        def format(self, values):
            id = forceRef(values[0])
            record = self.recordCache.get(id) if self.recordCache and id else None
            if record:
                return QVariant(forceString(record.value('serial')) + ' ' + forceString(record.value('number')))
            return CCol.invalid

    class CLocCountVisitsColumn(CCol):
        def __init__(self, title, fields, defaultWidth, alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self._cache = {}

        def format(self, values):
            tempInvalidId = forceRef(values[0])
            result = self._cache.get(tempInvalidId, 0)
            db = QtGui.qApp.db
            stmt = u'''SELECT COUNT(Visit.id)
            FROM TempInvalid
            INNER JOIN Event ON TempInvalid.client_id = Event.client_id
            INNER JOIN Visit ON Visit.event_id = Event.id
            WHERE TempInvalid.id = %s AND Event.deleted = 0 AND Visit.deleted = 0 AND TempInvalid.deleted = 0
            AND DATE(Visit.date) >= TempInvalid.begDate AND DATE(Visit.date) <= TempInvalid.endDate'''%(tempInvalidId)
            query = db.query(stmt)
            if query.next():
                result = forceInt(query.value(0))
                self._cache[tempInvalidId] = result
            else:
                result = CCol.invalid
                self._cache[tempInvalidId] = result
            return result

        def invalidateRecordsCache(self):
            self._cache.clear()

        def updateRecordsCache(self, id):
            if id in self._cache.keys():
                self._cache.pop(id, None)

    Col_CountVisits = 12

    def __init__(self, parent, clientCache):
        CTableModel.__init__(self, parent)
        clientCol   = CExpertTempInvalidTableModel.CLocClientColumn( u'Ф.И.О.', ['client_id'], 60, clientCache)
        clientBirthDateCol = CExpertTempInvalidTableModel.CLocClientBirthDateColumn(u'Дата рожд.', ['client_id'], 20, clientCache)
        clientSexCol = CExpertTempInvalidTableModel.CLocClientSexColumn(u'Пол', ['client_id'], 5, clientCache)
        self.addColumn(clientCol)
        self.addColumn(clientBirthDateCol)
        self.addColumn(clientSexCol)
        self.addColumn(CBoolCol(u'СтСт',                                             ['insuranceOfficeMark'],                                                  6))
        self.addColumn(CRefBookCol(u'Причина нетрудоспособности',                    ['tempInvalidReason_id'], 'rbTempInvalidReason',                          15))
        self.addColumn(CDateCol(u'Дата начала ВУТ',                                  ['caseBegDate'],                                                          10))
        self.addColumn(CDateCol(u'Начало',                                           ['begDate'],                                                              12))
        self.addColumn(CDateCol(u'Окончание',                                        ['endDate'],                                                              12))
        self.addColumn(CRefBookCol(u'Врач',                                          ['person_id'],            'vrbPerson',                                    6))
        self.addColumn(CDesignationCol(u'МКБ',                                       ['diagnosis_id'],         ('Diagnosis', 'MKB'),                           6))
        self.addColumn(CEnumCol(u'Состояние',                                        ['closed'],               [u'открыт', u'закрыт', u'продлён', u'передан'], 4))
        self.addColumn(CNumCol(u'Длительность',                                      ['duration'],                                                             10))
        self.addColumn(CExpertTempInvalidTableModel.CLocCountVisitsColumn(u'Визиты', ['id'],                                                                   10))
        self.addColumn(CRefBookCol(u'Тип',                                           ['doctype_id'],           'rbTempInvalidDocument',                        15))
        self.addColumn(CDateCol(u'В стационаре "с"',                                 ['begDateStationary'],                                                    12))
        self.addColumn(CDateCol(u'В стационаре "по"',                                ['endDateStationary'],                                                    12))
        self.addColumn(CRefBookCol(u'Нарушение режима',                              ['break_id'],             'rbTempInvalidBreak',                           15))
        self.addColumn(CDateCol(u'Дата нарушения режима',                            ['breakDate'],                                                            12))
        self.addColumn(CRefBookCol(u'Результат',                                     ['result_id'],            'rbTempInvalidResult',                          15))
        self.addColumn(CDateCol(u'Дата результата - Приступить к работе',            ['resultDate'],                                                           12))
        self.addColumn(CDateCol(u'Дата результата - Иное',                           ['resultOtherwiseDate'],                                                  12))
        self.addColumn(CTextCol(u'Номер путевки',                                    ['numberPermit'],                                                         12))
        self.addColumn(CDateCol(u'Дата начала путевки',                              ['begDatePermit'],                                                        12))
        self.addColumn(CDateCol(u'Дата окончания путевки',                           ['endDatePermit'],                                                        12))
        self.addColumn(CRefBookCol(u'Инвалидность',                                  ['disability_id'],        'rbTempInvalidRegime',                          15))
        self.setTable('TempInvalid')


    def updateColumnsCaches(self, id):
        self._cols[CExpertTempInvalidTableModel.Col_CountVisits].updateRecordsCache(id)


    def clearColumnsCaches(self):
        self._cols[CExpertTempInvalidTableModel.Col_CountVisits].invalidateRecordsCache()


class CExpertTempInvalidPeriodsTableModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)
        self.addColumn(CDateCol(   u'Дата открытия',    ['begDate'], 10))
        self.addColumn(CDateCol(   u'Дата закрытия',    ['endDate'], 10))
        self.addColumn(CRefBookCol(u'Закрыл',           ['endPerson_id'], 'vrbPerson', 6))
        self.addColumn(CRefBookCol(u'Председатель ВК',  ['chairPerson_id'],'vrbPerson', 6))
        self.addColumn(CBoolCol(   u'Внешний',          ['isExternal'], 10))
        self.addColumn(CRefBookCol(u'Режим',            ['regime_id'], 'rbTempInvalidRegime', 15))
        self.addColumn(CRefBookCol(u'Результат периода',['result_id'], 'rbTempInvalidResult', 15))
        self.addColumn(CTextCol(   u'Примечания',       ['note'], 6))
        self.setTable('TempInvalid_Period')


class CExpertTempInvalidDuplicatesTableModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)
        self.addColumn(CBoolCol(u'СтСт',      ['insuranceOfficeMark'],                6))
        self.addColumn(CTextCol(u'Серия',     ['serial'],                             6))
        self.addColumn(CTextCol(u'Номер',     ['number'],                             6))
        self.addColumn(CDateCol(u'Дата',      ['date'],                              12))
        self.addColumn(CRefBookCol(u'Выдал',  ['person_id'], 'vrbPerson',             6))
        self.setTable('TempInvalidDuplicate')


class CExpertTempInvalidDocumentsTableModel(CTableModel):
    class CLocClientCol(CDesignationCol):
        def __init__(self, title, fields, designationChain, defaultWidth, alignment='l'):
            CDesignationCol.__init__(self, title, fields, designationChain, defaultWidth, alignment)

        def format(self, values):
            val = values[0]
            for cache in self._caches:
                recordId  = val.toInt()[0]
                if recordId:
                    record = cache.get(recordId)
                    if record:
                        birthDate = forceString(record.value('birthDate'))
                        sex = formatSex(record.value('sex'))
                        val = toVariant(formatName(forceString(record.value('lastName')),
                               forceString(record.value('firstName')),
                               forceString(record.value('patrName'))) + u', ' + birthDate + u', ' + sex)
                    else:
                        return QVariant()
                else:
                    return QVariant()
            return val

    class CLocParentClientCol(CDesignationCol):
        def __init__(self, title, fields, designationChain, defaultWidth, alignment='l'):
            CDesignationCol.__init__(self, title, fields, designationChain, defaultWidth, alignment)

        def format(self, values):
            val = values[0]
            tempInvalidId = val.toInt()[0]
            if tempInvalidId:
                tempInvalidRecord = self._caches[0].get(tempInvalidId)
                clientId = forceRef(tempInvalidRecord.value('client_id')) if tempInvalidRecord else None
                if clientId:
                    record = self._caches[1].get(clientId)
                    if record:
                        birthDate = forceString(record.value('birthDate'))
                        sex = formatSex(record.value('sex'))
                        return toVariant(formatName(forceString(record.value('lastName')),
                               forceString(record.value('firstName')),
                               forceString(record.value('patrName'))) + u', ' + birthDate + u', ' + sex)
            return QVariant()

    class CLocDocumentCol(CDesignationCol):
        def __init__(self, title, fields, designationChain, defaultWidth, alignment='l'):
            CDesignationCol.__init__(self, title, fields, designationChain, defaultWidth, alignment)

        def format(self, values):
            val = values[0]
            for cache in self._caches:
                recordId  = val.toInt()[0]
                if recordId:
                    record = cache.get(recordId)
                    if record:
                        issueDate = forceString(record.value('issueDate'))
                        val = toVariant(forceString(record.value('serial')) + u'-' + forceString(record.value('number')) + u', ' + forceString(record.value('placeWork')) + u', ' + issueDate)
                    else:
                        return QVariant()
                else:
                    return QVariant()
            return val

    def __init__(self, parent):
        CTableModel.__init__(self, parent)
        self.addColumn(CExpertTempInvalidDocumentsTableModel.CLocParentClientCol(u'ФИО', ['master_id'], [('TempInvalid', 'id, client_id'), ('Client', 'id, birthDate, lastName, firstName, patrName, birthDate, sex')], 30))
        self.addColumn(CDateCol(       u'Дата выдачи',                         ['issueDate'],                                                                          12))
        self.addColumn(CTextCol(       u'Серия',                               ['serial'],                                                                              6))
        self.addColumn(CTextCol(       u'Номер',                               ['number'],                                                                              6))
        self.addColumn(CBoolCol(       u'Д',                                   ['duplicate'],                                                                           3).setToolTip(u'Дубликат'))
        self.addColumn(CRefBookCol(    u'Причина выдачи дубликата',            ['duplicateReason_id'], 'rbTempInvalidDuplicateReason',                                 10))
        self.addColumn(CBoolCol(       u'Э',                                   ['electronic'],                                                                          3).setToolTip(u'Электронный'))
        self.addColumn(CEnumCol(       u'Занятость',                           ['busyness'],        [u'', u'основное', u'совместитель', u'на учете'],                  10))
        self.addColumn(CTextCol(       u'Место работы',                        ['placeWork'],                                                                          20))
        self.addColumn(CTextCol(       u'ПДН',                                 ['prevNumber'],                                                                         12).setToolTip(u'Продолжение документа с номером'))
        self.addColumn(CExpertTempInvalidDocumentsTableModel.CLocDocumentCol(u'ИПД',   ['prev_id'], ('TempInvalidDocument', 'serial, number, placeWork, issueDate'),   20).setToolTip(u'Идентификатор продолжения документа'))
        self.addColumn(CRefBookCol(    u'Выдал',                               ['person_id'],       'vrbPersonWithSpeciality',                                         20))
        self.addColumn(CRefBookCol(    u'Закрыл',                              ['execPerson_id'],   'vrbPersonWithSpeciality',                                         20))
        self.addColumn(CRefBookCol(    u'Председатель ВК',                     ['chairPerson_id'],  'vrbPersonWithSpeciality',                                         20))
        self.addColumn(CExpertTempInvalidDocumentsTableModel.CLocClientCol(u'Пациент 1', ['clientPrimum_id'], ('Client', 'id, birthDate, lastName, firstName, patrName, birthDate, sex'), 30))
        self.addColumn(CExpertTempInvalidDocumentsTableModel.CLocClientCol(u'Пациент 2', ['clientSecond_id'], ('Client', 'id, birthDate, lastName, firstName, patrName, birthDate, sex'), 30))
        self.addColumn(CExpertTempInvalidDocumentsTableModel.CLocDocumentCol(u'ИВД',     ['last_id'], ('TempInvalidDocument', 'serial, number, placeWork, issueDate'), 20).setToolTip(u'Идентификатор выданного документа'))
        self.addColumn(CTextCol(       u'Примечание',                          ['note'],                                                                               20))
        self.setTable('TempInvalidDocument')


    def getCurrentTempInvalidId(self, tempInvalidDocumentId):
        if tempInvalidDocumentId:
            db = QtGui.qApp.db
            table = db.table('TempInvalidDocument')
            record = db.getRecordEx(table, [table['master_id']], [table['id'].eq(tempInvalidDocumentId), table['deleted'].eq(0)], table['issueDate'].name())
            return forceRef(record.value('master_id')) if record else None
        return None


class CVisitsTableModel(CTableModel):
    class CLocClientColumn(CEventsTableModel.CLocClientColumn):
        def __init__(self, title, fields, defaultWidth, clientCache, eventCache):
            CEventsTableModel.CLocClientColumn.__init__(self, title, fields, defaultWidth, clientCache)
            self.eventCache = eventCache


        def format(self, values):
            val = values[0]
            eventId  = forceRef(val)
            eventRecord = self.eventCache.get(eventId)
            if eventRecord:
                clientValues = [eventRecord.value('client_id')]
                clientVal = clientValues[0]
                clientId  = forceRef(clientVal)
                clientRecord = self.clientCache.get(clientId)
                if clientRecord:
                    name  = formatShortNameInt(forceString(clientRecord.value('lastName')),
                                       forceString(clientRecord.value('firstName')),
                                       forceString(clientRecord.value('patrName')))
                    return toVariant(name)
                return CCol.invalid
            return CCol.invalid


    class CLocClientIdentifierColumn(CCol):
        def __init__(self, title, fields, defaultWidth, eventCache):
            CCol.__init__(self, title, fields, defaultWidth, 'l')
            self.eventCache = eventCache
            self.identifiersCache = CRecordCache()


        def getClientIdentifier(self, clientId):
            identifier = self.identifiersCache.get(clientId)
            if identifier is None:
                accountingSystemId = QtGui.qApp.defaultAccountingSystemId()
                if accountingSystemId is None:
                    identifier = clientId
                else:
                    db = QtGui.qApp.db
                    tableClientIdentification = db.table('ClientIdentification')
                    cond = [tableClientIdentification['client_id'].eq(clientId),
                            tableClientIdentification['accountingSystem_id'].eq(accountingSystemId)]
                    record = db.getRecordEx(tableClientIdentification, tableClientIdentification['identifier'], cond)
                    identifier = forceString(record.value(0)) if record else ''
                self.identifiersCache.put(clientId, identifier)
            return identifier


        def format(self, values):
            val = values[0]
            eventId = forceRef(val)
            eventRecord = self.eventCache.get(eventId)
            if eventRecord:
                clientId = forceRef(eventRecord.value('client_id'))
                return toVariant(self.getClientIdentifier(clientId))
            return CCol.invalid


    class CLocClientBirthDateColumn(CEventsTableModel.CLocClientBirthDateColumn):
        def __init__(self, title, fields, defaultWidth, clientCache, eventCache):
            CEventsTableModel.CLocClientBirthDateColumn.__init__(self, title, fields, defaultWidth, clientCache)
            self.eventCache = eventCache

        def format(self, values):
            val = values[0]
            eventId  = forceRef(val)
            eventRecord = self.eventCache.get(eventId)
            if eventRecord:
                return CEventsTableModel.CLocClientBirthDateColumn.format(self, [eventRecord.value('client_id')])
            return CCol.invalid


    class CLocClientSexColumn(CEventsTableModel.CLocClientSexColumn):
        def __init__(self, title, fields, defaultWidth, clientCache, eventCache):
            CEventsTableModel.CLocClientSexColumn.__init__(self, title, fields, defaultWidth, clientCache)
            self.eventCache = eventCache

        def format(self, values):
            val = values[0]
            eventId  = forceRef(val)
            eventRecord = self.eventCache.get(eventId)
            if eventRecord:
                return CEventsTableModel.CLocClientSexColumn.format(self, [eventRecord.value('client_id')])
            return CCol.invalid


    def __init__(self, parent, clientCache, eventCache):
        CTableModel.__init__(self, parent)
        clientCol   = CVisitsTableModel.CLocClientColumn( u'Ф.И.О.', ['event_id'], 60, clientCache, eventCache)
#        clientIdentifierCol = CVisitsTableModel.CLocClientIdentifierColumn(u'Идентификатор', ['event_id'], 30, eventCache)
        clientBirthDateCol = CVisitsTableModel.CLocClientBirthDateColumn(u'Дата рожд.', ['event_id'], 20, clientCache, eventCache)
        clientSexCol = CVisitsTableModel.CLocClientSexColumn(u'Пол', ['event_id'], 5, clientCache, eventCache)
        self.addColumn(clientCol)
        self.addColumn(clientBirthDateCol)
        self.addColumn(clientSexCol)
        self.addColumn(CDateCol(u'Дата',                 ['date'],                                    15))
        self.addColumn(CRefBookCol(u'Тип',               ['visitType_id'], 'rbVisitType',             15))
        self.addColumn(CRefBookCol(u'Исполнитель',       ['person_id'],    'vrbPersonWithSpecialityAndOrgStr', 20))
        self.addColumn(CRefBookCol(u'Место',             ['scene_id'],     'rbScene',                 15))
        self.addColumn(CBoolCol(u'Признак первичности',  ['isPrimary'],                               15))
        self.addColumn(CRefBookCol(u'Услуга',            ['service_id'],   'rbService',               15))
        self.addColumn(CBoolCol(u'Флаги финансирования', ['payStatus'],                               15))
        self.addColumn(CRefBookCol(u'Ассистент',         ['assistant_id'], 'vrbPersonWithSpecialityAndOrgStr', 20))
        self.addColumn(CDesignationCol(u'Номер документа', ['event_id'],   ('Event', 'externalId'), 20))
        self.addColumn(CRefBookCol(u'Тип финансирования', ['finance_id'],   'rbFinance',               10))

        self.loadField('id')
        self.loadField('event_id')
        self.setTable('Visit')


    def headerData(self, section, orientation, role=Qt.DisplayRole):
        return CTableModel.headerData(self, section, orientation, role)


