# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import math
import sip
from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QAbstractTableModel, QDate, QDateTime, QModelIndex, QObject, QSize, QTimer, QVariant, pyqtSignature, SIGNAL

from library.crbcombobox                  import CRBModelDataCache, CRBComboBox
from library.DialogBase                   import CConstructHelperMixin
from library.DockWidget                   import CDockWidget
from library.PreferencesMixin             import CContainerPreferencesMixin, CPreferencesMixin
from library.PrintInfo                    import CDateInfo, CInfo, CInfoContext, CTimeInfo, CDateTimeInfo
from library.PrintTemplates               import getPrintAction, applyTemplate
from library.RecordLock                   import CRecordLockMixin
from library.TreeModel                    import CTreeItemWithId, CTreeModel
from library.Utils                        import calcAgeTuple, forceBool, forceDate, forceDouble, forceInt, forceRef, forceString, formatTime, getPref, setPref, sorry, toVariant

from Events.CreateEvent                   import requestNewEvent
from Events.Utils                         import getDeathDate
from Notifications.NotifyDialog import CNotifyDialog
from Notifications.NotificationRule import CNotificationRule
from Orgs.OrgPersonnel                    import COrgPersonnelModel, CFlatOrgPersonnelModel
from Orgs.OrgStructComboBoxes             import COrgStructureModel
from Orgs.PersonInfo                      import CPersonInfo
from Orgs.Utils                           import findOrgStructuresByAddress, getPersonInfo
from Registry.BeforeRecordClient          import CQueue, printOrderByScheduleItem
from Registry.ComplaintsEditDialog        import CComplaintsEditDialog
from Registry.ReferralEditDialog          import inputReferral, CReferral, CReferralEditDialog
from Registry.RegistrySuspenedAppointment import setRegistrySuspenedAppointment
from Registry.ShowScheduleItemInfo        import showScheduleItemInfo
from Registry.Utils                       import CAppointmentPurposeCache, CCheckNetMixin, CClientInfo, getClientAddressEx, getClientInfoEx, getClientMiniInfo, getClientInfo2
from Reports.ReportBase                   import CReportBase, createTable
from Reports.ReportBeforeRecord           import CReportBeforeRecord
from Reports.ReportView                   import CReportViewDialog
from RefBooks.RBAppointmentPurpose import CAppointmentPurposeInfo
from Timeline.Schedule                    import CSchedule, CScheduleItem, confirmAndFreeScheduleItem, getScheduleItemIdListForClient
from Timeline.TimeTable                   import formatTimeRange
from Users.Rights                         import urAdmin, urQueueCancelMaxDateLimit, urQueueCancelMinDateLimit, urQueueOverTime, urQueueToSelfOverTime, urRegTabWriteEvents, urSendInternalAmbNotifications


from Registry.Ui_ResourcesDockContent     import Ui_Form

# примечание 1:
#   при уменьшении количества строк в таблице при некоторых случаях
#   verticalHeader().updatesEnabled() остаётся False
#   эта ошибка наблюдается в Alt linux 6, причём в PyQt она есть, а в Qt - нет.
#   когда эта ошибка будет исправлена можно будет удалить строки с этой пометкой

class CResourcesDockWidget(CDockWidget):
    def __init__(self, parent):
        CDockWidget.__init__(self, parent)
        self.setWindowTitle(u'График')
        self.setFeatures(QtGui.QDockWidget.AllDockWidgetFeatures)
        self.setAllowedAreas(Qt.LeftDockWidgetArea|Qt.RightDockWidgetArea)
        self.content = None
        self.contentPreferences = {}
        self.connect(QtGui.qApp, SIGNAL('dbConnectionChanged(bool)'), self.onConnectionChanged)


    def loadPreferences(self, preferences):
        self.contentPreferences = getPref(preferences, 'content', {})
        CDockWidget.loadPreferences(self, preferences)
        if isinstance(self.content, CPreferencesMixin):
            self.content.loadPreferences(self.contentPreferences)


    def savePreferences(self):
        result = CDockWidget.savePreferences(self)
        self.updateContentPreferences()
        setPref(result,'content',self.contentPreferences)
        return result


    def updateContentPreferences(self):
        if isinstance(self.content, CPreferencesMixin):
            self.contentPreferences = self.content.savePreferences()


    def onConnectionChanged(self, value):
        if value:
            self.onDBConnected()
        else:
            self.onDBDisconnected()


    def onDBConnected(self):
        self.setWidget(None)
        if self.content:
            self.content.setParent(None)
            sip.delete(self.content)
        self.content = CResourcesDockContent(self)
        self.content.loadPreferences(self.contentPreferences)
        self.setWidget(self.content)


    def onDBDisconnected(self):
        self.setWidget(None)
        if self.content:
            self.updateContentPreferences()
            self.content.setParent(None)
            sip.delete(self.content)
        self.content = QtGui.QLabel(u'необходимо\nподключение\nк базе данных', self)
        self.content.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
        self.content.setDisabled(True)
        self.setWidget(self.content)


class CResourcesDockContent(QtGui.QWidget, Ui_Form, CConstructHelperMixin, CContainerPreferencesMixin, CCheckNetMixin, CRecordLockMixin):
    def __init__(self, parent):
        QtGui.QWidget.__init__(self, parent)
        CCheckNetMixin.__init__(self)
        CRecordLockMixin.__init__(self)

        self.groupingSpeciality = forceBool(QtGui.qApp.preferences.appPrefs.get('groupingSpeciality', True))
        self.activityListIsShown = forceBool(QtGui.qApp.preferences.appPrefs.get('activityListIsShown', False))

        self.addModels('OrgStructure',  COrgStructureModel(self, QtGui.qApp.currentOrgId()))
        self.addModels('Activity',      CActivityModel(self))
        self.addModels('AmbTimeTable',  CTimeTableModel(self, CSchedule.atAmbulance))
        self.addModels('AmbQueue',      CQueueModel(self, CSchedule.atAmbulance))
        self.addModels('HomeTimeTable', CTimeTableModel(self, CSchedule.atHome))
        self.addModels('HomeQueue',     CQueueModel(self, CSchedule.atHome))

        self.addObject('actFindArea',       QtGui.QAction(u'Найти участок', self))
        self.addObject('actAmbCreateOrder', QtGui.QAction(u'Поставить в очередь', self))
        self.addObject('actAmbDeleteOrder', QtGui.QAction(u'Удалить из очереди', self))
        self.addObject('actAmbChangeComplaint', QtGui.QAction(u'Изменить жалобы', self))
        self.addObject('actAmbChangeReferral',  QtGui.QAction(u'Изменить данные направления', self))
        self.addObject('actAmbPrintOrder',  QtGui.QAction(u'Напечатать направление', self))
        self.addObject('actAmbPrintOrderByTemplate', getPrintAction(self, 'ambOrder', u'Напечатать направление по шаблону', False))
        self.addObject('actAmbPrintQueue',  QtGui.QAction(u'Напечатать полный список', self))
        self.addObject('actAmbPrintQueuePass',  QtGui.QAction(u'Напечатать список подтвержденных записей', self))
        self.addObject('actAmbPrintQueueNoPass',  QtGui.QAction(u'Напечатать список неподтвержденных записей', self))
        self.addObject('actAmbPrintPreRecordQueue',  QtGui.QAction(u'Напечатать список предварительной записи', self))
        self.addObject('actAmbPrintQueueByTemplate', getPrintAction(self, 'ambQueue', u'Напечатать список по шаблону', False))
        self.addObject('actAmbFindClient',  QtGui.QAction(u'Перейти в картотеку', self))
        self.addObject('actAmbFindClientEvents',  QtGui.QAction(u'Перейти на вкладку "Обращение"', self))
        self.addObject('actAmbCreateEvent', QtGui.QAction(u'Новое обращение', self))
        self.addObject('actHomeCreateOrder',QtGui.QAction(u'Поставить в очередь', self))
        self.addObject('actHomeDeleteOrder',QtGui.QAction(u'Удалить из очереди', self))
        self.addObject('actHomeChangeComplaint',QtGui.QAction(u'Изменить жалобы', self))
        self.addObject('actHomeChangeReferral', QtGui.QAction(u'Изменить данные направления', self))
        self.addObject('actHomePrintOrder', QtGui.QAction(u'Напечатать направление', self))
        self.addObject('actHomePrintOrderByTemplate', getPrintAction(self, 'homeOrder', u'Напечатать направление по шаблону', False))
        self.addObject('actHomePrintQueue', QtGui.QAction(u'Напечатать полный список', self))
        self.addObject('actHomePrintQueuePass', QtGui.QAction(u'Напечатать список переданных врачу', self))
        self.addObject('actHomePrintQueueNoPass', QtGui.QAction(u'Напечатать список непереданных врачу', self))
        self.addObject('actHomePrintQueueByTemplate', getPrintAction(self, 'homeQueue', u'Напечатать список по шаблону', False))
        self.addObject('actHomeFindClient', QtGui.QAction(u'Перейти в картотеку', self))
        self.addObject('actHomeFindClientEvents', QtGui.QAction(u'Перейти на вкладку "Обращение"', self))
        self.addObject('actHomeCreateEvent', QtGui.QAction(u'Новое обращение', self))
        self.addObject('actServiceReport', QtGui.QAction(u'Сводка об обслуживании', self))
        self.addObject('actAmbInfo', QtGui.QAction(u'Свойства записи', self))
        self.addObject('actHomeInfo', QtGui.QAction(u'Свойства записи', self))
        self.addObject('actRegistrySuspenedAppointment', QtGui.QAction(u'Зарегистрировать пациента в Журнале отложной записи', self))
        self.addObject('actAmbNotification', QtGui.QAction(u'Сообщение по списку', self))

        self.ambTimeTableStoredPos = None
        self.homeTimeTableStoredPos = None
        self.enableQueueing = False

        self.timer = QTimer(self)
        self.timer.setObjectName('timer')
        self.timer.setInterval(60*1000) # раз в минуту
        self.activityPref = QtGui.qApp.getGlobalPreference('23')

        self.setupUi(self)
        for treeWidget in (self.treeOrgStructure, self.treeOrgPersonnel):
            treeWidget.setIndentation(12)
            treeWidget.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        for tableWidget in (self.tblAmbTimeTable, self.tblAmbQueue, self.tblHomeTimeTable, self.tblHomeQueue):
            verticalHeader = tableWidget.verticalHeader()
            verticalHeader.show()
            verticalHeader.setResizeMode(QtGui.QHeaderView.Fixed)
        self.setModels(self.treeOrgStructure, self.modelOrgStructure, self.selectionModelOrgStructure)
        self.setModels(self.tblAmbTimeTable,  self.modelAmbTimeTable, self.selectionModelAmbTimeTable)
        self.setModels(self.tblAmbQueue,      self.modelAmbQueue,     self.selectionModelAmbQueue)
        self.setModels(self.tblHomeTimeTable, self.modelHomeTimeTable,self.selectionModelHomeTimeTable)
        self.setModels(self.tblHomeQueue,     self.modelHomeQueue,    self.selectionModelHomeQueue)

        self.onTreePersonnelSectionClicked(0, False)

        self.tblHomeTimeTable.setColumnHidden(1,True) # кабинет в вызовах на дом

        self.treeOrgStructure.createPopupMenu([self.actFindArea])
        self.tblAmbQueue.createPopupMenu([self.actAmbCreateOrder,
                                          self.actAmbDeleteOrder,
                                          '-',
                                          self.actAmbChangeComplaint,
                                          self.actAmbChangeReferral,
                                          '-',
                                          self.actAmbPrintOrder,
                                          self.actAmbPrintOrderByTemplate,
                                          '-',
                                          self.actAmbPrintQueue,
                                          self.actAmbPrintQueuePass,
                                          self.actAmbPrintQueueNoPass,
                                          self.actAmbPrintPreRecordQueue,
                                          self.actAmbPrintQueueByTemplate,
                                          '-',
                                          self.actAmbFindClient,
                                          self.actAmbFindClientEvents, 
                                          self.actAmbCreateEvent,
                                          '-',
                                          self.actServiceReport,
                                          self.actAmbInfo,
                                          #self.actRegistrySuspenedAppointment,
                                          '-',
                                          self.actAmbNotification])
        self.tblHomeQueue.createPopupMenu([self.actHomeCreateOrder,
                                          self.actHomeDeleteOrder,
                                          '-',
                                          self.actHomeChangeComplaint,
                                          self.actHomeChangeReferral,
                                          '-',
                                          self.actHomePrintOrder,
                                          self.actHomePrintOrderByTemplate, 
                                          '-',
                                          self.actHomePrintQueue,
                                          self.actHomePrintQueuePass,
                                          self.actHomePrintQueueNoPass,
                                          self.actHomePrintQueueByTemplate,
                                          '-',
                                          self.actHomeFindClient,
                                          self.actHomeFindClientEvents,
                                          self.actHomeCreateEvent,
                                          '-',
                                          self.actServiceReport,
                                          self.actHomeInfo,
                                          #self.actRegistrySuspenedAppointment
                                          ]
                                          )

        self.onTreeOSSectionClicked(0, False)
        self.onCurrentUserIdChanged()

        self.connect(self.treeOrgStructure.popupMenu(), SIGNAL('aboutToShow()'), self.popupMenuTreeOrgStructureAboutToShow)
        self.connect(self.tblAmbQueue.popupMenu(), SIGNAL('aboutToShow()'), self.popupMenuAmbAboutToShow)
        self.connect(self.tblHomeQueue.popupMenu(), SIGNAL('aboutToShow()'), self.popupMenuHomeAboutToShow)

        self.headerTreeOS = self.treeOrgStructure.header()
        self.headerTreeOS.setClickable(True)
        QObject.connect(self.headerTreeOS, SIGNAL('sectionClicked(int)'), self.onTreeOSSectionClicked)

        self.treeOrgPersonnel.createPopupMenu([self.actRegistrySuspenedAppointment])
        self.connect(self.treeOrgPersonnel.popupMenu(), SIGNAL('aboutToShow()'), self.popupMenuOrgPersonnelAboutToShow)
        self.headerTreePersonnel = self.treeOrgPersonnel.header()
        self.headerTreePersonnel.setClickable(True)
        QObject.connect(self.headerTreePersonnel, SIGNAL('sectionClicked(int)'),
                               self.onTreePersonnelSectionClicked)

        self.connect(QtGui.qApp, SIGNAL('currentOrgIdChanged()'), self.onCurrentOrgIdChanged)
        self.connect(QtGui.qApp, SIGNAL('currentUserIdChanged()'), self.onCurrentUserIdChanged)

        self.prevClientId = None
        self.prevComplaint = ''

        self.timer.start()
        self.updateTimeTable()


    def onTreePersonnelSectionClicked(self, col, reverse=True):
        if reverse:
            self.groupingSpeciality = not self.groupingSpeciality

        if hasattr(self, 'selectionModelPersonnel'):
            self.disconnect(self.selectionModelPersonnel,
                            SIGNAL('currentChanged(QModelIndex, QModelIndex)'),
                            self.on_selectionModelPersonnel_currentChanged)
            del self.selectionModelPersonnel

        self.treeOrgPersonnel.setModel(None)
        if self.groupingSpeciality:
            self.addModels('Personnel', CResourcesPersonnelModel([], self))
        else:
            self.addModels('Personnel', CFlatResourcesPersonnelModel([], self))

        self.setModels(self.treeOrgPersonnel, self.modelPersonnel, self.selectionModelPersonnel)
        self.connect(self.selectionModelPersonnel,
                     SIGNAL('currentChanged(QModelIndex, QModelIndex)'),
                     self.on_selectionModelPersonnel_currentChanged)


        QtGui.qApp.preferences.appPrefs['groupingSpeciality'] = self.groupingSpeciality

        if self.activityListIsShown:
            self.updatePersonnelByActivity()
        else:
            self.updatePersonnelByOrgStructure()


    def onTreeOSSectionClicked(self, col, reverse=True):
        if reverse:
            self.activityListIsShown = not self.activityListIsShown

        if not self.activityListIsShown:
            self.treeOrgStructure.setModel(None)
            self.setModels(self.treeOrgStructure, self.modelOrgStructure, self.selectionModelOrgStructure)
            orgStructureIndex = self.modelOrgStructure.findItemId(QtGui.qApp.currentOrgStructureId())
            if orgStructureIndex and orgStructureIndex.isValid():
                self.treeOrgStructure.setCurrentIndex(orgStructureIndex)
                self.treeOrgStructure.setExpanded(orgStructureIndex, True)
            self.updatePersonnelByOrgStructure()
        else:
             self.treeOrgStructure.setModel(None)
             self.setModels(self.treeOrgStructure, self.modelActivity, self.selectionModelActivity)
             self.updatePersonnelByActivity()
        QtGui.qApp.preferences.appPrefs['activityListIsShown'] = self.activityListIsShown


    def sizeHint(self):
        return QSize(10, 10)


    def onCurrentOrgIdChanged(self):
        self.modelOrgStructure.setOrgId(QtGui.qApp.currentOrgId())
        self.updatePersonnelByOrgStructure()
        self.updateTimeTable()
        self.actAmbNotification.setEnabled(QtGui.qApp.userHasAnyRight([
                        urAdmin, urSendInternalAmbNotifications]))



    def onCurrentUserIdChanged(self):
        if QtGui.qApp.userOrgStructureId:
            index = self.modelOrgStructure.findItemId(QtGui.qApp.userOrgStructureId)
            if index and index.isValid():
                self.treeOrgStructure.setCurrentIndex(index)
        if QtGui.qApp.userSpecialityId:
            index = self.modelPersonnel.findPersonId(QtGui.qApp.userId)
            if index and index.isValid():
                self.treeOrgPersonnel.setCurrentIndex(index)
        self.setCaliendarDataRange()


    def setCaliendarDataRange(self):
        today = QDate.currentDate()
        if QtGui.qApp.userHasRight(urQueueCancelMinDateLimit):
            minDate = today.addYears(-10)
        else:
            minDate = today.addDays(-7)
        queueCancelMaxDateLimit = QtGui.qApp.userHasRight(urQueueCancelMaxDateLimit)
        if queueCancelMaxDateLimit:
            maxDate = today.addYears(10)
        else:
            maxDate = today.addMonths(2)
# это скверная мысль "рулить" датами в зависимости от врача -
# в зависимости от свойств врача это может сбивать дату в календаре.
# но к сожалению требуют - задача №0007142: Право пользователей queueCancelMaxDateLimit
#        personId = self.getCurrentPersonId()
#        if personId and QtGui.qApp.isTimelineAccessibilityDays() == 1:
#            db = QtGui.qApp.db
#            table = db.table('Person')
#            record = db.getRecordEx(table, [table['lastAccessibleTimelineDate'], table['timelineAccessibleDays']], [table['deleted'].eq(0), table['id'].eq(personId)])
#            if record:
#                lastAccessibleTimelineDate = forceDate(record.value('lastAccessibleTimelineDate'))
#                timelineAccessibleDays = forceInt(record.value('timelineAccessibleDays'))
#                if lastAccessibleTimelineDate:
#                    maxDate = lastAccessibleTimelineDate
#                if timelineAccessibleDays and not queueCancelMaxDateLimit:
#                    accessibleDays = today.addDays(timelineAccessibleDays-1)
#                    if lastAccessibleTimelineDate:
#                        maxDate = min(maxDate, accessibleDays)
#                    else:
#                        maxDate = accessibleDays
        self.calendarWidget.setDateRange(minDate, maxDate)


    def getCurrentPersonId(self):
        treeIndex = self.treeOrgPersonnel.currentIndex()
        personIdList = self.modelPersonnel.getItemIdList(treeIndex)
        if personIdList and len(personIdList) == 1:
            return personIdList[0]
        else:
            return None


    def getCurrentSpecialityId(self):
        treeIndex = self.treeOrgPersonnel.currentIndex()
        personIdList = self.modelPersonnel.getItemIdList(treeIndex)
        if personIdList:
            personId = personIdList[0]
            if personId:
                return self.getPersonSpecialityId(personId)
        return None


    def getCurrentDate(self):
        return self.calendarWidget.selectedDate()


    def getCurrentPersonIdList(self):
        treeIndex = self.treeOrgPersonnel.currentIndex()
        return self.modelPersonnel.getItemIdList(treeIndex)


    def getCurrentOrgSrtuctureId(self):
        treeIndex = self.treeOrgStructure.currentIndex()
        return self.modelOrgStructure.itemId(treeIndex)


    def checkApplicable(self, clientId, date):
        personId = self.getCurrentPersonId()
        return (     self.checkClientAttach(personId, clientId, date, True)
                 and (    self.checkClientAttendance(personId, clientId)
                       or self.confirmClientAttendance(self, personId, clientId)
                     )
                 and self.confirmClientPolicyConstraint(self, clientId)
               )


    def updatePersonnelByOrgStructure(self):
        personId = self.getCurrentPersonId()
        if self.groupingSpeciality and not personId:
            specialityId = self.getCurrentSpecialityId()
        treeIndex = self.treeOrgStructure.currentIndex()
        orgStructureIdList = self.modelOrgStructure.getItemIdList(treeIndex)
        date = self.calendarWidget.selectedDate()
        self.modelPersonnel.setOrgStructureIdList(self.modelOrgStructure.orgId, orgStructureIdList, date)
        if self.modelOrgStructure.isLeaf(treeIndex):
            self.treeOrgPersonnel.expandAll()
            self.treeOrgPersonnel.setCurrentIndex(self.modelPersonnel.getFirstLeafIndex())
        index = self.modelPersonnel.findPersonId(personId)
        if index and index.isValid():
            self.treeOrgPersonnel.setCurrentIndex(index)
        if self.groupingSpeciality and not personId:
            index = self.modelPersonnel.findSpecialityId(specialityId)
            if index and index.isValid():
                self.treeOrgPersonnel.setCurrentIndex(index)
        self.updateTimeTable()


    def updatePersonnelByActivity(self):
        personId = self.getCurrentPersonId()
        if self.groupingSpeciality and not personId:
            specialityId = self.getCurrentSpecialityId()
        treeIndex = self.treeOrgStructure.currentIndex()
        activityIdList = self.modelActivity.getItemIdList(treeIndex)
        date = self.calendarWidget.selectedDate()
        self.modelPersonnel.setActivityIdList(self.modelOrgStructure.orgId, activityIdList, date)
        if self.modelActivity.isLeaf(treeIndex):
            self.treeOrgPersonnel.expandAll()
            self.treeOrgPersonnel.setCurrentIndex(self.modelPersonnel.getFirstLeafIndex())
        index = self.modelPersonnel.findPersonId(personId)
        if index and index.isValid():
            self.treeOrgPersonnel.setCurrentIndex(index)
        if self.groupingSpeciality and not personId:
            index = self.modelPersonnel.findSpecialityId(specialityId)
            if index and index.isValid():
                self.treeOrgPersonnel.setCurrentIndex(index)
        self.updateTimeTable()


    def getActivityId(self):
        if self.activityListIsShown and self.activityPref == u'да':
            treeIndex = self.treeOrgStructure.currentIndex()
            activityIdList = self.modelActivity.getItemIdList(treeIndex)
            return activityIdList[0] if activityIdList else None
        else:
            return None


    def updateTimeTableEx(self, tblTimeTable, modelTimeTable, tblQueue, modelQueue):
        personId = self.getCurrentPersonId()
        date = self.calendarWidget.selectedDate()
        minDate = self.calendarWidget.minimumDate()
        maxDate = self.calendarWidget.maximumDate()

        row = tblTimeTable.currentIndex().row()
        activityId = self.getActivityId()
        modelTimeTable.setPersonAndDate(personId, date, minDate, maxDate, activityId)
        row = modelTimeTable.getRowForDate(date, row)
        tblTimeTable.setCurrentIndex(modelTimeTable.index(row, 0))
        self.updateQueueEx(tblTimeTable, modelTimeTable, tblQueue, modelQueue)


    def updateQueueEx(self, tblTimeTable, modelTimeTable, tblQueue, modelQueue):
        row = tblTimeTable.currentIndex().row()
        personId = self.getCurrentPersonId()
        schedule = modelTimeTable.getSchedule(row) if self.isRightMaxDateLimit(personId) else None
        modelQueue.setSchedule(schedule)
        if personId and schedule is not None and schedule.reasonOfAbsenceId is None:
            appointmentPurposeId = schedule.appointmentPurposeId
            self.enableQueueing = isAppointmentEnabled(appointmentPurposeId, personId) # тип записи разрешён
        else:
            self.enableQueueing = False
        tblQueue.setEnabled(modelQueue.rowCount()>0)


    def isRightMaxDateLimit(self, personId):
        if personId and QtGui.qApp.isTimelineAccessibilityDays() == 1:
            date = self.calendarWidget.selectedDate()
            today = QDate.currentDate()
            queueCancelMaxDateLimit = QtGui.qApp.userHasRight(urQueueCancelMaxDateLimit)
            if queueCancelMaxDateLimit:
                maxDate = today.addYears(10)
            else:
                maxDate = today.addDays(7)
            db = QtGui.qApp.db
            table = db.table('Person')
            record = db.getRecordEx(table, [table['lastAccessibleTimelineDate'], table['timelineAccessibleDays']], [table['deleted'].eq(0), table['id'].eq(personId)])
            if record:
                lastAccessibleTimelineDate = forceDate(record.value('lastAccessibleTimelineDate'))
                timelineAccessibleDays = forceInt(record.value('timelineAccessibleDays'))
                if lastAccessibleTimelineDate:
                    maxDate = lastAccessibleTimelineDate
                if timelineAccessibleDays and not queueCancelMaxDateLimit:
                    accessibleDays = today.addDays(timelineAccessibleDays-1)
                    if lastAccessibleTimelineDate:
                        maxDate = min(maxDate, accessibleDays)
                    else:
                        maxDate = accessibleDays
            return maxDate >= date
        return True


    def updateTimeTable(self):
        self.timer.stop()
        try:
            self.setCaliendarDataRange()

            place = self.tabPlace.currentIndex()
            if place == 0: # амбулаторно
                self.updateTimeTableEx(self.tblAmbTimeTable, self.modelAmbTimeTable, self.tblAmbQueue, self.modelAmbQueue)
            else: # place == 1: # на дому
                self.updateTimeTableEx(self.tblHomeTimeTable, self.modelHomeTimeTable, self.tblHomeQueue, self.modelHomeQueue)
        finally:
            self.timer.start()


    def showQueueItem2(self, scheduleItemId):
        if scheduleItemId:
            db = QtGui.qApp.db
            record = db.getRecord('vScheduleItem', 'date, person_id, activity_id',  scheduleItemId)
            if record:
                date = forceDate(record.value('date'))
                personId = forceRef(record.value('person_id'))
                if self.activityPref == u'да':
                    activityId = forceRef(record.value('activity_id'))
                    personRecord = db.getRecord('Person', 'orgStructure_id, speciality_id', personId)
                else:
                    tablePersonActivity = db.table('Person_Activity')
                    tablePerson = db.table('Person')
                    table = tablePerson.leftJoin(tablePersonActivity, tablePerson['id'].eq(tablePersonActivity['master_id']))
                    personRecord = db.getRecord(table, 'orgStructure_id, speciality_id, activity_id', personId)
                if personRecord:
                    orgStructureId = forceRef(personRecord.value('orgStructure_id'))
                    specialityId = forceRef(personRecord.value('speciality_id'))
                    if self.activityPref != u'да':
                        activityId = forceRef(personRecord.value('activity_id'))
                    self.showQueueItem(orgStructureId, specialityId, activityId, personId, date, scheduleItemId)


    def showQueueItem(self, orgStructureId, specialityId, activityId, personId, date, scheduleItemId):
        self.setPersonAttributes(orgStructureId, specialityId, activityId, personId)
        if date >= QDate.currentDate():
            self.calendarWidget.setSelectedDate(date)
        if scheduleItemId:
            self.selectScheduleItem(scheduleItemId)


    def setPersonAttributes(self, orgStructureId, specialityId, activityId, personId):
        index = None
        if self.activityListIsShown and activityId:
            index = self.modelActivity.findItemId(activityId)
        else:
            if orgStructureId:
                index = self.modelOrgStructure.findItemId(orgStructureId)
        if index and index.isValid():
            self.treeOrgStructure.setCurrentIndex(index)
        if specialityId and personId:
            index = self.modelPersonnel.findPersonId(personId)
            if index and index.isValid():
                self.treeOrgPersonnel.setCurrentIndex(index)


    def selectScheduleItem(self, scheduleItemId):
        db = QtGui.qApp.db
        record = db.getRecord('vScheduleItem', 'appointmentType, master_id, idx', scheduleItemId)
        if record:
            appointmentType = forceInt(record.value('appointmentType'))
            scheduleId = forceRef(record.value('master_id'))
            idx = forceInt(record.value('idx'))

            if appointmentType == CSchedule.atAmbulance:
                self.tabPlace.setCurrentIndex(0)
            elif appointmentType == CSchedule.atHome:
                self.tabPlace.setCurrentIndex(1)

            self.updateTimeTable()
            place = self.tabPlace.currentIndex()
            if place == 0:
                tblTimeTable, tblQueue = self.tblAmbTimeTable, self.tblAmbQueue
            else:
                tblTimeTable, tblQueue = self.tblHomeTimeTable, self.tblHomeQueue

            scheduleRow = tblTimeTable.model().findRowForScheduleId(scheduleId)
            if scheduleRow>=0:
                tblTimeTable.selectRow(scheduleRow)
                tblQueue.selectRow(idx)
                tblQueue.setFocus(Qt.OtherFocusReason)


    def queueingEnabled(self, date, orgStructureId, specialityId, activityId, personId, queueWidget, row, clientId):
        queueModel = queueWidget.model()
        if queueModel.getClientId(row):
            return False # кто-то уже записан
        deathDate = getDeathDate(clientId)
        if deathDate:
            QtGui.QMessageBox.warning(self, u'Внимание!', u'Этот пациент не может быть записан к врачу, потому что есть отметка что он уже умер %s' % forceString(deathDate))
            return False

        appointmentPurposeId = queueModel.schedule.appointmentPurposeId
        if not isAppointmentEnabledForClient(appointmentPurposeId, personId, date, clientId):
            QtGui.QMessageBox.warning(self, u'Внимание!', u'Назначение приёма препятствует записи пациента')
            return False

        scheduleItemIdList = getScheduleItemIdListForClient(clientId, specialityId, date)
        if scheduleItemIdList:
            if QtGui.qApp.isReStagingInQueue(): # Повторная постановка в очередь(да, нет)
                textMessage = u'Этот пациент уже записан к врачу этой специальности\nПодтвердите повторную запись'
            else:
                textMessage = u'Этот пациент уже записан к врачу этой специальности'
            messageBox = QtGui.QMessageBox(QtGui.QMessageBox.Warning, u'Внимание!', textMessage)
            if QtGui.qApp.isReStagingInQueue():
                messageBox.addButton(u'Ок', QtGui.QMessageBox.YesRole)
            messageBox.addButton(u'Отмена', QtGui.QMessageBox.NoRole)
            messageBox.setDefaultButton(messageBox.addButton(u'Просмотр', QtGui.QMessageBox.ActionRole))
            confirmation = messageBox.exec_()
            if not QtGui.qApp.isReStagingInQueue(): # Повторная постановка в очередь(да, нет)
                confirmation += 1
            if confirmation == 1:
                return False
            if confirmation == 2: # просмотр
                place = self.tabPlace.currentIndex()
                result = CQueue(self, clientId, scheduleItemIdList, QtGui.qApp.isReStagingInQueue()).exec_()
                if not result:
                    return False
                self.tabPlace.setCurrentIndex(place)
                self.showQueueItem(orgStructureId, specialityId, activityId, personId, date, None) # при некоторых условиях это не будет работать
                queueWidget.selectRow(row)

        if not ( queueModel.overtimeEnabled and queueModel.getTime(row).isNull() ):
            quota = getQuota(personId, queueModel.schedule.capacity, queueModel.schedule.appointmentPurposeId)
            queued = queueModel.getQueuedCount()
            if quota<=queued:
                message = u'Квота на запись исчерпана!'
                if QtGui.qApp.userHasRight('queueingOutreaching'):
                    message += u'\nВсё равно продолжить?'
                    buttons = QtGui.QMessageBox.Yes | QtGui.QMessageBox.No
                else:
                    buttons = QtGui.QMessageBox.Ok
                if QtGui.QMessageBox.critical(self,
                                                  u'Внимание!',
                                                  message,
                                                  buttons) != QtGui.QMessageBox.Yes:
                    return False
        return True


    def createOrder(self, timeTableWidget, queueWidget, clientId):
        def fillScheduleItem(scheduleItem, clientId, complaint, referral):
            scheduleItem.clientId = clientId
            scheduleItem.recordDatetime = QDateTime.currentDateTime()
            scheduleItem.recordPersonId = QtGui.qApp.userId
            scheduleItem.complaint = complaint
            scheduleItem.checked = False
            if referral:
                scheduleItem.srcOrgId        = referral.srcOrgId
                scheduleItem.srcPerson       = referral.srcPerson
                scheduleItem.srcSpecialityId = referral.srcSpecialityId
                scheduleItem.srcNumber       = referral.srcNumber
                scheduleItem.srcDate         = referral.srcDate

        date = self.calendarWidget.selectedDate()
        if self.checkApplicable(clientId, date):
            db = QtGui.qApp.db
            personId = self.getCurrentPersonId()
            specialityId = self.getPersonSpecialityId(personId)
            activityId = None
            orgStructureId = self.getPersonOrgStructureId(personId)
            model = queueWidget.model()
            row = queueWidget.currentIndex().row()
            if row == len(model.schedule.items):
                scheduleItem = model.createOvertimeScheduleItem()
            else:
                scheduleItem = model.getScheduleItem(row)
            if (    self.enableQueueing
                and personId
                and specialityId
                and orgStructureId
                and clientId
                and scheduleItem
                and self.queueingEnabled(date, orgStructureId, specialityId, activityId, personId, queueWidget, row, clientId)):

                appointmentPurposeId = model.schedule.appointmentPurposeId
                if isReferralRequired(appointmentPurposeId):
                    referral = inputReferral(self)
                    if not referral:
                        return False
                else:
                    referral = None

                if model.complaintRequired():
                    dlg = CComplaintsEditDialog(self)
                    if self.prevClientId == clientId:
                        dlg.setComplaints(self.prevComplaint)
                    dlg.disableCancel()
                    dlg.exec_()
                    complaint = dlg.getComplaints()
                    self.prevClientId = clientId
                    self.prevComplaint = complaint
                else:
                    complaint = ''

                if scheduleItem.id is None:
                    if self.lock('Schedule', model.schedule.id):
                        try:
                            schedule = model.schedule
                            schedule.reloadItems()
                            row = len(schedule.items)
                            scheduleItem.idx = schedule.items[-1].idx+1 if schedule.items else 0
                            fillScheduleItem(scheduleItem, clientId, complaint, referral)
                            QtGui.qApp.emitCurrentClientInfoJLWChanged(scheduleItem.save())
                            self.updateTimeTableRow(timeTableWidget)
                            model.emitDataChanged(row)
                            model.updateData()
                        finally:
                            self.releaseLock()
                    queueWidget.setCurrentIndex(model.index(row, 0))
                    return True
                else:
                    if self.lock('Schedule_Item', scheduleItem.id):
                        dataChanged = False
                        try:
                            model.updateData()
                            scheduleItem = model.getScheduleItem(row)
                            if scheduleItem.clientId or forceBool(scheduleItem.value('deleted')):
                                dataChanged = True
                            else:
                                db.transaction()
                                try:
                                    fillScheduleItem(scheduleItem, clientId, complaint, referral)
                                    QtGui.qApp.emitCurrentClientInfoJLWChanged(scheduleItem.save())
                                    db.commit()
                                    self.updateTimeTableRow(timeTableWidget)
                                    model.emitDataChanged(row)
                                    return True
                                except:
                                    db.rollback()
                                    raise
                        finally:
                            self.releaseLock()
                        if dataChanged:
                            QtGui.QMessageBox.warning( self,
                                u'Внимание!',
                                u'Запись на это время невозможна, так как оно уже занято',
                                QtGui.QMessageBox.Ok,
                                QtGui.QMessageBox.Ok)
        return False


    def deleteOrder(self, timeTableWidget, queueWidget):
        model = queueWidget.model()
        row = queueWidget.currentIndex().row()
        scheduleItem = model.getScheduleItem(row)
        if scheduleItem:
            confirmAndFreeScheduleItem(self, scheduleItem.id, scheduleItem.recordPersonId, scheduleItem.clientId)
            model.updateData()
            QtGui.qApp.emitCurrentClientInfoJLWDeleted(scheduleItem.id)


    def changeComplaint(self, queueWidget):
        model = queueWidget.model()
        model.updateData()

        row = queueWidget.currentIndex().row()
        complaint = model.getComplaint(row)
        if complaint is not None:
            dlg = CComplaintsEditDialog(self)
            dlg.setComplaints(complaint)
            if dlg.exec_():
                model.setComplaint(row, dlg.getComplaints())


    def changeReferral(self, queueWidget):
        model = queueWidget.model()
        model.updateData()
        row = queueWidget.currentIndex().row()
        referral = model.getReferral(row)
        dlg = CReferralEditDialog(self)
        dlg.setReferral(referral)
        if dlg.exec_():
            model.setReferral(row, dlg.getReferral())


    def updateTimeTableRow(self, timeTableWidget):
        timeTableWidget.model().updateAvail(timeTableWidget.currentIndex().row())


    def getCurrentQueuedClientId(self, queueWidget):
        row = queueWidget.currentIndex().row()
        if row>=0:
            return queueWidget.model().getClientId(row)
        else:
            return None


    def getCurrentScheduleItem(self, queueWidget):
        row = queueWidget.currentIndex().row()
        return queueWidget.model().getScheduleItem(row)


    def getQueuedClientsCount(self, queueWidget):
        return queueWidget.model().getQueuedClientsCount()


    def printQueue(self, timeTableWidget, queueWidget, variantPrint, nameMenuPrintQueue = u''):
        def _formatNote(note):
            target = u'Логин записавшего администратора'
            if target in note:
                return target + note[note.find(target)+len(target):]
            return note

        def _formatPerson(recordClass, recordPersonId, note):
            if recordClass:
                return {CScheduleItem.rcInfomat:u'Инфомат',
                        CScheduleItem.rcCallCenter:u'Call-центр; %s'%_formatNote(note),
                        CScheduleItem.rcInternet:u'Интернет'}.get(recordClass, u'')
            else:
                return forceString(QtGui.qApp.db.translate('vrbPersonWithSpeciality', 'id', recordPersonId, 'name'))

        receptionInfo = self.getReceptionInfo(timeTableWidget)
        personId = receptionInfo['personId']

        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        queueModel = queueWidget.model()
        cursor.setCharFormat(CReportBase.ReportSubTitle)
        if timeTableWidget == self.tblAmbTimeTable:
            cursor.insertText(u'запись на амбулаторный приём' + nameMenuPrintQueue)
            office = u'к. '+receptionInfo['office']
        else:
            cursor.insertText(u'вызовы на дом' + nameMenuPrintQueue)
            office = ''
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportTitle)
        personInfo = self.getPersonInfo(personId)
        cursor.insertText(personInfo['fullName'])
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportSubTitle)
        cursor.insertText(personInfo['specialityName'])
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportSubTitle)
        cursor.insertText(forceString(receptionInfo['date']) +' '+receptionInfo['timeRange']+' '+office)
#        complaintRequired = queueModel.complaintRequired()
        cols = [('5%',  [u'№'],       CReportBase.AlignRight),
                ('5%',  [u'время'],   CReportBase.AlignRight),
                ('55%', [u'пациент'], CReportBase.AlignLeft),
                ('20%', [u'жалобы'],  CReportBase.AlignLeft),
                ('15%', [u'записал'],  CReportBase.AlignLeft),
               ]

        table = createTable(cursor, cols)
        cnt = 0
        for row in xrange(queueModel.realRowCount()):
            scheduleItem = queueModel.getScheduleItem(row)
            if not scheduleItem:
                continue
            clientId = scheduleItem.clientId
            if clientId:
                if scheduleItem.overtime:
                    time = QDateTime()
                else:
                    time = scheduleItem.time

                recordClass = scheduleItem.recordClass
                recordPersonId = scheduleItem.recordPersonId
                note = scheduleItem.note
                if (((variantPrint == 0 or variantPrint == 1) and queueWidget == self.tblHomeQueue) or
                    ((variantPrint == 0 or variantPrint == 1) and (queueWidget == self.tblAmbQueue and
                      QtGui.qApp.ambulanceUserCheckable()))):
                    checked = scheduleItem.checked
                    if checked == variantPrint:
                        recordPerson = _formatPerson(recordClass, recordPersonId, note)
                        clientInfo = getClientInfoEx(clientId, self.calendarWidget.selectedDate())
                        clientText = u'%(fullName)s, %(birthDateStr)s (%(age)s), %(sex)s, код: %(id)s, СНИЛС: %(SNILS)s\n' \
                                     u'документ %(document)s, полис %(policy)s\n' \
                                     u'адр.рег. %(regAddress)s\n' \
                                     u'адр.прож. %(locAddress)s\n' \
                                     u'занятость %(work)s\n' \
                                     u'%(phones)s' % clientInfo
                        i = table.addRow()
                        cnt+=1
                        table.setText(i, 0, cnt)
                        table.setText(i, 1, locFormatTime(time))
                        table.setText(i, 2, clientText)
                        table.setText(i, 3, scheduleItem.complaint)
                        table.setText(i, 4, recordPerson)
                else:
                    if variantPrint == 2:
                        recordPerson = _formatPerson(recordClass, recordPersonId, note)
                        clientInfo = getClientInfoEx(clientId, self.calendarWidget.selectedDate())
                        clientText = u'%(fullName)s, %(birthDateStr)s (%(age)s), %(sex)s, код: %(id)s, СНИЛС: %(SNILS)s\n' \
                                     u'документ %(document)s, полис %(policy)s\n' \
                                     u'адр.рег. %(regAddress)s\n' \
                                     u'адр.прож. %(locAddress)s\n' \
                                     u'занятость %(work)s\n' \
                                     u'%(phones)s' % clientInfo
                        i = table.addRow()
                        cnt+=1
                        table.setText(i, 0, cnt)
                        table.setText(i, 1, locFormatTime(time))
                        table.setText(i, 2, clientText)
                        table.setText(i, 3, scheduleItem.complaint)
                        table.setText(i, 4, recordPerson)
        reportView = CReportViewDialog(self)
        reportView.setWindowTitle(u'Очередь')
        reportView.setText(doc)
        reportView.exec_()


    def printQueueByTemplate(self, timeTableWidget, queueWidget, templateId):
        receptionInfo = self.getReceptionInfo(timeTableWidget)
        queueModel = queueWidget.model()
        context = CInfoContext()
        queue = []
        if queueModel.schedule:
            for item in queueModel.schedule.items:
                time       = item.time
                overtime = item.overtime
                clientId   = item.clientId
                if clientId:
                    checked        = item.checked
                    complaint      = item.complaint
                    recordPersonId = item.recordPersonId
                    recordDatetime = item.recordDatetime
                else:
                    checked        = False
                    complaint      = ''
                    recordPersonId = None
                    recordDatetime = None
                queue.append(CQueueItemInfo(context, time, clientId, checked, complaint, recordPersonId, recordDatetime, overtime))

        data = {'date'    : CDateInfo(receptionInfo['date']),
                'office'  : receptionInfo['office'],
                'person'  : context.getInstance(CPersonInfo, receptionInfo['personId']),
                'timeRange': receptionInfo['timeRange'],
                'queue'   : queue,
                'recordDatetime' : CDateTimeInfo(recordDatetime)
               }
        applyTemplate(self, templateId, data)

    def printOrderByTemplate(self, timeTableWidget, queueWidget, templateId):
        scheduleItem = self.getCurrentScheduleItem(queueWidget)
        if scheduleItem:
            clientId = scheduleItem.clientId
            if clientId:
                visitInfo = self.getReceptionInfo(timeTableWidget)
                visit = {'date': CDateInfo(visitInfo['date']),
                           'office': visitInfo['office'],
                           'num': scheduleItem.idx+1,
                           'time': CTimeInfo(scheduleItem.time),
                           'overtime': scheduleItem.overtime,
                           'timeRange': visitInfo['timeRange'],
                           'complaints': scheduleItem.complaint,
                           'recordDatetime' : CDateTimeInfo(scheduleItem.recordDatetime),
                           'purpose' : CAppointmentPurposeInfo(self, visitInfo['purpose'])
                           }
                clientInfo = getClientInfo2(clientId)
                personInfo = clientInfo.getInstance(CPersonInfo, visitInfo['personId'])
                data = {'client':clientInfo, 'person':personInfo, 'visit': visit}
                applyTemplate(self, templateId, data)

    def getReceptionInfo(self, timeTableWidget):
        timeTableModel = timeTableWidget.model()
        timeTableRow = timeTableWidget.currentIndex().row()
        timeRange = unicode(formatTimeRange(timeTableModel.getTimeRange(timeTableRow)))
        return { 'date'     : self.calendarWidget.selectedDate(),
                 'office'   : timeTableModel.getOffice(timeTableRow),
                 'personId' : timeTableModel.personId,
                 'timeRange': timeRange,
                 'purpose': timeTableModel.getSchedule(timeTableRow).appointmentPurposeId}


    def getPersonInfo(self, personId):
        return getPersonInfo(personId)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelOrgStructure_currentChanged(self, current, previous):
        if not hasattr(self, 'groupingSpeciality'):
            self.groupingSpeciality = forceBool(QtGui.qApp.preferences.appPrefs.get('groupingSpeciality', True))
        self.updatePersonnelByOrgStructure()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelActivity_currentChanged(self, current, previous):
        if not hasattr(self, 'groupingSpeciality'):
            self.groupingSpeciality = forceBool(QtGui.qApp.preferences.appPrefs.get('groupingSpeciality', True))
        self.updatePersonnelByActivity()


#    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelPersonnel_currentChanged(self, current, previous):
        self.updateTimeTable()


    @pyqtSignature('')
    def on_timer_timeout(self):
        def update(tblTimeTable, tblQueue):
            if tblTimeTable.isEnabled():
                modelTimeTable = tblTimeTable.model()
                row  = tblTimeTable.currentIndex().row()
                if row>=0:
                    schedule = modelTimeTable.getSchedule(row)
                    date = schedule.date
                    scheduleId = schedule.id
                else:
                    date = self.calendarWidget.selectedDate()
                    scheduleId = None
                modelTimeTable.updateData()
                rowForScheduleId = modelTimeTable.findRowForScheduleId(scheduleId)
                newRow = modelTimeTable.getRowForDate(date, rowForScheduleId if rowForScheduleId>=0 else row)
                tblTimeTable.setCurrentIndex(modelTimeTable.index(newRow, 0))
                modelQueue = tblQueue.model()
                schedule = modelTimeTable.getSchedule(newRow)
                if schedule and schedule.id == scheduleId:
                    modelQueue.updateData()
                else:
                    modelQueue.setSchedule(schedule)
        self.timer.stop()
        try:
            self.setCaliendarDataRange()
            place = self.tabPlace.currentIndex()
            if place == 0:
                update(self.tblAmbTimeTable, self.tblAmbQueue)
            elif place == 1:
                update(self.tblHomeTimeTable, self.tblHomeQueue)
        finally:
            self.timer.start()


    @pyqtSignature('')
    def on_calendarWidget_selectionChanged(self):
        if self.activityListIsShown:
            self.updatePersonnelByActivity()
        else:
            self.updatePersonnelByOrgStructure()


    @pyqtSignature('int')
    def on_tabPlace_currentChanged(self, index):
        self.updateTimeTable()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelAmbTimeTable_currentChanged(self, current, previous):
        row = current.row()
        if row != previous.row() and 0<=row<self.modelAmbTimeTable.rowCount():
            date = self.modelAmbTimeTable.getDate(row)
            prevBlock = self.calendarWidget.blockSignals(True)
            try:
                self.calendarWidget.setSelectedDate(date)
            finally:
                self.calendarWidget.blockSignals(prevBlock)
            self.updateQueueEx(self.tblAmbTimeTable, self.modelAmbTimeTable, self.tblAmbQueue, self.modelAmbQueue)


    def popupMenuTreeOrgStructureAboutToShow(self):
        currentClientId = QtGui.qApp.currentClientId()
        self.actFindArea.setEnabled(bool(currentClientId))


    @pyqtSignature('')
    def on_actFindArea_triggered(self):
        currentClientId = QtGui.qApp.currentClientId()
        if currentClientId:
            r = getClientAddressEx(currentClientId)
            addressId = forceRef(r.value('address_id')) if r else None
            idList = findOrgStructuresByAddress(addressId, QtGui.qApp.currentOrgId())
            id = self.filterStructureId(idList, QtGui.qApp.currentOrgStructureId())
            if id:
                index = self.modelOrgStructure.findItemId(id)
                self.treeOrgStructure.setCurrentIndex(index)
            else:
                msg = u'Адрес пациента не соответствует никакому участку' if addressId else u'Адрес пациента пуст'
                QtGui.QMessageBox.warning( self.parent(),
                                           u'Поиск участка',
                                           msg,
                                           QtGui.QMessageBox.Ok,
                                           QtGui.QMessageBox.Ok)


    def filterStructureId(self, idList, topOrgStructureId):
        index = self.modelOrgStructure.findItemId(topOrgStructureId)
        childrenIdList = self.modelOrgStructure.getItemIdList(index)
        filteredList = list(set(childrenIdList).intersection(set(idList)))
        if filteredList:
            filteredList.sort()
            return filteredList[0]
        else:
            return None


    def popupMenuAboutToShow(self,
                             queueWidget,
                             actCreateOrder,
                             actDeleteOrder,
                             actChangeComplaint,
                             actChangeReferral,
                             actPrintOrder,
                             actPrintOrderByTemplate,
                             actPrintQueue,
                             actPrintQueuePass,
                             actPrintQueueNoPass,
                             actPrintQueueByTemplate,
                             actFindClient,
                             actFindClientEvents, 
                             actCreateEvent,
                             actServiceReport,
                             actInfo,
                             #actRegistrySuspenedAppointment
                             ):
        focusedVaidItem = 0<=queueWidget.currentIndex().row()<queueWidget.model().rowCount()
        currentClientId = QtGui.qApp.currentClientId()
        orderPresent   = bool(self.getCurrentQueuedClientId(queueWidget))
        anyOrderPresent= bool(self.getQueuedClientsCount(queueWidget))
        variantPrintQueue = QtGui.qApp.ambulanceUserCheckable()
        canFindClient  = QtGui.qApp.canFindClient()

        actCreateOrder.setEnabled(bool(currentClientId) and focusedVaidItem and not orderPresent and self.enableQueueing)
        actDeleteOrder.setEnabled(orderPresent)
        actChangeComplaint.setEnabled(orderPresent)
        actChangeReferral.setEnabled(orderPresent)
        actPrintOrder.setEnabled(orderPresent)
        if actPrintOrderByTemplate:
            actPrintOrderByTemplate.setEnabled(orderPresent)
        actPrintQueue.setEnabled(anyOrderPresent)
        if queueWidget == self.tblAmbQueue:
            actPrintQueuePass.setEnabled(anyOrderPresent and variantPrintQueue)
            actPrintQueueNoPass.setEnabled(anyOrderPresent and variantPrintQueue)
        else:
            actPrintQueuePass.setEnabled(anyOrderPresent)
            actPrintQueueNoPass.setEnabled(anyOrderPresent)
        actPrintQueueByTemplate.setEnabled(anyOrderPresent)
        actFindClient.setEnabled(canFindClient and bool(currentClientId))
        actCreateEvent.setEnabled(focusedVaidItem and orderPresent and bool(currentClientId) and QtGui.qApp.userHasAnyRight([urAdmin, urRegTabWriteEvents]))
        actServiceReport.setEnabled(anyOrderPresent)
        actInfo.setEnabled(focusedVaidItem and orderPresent)
        #actRegistrySuspenedAppointment.setEnabled(bool(currentClientId))


    def popupMenuAmbAboutToShow(self):
        self.popupMenuAboutToShow(self.tblAmbQueue,
                                  self.actAmbCreateOrder,
                                  self.actAmbDeleteOrder,
                                  self.actAmbChangeComplaint,
                                  self.actAmbChangeReferral,
                                  self.actAmbPrintOrder,
                                  self.actAmbPrintOrderByTemplate,
                                  self.actAmbPrintQueue,
                                  self.actAmbPrintQueuePass,
                                  self.actAmbPrintQueueNoPass,
                                  self.actAmbPrintQueueByTemplate,
                                  self.actAmbFindClient,
                                  self.actAmbFindClientEvents, 
                                  self.actAmbCreateEvent,
                                  self.actServiceReport,
                                  self.actAmbInfo)
                                  #self.actRegistrySuspenedAppointment)


    def popupMenuHomeAboutToShow(self):
        self.popupMenuAboutToShow(self.tblHomeQueue,
                                  self.actHomeCreateOrder,
                                  self.actHomeDeleteOrder,
                                  self.actHomeChangeComplaint,
                                  self.actHomeChangeReferral,
                                  self.actHomePrintOrder,
                                  self.actHomePrintOrderByTemplate,
                                  self.actHomePrintQueue,
                                  self.actHomePrintQueuePass,
                                  self.actHomePrintQueueNoPass,
                                  self.actHomePrintQueueByTemplate,
                                  self.actHomeFindClient,
                                  self.actHomeFindClientEvents,
                                  self.actHomeCreateEvent,
                                  self.actServiceReport,
                                  self.actHomeInfo)
                                  #self.actRegistrySuspenedAppointment)


    def popupMenuOrgPersonnelAboutToShow(self):
        currentClientId = QtGui.qApp.currentClientId()
        self.actRegistrySuspenedAppointment.setEnabled(bool(currentClientId))


    def printOrder(self, timeTableWidget, queueWidget):
        scheduleItem = self.getCurrentScheduleItem(queueWidget)
        if scheduleItem:
            printOrderByScheduleItem(self, scheduleItem.id)


    @pyqtSignature('')                                     # см. примечание 1
    def on_modelAmbTimeTable_rowCountChanged(self):               # см. примечание 1
        self.tblAmbTimeTable.verticalHeader().setUpdatesEnabled(True) # см. примечание 1


    @pyqtSignature('')                                     # см. примечание 1
    def on_modelAmbQueue_rowCountChanged(self):                   # см. примечание 1
        self.tblAmbQueue.verticalHeader().setUpdatesEnabled(True) # см. примечание 1


    @pyqtSignature('')                                     # см. примечание 1
    def on_modelHomeTimeTable_rowCountChanged(self):                  # см. примечание 1
        self.tblHomeTimeTable.verticalHeader().setUpdatesEnabled(True)# см. примечание 1

    @pyqtSignature('')                                     # см. примечание 1
    def on_modelHomeQueue_rowCountChanged(self):                  # см. примечание 1
        self.tblHomeQueue.verticalHeader().setUpdatesEnabled(True)# см. примечание 1


    @pyqtSignature('QModelIndex')
    def on_tblAmbQueue_doubleClicked(self, index):
        clientId = self.getCurrentQueuedClientId(self.tblAmbQueue)
        if clientId:
            if QtGui.qApp.doubleClickQueuePerson() == 0:
                self.on_actAmbChangeComplaint_triggered()
            elif QtGui.qApp.doubleClickQueuePerson() == 1:
                    self.on_actAmbFindClient_triggered()
            elif QtGui.qApp.doubleClickQueuePerson() == 2:
                   self.on_actAmbCreateEvent_triggered()
        else:
            self.on_actAmbCreateOrder_triggered()


    @pyqtSignature('QModelIndex')
    def on_tblHomeQueue_doubleClicked(self, index):
        clientId = self.getCurrentQueuedClientId(self.tblHomeQueue)
        if clientId:
            if QtGui.qApp.doubleClickQueuePerson() == 0:
                self.on_actHomeChangeComplaint_triggered()
            elif QtGui.qApp.doubleClickQueuePerson() == 1:
                    self.on_actHomeFindClient_triggered()
            elif QtGui.qApp.doubleClickQueuePerson() == 2:
                   self.on_actHomeCreateEvent_triggered()
        else:
            self.on_actHomeCreateOrder_triggered()


    @pyqtSignature('')
    def on_actAmbCreateOrder_triggered(self):
        if self.createOrder(self.tblAmbTimeTable, self.tblAmbQueue, QtGui.qApp.currentClientId()):
            self.printOrder(self.tblAmbTimeTable, self.tblAmbQueue)
        QtGui.qApp.emitCurrentClientInfoChanged()


    @pyqtSignature('')
    def on_actRegistrySuspenedAppointment_triggered(self):
        setRegistrySuspenedAppointment(self, QtGui.qApp.currentClientId())


    @pyqtSignature('')
    def on_actAmbDeleteOrder_triggered(self):
        self.deleteOrder(self.tblAmbTimeTable, self.tblAmbQueue)
        QtGui.qApp.emitCurrentClientInfoChanged()


    @pyqtSignature('')
    def on_actAmbChangeComplaint_triggered(self):
        self.changeComplaint(self.tblAmbQueue)
        QtGui.qApp.emitCurrentClientInfoChanged()


    @pyqtSignature('')
    def on_actAmbChangeReferral_triggered(self):
        self.changeReferral(self.tblAmbQueue)
        QtGui.qApp.emitCurrentClientInfoChanged()


    @pyqtSignature('')
    def on_actAmbPrintOrder_triggered(self):
        self.printOrder(self.tblAmbTimeTable, self.tblAmbQueue)


    @pyqtSignature('')
    def on_actAmbPrintQueue_triggered(self):
        self.printQueue(self.tblAmbTimeTable, self.tblAmbQueue, 2)


    @pyqtSignature('int')
    def on_actAmbPrintOrderByTemplate_printByTemplate(self, templateId):
        self.printOrderByTemplate(self.tblAmbTimeTable, self.tblAmbQueue, templateId)


    @pyqtSignature('')
    def on_actAmbPrintQueuePass_triggered(self):
        nameMenuPrintQueue = u', список подтвержденных записей'
        self.printQueue(self.tblAmbTimeTable, self.tblAmbQueue, 1, nameMenuPrintQueue)


    @pyqtSignature('')
    def on_actAmbPrintQueueNoPass_triggered(self):
        nameMenuPrintQueue = u', список неподтвержденных записей'
        self.printQueue(self.tblAmbTimeTable, self.tblAmbQueue, 0, nameMenuPrintQueue)


    @pyqtSignature('')
    def on_actAmbPrintPreRecordQueue_triggered(self):
        currentRow = self.tblAmbTimeTable.currentIndex().row()
        currentSchedule = self.modelAmbTimeTable.getSchedule(currentRow)
        scheduleList = [self.modelAmbTimeTable.getSchedule(row) for row in self.modelAmbTimeTable.getRowListForDate(currentSchedule.date)]
        if currentSchedule:
            CReportBeforeRecord(currentSchedule, scheduleList).exec_()


    @pyqtSignature('int')
    def on_actAmbPrintQueueByTemplate_printByTemplate(self, templateId):
        self.printQueueByTemplate(self.tblAmbTimeTable, self.tblAmbQueue, templateId)

    @pyqtSignature('')
    def on_actAmbCreateEvent_triggered(self):
        personId = self.getCurrentPersonId()
        specialityId = self.getPersonSpecialityId(personId)
        row = self.tblAmbQueue.currentIndex().row()
        time = self.tblAmbQueue.model().getTime(row) if row>=0 else None
        date = self.calendarWidget.selectedDate()
        scheduleItem = self.tblAmbQueue.model().getScheduleItem(row) if row>=0 else None
        if scheduleItem and specialityId:
            params = {'widget'             : self,
                      'clientId'           : scheduleItem.clientId,
                      'flagHospitalization':False,
                      'actionTypeId'       : None,
                      'valueProperties'    : [],
                      'eventTypeFilterHospitalization': 0,
                      'dateTime'           : time if time else QDateTime(date),
                      'personId'           : personId,
                      'typeQueue'          : 0,
                      'srcOrgId'           : scheduleItem.srcOrgId,
                      'srcPerson'          : scheduleItem.srcPerson,
                      'srcSpeciality'      : scheduleItem.srcSpecialityId,
                      'srcNumber'          : scheduleItem.srcNumber,
                      'srcDate'            : scheduleItem.srcDate,
                     }
            requestNewEvent(params)
        else:
            QtGui.qApp.requestNewEvent(scheduleItem.clientId, 0)


    @pyqtSignature('')
    def on_actHomeCreateEvent_triggered(self):
        clientId = self.getCurrentQueuedClientId(self.tblHomeQueue)
        personId = self.getCurrentPersonId()
        specialityId = self.getPersonSpecialityId(personId)
        row = self.tblHomeQueue.currentIndex().row()
        time = self.tblHomeQueue.model().getTime(row) if row>=0 else None
        date = self.calendarWidget.selectedDate()
        #WTF? когда specialityId бывает пусто при непустом personId?
        if specialityId:
            params = {'widget'             : self,
                      'clientId'           : clientId,
                      'flagHospitalization':False,
                      'actionTypeId'       : None,
                      'valueProperties'    : [],
                      'eventTypeFilterHospitalization': 0,
                      'dateTime'           : time if time else QDateTime(date),
                      'personId'           : personId,
                      'typeQueue'          : 1
                     }
            requestNewEvent(params)
        else:
            QtGui.qApp.requestNewEvent(clientId, 1)


    @pyqtSignature('')
    def on_actAmbFindClient_triggered(self):
        clientId = self.getCurrentQueuedClientId(self.tblAmbQueue)
        QtGui.qApp.findClient(clientId)


    @pyqtSignature('')
    def on_actAmbFindClientEvents_triggered(self):
        clientId = self.getCurrentQueuedClientId(self.tblAmbQueue)
        QtGui.qApp.findClient(clientId)
        QtGui.qApp.mainWindow.registry.tabMain.setCurrentIndex(1)


    @pyqtSignature('')
    def on_actAmbNotification_triggered(self):
        info = [(i.clientId, i) for i in self.tblAmbQueue.model().schedule.items if i.clientId]
        CNotifyDialog(info, CNotificationRule.ntSchedule,
            self.tblAmbQueue.model().schedule).exec_()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelHomeTimeTable_currentChanged(self, current, previous):
        row = current.row()
        if row != previous.row() and 0<=row<self.modelHomeTimeTable.rowCount():
            date = self.modelHomeTimeTable.getDate(row)
            prevBlock = self.calendarWidget.blockSignals(True)
            try:
                self.calendarWidget.setSelectedDate(date)
            finally:
                self.calendarWidget.blockSignals(prevBlock)
            self.updateQueueEx(self.tblHomeTimeTable, self.modelHomeTimeTable, self.tblHomeQueue, self.modelHomeQueue)


    @pyqtSignature('')
    def on_actHomeCreateOrder_triggered(self):
        self.createOrder(self.tblHomeTimeTable, self.tblHomeQueue, QtGui.qApp.currentClientId())


    @pyqtSignature('')
    def on_actHomeDeleteOrder_triggered(self):
        self.deleteOrder(self.tblHomeTimeTable, self.tblHomeQueue)


    @pyqtSignature('')
    def on_actHomeChangeComplaint_triggered(self):
        self.changeComplaint(self.tblHomeQueue)
        QtGui.qApp.emitCurrentClientInfoChanged()


    @pyqtSignature('')
    def on_actHomeChangeReferral_triggered(self):
        self.changeReferral(self.tblHomeQueue)
        QtGui.qApp.emitCurrentClientInfoChanged()


    @pyqtSignature('')
    def on_actHomePrintOrder_triggered(self):
        self.printOrder(self.tblHomeTimeTable, self.tblHomeQueue)


    @pyqtSignature('int')
    def on_actHomePrintOrderByTemplate_printByTemplate(self, templateId):
        self.printOrderByTemplate(self.tblHomeTimeTable, self.tblHomeQueue, templateId)


    @pyqtSignature('')
    def on_actHomePrintQueue_triggered(self):
        self.printQueue(self.tblHomeTimeTable, self.tblHomeQueue, 2)


    @pyqtSignature('')
    def on_actHomePrintQueuePass_triggered(self):
        nameMenuPrintQueue = u', список переданных врачу'
        self.printQueue(self.tblHomeTimeTable, self.tblHomeQueue, 0, nameMenuPrintQueue)


    @pyqtSignature('')
    def on_actHomePrintQueueNoPass_triggered(self):
        nameMenuPrintQueue = u', список непереданных врачу'
        self.printQueue(self.tblHomeTimeTable, self.tblHomeQueue, 1, nameMenuPrintQueue)


    @pyqtSignature('int')
    def on_actHomePrintQueueByTemplate_printByTemplate(self, templateId):
        self.printQueueByTemplate(self.tblHomeTimeTable, self.tblHomeQueue, templateId)


    @pyqtSignature('')
    def on_actHomeFindClient_triggered(self):
        clientId = self.getCurrentQueuedClientId(self.tblHomeQueue)
        QtGui.qApp.findClient(clientId)


    @pyqtSignature('')
    def on_actHomeFindClientEvents_triggered(self):
        clientId = self.getCurrentQueuedClientId(self.tblHomeQueue)
        QtGui.qApp.findClient(clientId)
        QtGui.qApp.mainWindow.registry.tabMain.setCurrentIndex(1)


# какое редкостное Г..., нужно вынести построение отчёта из этого модуля
    @pyqtSignature('')
    def on_actServiceReport_triggered(self):
        db = QtGui.qApp.db
        selectedDate = self.calendarWidget.selectedDate()
        personId = self.getCurrentPersonId()
        specialityId = self.getPersonSpecialityId(personId)

        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)

        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(u'Сводка об обслуживании на '+forceString(selectedDate)+'\n')
        cursor.insertBlock()

        cursor.setCharFormat(CReportBase.ReportBody)
        personInfo = self.getPersonInfo(personId)

        cursor.insertText(u'врач: %s, %s\n' % (personInfo['fullName'], personInfo['specialityName']))
        cursor.insertText(u'подразделение: %s, %s\n' % (personInfo['orgStructureName'], personInfo['postName']))
        cursor.insertText(u'дата формирования: %s\n' % forceString(QDate.currentDate()))
        cursor.insertBlock()

        cols = [
            ('2%',[u'№'], CReportBase.AlignRight),
            ('4%', [u'Время записи'], CReportBase.AlignLeft),
            ('10%', [u'ФИО'], CReportBase.AlignLeft),
            ('4%', [u'Д/р'], CReportBase.AlignLeft),
            ('3%', [u'Возраст'], CReportBase.AlignLeft),
            ('1%', [u'Пол'], CReportBase.AlignLeft),
            ('15%', [u'Адрес'], CReportBase.AlignLeft),
            ('25%', [u'Полис'], CReportBase.AlignLeft),
            ('10%', [u'Паспорт'], CReportBase.AlignLeft),
            ('10%', [u'СНИЛС'], CReportBase.AlignLeft),
            ('10%', [u'Врач'], CReportBase.AlignLeft),
            ('4%', [u'Время обслуживания'], CReportBase.AlignLeft),
            ('10%', [u'Тип'], CReportBase.AlignLeft),
            ('5%', [u'Диагноз'], CReportBase.AlignLeft),
            ('10%', [u'Результат'], CReportBase.AlignLeft),
               ]
        table = createTable(cursor, cols)
        cnt = 0

        clients=[]
        for (timeTableWidget, queueWidget) in ((self.tblAmbTimeTable, self.tblAmbQueue), (self.tblHomeTimeTable, self.tblHomeQueue)):
            queueModel = queueWidget.model()
            for row in xrange(queueModel.realRowCount()):
                if queueModel.getClientId(row):
                    time = queueModel.getTime(row)
                    clientId = queueModel.getClientId(row)
                    if clientId:
                        clients.append((clientId, time))

        num_same=0
        num_other=0
        num_no=0
        for (clientId, time) in clients:#TODO: Возможно сделать вывод данных по другому
            i = table.addRow()
            cnt+=1
            table.setText(i, 0, i)
            clientInfo = getClientInfoEx(clientId, selectedDate)
            table.setText(i, 2, clientInfo['fullName'])
            table.setText(i, 3, clientInfo['birthDateStr'])
            table.setText(i, 4, clientInfo['age'])
            table.setText(i, 5, clientInfo['sex'])
            table.setText(i, 6, clientInfo['regAddress'])
            table.setText(i, 7, clientInfo['policy'])
            table.setText(i, 8, clientInfo['document'])
            table.setText(i, 9, clientInfo['SNILS'])
            stmt="""
SELECT
    Person.id AS person_id, Diagnosis.MKB, Visit.date, Event.result_id, rbDiagnosticResult.name AS result, rbScene.name AS scene, Visit.createDatetime, Person.speciality_id
FROM
    Visit
    JOIN Event ON Event.id=Visit.event_id
    JOIN Person ON Person.id=Visit.person_id
    LEFT JOIN Diagnosis on Diagnosis.id=getEventPersonDiagnosis(Event.id, Person.id)
    LEFT JOIN Diagnostic ON Diagnosis.id = Diagnostic.diagnosis_id
    LEFT JOIN rbDiagnosticResult ON rbDiagnosticResult.id=Diagnostic.result_id
    LEFT JOIN rbScene ON rbScene.id = Visit.scene_id
WHERE
    Visit.deleted = 0
AND Event.deleted = 0
AND (Person.id=%(personId)d OR Person.speciality_id=%(specialityId)d)
AND DATE(Visit.date) = %(visitDate)s
AND Event.client_id=%(clientId)d
ORDER BY (Person.id=%(personId)d)
LIMIT 1""" % {'personId'    : personId,
              'specialityId': specialityId,
              'visitDate'   : db.formatDate(selectedDate),
              'clientId'    : clientId,
             }
            query = db.query(stmt)
            withoutVisit = True
            table.setText(i, 1, formatTime(time) if time else '--:--')
            if query.next():
                record = query.record()
                visitPersonId = forceRef(record.value('person_id'))
                if visitPersonId:
                    visitPersonInfo = self.getPersonInfo(visitPersonId)
                    if visitPersonInfo:
                        table.setText(i, 10, visitPersonInfo['fullName'])
                    table.setText(i, 11, formatTime(record.value('createDatetime').toTime()))
                    table.setText(i, 12, forceString(record.value('scene')))
                    table.setText(i, 13, forceString(record.value('MKB')))
                    table.setText(i, 14, forceString(record.value('result')))
                    if personId==visitPersonId:
                        num_same+=1
                    elif forceRef(record.value('speciality_id'))==specialityId:
                        num_other+=1
                    withoutVisit = False
            if withoutVisit:
                num_no+=1
#                table.setText(i, 1, '-')
                table.setText(i, 10, '-')
                table.setText(i, 11, '-')
                table.setText(i, 12, '-')
                table.setText(i, 13, '-')
                table.setText(i, 14, '-')

        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.setCharFormat(CReportBase.ReportBody)
        cursor.insertText(u'обслужено врачом к которому была запись: %d; обслужено коллегами: %d; не было обслужено: %d\n' % (num_same, num_other, num_no))
        reportView = CReportViewDialog(self)
        reportView.setWindowTitle(u'Сводка об обслуживании')
        reportView.setText(doc)
        reportView.exec_()


    @pyqtSignature('')
    def on_actAmbInfo_triggered(self):
        self.showInfo(self.tblAmbTimeTable, self.tblAmbQueue)


    @pyqtSignature('')
    def on_actHomeInfo_triggered(self):
        self.showInfo(self.tblHomeTimeTable, self.tblHomeQueue)


    def showInfo(self, timeTableWidget, queueWidget):
        queueModel = queueWidget.model()
        row = queueWidget.currentIndex().row()
        scheduleItem = queueModel.getScheduleItem(row)
        if scheduleItem:
            showScheduleItemInfo(scheduleItem.id, self)


class CTimeTableModel(QAbstractTableModel):
    __pyqtSignals__ = ( 'rowCountChanged()',) # см. примечание 1

    headerText = ( u'Назначение', u'Время', u'Каб', u'План', u'Готов' )

    def __init__(self, parent, appointmentType):
        QAbstractTableModel.__init__(self, parent)
        self.appointmentType = appointmentType
        self.personId   = None
        self.begDate    = None
        self.endDate    = None
        self.activityId = None
        self.scheduleList = []
        self.redBrush = QtGui.QBrush(Qt.red)


    def columnCount(self, index = None):
        return 5


    def rowCount(self, index = None):
        return len(self.scheduleList)


    def flags(self, index):
        return Qt.ItemIsSelectable|Qt.ItemIsEnabled


    def headerData(self, section, orientation, role = Qt.DisplayRole):
        if orientation == Qt.Horizontal:
            if role == Qt.DisplayRole:
                return QVariant(self.headerText[section])
        if orientation == Qt.Vertical:
            date = self.scheduleList[section].date
            if role == Qt.DisplayRole:
                return QVariant(QDate.shortDayName(date.dayOfWeek()))
            elif role == Qt.ToolTipRole:
                if self.begDate:
                    return QVariant(date)
            elif role == Qt.ForegroundRole:
                if date.dayOfWeek()>=6:
                    return QVariant(self.redBrush)
        return QVariant()


    def data(self, index, role=Qt.DisplayRole):
        column = index.column()
        row = index.row()
        if role == Qt.DisplayRole:
            schedule = self.scheduleList[row]
            if column == 0: # Назначение
                appointmentPurposeId = schedule.appointmentPurposeId
                return toVariant(self.getAppointmentPurposeName(appointmentPurposeId))
            if column == 1: # Время
                timeRange = schedule.begTime, schedule.endTime
                reasonOfAbsenceId = schedule.reasonOfAbsenceId
                if reasonOfAbsenceId:
                    return toVariant(formatTimeRange(timeRange)+', '+self.getReasonOfAbsenceCode(reasonOfAbsenceId))
                else:
                    return toVariant(formatTimeRange(timeRange))
            elif column == 2: # Каб
                return toVariant(schedule.office)
            elif column == 3: # План
                return toVariant(schedule.capacity)
            elif column == 4: # Готов
                return toVariant(schedule.capacity-schedule.getQueuedClientsCount())
        elif role == Qt.FontRole:
            schedule = self.scheduleList[row]
            if schedule.reasonOfAbsenceId:
                result = QtGui.QFont()
                result.setBold(True)
                result.setStrikeOut(True)
                return QVariant(result)
        return QVariant()


    def setPersonAndDate(self, personId, date, minDate, maxDate, activityId = None):
        if self.personId == personId and self.begDate <= date <= self.endDate and activityId == self.activityId:
            self.updateData()
        else:
            # self.beginResetModel()
            try:
                self.personId = personId
                self.begDate = max(date.addDays(1-date.dayOfWeek()), minDate)
                self.endDate = min(self.begDate.addDays(6), maxDate)
                self.activityId = activityId
                self.loadData()
            finally:
                #self.endResetModel()
                self.reset()


    def getAppointmentPurposeName(self, appointmentPurposeId):
        return CAppointmentPurposeCache.getName(appointmentPurposeId)


    def getReasonOfAbsenceCode(self, reasonOfAbsenceId):
        cache = CRBModelDataCache.getData('rbReasonOfAbsence', True)
        text = cache.getStringById(reasonOfAbsenceId, CRBComboBox.showCode)
        return text


    def getRowForDate(self, date, prevRow=-1):
        if 0<=prevRow<len(self.scheduleList):
            if self.scheduleList[prevRow].date == date:
                return prevRow
        for row, schedule in enumerate(self.scheduleList):
            if schedule.date == date:
                return row
        return -1


    def getRowListForDate(self, date):
        result = []
        for row, schedule in enumerate(self.scheduleList):
            if schedule.date == date:
                result.append(row)
        return result


    def loadSchedules(self, begDate, endDate, personId, activityId):
        if begDate and endDate and personId:
            db = QtGui.qApp.db
            tableSchedule = db.table('Schedule')
            tablePerson   = db.table('Person')
            table = tableSchedule.leftJoin(tablePerson, tablePerson['id'].eq(tableSchedule['person_id']))
            cond = [tableSchedule['deleted'].eq(0),
                    tableSchedule['appointmentType'].eq(self.appointmentType),
                    tableSchedule['date'].between(begDate, endDate),
                    tableSchedule['person_id'].eq(personId)
                   ]
            activityPref = QtGui.qApp.getGlobalPreference('23')
            if activityPref == u'да' and activityId:
                cond.append(db.joinOr([tableSchedule['activity_id'].eq(activityId), tableSchedule['activity_id'].isNull()]))
            cond.append( '`Person`.`lastAccessibleTimelineDate` IS NULL OR `Person`.`lastAccessibleTimelineDate`=0 OR `Schedule`.`date`<=`Person`.`lastAccessibleTimelineDate`')
            if QtGui.qApp.isTimelineAccessibilityDays() == 1:
                cond.append( '`Person`.`timelineAccessibleDays` <= 0 OR `Schedule`.`date`<%s'%db.dateAdd('current_date', 'day', 'Person.timelineAccessibleDays'))

            return [CSchedule(record) for record in db.getRecordList(table,
                                                                     'Schedule.*',
                                                                     cond,
                                                                     'Schedule.date, Schedule.begTime, Schedule.id'
                                                                    )
                   ]
        else:
            return []


    def loadDataInt(self):
        return self.loadSchedules(self.begDate, self.endDate, self.personId, self.activityId)


    def loadData(self):
        self.scheduleList = self.loadDataInt()


    def updateData(self):
        oldList = self.scheduleList
        self.scheduleList = self.loadDataInt()
        oldLen = len(oldList)
        newLen = len(self.scheduleList)
        if oldLen<newLen:
            # добавилось
            self.beginInsertRows(QModelIndex(), oldLen, newLen-1)
            self.endInsertRows()
        elif oldLen>newLen:
            # удалилось
            self.beginRemoveRows(QModelIndex(), newLen, oldLen-1)
            self.endRemoveRows()
        for row in xrange(min(oldLen, newLen)):
            oldSchedule = oldList[row]
            newSchedule = self.scheduleList[row]
            if oldSchedule != newSchedule:
                self.emitDataChanged(row)


    def updateAvail(self, row):
        index = self.index(row, 4)
        self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index, index)


    def getDate(self, row):
        if 0<=row<len(self.scheduleList):
            schedule = self.scheduleList[row]
            return schedule.date
        else:
            return None


    def getTimeRange(self, row):
        schedule = self.scheduleList[row]
        return schedule.begTime, schedule.endTime


    def getOffice(self, row):
        schedule = self.scheduleList[row]
        return schedule.office


    def getSchedule(self, row):
        if 0<=row<len(self.scheduleList):
            return self.scheduleList[row]
        else:
            return None


    def findRowForScheduleId(self, scheduleId):
        for row, schedule in enumerate(self.scheduleList):
            if schedule.id == scheduleId:
                return row
        return -1


    def reset(self):                                # см. примечание 1
        QAbstractTableModel.reset(self)             # см. примечание 1
        self.emitRowCountChanged()                  # см. примечание 1


    def endInsertRows(self):                        # см. примечание 1
        QAbstractTableModel.endInsertRows(self)     # см. примечание 1
        self.emitRowCountChanged()                  # см. примечание 1


    def endRemoveRows(self):                        # см. примечание 1
        QAbstractTableModel.endRemoveRows(self)     # см. примечание 1
        self.emitRowCountChanged()                  # см. примечание 1


    def emitRowCountChanged(self):                      # см. примечание 1
        self.emit(SIGNAL('rowCountChanged()'))   # см. примечание 1



class CQueueModel(QAbstractTableModel):
    __pyqtSignals__ = ( 'rowCountChanged()',) # см. примечание 1

    def __init__(self, parent, appointmentType):
        QAbstractTableModel.__init__(self, parent)
        self.appointmentType = appointmentType
        #FIXME: реализовать другой механизм подбора и кеширования специальности
        self.personId = None
        self.personSpecialityId = None
        self.schedule = None
        self.overtimeEnabled = False
        self.mapClientIdToName = {}
        self.itemsAreCheckable = (self.appointmentType == CSchedule.atHome) or QtGui.qApp.ambulanceUserCheckable()
        self.getPersonSpecialityId = parent.getPersonSpecialityId


    def complaintRequired(self):
        return self.appointmentType == CSchedule.atHome


    def columnCount(self, index=None):
        return 1


    def rowCount(self, index=None):
        if self.schedule:
            return len(self.schedule.items)+(1 if self.overtimeEnabled else 0)
        else:
            return 0


    def realRowCount(self):
        return len(self.schedule.items) if self.schedule else 0


    def flags(self, index):
        result = Qt.ItemIsSelectable|Qt.ItemIsEnabled
        if self.itemsAreCheckable:
            row = index.row()
            if 0<=row<len(self.schedule.items):
                item = self.schedule.items[row]
                if item.clientId:
                    result = result | Qt.ItemIsUserCheckable
        return result


    def headerData(self, section, orientation, role = Qt.DisplayRole):
        if orientation == Qt.Horizontal:
            if role == Qt.DisplayRole:
                return QVariant(u'ФИО')
        if orientation == Qt.Vertical:
            if role == Qt.DisplayRole:
                if 0<=section<len(self.schedule.items):
                    item = self.schedule.items[section]
                    time = None if item.overtime else item.time
                else:
                    time = None
                return QVariant(locFormatTime(time))
            if role == Qt.TextAlignmentRole:
                return QVariant(Qt.AlignRight)
        return QVariant()


    def getTextForClientId(self, clientId):
        if clientId:
            result = self.mapClientIdToString.get(clientId, False)
            if result == False:
                result = getClientMiniInfo(clientId)
                self.mapClientIdToString[clientId] = result
            return result
        return None


    def data(self, index, role=Qt.DisplayRole):
#        column = index.column()
        row = index.row()
        item = self.schedule.items[row] if 0<=row<len(self.schedule.items) else None

        if role == Qt.DisplayRole:
            if item:
                return toVariant(self.getTextForClientId(item.clientId))
        elif role == Qt.CheckStateRole and self.itemsAreCheckable:
            if item and item.clientId:
                return toVariant(Qt.Checked if item.checked else Qt.Unchecked)
        elif role == Qt.ToolTipRole:
            if item and item.clientId:
                cache = CRBModelDataCache.getData('vrbPersonWithSpeciality', True)
                text = u'Записал: '+cache.getStringById(item.recordPersonId, CRBComboBox.showName)
                if item.complaint:
                    text += u'\nЖалобы: '+item.complaint
                if item.note:
                    text += u'\nПримечание: '+item.note
                return toVariant(text)
        elif role == Qt.FontRole:
            if item and item.clientId and item.recordClass == item.rcSamson:
                recordPersonId = item.recordPersonId
                result = QtGui.QFont()
                if self.schedule.personId == recordPersonId:
                    result.setBold(True)
                else:
                    recordPersonSpecialityId = self.getPersonSpecialityId(recordPersonId)
                    if recordPersonSpecialityId:
                        result.setItalic(True)
                        if self.personSpecialityId == recordPersonSpecialityId:
                            result.setBold(True)
                return QVariant(result)
        elif role == Qt.DecorationRole:
            if item and item.clientId:
                if  item.recordClass==item.rcSamson:
                    return QVariant(QtGui.QColor(Qt.transparent))
                else:
                    return QVariant(QtGui.QIcon(':icons/redBullet.svg'))
#                return QVariant(QtGui.QColor(['green', 'blue', 'yellow', 'red'][item.recordClass]))


        return QVariant()


    def setData(self, index, value, role=Qt.EditRole):
#        column = index.column()
        row = index.row()
        item = self.schedule.items[row] if 0<=row<len(self.schedule.items) else None
        if role == Qt.CheckStateRole and self.itemsAreCheckable and item:
            item.checked = forceBool(value)
            item.save()
            QtGui.qApp.emitCurrentClientInfoChanged()
            return True
        return False


    def setSchedule(self, schedule):
        if self.schedule is not None and schedule is not None and self.schedule == schedule:
            self.updateData()
        else:
            self.schedule = schedule
            self.mapClientIdToString = {}
            if schedule:
                personId = schedule.personId
                if self.personId != personId:
                    self.personSpecialityId = forceRef(QtGui.qApp.db.translate('Person', 'id', personId, 'speciality_id'))
                    self.personId = personId
                self.loadData()
            else:
                self.personId = None
                self.personSpecialityId = None
                self.loadData()
            self.reset()


    def loadData(self):
        if self.schedule:
            self.overtimeEnabled = len(self.schedule.items)>0 and (
                QtGui.qApp.userHasRight(urQueueOverTime) or
                self.schedule.personId == QtGui.qApp.userId and QtGui.qApp.userHasRight(urQueueToSelfOverTime)
                )
        else:
            self.overtimeEnabled = False


    def updateData(self):
        if self.schedule:
            oldItems = self.schedule.items[:]
            oldLen = len(oldItems)
            self.schedule.reloadItems()
            newLen = len(self.schedule.items)
            if oldLen<newLen:
                # добавилось
                self.beginInsertRows(QModelIndex(), oldLen, newLen-1)
                self.endInsertRows()
            elif oldLen>newLen:
                # удалилось
                self.beginRemoveRows(QModelIndex(), newLen, oldLen-1)
                self.endRemoveRows()
            for row in xrange(min(oldLen, newLen)):
                oldItem = oldItems[row]
                newItem = self.schedule.items[row]
                if oldItem.time != newItem.time:
                    self.emitVerticalHeaderChanged(row)
                if oldItem.clientId != newItem.clientId or oldItem.checked != newItem.checked:
                    self.emitDataChanged(row)


    def reset(self):                                # см. примечание 1
        QAbstractTableModel.reset(self)             # см. примечание 1
        self.emitRowCountChanged()                  # см. примечание 1


    def endInsertRows(self):                        # см. примечание 1
        QAbstractTableModel.endInsertRows(self)     # см. примечание 1
        self.emitRowCountChanged()                  # см. примечание 1


    def endRemoveRows(self):                        # см. примечание 1
        QAbstractTableModel.endRemoveRows(self)     # см. примечание 1
        self.emitRowCountChanged()                  # см. примечание 1


    def emitRowCountChanged(self):                      # см. примечание 1
        self.emit(SIGNAL('rowCountChanged()'))   # см. примечание 1


    def emitDataChanged(self, row):
        index = self.index(row, 0)
        self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index, index)


    def emitVerticalHeaderChanged(self, row):
        self.emit(SIGNAL('headerDataChanged(Qt::Orientation, int, int)'), Qt.Vertical,  row, row)


    def getScheduleItem(self, row):
        if self.schedule and 0<=row<len(self.schedule.items):
            return self.schedule.items[row]
        else:
            return None


    def createOvertimeScheduleItem(self):
        result = CScheduleItem()
        result.scheduleId = self.schedule.id
        result.time = QDateTime(self.schedule.date, self.schedule.endTime)
        result.overtime = True
        return result


    def getRecordClass(self, row):
        item = self.getScheduleItem(row)
        if item:
            return item.recordClass
        return None


    def getTime(self, row):
        if self.schedule and 0<=row<len(self.schedule.items):
            item = self.schedule.items[row]
            if item.overtime:
                return QDateTime()
            return item.time
        else:
            return QDateTime()


    def getClientId(self, row):
        if self.schedule and 0<=row<len(self.schedule.items):
            return self.schedule.items[row].clientId
        else:
            return None


    def getRecordPersonId(self, row):
        if self.schedule and 0<=row<len(self.schedule.items):
            return self.schedule.items[row].recordPersonId
        else:
            return None


    def isChecked(self, row):
        if self.schedule and 0<=row<len(self.schedule.items):
            return self.schedule.items[row].checked
        else:
            return None


    def setChecked(self, row, checked):
        if self.schedule and 0<=row<len(self.schedule.items):
            item = self.schedule.items[row]
            item.checked = checked
            item.save()


    def getClientText(self, row):
        clientId = self.getClientId(row)
        return self.getTextForClientId(clientId) if clientId else None


    def getComplaint(self, row):
        if self.schedule and 0<=row<len(self.schedule.items):
            return self.schedule.items[row].complaint
        else:
            return None


    def setComplaint(self, row, complaint):
        if self.schedule and 0<=row<len(self.schedule.items):
            item = self.schedule.items[row]
            item.complaint = complaint
            item.save()


    def getReferral(self, row):
        referral = CReferral
        if self.schedule and 0<=row<len(self.schedule.items):
            item = self.schedule.items[row]
            referral.srcOrgId        = item.srcOrgId
            referral.srcPerson       = item.srcPerson
            referral.srcSpecialityId = item.srcSpecialityId
            referral.srcNumber       = item.srcNumber
            referral.srcDate         = item.srcDate
        return referral


    def setReferral(self, row, referral):
        if self.schedule and 0<=row<len(self.schedule.items):
            item = self.schedule.items[row]
            item.srcOrgId        = referral.srcOrgId
            item.srcPerson       = referral.srcPerson
            item.srcSpecialityId = referral.srcSpecialityId
            item.srcNumber       = referral.srcNumber
            item.srcDate         = referral.srcDate
            item.save()


    def getQueuedClientsCount(self):
        if self.schedule:
            return self.schedule.getQueuedClientsCount()
        else:
            return 0


    def getPrimaryQueuedCount(self):
        # первичная запись:
        # recordClass = самсон и записавшее лицо не имеет специальности
        result = 0
        for item in self.schedule.items:
            if (    item.clientId
                and item.recordClass == CScheduleItem.rcSamson
                and self.getPersonSpecialityId(item.recordPersonId) is None
               ):
                result += 1
        return result


    def getOwnQueuedCount(self):
        # записано из самсона самим пользователем
        result = 0
        for item in self.schedule.items:
            if (    item.clientId
                and item.recordClass == CScheduleItem.rcSamson
                and item.recordPersonId == QtGui.qApp.userId
               ):
                result += 1
        return result


    def getConsultancyQueuedCount(self):
        # консультативная запись:
        # записано из самсона пользователем со специальностью
        result = 0
        for item in self.schedule.items:
            if (    item.clientId
                and item.recordClass == CScheduleItem.rcSamson
                and self.getPersonSpecialityId(item.recordPersonId)
                and item.recordPersonId != self.schedule.personId
               ):
                result += 1
        return result


    def getExternalQueuedCount(self):
        # внешняя запись:
        result = 0
        for item in self.schedule.items:
            if (    item.clientId
                and item.recordClass != CScheduleItem.rcSamson
               ):
                result += 1
        return result


    def getQueuedCount(self):
        # количество записанных
        specialityId = QtGui.qApp.userSpecialityId
        if specialityId:
            if self.schedule.personId == QtGui.qApp.userId:
                return self.getOwnQueuedCount()
            else:
                return self.getConsultancyQueuedCount()
        else:
            return self.getPrimaryQueuedCount()


    def findClientRow(self, clientId):
        for row, item in enumerate(self.schedule.items):
            if item.clientId == clientId:
                return row
        return -1


    def findActionIdRow(self, actionId):
        sorry()
        for row, actionInfo in enumerate(self.actionInfoList):
            if actionInfo.actionId == actionId:
                return row
        return -1


# ######################################################################


def locFormatTime(time):
    return time.toString('H:mm') if time else '--:--'


def getQuota(personId, count, appointmentPurposeId):
    def divideQoutes(quotes, enablements):
        assert len(quotes) == len(enablements)

        t = sum(enablements)
        if t == len(quotes):
            return quotes
        elif t == 0:
            return [0.0]*len(quotes)
        elif t == 1:
            return [ 100.0 if enable else 0.0 for enable in enablements ]
        else:
            enabled = disabled = 0.0
            for i, enable in enumerate(enablements):
                if enable:
                   enabled += quotes[i]
                else:
                   disabled += quotes[i]
            if enabled>0:
                for i, enable in enumerate(enablements):
                    if enable:
                       quotes[i] += disabled*quotes[i]/enabled
                    else:
                       quotes[i] = 0.0
            return quotes

    db = QtGui.qApp.db
    table = db.table('Person')
    record = db.getRecord(table, ['ownQuota', 'consultancyQuota', 'primaryQuota', 'externalQuota'], personId)
    if not record:
        return 0

    quotes = [ forceDouble(record.value('ownQuota')),
               forceDouble(record.value('consultancyQuota')),
               forceDouble(record.value('primaryQuota')),
               forceDouble(record.value('externalQuota'))
             ]

    appointmentPurpose = CAppointmentPurposeCache.getItem(appointmentPurposeId)
    if appointmentPurpose:
        enablements = (appointmentPurpose.enableOwnRecord,
                       appointmentPurpose.enableConsultancyRecord,
                       appointmentPurpose.enablePrimaryRecord,
                       appointmentPurpose.enableRecordViaInfomat or appointmentPurpose.enableRecordViaCallCenter or appointmentPurpose.enableRecordViaInternet
                      )
        quotes = divideQoutes(quotes, enablements)

    if personId == QtGui.qApp.userId:
        quota = quotes[0]
    elif QtGui.qApp.userSpecialityId:
        quota = quotes[1]
    else:
        quota = quotes[2]
    return math.ceil(quota*count/100.0)


def isAppointmentEnabled(appointmentPurposeId, personId):
    appointmentPurpose = CAppointmentPurposeCache.getItem(appointmentPurposeId)
    if appointmentPurpose:
        if personId == QtGui.qApp.userId:
            return appointmentPurpose.enableOwnRecord
        elif QtGui.qApp.userSpecialityId:
            return appointmentPurpose.enableConsultancyRecord
        else:
            return appointmentPurpose.enablePrimaryRecord
    else:
        return True


def isAppointmentEnabledForClient(appointmentPurposeId, personId, date, clientId):
    appointmentPurpose = CAppointmentPurposeCache.getItem(appointmentPurposeId)
    if appointmentPurpose:
        if clientId:
            db = QtGui.qApp.db
            clientRecord = db.getRecord('Client', ('sex', 'birthDate'), clientId)
            clientSex = forceInt(clientRecord.value('sex'))
            clientAge = calcAgeTuple(forceDate(clientRecord.value('birthDate')), date)
            if not appointmentPurpose.applicable(clientSex, clientAge):
                return False

        if personId == QtGui.qApp.userId:
            return appointmentPurpose.enableOwnRecord
        elif QtGui.qApp.userSpecialityId:
            return appointmentPurpose.enableConsultancyRecord
        else:
            return appointmentPurpose.enablePrimaryRecord
    return True


def isReferralRequired(appointmentPurposeId):
    appointmentPurpose = CAppointmentPurposeCache.getItem(appointmentPurposeId)
    if appointmentPurpose:
        return appointmentPurpose.requireReferral
    return False


class CActivityTreeItem(CTreeItemWithId):
    def __init__(self, parent, name, id):
        CTreeItemWithId.__init__(self, parent, name, id)


    def loadChildren(self):
        return []


class CActivityRootTreeItem(CActivityTreeItem):
    def __init__(self):
        CActivityTreeItem.__init__(self, None, u'-', None)


    def loadChildren(self):
        items = []
        db = QtGui.qApp.db
        tableRBActivity = db.table('rbActivity')
        query = db.query(db.selectStmt(tableRBActivity, [tableRBActivity['id'], tableRBActivity['name'] ]))
        while query.next():
            record = query.record()
            id   = forceInt(record.value('id'))
            name = forceString(record.value('name'))
            items.append(CActivityTreeItem(self, name, id))
        return items


class CActivityModel(CTreeModel):
    def __init__(self, parent=None):
        CTreeModel.__init__(self, parent, CActivityRootTreeItem())
        self.rootItemVisible = False


    def reload(self):
        self._rootItem.loadChildren()
        self.reset()


    def headerData(self, section, orientation, role):
        if role == Qt.DisplayRole:
            return QVariant(u'Виды деятельности')
        return QVariant()


class CResourcesPersonnelModelMixin(object):
    def data(self, index, role):
        if not index.isValid():
            return QVariant()
        if role != Qt.DisplayRole:
            if role == Qt.ToolTipRole:
                personIdList = self.getItemIdList(index)
                if personIdList and len(personIdList) == 1:
                    text = self.getPersonStr(personIdList[0])
                    return toVariant(text)
            elif role == Qt.FontRole:
                item = index.internalPointer()
                if len(item._items) > 0:
                    ret = QtGui.QFont()
                    ret.setBold( 1 )
                    return ret
            else:
                return QVariant()
        item = index.internalPointer()
        return item.data(index.column())


    def getPersonStr(self, id):
        result = []
        if id:
            personInfo = getPersonInfo(id)
            result.append( personInfo['fullName'] )
            if personInfo['specialityName']:
                result.append(personInfo['specialityName'])
            if personInfo['orgStructureName']:
                result.append(u'подразделение: ' + personInfo['orgStructureName'])
            if personInfo['postName']:
                result.append(u'должность: ' + personInfo['postName'])
            if personInfo['tariffCategoryName']:
                result.append(u'категория: ' + personInfo['tariffCategoryName'])
        return ', '.join(result)


class CFlatResourcesPersonnelModel(CResourcesPersonnelModelMixin, CFlatOrgPersonnelModel):
    pass


class CResourcesPersonnelModel(CResourcesPersonnelModelMixin, COrgPersonnelModel):
    pass


class CQueueItemInfo(CInfo):
    def __init__(self, context, time, clientId, checked, complaint, recordPersonId, recordDatetime, overtime=None):
        CInfo.__init__(self, context)
        self.setOkLoaded()
        self._clientId = clientId
        self.time = CTimeInfo(time)
        self.client = self.getInstance(CClientInfo, clientId)
        self.checked = checked
        self.complaints = complaint
        self.overtime = overtime
        self.recordDatetime = CDateTimeInfo(recordDatetime)
        self.setPerson = self.getInstance(CPersonInfo, recordPersonId)

    def __nonzero__(self):
        return self._clientId is not None

