# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import pyqtSignature, QDate, QDateTime

from library.DialogBase import CDialogBase
from library.Utils      import firstMonthDay, forceInt, forceRef, forceString, formatName, lastMonthDay, forceDateTime, disassembleSeconds
from Orgs.Utils         import getOrgStructureDescendants, getOrgStructureFullName, getPersonInfo
from Reports.Report     import CReport
from Reports.ReportBase import CReportBase, createTable
from Reports.Utils      import dateRangeAsStr
from Reports.ReportView import CPageFormat

#from Registry.Utils     import getClientPhonesEx
from Timeline.Schedule  import CSchedule

from Reports.Ui_JournalBeforeRecordDialog  import Ui_JournalBeforeRecordDialog


def getClientSocStatuses(clientIdList):
    db = QtGui.qApp.db
    table = db.table('ClientSocStatus')
    cond  = [ table['client_id'].inlist(clientIdList),
              table['deleted'].eq(0),
              db.joinOr([table['begDate'].isNull(), table['begDate'].le(QDate.currentDate())]),
              db.joinOr([table['endDate'].isNull(), table['endDate'].ge(QDate.currentDate())]),
            ]
    return db.getRecordList(table, 'DISTINCT socStatusType_id, client_id', where=cond, order='begDate')


def formatSocStatuses(socStatuses, sstIdList, asHtml=False):
    if socStatuses:
        if QtGui.qApp.showingInInfoBlockSocStatus() == 0:
            if asHtml:
                lines = [(('<B>' + CSocStatusTypeCache.getCode(socStatusTypeId) + u'</B>') if socStatusTypeId not in sstIdList
                                                            else CSocStatusTypeCache.getCode(socStatusTypeId)) for socStatusTypeId in socStatuses]
            else:
                lines = [CSocStatusTypeCache.getCode(socStatusTypeId) for socStatusTypeId in socStatuses]
        elif QtGui.qApp.showingInInfoBlockSocStatus() == 1:
            if asHtml:
                lines = [((u'<B>' + CSocStatusTypeCache.getName(socStatusTypeId) + u'</B>') if (socStatusTypeId in sstIdList and CSocStatusTypeCache.getCode(socStatusTypeId) != u'м643')
                                                            else CSocStatusTypeCache.getName(socStatusTypeId)) for socStatusTypeId in socStatuses]
            else:
                lines = [CSocStatusTypeCache.getName(socStatusTypeId) for socStatusTypeId in socStatuses]
        else:
            if asHtml:
                lines = [((('<B>' + CSocStatusTypeCache.getCode(socStatusTypeId) + u'</B>-') if socStatusTypeId not in sstIdList
                                                            else (CSocStatusTypeCache.getCode(socStatusTypeId) + '-'))
                                + ((u'<B>' + CSocStatusTypeCache.getName(socStatusTypeId) + u'</B>') if (socStatusTypeId in sstIdList and CSocStatusTypeCache.getCode(socStatusTypeId) != u'м643')
                                                            else CSocStatusTypeCache.getName(socStatusTypeId))) for socStatusTypeId in socStatuses]
            else:
                lines = [CSocStatusTypeCache.getCode(socStatusTypeId) + '-' + CSocStatusTypeCache.getName(socStatusTypeId) for socStatusTypeId in socStatuses]
        lines.sort()
        return ', '.join(lines)
    else:
        return u'нет'


def selectData(params, AV = False):
    useRecordPeriod   = params.get('useRecordPeriod', False)
    begRecordDate     = params.get('begRecordDate', None)
    endRecordDate     = params.get('endRecordDate', None)
    useSchedulePeriod = params.get('useSchedulePeriod', False)
    begScheduleDate   = params.get('begScheduleDate', None)
    endScheduleDate   = params.get('endScheduleDate', None)
    orgStructureId    = params.get('orgStructureId', None)
    specialityId      = params.get('specialityId', None)
    personId          = params.get('personId', None)
    appointmentType   = params.get('appointmentType', None)
    recordPersonId    = params.get('recordPersonId', None)
    recordPersonProfileId  = params.get('recordPersonProfileId', None)
    detailSorted      = params.get('detailSorted', 0)
    userParams        = params.get('userParams', 0)
    appointmentPurposeId = params.get('appointmentPurposeId', None)
    socStatusClassId  = params.get('socStatusClassId', None)
    socStatusTypeId   = params.get('socStatusTypeId', None)

    db = QtGui.qApp.db
    tablePerson = db.table('vrbPersonWithSpeciality').alias('P')
    tableSchedule = db.table('Schedule')
    tableScheduleItem = db.table('Schedule_Item')
    tableRecordPerson = db.table('vrbPersonWithSpeciality').alias('RP')

    cond = [ tableSchedule['deleted'].eq(0),
             tableSchedule['appointmentType'].eq(appointmentType),
             tableScheduleItem['deleted'].eq(0),
             tableScheduleItem['client_id'].isNotNull(),
           ]

    if useRecordPeriod:
        if begRecordDate:
            cond.append(tableScheduleItem['recordDatetime'].ge(begRecordDate))
        if endRecordDate:
            cond.append(tableScheduleItem['recordDatetime'].lt(endRecordDate.addDays(1)))
    if useSchedulePeriod:
        if begScheduleDate:
            cond.append(tableSchedule['date'].ge(begScheduleDate))
        if endScheduleDate:
            cond.append(tableSchedule['date'].le(endScheduleDate))
    if personId:
        cond.append(tableSchedule['person_id'].eq(personId))
    if specialityId:
        cond.append(tablePerson['speciality_id'].eq(specialityId))
    if orgStructureId:
        cond.append(tablePerson['orgStructure_id'].inlist(getOrgStructureDescendants(orgStructureId)))
    else:
        if not personId:
            cond.append(tablePerson['org_id'].eq(QtGui.qApp.currentOrgId()))
    if recordPersonId:
        cond.append(tableScheduleItem['recordPerson_id'].eq(recordPersonId))
    if recordPersonProfileId:
        cond.append(tableRecordPerson['userProfile_id'].eq(recordPersonProfileId))
    joinAP = u''
    if detailSorted == 3 or appointmentPurposeId:
        joinAP = u'''LEFT JOIN rbAppointmentPurpose ON rbAppointmentPurpose.id = Schedule.appointmentPurpose_id'''
    if appointmentPurposeId:
        cond.append(tableSchedule['appointmentPurpose_id'].eq(appointmentPurposeId))
    if detailSorted  == 1:
        sorted = ' ORDER BY Client.lastName, Client.firstName, Client.patrName'
    elif detailSorted  == 2:
        sorted = ' ORDER BY Client.birthDate'
    elif detailSorted  == 3:
        joinAP = u'''LEFT JOIN rbAppointmentPurpose ON rbAppointmentPurpose.id = Schedule.appointmentPurpose_id'''
        sorted = ' ORDER BY rbAppointmentPurpose.code, rbAppointmentPurpose.name'
    else:
        sorted = ' ORDER BY Schedule_Item.time, P.name'
    if userParams == 1:
        cond.append(tableScheduleItem['recordClass'].eq(2))
    elif userParams == 2:
        cond.append(tableScheduleItem['recordClass'].eq(3))
    if AV:
        stmtTelefone = u'''(SELECT GROUP_CONCAT( ClientContact.contact SEPARATOR ', ')
        FROM ClientContact
        LEFT JOIN rbContactType ON rbContactType.id = ClientContact.contactType_id
        WHERE ClientContact.client_id = Client.id AND ClientContact.deleted=0
         AND rbContactType.code IN ('1','2','3')
        ORDER BY ClientContact.id) AS clientPhones,'''
    else:
        stmtTelefone = u'''getClientContacts(Client.id) AS clientPhones,'''

    if socStatusTypeId:
        subStmt = ('SELECT ClientSocStatus.id FROM ClientSocStatus WHERE '
                  +'ClientSocStatus.deleted=0 AND ClientSocStatus.client_id=Client.id AND '
                  +'ClientSocStatus.socStatusType_id=%d' % socStatusTypeId)
        cond.append('EXISTS('+subStmt+')')
    elif socStatusClassId:
        subStmt = ('SELECT ClientSocStatus.id FROM ClientSocStatus WHERE '
                  +'ClientSocStatus.deleted=0 AND ClientSocStatus.client_id=Client.id AND '
                  +'ClientSocStatus.socStatusClass_id=%d' % socStatusClassId)
        cond.append('EXISTS('+subStmt+')')

    stmt =  u'''SELECT
             Schedule_Item.recordDatetime,
             Schedule_Item.recordClass,
             Schedule_Item.recordPerson_id AS recordPerson_id,
             IF(Schedule_Item.recordClass = 2, Schedule_Item.note, RP.name) AS recordPersonName,
             Client.lastName, Client.firstName, Client.patrName, Client.birthDate, Client.id AS clientId,
             %s
             getClientRegAddress(Client.id) AS clientAddress,
             rbPolicyType.name AS policyType, ClientPolicy.serial, ClientPolicy.number,
             Schedule_Item.time AS scheduleDatetime,
             P.name AS personName
            FROM
             Schedule_Item
             LEFT JOIN Schedule     ON Schedule.id = Schedule_Item.master_id
             LEFT JOIN vrbPersonWithSpeciality AS P  ON P.id = Schedule.person_id
             LEFT JOIN vrbPersonWithSpeciality AS RP ON RP.id = Schedule_Item.recordPerson_id
             LEFT JOIN Client       ON Client.id = Schedule_Item.client_id
             LEFT JOIN ClientPolicy ON ClientPolicy.client_id = Client.id
                                      AND ClientPolicy.id = (SELECT MAX(CP.id) FROM ClientPolicy AS CP WHERE  CP.client_id = Client.id AND CP.deleted=0)
             LEFT JOIN rbPolicyType ON rbPolicyType.id = ClientPolicy.policyType_id
             %s
             WHERE %s
            %s''' % (stmtTelefone, joinAP, db.joinAnd(cond),  sorted)
    return db.query(stmt)


class CJournalBeforeRecord(CReport):
    def __init__(self, parent):
        CReport.__init__(self, parent)
        self.setTitle(u'Журнал предварительной записи')
        self.doctorsDict = {}
        self.callCentrDict = {}
        self.pageFormat = CPageFormat(pageSize=CPageFormat.A4, orientation=CPageFormat.Landscape, leftMargin=1, topMargin=1, rightMargin=1,  bottomMargin=1)


    def dumpParams(self, cursor, params):
        db = QtGui.qApp.db
        useRecordPeriod   = params.get('useRecordPeriod', False)
        begRecordDate     = params.get('begRecordDate', None)
        endRecordDate     = params.get('endRecordDate', None)
        useSchedulePeriod = params.get('useSchedulePeriod', False)
        begScheduleDate   = params.get('begScheduleDate', None)
        endScheduleDate   = params.get('endScheduleDate', None)
        orgStructureId    = params.get('orgStructureId', None)
        specialityId      = params.get('specialityId', None)
        personId          = params.get('personId', None)
        appointmentType   = params.get('appointmentType', None)
        recordPersonId    = params.get('recordPersonId', None)
        recordPersonProfileId  = params.get('recordPersonProfileId', None)
        detailCallCenter  = params.get('detailCallCenter', False)
        appointmentPurposeId = params.get('appointmentPurposeId', None)
        socStatusClassId  = params.get('socStatusClassId', None)
        socStatusTypeId   = params.get('socStatusTypeId', None)

        description = []
        if useRecordPeriod:
            if begRecordDate or endRecordDate:
                description.append(dateRangeAsStr(u'за период', begRecordDate, endRecordDate))
        if useSchedulePeriod:
            if begScheduleDate or endScheduleDate:
                description.append(dateRangeAsStr(u'период предварительной записи', begScheduleDate, endScheduleDate))
        if orgStructureId:
            description.append(u'подразделение: ' + getOrgStructureFullName(orgStructureId))
        else:
            description.append(u'подразделение: ЛПУ')
        if specialityId:
            description.append(u'специальность: ' + forceString(db.translate('rbSpeciality', 'id', specialityId, 'name')))
        if personId:
            personInfo = getPersonInfo(personId)
            description.append(u'врач: ' + personInfo['shortName']+', '+personInfo['specialityName'])
        if appointmentPurposeId:
            description.append(u'назначение: ' + forceString(db.translate('rbAppointmentPurpose', 'id', appointmentPurposeId, 'name')))
        if appointmentType:
            description.append(u'тип приёма: '+CSchedule.atNames[appointmentType])
        if socStatusTypeId:
            description.append(u'Тип соц.статуса: ' + forceString(db.translate('vrbSocStatusType', 'id', socStatusTypeId, 'name')))
        if socStatusClassId:
            description.append(u'Класс соц.статуса: ' + forceString(db.translate('rbSocStatusClass', 'id', socStatusClassId, 'name')))
        if recordPersonId:
            personInfo = getPersonInfo(recordPersonId)
            description.append(u'пользователь: ' + personInfo['shortName'])
        if recordPersonProfileId:
            description.append(u'профиль прав пользователя: ' + forceString(db.translate('rbUserProfile', 'id', recordPersonProfileId, 'name')))
        if detailCallCenter:
            description.append(u'Детализировать сall-центр')

        description.append(u'отчёт составлен: '+forceString(QDateTime.currentDateTime()))
        columns = [ ('100%', [], CReportBase.AlignLeft) ]
        table = createTable(cursor, columns, headerRowCount=len(description), border=0, cellPadding=2, cellSpacing=0)
        for i, row in enumerate(description):
            table.setText(i, 0, row)
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertBlock()


    def build(self, params):
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.title())
        cursor.insertBlock()
        self.dumpParams(cursor, params)
        cursor.setCharFormat(CReportBase.ReportBody)
        cursor.insertBlock()
        detailCallCenter = params.get('detailCallCenter', False)
        tableColumns = [( '2%', [u'№'],             CReportBase.AlignLeft),
                        ( '7%', [u'Дата записи'],   CReportBase.AlignLeft),
                        (' 9%', [u'Пользователь'],  CReportBase.AlignLeft),
                        (' 9%', [u'ФИО'],           CReportBase.AlignLeft),
                        ( '8%', [u'Д/р'],           CReportBase.AlignLeft),
                        ( '8%', [u'Полис'],         CReportBase.AlignLeft),
                        ( '8%', [u'Статус'],        CReportBase.AlignLeft),
                        ('14%', [u'Адрес'],         CReportBase.AlignLeft),
                        ( '7%', [u'Телефон'],       CReportBase.AlignLeft),
                        ( '8%', [u'Дата приема'],   CReportBase.AlignLeft),
                        ('10%', [u'Врач'],          CReportBase.AlignLeft),
                        ('10%', [u'Время ожидания записи в днях'], CReportBase.AlignLeft),
                        ]
        table = createTable(cursor, tableColumns)
        query = selectData(params)
        db = QtGui.qApp.db
        tableSSC = db.table('rbSocStatusClass')
        tableSSCT = db.table('rbSocStatusClassTypeAssoc')
        queryTable = tableSSC.innerJoin(tableSSCT, tableSSCT['class_id'].eq(tableSSC['id']))
        sstIdList = db.getDistinctIdList(queryTable, [tableSSCT['type_id']], [tableSSC['code'].eq('8')])
        n = 1
        clientIdList = []
        reportLines = []
        while query.next():
            reportLine = [u'']*11
            record = query.record()
            reportLine[0] = forceString(record.value('recordDatetime'))
            recordClass  =  forceInt(record.value('recordClass'))
            if recordClass == 1: # Инфомат
                recordPersonName = u'Инфомат'
            elif recordClass == 2: # call-центр
                if detailCallCenter:
                    recordPersonName = u'Call-центр ' + forceString(record.value('recordPersonName'))
                else:
                    recordPersonName = u'Call-центр'
            elif recordClass == 3: # интернет
                recordPersonName = u'Интернет'
            else: #
                if forceRef(record.value('recordPerson_id')):
                    recordPersonName = forceString(record.value('recordPersonName'))
                else:
                    recordPersonName = 'demo'
            reportLine[1] = recordPersonName
            reportLine[2] =    formatName(record.value('lastName'),
                                          record.value('firstName'),
                                          record.value('patrName')
                                         )+ u', ' + forceString(record.value('clientId'))
            reportLine[3] = forceString(record.value('birthDate'))
            reportLine[4] = ' '.join(name for name in (forceString(record.value('policyType')),
                                                          forceString(record.value('serial')),
                                                          forceString(record.value('number'))
                                                         )
                                       )
            clientId = forceRef(record.value('clientId'))
            if clientId and clientId not in clientIdList:
                clientIdList.append(clientId)
            reportLine[5] = clientId
            reportLine[6] = forceString(record.value('clientAddress'))
            reportLine[7] = forceString(record.value('clientPhones'))
            reportLine[8] = forceString(record.value('scheduleDatetime'))
            reportLine[9] = forceString(record.value('personName'))
            reportLine[10] = disassembleSeconds(forceDateTime(record.value('scheduleDatetime')).secsTo(forceDateTime(record.value('recordDatetime'))))
            reportLines.append(reportLine)
        clientSocStatuses = {}
        if clientIdList:
            records = getClientSocStatuses(clientIdList)
            for clientSocStatus in records:
                clientIdStatus = forceRef(clientSocStatus.value('client_id'))
                clientSocStatusLine = clientSocStatuses.get(clientIdStatus, [])
                socStatusTypeId = forceRef(clientSocStatus.value('socStatusType_id'))
                if socStatusTypeId and socStatusTypeId not in clientSocStatusLine:
                    clientSocStatusLine.append(socStatusTypeId)
                clientSocStatuses[clientIdStatus] = clientSocStatusLine
        for reportLine in reportLines:
            i = table.addRow()
            table.setText(i, 0, n)
            table.setText(i, 1, reportLine[0])
            table.setText(i, 2, reportLine[1])
            table.setText(i, 3, reportLine[2])
            table.setText(i, 4, reportLine[3])
            table.setText(i, 5, reportLine[4])
            table.setText(i, 6, formatSocStatuses(clientSocStatuses.get(reportLine[5]), sstIdList))
            table.setText(i, 7, reportLine[6])
            table.setText(i, 8, reportLine[7])
            table.setText(i, 9, reportLine[8])   #daysTo
            table.setText(i, 10, reportLine[9])
            table.setText(i, 11, reportLine[10])
            n += 1
        return doc


class CJournalBeforeRecordEx(CJournalBeforeRecord):
    def exec_(self):
        CJournalBeforeRecord.exec_(self)


    def getSetupDialog(self, parent):
        result = CJournalBeforeRecordDialog(parent)
        return result


    def build(self, params):
        return CJournalBeforeRecord.build(self, params)


class CJournalBeforeRecordAV(CJournalBeforeRecord):
    def __init__(self, parent):
        CJournalBeforeRecord.__init__(self, parent)
        self.setTitle(u'Журнал предварительной записи (сокращённый вариант)')


    def exec_(self):
        CJournalBeforeRecord.exec_(self)


    def getSetupDialog(self, parent):
        result = CJournalBeforeRecordDialog(parent)
        return result


    def build(self, params):
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.title())
        cursor.insertBlock()
        self.dumpParams(cursor, params)
        cursor.setCharFormat(CReportBase.ReportBody)
        cursor.insertBlock()
        detailCallCenter = params.get('detailCallCenter', False)
        tableColumns = [
                        ('45%', [u'ФИО'],           CReportBase.AlignLeft),
                        ('10%', [u'Код'],           CReportBase.AlignLeft),
                        ('20%', [u'Телефон'],       CReportBase.AlignLeft),
                        ('25%', [u'Дата приема'],   CReportBase.AlignLeft),
                        ]
        table = createTable(cursor, tableColumns)
        query = selectData(params, AV = True)
        while query.next():
            record = query.record()
            i = table.addRow()
            clientName       = formatName(record.value('lastName'),
                                          record.value('firstName'),
                                          record.value('patrName')
                                         )# + u', ' + forceString(record.value('clientId'))
            clientPhones     = forceString(record.value('clientPhones'))
            scheduleDatetime = forceString(record.value('scheduleDatetime'))

            table.setText(i, 0, clientName)
            table.setText(i, 1, forceString(record.value('clientId')))
            table.setText(i, 2, clientPhones)
            table.setText(i, 3, scheduleDatetime)
        return doc


class CJournalBeforeRecordDialog(CDialogBase, Ui_JournalBeforeRecordDialog):
    def __init__(self, parent=None):
        CDialogBase.__init__(self, parent)
        self.setupUi(self)
        self.cmbRecordPersonProfile.setTable('rbUserProfile')
        self.cmbSpeciality.setTable('rbSpeciality', True)
        self.cmbAppointmentPurpose.setTable('rbAppointmentPurpose', True)
        self.cmbOrgStructure.setOrgId(QtGui.qApp.currentOrgId())
        self.cmbOrgStructure.setValue(QtGui.qApp.currentOrgStructureId())
        self.cmbRecordPerson.setSpecialityPresent(False)
        self.cmbSocStatusType.setTable('vrbSocStatusType', True)


    def setParams(self, params):
        date = QDate.currentDate().addDays(-3)
        self.chkRecordPeriod.setChecked(params.get('useRecordPeriod', True))
        self.edtBegRecordDate.setDate(params.get('begRecordDate', firstMonthDay(date)))
        self.edtEndRecordDate.setDate(params.get('endRecordDate', lastMonthDay(date)))
        self.chkSchedulePeriod.setChecked(params.get('useSchedulePeriod', False))
        self.edtBegScheduleDate.setDate(params.get('begScheduleDate', firstMonthDay(date)))
        self.edtEndScheduleDate.setDate(params.get('endScheduleDate', lastMonthDay(date)))
        self.cmbOrgStructure.setValue(params.get('orgStructureId', None))
        self.cmbSpeciality.setValue(params.get('specialityId', None))
        self.cmbPerson.setValue(params.get('personId', None))
        self.cmbAppointmentType.setCurrentIndex(0 if params.get('appointmentType', CSchedule.atAmbulance)  == CSchedule.atAmbulance else 1)
        self.cmbRecordPerson.setValue(params.get('recordPersonId', None))
        self.cmbRecordPersonProfile.setValue(params.get('recordPersonProfileId', None))
        self.chkDetailCallCenter.setChecked(params.get('detailCallCenter', False))
        self.cmbSorted.setCurrentIndex(params.get('detailSorted', 0))
        self.cmbUserParams.setCurrentIndex(params.get('userParams', 0))
        self.cmbAppointmentPurpose.setValue(params.get('appointmentPurposeId', None))
        self.cmbSocStatusClass.setValue(params.get('socStatusClassId', None))
        self.cmbSocStatusType.setValue(params.get('socStatusTypeId', None))


    def params(self):
        return dict(useRecordPeriod     = self.chkRecordPeriod.isChecked(),
                    begRecordDate       = self.edtBegRecordDate.date(),
                    endRecordDate       = self.edtEndRecordDate.date(),
                    useSchedulePeriod   = self.chkSchedulePeriod.isChecked(),
                    begScheduleDate     = self.edtBegScheduleDate.date(),
                    endScheduleDate     = self.edtEndScheduleDate.date(),
                    orgStructureId      = self.cmbOrgStructure.value(),
                    specialityId        = self.cmbSpeciality.value(),
                    personId            = self.cmbPerson.value(),
                    appointmentType     = (CSchedule.atAmbulance, CSchedule.atHome)[self.cmbAppointmentType.currentIndex()],
                    recordPersonId      = self.cmbRecordPerson.value(),
                    recordPersonProfile = self.cmbRecordPersonProfile.value(),
                    detailCallCenter    = self.chkDetailCallCenter.isChecked(),
                    detailSorted        = self.cmbSorted.currentIndex(),
                    userParams          = self.cmbUserParams.currentIndex(),
                    appointmentPurposeId = self.cmbAppointmentPurpose.value(),
                    socStatusClassId    = self.cmbSocStatusClass.value(),
                    socStatusTypeId     = self.cmbSocStatusType.value()
                   )


    @pyqtSignature('int')
    def on_cmbSocStatusClass_currentIndexChanged(self, index):
        socStatusClassId = self.cmbSocStatusClass.value()
        filter = ('class_id=%d' % socStatusClassId) if socStatusClassId else ''
        self.cmbSocStatusType.setFilter(filter)


    @pyqtSignature('QDate')
    def on_edtBegDate_dateChanged(self, date):
        self.cmbPerson.setBegDate(date)
        self.cmbRecordPerson.setBegDate(date)


    @pyqtSignature('QDate')
    def on_edtEndDate_dateChanged(self, date):
        self.cmbPerson.setEndDate(date)
        self.cmbRecordPerson.setEndDate(date)


    @pyqtSignature('int')
    def on_cmbOrgStructure_currentIndexChanged(self, index):
        orgStructureId = self.cmbOrgStructure.value()
        self.cmbPerson.setOrgStructureId(orgStructureId)


    @pyqtSignature('int')
    def on_cmbSpeciality_currentIndexChanged(self, index):
        specialityId = self.cmbSpeciality.value()
        self.cmbPerson.setSpecialityId(specialityId)

