# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import codecs
import os.path
import re

import sip
from PyQt4 import QtGui
from PyQt4.QtCore import Qt, pyqtSignature, QByteArray, QFileInfo, QSizeF, QString, QUrl

from library.Utils import forceBool

from Users.Rights import urAdmin, urEditReportForm

from Reports.Ui_ReportView import Ui_ReportViewDialog


class CPageFormat(object):
    # page size
    A2 = QtGui.QPrinter.A2
    A3 = QtGui.QPrinter.A3
    A4 = QtGui.QPrinter.A4
    A5 = QtGui.QPrinter.A5
    A6 = QtGui.QPrinter.A6
    # page orientation
    Portrait = QtGui.QPrinter.Portrait
    Landscape = QtGui.QPrinter.Landscape
    #
    validPageSizes = { A2: A2, 'A2': A2,
                       A3: A3, 'A3': A3,
                       A4: A4, 'A4': A4,
                       A5: A5, 'A5': A5,
                       A6: A6, 'A6': A6,
                     }

    paperSizes =     { A2: (420, 594),
                       A3: (297, 420),
                       A4: (210, 297),
                       A5: (148, 210),
                       A6: (105, 148),
                     }

    validOrientations = { Portrait: Portrait,  'PORTRAIT': Portrait,  'P':Portrait,
                          Landscape:Landscape, 'LANDSCAPE':Landscape, 'L':Landscape
                     }


    @classmethod
    def getPaperRect(cls, paperSize):
        width, hieght = cls.paperSizes.get(paperSize, cls.paperSizes[cls.A4])
        return QSizeF(width, hieght)


    def __init__(self, pageSize=QtGui.QPrinter.A4,
                        orientation=QtGui.QPrinter.Portrait,
                        leftMargin=10,
                        topMargin=10,
                        rightMargin=10,
                        bottomMargin=10
                ):
        self.pageSize = pageSize
        self.pageRect = None # для custom size
        self.orientation = orientation
        self.leftMargin = leftMargin
        self.topMargin = topMargin
        self.rightMargin = rightMargin
        self.bottomMargin = bottomMargin


    def setupPrinter(self, printer):
        printerInfo = QtGui.QPrinterInfo(printer)
        if self.pageSize == QtGui.QPrinter.Custom:
            printer.setPaperSize(self.pageRect, QtGui.QPrinter.Millimeter)
        elif self.pageSize in printerInfo.supportedPaperSizes():
            printer.setPaperSize(self.pageSize)
        else:
            paperRect = self.getPaperRect(self.pageSize)
            printer.setPaperSize(paperRect, QtGui.QPrinter.Millimeter)
        printer.setOrientation(self.orientation)


    def updateFromPrinter(self, printer):
        self.pageSize = printer.pageSize()
        if self.pageSize == QtGui.QPrinter.Custom:
            self.pageRect = printer.paperSize(QtGui.QPrinter.Millimeter)
        else:
            self.pageRect = None
        self.orientation = printer.orientation()


    def setPageSize(self, size):
        if isinstance(size, basestring):
            size = size.upper().strip()
            customSize = re.match(r'^(\d+)\s*[xX]\s*(\d+)$', size)
            if customSize:
                self.pageSize = QtGui.QPrinter.Custom
                sizes = customSize.groups()
                self.pageRect = QSizeF(float(sizes[0]), float(sizes[1]))
                return ''
        validPageSize = self.validPageSizes.get(size, None)
        if validPageSize is not None:
            self.pageSize = validPageSize
            self.pageRect = None
            return ''
        else:
            return u'[Invalid page size "%s"]' % size


    def setOrientation(self, orientation):
        if isinstance(orientation, basestring):
            orientation = orientation.upper().strip()
        validOrientation = self.validOrientations.get(orientation, None)
        if validOrientation is not None:
            self.orientation = validOrientation
            return ''
        else:
            return u'[Invalid orientation "%s"]' % orientation


    def setMargins(self, margin):
        if isinstance(margin, (int, float)):
            self.leftMargin = margin
            self.topMargin = margin
            self.rightMargin = margin
            self.bottomMargin = margin
            return ''
        else:
            return u'[Invalid margin "%s"]' % margin


    def setLeftMargin(self, margin):
        if isinstance(margin, (int, float)):
            self.leftMargin = margin
            return ''
        else:
            return u'[Invalid left margin "%s"]' % margin


    def setTopMargin(self, margin):
        if isinstance(margin, (int, float)):
            self.topMargin = margin
            return ''
        else:
            return u'[Invalid top margin "%s"]' % margin


    def setRightMargin(self, margin):
        if isinstance(margin, (int, float)):
            self.rightMargin = margin
            return ''
        else:
            return u'[Invalid right margin "%s"]' % margin


    def setBottomMargin(self, margin):
        if isinstance(margin, (int, float)):
            self.bottomMargin = margin
            return ''
        else:
            return u'[Invalid bottom margin "%s"]' % margin


def odtAvailable():
    try:
        if 'QTextDocumentWriter' not in QtGui.__dict__:
            return False
        return 'ODF' in QtGui.QTextDocumentWriter.supportedDocumentFormats()
    except:
        pass
    return False


class CReportViewDialog(QtGui.QDialog, Ui_ReportViewDialog):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)

        self.btnPrint = QtGui.QPushButton(u'Печатать', self)
        self.btnPrint.setObjectName('btnPrint')
        self.btnPrint.setDefault(True)

        self.btnEMail = QtGui.QPushButton(u'Отправить', self)
        self.btnEMail.setObjectName('btnEMail')
        self.btnEMail.setEnabled(False)

        self.btnEdit = QtGui.QPushButton(u'Редактировать', self)
        self.btnEdit.setObjectName('btnEdit')
        self.btnEdit.setEnabled((QtGui.qApp.userHasRight(urEditReportForm) or QtGui.qApp.userHasRight(urAdmin)) and bool(QtGui.qApp.documentEditor()))

        self.setupUi(self)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)
        self.buttonBox.addButton(self.btnEMail, QtGui.QDialogButtonBox.ActionRole)
        self.buttonBox.addButton(self.btnEdit, QtGui.QDialogButtonBox.ActionRole)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Close|QtGui.QDialogButtonBox.Save)
        self.fileName = ''
        self.pageFormat = None
        self.orientation = QtGui.QPrinter.Portrait
        self.actionButtons = []
        self.actions = []

#        if not QtGui.QPrinterInfo.availablePrinters():
#            self.btnPrint.setEnabled(False)
        self.btnPrint.setFocus(Qt.OtherFocusReason)


    def setWindowTitle(self, title):
        QtGui.QDialog.setWindowTitle(self, title)
        for x in ':?|()[]\"\';!*/\\':
            title = title.replace(x, '')
        self.fileName = title


    def setOrientation(self, orientation):
        self.orientation = orientation


    def setActions(self, actions):
        for action in actions:
            button = QtGui.QPushButton(action.getName(), self)
            button.setEnabled(action.isEnabled())
            self.buttonBox.addButton(button, QtGui.QDialogButtonBox.ActionRole)
            self.actionButtons.append(button)
            self.actions.append(action)


    def setRepeatButtonVisible(self, value=True):
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Close|QtGui.QDialogButtonBox.Save|QtGui.QDialogButtonBox.Retry)


    def setText(self, reportResult):
        if isinstance(reportResult, (list, tuple)):
            text = reportResult[0]
            actions = reportResult[1:]
        else:
            text = reportResult
            actions = []

        if actions:
            self.setActions(actions)

        if isinstance(text, QtGui.QTextDocument):
            self.txtReport.setDocument(text.clone(self.txtReport))
#            text.setParent(self.txtReport)
        else:
            self.txtReport.setHtml(text)


    def setCanvases(self, canvases):
        self.txtReport.setCanvases(canvases)


    def setPageFormat(self, format):
        self.pageFormat = format


    def saveAsFile(self):
        defaultFileName = QtGui.qApp.getSaveDir()
        if self.fileName:
            defaultFileName = os.path.join(defaultFileName, self.fileName)

        saveFormats = [u'Веб-страница (*.html;*.htm)',
                       u'Portable Document Format (*.pdf)',
                       u'PostScript (*.ps)',
                      ]
        if odtAvailable():
            saveFormats.insert(-1, u'Текстовый документ OpenOffice.org (*.odt)')
        selectedFilter = QString('')

        fileName = QtGui.QFileDialog.getSaveFileName(
            self,
            u'Выберите имя файла',
            defaultFileName,
            ';;'.join(saveFormats),
            selectedFilter)
        if not fileName.isEmpty():
            exts = selectedFilter.section('(*.',1,1).section(')',0,0).split(';*.')
            ext = QFileInfo(fileName).suffix()
            if exts and not exts.contains(ext, Qt.CaseInsensitive):
                ext = exts[0]
                fileName.append('.'+ext)
            fileName = unicode(fileName)
            ext = unicode(ext)
            if ext.lower() == 'pdf':
                self.saveAsPdfOrPS(fileName, QtGui.QPrinter.PdfFormat)
            elif ext.lower() == 'ps':
                self.saveAsPdfOrPS(fileName, QtGui.QPrinter.PostScriptFormat)
            elif ext.lower() == 'odt':
                self.saveAsOdt(fileName)
            else:
                self.saveAsHtml(fileName)


    def saveAsHtml(self, fileName):
        QtGui.qApp.setSaveDir(fileName)
        baseDir, name = os.path.split(fileName)
        imageRelativePath = name.split('.', 1)[0]+'.files'
        imagesPath = os.path.join(baseDir, imageRelativePath)
        imagesDirCreated = False
        imageCounter = 0
        textDocument = self.txtReport.document()
        outTextDocument = textDocument.clone(textDocument)
        try:
            block = outTextDocument.begin()
            while block != outTextDocument.end():
                it = block.begin()
                while not it.atEnd():
                    fragment = it.fragment()
                    if fragment.isValid():
                        fm = fragment.charFormat().toImageFormat()
                        if fm.isValid() and not fm.name().isEmpty():
                            if not imagesDirCreated:
                                try:
                                    os.mkdir(imagesPath)
                                except:
                                    pass
                                imagesDirCreated = True
                            image = textDocument.resource(QtGui.QTextDocument.ImageResource, QUrl.fromEncoded(fm.name().toUtf8()))
                            imageFileName = imagesPath + '/'+str(imageCounter)+'.png'
                            writer = QtGui.QImageWriter(imageFileName, 'png')
                            writer.write(QtGui.QImage(image))
                            fm.setName('./'+imageRelativePath+'/'+str(imageCounter)+'.png');
                            cursor = QtGui.QTextCursor(outTextDocument)
                            cursor.setPosition(fragment.position())
                            cursor.setPosition(fragment.position() + fragment.length(), QtGui.QTextCursor.KeepAnchor);
                            cursor.setCharFormat(fm)
                            imageCounter+=1
                    it+=1
                block = block.next()
            txt = outTextDocument.toHtml(QByteArray('utf-8'))
            file = codecs.open(unicode(fileName), encoding='utf-8', mode='w+')
            file.write(unicode(txt))
            file.close()
        finally:
            sip.delete(outTextDocument)


    def saveAsOdt(self, fileName):
        writer = QtGui.QTextDocumentWriter();
        try:
            QtGui.qApp.setSaveDir(fileName)
            writer.setFormat('odf')
            writer.setFileName(fileName)
            writer.write(self.txtReport.document())
        finally:
            writer = None
            # sip.delete(writer)
            pass


    def setupPage(self, printer):
        printer.setOrientation(self.orientation)
        if self.pageFormat:
            self.pageFormat.setupPrinter(printer)


    def saveAsPdfOrPS(self, fileName, format):
        QtGui.qApp.setSaveDir(fileName)
        printer = QtGui.QPrinter(QtGui.QPrinter.HighResolution)
        printer.setOutputFormat(format)
        printer.setOutputFileName(fileName)
        self.setupPage(printer)
        self.printReport(printer)


    def printReport(self, printer):
        printTextDocument(self.txtReport.document(), self.fileName, self.pageFormat, printer)


    @pyqtSignature('')
    def on_btnPrint_clicked(self):
        defaultPrinterInfo = QtGui.QPrinterInfo.defaultPrinter()
        if not defaultPrinterInfo.isNull():
            printer = QtGui.QPrinter(defaultPrinterInfo, QtGui.QPrinter.HighResolution)
        else:
            printer = QtGui.QPrinter(QtGui.QPrinter.HighResolution)
        printer.setPrintRange(printer.AllPages)
        printer.setPaperSize(printer.A4)
        printer.setFullPage(False)
        if hasattr(printer, 'setNumCopies'):
            printer.setNumCopies(1)
        if hasattr(printer, 'setCopyCount'):
            printer.setCopyCount(1)
        self.setupPage(printer)
        if (defaultPrinterInfo.isNull() or (QtGui.qApp.keyboardModifiers() & Qt.ControlModifier)) or not QtGui.qApp.enableFastPrint():
            dialog = QtGui.QPrintDialog(printer, self)
            if dialog.exec_() != QtGui.QDialog.Accepted:
                return
        self.printReport(printer)


    @pyqtSignature('')
    def on_btnEMail_clicked(self):
        pass


    @pyqtSignature('')
    def on_btnEdit_clicked(self):
        editor = QtGui.qApp.documentEditor()
        if editor is not None:
            tmpDir = QtGui.qApp.getTmpDir('reportEdit')
            tmpFileName = os.path.join(tmpDir, 'report.html')
            self.saveAsHtml(tmpFileName)
            cmdLine = u'"%s" "%s"'% (editor, tmpFileName)
            prg=QtGui.qApp.execProgram(cmdLine)
            if not prg[0]:
                QtGui.QMessageBox.critical(self,
                                       u'Внимание!',
                                       u'Не удалось запустить "%s"' % editor,
                                       QtGui.QMessageBox.Close)
            if prg[2]:
                QtGui.QMessageBox.critical(self,
                                       u'Внимание!',
                                       u'Ошибка при выполнении "%s"' % editor,
                                       QtGui.QMessageBox.Close)
            QtGui.qApp.removeTmpDir(tmpDir)
        else:
            QtGui.QMessageBox.critical(self,
                                    u'Ошибка!',
                                    u'Не указан исполняемый файл редактора документов\n'+
                                    u'Смотрите пункт меню "Настройки.Умолчания", закладка "Прочие настройки",\n'+
                                    u'строка "Внешний редактор документов".',
                                    QtGui.QMessageBox.Close)


    @pyqtSignature('QAbstractButton*')
    def on_buttonBox_clicked(self, button):
        buttonCode = self.buttonBox.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Close:
            self.reject()
        elif buttonCode == QtGui.QDialogButtonBox.Retry:
            self.accept()
        elif buttonCode == QtGui.QDialogButtonBox.Save:
            QtGui.qApp.call(self, self.saveAsFile)
        else:
            if button in self.actionButtons:
                i = self.actionButtons.index(button)
                self.actions[i].exec_(self)


def printTextDocument(document, documentName, pageFormat, printer):
    def uncolorCharFormat(charFormat):
        foreground = charFormat.foreground()
        background = charFormat.background()
        foreground.setColor(QtGui.QColor(0, 0, 0))
        background.setColor(QtGui.QColor(255, 255, 255))
        charFormat.setForeground(foreground)
        charFormat.setBackground(background)

    def uncolorDocument(document):
        cursor = QtGui.QTextCursor(document)
        while not cursor.atEnd(): # заменяем все заливки на белые, а все тексты - на чёрные
            charFormat = cursor.blockCharFormat()
            uncolorCharFormat(charFormat)
            cursor.setBlockCharFormat(charFormat)
            while not cursor.atBlockEnd(): # заменяем все заливки на белые, а все тексты - на чёрные
                cursor.movePosition(QtGui.QTextCursor.NextCharacter, QtGui.QTextCursor.KeepAnchor)
                charFormat = cursor.charFormat()
                uncolorCharFormat(charFormat)
                cursor.setCharFormat(charFormat)
                cursor.clearSelection()
            cursor.movePosition(QtGui.QTextCursor.NextBlock)

    printer.setCreator(u'САМСОН')
    printer.setDocName(documentName)
    outDocument = document.clone(document)
    if forceBool(QtGui.qApp.preferences.appPrefs.get('printBlackWhite', True)):
        uncolorDocument(outDocument)
    if pageFormat:
        pageFormat.updateFromPrinter(printer)
        pd = document.documentLayout().paintDevice()
        if pd is None:
            pd = QtGui.qApp.desktop()
            document.documentLayout().setPaintDevice(pd)
        pd_logicalDpiX = pd.logicalDpiX()
        pd_logicalDpiY = pd.logicalDpiY()
        p_logicalDpiX = printer.logicalDpiX()
        p_logicalDpiY = printer.logicalDpiY()

        pageRect = printer.pageRect() # in pixels
        paperRect = printer.paperRect() # in pixels

        # hardware defined margins, in printer pixels
        hl = (pageRect.left()   - paperRect.left())
        ht = (pageRect.top()    - paperRect.top())
        hr = (paperRect.right() - pageRect.right())
        hb = (paperRect.bottom() -pageRect.bottom())

        # software defined margins, in printer pixels
        sl = pageFormat.leftMargin * p_logicalDpiX / 25.4 # 25.4 mm = 1 inch
        st = pageFormat.topMargin * p_logicalDpiY / 25.4
        sr = pageFormat.rightMargin * p_logicalDpiX / 25.4
        sb = pageFormat.bottomMargin * p_logicalDpiY / 25.4

        # margins
        ml = max(0, sl-hl)
        mt = max(0, st-ht)
        mr = max(0, sr-hr)
        mb = max(0, sb-hb)

        fmt = outDocument.rootFrame().frameFormat()
        fmt.setLeftMargin(ml / p_logicalDpiX * pd_logicalDpiX) #Sets the frame's left margin in in some parrots (screen pixels?)
        fmt.setTopMargin(mt/ p_logicalDpiY * pd_logicalDpiY)
        fmt.setRightMargin(mr / p_logicalDpiX * pd_logicalDpiX)
        fmt.setBottomMargin(mb / p_logicalDpiY * pd_logicalDpiY)
        outDocument.rootFrame().setFrameFormat(fmt)
        # Calculate page width and height, in screen pixels
        pw = float(pageRect.width()) / p_logicalDpiX * pd_logicalDpiX
        ph = float(pageRect.height()) / p_logicalDpiY * pd_logicalDpiY
        # setup page size
        outDocument.setPageSize(QSizeF(pw, ph))
    outDocument.print_(printer)
    sip.delete(outDocument)
