# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui, QtCore

from Events.Action import CAction

from library.Utils import forceRef, forceInt, forceDate, forceDateTime, toVariant


class AddingInfo(object):
    def __init__(self, addingDate, duration, amount, financeId, contractId, directionDate):
        self.addingDate = addingDate
        self.duration = duration
        self.amount = amount
        self.financeId = financeId
        self.contractId = contractId
        self.directionDate = directionDate


class CJobActionsCoursesMixin(object):
    """
    Класс наследник должен обладать интерфейсом:
    getClientId() - возвращает идентификатор клиента, чьи курсовые назначения мы анализируем

    Иметь свойство _jobTypeActionsAddingHelper <CJobTypeActionsAddingHelper>
    """

    def getClientId(self):
        """
        Обязательно переопределим в наследнике
        """
        raise NotImplementedError

    def _getLoadedActions(self):
        """
        Переопределим в наследнике по необходимости
        :return list<CAction>
        """
        return []

    def _getLoadedEventIdList(self):
        """
        Переопределим в наследнике по необходимости
        :return list<int>
        """
        return []
    #
    # def _checkJobActionsExecutionPlan2(self, jobTicketId):
    #     """
    #     В рамках одного события могут проходить несколько курсов. Теоретически.
    #     Или с одной работой могут быть связаны обычные действия и действие из курса
    #     и они будут из разных событий.
    #     И только один курс по типу действия в событии
    #     """
    #
    #     db = QtGui.qApp.db
    #
    #     loadedActionList = self._getLoadedActions()
    #     eventIdList = self._getLoadedEventIdList()
    #     actionIdList = []
    #     # courses - dict[(courseMasterId, actionTypeId)] -> dict[actionBegDate] -> actionList<CAction>
    #     courses = {}
    #
    #     # Теперь соберем все основываясь на планах
    #     tableAction = db.table('Action')
    #     tableEvent = db.table('Event')
    #     tableActionExecutionPlan = db.table('Action_ExecutionPlan')
    #     queryTable = tableActionExecutionPlan.innerJoin(tableAction,
    #                                                     tableAction['id'].eq(tableActionExecutionPlan['action_id']))
    #     queryTable = queryTable.innerJoin(tableEvent, tableEvent['id'].eq(tableAction['event_id']))
    #
    #     cond = [tableAction['deleted'].eq(0),
    #             tableEvent['deleted'].eq(0),
    #             tableActionExecutionPlan['action_id'].inlist([forceRef(action.getRecord().value('id'))
    #                                                           for action in loadedActionList])]
    #
    #     # Отсортируем по типу действия, при этом первая запись среди одинаковых типов действий будет началом курса.
    #     # Дальше все сортируется по времени
    #     order = [tableAction['actionType_id'].name(),
    #              '%s DESC' % tableActionExecutionPlan['master_id'].eq(tableActionExecutionPlan['action_id']),
    #              tableActionExecutionPlan['execDate'].name()]


    def _checkJobActionsExecutionPlan(self, jobTicketId):
        """
        В рамках одного события могут проходить несколько курсов. Теоретически.
        Или с одной работой могут быть связаны обычные действия и действие из курса
        и они будут из разных событий.
        Но один курс проходит в рамках одного события
        И только один курс по типу действия в событии
        """

        db = QtGui.qApp.db

        loadedActionList = self._getLoadedActions()
        eventIdList = self._getLoadedEventIdList()
        actionIdList = []
        # courses - dict[(eventId, actionTypeId)] -> dict[actionBegDate] -> actionList<CAction>
        courses = {}

        # Сгруппируем курсовые назначения
        for action in loadedActionList:
            self._groupActions(courses, actionIdList, eventIdList, action=action)

        # Теперь соберем все остальные действия связанные с курсами
        tableAction = db.table('Action')
        tableAPJT = db.table('ActionProperty_Job_Ticket')
        tableEvent = db.table('Event')
        queryTable, cond = self._getMainQueryTableAndCond()
        cond.extend([tableAPJT['value'].isNotNull(),
                     tableAPJT['value'].ne(jobTicketId),
                     tableAction['event_id'].inlist(eventIdList),
                     tableAction['id'].notInlist(actionIdList),
                     tableAction['duration'].gt(0),
                     tableEvent['client_id'].eq(self.getClientId())])

        query = db.query(db.selectStmt(queryTable, 'Action.*', cond, order=tableAction['id'].name()))
        # Сгруппируем курсовые назначения
        while query.next():
            self._groupActions(courses, actionIdList, eventIdList, record=query.record())

        for (eventId, actionTypeId), courseData in courses.items():
            addingInfo = self._getNewCourseActionInfo(eventId, actionTypeId, courseData)
            if addingInfo is None:
                continue
            self.addNewCourseAction(addingInfo, eventId, actionTypeId)

    def _getNewCourseActionInfo(self, eventId, actionTypeId, courseData):
        result = None
        currentDate = QtCore.QDate.currentDate()

        sortedDates = sorted(courseData.keys())
        startDate = sortedDates[0]
        startDayData = courseData[startDate]
        startAction = self._getStartAction(startDayData)
        startRecord = startAction.getRecord()
        aliquoticity = forceInt(startRecord.value('aliquoticity'))
        duration = forceInt(startRecord.value('duration')) or 1
        amount = forceInt(startRecord.value('amount'))
        financeId = forceRef(startRecord.value('finance_id'))
        contractId = forceRef(startRecord.value('contract_d'))
        directionDate = forceDateTime(startRecord.value('directionDate'))

        executionPlan = startAction.getExecutionPlan()
        if not executionPlan:
            return
        executionPlanDateList = sorted(executionPlan.keys())
        lastDate = sortedDates[-1]
        if lastDate in executionPlan:
            planDayTimes = sorted(executionPlan[lastDate].keys())
            planDayCount = self._getPlanDayCount(planDayTimes, aliquoticity)
            doneDayCount = len(courseData[lastDate])
            if doneDayCount < planDayCount:
                # У нас на день lastDate еще есть запланированные направления
                if doneDayCount < len(planDayTimes) and self._checkPlanTime(planDayTimes[doneDayCount]):
                    # У нас есть запланированное конкретное время
                    result = QtCore.QDateTime(lastDate)
                    result.setTime(planDayTimes[doneDayCount])
                else:
                    # У нас не запланировано конкретное время
                    result = QtCore.QDate(lastDate)
            else:
                # У нас на день lastDate больше нет запланированных направлений
                # Ищем последующий запланированный день
                for executionDate in executionPlanDateList:
                    # Так же мы проверим не просрочили ли мы день.
                    if executionDate > lastDate and QtCore.QDate(executionDate) >= currentDate:
                        planDayTimes = sorted(executionPlan[executionDate].keys())
                        if len(planDayTimes) and self._checkPlanTime(planDayTimes[0]):
                            # У нас есть запланированное конкретное время
                            result = QtCore.QDateTime(executionDate)
                            result.setTime(planDayTimes[0])
                        else:
                            # У нас не запланировано конкретное время
                            result = QtCore.QDate(executionDate)
                        break

        return result and AddingInfo(result, duration, amount, financeId, contractId, directionDate)

    def _getStartAction(self, actionList):
        return sorted(actionList, key=lambda action: action.getRecord().value('id').toInt()[0])[0]

    def _getPlanDayCount(self, timesList, aliquoticity):
        # По хорошому, для корректной работы по такому алгоритму,
        # в таблице Action_ExecutionPlan Дату и время бы разделить в разные поля.
        # Так как в случае когда у нас не указаны в F2 из F9 конкретные времена назначений,
        # у нас получается одна запись со временем QTime(0, 0). А это для нас опасно,
        # Хорошо что хоть номерков на это время не планируют.

        timesListLen = len(timesList)
        if timesListLen == 1:
            if not self._checkPlanTime(timesList[0]):
                return aliquoticity or 1
        return timesListLen or 1


    def _checkPlanTime(self, time, zeroTime=QtCore.QTime(0, 0)):
        # По хорошому, для корректной работы по такому алгоритму,
        # в таблице Action_ExecutionPlan Дату и время бы разделить в разные поля.
        # Но пока предположим что время QTime(0, 0) - не валидное,
        # в полночь работы не выполняем

        return time and time.isValid() and time != zeroTime

    def addNewCourseAction(self, addingInfo, eventId, actionTypeId):
        # Ввиду того, что у нас связанность действий и событий,
        # нам никаких дополнительных блокировок брать не нужно.
        # По крайней мере в единственном сейчас случае использования в редакторе работы.

        # Чтобы выполнение ниже следующего когда не поламало случайно что-нибудь сохраним
        # созданное ранее helper-ом, а потом обязательно возвернем назад
        oldEventEditor = getattr(self, 'eventEditor', None)
        oldActionTemplateCache = getattr(self, 'actionTemplateCache', None)
        QtGui.qApp.setJTR(self)
        try:
            self._jobTypeActionsAddingHelper.creatEventPossibilities(eventId)

            db = QtGui.qApp.db
            tableAction = db.table('Action')
            record = tableAction.newRecord()
            newAction = CAction.getFilledAction(self.eventEditor, record, actionTypeId, addingInfo.amount,
                                                addingInfo.financeId, addingInfo.contractId, initPresetValues=False)
            record.setValue('duration', addingInfo.duration)
            record.setValue('event_id', eventId)
            record.setValue('directionDate', toVariant(addingInfo.directionDate))

            addingDate = addingInfo.addingDate
            record.setValue('begDate', forceDateTime(addingDate))
            if isinstance(addingInfo.addingDate, QtCore.QDate):
                presetValueDate, presetValueTime = addingDate, None
            elif isinstance(addingInfo.addingDate, QtCore.QDateTime):
                presetValueDate, presetValueTime = addingDate.date(), addingDate.time()
            else:
                presetValueDate, presetValueTime = None, None

            newAction.updatePresetValuesConditions({'courseDate': presetValueDate,
                                                    'courseTime': presetValueTime})

            newAction.initPresetValues()
            newAction.save()
        finally:
            self.eventEditor = oldEventEditor
            self.actionTemplateCache = oldActionTemplateCache
            QtGui.qApp.unsetJTR(self)

    @staticmethod
    def _groupActions(courses, actionIdList, eventIdList, action=None, record=None):
        """
        Курсовые назначения имеют общее событие и один и тот же тип действия
        Группируем по датам для дальнейшего анализа курса.
        """

        action = action or CAction(record=record)
        record = record or action.getRecord()
        duration = forceInt(record.value('duration'))
        if duration < 1:
            return
        actionTypeId = forceRef(record.value('actionType_id'))
        eventId = forceRef(record.value('event_id'))
        actionId = forceRef(record.value('id'))
        eventId not in eventIdList and eventIdList.append(eventId)
        actionId not in actionIdList and actionIdList.append(actionId)
        dateKey = forceDate(record.value('begDate')).toPyDate()
        courseData = courses.setdefault((eventId, actionTypeId), {})
        courseData.setdefault(dateKey, []).append(action)

    def _getMainQueryTableAndCond(self):
        db = QtGui.qApp.db
        table = db.table('Job_Ticket')
        tableAPJT = db.table('ActionProperty_Job_Ticket')
        tableAP = db.table('ActionProperty')
        tableAction = db.table('Action')
        tableActionType = db.table('ActionType')
        tablePerson = db.table('vrbPersonWithSpeciality')
        tableEvent = db.table('Event')
        tableClient = db.table('Client')

        queryTable = table
        queryTable = queryTable.leftJoin(tableAPJT, tableAPJT['value'].eq(table['id']))
        queryTable = queryTable.leftJoin(tableAP, tableAP['id'].eq(tableAPJT['id']))
        queryTable = queryTable.leftJoin(tableAction, tableAction['id'].eq(tableAP['action_id']))
        queryTable = queryTable.leftJoin(tableActionType, tableActionType['id'].eq(tableAction['actionType_id']))
        queryTable = queryTable.leftJoin(tablePerson, tablePerson['id'].eq(tableAction['person_id']))
        queryTable = queryTable.leftJoin(tableEvent, tableEvent['id'].eq(tableAction['event_id']))
        queryTable = queryTable.leftJoin(tableClient, tableClient['id'].eq(tableEvent['client_id']))

        cond = [tableAction['deleted'].eq(0),
                tableEvent['deleted'].eq(0),
                tableClient['deleted'].eq(0)]

        return queryTable, cond
