# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from decimal import Decimal
from contextlib import contextmanager

from PyQt4 import QtGui, QtCore, QtSql

from Events.ActionStatus import CActionStatus
from Events.Action import CAction

from library.DialogBase import CDialogBase
from library.RecordLock import CRecordLockMixin
from library.InDocTable import CRecordListModel, CInDocTableCol, CBoolInDocTableCol
from library.ProgressBar import CProgressBar
from library.PrintTemplates import CPrintAction, getPrintButton
from Reports.PlannedClientInvoiceNomenclaturesReport import CPlannedClientInvoiceNomenclaturesReport

from library.Utils import formatNameByRecord, forceRef, toVariant, forceString, forceDouble, forceBool

from Stock.Service import CStockService

from Stock.Ui_GroupClientInvoice import Ui_GroupClientInvoice


class _CTables(object):
    def __init__(self):
        db = QtGui.qApp.db

        self.E = db.table('Event')
        self.A = db.table('Action')
        self.AT = db.table('ActionType')
        self.C = db.table('Client')

        self.APn = db.table('ActionProperty').alias('APn')
        self.APTn = db.table('ActionPropertyType').alias('APTn')
        self.APN = db.table('ActionProperty_rbNomenclature')
        self.N = db.table('rbNomenclature')

        self.APd = db.table('ActionProperty').alias('APd')
        self.APTd = db.table('ActionPropertyType').alias('APTd')
        self.APD = db.table('ActionProperty_Double')

        self.APs = db.table('ActionProperty').alias('APs')
        self.APTs = db.table('ActionPropertyType').alias('APTs')
        self.APS = db.table('ActionProperty_String')


class _CInvoiceProcessContext(object):
    def __init__(self, locker, tables):
        self._locker = locker
        self._tables = tables
        self._lockId = None

        self._oldConfirmLock = locker._confirmLock

        self._action = None
        self._failedData = {}
        self._successActionIds = set()

    @property
    def successActionIds(self):
        return self._successActionIds

    @property
    def failedData(self):
        return self._failedData

    @contextmanager
    def actionLock(self, action):
        if self._lockId:
            try:
                self._locker.releaseLock(self._lockId)
            finally:
                self._lockId = None
                raise RuntimeError(
                    "Could not process action lock. Previous lock %d was not released" % self._lockId
                )

        try:
            self._action = action
            lockId = self._locker.lock(self._tables.A.name(), action.getId())
            if lockId:
                self._successActionIds.add(action.getId())
                self._lockId = lockId

            yield bool(lockId)

        finally:
            if self._lockId:
                self._locker.releaseLock(self._lockId)

            self._action = self._lockId = None

    def _confirmLock(self, message):
        if self._action:
            self.fail(self._action.getId(), message)

    def fail(self, actionId, message):
        if actionId in self.successActionIds:
            self._successActionIds.remove(actionId)

        self._failedData[actionId] = message

    def __enter__(self):
        self._locker._confirmLock = self._confirmLock

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._locker._confirmLock = self._oldConfirmLock
        self._successActionIds.clear()
        self._failedData.clear()


class CGroupClientInvoice(CDialogBase, CRecordLockMixin, Ui_GroupClientInvoice):
    def __init__(self, orgStructureId, parent=None):
        CDialogBase.__init__(self, parent)
        CRecordLockMixin.__init__(self)

        self._filters = {}
        self._orgStructureId = orgStructureId
        self.setupUi(self)
        self.setWindowTitle(u'Списание ЛСиИМН')
        self.addModels('ClientNomenclatures', CGroupClientInvoiceModel(self))
        self.setModels(self.tblClientNomenclatures, self.modelClientNomenclatures)

        self.addObject('btnDoInvoices', QtGui.QPushButton(u'Списать', self))
        self.buttonBox.addButton(self.btnDoInvoices, QtGui.QDialogButtonBox.ActionRole)
        self.connect(self.btnDoInvoices, QtCore.SIGNAL('clicked()'), self._doInvoices)

        self.addObject('btnRefresh', QtGui.QPushButton(u'Обновить', self))
        self.buttonBox.addButton(self.btnRefresh, QtGui.QDialogButtonBox.ActionRole)
        self.connect(self.btnRefresh, QtCore.SIGNAL('clicked()'), self.load)

        self.addObject('btnPrint', getPrintButton(self, '', u'Печать'))
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)
        self.btnPrint.setEnabled(True)
        self.preparePrintBtn()

        self._tables = _CTables()

        self._setResultColumnVisible(False)

        self.actSetCheckAll = QtGui.QAction(u'Выделить все', self)
        self.actSetUnCheckedAll = QtGui.QAction(u'Снять все выделения', self)

        self.tblClientNomenclatures.createPopupMenu([self.actSetCheckAll, self.actSetUnCheckedAll])

        self.connect(self.actSetCheckAll, QtCore.SIGNAL('triggered()'), self.on_setCheckedAll)
        self.connect(self.actSetUnCheckedAll, QtCore.SIGNAL('triggered()'), self.on_setUnCheckedAll)
        self.connect(
            self.tblClientNomenclatures.popupMenu(), QtCore.SIGNAL('aboutToShow()'), self.on_popupMenuAboutToShow
        )

        self._progressBar = CProgressBar()
        self._progressBar.setFormat(u'%v из %m')
        self._progressBar.setMinimum(0)

    def _prepareProgressBar(self, steps):
        self._progressBar.setMaximum(steps)
        self._progressBar.setValue(0)
        self._progressBar.setMaximumHeight(self.statusBar.height() / 2)

        return self._progressBar

    def on_popupMenuAboutToShow(self):
        hasChecked = self.modelClientNomenclatures.hasChecked()
        allChecked = self.modelClientNomenclatures.allChecked()
        self.actSetCheckAll.setEnabled(not allChecked)
        self.actSetUnCheckedAll.setEnabled(hasChecked)

    def on_setCheckedAll(self):
        self.modelClientNomenclatures.setCheckedAll()

    def on_setUnCheckedAll(self):
        self.modelClientNomenclatures.setUnCheckedAll()

    def preparePrintBtn(self):
        btn = self.btnPrint
        menu = QtGui.QMenu()
        action = QtGui.QAction(u'Отчет о выдаче ЛС на выбранный период',  self)
        action.triggered.connect(self.plannedClientInvoiceNomenclaturesReport)
        menu.addAction(action)
        btn.setMenu(menu)

    def plannedClientInvoiceNomenclaturesReport(self):
        CPlannedClientInvoiceNomenclaturesReport(self).exec_()

    def _setResultColumnVisible(self, value):
        header = self.tblClientNomenclatures.horizontalHeader()
        header.setSectionHidden(CGroupClientInvoiceModel.RESULT_COLUMN, not value)

    def _doInvoices(self):
        items = [
            record for record in self.modelClientNomenclatures.items()
            if forceBool(record.value('isChecked'))
            ]

        if not items:
            return

        progressBar = self._prepareProgressBar(len(items))
        self.statusBar.addWidget(progressBar)

        try:
            progressBar.show()
            QtGui.qApp.callWithWaitCursor(self, self.__doInvoices, items, progressBar)

        finally:
            self.statusBar.removeWidget(progressBar)


    def __doInvoices(self, items, progressBar):
        processContext = _CInvoiceProcessContext(self, self._tables)

        actionIdsToReload = []

        with processContext:
            for record in items:
                progressBar.step()

                actionId = forceRef(record.value('actionId'))
                action = CAction.getActionById(actionId)
                if not action:
                    processContext.fail(actionId, u"Действие не найдено!")
                    continue

                with processContext.actionLock(action) as isLocked:
                    if not isLocked:
                        continue

                    if not action.checkOptimisticLock():
                        processContext.fail(
                            actionId, u'В процессе списания данные действия были изменены другим пользователем! Повторите попытку.')
                        actionIdsToReload.append(actionId)
                        continue

                    try:
                        CStockService.doClientInvoice(action, self._orgStructureId, self._filters['date'])

                    except:
                        processContext.fail(actionId, u'Ошибка в процессе списания!')
                        QtGui.qApp.logCurrentException()

            self.modelClientNomenclatures.setResults(processContext.failedData)
            self.modelClientNomenclatures.removeIds(processContext.successActionIds)
            self.modelClientNomenclatures.reloadActionIds(actionIdsToReload)
            self._setResultColumnVisible(True)

    def load(self, **filters):
        if filters:
            self._filters = filters
        self.modelClientNomenclatures.load(filters if filters else self._filters)


class CLocClientCol(CInDocTableCol):
    def __init__(self, model):
        CInDocTableCol.__init__(self, u'ФИО пациента', 'actionId', 15, readOnly=True)
        self._model = model

    def toString(self, val, record):
        return toVariant(self._model.getClientName(forceRef(val)))


class CLocNomenclatureCol(CInDocTableCol):
    def __init__(self, model):
        CInDocTableCol.__init__(self, u'Наименование ЛС', 'actionId', 15, readOnly=True)
        self._model = model

    def toString(self, val, record):
        return toVariant(self._model.getNomenclatureName(forceRef(val)))


class CLocQntCol(CInDocTableCol):
    def __init__(self, model):
        CInDocTableCol.__init__(self, u'Количество', 'actionId', 15, readOnly=True)
        self._model = model

    def toString(self, val, record):
        return toVariant(self._model.getQnt(forceRef(val)))


class CLocResultCol(CInDocTableCol):
    def __init__(self, model):
        CInDocTableCol.__init__(self, u'Результат', 'actionId', 20, readOnly=True)
        self._model = model

    def toString(self, val, record):
        return toVariant(self._model.getResult(forceRef(val)))


class _CItemsContainer(object):
    def __init__(self, model, items=None):
        self._model = model
        self._id2item = {}
        self._records = []
        self._extSqlFields = []

        if self._model._extColsPresent:
            for col in self._model.cols():
                if col.external():
                    fieldName = col.fieldName()
                    self._extSqlFields.append(QtSql.QSqlField(fieldName, col.valueType()))

        items and map(self.add, items)

    def removeIds(self, ids):
        self._records = [r for r in self._records if forceRef(r.value('actionId')) not in ids]
        for id in ids:
            del self._id2item[id]

    def __len__(self):
        return len(self._records)

    def __iter__(self):
        return iter(self._records)

    def __getitem__(self, index):
        return self._records[index]

    def getQnt(self, actionId):
        return self._id2item[actionId][CGroupClientInvoiceModel.QNT_COLUMN]

    def getNomenclatureName(self, actionId):
        return self._id2item[actionId][CGroupClientInvoiceModel.NOMENCLATURE_COLUMN]

    def getClientName(self, actionId):
        return self._id2item[actionId][CGroupClientInvoiceModel.CLIENT_COLUMN]

    def getResult(self, actionId):
        return self._id2item[actionId].get(CGroupClientInvoiceModel.RESULT_COLUMN)

    def add(self, record):
        self._addExtFields(record)

        self._records.append(record)

        self._setItem(record)

    def _addExtFields(self, record):
        for field in self._extSqlFields:
            record.append(field)

    def _setItem(self, record):
        actionId = forceRef(record.value('actionId'))
        clientName = formatNameByRecord(record)
        nomenclatureName = ' | '.join([forceString(record.value('nCode')), forceString(record.value('nName'))])
        doses = Decimal(str(forceDouble(record.value('doses'))))
        dosageValue = Decimal(str(forceDouble(record.value('dosageValue'))))
        clientQnt = Decimal('1')
        if dosageValue:
            clientQnt = doses / dosageValue

        self._id2item[actionId] = {
            CGroupClientInvoiceModel.CLIENT_COLUMN: clientName,
            CGroupClientInvoiceModel.NOMENCLATURE_COLUMN: nomenclatureName,
            CGroupClientInvoiceModel.QNT_COLUMN: float(clientQnt)
        }

    def reloadRecords(self, records):
        newRecordsMap = {}
        for r in records:
            self._addExtFields(r)
            newRecordsMap[forceRef(r.value('actionId'))] = r
            self._setItem(r)

        self._records = [newRecordsMap.get(forceRef(r.value('actionId')), r) for r in self._records]

    def setResults(self, results):
        for actionId, value in results.items():
            if actionId in self._id2item:
                self._id2item[actionId][CGroupClientInvoiceModel.RESULT_COLUMN] = value


class CGroupClientInvoiceModel(CRecordListModel):
    CHECKED_COLUMN = 0
    CLIENT_COLUMN = 1
    NOMENCLATURE_COLUMN = 2
    QNT_COLUMN = 3
    RESULT_COLUMN = 4

    def __init__(self, parent=None):
        CRecordListModel.__init__(self, parent)
        self._filters = None
        self._tables = _CTables()

        self.addExtCol(CBoolInDocTableCol(u'Списать', 'isChecked', 5), QtCore.QVariant.Bool)
        self.addCol(CLocClientCol(self))
        self.addCol(CLocNomenclatureCol(self))
        self.addCol(CLocQntCol(self))
        self.addCol(CLocResultCol(self))

    def getQnt(self, actionId):
        return self._items.getQnt(actionId)

    def getNomenclatureName(self, actionId):
        return self._items.getNomenclatureName(actionId)

    def getClientName(self, actionId):
        return self._items.getClientName(actionId)

    def getResult(self, actionId):
        return self._items.getResult(actionId)

    def setResults(self, results):
        self._items.setResults(results)

    def removeIds(self, ids):
        self._items.removeIds(ids)
        self.reset()

    def _iterItems(self, key=None):
        for item in self._items:
            yield key(item) if key else item

    def hasChecked(self):
        return any(self._iterItems(key=lambda i: forceBool(i.value('isChecked'))))

    def allChecked(self):
        return not any(self._iterItems(key=lambda i: not forceBool(i.value('isChecked'))))

    def setCheckedAll(self):
        self._setCheckValueAll(True)

    def setUnCheckedAll(self):
        self._setCheckValueAll(False)

    def _setCheckValueAll(self, value):
        map(lambda i: i.setValue('isChecked', toVariant(value)), self._items)
        self.emitColumnChanged(self.CHECKED_COLUMN)

    def setItems(self, records):
        CRecordListModel.setItems(self, _CItemsContainer(self, records))

    def _queryTable(self):
        tables = self._tables

        db = QtGui.qApp.db

        queryTable = tables.E
        queryTable = queryTable.innerJoin(tables.C, tables.C['id'].eq(tables.E['client_id']))
        queryTable = queryTable.innerJoin(tables.A, tables.A['event_id'].eq(tables.E['id']))
        queryTable = queryTable.innerJoin(tables.AT, tables.AT['id'].eq(tables.A['actionType_id']))
        queryTable = queryTable.innerJoin(tables.APTn, tables.APTn['actionType_id'].eq(tables.AT['id']))
        queryTable = queryTable.innerJoin(
            tables.APn,
            db.joinAnd(
                [tables.APn['action_id'].eq(tables.A['id']), tables.APn['type_id'].eq(tables.APTn['id'])]
            )
        )
        queryTable = queryTable.innerJoin(tables.APN, tables.APN['id'].eq(tables.APn['id']))
        queryTable = queryTable.innerJoin(tables.N, tables.N['id'].eq(tables.APN['value']))

        queryTable = queryTable.innerJoin(
            tables.APTd,
            # Доза это 2
            db.joinAnd([tables.APTd['actionType_id'].eq(tables.AT['id']), tables.APTd['inActionsSelectionTable'].eq(2)])
        )
        queryTable = queryTable.innerJoin(
            tables.APd,
            db.joinAnd(
                [tables.APd['action_id'].eq(tables.A['id']), tables.APd['type_id'].eq(tables.APTd['id'])]
            )
        )
        queryTable = queryTable.innerJoin(tables.APD, tables.APD['id'].eq(tables.APd['id']))

        return queryTable

    def load(self, filters):
        self._filters = filters
        self._load(self._filters)

    def _load(self, filters):
        records = self._getRecordList(filters)
        self.setItems(records)

    def _getRecordList(self, filters):
        queryTable = self._queryTable()

        tables = self._tables

        date = filters.get('date')
        nomenclatureId = filters.get('nomenclatureId')
        clientIds = filters.get('clientIds')
        actionIds = filters.get('actionIds')

        cond = [tables.A['status'].inlist([CActionStatus.started, CActionStatus.appointed])]

        if date:
            cond.append(tables.A['begDate'].dateEq(date))

        if nomenclatureId:
            cond.append(tables.APN['value'].eq(nomenclatureId))

        if clientIds:
            cond.append(tables.E['client_id'].inlist(clientIds))

        if actionIds:
            cond.append(tables.A['id'].inlist(actionIds))

        order = [
            tables.C['lastName'].name(),
            tables.C['firstName'].name(),
            tables.C['patrName'].name(),
            tables.A['id'].name()
        ]

        fields = [
            tables.N['id'].alias('nId'),
            tables.N['name'].alias('nName'),
            tables.N['code'].alias('ncode'),
            tables.N['dosageValue'],
            tables.A['id'].alias('actionId'),
            tables.C['lastName'],
            tables.C['firstName'],
            tables.C['patrName'],
            tables.APD['value'].alias('doses')
        ]

        return QtGui.qApp.db.getRecordList(queryTable, fields, where=cond, order=order)

    def reloadActionIds(self, actionIds):
        filters = dict(self._filters) if self._filters else {}
        filters['actionIds'] = actionIds

        records = self._getRecordList(filters)

        self._items.reloadRecords(records)

        self.emitAllChanged()




