# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui, QtCore

from library.Counter import CCounterController
from library.Utils import forceDateTime

from Stock.StockModel import CStockMotion, CStockMotionItem
from Stock.Utils import (
    findFinanceBatchShelfTime, UTILIZATION, 
    FILTER_FOR_FINANCE_AND_SHELF_TIME_FOR_UTILIZATION,  
    FILTER_FOR_BATCH_AND_FINANCE_FOR_UTILIZATION
)

from Stock.Utils import CStockCache, getNomenclatureUnitRatio


def _getSockMotionItemUniqueValues(stockMotionItem, fieldName, cond=None):
    queryTable = stockMotionItem.getTable()

    _cond = [
        CStockMotionItem.master_id == stockMotionItem.master_id,
    ]

    if cond:
        if isinstance(cond, list):
            _cond.extend(cond)
        else:
            _cond.append(cond)

    fields = [fieldName]

    return [
        getattr(CStockMotionItem, fieldName).attributeType.fromQVariantValue(getattr(r, fieldName))
        for r in CStockMotionItem
            .query(queryTable, fields=fields, cond=_cond)
            .getDistinctList()
        ]


class CStockService(object):
    def __init__(self, stockMotion):
        self._stockMotion = stockMotion
        self.stockCache = CStockCache()

    @staticmethod
    def getStockMotionByRecord(stockMotionRecord):
        return CStockMotion(stockMotionRecord)

    def getStockMotionItemByRecord(self, stockMotionItemRecord):
        result = CStockMotionItem(stockMotionItemRecord)
        result.stockMotion = self._stockMotion
        return result

    @staticmethod
    def setFinanceBatchShelfTime(stockMotionItem, financeId=None):
        stockMotion = stockMotionItem.stockMotion

        if QtGui.qApp.controlSMFinance() == 0:
            result = findFinanceBatchShelfTime(
                stockMotion.supplier_id, stockMotionItem.nomenclature_id, 
                stockMotionItem=stockMotionItem, filterFor=UTILIZATION, first=False
                )
        else:
            result = findFinanceBatchShelfTime(
                stockMotion.supplier_id, stockMotionItem.nomenclature_id, 
                stockMotionItem=stockMotionItem, financeId=financeId,  first=False
                )

        infinity = []
        withShelfTime = []

        for v in result:
            _, _, shelfTime = v
            if shelfTime.isNull():
                infinity.append(v)
            else:
                withShelfTime.append(v)

        if withShelfTime:
            v = sorted(result, key=lambda v: v[2])[0]
        else:
            v = infinity[0] if infinity else (None, None, None)

        financeId, batch, shelfTime = v

        stockMotionItem.finance_id = financeId
        stockMotionItem.batch = batch
        stockMotionItem.shelfTime = shelfTime

    @staticmethod
    def setFinanceAndShelfTimeByBatch(stockMotionItem):
        stockMotion = stockMotionItem.stockMotion
        financeId, batch, shelfTime = findFinanceBatchShelfTime(
            stockMotion.supplier_id, stockMotionItem.nomenclature_id,
            stockMotionItem=stockMotionItem, filterFor=FILTER_FOR_FINANCE_AND_SHELF_TIME_FOR_UTILIZATION
        )
        stockMotionItem.finance_id = financeId
        stockMotionItem.shelfTime = shelfTime

    @staticmethod
    def setFinanceAndBatchByShelfTime(stockMotionItem):
        stockMotion = stockMotionItem.stockMotion
        financeId, batch, shelfTime = findFinanceBatchShelfTime(
            stockMotion.supplier_id, stockMotionItem.nomenclature_id,
            stockMotionItem=stockMotionItem, filterFor=FILTER_FOR_BATCH_AND_FINANCE_FOR_UTILIZATION
        )
        stockMotionItem.finance_id = financeId
        stockMotionItem.batch = batch

    @staticmethod
    def setBatchAndShelfTimeByFinance(stockMotionItem):
        stockMotion = stockMotionItem.stockMotion
        financeId, batch, shelfTime = findFinanceBatchShelfTime(
            stockMotion.supplier_id, stockMotionItem.nomenclature_id,
            stockMotionItem=stockMotionItem, filterFor=FILTER_FOR_BATCH_AND_FINANCE_FOR_UTILIZATION
        )
        stockMotionItem.shelfTime = shelfTime
        stockMotionItem.batch = batch

    @staticmethod
    def getBatchListDependOnFinanceAndShelfTime(stockMotionItem):
        stockMotion = stockMotionItem.stockMotion
        if stockMotion.supplier_id:
            result = findFinanceBatchShelfTime(
                stockMotion.supplier_id, stockMotionItem.nomenclature_id,#FILTER_FOR_BATCH_AND_SHELF_TIME_FOR_UTILIZATION
                stockMotionItem=stockMotionItem, filterFor=UTILIZATION, first=False
            )
            return [r[1] for r in result]

        cond = [
            CStockMotionItem.finance_id == stockMotionItem.finance_id,
            CStockMotionItem.shelfTime == stockMotionItem.shelfTime
        ]
        return _getSockMotionItemUniqueValues(stockMotionItem, CStockMotionItem.batch.name, cond)


    @staticmethod
    def getShelfTimeListDependOnFinanceAndBatch(stockMotionItem, orgStructureId=None):
        stockMotion = stockMotionItem.stockMotion
        if stockMotion.supplier_id:
            result = findFinanceBatchShelfTime(
                stockMotion.supplier_id, stockMotionItem.nomenclature_id,#FILTER_FOR_BATCH_AND_FINANCE_FOR_UTILIZATION
                stockMotionItem=stockMotionItem, filterFor=UTILIZATION, first=False
            )
            return [r[2] for r in result]

        cond = [
            CStockMotionItem.finance_id == stockMotionItem.finance_id,
            CStockMotionItem.batch == stockMotionItem.batch
        ]
        return _getSockMotionItemUniqueValues(stockMotionItem, CStockMotionItem.shelfTime.name, cond)

    @staticmethod
    def getFinanceIdListDependOnBatchAndShelfTime(stockMotionItem, orgStructureId):
        stockMotion = stockMotionItem.stockMotion
        if stockMotion.supplier_id:
            result = findFinanceBatchShelfTime(
                stockMotion.supplier_id, stockMotionItem.nomenclature_id,#FILTER_FOR_FINANCE_AND_SHELF_TIME_FOR_UTILIZATION
                stockMotionItem=stockMotionItem, filterFor=UTILIZATION, first=False
            )
            return [r[0] for r in result]

        cond = [
            CStockMotionItem.shelfTime == stockMotionItem.shelfTime,
            CStockMotionItem.batch == stockMotionItem.batch
        ]

        return _getSockMotionItemUniqueValues(stockMotionItem, CStockMotionItem.finance_id.name, cond)

    @classmethod
    def doClientInvoice(cls, action, supplierId, date=None):
        """
        :param action: Events.Action.CAction
        :param supplierId: supplier org structure id
        :return: bool
        """
        nomenclatureExpense = action.nomenclatureExpense
        if not nomenclatureExpense:
            return False

        event = action.event
        if not event:
            return False

        QtGui.qApp.setCounterController(CCounterController())

        stockMotionRecord = nomenclatureExpense.stockMotionRecord()
        stockMotionItemRecords = nomenclatureExpense.stockMotionItems()

        stockMotion = CStockMotion(stockMotionRecord)
        stockMotion.supplier_id = supplierId
        stockMotion.stockDocumentType = 4
        stockMotion.date = QtCore.QDateTime.currentDateTime()
        stockMotion.generateStockMotionNumber()
        stockMotion.client_id = event.client_id

        service = cls(stockMotion)
        for stockMotionItemRecord in stockMotionItemRecords:
            stockMotionItem = service.getStockMotionItemByRecord(stockMotionItemRecord)
            stockMotionItem.sum = service.calcStockMotionItemSum(stockMotionItem)

        db = QtGui.qApp.db
        db.transaction()
        try:
            nextAction = action.finishAction(event.client_id, date)
            action.save(idx=-1)

            if nextAction:
                idx = nextAction.countIdx(event.id)
                nextAction.save(eventId=event.id, idx=idx)

        except:
            db.rollback()
            QtGui.qApp.resetAllCounterValueIdReservation()
            raise

        db.commit()

        QtGui.qApp.delAllCounterValueIdReservation()

        QtGui.qApp.setCounterController(None)
        
        newExecutionPlan = action.getExecutionPlanForNewAction(forceDateTime(action._record.value('begDate')))

        if not newExecutionPlan:
            action.nomenclatureClientReservation.cancel()

        return True

    def calcStockMotionItemSum(self, stockMotionItem):
        if not stockMotionItem.nomenclature_id:
            return None

        stockPrice = self.stockCache.getPrice(
            self._stockMotion.supplier_id, stockMotionItem.nomenclature_id, stockMotionItem.finance_id
        )
        stockUnitId = stockMotionItem.nomenclature.defaultStockUnit_id
        if stockMotionItem.unit_id == stockUnitId:
            ratio = 1
        else:
            ratio = getNomenclatureUnitRatio(stockMotionItem.nomenclature_id, stockUnitId, stockMotionItem.unit_id)
        return stockMotionItem.qnt * ratio * stockPrice
