# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

u'Работа: складской учёт'

import sip
from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt, QByteArray, QDate, QDateTime, QTime, QVariant, pyqtSignature

from library.crbcombobox              import CRBComboBox, CRBModelDataCache
from library.DbComboBox               import CDbDataCache
from library.DialogBase               import CDialogBase
from library.InDocTable               import CRecordListModel, CInDocTableCol, CRBInDocTableCol, CFloatInDocTableCol, CDateInDocTableCol
from library.PrintInfo                import CInfoContext
from library.PrintTemplates           import applyTemplate, CPrintAction, CPrintButton, getPrintTemplates, getPrintButton
from library.TableModel               import CTableModel, CDateCol, CDateTimeCol, CEnumCol, CRefBookCol, CTextCol, CCol
from library.Utils                    import forceDouble, forceInt, forceRef, forceString, forceStringEx, formatRecordsCount, smartDict, toVariant
from library.Counter import CCounterController

from Orgs.Utils                       import getOrgStructureDescendants, getOrgStructureFullName
from Registry.Utils                   import getClientMiniInfo
from Reports.ReportBase               import CReportBase, createTable
from Reports.ReportView               import CReportViewDialog
from Reports.Utils                    import dateRangeAsStr
from Reports.ReportStockM11           import CReportStockM11
from Stock.ClientInvoiceEditDialog    import CClientRefundInvoiceEditDialog
from Stock.FinTransferEditDialog      import CFinTransferEditDialog
from Stock.InventoryEditDialog        import CInventoryEditDialog
from Stock.InvoiceEditDialog          import CInvoiceEditDialog
from Stock.NomenclatureComboBox       import CNomenclatureInDocTableCol
from Stock.ProductionEditDialog       import CProductionEditDialog
from Stock.StockUtilizationEditDialog import CStockUtilizationEditDialog, CStockInternalConsumptionEditDialog
from Stock.StockMotion                import editStockMotion, getDialogName, stockMotionType, openReadOnlyMotion
from Stock.StockRequisitionEditDialog import CStockRequisitionEditDialog
from Stock.StockMotionInfo            import CStockMotionInfoList, CStockRemainingsInfoList, CStockRequisitionsInfoList 
from Users.Rights                     import urEditOwnMotions, urEditOtherMotions, urDeleteMotions

from Stock.Utils import (
    getStockMotionItemQuantityColumn, getRemainingHistory, showRemainingHistoryReport, tuneSatisfiedFilterCmb, CSatisifiedFilter
)

from Stock.Ui_StockDialog             import Ui_StockDialog


class CSorterData(object):
    def __init__(self):
        self._columnIndex = None
        self._ascending = False

    @property
    def columnIndex(self):
        return self._columnIndex

    @property
    def ascending(self):
        return self._ascending

    def apply(self, tableView, columnIndex):
        header = tableView.horizontalHeader()

        if self._columnIndex == columnIndex:
            self._ascending = not self._ascending
        else:
            self._columnIndex = columnIndex
            self._ascending = True
        header.setSortIndicatorShown(True)
        header.setSortIndicator(
            self._columnIndex, Qt.AscendingOrder if self._ascending else Qt.DescendingOrder
        )


class CTableViewSorter(object):
    def __init__(self):
        self._cache = {}

    def __call__(self, key, tableView, columnIndex):
        if key not in self._cache:
            self._cache[key] = CSorterData()

        self._cache[key].apply(tableView, columnIndex)

    def getData(self, key):
        return self._cache.get(key, None)


class CInMemoryStorageMixin(object):
    def setCopiedData(self, key, data):
        storage = self.__getStorage()
        storage[key] = data

    def getCopiedData(self, key, default=None):
        storage = self.__getStorage()
        return storage.get(key, default)

    def __getStorage(self):
        if not '_CInMemoryStorageMixin__storage' in self.__dict__:
            self.__dict__['_CInMemoryStorageMixin__storage'] = {}
        return self.__storage


class CStockDialog(CDialogBase, CInMemoryStorageMixin, Ui_StockDialog):
    def __init__(self, parent):
        CDialogBase.__init__(self, parent)
        self._tableViewSorter = CTableViewSorter()

        self.motionsFilter = smartDict()
        self.remainingsFilter = smartDict()
        self.MRsFilter = smartDict()
        self.RTMsFilter = smartDict()

        self.addModels('Motions',   CMyMotionsModel(self))
        self.addModels('MotionsItems',   CMyMotionsItemsModel(self))
        self.addObject('mnuMotions',        QtGui.QMenu(self))
        self.addObject('mnuMotionsItems',        QtGui.QMenu(self))
        self.addObject('actAddInvoice',     QtGui.QAction(u'Создать накладную', self))
        self.addObject('actAddInventory',   QtGui.QAction(u'Создать инвентаризацию', self))
        self.addObject('actAddFinTransfer', QtGui.QAction(u'Создать фин.перенос', self))
        self.addObject('actAddProduction',  QtGui.QAction(u'Создать производство', self))
        self.addObject('actAddClientRefundInvoice',  QtGui.QAction(u'Создать возврат', self))
        self.addObject('actAddUtilization', QtGui.QAction(u'Утилизация', self))
        self.addObject('actAddInternalConsumption', QtGui.QAction(u'Внутреннее потребление', self))
        self.addObject('actEditMotion',     QtGui.QAction(u'Редактировать движение', self))
        self.addObject('actPrintInvoice',   QtGui.QAction(u'Печать М11', self))
        self.addObject('actOpenMotion',        QtGui.QAction(u'Открыть накладную', self))
        self.addObject('actDeleteRow', QtGui.QAction(u'Удалить запись', self))

        self.mnuMotions.addAction(self.actAddInvoice)
        self.mnuMotions.addAction(self.actAddInventory)
        self.mnuMotions.addAction(self.actAddFinTransfer)
        self.mnuMotions.addAction(self.actAddProduction)
        self.mnuMotions.addAction(self.actAddClientRefundInvoice)
        self.mnuMotions.addAction(self.actAddUtilization)
        self.mnuMotions.addAction(self.actAddInternalConsumption)
        self.mnuMotions.addSeparator()
        self.mnuMotions.addAction(self.actEditMotion)
        self.mnuMotions.addAction(self.actDeleteRow)
        self.actDeleteRow.setEnabled(QtGui.qApp.userHasAnyRight([urDeleteMotions]))
        self.mnuMotions.addSeparator()
        self.mnuMotionsItems.addAction(self.actOpenMotion)
        
        self.mnuMotions.addAction(self.actPrintInvoice)

        self.addModels('Remainings', CRemainingsModel(self))

        self.addModels('MRs',       CMyRequisitionsModel(self))
        self.addObject('mnuMRs',    QtGui.QMenu(self))
        self.addObject('actAddRequisition', QtGui.QAction(u'Создать требование', self))
        self.addObject('actEditRequisition', QtGui.QAction(u'Редактировать требование', self))
        self.addObject('actRevokeRequisition', QtGui.QAction(u'Отменить требование', self))
        self.addObject('actShowOverhead', QtGui.QAction(u'Показать связанные накладные', self))
        self.mnuMRs.addAction(self.actAddRequisition)
        self.mnuMRs.addAction(self.actEditRequisition)
        self.mnuMRs.addAction(self.actRevokeRequisition)
        self.mnuMRs.addAction(self.actShowOverhead)
        self.mnuMRs.addAction(self.actPrintInvoice)
        self.addModels('MRContent', CRequisitionContentModel(self))

        self.addModels('RTMs',       CRequisitionsToMeModel(self))
        self.addObject('mnuRTMs', QtGui.QMenu(self))
#        self.addObject('actEditClient', QtGui.QAction(u'Открыть регистрационную карточку', self))
        self.addObject('actCreateMotionByRequisition', QtGui.QAction(u'Создать движение по требованию', self))
#        self.addObject('actRejectRequisition', QtGui.QAction(u'Отказать в требование', self))
        self.mnuRTMs.addAction(self.actCreateMotionByRequisition)
#        self.mnuRTMs.addAction(self.actRejectRequisition)
        self.addModels('RTMContent', CRequisitionContentModel(self))
        self.templateNames = {0:'StockMotions', 1:'StockRemainings', 2:'StockMyRequisitions', 3:'StockRequirementsToMe'}
        self.addObject('btnPrint', getPrintButton(self, ''))
        self.btnPrint.setShortcut('F6')
        self.setupUi(self)

        tuneSatisfiedFilterCmb(self.cmbMRsFilterSatisfied)
        tuneSatisfiedFilterCmb(self.cmbRTMsFilterSatisfied)

        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)
        self.printMenu = {}
        self.preparePrintBtn(self.tabWidget.currentIndex())
        if QtGui.qApp.currentOrgStructureId():
            self.setWindowTitle(u'Склад ЛСиИМН: %s' % getOrgStructureFullName(QtGui.qApp.currentOrgStructureId()))

        self.bbxMotionsFilter.button(QtGui.QDialogButtonBox.Apply).setDefault(True)
        self.bbxRemainingsFilter.button(QtGui.QDialogButtonBox.Apply).setDefault(True)
        self.bbxMRsFilter.button(QtGui.QDialogButtonBox.Apply).setDefault(True)
        self.bbxRTMsFilter.button(QtGui.QDialogButtonBox.Apply).setDefault(True)

        self.edtMotionsFilterBegReasonDate.canBeEmpty(True)
        self.edtMotionsFilterEndReasonDate.canBeEmpty(True)

        self.cmbMotionsFilterNomenclature.setTable('rbNomenclature', order='name')
        self.cmbMotionsFilterFinance.setTable('rbFinance', True)
        self.cmbMotionsFilterSupplierOrg.setFilter('isSupplier = 1')

        self.cmbRemainingsFilterNomenclature.setTable('rbNomenclature', order='name')
        self.cmbRemainingsFilterFinance.setTable('rbFinance', True)
        self.cmbRemainingsFilterUnit.setTable('rbUnit', True)

        self.cmbMRsFilterNomenclature.setTable('rbNomenclature', order='name')
        self.cmbRTMsFilterNomenclature.setTable('rbNomenclature', order='name')

        self.tblMotions.setModel(self.modelMotions)
        self.tblMotions.setSelectionModel(self.selectionModelMotions)
        self.tblMotions.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.tblMotions.setPopupMenu(self.mnuMotions)

        self.tblMotionsItems.setModel(self.modelMotionsItems)
        self.tblMotionsItems.setSelectionModel(self.selectionModelMotionsItems)
        self.tblMotionsItems.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.tblMotionsItems.setPopupMenu(self.mnuMotionsItems)

        self.tblRemainings.setModel(self.modelRemainings)
        self.tblRemainings.setSelectionModel(self.selectionModelRemainings)

        self.tblMRs.setModel(self.modelMRs)
        self.tblMRs.setSelectionModel(self.selectionModelMRs)
        self.tblMRs.setPopupMenu(self.mnuMRs)
        self.tblMRContent.setModel(self.modelMRContent)
        self.tblMRContent.setSelectionModel(self.selectionModelMRContent)

        self.tblRTMs.setModel(self.modelRTMs)
        self.tblRTMs.setSelectionModel(self.selectionModelRTMs)
        self.tblRTMs.setPopupMenu(self.mnuRTMs)

        self.tblRTMContent.setModel(self.modelRTMContent)
        self.tblRTMContent.setSelectionModel(self.selectionModelRTMContent)



        self.controlSplitter = None
        self._recordsIdDict = {}

        self.visitedMRs = False
        self.visitedRTMs = False


        # defaults
        self.edtRemainingsFilterBegShelfTime.setDate(QDate())
        self.edtRemainingsFilterEndShelfTime.setDate(QDate())

        self.chkRemainingsFilterGroupByBatch.setChecked(True)

        self.cmbRemainingsFilterStorage.setValue(QtGui.qApp.currentOrgStructureId())

        self.tblMotions.enableColsHide()
        self.tblMotions.enableColsMove()
        self.tblRemainings.enableColsHide()
        self.tblRemainings.enableColsMove()

        self.addObject('actRemainingsShowStockMotions', QtGui.QAction(u'Показать накладные', self))
        self.tblRemainings.addPopupAction(self.actRemainingsShowStockMotions)

        self.connect(self.actRemainingsShowStockMotions, QtCore.SIGNAL('triggered()'), self.on_actRemainingsShowStockMotions_triggered)

        self.connect(self.tblMotions.horizontalHeader(), QtCore.SIGNAL('sectionClicked(int)'), self.on_motionsSortByColumn)


    def exec_(self):
        self.tabWidget.setCurrentIndex(0)
        self.resetMotionsFilter()
        self.applyMotionsFilter()
        CDialogBase.exec_(self)


    def syncSplitters(self, nextSplitter):
        if self.controlSplitter and nextSplitter != self.controlSplitter:
            nextSplitter.setSizes(self.controlSplitter.sizes())
            self.controlSplitter = nextSplitter
        else:
            self.controlSplitter = nextSplitter


    def composeDateTime(self, date, time):
        if date:
            return QDateTime(date, time)
        else:
            return QDateTime()


    def getRequisitionItemIdList(self, requisitionId):
        if requisitionId:
            db = QtGui.qApp.db
            table = db.table('StockRequisition_Item')
            cond = table['master_id'].inlist(requisitionId)
            return db.getIdList(table, 'id', where=cond, order='idx, id')
        else:
            return []


    def preparePrintBtn(self,  index):
        btn = self.btnPrint
        enabled = True
        menu = self.getPrintBtnMenu(index)
        if not menu:
            menu = QtGui.QMenu()
            act = menu.addAction(u'Нет шаблонов печати')
            act.setEnabled(False)
            enabled = False
        btn.setMenu(menu)
        btn.setEnabled(enabled)
        self.printMenu[index] = menu


    def getPrintBtnMenu(self, index):
        self.reportFunc = {
                      -1:[u'Напечатать список', self.actPrintMotions]
                      }
        if index in self.printMenu:
            menu = self.printMenu[index]
        else:
            menu = QtGui.QMenu()
            subMenuDict={}
            templates = getPrintTemplates(self.templateNames[index])
            if templates:
                for i, template in enumerate(templates):
                    if template[2]==u'':
                        action = CPrintAction(template[0], template[1], self.btnPrint,  self.btnPrint)
                        menu.addAction(action)
                    else:
                        subMenu = subMenuDict.get(template[2], QtGui.QMenu(template[2], self.parentWidget()))
                        action = CPrintAction(template[0], template[1], self.btnPrint,  self.btnPrint)
                        subMenu.addAction(action)
                        subMenuDict[template[2]] = subMenu
                if len(subMenuDict):
                    for subMenuKey in subMenuDict:
                        menu.addMenu(subMenuDict[subMenuKey])
            else:
                act = menu.addAction(u'Нет шаблонов печати')
                act.setEnabled(False)
            menu.addSeparator()
            for key in sorted(self.reportFunc):
                menu.addAction( CPrintAction(self.reportFunc[key][0],  key,  self.btnPrint,  self.btnPrint) )
        return menu

    @pyqtSignature('int')
    def on_btnPrint_printByTemplate(self, templateId):
        if templateId < 0:
            self.reportFunc[templateId][1]()
        else:
            context = forceString(QtGui.qApp.db.translate('rbPrintTemplate', 'id', templateId, 'context'))
            data = None
            if context == u'StockMotions':
                data = self.getContextData(self.tblMotions)
            elif context == u'StockRemainings':
                data = self.getContextData(self.tblRemainings)
            elif context == u'StockMyRequisitions':
                data = self.getContextData(self.tblMRContent)
            elif context == u'StockRequirementsToMe':
                data = self.getContextData(self.tblRTMContent)
            if data:
                QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    def getTableToWidget(self, widgetIndex):
        return [self.tblMotions, self.tblRemainings, self.tblMRContent, self.tblRTMContent][widgetIndex]


    def actPrintMotions(self):
        widget = self.getTableToWidget(self.tabWidget.currentIndex())
        if self.tabWidget.currentIndex() == 1:
            self.getRemainingsPrint()
        else:
            widget.setReportHeader(u'Список')
            widget.setReportDescription(self.getStockFilterAsText(self.tabWidget.currentIndex()))
            widget.printContent()


    def getContextData(self, table):
        context = CInfoContext()
        if self.tabWidget.currentIndex() == 0:
            idList = self.modelMotions.idList()
            selectedIdList = self.tblMotions.selectedItemIdList()
            data = { 'motionsList': CStockMotionInfoList(context, idList), 
                          'selectedMotionsList' : CStockMotionInfoList(context, selectedIdList)}
        elif self.tabWidget.currentIndex() == 1:
            items = self.modelRemainings._items
            data = { 'remainingsList': CStockRemainingsInfoList(context, items)}
        elif self.tabWidget.currentIndex() == 2:
            idList = self.modelMRs.idList()
            selectedIdList = self.tblMRs.selectedItemIdList()
            data = { 'requisitionsList': CStockRequisitionsInfoList(context, idList), 
                         'selectedRequisitionsList': CStockRequisitionsInfoList(context, selectedIdList)}
        elif self.tabWidget.currentIndex() == 3:
            idList = self.modelRTMs.idList()
            selectedIdList = self.tblRTMs.selectedItemIdList()
            data = { 'requisitionsList': CStockRequisitionsInfoList(context, idList), 
                        'selectedRequisitionsList': CStockRequisitionsInfoList(context, selectedIdList)}
        return data


    def dumpParams(self, cursor):
        db = QtGui.qApp.db
        description = []
        remainingsFilterDate = self.edtRemainingsFilterDate.date()
        if remainingsFilterDate:
            description.append(u'На дату ' + forceString(remainingsFilterDate))
        remainingsFilterStorage = self.cmbRemainingsFilterStorage.value()
        if remainingsFilterStorage:
            description.append(u'Склад %s'%forceString(db.translate('OrgStructure', 'id', remainingsFilterStorage, 'name')))
        remainingsFilterNomenclature = self.cmbRemainingsFilterNomenclature.value()
        if remainingsFilterNomenclature:
            description.append(u'Номенклатура %s'%forceString(db.translate('rbNomenclature', 'id', remainingsFilterNomenclature, 'name')))
        if self.chkRemainingsFilterGroupByBatch.isChecked():
            description.append(u'Разделять по партиям и годности')
        remainingsFilterBatch = self.edtRemainingsFilterBatch.text()
        if remainingsFilterBatch:
            description.append(u'Партия %s'%remainingsFilterBatch)
        begDate = self.edtRemainingsFilterBegShelfTime.date()
        endDate = self.edtRemainingsFilterEndShelfTime.date()
        if begDate or endDate:
            description.append(dateRangeAsStr(u'Годен', begDate, endDate))
        remainingsFilterFinance = self.cmbRemainingsFilterFinance.value()
        if remainingsFilterFinance:
            description.append(u'Тип финансирования %s'%forceString(db.translate('rbFinance', 'id', id, 'name')))
        if self.chkRemainingsFilterConstrainedQnt.isChecked():
            description.append(u'Запас ниже точки заказа')
        if self.chkRemainingsFilterOrderQnt.isChecked():
            description.append(u'Запас ниже гантированного')
        if self.chkRemainingsFilterAvailable.isChecked():
            description.append(u'Имеющиеся в наличии')
        remainingsFilterUnit = self.cmbRemainingsFilterUnit.value()
        if remainingsFilterUnit:
            description.append(u'Ед. учета %s'%forceString(db.translate('rbUnit', 'id', id, 'name')))
        description.append(u'отчёт составлен: ' + forceString(QDateTime.currentDateTime()))
        columns = [ ('100%', [], CReportBase.AlignLeft) ]
        table = createTable(cursor, columns, headerRowCount=len(description), border=0, cellPadding=2, cellSpacing=0)
        for i, row in enumerate(description):
            table.setText(i, 0, row)
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertBlock()


    def getRemainingsPrint(self):
        model = self.tblRemainings.model()
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(u'Остатки\n')
        self.dumpParams(cursor)
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportBody)
        colWidths  = [ self.tblRemainings.columnWidth(i) for i in xrange(model.columnCount()-1) ]
        colWidths.insert(0,10)
        totalWidth = sum(colWidths)
        tableColumns = []
        iColNumber = False
        for iCol, colWidth in enumerate(colWidths):
            widthInPercents = str(max(1, colWidth*90/totalWidth))+'%'
            if iColNumber == False:
                tableColumns.append((widthInPercents, [u'№'], CReportBase.AlignRight))
                iColNumber = True
            tableColumns.append((widthInPercents, [forceString(model._cols[iCol].title())], CReportBase.AlignLeft))
        table = createTable(cursor, tableColumns)
        for iModelRow in xrange(model.rowCount()):
            iTableRow = table.addRow()
            table.setText(iTableRow, 0, iModelRow+1)
            for iModelCol in xrange(model.columnCount()):
                index = model.createIndex(iModelRow, iModelCol)
                text = forceString(model.data(index))
                table.setText(iTableRow, iModelCol+1, text)
        html = doc.toHtml(QByteArray('utf-8'))
        view = CReportViewDialog(self)
        view.setText(html)
        view.exec_()


    def getStockFilterAsText(self, widget):
        db = QtGui.qApp.db
        if widget == 0:
            filter = self.motionsFilter
            tmpList = [('number', u'Номер', forceString),
                        ('begDate', u'Дата с', forceString),
                        ('endDate', u'Дата по', forceString),
                        ('reason',   u'Основание', forceString),
                        ('supplierId', u'Источник',
                            lambda id: forceString(db.translate('OrgStructure', 'id', id, 'name'))),
                        ('receiverId', u'Получатель',
                            lambda id: forceString(db.translate('OrgStructure', 'id', id, 'name'))),
                        ('clientId',   u'Код пациента', forceString),
                        ('note',   u'Примечание', forceString),
                        ('nomenclatureId', u'Номенклатура',
                            lambda id: forceString(db.translate('rbNomenclature', 'id', id, 'name'))),
                        ('financeId', u'Тип финансирования',
                            lambda id: forceString(db.translate('rbFinance', 'id', id, 'name'))),
                        ('type', u'Тип документа', getDialogName), 
                        ('groupMotions', u'Группировать накладные в рамках одного курса', getDialogName), 
                        ]
        if widget == 1:
            filter = self.remainingsFilter
            tmpList = [('date', u'На дату', forceString),
                        ('storageId', u'Склад',
                            lambda id: forceString(db.translate('OrgStructure', 'id', id, 'name'))),
                        ('nomenclatureId', u'Номенклатура',
                            lambda id: forceString(db.translate('rbNomenclature', 'id', id, 'name'))),
                        ('groupByBatch',  u'Разделять по партиям и годности', lambda dummy: u'+'),
                        ('batch', u'Партия', forceString),
                        ('begShelfTime',   u'Годен с', forceString),
                        ('endShelfTime',   u'Годен по', forceString),
                        ('financeId', u'Тип финансирования',
                            lambda id: forceString(db.translate('rbFinance', 'id', id, 'name'))),
                        ('constrainedQnt',  u'Запас ниже точки заказа', lambda dummy: u'+'),
                        ('orderQnt',  u'Запас ниже гантированного', lambda dummy: u'+')
                        ]
        if widget == 2:
            filter = self.MRsFilter
            tmpList = [('onlyActive',   u'Только действующие', lambda dummy: u'+'),
                        ('begDate', u'Период с', forceString),
                        ('endDate', u'Период по', forceString),
                        ('begDeadline', u'В срок с', forceString),
                        ('endDeadline', u'В срок по', forceString),
                        ('orgStructureId', u'Получатель',
                            lambda id: forceString(db.translate('OrgStructure', 'id', id, 'name'))),
                        ('nomenclatureId', u'Номенклатура',
                            lambda id: forceString(db.translate('rbNomenclature', 'id', id, 'name')))
                        ]
        if widget == 3:
            filter = self.RTMsFilter
            tmpList = [('onlyActive',   u'Только действующие', lambda dummy: u'+'),
                        ('begDate', u'Период с', forceString),
                        ('endDate', u'Период по', forceString),
                        ('begDeadline', u'В срок с', forceString),
                        ('endDeadline', u'В срок по', forceString),
                        ('orgStructureId', u'Заказчик',
                            lambda id: forceString(db.translate('OrgStructure', 'id', id, 'name'))),
                        ('nomenclatureId', u'Номенклатура',
                            lambda id: forceString(db.translate('rbNomenclature', 'id', id, 'name')))
                        ]
        resList = [(u'Список', u'')]
        for (x, y, z) in tmpList:
            self.convertFilterToTextItem(resList, filter, x, y, z)
        return '\n'.join([ ': '.join(item) for item in resList])


    def convertFilterToTextItem(self, resList, filter, prop, propTitle, format=None):
        val = filter.get(prop, None)
        if val:
            if format:
                resList.append((propTitle, format(val)))
            else:
                resList.append((propTitle, val))


    # MR ##(my requests)########################################################

    def resetMRsFilter(self):
        self.chkMRsFilterOnlyActive.setChecked(True)
        self.edtMRsFilterBegDate.setDate(QDate())
        self.edtMRsFilterEndDate.setDate(QDate())
        self.edtMRsFilterBegDeadlineDate.setDate(QDate())
        self.edtMRsFilterEndDeadlineDate.setDate(QDate())
        self.edtMRsFilterBegDeadlineTime.setTime(QTime())
        self.edtMRsFilterEndDeadlineTime.setTime(QTime())
        self.cmbMRsFilterOrgStructure.setValue(None)
        self.cmbMRsFilterNomenclature.setValue(None)
        self.cmbMRsFilterSatisfied.setCurrentIndex(CSatisifiedFilter.ALL)


    def applyMRsFilter(self):
        self.MRsFilter.onlyActive = self.chkMRsFilterOnlyActive.isChecked()
        self.MRsFilter.begDate = self.edtMRsFilterBegDate.date()
        self.MRsFilter.endDate = self.edtMRsFilterEndDate.date()
        self.MRsFilter.begDeadline = self.composeDateTime(self.edtMRsFilterBegDeadlineDate.date(), self.edtMRsFilterBegDeadlineTime.time())
        self.MRsFilter.endDeadline = self.composeDateTime(self.edtMRsFilterEndDeadlineDate.date(), self.edtMRsFilterEndDeadlineTime.time())
        self.MRsFilter.orgStructureId = self.cmbMRsFilterOrgStructure.value()
        self.MRsFilter.nomenclatureId = self.cmbMRsFilterNomenclature.value()
        self.MRsFilter.satisfiedFilter = self.cmbMRsFilterSatisfied.currentIndex()
        self.updateMRsList()


#    def setMRsSort(self, col):
#        self.order = col
#        header=self.tblItems.horizontalHeader()
#        header.setSortIndicatorShown(True)
#        header.setSortIndicator(col, Qt.AscendingOrder)
#        self.updateMRsList()

    def updateMRsList(self, currentId=None):
        filter = self.MRsFilter
        db = QtGui.qApp.db
        table = db.table('StockRequisition')
        tableItems = db.table('StockRequisition_Item')
        cond = [table['deleted'].eq(0),
                table['recipient_id'].eq(QtGui.qApp.currentOrgStructureId()),
               ]
        if filter.orgStructureId:
            cond.append(table['supplier_id'].inlist(getOrgStructureDescendants(filter.orgStructureId)))
        if filter.onlyActive:
            cond.append(table['revoked'].eq(0))
        if filter.begDate:
            cond.append(table['date'].ge(filter.begDate))
        if filter.endDate:
            cond.append(table['date'].le(filter.endDate))
        if filter.begDeadline:
            cond.append(table['deadline'].ge(filter.begDeadline))
        if filter.endDeadline:
            cond.append(table['deadline'].le(filter.endDeadline))
        if filter.nomenclatureId:
            subcond = [tableItems['master_id'].eq(table['id']),
                       tableItems['nomenclature_id'].eq(filter.nomenclatureId),
                      ]
            if filter.onlyActive:
                subcond.append(tableItems['qnt'].ge(tableItems['satisfiedQnt']))
            cond.append(db.existsStmt(tableItems, subcond))
        if filter.satisfiedFilter:
            c = CSatisifiedFilter.getFilter(filter.satisfiedFilter, filter.nomenclatureId)
            if c is not None:
                if  filter.satisfiedFilter in (1, 2, 3, 4):
                    cond.append(u'StockRequisition.id in (%s)'%c)
                else:
                    cond.append(c)

        idList = db.getIdList(table, idCol=table['id'], where=cond, order='date, deadline')
        self.tblMRs.setIdList(idList, currentId)
        self.lblMRs.setText(formatRecordsCount(len(idList)))


    def addRequisition(self):
        counterController = QtGui.qApp.counterController()
        if not counterController:
            QtGui.qApp.setCounterController(CCounterController(self))
        dialog = CStockRequisitionEditDialog(self)
        try:
            dialog.setDefaults()
            dialog._generateStockMotionNumber()
            id = dialog.exec_()
            if id:
                self.updateMRsList(id)
                QtGui.qApp.delAllCounterValueIdReservation()
            else:
                QtGui.qApp.resetAllCounterValueIdReservation()
        finally:
            sip.delete(dialog)


    def editRequisition(self, id):
        dialog = CStockRequisitionEditDialog(self)
        try:
            dialog.load(id)
            id = dialog.exec_()
            if id:
                self.updateMRsList(id)
                QtGui.qApp.delAllCounterValueIdReservation()
            else:
                QtGui.qApp.resetAllCounterValueIdReservation()
        finally:
            sip.delete(dialog)


    def updateMRContent(self, requisitionId):
        self.tblMRContent.setIdList(self.getRequisitionItemIdList([requisitionId]))


    # Motions ########################################################

    def on_motionsSortByColumn(self, columnIndex):
        self._tableViewSorter('motions', self.tblMotions, columnIndex)
        self.updateMotionsList()

    def resetMotionsFilter(self):
        self.edtMotionsFilterNumber.setText('')
        self.edtMotionsFilterBegDate.setDate(QDate.currentDate().addDays(-7))
        self.edtMotionsFilterEndDate.setDate(QDate())
        self.edtMotionsFilterReason.setText('')
        self.edtMotionsFilterBegReasonDate.setDate(QDate())
        self.edtMotionsFilterEndReasonDate.setDate(QDate())
        self.cmbMotionsFilterSupplier.setValue(None)
        self.cmbMotionsFilterReceiver.setValue(None)
        self.cmbMotionsFilterNomenclature.setValue(None)
        self.cmbMotionsFilterFinance.setValue(None)
        self.edtMotionsClientFilterReceiver.setText('')
        self.edtMotionsFilterNote.setText('')
        self.cmbMotionsFilterType.setCurrentIndex(0)
        self.cmbMotionsFilterSupplierOrg.setValue(None)
        self.chkGroupMotions.setChecked(True)


    def applyMotionsFilter(self):
        filter = self.motionsFilter
        filter.number  = forceString(self.edtMotionsFilterNumber.text())
        filter.begDate = self.edtMotionsFilterBegDate.date()
        filter.endDate = self.edtMotionsFilterEndDate.date()
        filter.reason  = forceString(self.edtMotionsFilterReason.text())
        filter.begReasonDate = self.edtMotionsFilterBegReasonDate.date()
        filter.endReasonDate = self.edtMotionsFilterEndReasonDate.date()

        filter.supplierId = self.cmbMotionsFilterSupplier.value()
        filter.receiverId = self.cmbMotionsFilterReceiver.value()
        filter.clientId = forceString(self.edtMotionsClientFilterReceiver.text())
        filter.note = forceString(self.edtMotionsFilterNote.text())
        filter.nomenclatureId = self.cmbMotionsFilterNomenclature.value()
        filter.financeId = self.cmbMotionsFilterFinance.value()
        filter.type = self.cmbMotionsFilterType.currentIndex()-1
        filter.supplierOrgId = self.cmbMotionsFilterSupplierOrg.value()
        filter.groupMotions = self.chkGroupMotions.isChecked()
        self.updateMotionsList()


    def updateMotionsList(self, currentId=None):
        filter = self.motionsFilter
        db = QtGui.qApp.db
        table = db.table('StockMotion')
        tableItems = db.table('StockMotion_Item')
        cond = [
            table['deleted'].eq(0),
        ]
        if QtGui.qApp.currentOrgStructureId():
            cond.append(db.joinOr([ table['supplier_id'].eq(QtGui.qApp.currentOrgStructureId()),
                                    table['receiver_id'].eq(QtGui.qApp.currentOrgStructureId())
                                  ]
                                 )
                       )
        if filter.number:
            cond.append(table['number'].like(filter.number))
        if filter.begDate:
            cond.append(table['date'].ge(filter.begDate))
        if filter.endDate:
            cond.append(table['date'].lt(filter.endDate.addDays(1)))
        if filter.reason:
            cond.append(table['reason'].like(filter.reason))
        if filter.begReasonDate:
            cond.append(table['reasonDate'].ge(filter.begReasonDate))
        if filter.endReasonDate:
            cond.append(table['reasonDate'].lt(filter.endReasonDate.addDays(1)))
        if not filter.supplierOrgId and filter.supplierId:
            cond.append(table['supplier_id'].eq(filter.supplierId))
        elif filter.supplierOrgId:
            cond.append(table['supplierOrg_id'].eq(filter.supplierOrgId))
        if filter.receiverId:
#            cond.append(table['supplier_id'].inlist(getOrgStructureDescendants(filter.orgStructureId)))
            cond.append(table['receiver_id'].eq(filter.receiverId))
        if filter.note:
            cond.append(table['note'].like(filter.note))
        if filter.clientId:
            cond.append(table['client_id'].eq(filter.clientId))
        if filter.type >=0:
            cond.append(table['type'].eq(filter.type))
        if filter.nomenclatureId or filter.financeId:
            subcond = [tableItems['master_id'].eq(table['id'])]
            if filter.nomenclatureId:
                subcond.append(tableItems['nomenclature_id'].eq(filter.nomenclatureId))
            if filter.financeId:
                subcond.append(tableItems['finance_id'].eq(filter.financeId))
            cond.append(db.existsStmt(tableItems, subcond))

        sorterData = self._tableViewSorter.getData('motions')
        if sorterData is None:
            order = 'date DESC, id DESC'
        else:
            columnIndex = sorterData.columnIndex
            ascending = sorterData.ascending

            order = []
            if columnIndex in (4, 5):
                order.extend(['date', 'id'])
                ascending = False
            elif columnIndex == 0:
                data = sorted(stockMotionType.items(), key=lambda v: v[1][0])
                o = ['CASE']
                for idx, d in enumerate(data):
                    o.append('WHEN type = %s THEN %s' % (d[0], idx))
                o.append('ELSE 0 END')
                order.append(' '.join(o))
            elif columnIndex == 1:
                order.append('number')
            elif columnIndex == 2:
                order.append('date')
            elif columnIndex == 3:
                order.append('reason')
            elif columnIndex == 6:
                order.append('note')

            order = ', '.join(['{0} {1}'.format(v, ['DESC', 'ASC'][ascending]) for v in order])

        cols = [u'IF(StockMotion.`client_id`, StockMotion.`client_id`, StockMotion.`id`) as ident', 
                    table['id'], 
                    table['type'], 
                    table['number'], 
                    table['date'], 
                    table['reason'], 
                    table['supplierOrg_id'], 
                    table['supplier_id'], 
                    table['receiver_id'], 
                    table['client_id'], 
                    table['note'], 
                    u'''    (SELECT 
                        GROUP_CONCAT(DISTINCT StockMotion_Item.`nomenclature_id`
                                ORDER BY StockMotion_Item.`nomenclature_id` ASC
                                SEPARATOR ', ')
                    FROM
                        StockMotion_Item
                    WHERE
                        StockMotion_Item.`master_id` = StockMotion.`id`) AS motionItems'''
                    ]

        if filter.groupMotions:
            cols.append(u'GROUP_CONCAT(DISTINCT StockMotion.`id` ORDER BY StockMotion.`id` ASC SEPARATOR \', \') as idList')
            cols.append(u'COUNT(DISTINCT StockMotion.id) as motionsCount')
            group = u'StockMotion.type, ident, StockMotion.reasonDate, motionItems'
            groupCond = cond[:]
            queryTable = table
            records = db.getRecordListGroupBy(queryTable, cols, where=groupCond, group=group, order=order)
            currentMotionId = self.tblMotions.currentItemId()
            for rec in records:
                self._recordsIdDict[forceInt(rec.value('id'))] = rec
            self.records = records
            self.setGroupedMotionsToTable(records, currentMotionId=currentMotionId)
            self.lblMotions.setText(formatRecordsCount(len(records)))
        else:
            cols.append(u'StockMotion.`id` as idList')
            records = db.getRecordList(table, cols, where=cond, order=order)
            for rec in records:
                self._recordsIdDict[forceInt(rec.value('id'))] = rec
            currentMotionId = self.tblMotions.currentItemId()
            self.setGroupedMotionsToTable(records, currentMotionId)
            self.lblMotions.setText(formatRecordsCount(len(records)))
            self.tblMotionsItems.setCurrentItemId(currentMotionId)
        self.tblMotionsItems.setColumnHidden(0, not filter.groupMotions)


    def setGroupedMotionsToTable(self, records, currentMotionId=None):
        idList = []
        if records:
            for record in records:
                id = forceInt(record.value('id'))
                idList.append(id)
        self.tblMotions.setIdList(idList, currentMotionId)
        self.modelMotions.setRecords(records)
        self.tblMotionsItems.setCurrentItemId(currentMotionId)


    def addMotion(self, dialogClass, requisitionIdList = None):
        dialog = dialogClass(self)
        try:
            dialog.setDefaults()
            if requisitionIdList:
                dialog.setRequsitions(requisitionIdList)
            id = dialog.exec_()
            if id:
                self.updateMotionsList(id)
                QtGui.qApp.delAllCounterValueIdReservation()
            else:
                QtGui.qApp.resetAllCounterValueIdReservation()
        finally:
            sip.delete(dialog)


    def addInvoice(self, requisitionIdList = None):
        self.addMotion(CInvoiceEditDialog, requisitionIdList)


    def addInventory(self):
        self.addMotion(CInventoryEditDialog)


    def addFinTransfer(self):
        self.addMotion(CFinTransferEditDialog)


    def addProduction(self):
        self.addMotion(CProductionEditDialog)

    def addUtilization(self):
        self.addMotion(CStockUtilizationEditDialog)

    def addInternalConsumption(self):
        self.addMotion(CStockInternalConsumptionEditDialog)


    def addAddClientRefundInvoice(self, clientInvoiceId):
        dialog = CClientRefundInvoiceEditDialog(self)
        try:
            dialog.setDefaults()
            if clientInvoiceId:
                dialog.loadDataFromClientInvoice(clientInvoiceId)
            id = dialog.exec_()
            if id:
                self.updateMotionsList(id)
                QtGui.qApp.delAllCounterValueIdReservation()
            else:
                QtGui.qApp.resetAllCounterValueIdReservation()
        finally:
            sip.delete(dialog)


    @pyqtSignature('')
    def on_actDeleteRow_triggered(self):
        self.tblMotions.removeSelectedRows()
        self.updateMotionsList()


    # Remainings #####################################################

    def resetRemainingsFilter(self):
        self.edtRemainingsFilterDate.setDate(QDate())
        self.cmbRemainingsFilterStorage.setValue(QtGui.qApp.currentOrgStructureId())
        self.cmbRemainingsFilterNomenclature.setValue(None)
        self.chkRemainingsFilterGroupByBatch.setChecked(False)
        self.edtRemainingsFilterBatch.setText('')
        self.edtRemainingsFilterBegShelfTime.setDate(QDate())
        self.edtRemainingsFilterEndShelfTime.setDate(QDate())
        self.cmbRemainingsFilterFinance.setValue(None)
        self.chkRemainingsFilterConstrainedQnt.setChecked(False)
        self.chkRemainingsFilterOrderQnt.setChecked(False)
        self.chkRemainingsFilterAvailable.setChecked(False)
        self.cmbRemainingsFilterUnit.setValue(None)
        self.edtMotionsClientFilterReceiver.setText('')
        self.edtMotionsFilterNote.setText('')


    def applyRemainingsFilter(self):
        filter = self.remainingsFilter
        filter.date = self.edtRemainingsFilterDate.date()
        filter.storageId = self.cmbRemainingsFilterStorage.value()
        filter.nomenclatureId = self.cmbRemainingsFilterNomenclature.value()
        filter.groupByBatch = self.chkRemainingsFilterGroupByBatch.isChecked()
        filter.batch = forceStringEx(self.edtRemainingsFilterBatch.text())
        filter.begShelfTime = self.edtRemainingsFilterBegShelfTime.date()
        filter.endShelfTime = self.edtRemainingsFilterEndShelfTime.date()
        filter.financeId = self.cmbRemainingsFilterFinance.value()
        filter.constrainedQnt = self.chkRemainingsFilterConstrainedQnt.isChecked()
        filter.orderQnt = self.chkRemainingsFilterOrderQnt.isChecked()
        filter.available = self.chkRemainingsFilterAvailable.isChecked()
        filter.unit = self.cmbRemainingsFilterUnit.value()
        self.updateRemainingsList()


    def updateRemainingsList(self, currentId=None):
        filter = self.remainingsFilter
        self.modelRemainings.loadData(filter.date,
                                      filter.storageId,
                                      filter.nomenclatureId,
                                      filter.groupByBatch,
                                      filter.batch,
                                      filter.begShelfTime,
                                      filter.endShelfTime,
                                      filter.financeId,
                                      filter.constrainedQnt,
                                      filter.orderQnt,
                                      filter.available, 
                                      filter.unit
                                      )


    # RTMs ##(requests to me)####################################################

    def resetRTMsFilter(self):
        self.chkRTMsFilterOnlyActive.setChecked(True)
        self.edtRTMsFilterBegDate.setDate(QDate())
        self.edtRTMsFilterEndDate.setDate(QDate())
        self.edtRTMsFilterBegDeadlineDate.setDate(QDate())
        self.edtRTMsFilterEndDeadlineDate.setDate(QDate())
        self.edtRTMsFilterBegDeadlineTime.setTime(QTime())
        self.edtRTMsFilterEndDeadlineTime.setTime(QTime())
        self.cmbRTMsFilterOrgStructure.setValue(None)
        self.cmbRTMsFilterNomenclature.setValue(None)
        self.cmbRTMsFilterSatisfied.setCurrentIndex(CSatisifiedFilter.ALL)


    def applyRTMsFilter(self):
        self.RTMsFilter.onlyActive = self.chkRTMsFilterOnlyActive.isChecked()
        self.RTMsFilter.begDate = self.edtRTMsFilterBegDate.date()
        self.RTMsFilter.endDate = self.edtRTMsFilterEndDate.date()
        self.RTMsFilter.begDeadline = self.composeDateTime(self.edtRTMsFilterBegDeadlineDate.date(), self.edtRTMsFilterBegDeadlineTime.time())
        self.RTMsFilter.endDeadline = self.composeDateTime(self.edtRTMsFilterEndDeadlineDate.date(), self.edtRTMsFilterEndDeadlineTime.time())
        self.RTMsFilter.orgStructureId = self.cmbRTMsFilterOrgStructure.value()
        self.RTMsFilter.nomenclatureId = self.cmbRTMsFilterNomenclature.value()
        self.RTMsFilter.satisfiedFilter = self.cmbRTMsFilterSatisfied.currentIndex()
        self.updateRTMsList()


    def updateRTMsList(self, currentId=None):
        filter = self.RTMsFilter
        db = QtGui.qApp.db
        table = db.table('StockRequisition')
        tableItems = db.table('StockRequisition_Item')
        cond = [table['deleted'].eq(0),
                table['supplier_id'].eq(QtGui.qApp.currentOrgStructureId()),
               ]
        if filter.orgStructureId:
            cond.append(table['recipient_id'].inlist(getOrgStructureDescendants(filter.orgStructureId)))
        if filter.onlyActive:
            cond.append(table['revoked'].eq(0))
        if filter.begDate:
            cond.append(table['date'].ge(filter.begDate))
        if filter.endDate:
            cond.append(table['date'].le(filter.endDate))
        if filter.begDeadline:
            cond.append(table['deadline'].ge(filter.begDeadline))
        if filter.endDeadline:
            cond.append(table['deadline'].le(filter.endDeadline))
        if filter.nomenclatureId:
            subcond = [tableItems['master_id'].eq(table['id']),
                       tableItems['nomenclature_id'].eq(filter.nomenclatureId),
                      ]
            if filter.onlyActive:
                subcond.append(tableItems['qnt'].ge(tableItems['satisfiedQnt']))
            cond.append(db.existsStmt(tableItems, subcond))
        if filter.satisfiedFilter:
            c = CSatisifiedFilter.getFilter(filter.satisfiedFilter, filter.nomenclatureId)
            if c is not None:
                cond.append(c)
        idList = db.getIdList(table, idCol=table['id'], where=cond, order='date, deadline')
        self.tblRTMs.setIdList(idList, currentId)
        self.lblRTMs.setText(formatRecordsCount(len(idList)))


    def updateRTMContent(self, requisitionId):
        self.tblRTMContent.setIdList(self.getRequisitionItemIdList(requisitionId))


    # slots ####################################################################

    def on_actRemainingsShowStockMotions_triggered(self):
        item = self.tblRemainings.currentItem()
        nomenclatureId = forceRef(item.value('nomenclature_id'))
        batch = forceString(item.value('batch'))
        financeId = forceRef(item.value('finance_id'))
        orgStructureId = self.remainingsFilter.storageId
        records = getRemainingHistory(nomenclatureId, batch=batch, financeId=financeId, orgStructureId=orgStructureId)
        showRemainingHistoryReport(records, self)



    @pyqtSignature('int')
    def on_tabWidget_currentChanged(self, index):
        if index == 0:
            self.syncSplitters(self.splitterMotions)
#            self.btnPrint.setEnabled(True)
#            self.btnCash.setEnabled(False)
            self.applyMotionsFilter()
        elif index == 1:
            self.syncSplitters(self.splitterRemainings)
            self.applyRemainingsFilter() # обновляем всегда
        elif index == 2:
            self.syncSplitters(self.splitterMRs)
            if not self.visitedMRs:
                self.resetMRsFilter()
                self.visitedMRs = True
            self.applyMRsFilter()

        elif index == 3:
            self.syncSplitters(self.splitterRTMs)
            if not self.visitedRTMs:
                self.resetRTMsFilter()
                self.visitedRTMs = True
            self.applyRTMsFilter()
        
        self.preparePrintBtn(index)


    # Motions ########################################################

    @pyqtSignature('int')
    def on_cmbMotionsFilterSupplierOrg_currentIndexChanged(self):
        value = self.cmbMotionsFilterSupplierOrg.value()
        self.cmbMotionsFilterSupplier.setEnabled(not value)

    @pyqtSignature('QAbstractButton*')
    def on_bbxMotionsFilter_clicked(self, button):
        buttonCode = self.bbxMotionsFilter.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.applyMotionsFilter()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.resetMotionsFilter()
            self.applyMotionsFilter()


    @pyqtSignature('')
    def on_actAddInvoice_triggered(self):
        self.addInvoice()


    @pyqtSignature('')
    def on_actPrintInvoice_triggered(self):
        currWidget = self.tabWidget.currentIndex()
        if currWidget == 0:
            tbl = self.tblMotions
        elif currWidget == 2:
            tbl = self.tblMRs
        elif currWidget == 3:
            tbl = self.tblRTMs
        id = tbl.currentItemId()
        if id:
            if currWidget == 0:
                motionType = forceInt(QtGui.qApp.db.translate('StockMotion', 'id', id, 'type'))
                if motionType == 0:
                    CReportStockM11(self, id, 0).exec_()
            elif currWidget in [2, ]:
                CReportStockM11(self, id, 1).exec_()


    @pyqtSignature('')
    def on_actAddInventory_triggered(self):
        self.addInventory()


    @pyqtSignature('')
    def on_actAddFinTransfer_triggered(self):
        self.addFinTransfer()


    @pyqtSignature('')
    def on_actAddProduction_triggered(self):
        self.addProduction()

    @pyqtSignature('')
    def on_actAddUtilization_triggered(self):
        self.addUtilization()

    @pyqtSignature('')
    def on_actAddInternalConsumption_triggered(self):
        self.addInternalConsumption()

    @pyqtSignature('')
    def on_actEditMotion_triggered(self):
        id = self.tblMotions.currentItemId()
        if id:
            personId = forceInt(QtGui.qApp.db.translate('StockMotion', 'id', id, 'createPerson_id'))
            if (QtGui.qApp.userHasRight(urEditOwnMotions) and QtGui.qApp.userId == personId) or (QtGui.qApp.userHasRight(urEditOtherMotions) and  QtGui.qApp.userId != personId):
                if editStockMotion(self, id):
                    self.updateMotionsList(id)
            else:
                openReadOnlyMotion(self, id)


    @pyqtSignature('')
    def on_actAddClientRefundInvoice_triggered(self):
        id = self.tblMotions.currentItemId()
        if id:
            motionType = forceInt(QtGui.qApp.db.translate('StockMotion', 'id', id, 'type'))
            if motionType == 4:
                self.addAddClientRefundInvoice(id)


    @pyqtSignature('QModelIndex')
    def on_tblMotions_doubleClicked(self, index):
        self.on_actEditMotion_triggered()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelMotions_currentRowChanged(self, current, previous):
        motionId = self.tblMotions.itemId(current)
        self.updateMotionsItemsList(motionId)


    def updateMotionsItemsList(self, motionId):
        db = QtGui.qApp.db
        motionIdList = []
        tableStockMotionItem = db.table('StockMotion_Item')
        
        motionsString = forceString(self._recordsIdDict[motionId].value('idList')).split(",")
        
        for motion in motionsString:
            motionIdList.append(motion)
        
        queryTable = tableStockMotionItem
        cond = queryTable['master_id'].inlist(motionIdList)
        idList = db.getIdList(queryTable, tableStockMotionItem['id'], cond)
        currentMotionItemId = self.tblMotions.currentItemId()
        self.modelMotionsItems.setIdList(idList)
        self.tblMotionsItems.setCurrentItemId(currentMotionItemId)


    @pyqtSignature('')
    def on_actOpenMotion_triggered(self):
        db = QtGui.qApp.db
        id = self.tblMotionsItems.currentItemId()
        motionId = db.translate('StockMotion_Item', 'id', id, 'master_id')
        if id:
            personId = forceInt(QtGui.qApp.db.translate('StockMotion', 'id', id, 'createPerson_id'))
            if (QtGui.qApp.userHasRight(urEditOwnMotions) and QtGui.qApp.userId == personId) or (QtGui.qApp.userHasRight(urEditOtherMotions) and  QtGui.qApp.userId != personId):
                if editStockMotion(self, motionId):
                    self.updateMotionsList(motionId)
            else:
                openReadOnlyMotion(self, motionId)


    # Remainings #####################################################


    @pyqtSignature('bool')
    def on_chkRemainingsFilterGroupByBatch_toggled(self, val):
        self.lblRemainingsFilterBatch.setEnabled(val)
        self.edtRemainingsFilterBatch.setEnabled(val)
        self.lblRemainingsFilterBegShelfTime.setEnabled(val)
        self.edtRemainingsFilterBegShelfTime.setEnabled(val)
        self.lblRemainingsFilterEndShelfTime.setEnabled(val)
        self.edtRemainingsFilterEndShelfTime.setEnabled(val)


    @pyqtSignature('QAbstractButton*')
    def on_bbxRemainingsFilter_clicked(self, button):
        buttonCode = self.bbxRemainingsFilter.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.applyRemainingsFilter()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.resetRemainingsFilter()
            self.applyRemainingsFilter()


    # MR ##(my requests)########################################################

    @pyqtSignature('const QDate&')
    def on_edtMRsFilterBegDeadlineDate_dateChanged(self, date):
        self.edtMRsFilterBegDeadlineTime.setEnabled(bool(date))


    @pyqtSignature('const QDate&')
    def on_edtMRsFilterEndDeadlineDate_dateChanged(self, date):
        self.edtMRsFilterEndDeadlineTime.setEnabled(bool(date))


    @pyqtSignature('QAbstractButton*')
    def on_bbxMRsFilter_clicked(self, button):
        buttonCode = self.bbxMRsFilter.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.applyMRsFilter()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.resetMRsFilter()
            self.applyMRsFilter()

    @pyqtSignature('')
    def on_actAddRequisition_triggered(self):
        self.addRequisition()

    @pyqtSignature('')
    def on_actEditRequisition_triggered(self):
        id = self.tblMRs.currentItemId()
        if id:
            self.editRequisition(id)
            self.updateMRContent(id)


    @pyqtSignature('')
    def on_actRevokeRequisition_triggered(self):
        id = self.tblMRs.currentItemId()
        if id:
            db = QtGui.qApp.db
            record = self.modelMRs.getRecordById(id)
            record.setValue('revoked', QVariant(True))
            table = db.table('StockRequisition')
            db.updateRecord(table, record)
            self.applyMRsFilter()


    @pyqtSignature('')
    def on_actShowOverhead_triggered(self):
        id = self.tblMRs.currentItemId()
        if id:
            record = self.modelMRs.getRecordById(id)
            if not record:
                db = QtGui.qApp.db
                tableSR = db.table('StockRequisition')
                tableOSM = db.table('vStockMotion')



    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelMRs_currentRowChanged(self, current, previous):
        id = self.tblMRs.itemId(current)
        self.updateMRContent(id)


    # RTMs ##(requests to me)####################################################

    @pyqtSignature('const QDate&')
    def on_edtRTMsFilterBegDeadlineDate_dateChanged(self, date):
        self.edtRTMsFilterBegDeadlineTime.setEnabled(bool(date))


    @pyqtSignature('const QDate&')
    def on_edtRTMsFilterEndDeadlineDate_dateChanged(self, date):
        self.edtRTMsFilterEndDeadlineTime.setEnabled(bool(date))


    @pyqtSignature('QAbstractButton*')
    def on_bbxRTMsFilter_clicked(self, button):
        buttonCode = self.bbxRTMsFilter.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.applyRTMsFilter()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.resetRTMsFilter()
            self.applyRTMsFilter()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelRTMs_currentRowChanged(self, current, previous):
        id = self.tblRTMs.itemId(current)
        self.updateRTMContent([id])


    @pyqtSignature('')
    def on_actCreateMotionByRequisition_triggered(self):
        RTMsIdList = self.tblRTMs.selectedItemIdList()
        if RTMsIdList:
            self.addInvoice(RTMsIdList)
            self.updateRTMContent(RTMsIdList)

#
# ##############################################################################
#

class CReceiverCol(CRefBookCol):
    def __init__(self,  *args):
        CRefBookCol.__init__(self, *args)
        self.mapClientIdToText = {}


    def format(self, values):
        orgStructureId = forceRef(values[0])
        if orgStructureId:
            return CRefBookCol.format(self, values)
        clientId = forceRef(values[1])
        if clientId:
            text = self.mapClientIdToText.get(clientId, None)
            if text is None:
                text = getClientMiniInfo(clientId)
                self.mapClientIdToText[clientId] = text
            return QVariant(text)
        return QVariant()


    def invalidateRecordsCache(self):
        self.mapClientIdToText = {}
        CRefBookCol.invalidateRecordsCache(self)


class CMyMotionsModel(CTableModel):
    class CSupplierCol(CCol):
        def __init__(self):
            CCol.__init__(self, u'Поставщик', ['supplierOrg_id', 'supplier_id'], defaultWidth=15, alignment='l')
            self.orgStructureCache = CRBModelDataCache.getData('OrgStructure', True, '')
            self.organisationCache = CDbDataCache.getData('Organisation', 'shortName', filter='isSupplier=1', addNone=True, noneText='', order='id')

        def format(self, values):
            supplierOrgId, supplierId = forceRef(values[0]), forceRef(values[1])
            if supplierOrgId:
                if supplierOrgId in self.organisationCache.idList:
                    dataIndex = self.organisationCache.idList.index(supplierOrgId)
                    value = self.organisationCache.strList[dataIndex]
                else:
                    value = u'{ %d }' % supplierOrgId
                return toVariant(value)

            elif supplierId:
                return toVariant(self.orgStructureCache.getStringById(int(supplierId), CRBComboBox.showName))

            else:
                return CCol.invalid

        def invalidateRecordsCache(self):
            self.orgStructureCache = CRBModelDataCache.getData('OrgStructure', True, '')
            self.organisationCache = CDbDataCache.getData('Organisation', 'shortName', filter='isSupplier=1', addNone=True, noneText='', order='id')


    def __init__(self, parent):
        CTableModel.__init__(self, parent, [
            CEnumCol(     u'Тип',           ['type'],  (u'Накладная',
                                                        u'Инвентаризация',
                                                        u'Фин.переброска',
                                                        u'Производство',
                                                        u'Списание',
                                                        u'Возврат',
                                                        u'Резерв',
                                                        u'Утилизация', 
                                                        u'Внутреннее потребление'
                                                       ),  20),

            CTextCol(     u'Номер',         ['number'], 20),
            CDateTimeCol( u'Дата и время',  ['date'],   20),
            CTextCol(     u'Основание',     ['reason'], 20),
            CMyMotionsModel.CSupplierCol(),
            CReceiverCol( u'Получатель',    ['receiver_id', 'client_id'], 'OrgStructure', 15),
            CTextCol(     u'Примечание',    ['note'],  20),
            ], 'StockMotion' )
        self._records = {}
        self._boldRows = {}
    
    
    def setRecords(self, records):
        self._records = {}
        self._boldRows = {}
        for row, record in enumerate(records):
            self._records[forceInt(record.value("id"))] = record
            if forceInt(record.value('motionsCount'))>1:
                self._boldRows[row] = True
    
    
    def getRecordValues(self, column, row):
        col = self._cols[column]
        if self._prevColumn != column or self._prevRow != row or self._prevData is None:
            recordId = self._idList[row]
            record = self._records.get(recordId)
            self._prevData   = col.extractValuesFromRecord(record)
            self._prevColumn = column
            self._prevRow    = row
        return (col, self._prevData)
    
    
    def data(self, index, role=Qt.DisplayRole):
        if not index.isValid():
            return QVariant()
        #column = index.column()
        row    = index.row()
        if role == Qt.FontRole:
            font = QtGui.QFont()
            if self._boldRows.get(row, False):
                font.setBold(True)
            return font
        else:
            return CTableModel.data(self, index, role)


class CMyMotionsItemsModel(CTableModel):
    class CDateCol(CCol):
        def __init__(self, title, fields, defaultWidth, alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.date = None

        def format(self, values):
            recordId = forceRef(values[1].value('master_id'))
            if recordId:
                if self.date:
                    return self.date
                else:
                    self.data = QtGui.qApp.db.translate('StockMotion',  'id', recordId, 'date')  #CRBModelDataCache.getData(tableName, True, '')
                    return self.data
            else:
                return CCol.invalid

        def invalidateRecordsCache(self):
            self.data = None


    def __init__(self, parent):
        CTableModel.__init__(self, parent, [
            CMyMotionsItemsModel.CDateCol( u'Дата и время',  ['master_id'],   20),
            CRefBookCol(     u'ЛСиИМН',           ['nomenclature_id'],  'rbNomenclature',  20),
            CTextCol(     u'Партия',     ['batch'], 20),
            CDateCol( u'Годен до',  ['shelfTime'],   20),
            CRefBookCol(     u'Тип финансирования',     ['finance_id'], 'rbFinance',  20),
            CTextCol(     u'Кол-во',     ['qnt'], 20),
            CRefBookCol(     u'Ед.Учета',         ['unit_id'], 'rbUnit', 20),
            CTextCol(     u'Сумма',     ['sum'], 20),
            ], 'StockMotion_Item' )


#
# ##############################################################################
#

class CRemainingsModel(CRecordListModel):
    class CSumCol(CFloatInDocTableCol):
        def _toString(self, value):
            if value.isNull():
                return None
            sum = forceDouble(value)
            if sum == -0:
                sum = 0.0
            return format(sum, '.2f')

    class CBaseLocNomenclatureValuesCol(CInDocTableCol):
        def __init__(self, nomenclatureCache, *args, **params):
            CInDocTableCol.__init__(self, *args, **params)
            self._cache = {None: QVariant()}
            self._nomenclatureCache = nomenclatureCache

        def getNomenclatureValues(self, nomenclatureId):
            if nomenclatureId not in self._nomenclatureCache:
                record = QtGui.qApp.db.getRecord('rbNomenclature', 'lfForm_id, dosageValue', nomenclatureId)
                self._nomenclatureCache[nomenclatureId] = {'lfForm_id': record.value('lfForm_id'),
                                                           'dosageValue': record.value('dosageValue')}
            return self._nomenclatureCache[nomenclatureId]

    class CNomenclatureFormCol(CBaseLocNomenclatureValuesCol):
        def toString(self, val, record):
            nomenclatureId = forceRef(val)
            if not nomenclatureId in self._cache:
                nomenclatureValues = self.getNomenclatureValues(nomenclatureId)
                lfFormId = forceRef(nomenclatureValues['lfForm_id'])
                if lfFormId:
                    cache = CRBModelDataCache.getData('rbLfForm')
                    result = QVariant(cache.getStringById(lfFormId, CRBComboBox.showName))
                else:
                    result = QVariant()
                self._cache[nomenclatureId] = result
            return self._cache[nomenclatureId]

    class CNomenclatureDosageCol(CBaseLocNomenclatureValuesCol):
        def toString(self, val, record):
            nomenclatureId = forceRef(val)
            nomenclatureValues = self.getNomenclatureValues(nomenclatureId)
            return nomenclatureValues['dosageValue']


    def __init__(self, parent):
        CRecordListModel.__init__(self, parent)
        self._cacheNomenclatureValues = {}
        self.addCol(CRBInDocTableCol(u'Подразделение', 'orgStructure_id', 50, 'OrgStructure', showFields = CRBComboBox.showCode).setSortable())
        self.addCol(CNomenclatureInDocTableCol(u'ЛСиИМН', 'nomenclature_id', 50, showFields = CRBComboBox.showName).setSortable())
        self.addCol(CInDocTableCol( u'Партия', 'batch', 16).setSortable())
        self.addCol(CDateInDocTableCol( u'Годен до', 'shelfTime', 12, canBeEmpty=True).setSortable())
        self.addCol(CRBInDocTableCol(    u'Тип финансирования', 'finance_id', 15, 'rbFinance').setSortable())
        self.addCol(getStockMotionItemQuantityColumn( u'Кол-во', 'qnt', 12).setSortable())
        self.addCol(CRBInDocTableCol(    u'Ед.Учета.', 'unitId', 15, 'rbUnit').setSortable())
        self.addCol(CRemainingsModel.CSumCol( u'Сумма', 'sum', 12).setSortable())
        self.addCol(CFloatInDocTableCol( u'Гантированный запас', 'constrainedQnt', 12).setSortable())
        self.addCol(CFloatInDocTableCol( u'Точка заказа', 'orderQnt', 12).setSortable())
        self.addCol(CRemainingsModel.CNomenclatureDosageCol(
            self._cacheNomenclatureValues, u'Дозировка', 'nomenclature_id', 12).setSortable()
        )
        self.addCol(CRemainingsModel.CNomenclatureFormCol(
            self._cacheNomenclatureValues, u'Форма выпуска', 'nomenclature_id', 12).setSortable()
        )
        self._cachedRow = None
        self._cachedRowColor = None


    def flags(self, index):
        return Qt.ItemIsSelectable|Qt.ItemIsEnabled


    def getRowColor(self, row):
        if self._cachedRow != row:
            self._cachedRow = row
            record = self._items[row]
            varQnt = record.value('qnt')
            varConstrainedQnt = record.value('constrainedQnt')
            varOrderQnt = record.value('orderQnt')
            self._cachedRowColor = None
            if not (varConstrainedQnt.isNull() or varOrderQnt.isNull()):
                qnt = forceDouble(varQnt)
                constrainedQnt = forceDouble(varConstrainedQnt)
                orderQnt = forceDouble(varOrderQnt)
                if qnt<constrainedQnt:
                    self._cachedRowColor = QtGui.QColor(Qt.darkRed)
                elif qnt<orderQnt:
                    self._cachedRowColor = QtGui.QColor(Qt.darkGreen)
        return self._cachedRowColor


    def data(self, index, role=Qt.DisplayRole):
        if role == Qt.ForegroundRole:
            row = index.row()
            return toVariant(self.getRowColor(row))
        else:
            return CRecordListModel.data(self, index, role)


    def loadData(self, date, orgStructureId, nomenclatureId,
                 groupByBatch, batch, begShelfTime, endShelfTime,
                 financeId, constrainedQnt, orderQnt, available, unit):
        self._items = []
        db = QtGui.qApp.db
        table = db.table('StockTrans')
        tableOSS = db.table('OrgStructure_Stock')
        debCond = []
        creCond = []
        ossCond = []
        if date:
            debCond.append(table['date'].dateLe(date))
            creCond.append(table['date'].dateLe(date))
        if orgStructureId:
            debCond.append(table['debOrgStructure_id'].eq(orgStructureId))
            creCond.append(table['creOrgStructure_id'].eq(orgStructureId))
            ossCond.append(tableOSS['master_id'].eq(orgStructureId))
        else:
            debCond.append(table['debOrgStructure_id'].isNotNull())
            creCond.append(table['creOrgStructure_id'].isNotNull())
        if nomenclatureId:
            debCond.append(table['debNomenclature_id'].eq(nomenclatureId))
            creCond.append(table['creNomenclature_id'].eq(nomenclatureId))
            ossCond.append(tableOSS['nomenclature_id'].eq(nomenclatureId))

        if groupByBatch:
            batchFields = 'StockTrans.batch, StockTrans.shelfTime,'
            sqlGroupByBatch = 'batch, shelfTime, '
            if batch:
                t = table['batch'].eq(batch)
                debCond.append(t)
                creCond.append(t)
            if begShelfTime:
                t = table['shelfTime'].ge(begShelfTime)
                debCond.append(t)
                creCond.append(t)
            if endShelfTime:
                t = table['shelfTime'].le(endShelfTime)
                debCond.append(t)
                creCond.append(t)
        else:
            batchFields = '\'\' AS batch, NULL AS shelfTime,'
            sqlGroupByBatch = ''
        if financeId:
            debCond.append(table['debFinance_id'].eq(financeId))
            creCond.append(table['creFinance_id'].eq(financeId))
            ossCond.append(tableOSS['finance_id'].eq(financeId))
        havCond = ['(`qnt` != 0 OR `sum` != 0 OR `constrainedQnt` IS NOT NULL)']
        if constrainedQnt:
            havCond.append('`qnt`<`constrainedQnt`')
        if orderQnt:
            havCond.append('`qnt`<`orderQnt`')
        
        if unit:
            unitId = unit
            unitCol = '%s AS unitId' %unitId
            unitParams = '''
    (SELECT 
                rbNomenclature_UnitRatio.ratio
            FROM
                rbNomenclature_UnitRatio
            WHERE
                rbNomenclature_UnitRatio.sourceUnit_id = RBUSource.id
                    AND rbNomenclature_UnitRatio.targetUnit_id =  %(unitId)s
                    AND rbNomenclature_UnitRatio.master_id = rbNomenclature.id) AS `ratio`,
    (SELECT 
            rbUnit.name
        FROM
            rbNomenclature_UnitRatio
                LEFT JOIN
            rbUnit ON rbUnit.id = rbNomenclature_UnitRatio.targetUnit_id
        WHERE
            rbNomenclature_UnitRatio.sourceUnit_id = RBUSource.id
                AND rbNomenclature_UnitRatio.targetUnit_id = %(unitId)s
                AND rbNomenclature_UnitRatio.master_id = rbNomenclature.id) AS `unitName`
            '''%{
            'unitId':unitId, 
            }
        else:
            unitCol = 'RBUSource.id AS `unitId`'
            unitParams = '1'

        stmt = u'''
SELECT T.orgStructure_id,
       T.nomenclature_id,
       T.batch,
       T.shelfTime,
       T.finance_id,
       sum(T.qnt) AS `qnt`,
       sum(T.`sum`) AS `sum`,
       %(unitCol)s,
       %(unitParams)s,
       OrgStructure_Stock.constrainedQnt AS constrainedQnt,
       OrgStructure_Stock.orderQnt AS orderQnt
FROM
    (
    SELECT debOrgStructure_id AS orgStructure_id,
           debNomenclature_id AS nomenclature_id,
           %(batchFields)s
           debFinance_id      AS finance_id,
           sum(StockTrans.qnt)           AS `qnt`,
           sum(StockTrans.`sum`)         AS `sum`
    FROM StockTrans
    WHERE %(debCond)s
    GROUP BY debOrgStructure_id, debNomenclature_id, %(groupByBatch)s debFinance_id
    UNION ALL
    SELECT creOrgStructure_id AS orgStructure_id,
           creNomenclature_id AS nomenclature_id,
           %(batchFields)s
           creFinance_id      AS finance_id,
           -sum(StockTrans.qnt)          AS `qnt`,
           -sum(StockTrans.`sum`)        AS `sum`
    FROM StockTrans
    WHERE %(creCond)s
    GROUP BY creOrgStructure_id, creNomenclature_id, %(groupByBatch)s creFinance_id
    UNION ALL
    SELECT master_id          AS orgStructure_id,
           nomenclature_id    AS nomenclature_id,
           ''                 AS batch,
           NULL               AS shelfTime,
           finance_id         AS finance_id,
           0                  AS `qnt`,
           0                  AS `sum`
    FROM OrgStructure_Stock
    WHERE %(ossCond)s
    GROUP BY master_id, nomenclature_id, finance_id
    ) AS T
    LEFT JOIN OrgStructure ON OrgStructure.id = T.orgStructure_id
    LEFT JOIN rbNomenclature ON rbNomenclature.id = T.nomenclature_id
    LEFT JOIN rbUnit AS RBUSource ON rbNomenclature.defaultStockUnit_id = RBUSource.id
    LEFT JOIN rbFinance ON rbFinance.id = T.finance_id
    LEFT JOIN OrgStructure_Stock ON OrgStructure_Stock.master_id = T.orgStructure_id
          AND OrgStructure_Stock.nomenclature_id = T.nomenclature_id
          AND OrgStructure_Stock.finance_id = T.finance_id
GROUP BY orgStructure_id, nomenclature_id, %(groupByBatch)s finance_id
HAVING (%(havCond)s)
ORDER BY OrgStructure.code, rbNomenclature.code, rbNomenclature.name, %(groupByBatch)s rbFinance.code
''' % {
        'debCond' : db.joinAnd(debCond) if debCond else '1',
        'creCond' : db.joinAnd(creCond) if creCond else '1',
        'ossCond' : db.joinAnd(ossCond) if ossCond else '1',
        'havCond' : db.joinAnd(havCond),
        'groupByBatch' : sqlGroupByBatch,
        'batchFields'  : batchFields,
        'unitCol':unitCol,
        'unitParams':unitParams, 
      }
        query = db.query(stmt)
        if available:
            while query.next():
                record = query.record()
                # Отображаем теперь только те которые в наличии, то есть положительное количество.
                if round(forceDouble(record.value('qnt')), 2) > 0:
                    if unit:
                        if forceDouble(record.value('ratio')):
                            itemQnt = forceDouble(record.value('qnt'))*forceDouble(record.value('ratio'))
                            record.setValue('qnt', toVariant(itemQnt))
                            self._items.append(record)
                    else:
                        self._items.append(record)
        else:
            while query.next():
                record = query.record()
                if abs(round(forceDouble(record.value('qnt')), 2)) > 0:
                    if unit:
                        if forceDouble(record.value('ratio')):
                            itemQnt = forceDouble(record.value('qnt'))*forceDouble(record.value('ratio'))
                            record.setValue('qnt', toVariant(itemQnt))
                            self._items.append(query.record())
                    else:
                        self._items.append(query.record())
        self.reset()
#
# ##############################################################################
#

class CMyRequisitionsModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent, [
            CTextCol(     u'Номер',         ['number'], 20),
            CDateCol(     u'Дата',          ['date'],  20),
            CDateTimeCol( u'Срок',          ['deadline'], 20),
            CRefBookCol(  u'Поставщик',     ['supplier_id'], 'OrgStructure', 15),
            CTextCol(     u'Примечание',    ['note'],  20),
            ], '' )
        self.loadField('revoked')
        self.setTable('StockRequisition')

#
# ##############################################################################
#

class CRequisitionsToMeModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent, [
            CTextCol(     u'Номер',         ['number'], 20),
            CDateCol(     u'Дата',          ['date'],  20),
            CDateTimeCol( u'Срок',          ['deadline'], 20),
            CRefBookCol(  u'Заказчик',      ['recipient_id'], 'OrgStructure', 15),
            CTextCol(     u'Примечание',    ['note'],  20),
            ], 'StockRequisition' )


#
# ##############################################################################
#

class CRequisitionContentModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent, [
            CRefBookCol(  u'ЛСиИМН',             ['nomenclature_id'], 'rbNomenclature', 50),
            CRefBookCol(  u'Тип финансирования', ['finance_id'],      'rbFinance', 15),
            CTextCol(     u'Кол-во',             ['qnt'],  12),
            CTextCol(     u'Отпущено',           ['satisfiedQnt'],  12),
            ], 'StockRequisition_Item' )
