# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4 import QtCore
from PyQt4.QtCore import QDateTime, pyqtSignature

from library.ItemsListDialog               import CItemEditorBaseDialog
from library.interchange                   import (getDateEditValue, getDatetimeEditValue, getLineEditValue, getRBComboBoxValue,
                                                   setDateEditValue, setDatetimeEditValue, setLineEditValue, setRBComboBoxValue)
from library.InDocTable import CInDocTableModel

from library.Utils                         import forceDouble, forceRef, forceInt, toVariant
from library.Counter import CCounterController

from Stock.Utils import getNomenclatureUnitRatio, CStockCache, getStockMotionNumberCounterId


class CStockMotionItemsCopyPasteMixin(object):
    fieldsNotToPaste = ('qnt', 'sum')

    def __init__(self, parent):
        self._parent = parent

    def _initView(self):
        self.addObject('actCopyItems', QtGui.QAction(u'Копировать выделенное', self))
        self.addObject('actPasteItems', QtGui.QAction(u'Вставить из буфера', self))

        self.tblItems.addPopupAction(self.actCopyItems)
        self.tblItems.addPopupAction(self.actPasteItems)

        self.tblItems.setActionsWithCheckers(((self.actCopyItems, self._checkActionCopyItemsEnabled),
                                              (self.actPasteItems, self._checkActionPasteItemsEnabled)))

        self.connect(self.actCopyItems, QtCore.SIGNAL('triggered()'), self.on_actCopyItems)
        self.connect(self.actPasteItems, QtCore.SIGNAL('triggered()'), self.on_actPasteItems)

    def _checkActionCopyItemsEnabled(self):
        return len(self.tblItems.model().items()) > 0

    def _checkActionPasteItemsEnabled(self):
        return len(self._parent.getCopiedData('stockMotionItems', [])) > 0

    def on_actCopyItems(self):
        model = self.tblItems.model()
        notToCopyFields = (model.idFieldName, model.masterIdFieldName)
        result = []
        for item in self.tblItems.getSelectedItems():
            newRecord = model.getEmptyRecord()
            for fieldIndex in xrange(item.count()):
                fieldName = item.fieldName(fieldIndex)
                if fieldName in notToCopyFields:
                    continue
                # Создам новый объект QVariant. Вот не знаю может ли где изменится
                # его значение по ссылке в оригинальном наборе записей.
                newRecord.setValue(fieldName, QtCore.QVariant(item.value(fieldName)))
            result.append(newRecord)
        self._parent.setCopiedData('stockMotionItems', result)

    def on_actPasteItems(self):
        itemsToPast = self._parent.getCopiedData('stockMotionItems', [])
        if not itemsToPast:
            return

        model = self.tblItems.model()
        items = model.items()

        notToPasteFields = self.fieldsNotToPaste + (model.idFieldName, model.masterIdFieldName)

        for item in itemsToPast:
            newRecord = model.getEmptyRecord()
            for fieldIndex in xrange(newRecord.count()):
                fieldName = newRecord.fieldName(fieldIndex)
                if fieldName in notToPasteFields:
                    continue
                # Создам новый объект QVariant. Вот не знаю может ли где изменится
                # его значение по ссылке в оригинальном наборе записей.
                newRecord.setValue(fieldName, QtCore.QVariant(item.value(fieldName)))
            items.append(newRecord)

        rootIndex = QtCore.QModelIndex()
        begin = len(items)
        count = len(itemsToPast)
        model.beginInsertRows(rootIndex, begin, begin + count - 1)
        model.insertRows(begin, count, rootIndex)
        model.endInsertRows()

        model.emitAllChanged()


class CStockMotionBaseDialog(CItemEditorBaseDialog, CStockCache):
    def __init__(self,  parent):
        CItemEditorBaseDialog.__init__(self, parent, 'StockMotion')
        CStockCache.__init__(self)


    def prepareItemsPopupMenu(self, tblWidget):
        tblWidget.addPopupDuplicateCurrentRow()
        tblWidget.addPopupSeparator()
        tblWidget.addMoveRow()
        tblWidget.addPopupDelRow()


    def setDefaults(self):
        now = QDateTime.currentDateTime()
        self.edtDate.setDate(now.date())
        self.edtTime.setTime(now.time())
        self.cmbSupplier.setValue(QtGui.qApp.currentOrgStructureId())


    def getPrice(self, nomenclatureId, financeId):
        return CStockCache.getPrice(self, self.cmbSupplier.value(), nomenclatureId, financeId)


    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        setLineEditValue(    self.edtNumber,          record, 'number')
        setDatetimeEditValue(self.edtDate, self.edtTime, record, 'date')
        setLineEditValue(    self.edtReason,          record, 'reason')
        setDateEditValue(    self.edtReasonDate,      record, 'reasonDate')
        setRBComboBoxValue(  self.cmbSupplier,        record, 'supplier_id')
        setRBComboBoxValue(  self.cmbSupplierPerson,  record, 'supplierPerson_id')
        setLineEditValue(    self.edtNote,            record, 'note')


    def getRecord(self):
        record = CItemEditorBaseDialog.getRecord(self)
        getLineEditValue(   self.edtNumber,         record, 'number')
        getDatetimeEditValue(self.edtDate, self.edtTime, record, 'date', True)
        getLineEditValue(   self.edtReason,    record, 'reason')
        getDateEditValue(   self.edtReasonDate,record, 'reasonDate')
        getRBComboBoxValue( self.cmbSupplier,       record, 'supplier_id')
        getRBComboBoxValue( self.cmbSupplierPerson, record, 'supplierPerson_id')
        getLineEditValue(   self.edtNote,           record, 'note')
        return record


    def _checkStockMotionItemsData(self, tblItems):
        model = tblItems.model()
        items = model.items()
        for idx, item in enumerate(items):
            qnt = forceDouble(item.value('qnt'))
            if qnt <= 0:
                return self.checkValueMessage(
                    u"Количество должно быть больше нуля!", False, tblItems, idx, model.getColIndex('qnt')
                )
        return True

    @pyqtSignature('int')
    def on_cmbSupplier_currentIndexChanged(self, val):
        orgStructureId = self.cmbSupplier.value()
#        if orgStructureId:
        self.cmbSupplierPerson.setOrgStructureId(orgStructureId)
        if hasattr(self, 'modelItems'):
            self.modelItems.setSupplierId(orgStructureId)

        self._on_cmbSupplierChanged()

    def _on_cmbSupplierChanged(self):
        pass


    @pyqtSignature('const QDate&')
    def on_edtDate_dateChanged(self, date):
        self.edtTime.setEnabled(bool(date) if self.edtDate.isEnabled() else False)


    def _generateStockMotionNumber(self):
        if unicode(self.edtNumber.text()):
            return

        counterId = getStockMotionNumberCounterId(self.stockDocumentType)
        if not counterId:
            return

        number = QtGui.qApp.getDocumentNumber(None, counterId, date=QtCore.QDate.currentDate())
        self.edtNumber.setText(number)

    def resetCounterNumber(self):
        return False

    def exec_(self):
        counterController = QtGui.qApp.counterController()
        if not counterController:
            QtGui.qApp.setCounterController(CCounterController(self))
        result = None
        try:
            if not self._id or not hasattr(self, 'requisitionIdList'):
                self._generateStockMotionNumber()
            result = CItemEditorBaseDialog.exec_(self)
        finally:
            if not counterController:
                if result:
                    QtGui.qApp.delAllCounterValueIdReservation()
                else:
                    QtGui.qApp.resetAllCounterValueIdReservation()
            elif self.resetCounterNumber() and counterController.lastReservationId:
                QtGui.qApp.resetCounterValueIdReservation(counterController.lastReservationId)

        if not counterController:
            QtGui.qApp.setCounterController(None)
        return result


class CNomenclatureItemsBaseModel(CInDocTableModel):
    def setSupplierId(self, orgStructureId):
        self._supplierId = orgStructureId

    def createEditor(self, index, parent):
        editor = CInDocTableModel.createEditor(self, index, parent)
        column = index.column()
        if column == self.getColIndex('nomenclature_id'):
            filterSetter = getattr(editor, 'setOrgStructureId', None)
            if not filterSetter:
                return editor
            filterSetter(getattr(self, '_supplierId', None))

        elif column == self.getColIndex('unit_id'):
            self._setUnitEditorFilter(index.row(), editor)

        return editor

    def _setUnitEditorFilter(self, row, editor):
        if 0 <= row < len(self._items):
            item = self._items[row]
            nomenclatureId = forceRef(item.value('nomenclature_id'))
            if not nomenclatureId:
                return
            editor.setFilter(self._getNomenclatureUnitFilter(nomenclatureId))

    @staticmethod
    def _getNomenclatureUnitFilter(nomenclatureId):
        if not nomenclatureId:
            return None

        result = set()
        records = QtGui.qApp.db.getRecordList('rbNomenclature_UnitRatio', where='master_id=%d AND deleted=0' % nomenclatureId)
        for record in records:
            targetUnitId = forceRef(record.value('targetUnit_id'))
            sourceUnitId = forceRef(record.value('sourceUnit_id'))
            if targetUnitId:
                result.add(targetUnitId)
            if sourceUnitId:
                result.add(sourceUnitId)
        return QtGui.qApp.db.table('rbUnit')['id'].inlist(result)

    def cellReadOnly(self, index):
        row = index.row()
        if 0 <= row < len(self._items):
            column = index.column()
            if column == self.getColIndex('unit_id'):
                item = self._items[row]
                nomenclatureId = forceRef(item.value('nomenclature_id'))
                stockUnitId = self.getDefaultStockUnitId(nomenclatureId)
                return not bool(stockUnitId)

        return False

    def getDefaultStockUnitId(self, nomenclatureId):
        return self._getNomenclatureDefaultUnits(nomenclatureId).get('defaultStockUnitId')

    def getDefaultClientUnitId(self, nomenclatureId):
        return self._getNomenclatureDefaultUnits(nomenclatureId).get('defaultClientUnitId')

    def _getNomenclatureDefaultUnits(self, nomenclatureId):
        if not nomenclatureId:
            return {}
        if nomenclatureId not in self._getCache():
            record = QtGui.qApp.db.getRecord(
                'rbNomenclature', 'defaultStockUnit_id, defaultClientUnit_id', nomenclatureId
            )
            if record:
                defaultStockUnitId = forceRef(record.value('defaultStockUnit_id'))
                defaultClientUnitId = forceRef(record.value('defaultClientUnit_id'))
            else:
                defaultStockUnitId = defaultClientUnitId = None

            self._getCache()[nomenclatureId] = {
                'defaultStockUnitId': defaultStockUnitId,
                'defaultClientUnitId': defaultClientUnitId
            }
        return self._getCache()[nomenclatureId]

    def _getCache(self):
        if not hasattr(self, '_modelInfoCache'):
            self._modelInfoCache = {}
        return self._modelInfoCache

    def _applySumRatio(self, item, oldUnitId, newUnitId, sumCol='sum', qntCol='qnt'):
        sum = forceDouble(item.value(sumCol))
        qnt = forceDouble(item.value(qntCol))

        if not (sum and qnt):
            return

        nomenclatureId = forceRef(item.value('nomenclature_id'))
        ratio = self.getRatio(nomenclatureId, oldUnitId, newUnitId)

        item.setValue(sumCol, toVariant(sum/ratio))

    def getRatio(self, nomenclatureId, oldUnitId, newUnitId):
        if oldUnitId is None:
            oldUnitId = self.getDefaultStockUnitId(nomenclatureId)
        if newUnitId is None:
            newUnitId = self.getDefaultStockUnitId(nomenclatureId)

        if oldUnitId == newUnitId:
            return 1

        ratio = getNomenclatureUnitRatio(nomenclatureId, oldUnitId, newUnitId)
        if ratio is None:
            raise ValueError()

        return ratio

    def calcSum(self, item, qntCol='qnt'):
        nomenclatureId = forceRef(item.value('nomenclature_id'))
        financeId = forceRef(item.value('finance_id'))
        stockPrice = self.priceCache.getPrice(nomenclatureId, financeId)

        qnt = forceDouble(item.value(qntCol))
        unit_id = forceRef(item.value('unit_id'))
        stockUnitId = self.getDefaultStockUnitId(nomenclatureId)
        ratio = self.getRatio(nomenclatureId, unit_id, stockUnitId)
        stockQnt = qnt*ratio
        return stockQnt*stockPrice
