# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sys

from PyQt4 import QtCore, QtGui

from Reports.ReportBase import CReportBase, createTable

from library.InDocTable import CIntInDocTableCol, CFloatInDocTableCol
from library.Utils import forceRef, forceString, forceInt, forceDouble, forceDate

from Reports.ReportView import CReportViewDialog


def getStockMotionNumberCounterId(motionType):
    db = QtGui.qApp.db
    orgStructureId = QtGui.qApp.currentOrgStructureId()
    tableRbStockMotionNumber = db.table('rbStockMotionNumber')
    cond = [tableRbStockMotionNumber['motionType'].eq(motionType), 
                tableRbStockMotionNumber['orgStructure_id'].eq(orgStructureId)]
    record = db.getRecordEx(tableRbStockMotionNumber, 'counter_id', cond)
    counterId = forceInt(record.value('counter_id')) if record else None
    if counterId is None:
        condNull = [tableRbStockMotionNumber['motionType'].eq(motionType), 
                tableRbStockMotionNumber['orgStructure_id'].isNull()]
        record = db.getRecordEx(tableRbStockMotionNumber, 'counter_id', condNull)
        counterId = forceInt(record.value('counter_id')) if record else None
    return counterId


def getStockMotionItemQuantityColumn(title=u'Кол-во', fieldName='qnt', width=12):
    return CFloatInDocTableCol(title, fieldName, width, low=1, high=65535, precision=2)


def getRemainingHistory(nomenclatureId, **kwargs):
    orgStructureId = kwargs.get('orgStructureId')
    batch = kwargs.get('batch')
    financeId = kwargs.get('financeId')

    db = QtGui.qApp.db

    tableStockTrans = db.table('StockTrans')

    stockTransCond = [
        db.joinOr([tableStockTrans['debNomenclature_id'].eq(nomenclatureId),
                   tableStockTrans['creNomenclature_id'].eq(nomenclatureId)])
    ]

    if batch:
        stockTransCond.append(tableStockTrans['batch'].eq(batch))

    if financeId:
        stockTransCond.extend(
            [tableStockTrans['debFinance_id'].eq(financeId),
             tableStockTrans['creFinance_id'].eq(financeId)]
        )

    if orgStructureId:
        stockTransCond.append(
            db.joinOr([tableStockTrans['debOrgStructure_id'].eq(orgStructureId),
                       tableStockTrans['creOrgStructure_id'].eq(orgStructureId)])
        )

    stmt = """
    SELECT
        StockMotion.id AS stockMotionId,
        StockMotion.number AS stockMotionNumber,
        StockMotion.date AS stockMotionDate,
        StockMotion_Item.id AS stockMotionItemId,
        StockMotion_Item.qnt AS qnt,
        rbNomenclature.code AS NomenclatureCode,
        rbNomenclature.name AS NomenclatureName,
        rbLfForm.code AS lfFormCode,
        rbLfForm.name AS lfFormName,
        rbUnit.code AS unitCode,
        rbUnit.name AS unitName
    FROM StockMotion
    INNER JOIN StockMotion_Item ON StockMotion_Item.master_id = StockMotion.id
    INNER JOIN rbNomenclature ON rbNomenclature.id = StockMotion_Item.nomenclature_id
    LEFT JOIN rbLfForm ON rbLfForm.id = rbNomenclature.lfForm_id
    LEFT JOIN rbUnit ON rbUnit.id = rbNomenclature.unit_id

    WHERE StockMotion_Item.id IN (
        SELECT StockTrans.stockMotionItem_id
        FROM StockTrans
        WHERE {stockTransCond}
    )
    AND StockMotion.deleted = 0
    ORDER BY StockMotion.id
    """.format(stockTransCond=db.joinAnd(stockTransCond))

    query = db.query(stmt)

    result = []
    while query.next():
        result.append(query.record())

    return result


def showRemainingHistoryReport(records, parent=None):
    doc = QtGui.QTextDocument()
    cursor = QtGui.QTextCursor(doc)

    cursor.setCharFormat(CReportBase.ReportTitle)
    cursor.insertText(u'История по остаткам')
    cursor.insertBlock()

    tableColumns = [
        ('6%', [u'№ п/п'], CReportBase.AlignRight),
        ('20%', [u'Накладная'], CReportBase.AlignLeft),
        ('10%', [u'Дата'], CReportBase.AlignLeft),
        ('20%', [u'Товар'], CReportBase.AlignLeft),
        ('10%', [u'Количество'], CReportBase.AlignRight),
        ('15%', [u'Форма выпуска'], CReportBase.AlignLeft),
        ('15%', [u'Ед. изм.'], CReportBase.AlignLeft),
    ]

    table = createTable(cursor, tableColumns)

    prevStockMotionId = None
    stockMotionRows = 0
    for record in records:
        i = table.addRow()
        table.setText(i, 0, i)
        stockMotionId = forceRef(record.value('stockMotionId'))
        if stockMotionId != prevStockMotionId:
            if stockMotionRows:
                table.mergeCells(i - stockMotionRows, 1, stockMotionRows, 1)
                table.mergeCells(i - stockMotionRows, 2, stockMotionRows, 1)
            stockMotionNumber = forceString(record.value('stockMotionNumber'))
            stockMotionDate = forceString(record.value('stockMotionDate'))
            table.setText(i, 1, stockMotionNumber)
            table.setText(i, 2, stockMotionDate)

        qnt = forceInt(record.value('qnt'))
        nomenclatureCode = forceString(record.value('nomenclatureCode'))
        nomenclatureName = forceString(record.value('nomenclatureName'))
        lfFormCode = forceString(record.value('lfFormCode'))
        lfFormName = forceString(record.value('lfFormName'))
        unitCode = forceString(record.value('unitCode'))
        unitName = forceString(record.value('unitName'))

        table.setText(i, 3, ' | '.join([nomenclatureCode, nomenclatureName]))
        table.setText(i, 4, qnt)
        if lfFormCode or lfFormName:
            table.setText(i, 5, ' | '.join([v for v in [lfFormCode, lfFormName] if v]))
        if unitCode or unitName:
            table.setText(i, 6, ' | '.join([v for v in [unitCode, unitName] if v]))

    viewDialog = CReportViewDialog(parent)

    viewDialog.setWindowTitle(u'История по остаткам')
    viewDialog.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Close)
    viewDialog.setText(doc)
    viewDialog.setOrientation(QtGui.QPrinter.Portrait)
    viewDialog.exec_()


def getExistsNomenclatureIdList(orgStructureId=None, financeId=None):
    db = QtGui.qApp.db
    tableStocktrans = db.table('StockTrans')
    debCond = []
    creCond = []
    cond = []
    if orgStructureId:
        debCond.append(tableStocktrans['debOrgStructure_id'].eq(orgStructureId))
        creCond.append(tableStocktrans['creOrgStructure_id'].eq(orgStructureId))
        cond.append('T.orgStructure_id = %d' % orgStructureId)
    if financeId:
        debCond.append(tableStocktrans['debFinance_id'].eq(financeId))
        creCond.append(tableStocktrans['creFinance_id'].eq(financeId))
        cond.append('T.finance_id = %d' % financeId)

    stmt = u''' SELECT DISTINCT dataTable.nomenclature_id FROM (
        SELECT
               T.orgStructure_id,
               T.nomenclature_id,
               sum(T.qnt) AS `qnt`

        FROM
            (
            SELECT debOrgStructure_id AS orgStructure_id,
                   debNomenclature_id AS nomenclature_id,
                   sum(qnt)           AS `qnt`,
                   debFinance_id AS `finance_id`

            FROM StockTrans
            WHERE {debCond}
            GROUP BY debOrgStructure_id, debNomenclature_id
            UNION ALL
            SELECT creOrgStructure_id AS orgStructure_id,
                   creNomenclature_id AS nomenclature_id,
                   -sum(qnt)          AS `qnt`,
                   creFinance_id AS `finance_id`
            FROM StockTrans
            WHERE {creCond}
            GROUP BY creOrgStructure_id, creNomenclature_id
            ) AS T

        WHERE {cond}
        GROUP BY orgStructure_id, nomenclature_id
        HAVING `qnt` > 0
        ) AS dataTable
    '''.format(debCond=db.joinAnd(debCond) if debCond else '1',
               creCond=db.joinAnd(creCond) if creCond else '1',
               cond=db.joinAnd(cond) if creCond else '1')

    result = []
    query = db.query(stmt)
    while query.next():
        result.append(forceRef(query.value(0)))
    return result


# ##################################################################################


def getNomenclatureUnitRatio(nomenclatureId, sourceUnitId, targetUnitId):
    if sourceUnitId == targetUnitId:
        return 1

    db = QtGui.qApp.db

    tableNUR = db.table('rbNomenclature_UnitRatio')

    cond = [tableNUR['sourceUnit_id'].eq(sourceUnitId),
            tableNUR['targetUnit_id'].eq(targetUnitId),
            tableNUR['master_id'].eq(nomenclatureId)]

    record = db.getRecordEx(tableNUR, 'ratio', cond)
    ratio = forceDouble(record.value('ratio')) if record else None
    if ratio is not None:
        return ratio

    cond = [tableNUR['sourceUnit_id'].eq(targetUnitId),
            tableNUR['targetUnit_id'].eq(sourceUnitId),
            tableNUR['master_id'].eq(nomenclatureId)]

    record = db.getRecordEx(tableNUR, 'ratio', cond)
    ratio = forceDouble(record.value('ratio')) if record else None
    if ratio is not None:
        return 1/ratio

    return None


def applyNomenclatureUnitRatio(qnt, nomenclatureId, unitId, revert=False):
    db = QtGui.qApp.db

    stockUnitId = forceRef(db.translate('rbNomenclature', 'id', nomenclatureId, 'defaultStockUnit_id'))

    if revert:
        ratio = getNomenclatureUnitRatio(nomenclatureId, unitId, stockUnitId)
    else:
        ratio = getNomenclatureUnitRatio(nomenclatureId, stockUnitId, unitId)

    if ratio in (1, None):
        return qnt

    return qnt * ratio


def getExistsNomenclatureAmount(nomenclatureId, financeId=None, batch=None, orgStructureId=None, unitId=None):
    db = QtGui.qApp.db
    table = db.table('StockTrans')

    debCond = []
    creCond = []

    date = QtCore.QDate.currentDate()
    debCond.append(table['date'].dateLe(date))
    creCond.append(table['date'].dateLe(date))

    orgStructureId = orgStructureId or QtGui.qApp.currentOrgStructureId()

    if orgStructureId:
        debCond.append(table['debOrgStructure_id'].eq(orgStructureId))
        creCond.append(table['creOrgStructure_id'].eq(orgStructureId))
    else:
        debCond.append(table['debOrgStructure_id'].isNotNull())
        creCond.append(table['creOrgStructure_id'].isNotNull())

    if financeId:
        debCond.append(table['debFinance_id'].eq(financeId))
        creCond.append(table['creFinance_id'].eq(financeId))

    if batch:
        debCond.append(table['batch'].eq(batch))
        creCond.append(table['batch'].eq(batch))


    debCond.append(table['debNomenclature_id'].eq(nomenclatureId))
    creCond.append(table['creNomenclature_id'].eq(nomenclatureId))

    stmt = u'''
SELECT
   sum(T.qnt) AS `qnt`
FROM
(
SELECT
       sum(qnt)           AS `qnt`
FROM StockTrans
WHERE %(debCond)s
UNION
SELECT
       -sum(qnt)          AS `qnt`
FROM StockTrans
WHERE %(creCond)s
) AS T

''' % {
    'debCond' : db.joinAnd(debCond),
    'creCond' : db.joinAnd(creCond)
    }

    query = db.query(stmt)
    if query.next():
        qnt = forceDouble(query.record().value('qnt'))
        if unitId is not None:
            return applyNomenclatureUnitRatio(qnt, nomenclatureId, unitId)
        return qnt
    return 0


FILTER_FOR_BATCH = 0
FILTER_FOR_SHELF_TIME = 1
FILTER_FOR_FINANCE_ID = 2
FILTER_FOR_FINANCE_AND_SHELF_TIME = 3
FILTER_FOR_FINANCE_AND_BATCH = 4
FILTER_FOR_BATCH_AND_SHELF_TIME = 5
UTILIZATION = 6
FILTER_FOR_FINANCE_AND_SHELF_TIME_FOR_UTILIZATION = 7
FILTER_FOR_BATCH_AND_FINANCE_FOR_UTILIZATION = 8


def findFinanceBatchShelfTime(orgStructureId, nomenclatureId, qnt=None, stockMotionItem=None, filterFor=None, financeId=None, clientId=None,  first=True):
    db = QtGui.qApp.db
    table = db.table('StockTrans')
    tableOSS = db.table('OrgStructure_Stock')

    debCond = []
    creCond = []
    ossCond = []


    debCond.append(table['debOrgStructure_id'].eq(orgStructureId))
    creCond.append(table['creOrgStructure_id'].eq(orgStructureId))
    ossCond.append(tableOSS['master_id'].eq(orgStructureId))

    debCond.append(table['debNomenclature_id'].eq(nomenclatureId))
    creCond.append(table['creNomenclature_id'].eq(nomenclatureId))
    ossCond.append(tableOSS['nomenclature_id'].eq(nomenclatureId))


    batchFields = 'batch, shelfTime,'
    sqlGroupByBatch = 'batch, shelfTime, '

    shelfTimeFilterIsSet = False

    havCond = []
    if qnt:
        havCond.append('(`qnt` >= %s)' % qnt)
    else:
        havCond.append('(`qnt` > 0)')

    if stockMotionItem and filterFor is not None:
        if filterFor == FILTER_FOR_BATCH:
            debCond.append(table['debFinance_id'].eq(stockMotionItem.finance_id))
            creCond.append(table['creFinance_id'].eq(stockMotionItem.finance_id))

            debCond.append(table['shelfTime'].dateLe(stockMotionItem.shelfTime))
            creCond.append(table['shelfTime'].dateLe(stockMotionItem.shelfTime))

            shelfTimeFilterIsSet = True


        elif filterFor == FILTER_FOR_FINANCE_ID:
            debCond.append(table['batch'].eq(stockMotionItem.batch))
            creCond.append(table['batch'].eq(stockMotionItem.batch))

            debCond.append(table['shelfTime'].dateLe(stockMotionItem.shelfTime))
            creCond.append(table['shelfTime'].dateLe(stockMotionItem.shelfTime))

            shelfTimeFilterIsSet = True

        elif filterFor == FILTER_FOR_SHELF_TIME:
            debCond.append(table['debFinance_id'].eq(stockMotionItem.finance_id))
            creCond.append(table['creFinance_id'].eq(stockMotionItem.finance_id))

            debCond.append(table['batch'].eq(stockMotionItem.batch))
            creCond.append(table['batch'].eq(stockMotionItem.batch))

        elif filterFor == FILTER_FOR_FINANCE_AND_SHELF_TIME:
            debCond.append(table['batch'].eq(stockMotionItem.batch))
            creCond.append(table['batch'].eq(stockMotionItem.batch))

        elif filterFor == FILTER_FOR_FINANCE_AND_BATCH:
            debCond.append(table['shelfTime'].dateLe(stockMotionItem.shelfTime))
            creCond.append(table['shelfTime'].dateLe(stockMotionItem.shelfTime))

        elif filterFor == FILTER_FOR_BATCH_AND_SHELF_TIME:
            debCond.append(table['debFinance_id'].eq(stockMotionItem.finance_id))
            creCond.append(table['creFinance_id'].eq(stockMotionItem.finance_id))

            shelfTimeFilterIsSet = True
        
    if filterFor == FILTER_FOR_FINANCE_AND_SHELF_TIME_FOR_UTILIZATION:
        debCond.append(table['batch'].eq(stockMotionItem.batch))
        creCond.append(table['batch'].eq(stockMotionItem.batch))
        debCond.append(table['shelfTime'].isNotNull())
        creCond.append(table['shelfTime'].isNotNull())
        shelfTimeFilterIsSet = True
    elif filterFor == FILTER_FOR_BATCH_AND_FINANCE_FOR_UTILIZATION:
        debCond.append(table['debFinance_id'].eq(stockMotionItem.finance_id))
        creCond.append(table['creFinance_id'].eq(stockMotionItem.finance_id))
        debCond.append(table['batch'].eq(stockMotionItem.batch))
        creCond.append(table['batch'].eq(stockMotionItem.batch))
    elif filterFor == UTILIZATION:
        debCond.append(table['shelfTime'].isNotNull())
        creCond.append(table['shelfTime'].isNotNull())
        shelfTimeFilterIsSet = True
    
    if financeId:
        debCond.append(table['debFinance_id'].eq(financeId))
        creCond.append(table['creFinance_id'].eq(financeId))

    if not shelfTimeFilterIsSet:
        currentDate = QtCore.QDate.currentDate()
        debCond.append(table['shelfTime'].dateGe(currentDate))
        creCond.append(table['shelfTime'].dateGe(currentDate))
    
    if clientId:
        reservationStmt = u'''SELECT 
                StockMotion_Item.finance_id,
                StockMotion_Item.batch,
                StockMotion_Item.shelfTime
            FROM
                StockMotion
                    LEFT JOIN
                StockMotion_Item ON StockMotion_Item.master_id = StockMotion.id
            WHERE
                StockMotion_Item.`nomenclature_id` = %(nomenclatureId)s
                    AND StockMotion.type = 6
                    AND StockMotion_Item.qnt > 0
                    AND StockMotion.client_id = %(clientId)s
                    ORDER BY StockMotion_Item.shelfTime, StockMotion_Item.qnt LIMIT 1'''% {
        'nomenclatureId': nomenclatureId,
        'clientId': clientId,
            }
        reservationQuery = db.query(reservationStmt)
        while reservationQuery.next():
            reservationRecord = reservationQuery.record()
            if reservationRecord:
                financeId = forceRef(reservationRecord.value('finance_id'))
                batch = forceString(reservationRecord.value('batch'))
                shelfTime = forceDate(reservationRecord.value('shelfTime'))
            if financeId and batch and shelfTime:
                return financeId, batch, shelfTime

    stmt = u'''
    SELECT T.orgStructure_id,
           T.nomenclature_id,
           T.batch,
           T.shelfTime,
           T.finance_id,
           sum(T.qnt) AS `qnt`,
           sum(T.`sum`) AS `sum`,
           OrgStructure_Stock.constrainedQnt AS constrainedQnt,
           OrgStructure_Stock.orderQnt AS orderQnt
    FROM
        (
        SELECT debOrgStructure_id AS orgStructure_id,
               debNomenclature_id AS nomenclature_id,
               %(batchFields)s
               debFinance_id      AS finance_id,
               sum(qnt)           AS `qnt`,
               sum(`sum`)         AS `sum`
        FROM StockTrans
        WHERE %(debCond)s
        GROUP BY debOrgStructure_id, debNomenclature_id, %(groupByBatch)s debFinance_id
        UNION ALL
        SELECT creOrgStructure_id AS orgStructure_id,
               creNomenclature_id AS nomenclature_id,
               %(batchFields)s
               creFinance_id      AS finance_id,
               -sum(qnt)          AS `qnt`,
               -sum(`sum`)        AS `sum`
        FROM StockTrans
        WHERE %(creCond)s
        GROUP BY creOrgStructure_id, creNomenclature_id, %(groupByBatch)s creFinance_id
        UNION ALL
        SELECT master_id          AS orgStructure_id,
               nomenclature_id    AS nomenclature_id,
               ''                 AS batch,
               NULL               AS shelfTime,
               finance_id         AS finance_id,
               0                  AS `qnt`,
               0                  AS `sum`
        FROM OrgStructure_Stock
        WHERE %(ossCond)s
        GROUP BY master_id, nomenclature_id, finance_id
        ) AS T
        LEFT JOIN OrgStructure ON OrgStructure.id = T.orgStructure_id
        LEFT JOIN rbNomenclature ON rbNomenclature.id = T.nomenclature_id
        LEFT JOIN rbFinance ON rbFinance.id = T.finance_id
        LEFT JOIN OrgStructure_Stock ON OrgStructure_Stock.master_id = T.orgStructure_id
              AND OrgStructure_Stock.nomenclature_id = T.nomenclature_id
              AND OrgStructure_Stock.finance_id = T.finance_id
    GROUP BY orgStructure_id, nomenclature_id, %(groupByBatch)s finance_id
    HAVING (%(havCond)s)
    ORDER BY T.shelfTime, %(groupByBatch)s rbFinance.code
    %(limit)s
    ''' % {
        'debCond': db.joinAnd(debCond),
        'creCond': db.joinAnd(creCond),
        'ossCond': db.joinAnd(ossCond),
        'havCond': db.joinAnd(havCond),
        'groupByBatch': sqlGroupByBatch,
        'batchFields': batchFields,
        'limit': "LIMIT 1" if first else ''
    }
    query = db.query(stmt)

    if first:
        if query.next():
            record = query.record()
            financeId = forceRef(record.value('finance_id'))
            batch = forceString(record.value('batch'))
            shelfTime = forceDate(record.value('shelfTime'))
            return financeId, batch, shelfTime
        return None, None, None
    else:
        result = []
        while query.next():
            record = query.record()
            financeId = forceRef(record.value('finance_id'))
            batch = forceString(record.value('batch'))
            shelfTime = forceDate(record.value('shelfTime'))
            result.append((financeId, batch, shelfTime))
        return result


# ########################################


class CSatisifiedFilter(object):
    str_values = [u'', u'Частично', u'Полностью', u'Нет', u'С превышением']
    ALL = 0
    PARTIAL = 1
    FULL = 2
    NOT = 3
    EXCEEDING = 4

    NEED_FILTER_INDEXES = (PARTIAL, FULL, NOT, EXCEEDING)

    @classmethod
    def getFilter(cls, index, nomenclatureId=None):
        if index not in cls.NEED_FILTER_INDEXES:
            return None

        db = QtGui.qApp.db
        table = db.table('StockRequisition')
        tableItems = db.table('StockRequisition_Item')

        subcond = [
            tableItems['master_id'].eq(table['id'])
        ]

        if nomenclatureId:
            subcond.append(tableItems['nomenclature_id'].eq(nomenclatureId))

        if index == cls.PARTIAL:
            return db.selectStmtGroupBy(tableItems, u'IF((SUM(StockRequisition_Item.`satisfiedQnt`) > 0 AND SUM(StockRequisition_Item.`qnt`) > SUM(StockRequisition_Item.`satisfiedQnt`)), StockRequisition_Item.master_id, NULL)', subcond, u'StockRequisition_Item.master_id')

        elif index == cls.FULL:
            return db.selectStmtGroupBy(tableItems, u'IF(SUM(StockRequisition_Item.`qnt`) = SUM(StockRequisition_Item.`satisfiedQnt`), StockRequisition_Item.master_id, NULL)', subcond, u'StockRequisition_Item.master_id')

        elif index == cls.NOT:
            return db.selectStmtGroupBy(tableItems, u'IF(SUM(StockRequisition_Item.`satisfiedQnt`) = 0, StockRequisition_Item.master_id, NULL)', subcond, u'StockRequisition_Item.master_id')

        elif index == cls.EXCEEDING:
            return db.selectStmtGroupBy(tableItems, u'IF(SUM(StockRequisition_Item.`qnt`) < SUM(StockRequisition_Item.`satisfiedQnt`), StockRequisition_Item.master_id, NULL)', subcond, u'StockRequisition_Item.master_id')

        else:
            return None

        return db.existsStmt(tableItems, subcond)


def tuneSatisfiedFilterCmb(cmb):
    cmb.addItems(CSatisifiedFilter.str_values)


class CStockCache(object):
    def __init__(self):
        self._mapIdsToStock = {}


    def getStock(self, orgStructureId, nomenclatureId, financeId):
        key = orgStructureId, nomenclatureId, financeId
        result = self._mapIdsToStock.get(key, None)
        if result is None:
            result = self.readStock(orgStructureId, nomenclatureId, financeId)
            self._mapIdsToStock[key] = result
        return result


    def readStock(self, orgStructureId, nomenclatureId, financeId):
        qnt = 0.0
        sum = 0.0
        if orgStructureId and nomenclatureId:
            try:
                db = QtGui.qApp.db
                financeId = financeId if financeId is not None else 'NULL'
                db.query('CALL getStock(%s, %s, %s, NULL, @resQnt, @resSum)' % (orgStructureId, nomenclatureId, financeId))
                query = db.query('SELECT @resQnt, @resSum')
                if query.next():
                    record = query.record()
                    qnt = forceDouble(record.value(0))
                    sum = forceDouble(record.value(1))
            except:
                QtGui.qApp.logCurrentException()
        return qnt, sum


    def getPrice(self, orgStructureId, nomenclatureId, financeId):
        qnt, sum = self.getStock(orgStructureId, nomenclatureId, financeId)
        if qnt:
            return sum/qnt
        else:
            return 0.0

def calcNomenclatureSum(stockMotionItem):
    db = QtGui.qApp.db
    tableStockMotionItem = db.table('StockMotion_Item')
    tableStockMotion = db.table('StockMotion')
    cond = [tableStockMotion['type'].inlist((0, 1))]
    
    if stockMotionItem.batch:
        cond.append(tableStockMotionItem['batch'].eq(stockMotionItem.batch))
    if stockMotionItem.shelfTime:
        cond.append(tableStockMotionItem['shelfTime'].eq(stockMotionItem.shelfTime))
    if stockMotionItem.nomenclature_id:
        cond.append(tableStockMotionItem['nomenclature_id'].eq(stockMotionItem.nomenclature_id))
        
    queryTable = tableStockMotion
    queryTable = queryTable.leftJoin(tableStockMotionItem, tableStockMotionItem['master_id'].eq(tableStockMotion['id']))
    
    record = db.getRecordEx(queryTable, 'StockMotion_Item.sum / StockMotion_Item.qnt AS stockPrice', cond)
    if record:
        stockPrice = forceDouble(record.value('stockPrice'))

    stockUnitId = stockMotionItem.nomenclature.defaultStockUnit_id
    if stockMotionItem.unit_id == stockUnitId:
        ratio = 1
    else:
        ratio = getNomenclatureUnitRatio(stockMotionItem.nomenclature_id, stockMotionItem.unit_id, stockUnitId)
    return stockMotionItem.qnt * ratio * stockPrice
