# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2016-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
##
## Взаимодейсивие с лабораторной системой по протоколу FHIR:
## - отправка заказов
## - получение результатов
## - отправка результатов локальных лабораторий с целью "интерграции"
##
#############################################################################

import isodate
import json
import uuid
import xml.sax
from StringIO import StringIO
from collections import namedtuple

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QDateTime, QEventLoop

from library.Utils import exceptionToUnicode, forceBool, forceInt, forceRef, forceString, forceDate, forceDateTime, formatNameInt
from library.Identification import getIdentification, getIdentificationEx, findByIdentification, addIdentification

from Exchange.FHIRClient import client
from Exchange.FHIRClient.models102.address          import Address
from Exchange.FHIRClient.models102.attachment       import Attachment
from Exchange.FHIRClient.models102.binary           import Binary
from Exchange.FHIRClient.models102.bundle           import Bundle, BundleEntry, BundleEntryRequest
from Exchange.FHIRClient.models102.codeableconcept  import CodeableConcept
from Exchange.FHIRClient.models102.coding           import Coding
from Exchange.FHIRClient.models102.condition        import Condition
#from Exchange.FHIRClient.models102.conformance      import Conformance
from Exchange.FHIRClient.models102.diagnosticorder  import DiagnosticOrder, DiagnosticOrderItem
from Exchange.FHIRClient.models102.diagnosticreport import DiagnosticReport
from Exchange.FHIRClient.models102.encounter        import Encounter
from Exchange.FHIRClient.models102.extension        import Extension
from Exchange.FHIRClient.models102.fhirdate         import FHIRDate
from Exchange.FHIRClient.models102.fhirreference    import FHIRReference
from Exchange.FHIRClient.models102.humanname        import HumanName
from Exchange.FHIRClient.models102.identifier       import Identifier
from Exchange.FHIRClient.models102.meta             import Meta
from Exchange.FHIRClient.models102.observation      import Observation, ObservationReferenceRange
from Exchange.FHIRClient.models102.order            import Order, OrderWhen
from Exchange.FHIRClient.models102.orderresponse    import OrderResponse
from Exchange.FHIRClient.models102.organization     import Organization
from Exchange.FHIRClient.models102.patient          import Patient
from Exchange.FHIRClient.models102.period           import Period
from Exchange.FHIRClient.models102.practitioner     import Practitioner, PractitionerPractitionerRole
from Exchange.FHIRClient.models102.reference        import Reference
from Exchange.FHIRClient.models102.quantity         import Quantity
from Exchange.FHIRClient.models102.specimen         import Specimen, SpecimenCollection, SpecimenContainer

__all__ = ( 'sendOrdersOverFHIR',
            'pickupResultsOverFHIR',
            'sendLocalResultsOverFHIR'
          )


# обнаружено, что разные протоколы могут предусматривать разные
# способы группировки проб по заданиям.
# поэтому мы должны самостоятельно группировать пробы в задания
def sendOrdersOverFHIR(widget, equipmentInterface, clientInfo, probeIdList, probeSaver=None):
    opts = json.loads(equipmentInterface.address)
    orgId = QtGui.qApp.currentOrgId()
    interfaceObject = CFHIRExchange(opts, orgId)

    orderList = selectFHIROrders(probeIdList)
    for order in orderList:
        if probeSaver:
            probeSaver.clear()
        identifier = interfaceObject.sendOrderOverFHIR(clientInfo, order)
        if probeSaver:
            for probeId in order.probeIdSet:
                probeSaver.append(probeId)
            probeSaver.save(identifier)
    return bool(orderList)


def pickupResultsOverFHIR(widget, equipmentInterface, probeIdList, recordList = []):
    if not recordList:
        db = QtGui.qApp.db
        tableProbe = db.table('Probe')
        cond = [ tableProbe['status'].eq(6),  # Передано в ЛИС
                 tableProbe['equipment_id'].eq(equipmentInterface.id), # и ЛИС тот самый
                 tableProbe['exportName'].ne(''), # exportName заполнен
                 tableProbe['exportDatetime'].ge(QDateTime.currentDateTime().addDays(-7)), # прошло не более недели с заказа
                 tableProbe['id'].inlist(probeIdList) if probeIdList else '1'
               ]
        recordList = db.getRecordList(tableProbe,
                                      [ '*' ],
                                      cond
                                     )
    opts = json.loads(equipmentInterface.address)
    orgId = QtGui.qApp.currentOrgId()
    interfaceObject = CFHIRExchange(opts, orgId)
    setOfProcessedOrderIds = set()
    for record in recordList:
        fhirOrderId = forceString(record.value('exportName'))
        if fhirOrderId not in setOfProcessedOrderIds:
            if widget:
                QtGui.qApp.processEvents(QEventLoop.ExcludeUserInputEvents)
            try:
                fhirResult = interfaceObject.requestResultOverFHIR(fhirOrderId)
                fhirResult.storeToProbes(equipmentInterface.id)
                setOfProcessedOrderIds.add(fhirOrderId)
            except Exception, e:
                QtGui.qApp.log('importFhir', u'при получении результата по заказу %s произошла ошибка:\n%s' % (
                    fhirOrderId, exceptionToUnicode(e)))
    return setOfProcessedOrderIds


def sendLocalResultsOverFHIR(equipmentInterface, clientInfo, serviceId, results):
    opts = json.loads(equipmentInterface.address)
    orgId = QtGui.qApp.currentOrgId()
    interfaceObject = CFHIRExchange(opts, orgId)
    interfaceObject.sendLocalResultsOverFHIR(clientInfo, serviceId, results)


# ##############################################################################


class COrderDto:
    u'Заказ, состоит из событий'
    def __init__(self):
        self.takenDatetime = None
        self.mapEventIdtoEvent = {}
        self.probeIdSet = set([])


    def getEventList(self):
        return self.mapEventIdtoEvent.values()


    def addProbe(self,
                 eventId,
                 eventTypeId,
                 eventPurposeId,
                 actionId,
                 actionTypeId,
                 directionDatetime,
                 orgStructureId,
                 setPersonId,
                 MKB,
                 serviceId,
                 takenTissueJournalId,
                 takenDatetime,
                 externalId,
                 specimenTypeId,
                 containerTypeId,
                 probeId,
                 financeId
                ):
        if not externalId:
            raise Exception(u'В пробe Probe.id=%d пустой externalId' % probeId )
        if not specimenTypeId:
            raise Exception(u'В пробe Probe.id=%d пустой specimenType_id' % probeId)
        if not containerTypeId:
            raise Exception(u'В пробe Probe.id=%d пустой containerType_id' % probeId)
        if not takenTissueJournalId:
            raise Exception(u'В пробe Probe.id=%d пустой takenTissueJournal_id' % probeId)
        if not actionId:
            raise Exception(u'Для TakenTissueJournal.id=%d упомянутого в Probe.id=%d не найдено ни одного действия' % (takenTissueJournalId, probeId))
        if not actionTypeId:
            raise Exception(u'В действии Action.id=%d упомянутого в Probe.id=%d указан пустой actionType_id' % (actionId, probeId))
        if not serviceId:
            raise Exception(u'В типе действия ActionType.id=%d упомянутого в пробе Probe.id=%d не задана номенклатурная услуга' % (actionTypeId, probeId))
        if not setPersonId:
            raise Exception(u'В действии Action.id=%d упомянутого в пробе Probe.id=%d пустой setPerson_id' % (actionId, probeId))

        self.probeIdSet.add(probeId)
        self.takenDatetime = max(self.takenDatetime, takenDatetime)
        event = self.mapEventIdtoEvent.get(eventId)
        if event is None:
            event = self.mapEventIdtoEvent[eventId] = CEventDto(eventId, eventTypeId, eventPurposeId)
        action = event.getAction(actionId,
                                 actionTypeId,
                                 directionDatetime,
                                 orgStructureId,
                                 setPersonId,
                                 MKB,
                                 serviceId,
                                 financeId)
        action.addSpecimen(specimenTypeId,
                           containerTypeId,
                           externalId,
                           takenTissueJournalId,
                           takenDatetime,
                           probeId)


class CEventDto:
    u'описание события, во fhir превращается в encounter'
    def __init__(self, eventId, eventTypeId, eventPurposeId):
        self.eventId = eventId
        self.eventTypeId = eventTypeId
        self.eventPurposeId = eventPurposeId
        self.mapActionIdToAction = {}


    def getActionList(self):
        return self.mapActionIdToAction.values()


    def getAction(self,
                  actionId,
                  actionTypeId,
                  directionDatetime,
                  orgStructureId,
                  setPersonId,
                  diagnosis,
                  serviceId,
                  financeId):
        action = self.mapActionIdToAction.get(actionId)
        if action is None:
            action = self.mapActionIdToAction[actionId] = CActionDto(actionId,
                                                                      actionTypeId,
                                                                      directionDatetime,
                                                                      orgStructureId,
                                                                      setPersonId,
                                                                      diagnosis,
                                                                      serviceId,
                                                                      financeId)
        return action


class CActionDto:
    u'описание события, во fhir совокупность CActionDto превращается в diagnosticOrder'
    def  __init__(self,
                  actionId,
                  actionTypeId,
                  directionDatetime,
                  orgStructureId,
                  setPersonId,
                  diagnosis,
                  serviceId,
                  financeId):
        self.actionId = actionId
        self.actionTypeId = actionTypeId
        self.directionDatetime = directionDatetime
        self.orgStructureId = orgStructureId
        self.setPersonId = setPersonId
        self.diagnosis = diagnosis
        self.serviceId = serviceId
        self.financeId = financeId
        self.specimenList = []


    def addSpecimen(self, specimenTypeId,
                          containerTypeId,
                          externalId,
                          takenTissueJournalId,
                          takenDatetime,
                          probeId):
        self.specimenList.append(CSpecimenDto(specimenTypeId,
                                              containerTypeId,
                                              externalId,
                                              takenTissueJournalId,
                                              takenDatetime,
                                              probeId))


    def getSpecimenList(self):
        return self.specimenList


CSpecimenDto = namedtuple('CSpecimenDto',
                          ('specimenTypeId',
                           'containerTypeId',
                           'externalId',
                           'takenTissueJournalId',
                           'takenDatetime',
                           'probeId',
                          )
                         )


class CFHIRResultValue:
    def __init__(self, code):
        self.code  = code
        self.value = None
        self.unitId  = None
        self.referenceRange = None


class CFHIRServiceResult:
    def __init__(self, code, issued, conclusion):
        self.code = code
        self.issued = issued
        self.conclusion = conclusion
        self.values = []


class CFHIRResult:
    def __init__(self, fhirOrderId):
        self.fhirOrderId = fhirOrderId
        self.probeIdSet = set()
        self.finished = False # результатов больше не будет
        self.serviceResults = []


    def addProbeId(self, probeId):
        self.probeIdSet.add(probeId)


    def addServiceResult(self, serviceResult):
        self.serviceResults.append(serviceResult)


    @staticmethod
    def getMapTestCodeToTestId(eqipmentId):
        db = QtGui.qApp.db
        table = db.table('rbEquipment_Test')
        records = db.getRecordList(table,
                                   ['test_id', 'hardwareTestCode'],
                                   db.joinAnd([table['equipment_id'].eq(eqipmentId),
                                               table['type'].inlist((1, 2)),
                                               table['test_id'].isNotNull()
                                              ])
                                  )
        result = {}
        for record in records:
            code   = forceString(record.value('hardwareTestCode'))
            testId = forceRef(record.value('test_id'))
            result[code] = testId
        return result


    @staticmethod
    def determineTypeName(value):
        return 'String' if isinstance(value, basestring) else 'Double'


    @staticmethod
    def updateOrAddProbe(eqipmentId, exportName, testId, value, unitName, unitId, referenceRangeAsStr):
        db = QtGui.qApp.db

        tableProbe = db.table('Probe')
        probeList = db.getRecordList(tableProbe,
                                     '*',
                                     db.joinAnd([tableProbe['equipment_id'].eq(eqipmentId),
                                                 tableProbe['exportName'].eq(exportName),
                                                 tableProbe['test_id'].eq(testId),
                                                ]
                                               )
                                    )
        if probeList:
            for record in probeList:
                status  = forceInt(record.value('status'))
                if status in (6, 7):
                    record.setValue('status',          7)
                    record.setValue('resultIndex',     1)
                    record.setValue('result1',         value)
                    record.setValue('typeName',        CFHIRResult.determineTypeName(value))
                    record.setValue('unit_id',         unitId)
                    record.setValue('externalUnit',    unitName)
                    record.setValue('norm',            referenceRangeAsStr)
                    record.setValue('externalNorm',    referenceRangeAsStr)
                    record.setValue('importDatetime',  QDateTime.currentDateTime())
                    record.setValue('importPerson_id', QtGui.qApp.userId)
                    db.updateRecord(tableProbe, record)
        else:
            takenTissueJournalIdList = db.getDistinctIdList(tableProbe,
                                                            'takenTissueJournal_id',
                                                            db.joinAnd([tableProbe['equipment_id'].eq(eqipmentId),
                                                                        tableProbe['exportName'].eq(exportName),
                                                                       ]
                                                                      )
                                                           )
            record = tableProbe.newRecord()
            record.setValue('equipment_id',    eqipmentId)
            record.setValue('exportName',      exportName)
            record.setValue('test_id',         testId)
            record.setValue('workTest_id',     testId)
            record.setValue('takenTissueJournal_id', takenTissueJournalIdList[0])
            record.setValue('status',          7)
            record.setValue('resultIndex',     1)
            record.setValue('result1',         value)
            record.setValue('typeName',        CFHIRResult.determineTypeName(value))
            record.setValue('unit_id',         unitId)
            record.setValue('externalUnit',    unitName)
            record.setValue('norm',            referenceRangeAsStr)
            record.setValue('externalNorm',    referenceRangeAsStr)
            record.setValue('importDatetime',  QDateTime.currentDateTime())
            record.setValue('importPerson_id', QtGui.qApp.userId)
            db.insertRecord(tableProbe, record)


    @staticmethod
    def getReferenceRangeAsStr(referenceRange):
        if referenceRange:
            low, high = referenceRange
            if low is None:
                low = 0.0
            if high is None:
                high = 999.99
            return str(low) + ' - ' + str(high)
        else:
            return ''


    @staticmethod
    def markRemainedProbesAsAcceptedWithoutResult(eqipmentId, exportName):
        db = QtGui.qApp.db
        tableProbe = db.table('Probe')
        record = tableProbe.newRecord(['status', 'importDatetime', 'importPerson_id'])
        record.setValue('status',          7)
        record.setValue('resultIndex',     4)
        record.setValue('importDatetime',  QDateTime.currentDateTime())
        record.setValue('importPerson_id', QtGui.qApp.userId)

        cond = db.joinAnd([tableProbe['equipment_id'].eq(eqipmentId),
                           tableProbe['exportName'].eq(exportName),
                           tableProbe['status'].eq(6),
                          ]
                         )
        db.updateRecords(tableProbe, record,  cond)


    def storeToProbes(self, eqipmentId):
        freeResuts = {}
        mapTestCodeToTestId = self.getMapTestCodeToTestId(eqipmentId)
        db = QtGui.qApp.db
        db.transaction()
        try:
            for serviceResult in self.serviceResults:
                for value in serviceResult.values:
                    testId = mapTestCodeToTestId.get(value.code)
                    unit = forceString(db.translate('rbUnit', 'id', value.unitId, 'code'))
                    if testId:
                        referenceRangeAsStr = self.getReferenceRangeAsStr(value.referenceRange)
                        self.updateOrAddProbe(eqipmentId, self.fhirOrderId, testId, value.value, unit, value.unitId, referenceRangeAsStr)
                    else:
                        freeResuts[value.code] = { 'value' : value.value,
                                                   'unit'  : unit,
                                                   'referenceRange' : value.referenceRange
                                                 }
            if freeResuts:
                testId = mapTestCodeToTestId.get('blank')
                if testId:
                    self.updateOrAddProbe(eqipmentId, self.fhirOrderId, testId, json.dumps(freeResuts), '', None, '')

            if self.finished:
                self.markRemainedProbesAsAcceptedWithoutResult(eqipmentId, self.fhirOrderId)
            db.commit()
        except:
            db.rollback()
            raise


def selectFHIROrders(probeIdList):
    db = QtGui.qApp.db
    tableProbe = db.table('Probe')
    tableTakenTissueJournal = db.table('TakenTissueJournal')
    tableAction = db.table('Action')
    tableActionContract = db.table('Action').alias('ActionContract')
    tableActionType = db.table('ActionType')
    tableActionPropertyType = db.table('ActionPropertyType')
    tablePerson = db.table('Person')
    tableDiagnosis = db.table('Diagnosis')
    tableEvent = db.table('Event')
    tableEventContract = db.table('Contract').alias('EventContract')
    tableEventType = db.table('EventType')

    table = tableProbe
    table = table.leftJoin(tableTakenTissueJournal, tableTakenTissueJournal['id'].eq(tableProbe['takenTissueJournal_id']))
    table = table.innerJoin(tableAction,     tableAction['takenTissueJournal_id'].eq(tableProbe['takenTissueJournal_id']))
    table = table.innerJoin(tableEvent,      tableEvent['id'].eq(tableAction['event_id']))
    table = table.innerJoin(tableEventType,  tableEventType['id'].eq(tableEvent['eventType_id']))
    table = table.innerJoin(tableActionType, tableActionType['id'].eq(tableAction['actionType_id']))
    table = table.innerJoin(tableActionPropertyType, db.joinAnd([ tableActionPropertyType['actionType_id'].eq(tableActionType['id']),
                                                                  tableActionPropertyType['test_id'].eq(tableProbe['test_id']),
                                                                  tableActionPropertyType['deleted'].eq(0)
                                                                ]
                                                               )
                           )
    table = table.leftJoin(tablePerson,     tablePerson['id'].eq(tableAction['setPerson_id']))
    table = table.leftJoin(tableDiagnosis,   'Diagnosis.id = getEventDiagnosis(Action.event_id)')
    table = table.leftJoin(tableActionContract,  tableActionContract['id'].eq(tableAction['contract_id']))
    table = table.leftJoin(tableEventContract,   tableEventContract['id'].eq(tableEvent['contract_id']))

    cond = [ tableProbe['id'].inlist(probeIdList)
           ]

    financeIdDef = 'COALESCE(%s.finance_id, %s.finance_id, %s.finance_id) AS finance_id' % (
                    tableAction.tableName,
                    tableActionContract.tableName,
                    tableEventContract.tableName )


    recordList = db.getRecordList(table,
                                  [ tableProbe['id'].alias('probe_id'),
                                    tableProbe['externalId'],
                                    tableProbe['takenTissueJournal_id'],
                                    tableProbe['specimenType_id'],
                                    tableProbe['containerType_id'],
                                    tableTakenTissueJournal['datetimeTaken'],
                                    tableAction['id'].alias('action_id'),
                                    tableAction['actionType_id'],
                                    tableActionType['nomenclativeService_id'].alias('service_id'),
                                    tableAction['directionDate'].alias('directionDatetime'),
                                    tableAction['id'].alias('action_id'),
                                    tableAction['event_id'],
                                    tableEvent['eventType_id'],
                                    tableEventType['purpose_id'].alias('eventPurpose_id'),
                                    tablePerson['orgStructure_id'],
                                    tableAction['setPerson_id'],
                                    tableAction['MKB'].alias('actionMKB'),
                                    tableDiagnosis['MKB'].alias('eventMKB'),
                                    financeIdDef,
                                  ],
                                  cond
                                 )

    orderDict = {}
    for record in recordList:
        probeId              = forceRef(record.value('probe_id'))
        externalId           = forceString(record.value('externalId'))
        takenTissueJournalId = forceRef(record.value('takenTissueJournal_id'))
        specimenTypeId       = forceRef(record.value('specimenType_id'))
        containerTypeId      = forceRef(record.value('containerType_id'))
        takenDatetime        = forceDateTime(record.value('datetimeTaken'))
        actionId             = forceRef(record.value('action_id'))
        actionTypeId         = forceRef(record.value('actionType_id'))
        serviceId            = forceRef(record.value('service_id'))
        directionDatetime    = forceDateTime(record.value('directionDatetime'))
        eventId              = forceRef(record.value('event_id'))
        eventTypeId          = forceRef(record.value('eventType_id'))
        eventPurposeId       = forceRef(record.value('eventPurpose_id'))
        orgStructureId       = forceRef(record.value('orgStructure_id'))
        setPersonId          = forceRef(record.value('setPerson_id'))
        MKB                  = forceString(record.value('actionMKB')) or forceString(record.value('eventMKB')) or 'Z00'
        financeId            = forceRef(record.value('finance_id'))
#        key = (takenTissueJournalId, externalId)
        orderKey = takenTissueJournalId
        order = orderDict.get(orderKey)
        if order is None:
            orderDict[orderKey] = order = COrderDto()
        order.addProbe(eventId,
                       eventTypeId,
                       eventPurposeId,
                       actionId,
                       actionTypeId,
                       directionDatetime,
                       orgStructureId,
                       setPersonId,
                       MKB,
                       serviceId,
                       takenTissueJournalId,
                       takenDatetime,
                       externalId,
                       specimenTypeId,
                       containerTypeId,
                       probeId,
                       financeId
                      )
    return orderDict.values()


def dateToFHIRDate(date):
    result = FHIRDate()
    if date:
        if isinstance(date, QDateTime):
            date = date.date()
        pd = date.toPyDate()
        result.date = pd
    return result


def dateTimeToFHIRDate(dateTime):
    result = FHIRDate()
    if dateTime:
#        pdt = dateTime.toPyDateTime().replace(microsecond=0)
        pdt = dateTime.toPyDateTime()
        if pdt.tzinfo is None:
            pdt = pdt.replace(tzinfo=isodate.LOCAL)
        result.date = pdt
    return result


class CFHIRExchange:
    urnMisIdentifier  = 'urn:oid:1.2.643.5.1.13.2.7.100.5'  # OID для идентификатора в МИС/ЛИС
    urnDocuments      = 'urn:oid:1.2.643.2.69.1.1.1.6'      # OID для разных документов
    urnDocumentProviders='urn:netrika:documentProvider'     # УФМС, ЗАГС и т.п.
    urnSnils          = 'urn:oid:1.2.643.2.69.1.1.1.6.223'  # OID ПФР для СНИЛСа
    urnOldPolicy      = 'urn:oid:1.2.643.2.69.1.1.1.6.226'  # OID для страхового полиса ОМС старого образца
    urnTmpPolicy      = 'urn:oid:1.2.643.2.69.1.1.1.6.227'  # OID для временного свидетельства
    urnNewPolicy      = 'urn:oid:1.2.643.2.69.1.1.1.6.228'  # OID для страхового полиса ОМС единого образца
    urnVolPolicy      = 'urn:oid:1.2.643.2.69.1.1.1.6.240'  # OID для страхового полиса ДМС

    urnOrgs           = 'urn:oid:1.2.643.2.69.1.1.1.64'     # кодификатор организаций
    urnHicRegistry    = 'urn:oid:1.2.643.5.1.13.2.1.1.635'  # Реестр СМО (ФФОМС)
    HicRegistry       = '1.2.643.5.1.13.2.1.1.635'          # Реестр СМО (ФФОМС)

    urnSpecialities   = 'urn:oid:1.2.643.5.1.13.13.11.1066' # Специальности
    urnRoles          = 'urn:oid:1.2.643.5.1.13.13.11.1002' # Роли (Должности)
    urnReasons        = 'urn:oid:1.2.643.2.69.1.1.1.19'     # Назначение события
    urnServices       = 'urn:oid:1.2.643.2.69.1.1.1.31'     # Услуги
    urnSpecimenTypes  = 'urn:oid:1.2.643.5.1.13.13.11.1081' # типы образцов
    urnContainerTypes = 'urn:oid:1.2.643.2.69.1.1.1.34'     # типы контейнеров

    urnCaseTypes        = 'urn:oid:1.2.643.2.69.1.1.1.35'   # Тип случая обслуживания
    CaseTypeVersion     = '1'
    CasePolyclinic      = '2'
    CaseDispObservation = '4'

    urnConditions      = 'urn:oid:1.2.643.2.69.1.1.1.36'    # типы Condition
    ConditionDiagnosis = 'diagnosis'
    ConditionVersion   = '1'

#    urnLoinc          = 'urn:oid:1.2.643.2.69.1.1.1.1'
    urnICDs           = 'urn:oid:1.2.643.2.69.1.1.1.2'      # Коды диагнозов по МКБ
    ICDVersion        = ''

    urnInterpretations = 'urn:oid:1.2.643.5.1.13.13.11.1381'
    urnInterpretationsVersion = ''
    mapEvaluationToInterpretation = { None: 'IND',
                                      -2  : 'LU',
                                      -1  : 'L',
                                       0  : 'N',
                                      +1  : 'H',
                                      +2  : 'HU',
                                    }

    urnUnits           = 'urn:oid:1.2.643.5.1.13.13.11.1358' # Единицы измерения
    urnFinances        = 'urn:oid:1.2.643.2.69.1.1.1.32'     # Источники финансирования
    urnFinancesDefault = '6'
    urnFinancesVersion = ''

    orderProfile       = 'StructureDefinition/cd45a667-bde0-490f-b602-8d780acf4aa2'
    resultProfile      = 'StructureDefinition/21f687dd-0b3b-4a7b-af8f-04be625c0201'


    def __init__(self, opts, orgId):
        self.url            = self.getOpt(opts, 'url',  u'URL Сервиса ОДЛИ')
        self.terminologyUrl = self.getOpt(opts, 'terminology_url',  u'URL Сервиса терминологии') # , 'http://r78-rc.zdrav.netrika.ru/Terminology/term')
        self.authorization  = self.getOpt(opts, 'authorization',  u'Значение заголовка авторизации')
        target              = self.getOpt(opts, 'target',  u'GUID лаборатории')
        self.SAMSON         = self.getOpt(opts, 'mis_oid', u'OID МИС')
        self.urnTests       = self.getOpt(opts, 'tests_urn', u'URN справочника тестов')
        self.urnTestsVersion= self.getOpt(opts, 'tests_version', u'Версия справочника тестов', '')

        self.orgShortCode = forceString(QtGui.qApp.db.translate('Organisation', 'id', orgId, 'infisCode'))
        self.orgCode  = getIdentification('Organisation', orgId, self.urnOrgs)

        self.org = self.createOrganization(self.orgCode)
        self.lab = self.createOrganization(target)

        settings = { 'api_base': self.url,
                     'app_id'  : 'samson/0.1',
                     'headers' : { 'Authorization' : self.authorization,},
#                     'conformance' : Conformance,
                     'conformance' : None,
                   }

        self.smart = client.FHIRClient(settings=settings)
        self.smart.prepare()

        self._terminologyClient = None

        self.orgReference = self.createReference(self.org)
        self.labReference = self.createReference(self.lab)

        self._mapPolicyTypeIdToComp = {}
        self._mapPolicyKindIdToCode = {}
        self._mapUnitIdentificationToId = {}
        self._mapUriToVersion = downloadVersionsOfDictionaries(self.getTerminologyClient())


    def getOpt(self, opts, param, title, default=None):
        if param in opts:
            result = opts.get(param)
            return result
        if default is None:
            raise Exception(u'Не задано значение параметра %s (%s)' % (param, title))
        return default


    def getTerminologyClient(self):
        if self._terminologyClient is None:
            settings = { 'api_base': self.terminologyUrl,
                         'app_id'  : 'samson/0.1',
#                         'headers' : { 'Authorization' : self.authorization,},
#                 'conformance' : Conformance,
                         'conformance' : None,
                       }

            self._terminologyClient = client.FHIRClient(settings=settings)
            self._terminologyClient.prepare()
        return self._terminologyClient


    def _getPolicyTypeComp(self, policyTypeId):
        result = self._mapPolicyTypeIdToComp.get(policyTypeId)
        if result is None:
            result = forceBool(QtGui.qApp.db.translate('rbPolicyType', 'id', policyTypeId, 'isCompulsory'))
            self._mapPolicyTypeIdToComp[policyTypeId] = result
        return result


    def _getPolicyKindCode(self, policyKindId):
        code = self._mapPolicyKindIdToCode.get(policyKindId)
        if code is None:
            code = forceString(QtGui.qApp.db.translate('rbPolicyKind', 'id', policyKindId, 'code'))
            self._mapPolicyKindIdToCode[policyKindId] = code
        return code


    def _guessPolicyKindCode(self, serial, number):
        if number.isdigit():
            if ( not serial or serial.upper() == u'ЕП') and len(number) == 16:
                return '3'
            if ( not serial or serial.upper() == u'ВР') and len(number) == 9:
                return '2'
            if len(serial)>1 and len(number)>=6:
                return '1'
        return None


    def createReference(self, resource):
        if resource.id is None:
            resp = resource.create(self.smart.server)
            resource.update_with_json(resp)
        reference = FHIRReference()
        reference.reference =  '%s/%s' % (resource.resource_name, resource.id)
        return reference


    def createDispReference(self, display):
        reference = FHIRReference()
        reference.display = display
        return reference


    def newBundleReference(self, url):
        reference = FHIRReference()
        reference.reference = url
        return reference


#проверить!
    def createIdentifier(self, system, value, assigner=None):
        identifier = Identifier()
        identifier.system = system
        identifier.value  = value
        identifier.assigner = assigner
        return identifier


    def createPPMisIdentifier(self, value):
        identifier = Identifier()
        identifier.system = self.urnMisIdentifier
        identifier.value  = 'SAMSON:%s:%s' % (self.orgShortCode, value)
        identifier.assigner = self.createDispReference(self.SAMSON)
        return identifier


    def createMisIdentifier(self, value):
        identifier = Identifier()
        identifier.system = 'urn:oid:%s' % self.SAMSON
        identifier.value  = '%s:%s' % (self.orgShortCode, value)
        identifier.assigner = self.orgReference
        return identifier


    def createCoding(self, system, code, version):
        coding = Coding()
        coding.system=system
        coding.code=code
#        coding.version=version
#        coding.version=''
        coding.version = self._mapUriToVersion.get(system, version)
        return coding


    def createCodeableConcept(self, system, code, version):
        codeableConcept = CodeableConcept()
        codeableConcept.coding = [ self.createCoding(system,
                                                     code,
                                                     version
                                                    )
                                 ]
        return codeableConcept


    def createOrganization(self, fhirId):
        org = Organization()
        org.id = fhirId
        return org


#проверить!
    def createOrgStructureReference(self, orgStructureId):
#        if orgStructureId:
#            fhirId = getOrgStructureIdentification(self.urnOrgs, orgStructureId)
#            return self.createReference(self.createOrganization(fhirId))
        return self.orgReference


#проверить!
    def searchByIdentifier(self, cls, identifier):
        identifierStr = ('%s|%s' % (identifier.system,identifier.value)).encode('utf8')
        searchObject = cls.where({'identifier': identifierStr})
        searchBundle = searchObject.perform(self.smart.server)
        return searchBundle


    def documentAsIdentifier(self, documentRecord):
        serial = forceString(documentRecord.value('serial'))
        number = forceString(documentRecord.value('number'))
        documentTypeId = forceString(documentRecord.value('documentType_id'))
        date = forceDate(documentRecord.value('date'))
        code = None
        try:
            code, version = getIdentificationEx('rbDocumentType', documentTypeId, self.urnDocuments, True)
        except:
            QtGui.qApp.logCurrentException()
        if serial and number and code:
            provider, version = getIdentificationEx('rbDocumentType', documentTypeId, self.urnDocumentProviders, False)
            if provider is None:
                if code == '14':
                    provider = u'УФМС'
                elif code == '3':
                    provider = u'ЗАГС'
                else:
                    provider = u'-'
            document = ('%s:%s' % (serial, number)).replace(' ','')
            identifier = self.createIdentifier('%s.%s' % (self.urnDocuments, code),
                                               document,
                                               self.createDispReference(provider))
            if date:
                identifier.period = Period()
                identifier.period.start = dateToFHIRDate(date)
            return identifier
        return None


    def snilsAsIdentifier(self, snils):
        identifier = self.createIdentifier(self.urnSnils,
                                           snils,
                                           self.createDispReference(u'ПФР'))
        return identifier


    def policyAsIdentifier(self, clientId, policyRecord):
        insurerId = forceRef(policyRecord.value('insurer_id'))
        serial    = forceString(policyRecord.value('serial'))
        number    = forceString(policyRecord.value('number'))
        begDate   = forceDate(policyRecord.value('begDate'))
        endDate   = forceDate(policyRecord.value('endDate'))
        if insurerId is None:
            raise Exception(u'У пациента Client.id=%s\n'
                            u'в полисе серия:«%s», номер: «%s»\n'
                            u'не указана страховая компания' % (clientId, serial, number))

        policyTypeComp = self._getPolicyTypeComp(forceRef(policyRecord.value('policyType_id')))
        if policyTypeComp:
            policyKindCode = self._getPolicyKindCode(forceRef(policyRecord.value('policyKind_id')))
            if not policyKindCode:
                policyKindCode = self._guessPolicyKindCode(serial, number)
            if policyKindCode == '1': # старый
                system = self.urnOldPolicy
            elif policyKindCode == '2': # временный
                system = self.urnTmpPolicy
            elif policyKindCode == '3': # новый
                system = self.urnNewPolicy
            else:
                return None
        else:
            system = self.urnVolPolicy
        insurerCode = getIdentification('Organisation', insurerId, self.urnHicRegistry)
        value = number if system == self.urnNewPolicy else '%s:%s' % (serial, number)
        identifier = self.createIdentifier(system,
                                           value,
                                           self.createDispReference('%s.%s' % (self.HicRegistry, insurerCode)))
        if begDate or endDate:
            identifier.period = Period()
            if begDate:
                identifier.period.start = dateToFHIRDate(begDate)
            if endDate:
                identifier.period.end = dateToFHIRDate(endDate)
        return identifier


    def createPatient(self, clientInfo):
        patient = Patient()
        patient.active = True
        name = HumanName()
        name.family = [ clientInfo.lastName or '-', clientInfo.patrName or '-']
        name.given  = [ clientInfo.firstName or '-']
        name.text   = formatNameInt(clientInfo.lastName, clientInfo.firstName, clientInfo.patrName)
        name.use = 'official'
        patient.name = [ name ]
        patient.gender = {  1: 'male', 2:'female' }.get(clientInfo.sexCode, 'undefined')
        patient.birthDate = dateToFHIRDate(clientInfo.birthDate)

        patient.identifier = [ self.createPPMisIdentifier(clientInfo.id),  # идентификатор в МИС
#                               self.createMisIdentifier(clientInfo.id),
                             ]
        if clientInfo.documentRecord:
            documentAsIdentifier = self.documentAsIdentifier(clientInfo.documentRecord)
            if documentAsIdentifier is not None:
                patient.identifier.append(documentAsIdentifier)
        if clientInfo.SNILS:
            snils = clientInfo.SNILS.replace('-', '').replace(' ', '')
            patient.identifier.append(self.snilsAsIdentifier(snils))
        if clientInfo.compulsoryPolicyRecord:
            policyAsIdentifier = self.policyAsIdentifier(clientInfo.id, clientInfo.compulsoryPolicyRecord)
            if policyAsIdentifier:
                patient.identifier.append(policyAsIdentifier)
        if clientInfo.voluntaryPolicyRecord:
            policyAsIdentifier = self.policyAsIdentifier(clientInfo.id, clientInfo.voluntaryPolicyRecord)
            if policyAsIdentifier:
                patient.identifier.append(policyAsIdentifier)
        address = clientInfo.locAddress
        if address:
            patient.address = [ Address( { 'use' : 'home',
                                           'text': address
                                         }
                                       )
                              ]
        patient.managingOrganization = self.orgReference
        resp = patient.create(self.smart.server)
        assert resp['resourceType'] == 'Patient'
        patient.update_with_json(resp)
        return patient


    def newPractitioner(self, personId):
        db = QtGui.qApp.db
        record = db.getRecord('Person',
                             ('lastName','firstName','patrName','SNILS','post_id','speciality_id','orgStructure_id'),
                             personId
                             )
        lastName  = forceString(record.value('lastName'))
        firstName = forceString(record.value('firstName'))
        patrName  = forceString(record.value('patrName'))
        snils     = forceString(record.value('SNILS'))
        postId    = forceRef(record.value('post_id'))
        specialityId = forceRef(record.value('speciality_id'))
        orgStructureId = forceRef(record.value('orgStructure_id'))
        specialityCode, specialityVersion = getIdentificationEx('rbSpeciality',
                                                                specialityId,
                                                                self.urnSpecialities
                                                               )
        postCode,       postVersion       = getIdentificationEx('rbPost',
                                                                postId,
                                                                self.urnRoles
                                                               )
        practitioner = Practitioner()
        practitioner.active = True
        name = HumanName()
        name.family = [ lastName or '-', patrName or '-' ]
        name.given  = [ firstName or '-' ]
        name.text   = formatNameInt(lastName, firstName, patrName)
        practitioner.name = name

        practitioner.identifier = [ self.createPPMisIdentifier(personId),  # идентификатор в МИС
#                                    self.createMisIdentifier(personId),
                                  ]
        if snils:
            practitioner.identifier.append(self.snilsAsIdentifier(snils))

        ppr =  PractitionerPractitionerRole()
        ppr.managingOrganization = self.createOrgStructureReference(orgStructureId)
        ppr.specialty = [ self.createCodeableConcept(self.urnSpecialities,
                                                     specialityCode,
                                                     specialityVersion
                                                   )
                        ]
        ppr.role      = self.createCodeableConcept(self.urnRoles,
                                                   postCode,
                                                   postVersion
                                                  )

        practitioner.practitionerRole = [ ppr ]
        return practitioner


    def newConditionFromDiagnosis(self, patientReference, diagnosisCode):
        condition = Condition()
        condition.patient = patientReference
        condition.category = self.createCodeableConcept(self.urnConditions,
                                                        self.ConditionDiagnosis,
                                                        self.ConditionVersion
                                                       )
        condition.code = self.createCodeableConcept(self.urnICDs,
                                                    diagnosisCode,
                                                    self.ICDVersion
                                                   )
        condition.verificationStatus = 'confirmed'
        return condition


#проверить!
    def newEncounter(self, patientReference, eventId, eventTypeId, eventPurposeId, orgStructureId, diagnosisList):
        typeCode, typeVersion =  getIdentificationEx('EventType',
                                                     eventTypeId,
                                                     self.urnCaseTypes,
                                                     False
                                                    )
        if typeCode is None:
            typeCode, typeVersion =  getIdentificationEx('rbEventTypePurpose',
                                                         eventPurposeId,
                                                         self.urnCaseTypes,
                                                         False
                                                        )
        if typeCode is None:
            typeCode, typeVersion = self.CasePolyclinic, self.CaseTypeVersion

        reasonCode, reasonVersion = getIdentificationEx('EventType',
                                                        eventTypeId,
                                                        self.urnReasons,
                                                        False
                                                       )
        if reasonCode is None:
            reasonCode, reasonVersion = getIdentificationEx('rbEventTypePurpose',
                                                            eventPurposeId,
                                                            self.urnReasons,
                                                            True
                                                           )

        encounter = Encounter()
        encounter.identifier = [ self.createMisIdentifier(eventId) ]
        encounter.status     = 'in-progress'
        encounter.class_fhir = 'outpatient'
        encounter.type       = [ self.createCodeableConcept(self.urnCaseTypes, typeCode, typeVersion) ]
        encounter.patient    = patientReference
        encounter.reason     = [ self.createCodeableConcept(self.urnReasons, reasonCode, reasonVersion) ]
        encounter.indication = diagnosisList
        encounter.serviceProvider = self.createOrgStructureReference(orgStructureId)
        return encounter


    def newSpecimen(self,
                       patientReference,
                       specimenTypeId,
                       containerTypeId,
                       externalId,
                       takenDatetime,
                       probeId
                      ):
        specimenTypeCode, specimenTypeVersion = getIdentificationEx('rbSpecimenType',
                                                                    specimenTypeId,
                                                                    self.urnSpecimenTypes
                                                                   )
        containerTypeCode, containerTypeVersion = getIdentificationEx('rbContainerType',
                                                                      containerTypeId,
                                                                      self.urnContainerTypes,
                                                                     )
        specimen = Specimen()
        specimen.identifier = [ self.createMisIdentifier(probeId) ]
        specimen.subject = patientReference
        specimen.type = self.createCodeableConcept(self.urnSpecimenTypes,
                                                   specimenTypeCode,
                                                   specimenTypeVersion)

        specimen.collection = SpecimenCollection()
        specimen.collection.collectedDateTime = dateTimeToFHIRDate(takenDatetime)
        specimen.container = [ SpecimenContainer() ]
        specimen.container[0].identifier = [ self.createIdentifier('urn:uuid:'+ self.lab.id,
                                                              externalId,
                                                              self.labReference
                                                             )
                                           ]
        specimen.container[0].type = self.createCodeableConcept(self.urnContainerTypes, containerTypeCode, containerTypeVersion)
        specimen._key = (specimenTypeCode, containerTypeCode, externalId)
        return specimen


    def newDiagnosticOrderItem(self,
                               serviceCode,
                               serviceVersion,
                               financeCode,
                               financeVersion
                              ):
        diagnosticOrderItem  = DiagnosticOrderItem()
        diagnosticOrderItem.code = self.createCodeableConcept(self.urnServices,
                                                              serviceCode,
                                                              serviceVersion)

        e = Extension()
        e.url = 'urn:oid:1.2.643.2.69.1.100.1'
        e.valueCodeableConcept = self.createCodeableConcept(self.urnFinances,
                                                            financeCode,
                                                            financeVersion)

        diagnosticOrderItem.code.extension = [ e ]
        return diagnosticOrderItem


    def newDiagnosticOrder(self,
                           patientReference,
                           practitionerReference,
                           encounterReference,
                           observations,
                           specimenReferenceList,
                           serviceIdList,
                           financeId):
        diagnosticOrder = DiagnosticOrder()
        diagnosticOrder.subject   = patientReference
        diagnosticOrder.orderer   = practitionerReference
        diagnosticOrder.encounter = encounterReference
#        diagnosticOrder.supportingInformation = observations
        diagnosticOrder.specimen = specimenReferenceList
        diagnosticOrder.status    = 'requested'
        diagnosticOrder.priority  = 'routine'
        diagnosticOrder.item = []
        serviceCodeSet = set(getIdentificationEx('rbService', serviceId, self.urnServices)
                             for serviceId in serviceIdList
                            )
        serviceCodeList = list(serviceCodeSet)
        serviceCodeList.sort()
        if financeId:
            financeCode, financeVersion = getIdentificationEx('rbFinance', financeId, self.urnFinances, True)
        else:
            financeCode, financeVersion = self.urnFinancesDefault, self.urnFinancesVersion

        for serviceCode, serviceVersion in serviceCodeList:
            item = self.newDiagnosticOrderItem(serviceCode,
                                               serviceVersion,
                                               financeCode,
                                               financeVersion
                                              )
            diagnosticOrder.item.append(item)
        return diagnosticOrder


    def newOrder(self,
                 orderIdentifier,
                 orderDatetime,
                 patientReference,
                 practitionerReference,
                 diagnosticOrderReferenceList
                ):
        order = Order()
        order.identifier = [ orderIdentifier ]
        order.date = dateTimeToFHIRDate(orderDatetime)
#        order.date = dateTimeToFHIRDate(QDateTime.currentDateTime())
        order.subject = patientReference
        order.source = practitionerReference
        order.target = self.labReference
        order.when = OrderWhen()
        order.when.code = self.createCodeableConcept('urn:oid:1.2.643.2.69.1.1.1.30', 'Routine', version='1')
        order.detail = diagnosticOrderReferenceList
        return order


    def createEmptyTransactionBundle(self, profile):
        result = Bundle()
        result.meta  = Meta( dict(profile = [ profile ] ))
        result.type  = 'transaction'
        result.entry = []
        return result


    def addBundleEntry(self, bundle, resource):
        entry = BundleEntry()
        entry.fullUrl = 'urn:uuid:%s' % uuid.uuid4()
        entry.resource = resource
        entry.request = BundleEntryRequest()
        entry.request.method = 'POST'
        entry.request.url    = resource.resource_name
        bundle.entry.append(entry)
        return self.newBundleReference(entry.fullUrl)


    def newOrderBundle(self,
                       order,
                       orderIdentifier,
                       patientReference
                      ):
        bundle = self.createEmptyTransactionBundle(self.orderProfile)
        mapPersonIdToPractitionerReference = {}
        mapSpecimenKeyToSpecimenReference = {}
        mapDiagnosisToConditionReference = {}
#        mapEventIdToEncounterReference = {}
        mapActionToSpecimenReference = {}
        diagnosticOrderReferenceList = []
        orderDatetime = order.takenDatetime or QDateTime.currentDateTime()

        for event in order.getEventList():
            eventDiagnosisSet = set()
            orgStructureId = None
            for action in event.getActionList():
                personId = action.setPersonId
                if personId not in mapPersonIdToPractitionerReference:
                    practitioner = self.newPractitioner(personId)
                    reference = self.addBundleEntry(bundle, practitioner)
                    mapPersonIdToPractitionerReference[personId] = reference

                mapSpecimenKeyToSpecimenReferenceInAction = {}
                for sp in action.specimenList:
                    specimen = self.newSpecimen(patientReference,
                                                sp.specimenTypeId,
                                                sp.containerTypeId,
                                                sp.externalId,
                                                sp.takenDatetime,
                                                sp.probeId
                                               )
                    specimenKey = specimen._key
                    if specimenKey not in mapSpecimenKeyToSpecimenReference:
                        reference = self.addBundleEntry(bundle, specimen)
                        mapSpecimenKeyToSpecimenReference[specimenKey] = reference
                    else:
                        reference = mapSpecimenKeyToSpecimenReference[specimenKey]
                    mapSpecimenKeyToSpecimenReferenceInAction[specimenKey] = reference
                mapActionToSpecimenReference[id(action)] = mapSpecimenKeyToSpecimenReferenceInAction.values()

                diagnosis = action.diagnosis[:5]
                if diagnosis not in mapDiagnosisToConditionReference:
                   condition = self.newConditionFromDiagnosis(patientReference, diagnosis)
                   reference = self.addBundleEntry(bundle, condition)
                   mapDiagnosisToConditionReference[diagnosis] = reference
                eventDiagnosisSet.add(diagnosis)

                if not orgStructureId:
                    orgStructureId = action.orgStructureId

            encounter = self.newEncounter(patientReference,
                                          event.eventId,
                                          event.eventTypeId,
                                          event.eventPurposeId,
                                          orgStructureId,
                                          [ mapDiagnosisToConditionReference[ds]
                                            for ds in eventDiagnosisSet
                                          ]
                                         )
            encounterReference = self.addBundleEntry(bundle, encounter)
#            mapEventIdToEncounterReference[event.eventId] = encounterReference

            for action in event.getActionList():
                diagnosticOrder = self.newDiagnosticOrder(patientReference,
                                                          mapPersonIdToPractitionerReference[action.setPersonId],
                                                          encounterReference,
                                                          None,
                                                          mapActionToSpecimenReference.get(id(action), []),
                                                          [action.serviceId],
                                                          action.financeId
                                                         )
                diagnosticOrderReference = self.addBundleEntry(bundle, diagnosticOrder)
                diagnosticOrderReferenceList.append(diagnosticOrderReference)

        order = self.newOrder(orderIdentifier,
                              orderDatetime,
                              patientReference,
                              mapPersonIdToPractitionerReference.values()[0],
                              diagnosticOrderReferenceList
                             )
        self.addBundleEntry(bundle, order)
        return bundle


    def sendOrderOverFHIR(self, clientInfo, order):
        patient           = self.createPatient(clientInfo)
        patientReference  = self.createReference(patient)
#        probeId = min(order.probeIdSet)
#        orderIdentifier = self.createMisIdentifier(probeId)
        orderIdentifier = self.createMisIdentifier(str(uuid.uuid4()))
        bundle = self.newOrderBundle(order,
                                     orderIdentifier,
                                     patientReference)
        res = self.smart.server.post_json('', bundle.as_json())
        resAsJson = res.json()
        orderFullUrl = resAsJson['entry'][-1]['fullUrl']
        if orderFullUrl.startswith('Order/'):
            identifier = orderFullUrl.partition('/')[2]
            if identifier:
                return identifier
        return orderIdentifier.value


    def newObservation(self, res, mapPersonIdToPractitionerReference):
        db = QtGui.qApp.db
        observation = Observation()
        observation.code = self.createCodeableConcept(self.urnTests,
                                                      res.testCode,
                                                      self.urnTestsVersion
                                                    )
        observation.status = 'final'
        observation.issued = dateTimeToFHIRDate(res.datetime)
        if res.personId:
            observation.performer = [ mapPersonIdToPractitionerReference.get(res.personId)
                                    ]
        if isinstance(res.value, (float, int, long)):
            unitCode, unitVersion = getIdentificationEx('rbUnit', res.unitId, self.urnUnits)
            unitName = forceString(db.translate('rbUnit', 'id', res.unitId, 'code'))
            observation.valueQuantity = Quantity()
            observation.valueQuantity.value = float(res.value)
            observation.valueQuantity.system = self.urnUnits
            observation.valueQuantity.code  = unitCode
            observation.valueQuantity.unit  = unitName
            if res.norm != (None, None):
                observation.referenceRange = [ ObservationReferenceRange() ]
                if res.norm[0] is not None:
                    quantity = Quantity()
                    quantity.value = float(res.norm[0])
                    quantity.system = observation.valueQuantity.system
                    quantity.code   = observation.valueQuantity.code
                    quantity.unit   = observation.valueQuantity.unit
                    observation.referenceRange[0].low = quantity
                if res.norm[1] is not None:
                    quantity = Quantity()
                    quantity.value = float(res.norm[1])
                    quantity.system = observation.valueQuantity.system
                    quantity.code   = observation.valueQuantity.code
                    quantity.unit   = observation.valueQuantity.unit
                    observation.referenceRange[0].high = quantity
###            forceString(db.translate('rbUnit', 'id', res.unitId, 'code'))
            else:
                observation.referenceRange = [ ObservationReferenceRange(jsondict=dict(text='-')) ]
            observation.interpretation = self.createCodeableConcept(self.urnInterpretations,
                                                                    self.mapEvaluationToInterpretation.get(res.evaluation, 'E'),
                                                                    self.urnInterpretationsVersion
                                                                   )
        else:
            observation.valueString = unicode(res.value)
            observation.referenceRange = [ ObservationReferenceRange(jsondict=dict(text='-')) ]
            observation.interpretation = self.createCodeableConcept(self.urnInterpretations,
                                                                    self.mapEvaluationToInterpretation.get(res.evaluation, 'E'),
                                                                    self.urnInterpretationsVersion
                                                                   )
        return observation


    def newBinary(self, contentType, content):
        binary = Binary()
        binary.contentType = 'application/pdf'
        binary.content = 'JVBERi0xLjEKJcKlwrHDqwoKMSAwIG9iagogIDw8IC9UeXBlIC9DYXRhbG9nCiAgICAgL1BhZ2Vz' \
                         'IDIgMCBSCiAgPj4KZW5kb2JqCgoyIDAgb2JqCiAgPDwgL1R5cGUgL1BhZ2VzCiAgICAgL0tpZHMg' \
                         'WzMgMCBSXQogICAgIC9Db3VudCAxCiAgICAgL01lZGlhQm94IFswIDAgMzAwIDE0NF0KICA+Pgpl' \
                         'bmRvYmoKCjMgMCBvYmoKICA8PCAgL1R5cGUgL1BhZ2UKICAgICAgL1BhcmVudCAyIDAgUgogICAg' \
                         'ICAvUmVzb3VyY2VzCiAgICAgICA8PCAvRm9udAogICAgICAgICAgIDw8IC9GMQogICAgICAgICAg' \
                         'ICAgICA8PCAvVHlwZSAvRm9udAogICAgICAgICAgICAgICAgICAvU3VidHlwZSAvVHlwZTEKICAg' \
                         'ICAgICAgICAgICAgICAgL0Jhc2VGb250IC9UaW1lcy1Sb21hbgogICAgICAgICAgICAgICA+Pgog' \
                         'ICAgICAgICAgID4+CiAgICAgICA+PgogICAgICAvQ29udGVudHMgNCAwIFIKICA+PgplbmRvYmoK' \
                         'CjQgMCBvYmoKICA8PCAvTGVuZ3RoIDU1ID4+CnN0cmVhbQogIEJUCiAgICAvRjEgMTggVGYKICAg' \
                         'IDAgMCBUZAogICAgKEhlbGxvIFdvcmxkKSBUagogIEVUCmVuZHN0cmVhbQplbmRvYmoKCnhyZWYK' \
                         'MCA1CjAwMDAwMDAwMDAgNjU1MzUgZiAKMDAwMDAwMDAxOCAwMDAwMCBuIAowMDAwMDAwMDc3IDAw' \
                         'MDAwIG4gCjAwMDAwMDAxNzggMDAwMDAgbiAKMDAwMDAwMDQ1NyAwMDAwMCBuIAp0cmFpbGVyCiAg' \
                         'PDwgIC9Sb290IDEgMCBSCiAgICAgIC9TaXplIDUKICA+PgpzdGFydHhyZWYKNTY1CiUlRU9GCg=='
        return binary


    def newDiagnosticReport(self, patientReference, issued, serviceId, performer, observationReferences, attachmentReference):
        serviceCode, serviceVersion = getIdentificationEx('rbService', serviceId, self.urnServices)

        diagnosticReport = DiagnosticReport()
#        diagnosticReport.conclusion = u'Value should be no empty string on DiagnosticReport'
        diagnosticReport.issued     = issued
        diagnosticReport.effectiveDateTime = issued
        diagnosticReport.code       = self.createCodeableConcept(self.urnServices,
                                                                 serviceCode,
                                                                 serviceVersion)
        diagnosticReport.performer  = performer
        diagnosticReport.result     = observationReferences
        diagnosticReport.status     = 'final'
        diagnosticReport.subject    = patientReference
        diagnosticReport.presentedForm = [ Attachment(jsondict=dict(url=attachmentReference.reference))
                                         ]
        diagnosticReport.meta = Meta()
        diagnosticReport.meta.security = [ self.createCoding('urn:oud:1.2.643.5.1.13.13.11.1116', 'N', '1')
                                         ]
        return diagnosticReport


    def newOrderResponse(self, orderReference, diagnosticReportReference, actionId):
        orderResponse = OrderResponse()
        orderResponse.who  = self.orgReference
        orderResponse.date = dateTimeToFHIRDate(QDateTime.currentDateTime())
        orderResponse.fulfillment = [ diagnosticReportReference ]
#        orderResponse.identifier = [ createPPMisIdentifier(str(actionId)) ]
        orderResponse.identifier = [ self.createMisIdentifier(str(actionId)) ]
#        orderResponse.orderStatus = 'completed'
        orderResponse.orderStatus = 'accepted'
        orderResponse.request = orderReference
        return orderResponse


#проверить!
    def newResultBundle(self, patientReference, serviceId, results):
        bundle = self.createEmptyTransactionBundle(self.resultProfile)
        mapPersonIdToPractitionerReference = {}
        personIdSet = set(res.personId for res in results if res.personId)
        datetime = max(res.datetime for res in results) or QDateTime.currentDateTime()

        for personId in personIdSet:
            practitioner = self.newPractitioner(personId)
            reference = self.addBundleEntry(bundle, practitioner)
            mapPersonIdToPractitionerReference[personId] = reference

        practitionerReference = mapPersonIdToPractitionerReference.values()[0]

        observationReferences = []
        for res in results:
            observation = self.newObservation(res, mapPersonIdToPractitionerReference)
            reference = self.addBundleEntry(bundle, observation)
            observationReferences.append(reference)

        binary = self.newBinary('','')
        binaryReference = self.addBundleEntry(bundle, binary)

        diagnosticReport = self.newDiagnosticReport(patientReference,
                                                    dateTimeToFHIRDate(datetime),
                                                    serviceId,
                                                    practitionerReference,
                                                    observationReferences,
                                                    binaryReference)
        diagnosticReportReference = self.addBundleEntry(bundle, diagnosticReport)

        order = Order()
#        order.subject = patientReference
        order.source  = practitionerReference
        order.target  = self.orgReference
        order.detail  = [Reference(jsondict=dict(reference=''))]
        orderReference = self.addBundleEntry(bundle, order)

        orderResponse = self.newOrderResponse(orderReference, diagnosticReportReference, results[0].actionId)
        self.addBundleEntry(bundle, orderResponse)
        bundle.entry[-1].fullUrl = '' # стираю fullUrl в entry для orderResponse
        return bundle


    def sendLocalResultsOverFHIR(self, clientInfo, serviceId, results):
        patient           = self.createPatient(clientInfo)
        patientReference  = self.createReference(patient)
        resultBundle = self.newResultBundle(patientReference, serviceId, results)
        self.smart.server.post_json('$addresults', resultBundle.as_json())


    def extractCode(self, coding, uri, ref, path):
        if not coding:
            raise Exception(u'%s в %s пуст' % (path, ref))
        for c in coding:
            if c.system.lower() == uri:
                return c.code
        raise Exception(u'%s в %s не имеет кода для %s' % (path, ref, uri))


    def lookupInTerminology(self, urn, code):
        request = { 'resourceType': 'Parameters',
                    'parameter': [
                                    { 'name': 'system',
                                      'valueString': urn
                                    },
                                    { 'name': 'code',
                                      'valueString': code
                                    }
                                 ]
                  }

        response = self.getTerminologyClient().server.post_json('ValueSet/$lookup', request).json()
        if 'parameter' not in response:
            raise Exception('Lookup "%s" in %s failed.' % ( code, urn))

        result = {}
        for item in response['parameter']:
            name = item['name']
            value = item['valueString']
            result[name] = value
        return result


    def lookupUnitInTerminology(self, code):
        shortName = u'краткое наименование'
        longName  = u'display'

        d = self.lookupInTerminology(self.urnUnits, code)
        try:
            return d[shortName], d[longName]
        except:
            raise Exception('Lookup "%s" in %s returned unexpected result.' % (code, self.urnUnits))


    def registerUnit(self, code, name, identification):
        # если нужно - добавить ед.изм.
        # и добавить идентификацию...
        db = QtGui.qApp.db
        tableUnit = db.table('rbUnit')
        unitId = forceRef(db.translate(tableUnit, 'code', code, 'id'))
        if unitId is None:
            unitId = forceRef(db.translate(tableUnit, 'name', name, 'id'))
        if unitId is None:
            record = tableUnit.newRecord()
            record.setValue('code',  code)
            record.setValue('name',  name)
            unitId = db.insertRecord(tableUnit, record)
        addIdentification('rbUnit', unitId, self.urnUnits, identification)


    def getUnitId(self, identification):
        # получить rbUnit.id по коду в справочнике self.urnUnits
        # key = (code, shortName if not code else None)
        unitId = self._mapUnitIdentificationToId.get(identification, False)
        if unitId is not False:
            return unitId
        if identification:
            unitId = findByIdentification('rbUnit', self.urnUnits, identification, False)
            if unitId is None:
                unitShortName, unitLongName = self.lookupUnitInTerminology(identification)
                unitId = self.registerUnit(unitShortName, unitLongName, identification)
        else:
            unitId = None
        self._mapUnitIdentificationToId[identification] = unitId
        return unitId


    def processDiagnosticReport(self, result, diagnosticReport):
        refDiagnosticReport = diagnosticReport.relativePath()
        serviceCode = self.extractCode(diagnosticReport.code.coding,
                                       self.urnServices,
                                       refDiagnosticReport,
                                       'diagnosticReport.name.coding'
                                      )
        issued      = QDateTime.fromString(diagnosticReport.issued.isostring, Qt.ISODate)
        conclusion  = diagnosticReport.conclusion
        serviceResult = CFHIRServiceResult(serviceCode, issued, conclusion)
        for item in diagnosticReport.result:
            observation = Observation.read_from(item.reference, self.smart.server)
            refObservation = observation.relativePath()
            testCode = self.extractCode(observation.code.coding,
                                        self.urnTests,
                                        refObservation,
                                        'observation.code.coding'
                                   )
            value = CFHIRResultValue(testCode)
            if observation.valueQuantity is not None:
                valueQuantity = observation.valueQuantity
                value.value = float(valueQuantity.value)
                value.unitId = self.getUnitId(valueQuantity.code)
                if observation.referenceRange:
                    if len(observation.referenceRange) != 1:
                        raise Exception(u'неожиданный observation.referenceRange в %s' % refObservation)
                    rr = observation.referenceRange[0]
                    if ( rr.low is not None
                         and (rr.low.code  and rr.low.code  != valueQuantity.code)
                       ):
                        raise Exception(u'разные единицы измерения observation.valueQuantity и observation.referenceRange[0].low в %s' % refObservation)
                    if ( rr.high is not None
                         and (rr.high.code  and rr.high.code  != valueQuantity.code)
                       ):
                        raise Exception(u'разные единицы измерения observation.valueQuantity и observation.referenceRange[0].high в %s' % refObservation)
                    value.referenceRange = ( rr.low.value  if rr.low else None,
                                             rr.high.value if rr.high else None
                                           )
            elif observation.valueString is not None:
                value.value = unicode(observation.valueString)
            else:
                raise Exception(u'неожиданный тип observation.value в %s' % refObservation)
            serviceResult.values.append(value)
        result.addServiceResult(serviceResult)


    def requestResultOverFHIR(self, fhirOrderId):
        try:
#            params = { 'request:Order.id': fhirOrderId }
            params = { 'request': 'Order/%s' % fhirOrderId }
            searchObject = OrderResponse.where(params)
            searchBundle = searchObject.perform(self.smart.server)
            result = CFHIRResult(fhirOrderId)
            if searchBundle and searchBundle.entry is not None:
    #            if len(searchBundle.entry)>1:
    #                raise Exception(u'Для заказа %s получено более одного результата' % fhirOrderId)
                orderFinished = True
                for entry in searchBundle.entry:
                    orderResponse = entry.resource
                    orderFinished = orderFinished and orderResponse.orderStatus in ('completed', 'rejected', 'error', 'cancelled', 'replaced', 'aborted')
                    for item in orderResponse.fulfillment:
                        diagnosticReport = DiagnosticReport.read_from(item.reference, self.smart.server)
                        self.processDiagnosticReport(result, diagnosticReport)
                if result.serviceResults:
                    result.finished = orderFinished
            return result
        except Exception, e:
            raise Exception(u'при получении результата по заказу %s произошла ошибка:\n%s' % (fhirOrderId, exceptionToUnicode(e)))


def getOrgStructureIdentification(urn, orgStructureId):
    db = QtGui.qApp.db
    table = db.table('OrgStructure')
    code = None
    while True:
        try:
            code, version = getIdentificationEx('OrgStructure', orgStructureId, urn, False)
            if code:
                return code
        except:
            pass
        parentId = forceRef(db.translate(table, 'id', orgStructureId, 'parent_id'))
        if parentId:
            orgStructureId = parentId
        else:
            orgId = forceRef(db.translate(table, 'id', orgStructureId, 'organisation_id'))
            return getIdentification('Organisation', orgId, urn)


def downloadVersionsOfDictionaries(terminologyClient):

    request = { 'resourceType': 'Parameters',
                'parameter': [
                    { 'name': 'objectGUID',
                      'valueString': ''
                    }
                ]
              }

    response = terminologyClient.server.post_json('$dictionaries', request).json()

    assert response['parameter'][0]['name'] == 'result'
    dictionaries = response['parameter'][0]['valueString']

    parser = xml.sax.make_parser()
    handler = ArrayOfDictionaryContractHandler()
    parser.setContentHandler(handler)
    parser.setFeature(xml.sax.handler.feature_namespaces, True)
    parser.parse(StringIO(dictionaries.encode('utf-8')))
    return handler.mapUriToVersion



class ArrayOfDictionaryContractHandler(xml.sax.handler.ContentHandler):
    nsNetrika = 'http://schemas.datacontract.org/2004/07/Netrika.TS.Contracts.Contracts'
    nsXml     = 'http://www.w3.org/2001/XMLSchema-instance'

    ArrayOfDictionaryContract = (nsNetrika, 'ArrayOfDictionaryContract')
    DictionaryContract        = (nsNetrika, 'DictionaryContract')
    Comment                   = (nsNetrika, 'Comment')
    Id                        = (nsNetrika, 'Id')
    IsModify                  = (nsNetrika, 'IsModify')
    LastUpdate                = (nsNetrika, 'LastUpdate')
    Name                      = (nsNetrika, 'Name')
    ParentName                = (nsNetrika, 'ParentName')
    SystemName                = (nsNetrika, 'SystemName')
    Uri                       = (nsNetrika, 'Uri')
    Version                   = (nsNetrika, 'Version')

    structure = { None                      : set([ArrayOfDictionaryContract]),
                  ArrayOfDictionaryContract : set([DictionaryContract]),
                  DictionaryContract        : set([Comment, Id, IsModify, LastUpdate, Name, ParentName, SystemName, Uri, Version]),
                }

    xmlNil = ( nsXml, 'nil' )

    def __init__(self):
        self.path = [None]
        self.mapUriToVersion = {}
        self.dictDef = {}


    def startElementNS(self, name, qname, attrs):
        assert name in self.structure[self.path[-1]]
        if name in self.structure[self.DictionaryContract]:
            if attrs.get(self.xmlNil) == 'true':
                self.dictDef.setdefault(name, None)
            else:
                self.dictDef.setdefault(name, '')
        self.path.append(name)


    def endElementNS(self, name, qname):
        self.path.pop()

        if name == self.DictionaryContract:
            uri = self.dictDef.get(self.Uri)
            if ':' not in uri:
                uri = 'urn:oid:' + uri
            version = self.dictDef.get(self.Version)
            version = version.replace('"','')
            self.mapUriToVersion[uri] = version
            self.dictDef.clear()


    def characters(self, content):
        el = self.path[-1]
#        if el in ( self.Uri, self.Version ):
        self.dictDef[el] = content

