# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QDate, QDateTime, pyqtSignature

from library.database        import CTableRecordCache
from library.DialogBase      import CDialogBase
from library.interchange     import setLineEditValue, getLineEditValue, setTextEditHTML, getTextEditHTML
from library.ItemsListDialog import CItemsListDialog, CItemEditorBaseDialog
from library.TableModel      import CTextCol, CDateTimeCol, CRefBookCol
from library.Utils           import toVariant, forceString, forceDateTime, forceRef

from Users.Ui_InformerEditor import Ui_InformerMessageEditorDialog
from Users.Ui_InformerViewer import Ui_InformerPage
from Users.Ui_InformerFilter import Ui_InformerFilterDialog


class CInformerList(CItemsListDialog):
    def __init__(self, parent, forSelect=False):
        CItemsListDialog.__init__(self, parent, [
            CRefBookCol( u'Автор',        ['createPerson_id'], 'vrbPerson', 10),
            CDateTimeCol(u'Дата и время', ['createDatetime'], 20),
            CTextCol(    u'Тема',         ['subject'], 20),
            ], 'InformerMessage', ['createDatetime DESC'], filterClass=CInformerFilterClass)
        self.setWindowTitleEx(u'Список сообщений информатора')
        self.tblItems.addPopupDelRow()


    def getItemEditor(self):
        return CInformerEditor(self)


    def select(self, props):
        table = self.model.table()
        cond = []
        if props.get('chkAuthor', False):
            cond.append(table['createPerson_id'].eq(props.get('author', None)))
        if props.get('chkDate', False):
            cond.append(table['createDatetime'].dateGe(props.get('begDate', QDate())))
            cond.append(table['createDatetime'].dateLe(props.get('endDate', QDate())))
        if props.get('chkSubject', False):
            cond.append(table['subject'].like('%%%s%%'%props.get('subject', '')))
        if props.get('chkRevisionInfo', False):
            cond.append(table['isRevisionInfo'].eq(1 if props.get('revisionInfo', 0)==0 else 0))

        return QtGui.qApp.db.getIdList(table.name(), self.idFieldName, cond, self.order)


class CInformerFilterClass(CDialogBase, Ui_InformerFilterDialog):
    def __init__(self, parent = None):
        CDialogBase.__init__(self, parent)
        self.setupUi(self)
        self.cmbAuthor.setSpecialityPresent(False)


    def setProps(self, props):
        self.chkRevisionInfo.setChecked(props.get('chkRevisionInfo', False))
        self.cmbRevisionInfo.setCurrentIndex(props.get('revisionInfo', 0))
        self.chkSubject.setChecked(props.get('chkSubject', False))
        self.edtSubject.setText(props.get('subject', ''))
        self.chkAuthor.setChecked(props.get('chkAuthor', False))
        self.cmbAuthor.setValue(props.get('author', None))
        self.chkDate.setChecked(props.get('chkDate', False))
        self.edtBegDate.setDate(props.get('begDate', QDate()))
        self.edtEndDate.setDate(props.get('endDate', QDate()))


    def props(self):
        result = {}
        result['chkAuthor'] = self.chkAuthor.isChecked()
        result['chkRevisionInfo'] = self.chkRevisionInfo.isChecked()
        result['revisionInfo'] = self.cmbRevisionInfo.currentIndex()
        result['chkSubject'] = self.chkSubject.isChecked()
        result['subject'] = self.edtSubject.text()
        result['author'] = self.cmbAuthor.value()
        result['chkDate'] = self.chkDate.isChecked()
        result['begDate'] = self.edtBegDate.date()
        result['endDate'] = self.edtEndDate.date()
        return result


#
# ##########################################################################
#

class CInformerEditor(CItemEditorBaseDialog, Ui_InformerMessageEditorDialog):
    def __init__(self,  parent):
        CItemEditorBaseDialog.__init__(self, parent, 'InformerMessage')
        self.setupUi(self)
        self.setWindowTitleEx(u'Сообщение информатора')
        self.setupDirtyCather()


    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        setLineEditValue( self.edtSubject, record, 'subject')
        setTextEditHTML(  self.edtText,    record, 'text')
        self.setIsDirty(False)


    def getRecord(self):
        record = CItemEditorBaseDialog.getRecord(self)
        getLineEditValue( self.edtSubject, record, 'subject')
        getTextEditHTML(  self.edtText,    record, 'text')
        return record


    def checkDataEntered(self):
        return True

#
# ##########################################################################
#

def showInformer(widget, quiet):
    db = QtGui.qApp.db
    personId = QtGui.qApp.userId
    tablePersonOrder = db.table('Person_Order')
    personOrderRecord = db.getRecordEx(tablePersonOrder,
                                       'max(date)',
                                       [ tablePersonOrder['master_id'].eq(personId),
                                         tablePersonOrder['type'].eq(0),
                                       ],
                                      )
    if personOrderRecord and not personOrderRecord.isNull(0):
        minDateTime = forceDateTime(personOrderRecord.value(0))
    else:
        minDateTime = None

    if not minDateTime:
        # fallback: если приёма на работу нет, то берём дату создания записи
        minDateTime = forceDateTime(db.translate('Person', 'id', personId, 'createDatetime'))

    table = db.table('InformerMessage')
    tableReadMark = db.table('InformerMessage_readMark')
    cond = [ tableReadMark['id'].isNull(),
             table['createDatetime'].ge(minDateTime)
           ]
    messageIdList = db.getIdList( table.leftJoin(tableReadMark,
                                                 [tableReadMark['master_id'].eq(table['id']), tableReadMark['person_id'].eq(QtGui.qApp.userId)]),
                                  table['id'].name(),
                                  cond,
                                  table['createDatetime'].name()
                                )
    if messageIdList:
        informer = CInformer(widget)
        informer.setIdList(messageIdList)
        informer.exec_()
    elif not quiet:
        QtGui.QMessageBox.information(widget,
                    u'Информатор',
                    u'Нет непрочитанных сообщений',
                    QtGui.QMessageBox.Close)


class CInformer(CDialogBase, Ui_InformerPage):
    def __init__(self, parent=None):
        CDialogBase.__init__(self, parent)
        self.addObject('btnPrev', QtGui.QPushButton(u'Предыдущее', self))
        self.addObject('btnNext',  QtGui.QPushButton(u'Следующее',  self))
        self.setupUi(self)
        self.buttonBox.addButton(self.btnPrev, QtGui.QDialogButtonBox.ActionRole)
        self.buttonBox.addButton(self.btnNext, QtGui.QDialogButtonBox.ActionRole)
        db = QtGui.qApp.db
        self.messageCache = CTableRecordCache(db, 'InformerMessage')
        self.personCache = CTableRecordCache(db, 'vrbPerson')
        self.idList = []
        self.markList = []
        self.currentIndex = None
        self.setWindowTitleEx(u'Сообщение')


    def setIdList(self, idList):
        self.idList = idList
        self.markList = [None]*len(self.idList)
        self.setCurrentIndex(0)


    def setCurrentIndex(self, index):
        notFirst = index>0
        notLast  = index<len(self.idList)-1
        self.currentIndex = index
        self.btnPrev.setEnabled(notFirst)
        self.btnNext.setEnabled(notLast)
        message = self.messageCache.get(self.idList[self.currentIndex])
        if message:
            createPersonId = forceRef(message.value('createPerson_id'))
            person = self.personCache.get(createPersonId)
            if person:
                personName = forceString(person.value('name'))
            else:
                personName = ''
            createDateTime = message.value('createDatetime').toDateTime()
            subject        = forceString(message.value('subject'))
            text           = forceString(message.value('text'))
        else:
            personName     = ''
            createDateTime = QDateTime()
            subject        = ''
            text           = ''
        self.lblCreatePersonValue.setText(personName)
        self.lblCreateDatetimeValue.setText(createDateTime.toString(Qt.LocaleDate))
        self.lblSubjectValue.setText(subject)
        self.edtText.setHtml(text)
        self.chkMarkViewed.setChecked(self.markList[self.currentIndex]==False)
        if notLast:
            self.btnNext.setFocus(Qt.OtherFocusReason)
        else:
            self.buttonBox.button(QtGui.QDialogButtonBox.Close).setFocus(Qt.OtherFocusReason)


    def askSaveDiscardContinueEdit(self):
        for messageId, mark in zip(self.idList, self.markList):
            if mark == True: # т.к. бывает True/False/None
                self.markViewed(messageId)
            elif mark == False: # т.к. бывает True/False/None
                self.unmarkViewed(messageId)
        return self.cdDiscard


    def markViewed(self, messageId):
        db = QtGui.qApp.db
        table = db.table('InformerMessage_readMark')
        if not db.getRecordEx(table, '*', [table['master_id'].eq(messageId), table['person_id'].eq(QtGui.qApp.userId)]):
            record = table.newRecord()
            record.setValue('master_id', toVariant(messageId))
            record.setValue('person_id', toVariant(QtGui.qApp.userId))
            db.insertRecord(table, record)


    def unmarkViewed(self, messageId):
        db = QtGui.qApp.db
        table = db.table('InformerMessage_readMark')
        db.deleteRecord(table, [table['master_id'].eq(messageId), table['person_id'].eq(QtGui.qApp.userId)])


    @pyqtSignature('bool')
    def on_chkMarkViewed_toggled(self, checked):
        self.markList[self.currentIndex] = checked


    @pyqtSignature('')
    def on_btnPrev_clicked(self):
        if self.currentIndex>0:
            self.markList[self.currentIndex] = self.chkMarkViewed.isChecked()
            self.setCurrentIndex(self.currentIndex-1)


    @pyqtSignature('')
    def on_btnNext_clicked(self):
        if self.currentIndex<len(self.idList)-1:
            self.markList[self.currentIndex] = self.chkMarkViewed.isChecked()
            self.setCurrentIndex(self.currentIndex+1)


    @pyqtSignature('')
    def on_buttonBox_rejected(self):
        self.markList[self.currentIndex] = self.chkMarkViewed.isChecked()
        self.close()
