# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""Реализация импорта населения"""

import re

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, SIGNAL, QFile, pyqtSignature, QString, QDate

from appendix.regional.r60.logicalControl.Ui_Control import Ui_ProgressDialog
from appendix.regional.r60.logicalControl.XmlReader import CXmlStreamReader

from library.DbEntityCache import CDbEntityCache
from library.Utils import (forceString, forceDate, forceRef, toVariant,
                           forceInt, nameCase)

from Exchange.Export import CExportHelperMixin
from Exchange.Utils import tbl

def setPrbCount(progressBar, cnt):
    u'Выставляет значение полосы прогресса'
    if cnt:
        progressBar.setValue(0)
        progressBar.setMaximum(cnt)
        progressBar.setEnabled(True)
        progressBar.setTextVisible(True)
    else:
        progressBar.setValue(0)
        progressBar.setMaximum(1)
        progressBar.setEnabled(False)
        progressBar.setTextVisible(False)


class CProgressDialog(QtGui.QDialog, Ui_ProgressDialog, CExportHelperMixin):
    u'Реализация импорта населения'

    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        CExportHelperMixin.__init__(self)
        self._parent = parent
        self.setupUi(self)

        self.start()
        self.connect(self, SIGNAL('startWork()'), self.startWork,
                     Qt.QueuedConnection)
        self.nProcessed = 0

        setPrbCount(self.prbControl, 0)
        self.prbVisitCount.setVisible(False)
        self.prbMedicalAidType.setVisible(False)

        self._db = QtGui.qApp.db
        self._tblClient = tbl('Client')
        self._tblClientPolicy = tbl('ClientPolicy')
        self._tblOrg = tbl('Organisation')

        self._orgInfisToIdCache = {}
        self._policyKindToId = {}
        self._activeOrgCache = {}

        self._reCredentials = re.compile(u'ФИО: ([\\S -]+) (\\S+) (\\S+) '
                                         u'([0-9\\.]+)')
        self._reResultComment = re.compile(
            u'(^Идентифицирован в РС ЕРЗ$)|'
            u'(^Страховая принадлежность не определена Причина аннулирования: '
            u'[1-5]$)')

        self._reInsurerCredentials = re.compile(
            u'^Идентифицирован в ЦСЕРЗ Код ОГРН СМО: (?P<ogrn>\\d+) '
            u'ОКАТО территории: (?P<okato>\\d+)$')


    def start(self):
        self.processDone = False
        self.aborted = False
        self.btnCancel.setText(u'Отмена')


    def stop(self):
        self.processDone = True
        self.btnCancel.setText(u'Закрыть')


    def done(self, result):
        self.stop()
        QtGui.QDialog.done(self, result)


    def exec_(self):
        self.emit(SIGNAL('startWork()'))

        return QtGui.QDialog.exec_(self)


    def setCheckMode(self, mode):
        self._mode = mode


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        if self.processDone:
            self.accept()
        else:
            self.aborted = True


    def startWork(self):
        self.start()
        QtGui.qApp.call(self, self.work)
        self.stop()


    def work(self):
        u'Запускает рабочую функцию диалога'
        self.processXml(self.prbControl)


    def log(self, msg):
        u'Пишет сообщение в журнал'
        self.logBrowser.append(msg)
        self.logBrowser.update()

# ******************************************************************************

    def processXml(self, progressBar):
        u"""Разбирает указанный XML файл, отправляет данные в БД"""
        fileName = forceString(self._parent.edtFileName.text())

        if not fileName:
            return False

        inFile = QFile(fileName)

        if not inFile.open(QFile.ReadOnly | QFile.Text):
            QtGui.QMessageBox.warning(
                self, u'Импорт населения',
                QString(u'Не могу открыть файл для чтения %1:\n%2.')
                .arg(fileName)
                .arg(inFile.errorString()))
            return False

        progressBar.setFormat(u'%p%')
        setPrbCount(progressBar, max(inFile.size(), 1))
        reader = CXmlStreamReader(self, self.logBrowser, True)
        reader.setDevice(inFile)
        params = {}

        if reader.readHeader():
            for name, row in reader.readData():
                progressBar.setValue(inFile.pos())
                QtGui.qApp.processEvents()

                if row:
                    self.processXmlRow(row, name, params)

                if self.aborted:
                    break

        self.log(u'Импорт элементов: %d обработано' % self.nProcessed)

        if not (reader.hasError() or self.aborted):
            self.log(u'Готово')
            return True
        else:
            self.log(u'Прервано')

            if self.aborted:
                self.log(u'! Прервано пользователем.')
            else:
                self.log(u'! Ошибка: файл %s, %s' % (fileName,
                                                     reader.errorString()))

            return False


    def processXmlRow(self, row, name, params):
        self.nProcessed += 1

        if name == 'ZAP':
            eventList = row.get('SLUCH', [])

            if not isinstance(eventList, list):
                eventList = [eventList]

            for event in eventList:
                resultComment = event.get('RESULT_COM')

                if not (resultComment is None or
                        self._reResultComment.search(resultComment)):
                    break
            else:
                return

            client = row.get('PACIENT', {})
            clientId = forceInt(client.get('ID_PAC'))
            eventId = forceInt(event.get('IDCASE'))

            if not (clientId and eventId):
                return

            isUpdated, policyRecord = self.checkPolicy(
                clientId, client, eventId)

            if isUpdated:
                self.log(u'Обновлены данные полиса пациента '
                         u'ID_PAC={0}'.format(clientId))

            if self.checkCredentials(clientId, resultComment):
                self.log(u'Обновлены персональные данные пациента '
                         u'ID_PAC={0}'.format(clientId))

            if policyRecord:
                insurerId = self.checkInsurer(resultComment, clientId)

                if (insurerId and insurerId != forceRef(
                        policyRecord.value('insurer_id'))):
                    policyRecord.setValue('insurer_id', toVariant(insurerId))
                    self._db.updateRecord(self._tblClientPolicy, policyRecord)
                    self.log(u'Изменены данные СМО пациента ID_PAC={0}'.format(
                        clientId))


    def checkPolicy(self, clientId, client, eventId):
        eventExecDate = forceDate(self._db.translate('Event', 'id', eventId,
                                                     'execDate'))
        policyRecord = self.getClientPolicy(clientId, eventExecDate)
        isDirty = False

        if policyRecord:
            kindId = forceRef(policyRecord.value('policyKind_id'))
            ourKindCode = CPolicyKindRegionalCodeCache.getCode(kindId)
            theirKindCode = client.get('VPOLIS')

            if theirKindCode and theirKindCode != ourKindCode:
                kindId = self.getPolicyKindIdByRegionalCode(theirKindCode)

                if kindId:
                    isDirty = True
                    policyRecord.setValue('policyKind_id', toVariant(kindId))
                else:
                    self.log(u'В базе не найден Вид полиса с кодом '
                             u'`{code}`'.format(code=theirKindCode))

            theirSerial = client.get('SPOLIS')

            if (theirSerial and
                    theirSerial != forceString(policyRecord.value('serial'))):
                isDirty = True
                policyRecord.setValue('serial', toVariant(theirSerial))

            theirNumber = client.get('NPOLIS')

            if (theirNumber and
                    theirNumber != forceString(policyRecord.value('number'))):
                isDirty = True
                policyRecord.setValue('number', toVariant(theirNumber))

            theirInsurer = client.get('SMO')
            ourInsurer = COrganisationInfisCodeCache.getCode(
                forceRef(policyRecord.value('insurer_id')))

            if theirInsurer and theirInsurer != ourInsurer:
                insurerId = self.getOrgIdByInfis(theirInsurer)

                if insurerId:
                    isDirty = True
                    policyRecord.setValue('insurer_id', toVariant(insurerId))
                else:
                    self.log(u'В базе не найдена СМО с кодом '
                             u'`{code}`'.format(code=theirInsurer))

            if isDirty:
                self._db.updateRecord(self._tblClientPolicy, policyRecord)

        return isDirty, policyRecord


    def checkCredentials(self, clientId, resultComment):
        result = self._reCredentials.search(resultComment)
        isDirty = False

        if result:
            record = self._db.getRecord(self._tblClient, 'id, firstName, '
                                        'lastName, patrName, birthDate',
                                        clientId)

            if record:
                for (ourField, theirFiled) in (('firstName', 2),
                                               ('lastName', 1),
                                               ('patrName', 3)):
                    ourVal = forceString(record.value(ourField))
                    theirVal = nameCase(forceString(result.group(theirFiled)))

                    if theirVal and ourVal != theirVal:
                        record.setValue(ourField, toVariant(theirVal))
                        isDirty = True

                ourBirthDate = forceDate(record.value('birthDate'))
                theirBirthDate = QDate.fromString(forceString(result.group(4)),
                                                  'dd.MM.yyyy.')

                if theirBirthDate.isValid() and ourBirthDate != theirBirthDate:
                    record.setValue('birthDate', toVariant(theirBirthDate))
                    isDirty = True

                if isDirty:
                    self._db.updateRecord(self._tblClient, record)

            else:
                self.log(u'В базе не найден пациент с id='
                         u'`{clientId}`'.format(clientId=clientId))

        return isDirty


    def checkInsurer(self, resultComment, clientId):
        u'Получает информацию о СК из строки и ищет активную СК по реквизитам'
        result = self._reInsurerCredentials.search(resultComment)

        if result:
            ogrn = result.group('ogrn')
            okato = result.group('okato')
            result = self.getActiveOrgByOgrnAndOkato(ogrn, okato)

            if not result:
                self.log(u'В базе не найдена СМО с ОГРН {ogrn} ОКАТО {okato}.'
                         u' ID_PAC={clientId}'.format(
                             ogrn=ogrn, okato=okato, clientId=clientId))

        return result


    def getOrgIdByInfis(self, infis):
        return self.getTableFieldById(infis, 'Organisation', 'id',
                                      self._orgInfisToIdCache, 'infisCode')


    def getPolicyKindIdByRegionalCode(self, code):
        return self.getTableFieldById(code, 'rbPolicyKind', 'id',
                                      self._policyKindToId, 'regionalCode')


    def getClientPolicy(self, clientId, date):
        db = self._db
        stmt = ('SELECT * FROM ClientPolicy'
                ' WHERE ClientPolicy.id = (select getClientPolicyIdForDate'
                '(%d,1,%s))' % (clientId, db.formatDate(date)))
        query = db.query(stmt)
        return query.record() if query.first() else None


    def getActiveOrgByOgrnAndOkato(self, ogrn, okato):
        u'Ищет активную организацию по ОГРН и ОКАТО'
        key = (ogrn, okato)
        result = self._activeOrgCache.get(key, -1)

        if result == -1:
            cond = [self._tblOrg['isActive'].eq(1),
                    self._tblOrg['OGRN'].eq(ogrn),
                    self._tblOrg['OKATO'].eq(okato)]
            record = self._db.getRecordEx(self._tblOrg, 'id', cond)
            result = forceRef(record.value(0)) if record else None
            self._activeOrgCache[key] = result

        return result


class CPolicyKindRegionalCodeCache(CDbEntityCache):
    mapIdToCode = {}

    @classmethod
    def purge(cls):
        cls.mapIdToCode.clear()


    @classmethod
    def getCode(cls, _id):
        result = cls.mapIdToCode.get(_id, False)
        if result == False:
            cls.connect()
            result = forceString(QtGui.qApp.db.translate('rbPolicyKind', 'id',
                                                         _id, 'regionalCode'))
            cls.mapIdToCode[_id] = result
        return result


class COrganisationInfisCodeCache(CDbEntityCache):
    mapIdToCode = {}

    @classmethod
    def purge(cls):
        cls.mapIdToCode.clear()


    @classmethod
    def getCode(cls, _id):
        result = cls.mapIdToCode.get(_id, False)
        if result == False:
            cls.connect()
            result = forceString(QtGui.qApp.db.translate('Organisation', 'id',
                                                         _id, 'infisCode'))
            cls.mapIdToCode[_id] = result
        return result
