# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""Реализация ФЛК"""

import re

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QDate

from appendix.regional.r60.logicalControl.Control import (
    CControlDialog as CR60ControlDialog)
from library.Calendar import CCalendarInfo
from library.Utils import forceString, forceInt, lastMonthDay, forceRef

from Exchange.Utils import tbl

class CControlDialog(CR60ControlDialog):
    u'Реализация ФЛК'
    requiredTags = {
        'ZGLV': ('VERSION', 'DATA', 'FILENAME', 'SD_Z'),
        'SCHET': ('CODE', 'CODE_MO', 'YEAR', 'MONTH', 'NSCHET', 'DSCHET',
                  'SUMMAV'),
        'ZAP': ('N_ZAP', 'PR_NOV', 'PACIENT', 'SLUCH'),
        'PACIENT': ('ID_PAC', 'VPOLIS', 'NPOLIS', 'NOVOR'),
        'SLUCH': ('IDCASE', 'USL_OK', 'VIDPOM', 'FOR_POM', 'LPU', 'PROFIL',
                  'DET', 'NHISTORY', 'DATE_1', 'DATE_2', 'DS1', 'RSLT', 'ISHOD',
                  'PRVS', 'IDDOKT', 'IDSP', 'SUMV'),
        'USL': ('IDSERV', 'LPU', 'PROFIL', 'DET', 'DATE_IN', 'DATE_OUT',
                'DS', 'CODE_USL', 'KOL_USL', 'SUMV_USL', 'PRVS', 'CODE_MD')
    }

    tagsFormat = {
        'ZGLV': {'VERSION': ('T', 5, 0),
                 'DATA': ('D', 0, 0),
                 'FILENAME': ('T', 26, 0),
                 'SD_Z': ('N', 9, 0), },
        'SCHET': {'CODE': ('N', 8, 0),
                  'CODE_MO': ('T', 6, 0),
                  'YEAR': ('N', 4, 0),
                  'MONTH': ('N', 2, 0),
                  'NSCHET': ('T', 15, 0),
                  'DSCHET': ('D', 0, 0),
                  'PLAT': ('T', 5, 0),
                  'SUMMAV': ('N', 15, 2),
                  'COMMENTS': ('T', 250, 0),
                  'SUMMAP': ('N', 15, 2),
                  'SANK_MEK': ('N', 15, 2),
                  'SANK_MEE': ('N', 15, 2),
                  'SANK_EKM': ('N', 15, 2), },
        'ZAP' : {'N_ZAP': ('N', 8, 0),
                 'PR_NOV': ('N', 1, 0), },
        'PACIENT': {'ID_PAC': ('T', 36, 0),
                    'VPOLIS': ('N', 1, 0),
                    'SPOLIS': ('T', 10, 0),
                    'NPOLIS': ('T', 20, 0),
                    'ST_OKATO': ('T', 5, 0),
                    'SMO': ('T', 5, 0),
                    'SMO_OGRN': ('T', 15, 0),
                    'SMO_OK': ('T', 5, 0),
                    'SMO_NAM': ('T', 100, 0),
                    'INV': ('N', 1, 0),
                    'NOVOR': ('T', 9, 0),
                    'VNOV_D': ('N', 4, 0), },
        'SLUCH': {'IDCASE': ('N', 11, 0),
                  'USL_OK': ('N', 2, 0),
                  'VIDPOM': ('N', 4, 0),
                  'FOR_POM': ('N', 1, 0),
                  'NPR_MO': ('N', 6, 0),
                  'NPR_DATE': ('D', 0, 0),
                  'EXTR': ('N', 2, 0),
                  'LPU': ('T', 6, 0),
                  'LPU_1': ('T', 8, 0),
                  'PODR': ('N', 8, 0),
                  'PROFIL': ('N', 3, 0),
                  'DET': ('N', 1, 0),
                  'NHISTORY': ('T', 50, 0),
                  'P_PER': ('N', 1, 0),
                  'DATE_1': ('D', 0, 0),
                  'DATE_2': ('D', 0, 0),
                  'DS0': ('T', 10, 0),
                  'DS1': ('T', 10, 0),
                  'DS2': ('T', 10, 0),
                  'DS3': ('T', 10, 0),
                  'VNOV_M': ('N', 4, 0),
                  'CODE_DKK': ('T', 10, 0),
                  'CODE_MES1': ('T', 20, 0),
                  'CODE_MES2': ('T', 20, 0),
                  'RSLT': ('N', 3, 0),
                  'ISHOD': ('N', 3, 0),
                  'PRVS': ('N', 4, 0),
                  'VERS_SPEC': ('T', 4, 0),
                  'IDDOKT': ('T', 25, 0),
                  'OS_SLUSH': ('N', 1, 0),
                  'IDSP': ('N', 2, 0),
                  'ED_COL': ('N', 5, 2),
                  'TARIF': ('N', 15, 2),
                  'SUMV': ('N', 15, 2),
                  'OPLATA': ('N', 1, 0),
                  'SUMP': ('N', 15, 2),
                  'SANK_IT': ('N', 15, 2), },
        'SANK': {'S_CODE': ('T', 36, 0),
                 'S_SUM': ('N', 15, 2),
                 'S_TIP': ('N', 1, 0),
                 'S_OSN': ('N', 3, 0),
                 'S_COM': ('T', 250, 0),
                 'S_IST': ('N', 1, 0), },
        'USL': {'IDSERV': ('T', 36, 0),
                'LPU': ('T', 6, 0),
                'LPU_1': ('T', 8, 0),
                'PODR': ('N', 8, 0),
                'PROFIL': ('N', 3, 0),
                'VID_VME': ('T', 15, 0),
                'DET': ('N', 1, 0),
                'DATE_IN': ('D', 0, 0),
                'DATE_OUT': ('D', 0, 0),
                'DS': ('T', 10, 0),
                'CODE_USL': ('T', 20, 0),
                'KOL_USL': ('N', 6, 2),
                'TARIF': ('N', 15, 2),
                'SUMV_USL': ('N', 15, 2),
                'PRVS': ('N', 4, 0),
                'CODE_MD': ('T', 25, 0),
                'NPL': ('N', 1, 0),
                'COMENTU': ('T', 250, 0), },
    }

    reNumber = r'^\d{{0,{len}}}$'
    reFloat = r'^\d{{0,{len}}}(\.\d{{0,{dotlen}}})?$'
    reText = r'^.{{0,{len}}}$'
    reDate = r'^\d{4}-\d\d-\d\d$'

    def __init__(self, parent):
        CR60ControlDialog.__init__(self, parent)

        self.checkHolidays = False
        self.checkAmbulanceHospitalIntersect = False
        self.checkHospitalIntersect = False
        self.checkInsurer = False
        self.checkPeriod = False
        self.checkFormat = False
        self.checkVisit = False
        self.checkDiffVisitType = False
        self.checkTreatment = False
        self.checkMes = False
        self.calendarInfo = CCalendarInfo(self)

        self._db = QtGui.qApp.db

        if self._db:
            self.calendarInfo.load()
            self._tblVisit = tbl('Visit')
            self._tblRbVisitType = tbl('rbVisitType')
            self._tblEvent = tbl('Event')
            self._visitTypeVisit = self._db.translate(self._tblRbVisitType,
                                                      'code', u'ПОС', 'id')
            self._visitTypeTreatment = self._db.translate(self._tblRbVisitType,
                                                          'code', u'ОБР', 'id')
        else:
            self.log(u'Информация о праздничных днях не загружена: '
                     u'отсутствует подключение к БД.')


    def processXmlRow(self, row, name, params):
        CR60ControlDialog.processXmlRow(self, row, name, params)

        if self._mode == self.modeNormal:
            tasks = (
                (self.checkHolidays, self.processHolidays),
                (self.checkAmbulanceHospitalIntersect,
                 self.processAmbulanceHospitalIntersect),
                (self.checkHospitalIntersect,
                 self.processHospitalIntersect),
                (self.checkInsurer, self.processInsurer),
                (self.checkPeriod, self.processPeriod),
                (self.checkFormat, self.processFormat),
                (self.checkVisit, self.processVisit),
                (self.checkTreatment, self.processTreatment),
                (self.checkDiffVisitType, self.processDiffVisitType),
                (self.checkMes, self.processMes),
                (self.checkVisitPeriod, self.processVisitPeriod)
            )

            for (cond, worker) in tasks:
                if cond:
                    for err in worker(row, name, params):
                        self.log(err)


    def processHolidays(self, row, name, _):
        u'Проверка услуг в выходные и праздничные дни'
        result = []

        if name == 'ZAP':
            eventList = row.get('SLUCH', {})

            if not isinstance(eventList, list):
                eventList = [eventList]

            for event in eventList:
                eventCode = forceString(event.get('USL_OK'))

                if eventCode != '3':
                    continue

                service = event.get('USL', {})

                if not isinstance(service, list):
                    service = [service]

                for srv in service:
                    for date in (srv.get('DATE_IN', ''),
                                 srv.get('DATE_OUT', '')):
                        if (self.calendarInfo.getDayOfWeek(
                                QDate.fromString(date, Qt.ISODate)) in (6, 7)):
                            result.append(u'IDCASE {idcase} IDSERV={idserv} '
                                          u'даты выполнения соответствует '
                                          u'выходному дню'.format(
                                              idcase=event.get('IDCASE', '?'),
                                              idserv=srv.get('IDSERV', '?')))

        return result


    def processAmbulanceHospitalIntersect(self, row, name, _):
        u"""если в файле есть запись (блок ZAP), у которой имеется два и более
            блоков SLUCH с пересекающимися периодами(<DATE_1> и <DATE_2>), при
            этом один (или более) из них имеет <USL_OK>3</USL_OK>, а другой
            (или более) <USL_OK>1</USL_OK> или <USL_OK>2</USL_OK>"""
        result = []

        if name == 'ZAP':
            eventList = row.get('SLUCH', {})

            if isinstance(eventList, list):
                hospSrvList = []
                spareHospSrvList = []
                ambEvents = []
                flag = False

                for event in eventList:
                    eventCode = event.get('USL_OK')

                    if eventCode == '3':
                        srvList = event.get('USL', [])

                        if not isinstance(srvList, list):
                            srvList = [srvList]

                        for srv in srvList:
                            if srv.get('TARIF', '0') == '0':
                                hospSrvList.append(srv)
                            else:
                                spareHospSrvList.append(srv)
                    elif eventCode == '1':
                        ambEvents.append(event)

                if hospSrvList == []:
                    hospSrvList = spareHospSrvList

                if hospSrvList != [] and ambEvents != []:
                    for hospSrv in hospSrvList:
                        begDate = QDate.fromString(hospSrv.get('DATE_IN', ''),
                                                   Qt.ISODate)
                        endDate = QDate.fromString(hospSrv.get('DATE_OUT', ''),
                                                   Qt.ISODate)

                        for ambEvent in ambEvents:
                            ambBegDate = QDate.fromString(ambEvent.get(
                                'DATE_1', ''), Qt.ISODate).addDays(1)
                            ambEndDate = QDate.fromString(ambEvent.get(
                                'DATE_2', ''), Qt.ISODate).addDays(-1)

                            if begDate <= ambEndDate and endDate >= ambBegDate:
                                flag = True
                                break

                        if flag:
                            break

                if flag:
                    client = row.get('PACIENT', {})
                    result.append(u'ID_PAC={clientId} пересечение случаев '
                                  u'лечения в амбулатории и стационаре'.format(
                                      clientId=client.get('ID_PAC', '?')))

        return result


    def processHospitalIntersect(self, row, name, _):
        u"""Если в файле есть запись (блок ZAP), у которой имеются несколько
        блоков SLUCH, из которых один (или более) имеет <USL_OK>2</USL_OK>, в то
        время как один (или более) имеет <USL_OK>1</USL_OK>, необходимо
        произвести проверку на пересечение дат. В SLUCH, который имеет USL_OK=2,
        необходимо отобрать временной интервал [<DATE_1>;<DATE_2>]. И проверить
        его пересечение с временным интервалом [<DATE_1> +1день;<DATE_2>-1день],
        где <DATE_1> и <DATE_2> теги блока SLUCH, который имеет USL_OK = 1"""
        result = []

        if name == 'ZAP':
            eventList = row.get('SLUCH', {})

            if isinstance(eventList, list):
                hospEvents = []
                dayHospEvents = []
                flag = False

                for event in eventList:
                    eventCode = event.get('USL_OK')

                    if eventCode == '1':
                        dayHospEvents.append(event)
                    elif eventCode == '2':
                        hospEvents.append(event)

                if hospEvents != [] and dayHospEvents != []:
                    for hospEvent in hospEvents:
                        begDate = QDate.fromString(hospEvent.get('DATE_1', ''),
                                                   Qt.ISODate)
                        endDate = QDate.fromString(hospEvent.get('DATE_2', ''),
                                                   Qt.ISODate)

                        for event in dayHospEvents:
                            dBegDate = QDate.fromString(event.get(
                                'DATE_1', ''), Qt.ISODate).addDays(1)
                            dEndDate = QDate.fromString(event.get(
                                'DATE_2', ''), Qt.ISODate).addDays(-1)

                            if begDate <= dEndDate and endDate >= dBegDate:
                                flag = True
                                break

                        if flag:
                            break

                if flag:
                    client = row.get('PACIENT', {})
                    result.append(u'ID_PAC={clientId} пересечение случаев '
                                  u'лечения в амбулатории и стационаре'.format(
                                      clientId=client.get('ID_PAC', '?')))


        return result

    def processInsurer(self, row, name, params):
        u'Проверка соотвествия страховой из полиса и плательщика счета'
        result = []

        if name == 'SCHET':
            params['PLAT'] = row.get('PLAT')

        if name == 'ZAP':
            client = row.get('PACIENT', {})

            if client.get('SMO') != params['PLAT']:
                result.append(u'ID_PAC={clientId} страховая компания пациента '
                              u'не соответствует плательщику'.format(
                                  clientId=client.get('ID_PAC', '?')))


        return result


    def processPeriod(self, row, name, params):
        u"""В тегах <MONTH> и <YEAR> блока <SCHET> указаны месяц и год
            расчетного периода. Если дата окончания случая ( тег <DATE_2> блока
            <SLUCH> не соответствует по месяцу и году расчетному периоду, то
            необходимо выдавать ошибку."""
        result = []

        if name == 'SCHET':
            params['periodBegDate'] = QDate(forceInt(row.get('YEAR')),
                                            forceInt(row.get('MONTH')), 1)
            params['periodEndDate'] = lastMonthDay(params['periodBegDate'])

        if name == 'ZAP':
            eventList = row.get('SLUCH', {})
            if not isinstance(eventList, list):
                eventList = [eventList]

            for event in eventList:
                date = QDate.fromString(event.get('DATE_2', ''), Qt.ISODate)

                if date.isValid() and not (
                        date >= params.get('periodBegDate', QDate()) and
                        date <= params.get('periodEndDate', QDate())):
                    result.append(u'Выход за границы отчетного периода:'
                                  u' IDCASE={idCase}'.format(
                                      idCase=event.get('IDCASE', '?')))

        return result


    def processFormat(self, row, name, _):
        u"""Содержимое тегов должно проверяться на соотетствие формату,
        описанному в файле "Д1 Описание формата. V2018" в поле Формат.
            Обозначения форматов:
            Т - текст, в скобочках указывается максимальное количество симолов
            Д - дата в формате ГГГГ-ММ-ДД
            N - число, в скобочках указывается максимально число знаков.
                При этом N(15.2) означает формат максимум 15 знаков до
                запятой и 2 знака после запятой."""

        def _getRegExp(_type, _chars, _dotchars):
            u'Возвращает регулярное выражение по типу данных'
            regExp = None

            if _type == 'N':
                if _dotchars:
                    regExp = self.reFloat.format(len=_chars, dotlen=_dotchars)
                else:
                    regExp = self.reNumber.format(len=_chars)
            elif _type == 'T':
                regExp = self.reText.format(len=_chars)
            elif _type == 'D':
                regExp = self.reDate

            return re.compile(regExp)

        def _checkFormat(name, key, val, recordNum, eventId):
            u'Проверяет правильность формата поля'
            err = None
            fmt = self.tagsFormat.get(name, {}).get(key)

            if fmt:
                reFmt = _getRegExp(*fmt)

                if reFmt and not reFmt.match(val):
                    err = (u'Нарушение формата данных в поле {tag}: '
                           u'N_ZAP={recordNum} IDCASE={eventId}'.format(
                               tag=key, recordNum=recordNum, eventId=eventId))
                elif not reFmt:
                    err = (u'Нет правила для {name}.{key}={val}'.format(
                        name=name, key=key, val=val))

            return err

        def _checkGroup(name, row, recordNum, eventId):
            u'Проверяет правильность формата полей в группе'
            for key, val in row.iteritems():
                err = _checkFormat(name, key, val, recordNum, eventId)

                if err:
                    result.append(err)

        result = []

        if name in ('ZGLV', 'SCHET'):
            _checkGroup(name, row, '?', '?')
        elif name == 'ZAP':
            recordNum = row.get('N_ZAP', '?')
            _checkGroup(name, row, recordNum, '?')
            _checkGroup('PACIENT', row.get('PACIENT', {}), recordNum, '?')
            events = row.get('SLUCH', {})

            if events:
                if not isinstance(events, list):
                    events = [events]

                for event in events:
                    eventId = event.get('IDCASE', '?')
                    _checkGroup('SLUCH', event, recordNum, eventId)
                    services = event.get('USL')

                    if services:
                        if not isinstance(services, list):
                            services = [services]

                        for service in services:
                            _checkGroup('USL', service, recordNum, eventId)

        return result


    def processVisit(self, row, name, _):
        u""" Если в этом событии имеется больше одного визита с кодом ПОС,
             то необходимо выводить ошибку"""
        result = []

        if name == 'ZAP':
            eventList = row.get('SLUCH', {})

            if not isinstance(eventList, list):
                eventList = [eventList]

            for event in eventList:
                eventId = event.get('IDCASE')
                cond = [self._tblVisit['visitType_id'].eq(self._visitTypeVisit),
                        self._tblVisit['deleted'].eq(0),
                        self._tblVisit['event_id'].eq(eventId)]
                recordList = self._db.getRecordList(self._tblVisit, where=cond,
                                                    limit=2)

                if len(recordList) > 1:
                    result.append(u'Тарификация более одного визита с типом'
                                  u' `Посещение` в одном событии IDCASE='
                                  u'{eventId}'.format(eventId=eventId))

        return result


    def processTreatment(self, row, name, _):
        u"""Если в этом событии имеется только один визит с кодом ОБР,
            то необходимо выводить ошибку"""
        result = []

        if name == 'ZAP':
            eventList = row.get('SLUCH', {})

            if not isinstance(eventList, list):
                eventList = [eventList]

            for event in eventList:
                eventId = event.get('IDCASE')
                cond = [self._tblVisit['visitType_id'].eq(
                    self._visitTypeTreatment),
                        self._tblVisit['deleted'].eq(0),
                        self._tblVisit['event_id'].eq(eventId)]
                recordList = self._db.getRecordList(self._tblVisit, where=cond,
                                                    limit=2)

                if recordList and len(recordList) < 2:
                    result.append(u'Тарификация одного визита с типом '
                                  u'`Обращение` в одном событии IDCASE='
                                  u'{eventId}'.format(eventId=eventId))

        return result


    def processDiffVisitType(self, row, name, _):
        u"""Если в этом событии имеются визиты с разными типами,
            то необходимо выводить ошибку"""
        result = []

        if name == 'ZAP':
            eventList = row.get('SLUCH', {})

            if not isinstance(eventList, list):
                eventList = [eventList]

            for event in eventList:
                eventId = event.get('IDCASE')
                cond = [self._tblVisit['event_id'].eq(eventId),
                        self._tblVisit['deleted'].eq(0)]
                recordList = self._db.getRecordListGroupBy(self._tblVisit,
                                                           where=cond,
                                                           group='visitType_id',
                                                           limit=2)

                if len(recordList) > 1:
                    result.append(u'Разные типы визитов в одном событии IDCASE='
                                  u'{eventId}'.format(eventId=eventId))

        return result


    def processMes(self, row, name, _):
        u"""Если у этого Event.id указан МЭС (Event.MES_id), то смотрим, есть ли
            у этого МЭСа требование по услуге (Event.MES_id->mes.MES.id->
            mes.MES_service.master_id). Если требования по услугам есть,
            то смотрим код услуги (mes.MES_service.service_id->mrbService.code)
            Теперь смотрим, зарегистрирована ли в событии данная услуга. Если в
            событии нет Action, у ActionType которого
            ActionType.nomenclativeService_id->rbService.code=mrbService.code,
            то выдаем ошибку"""

        def getMesServiceCodeSet(eventId):
            u'Возвращает множество кодов услуг по МЭС'
            codeSet = set()

            stmt = """SELECT MS.code, Event.MES_id
                      FROM Event
                      LEFT JOIN mes.MES_service S ON S.master_id = Event.MES_id
                      LEFT JOIN mes.mrbService MS ON S.service_id = MS.id
                      WHERE Event.id = {eventId}""".format(eventId=eventId)
            query = self._db.query(stmt)

            while query.next():
                record = query.record()
                mesId = forceRef(record.value(1))

                if not mesId:
                    codeSet = None
                    break

                codeSet.add(forceString(record.value(0)))

            return codeSet

        def getNomenclativeServiceCodeSet(eventId):
            u'Возвращает множество кодов услуг по типам действий'
            codeSet = set()

            stmt = """SELECT rbService.code
                        FROM Action A
                        LEFT JOIN ActionType AT ON AT.id = A.actionType_id
                        LEFT JOIN rbService ON
                            rbService.id = AT.nomenclativeService_id
                        WHERE A.event_id = {eventId}""".format(eventId=eventId)
            query = self._db.query(stmt)

            while query.next():
                record = query.record()
                codeSet.add(forceString(record.value(0)))

            return codeSet


        result = []

        if name == 'ZAP':
            eventList = row.get('SLUCH', {})

            if not isinstance(eventList, list):
                eventList = [eventList]

            for event in eventList:
                eventId = event.get('IDCASE')
                mesServiceCodeSet = getMesServiceCodeSet(eventId)

                if mesServiceCodeSet is None:
                    continue

                nomenclativeServiceCodeSet = (
                    getNomenclativeServiceCodeSet(eventId))

                if not mesServiceCodeSet.issubset(nomenclativeServiceCodeSet):
                    result.append(u'Не выполнено требование МЭС по услуге '
                                  u'IDCASE={eventId}'.format(eventId=eventId))

        return result

    def processVisitPeriod(self, row, name, _):
        u"""Если в блоке <ZAP> имеется несколько тегов <SLUCH>, то смотрим их
            <IDCASE> и идем в базу s11. Смотрим на события с данными
            идентификаторами (Event.id=IDCASE). Далее смотрим на типы визитов в
            этих событиях. События, у которых типы визитов с кодом ПОС, мы
            рассматривать не будем. Проверяем только события, у которых есть
            визиты с типом Обращение (код типа визита ОБР). Берем дату окончания
            (Event.execDate) более раннего по времени обращения (обзначим ее за
            X) и дату начала (Event.setDate) более позднего по времени события
            (обозначим за Y). Если Y-X<=15 то выдаем ошибку."""

        def checkVisitPeriod(eventIdList):
            u'Возвращает количество дней между посещениями в событиях'
            stmt = u"""SELECT IF(A.id = B.id,
                16, datediff(B.setDate, A.execDate))
            FROM Event
            JOIN (SELECT Event.id, MIN(Event.execDate) AS execDate FROM Event
                LEFT JOIN Visit ON Visit.event_id = Event.id
                LEFT JOIN rbVisitType ON Visit.visitType_id = rbVisitType.id
                WHERE {eventIdList} AND rbVisitType.code = 'ОБР') AS A
            JOIN (SELECT Event.id, MAX(Event.setDate) AS setDate FROM Event
                LEFT JOIN Visit ON Visit.event_id = Event.id
                LEFT JOIN rbVisitType ON Visit.visitType_id = rbVisitType.id
            WHERE {eventIdList} AND rbVisitType.code = 'ОБР') AS B
            LIMIT 0,1
            """.format(eventIdList=self._tblEvent['id'].inlist(eventIdList))

            _result = False
            query = self._db.query(stmt)

            while query.next():
                record = query.record()
                val = forceInt(record.value(0))
                if val < 16:
                    _result = True

            return _result

        result = []

        if name == 'ZAP':
            eventList = row.get('SLUCH', {})

            if (isinstance(eventList, list) and
                    checkVisitPeriod(
                        [event.get('IDCASE') for event in eventList])):
                clientId = row.get('PACIENT', {}).get('ID_PAC', '-')
                result.append(u'ID_PAC={clientId} Период между обращениями '
                              u' менее 15 дней'.format(clientId=clientId))

        return result
