# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import QXmlStreamReader, pyqtSignature, SIGNAL, Qt, QString

from library.Utils import (forceRef, forceString, toVariant,
                    anyToUnicode)
from Exchange.Utils import tbl

from Ui_Progress import Ui_ProgressDialog

# ******************************************************************************

class CMyXmlStreamReader(QXmlStreamReader):
    insurerFields = ('tf_okato', 'smocod', 'nam_smop', 'nam_smok', 'inn',
        'Ogrn', 'KPP', 'fam_ruk', 'im_ruk', 'ot_ruk', 'phone', 'fax', 'www',
        'e_mail', 'okopf', 'org', 'd_edit')

    def __init__(self, parent, logBrowser, showLog=False):
        QXmlStreamReader.__init__(self)
        self._parent = parent
        self.db = QtGui.qApp.db
        self.tableOrgs = tbl('Organisation')
        self.mapOrgKeyToId = {}
        self.logBrowser = logBrowser
        self.showLog = showLog
        self.elementsList = []


    def raiseError(self, msg):
        QXmlStreamReader.raiseError(self, u'[%d:%d] %s' % (
            self.lineNumber(), self.columnNumber(), msg))


    def log(self, msg, forceLog=False):
        if self.showLog or forceLog:
            self.logBrowser.append(msg)
            self.logBrowser.update()


    def readHeader(self):
        u''' Разбирает заголовок'''

        xmlVersion = '1.0.1'

        try:
            while not self.atEnd():
                self.readNext()

                if self.isStartElement():
                    if self.name() == 'packet':
                        if self.attributes().value('version') == xmlVersion:
                            return True
                        else:
                            self.raiseError(u'Версия формата "%s" не '
                                u'поддерживается. Должна быть "%s"' % (
                               self.attributes().value('version').toString(),
                               xmlVersion))
                    else:
                        self.raiseError(u'Неверный формат экспорта данных.')

        except IOError, e:
            QtGui.qApp.logCurrentException()
            msg = ''
            if hasattr(e, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno,
                                               anyToUnicode(e.strerror))
            else:
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            QtGui.QMessageBox.critical(self._parent,
                u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
            self.log(u'! Ошибка ввода-вывода: %s' % msg, True)

        except Exception, e:
            QtGui.qApp.logCurrentException()
            QtGui.QMessageBox.critical(self._parent,
                u'Произошла ошибка', unicode(e), QtGui.QMessageBox.Close)
            self.log(u'! Ошибка: %s' % unicode(e), True)

        return False


    def readData(self):
        assert self.isStartElement() and self.name() == 'packet'

        # очищаем список событий перед заполнением

        while not self.atEnd():
            self.readNext()

            if self.isEndElement():
                break

            if self.isStartElement():
                if self.name() == 'insCompany':
                    yield self.readInsurer()
                else:
                    self.readUnknownElement()

            if self.hasError() or self._parent.aborted:
                break


    def readInsurer(self):
        assert self.isStartElement() and self.name() == 'insCompany'

        result = {}

        if self._parent.aborted:
            return None

        while not self.atEnd():
            QtGui.qApp.processEvents()
            self.readNext()

            if self.isEndElement():
                break

            if self.isStartElement():
                name = forceString(self.name())
                if name in self.insurerFields:
                    result[name] = forceString(self.readElementText())
                elif name in ('jurAddress', 'pstAddress'):
                    result[name] = self.readAddress()
                elif name in ('licenziy', 'insInclude', 'insAdvice'):
                    # игноруемые параметры,
                    # не сообщаем в журнал об ошибках
                    self.readUnknownElement(False)
                else:
                    self.readUnknownElement()

        return result


    def readAddress(self):
        assert self.isStartElement() and self.name() in (
            'jurAddress', 'pstAddress')

        result = {}

        if self._parent.aborted:
            return None

        while not self.atEnd():
            self.readNext()

            if self.isEndElement():
                break

            if self.isStartElement():
                name = forceString(self.name())
                if name in ('index_j', 'index_f', 'addr_j', 'addr_f'):
                    result[name] = forceString(self.readElementText())
                else:
                    self.readUnknownElement()

        return result


    def readUnknownElement(self, report=True):
        """ Читает неизвестный элемент, и сообщает об этом,
            если report ==True """

        assert self.isStartElement()

        if report:
            self.log(u'Неизвестный элемент: '+self.name().toString())

        while not self.atEnd():
            self.readNext()

            if self.isEndElement():
                break

            if self.isStartElement():
                self.readUnknownElement(report)

            if self.hasError() or self._parent.aborted:
                break

# ******************************************************************************

class CProgressDialog(QtGui.QDialog, Ui_ProgressDialog):
    __pyqtSignals__ = ('startWork()', )

    def __init__(self, parent, searchByInn):
        QtGui.QDialog.__init__(self, parent)
        self._parent = parent
        self.setupUi(self)

        self.nAdded = 0
        self.nCoincident = 0
        self.nProcessed = 0
        self.nUpdated = 0

        self.start()
        self.connect(self, SIGNAL('startWork()'), self.startWork,
            Qt.QueuedConnection)

        self.orgMap = {}
        self.showLog = True
        self.skipOrgs = False

        self.db = QtGui.qApp.db
        self.setOrgCount = lambda cnt: self._setCount(self.prbOrgs, cnt)
        self.setPlacesCount = lambda cnt: self._setCount(self.prbPlaces, cnt)

        if searchByInn:
            self.findOrg = self.findOrgByInn
            self.searchField = 'inn'
        else:
            self.findOrg = self.findOrgByMiacCode
            self.searchField = 'smocod'


    def log(self, msg, forceLog=False):
        if self.showLog or forceLog:
            self.logBrowser.append(msg)
            self.logBrowser.update()


    def _setCount(self, progressBar, cnt):
        if cnt:
            progressBar.setValue(0)
            progressBar.setMaximum(cnt)
            progressBar.setEnabled(True)
            progressBar.setTextVisible(True)
        else:
            progressBar.setValue(0)
            progressBar.setMaximum(1)
            progressBar.setEnabled(False)
            progressBar.setTextVisible(False)


    def start(self):
        self.processDone = False
        self.aborted = False
        self.btnCancel.setText(u'Отмена')


    def stop(self):
        self.processDone = True
        self.btnCancel.setText(u'Закрыть')


    def done(self, r):
        self.stop()
        QtGui.QDialog.done(self, r)


    def exec_(self):
        self.lblOrgs.setEnabled(True)
        self.prbOrgs.setEnabled(True)
        self.emit(SIGNAL('startWork()'))
        return QtGui.QDialog.exec_(self)


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        if self.processDone:
            self.accept()
        else:
            self.aborted = True


    def startWork(self):
        self.start()
        QtGui.qApp.call(self, self.work)
        self.stop()


    def work(self):
        fileName = forceString(self._parent.edtOrgFileName.text())

        if not fileName:
            return False

        self.setOrgCount(0)
        self.setPlacesCount(0)
        self.prbOrgs.setFormat(u'%v байт')
        inFile = QtCore.QFile(fileName)

        if not inFile.open(QtCore.QFile.ReadOnly | QtCore.QFile.Text):
            QtGui.QMessageBox.warning(self, u'Импорт страховых компаний',
                  QString(u'Не могу открыть файл для чтения %1:\n%2.')
                  .arg(fileName)
                  .arg(inFile.errorString()))
            return False

        self.setOrgCount(max(inFile.size(), 1))
        reader = CMyXmlStreamReader(self, self.logBrowser, True)
        reader.setDevice(inFile)

        if reader.readHeader():
            for org in reader.readData():
                self.prbOrgs.setValue(inFile.pos())
                self.processOrg(org)

                if self.aborted:
                    break

        self.log(u'импорт элементов: %d добавлено; %d обновлено;'
            u' %d обработано' % (
            self.nAdded, self.nUpdated, self.nProcessed))

        if not reader.hasError():
            self.log(u'Готово')
            return True
        else:
            self.log(u'Прервано')

            if self.aborted:
                self.log(u'! Прервано пользователем.')
            else:
                self.log(u'! Ошибка: файл %s, %s' % (fileName,
                    reader.errorString()))

            return False

# ******************************************************************************

    def processOrg(self, org):
        code = org.get(self.searchField)
        self.nProcessed += 1

        if code:
            _id = self.findOrg(code)

            shortName = org.get('nam_smok', '')
            fullName = org.get('name_smop', '')
            inn = org.get('inn', '')
            kpp = org.get('kpp', '')
            address = org.get('pstAddress', {}).get('addr_f', '')
            chief = u' '.join([org.get('fam_ruk', ''), org.get('im_ruk', ''),
                org.get('ot_ruk', '')])
            phone = org.get('phone', '')
            fax = org.get('fax', '')
            email = org.get('e_mail', '')
            infisCode = org.get('miacCode', '')
            accountant = ''
            masterId = None
            ogrn = org.get('Ogrn', '')
            okpo = ''
            okved = ''

            if _id:
                self.log(u'* Обновляем: %s (id=%d)' % (
                   org.get('nam_smok', '-'), _id))
                self.updateOrg(id, shortName, fullName, okpo, okved, ogrn,
                    address, chief, accountant, phone, fax, email, masterId,
                    infisCode)
                self.nUpdated += 1
            else:
                self.log(u'+ Добавляем: %s' % (
                   org.get('nam_smok', '-')))
                self.addOrg(shortName, fullName, inn, kpp, okpo, okved, ogrn,
                    address, chief, accountant, phone, fax, email, masterId,
                    infisCode)
                self.nAdded += 1

        else:
            self.log(u'Отсутствует поле `%s` для организации `%s`' %(
                self.searchField, org.get('nam_smok', '??')))

# ******************************************************************************

    def findOrgByInn(self, INN):
        table = self.db.table('Organisation')
        record = self.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
                                                         table['INN'].eq(INN)
                                                        ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# ******************************************************************************

    def findOrgByMiacCode(self, miacCode):
        table = self.db.table('Organisation')
        record = self.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
            table['miacCode'].eq(miacCode)], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# ******************************************************************************

    def addOrg(self, shortName, fullName, INN, KPP, OKPO, OKVED, OGRN,
                        address, chief, accountant, phone, fax, email, masterId,
                        infisCode):
        table = self.db.table('Organisation')
        record = table.newRecord()
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('INN', toVariant(INN))
        record.setValue('KPP', toVariant(KPP))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('accountant', toVariant(accountant))
        record.setValue('phone', toVariant(phone))
#        record.setValue('net_id', toVariant(self.matureNetId))

        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if masterId:
            record.setValue('head_id', toVariant(masterId))

        record.setValue('infisCode', toVariant(infisCode))
        return self.db.insertRecord(table, record)

# ******************************************************************************

    def updateOrg(self, orgId, shortName, fullName, OKPO, OKVED, OGRN,
            address, chief, accountant, phone, fax, email, masterId, infisCode):
        table = self.db.table('Organisation')
        record = self.db.getRecord(table, '*', orgId)
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('accountant', toVariant(accountant))
        record.setValue('phone', toVariant(phone))
#        record.setValue('net_id', toVariant(self.matureNetId))
        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if masterId:
            record.setValue('head_id', toVariant(masterId))

        record.setValue('infisCode', toVariant(infisCode))
        return self.db.updateRecord(table, record)
