# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import os.path
import shutil

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import Qt, QDate, pyqtSignature, SIGNAL

from library.dbfpy.dbf import Dbf

from library.Utils     import (forceString, forceInt, pyDate, forceDate,
        forceRef, forceStringEx, forceBool, forceDouble)
from zipfile import ZipFile, ZIP_DEFLATED

from Exchange.Export import CExportHelperMixin
from Ui_ExportEGISZ import Ui_ExportDialog


class CExportEGISZDialog(QtGui.QDialog, Ui_ExportDialog, CExportHelperMixin):
    __pyqtSignals__ = ('startWork()', )

    def __init__(self, parent, isHospital, begDate, endDate, destDir, packageNum):
        QtGui.QDialog.__init__(self, parent)
        CExportHelperMixin.__init__(self)
        self._parent = parent
        self.setupUi(self)

        self.setEventsCount(0)
        self.connect(self, SIGNAL('startWork()'), self.export, Qt.QueuedConnection)

        self.tmpDir = None
        self.zipFileName = None

        self.isHospital = isHospital
        self.begDate = begDate
        self.endDate = endDate
        self.destDir = destDir
        self.packageNum = packageNum
        self.exportedClientIdSet = set()


    def _setCount(self, progressBar, cnt):
        progressBar.reset()

        if cnt:
            progressBar.setValue(0)
            progressBar.setMaximum(cnt)
            progressBar.setEnabled(True)
            progressBar.setTextVisible(True)
        else:
            progressBar.setValue(0)
            progressBar.setMaximum(1)
            progressBar.setEnabled(False)
            progressBar.setTextVisible(False)


    def setEventsCount(self, cnt):
        self._setCount(self.prbEvents, cnt)


    def exec_(self):
        self.emit(SIGNAL('startWork()'))
        result = QtGui.QDialog.exec_(self)
        self.cleanup()
        return result


    def start(self):
        self.processDone = False
        self.aborted = False
        self.btnCancel.setText(u'Отмена')


    def stop(self):
        self.processDone = True
        self.btnCancel.setText(u'Закрыть')


    def done(self, r):
        self.stop()
        QtGui.QDialog.done(self, r)


    def getTmpDir(self):
        if not self.tmpDir:
            self.tmpDir = QtGui.qApp.getTmpDir('R51ExportEgisz')
        return self.tmpDir


    def getZipFileName(self):
        u"""имя должно задаваться как «YYYYMMDDNNN.zip»,
            где YYYY–текущий год, MM – текущий месяц,
            DD – текущий день, NNN – номер пакета за отчетный период."""

        if not self.zipFileName:
            self.zipFileName = '%s%03d.zip' % (
                       forceString(QDate.currentDate().toString('yyyyMMdd')),
                       self.packageNum)

        return self.zipFileName


    def cleanup(self):
        if self.tmpDir:
            QtGui.qApp.removeTmpDir(self.tmpDir)


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        if self.processDone:
            self.accept()
        else:
            self.aborted = True

# *****************************************************************************************

    def formatAddress(self, placeName, streetName, number, corpus, flat):
        parts = []
        if placeName:
            parts.append(placeName)
        if streetName:
            parts.append(streetName)
        if number:
            parts.append(u'д. '+number)
        if corpus:
            parts.append(u'к. '+corpus)
        if flat:
            parts.append(u'кв. '+flat)
        return ', '.join(parts)

# *****************************************************************************************

    def createDetDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'DET.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('FILENAME', 'C', 100), # Название файла пакета данных
            ('ENCODING', 'C', 100), # Кодировка файлов в пакете данных
            ('NUM', 'N', 1), # Порядковый номер передаваемого пакета данных в пределах отчетного периода
            ('TYPE', 'N', 1), # Тип пакета данных
                                    # 1 - Пакет данных по законченным случаям госпитализации
                                    # 2 - Пакет данных по амбулаторным случаям
            )
        return dbf


    def createPatDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'PAT.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('INS', 'C', 20), # Код СМО (по справочнику фонда)
            ('SERPOL', 'C', 10), # Серия полиса
            ('NMBPOL', 'C', 200), # Номер полиса
            ('DOGOVOR', 'C', 100), #    Номер договора
            ('SOC_STATUS', 'N', 1), # Социальный статус Возможные значения:
            ('FAM', 'C', 40), # Фамилия пациента
                                    # Для детей при отсутствии данных ФИО до
                                    # государственной регистрации рождения указывается «НЕТ».
                                    # Отчество указывается «НЕТ» при отсутствии в удостоверении
            ('IMM', 'C', 40), # Имя пациента
            ('OTC', 'C', 40), # Отчество пациента
            ('SER_PASP', 'C', 10), # Серия документа, удостоверяющего личность
            ('NUM_PASP', 'C', 20), # Номер документа, удостоверяющего личность
            ('TYP_DOC', 'N', 2), # Тип документа, удостоверяющего личность (приложение «Типы документов»)
            ('SS', 'C', 14), # Страховой номер индивидуального лицевого счета (СНИЛС)
            ('PIN', 'C', 16), # Персональный  индивидуальный номер застрахованного по ОМС
            ('BIRTHDAY', 'D'), # Дата рождения пациента Для новорожденных передавать время
            ('SEX', 'C', 1), # Пол пациента   («М», «Ж»)
            ('TAUN', 'C', 100), # Код населенного пункта проживания пациента по справочнику фонда   Код территории проживания
            ('NUMMEDKART', 'C', 10), # Номер мед.карты  Для идентификации пациента
            ('ADRES', 'C', 200), # Адрес регистрации по месту жительства
            ('GITEL_TAUN', 'N', 1), #Житель (город/село)    Возможные значения:
            ('KAT_LGOTA', 'N', 1), #Категория льготы    Возможные значения:
            ('WEEK_PREG', 'N', 1), #На какой неделе беременности родился    Для детей до 1 года
            ('PAT_INDUCT', 'N',1), #Призывник
            ('VIDDOKOMS', 'C', 3), # Тип ДУЛ    Код по справочнику ТФОМС
            ('NASP_OMS', 'C', 100), #Населенный пункт   Код населенного пункта проживания по справочнику ТФОМС
            ('POLITYPOMS', 'C', 3), #Тип полиса Код типа полиса по справочнику ТФОМС
            ('POLDATEB', 'D'), # Дата выдачи полиса
            ('POLDATEC', 'D'), # Дата аннулирования полиса  Заполняется, если полис аннулирован
            ('POLDATETO', 'D'), # Дата окончания полиса Заполняется при наличии (временный полис)
            ('LPU_LECH', 'C', 20), # Код ОМС ЛПУ лечения    В соответствии со справочниками ТФОМС
            ('LPU_PRIKR', 'C',  20), # Код ОМС ЛПУ прикрепления В соответствии со справочниками ТФОМС
            ('LPU_DIR', 'C', 20), # Код ОМС направившего ЛПУ    В соответствии со справочниками ТФОМС
            ('ORTOFBIRTH', 'C', 254), # Место рождения  В соответствии с записью в ДУЛ
            ('P_FAM', 'C', 40), # Фамилия представителя
            ('P_IM', 'C', 40)   ,  # Имя представителя
            ('P_OT', 'C', 40),  # Отчество представителя
            ('P_DR', 'D'),  # Дата рождения представителя
            ('P_POL', 'C', 1),  #   Пол представителя   «М» или «Ж»
            ('P_VIDDOK', 'C', 3),  #    Тип ДУЛ представителя Код в соответствии со справочником ТФОМС
            ('P_SERDOK', 'C', 10),  #   Серия ДУЛ представителя
            ('P_NUMDOK', 'C', 20),  #   Номер ДУЛ представителя
            ('NOVOR', 'C', 20),  #  Признак новорожденного
            ('Q_G', 'C', 2),  # Признак особого случая
            ('KOD_TER', 'C', 20),  #    Территория страхования Код по ОКАТО
            )
        return dbf


    def createHospDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'HOSP.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10),  # Номер мед.карты (ЭМК)
            ('CARD', 'C', 25),  # Номер истории болезни
            ('VID_OPLA', 'N', 1),  # Вид оплаты
            ('KEM_NAPR', 'C', 100),  #Направление на госпитализацию: Кем направлен
            ('NUM_NAPR', 'C', 100),  #Направление на госпитализацию: № направления
            ('DATE_NAPR', 'D'),  # Направление на госпитализацию: Дата направления
            ('DIAG_NAPR', 'C', 100),  # Диагноз направившего учреждения
            ('DIAG_PRIEM', 'C', 100),  # Диагноз приемного отделения
            ('SOST_OP', 'N', 1),  # Доставлен в состоянии опьянения
            ('H_CURRYEAR', 'N', 1),  # Госпитализирован по поводу данного заболевания в текущем году
            ('AFTER_HOUR', 'N', 1),  # Доставлен в стационар от начала заболевания (получения травмы)
            ('KANAL_HOSP', 'N', 1),  # Канал госпитализации
            ('TRAVMA', 'N', 1),  #Вид травмы
            ('DATEPOSTUP', 'D'),  #Дата и время поступления в приемное отделение
            ('OTD_CODE', 'C', 100),  #Отделение госпитализации
            ('DATE_HOSP', 'D'),  #Дата и время поступления в отделение
            ('VID_STAC', 'N', 1),  #Вид стационара
            ('DATE_VYPIS', 'D'),  #Дата и время выбытия (выписки или смерти)
            ('KOJKO_DNEJ', 'N', 1),  #Продолжительность госпитализации (койко-дней)
            ('ISHOD_HOSP', 'N', 1),  #Исход госпитализации
            ('REZ_HOSP', 'N', 1),  # Результат госпитализации
            ('LN_DATE_FR', 'D'),  # Дата начала случая ВН
            ('LN_DATE_TO', 'D'),  # Дата закрытия случая ВН
            ('DEF_DOHOSP', 'N', 1),  # Дефекты догоспитального этапа
            ('HOSP_GOAL', 'N', 1),  # Цель госпитализации
            ('PROF_POST', 'C', 20),  #Профиль поступления
            ('PROF_VYP', 'C', 20),  #Профиль выписки
            ('SIF_EXAM', 'N',1),  #Обследован серологически с целью выявления сифилиса
            ('ZAB_FROM', 'D'),  #Дата и время начала заболевания
            ('TROMB_THER', 'N',1),  #Проведена тромбологическая терапия
            ('STENTIR', 'N',1),  #Проведено стентирование
            ('PREG_WEEKS', 'N', 1),  #Срок беременности, недель
            ('BORN_W', 'N', 1),  #Масса при рождении, грамм
            ('KRIM', 'C', 1),  #Признак противоправных действий
            ('ISH_ZABOMS', 'C', 3),  #Исход заболевания
            ('PLAN_PREDV', 'C', 20),  #Предварительный МЭС
            ('CHILD', 'C', 100),  #Номер истории развития новорожденного
            ('OSOB_SL_CH', 'C', 2),  #Признак особенности
            ('PLAN_ZAKL', 'C', 20),  #Заключительный МЭС
            ('MED_CR', 'C', 10),  #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код лечащего врача
            )
        return dbf


    def createTransfDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'TRANSF.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), #Номер истории болезни
            ('ARRDEPCODE', 'C', 100), #Отделение размещения
            ('ARR_PROFIL', 'C', 100), #Профиль размещения
            ('FIO_MED', 'C', 100), #ФИО врача
            ('ARR_DATE', 'D'), #Дата поступления
            ('DEPARTDATE', 'D'), #Дата освобождения койки
            ('DIAG_CODE', 'C', 100), #Код диагноза по МКБ10 (клинического, во время нахождения в отделении)
            ('BEDTYPEOMS', 'C', 16), #Профиль койки
            ('DIALIZ', 'C', 1), #Признак отделения гемодиализа (размещение)
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код медицинского сотрудника (автор)
            )
        return dbf


    def createOperDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'OPER.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), #Номер истории болезни
            ('DATE_OPER', 'D'), # Дата и время начала операции
            ('DATEOPER_E', 'D'), # Дата и время окончания операции
            ('OPER_MED', 'C', 254), #ФИО хирурга
            ('OPERDEPCOD', 'C', 100), #Отделение
            ('OPER_CODE', 'C', 254), #Код операции
            ('OPER_NAME', 'C', 254), #Наименование операции
            ('COMPL_TEXT', 'C', 254), #Осложнение (текст)
            ('COMPL_CODE', 'C', 100), #Интраоперационное осложнение (код)
            ('ANESTESIA', 'C', 100), #Анестезия
            ('ENDOSCOPHI', 'N',1), #Использовалась аппаратура: эндоскопическая
            ('LAZER', 'N',1), #Использовалась аппаратура: лазерная
            ('CRYOGENIC', 'N',1), #Использовалась аппаратура: криогенная
            ('VID_OPLAT', 'N'), #Вид оплаты
            ('OPDIAGCODE', 'C', 100), #Повод операции
            ('ISHOD', 'C', 3), #Исход
            ('PLAN_OMS', 'C', 20), #Код оказанного МЭСа
            ('KOL_OPER', 'N'), # Количество оказаний
            ('LEVEL', 'C', 10), #Уровень оказанной помощи
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код медицинского сотрудника (автор)
            )
        return dbf


    def createDiagDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'DIAG.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), #Номер истории болезни
            ('VID_ZABOL', 'N', 1), #Вид диагноза
            ('DIAG_STAC', 'N', 1), #Диагноз стационара
            ('DIAG_STR', 'C', 254), #   Диагноз (текст)
            ('DIAG_CODE', 'C', 100), #Диагноз заболевания
            ('DIAGCODEXT', 'C', 10), #Расширение диагноза
            ('DIAGINJURY', 'N', 1), #Вид травмы
            ('FIRST_UST', 'N', 1), #Установлен впервые
            ('VYZV_SMERT', 'N', 1), #Привел к смерти
            ('DIAG_PERV', 'C', 100), #Первичный диагноз
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код медицинского сотрудника (автор)
            )
        return dbf


    def createAnatDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'ANAT.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), #Номер истории болезни
            ('VID_ZABOL', 'N'), # Вид диагноза
            ('DIAG_STR', 'C', 254), # Диагноз (текст)
            ('DIAG_CODE', 'C', 100), # Диагноз заболевания
            ('P_BORNDEAD', 'N',1), #Мертворожденный
            ('BORNDEAD_B', 'N',1), #Родился мертворожденным на сроке 22-27 недель
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C',10), #Код медицинского сотрудника (автор)
            )
        return dbf


    def createDispDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'DISP.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('DISP_FROM', 'D'), # Дата постановки на Д-учет
            ('DISP_DIAG', 'C', 100), #Диагноз заболевания
            ('MED_SPEC', 'C', 20), #Специальность врача
            ('MED_TYPE', 'N', 1), #Врач/средний медперсонал
            ('DATE_TO', 'D'), #Дата снятия с наблюдения
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код медицинского сотрудника (автор)
            )
        return dbf


    def createTapDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'TAP.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('CARD', 'C', 25), #Номер стат. талона
            ('DATE', 'D'), #Дата приема пациента
            ('NUMMEDKART', 'C', 10), # Номер мед.карты
            ('CEL_POSEH', 'N', 1), #Цель посещения
            ('SLUH_POS', 'N', 1), #Случай посещения
            ('REZUL_POS', 'N', 1), #Результат посещения
            ('MESTO_OBSL', 'N', 1), #Место обслуживания
            ('VID_OPLAT', 'N', 1), #Вид оплаты
            ('POVOD_OBR', 'N', 1), #Повод обращения
            ('LN_NUM', 'N', 1), #Лист нетрудоспособности
            ('CODE_HOSP', 'C', 20), # Код ЛПУ пребывания по справочнику фонда
            ('MED_SPEC', 'C', 20), #    Специальность врача
            ('MED_CODE', 'C', 10), #    Код врача
            ('MED_TYPE', 'N', 1), #Врач/средний медперсонал
            ('HOSP_CANC', 'N',1), #Отказ в госпитализации
            ('DATE_ZAKR', 'D'), #Дата закрытия талона
            ('DATE_OPEN', 'D'), #Дата открытия талона
            ('ISHOD', 'C', 3), # Исход заболевания
            ('MED_CR', 'C', 10), # Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #    Код медицинского сотрудника (автор)
            )
        return dbf


    def createDiscertDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'DISCERT.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), #Номер истории болезни или стат. талона
            ('DATE_BEGIN', 'D'), #Дата начала случая
            ('DATE_END', 'D'), #Дата окончания случая
            ('CIM10', 'C', 100), #Основной диагноз в рамках случая ВН
            ('PO_UHODU', 'N',1), #По уходу
            ('OTPUSK_SK', 'N',1), #Отпуск в связи с санаторно-курортным лечением (без туберкулеза и долечивания инфаркта миокарда)
            ('KARANTIN', 'N',1), #Освобождение от работы в  связи с карантином и бактерионосительством
            ('OTPUSKRODY', 'N',1), #Отпуск по беременности и родам (неосложненным)
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код медицинского сотрудника (автор)
            )
        return dbf


    def createLpuDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'LPU.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('LPU_CODE', 'C', 20), #Код ЛПУ
            ('TIP_LPU', 'N', 1), #Тип ЛПУ
            ('DNSTACMEST', 'N', 1), #Количество мест в дневных стационарах
            ('DNSTACSMEN', 'N', 1), #Количество смен в дневных стационарах
            ('DNSAPUMEST', 'N', 1), #Количество мест в дневных стационарах при АПУ
            ('DNSAPUSMEN', 'N', 1), #Количество смен в дневных стационарах при АПУ
            ('DNSDOMMEST', 'N', 1), #Количество мест в дневных стационарах на дому
            ('STAC_MDET', 'N', 1), #Количество мест в дневных стационарахдля детей(0-17 лет)
            ('STAC_SDET', 'N', 1), #Количество смен в дневных стационарах для детей(0-17 лет)
            ('APU_MDET', 'N', 1), #Количество мест в дневных стационарахпри АПУ  для детей(0-17 лет)
            ('APU_SDET', 'N', 1), #Количество смен в дневных стационарах при АПУ для детей(0-17 лет)
            ('DOM_MDET', 'N', 1), #Количество мест в дневных стационарах на дому для детей(0-17 лет)
            )
        return dbf


    def createBedsDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'BEDS.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('LPU_CODE', 'C', 20), #Код ЛПУ
            ('DEP_CODE', 'C', 20), #Код отделения
            ('PROF_CODE', 'C', 20), #Профиль отделения
            ('BED_PROF', 'C', 16), #Код профиля койки
            ('BPROF_NAME', 'C', 70), #Наименование профиля койки
            ('SMET_COUNT', 'N'), #Количество коек по смете
            ('FACT_COUNT', 'N'), #Количество коек фактическое
            ('DIS_COUNT', 'N'), #Количество свёрнутых коек
            )
        return dbf


    def createStomDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'STOM.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), #Номер стат. талона
            ('UET', 'N', 20, 6), # Объём работы в УЕТ
            ('C_ZUBOV', 'N'), #Вылечено зубов
            ('C_ZUB_POST', 'N'), #Вылечено постоянных зубов
            ('C_ZUB_KAR', 'N'), #Вылечено зубов по поводу осложнённого кариеса
            ('UD_ZUBOV', 'N'), #Удалено зубов
            ('UD_ZUBPOST', 'N'), #Удалено постоянных зубов
            ('SANIROVANO', 'N'), #Всего санировано зубов
            ('TREB_SANAC', 'N'), #Нуждается в санации, зубов
            ('PLAN_SANAC', 'N',1), #Плановая санация
            ('SANIR_NUGD', 'N'), #Санировано из числа нуждавшихся, зубов
            ('KURS_PROF', 'N',1), #Проведён курс профилактики
            ('PROTEZ', 'N',1), #Выдан зубной протез
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код медицинского сотрудника (автор)
            )
        return dbf


    def createBloodDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'BLOOD.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), # Номер истории болезни или номер статталона
            ('DATE_PROC', 'D'), #Дата и время переливания
            ('PERELIVAN', 'N'), #Кол-во переливаний
            ('PERELITO', 'N'), #Перелито литров
            ('OSLOGNENIE', 'N',1), #Наблюдалось осложнение
            ('SREDSTVO', 'N'), #Трансфузионное средство
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код медицинского сотрудника (автор)
            )
        return dbf


    def createRentDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'RENT.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), # Номер истории болезни или номер статталона
            ('DATE_PROC', 'D'), #Дата исследования
            ('VID_ISSLED', 'N'), #Вид исследования
            ('OBL_ISSLED', 'N'), #
            ('MESTPROVED', 'N'), #Место проведения
            ('CIM10', 'C', 100), #Код МКБ-10, с которым пациент направлен на исследование
            ('VIDSPECISS', 'N'), #
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код медицинского сотрудника (автор)
            )
        return dbf


    def createUsDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'US.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), # Номер истории болезни или номер статталона
            ('DATE_PROC', 'D'), #Дата исследования
            ('OBL_ISSLED', 'C', 254), #Область исследования
            ('OPISANIE', 'C', 254), #Описание исследования
            ('MESTPROVED', 'N'), #
            ('CIM10', 'C', 100), #Код МКБ-10, с которым пациент направлен на исследование
            ('INTRAOPER', 'N',1), #Интраоперационное исследование
            ('INTERV', 'N',1), #Интервенционное вмешательство
            ('PLOD_ANOM', 'N',1), #Выявлен плод с аномалиями пороков развития
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код медицинского сотрудника (автор)
            )
        return dbf


    def createRadDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'US.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), # Номер истории болезни или номер статталона
            ('DATE_PROC', 'D'), #Дата исследования
            ('VID_ISSLED', 'C', 254), #Вид исследования
            ('OPISANIE', 'C', 254), #Описание исследования
            ('MESTPROVED', 'N'),
            ('IN_VIVO', 'N',1), #Исследование на живом организме
            ('CIM1', 'C', 100), #Код МКБ10 с которым пациент поступил
            ('CIM10_OUT', 'C', 100), #Код МКБ10, выявленный во время исследования
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код медицинского сотрудника (автор)
                    )
        return dbf


    def createEndDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'END.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), # Номер истории болезни или номер статталона
            ('DATE_PROC', 'D'), #Дата исследования
            ('MESTPROVED', 'N'), #Место исследования
            ('VID_ISSLED', 'N'), #Вид исследования
            ('OPISANIE', 'C', 254), #Описание исследования(протокол)
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код медицинского сотрудника (автор)
                    )
        return dbf


    def createLabDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'LAB.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), # Номер истории болезни или номер статталона
            ('DATE_PROC', 'D'), #Дата исследования
            ('MESTPROVED', 'N'), #Место исследования
            ('VID_ISSLED', 'N'), #Вид исследования
            ('OPISANIE', 'C', 254), #Описание исследования(протокол)
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код медицинского сотрудника (автор)
            )
        return dbf


    def createFuncDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'FUNC.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), # Номер истории болезни или номер статталона
            ('DATE_PROC', 'D'), #Дата исследования
            ('MESTPROVED', 'N'), #Место исследования
            ('VID_ISSLED', 'N'), #Вид исследования
            ('OPISANIE', 'C', 254), #Описание исследования(протокол)
            ('MED_CR', 'C', 10), #Код медицинского сотрудника, создавшего запись
            ('MED_AUTH', 'C', 10), #Код медицинского сотрудника (автор)
            )
        return dbf


    def createInvDbf(self):
        dbfName = os.path.join(self.getTmpDir(), 'INV.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('INV_NUM', 'C', 100), #Номер счета
            ('INV_DATE', 'D'), #Дата выставления счета
            ('NUMMEDKART', 'C', 10), #Номер мед.карты(ЭМК)
            ('CARD', 'C', 25), # Номер истории болезни или номер статталона
            ('ORGPAY_OMS', 'C', 20), #ЛПУ, выставляющее счет
            ('ORG_OMS', 'C', 20), #СМО для выставления счета
            ('SERV_OMS', 'C', 50), #Оказанная услуга
            ('CNT', 'N', 1), #Количество оказаний
            ('DATE_CREAT', 'D'), #Дата и время начала оказания услуги
            ('DATE_END', 'D'), #Дата и время окончания оказания услуги
            ('ISHOD_SERV', 'C', 3), #Исход оказания
            ('PRICETOPAY', 'N', 15, 2), #Стоимость оказания
            ('STOIM_S', 'N', 15, 2), #Сумма из средств Федеральных субвенций
            ('STOIM_R', 'N', 15, 2), #Сумма из средств Субъекта РФ (статья 6 – иные расходы)
            ('STOIM_F', 'N', 15, 2), #Сумма доплаты из средств Федерального фонда ОМС
            ('MED_KOD', 'C', 20), #Код медицинского сотрудника, оказавшего услугу
            ('MEDSPECOMS', 'C', 20), #Специальность врача, оказавшего услугу/прием
            ('LEVEL', 'C', 10), #Уровень оказанной помощи
                    )
        return dbf

# ******************************************************************************

    def createDbf(self):
        if self.isHospital:
            return (self.createDetDbf(), self.createPatDbf(), self.createHospDbf(),
                    self.createDiagDbf(), self.createDispDbf(), self.createDiscertDbf(),
                    self.createLpuDbf(), self.createInvDbf())
        else:
            return (self.createDetDbf(), self.createPatDbf(), self.createTapDbf(),
                    self.createDiagDbf(), self.createDispDbf(), self.createDiscertDbf(),
                    self.createLpuDbf(), self.createInvDbf())

# ******************************************************************************

    def createQuery(self):
        db = QtGui.qApp.db
        tableEvent = db.table('Event')
        cond = []
        cond.append(tableEvent['execDate'].between(
                                       self.begDate.toString(Qt.ISODate),
                                       self.endDate.toString(Qt.ISODate)))
        cond.append(tableEvent['client_id'].isNotNull())
        cond.append(tableEvent['deleted'].eq(0))
        idList = db.getIdList('Event', where=cond)

        if idList == []:
           return None

        stmt = """SELECT Client.lastName AS lastName,
            Client.firstName AS firstName,
            Client.patrName AS patrName,
            Client.birthDate AS birthDate,
            Client.birthPlace AS birthPlace,
            Client.sex AS sex,
            Client.SNILS AS SNILS,
            age(Client.birthDate, Event.execDate) AS clientAge,
            ClientDocument.serial  AS documentSerial,
            ClientDocument.number  AS documentNumber,
            ClientPolicy.serial AS policySerial,
            ClientPolicy.number AS policyNumber,
            ClientPolicy.begDate AS policyBegDate,
            ClientPolicy.endDate AS policyEndDate,
            rbPolicyKind.regionalCode AS policyKindCode,
            rbDocumentType.regionalCode AS documentRegionalCode,
            RegAddressHouse.KLADRCode AS regKLADRCode,
            RegAddressHouse.number AS regNumber,
            RegAddressHouse.corpus AS regCorpus,
            RegAddress.flat AS regFlat,
            RegKLADR.infis AS regPlaceCode,
            RegKLADR.NAME AS regPlaceName,
            RegStreet.infis AS regStreetCode,
            RegStreet.NAME AS regStreetName,
            LocAddressHouse.KLADRCode AS locKLADRCode,
            LocKLADR.infis AS locPlaceCode,
            LocKLADR.NAME AS locPlaceName,
            LocStreet.infis AS locStreetCode,
            LocStreet.NAME AS locStreetName,
            LocAddressHouse.number AS locNumber,
            LocAddress.flat AS locFlat,
            Event.client_id AS clientId,
            Event.isPrimary AS eventPrimary,
            Insurer.OKATO AS insurerOKATO,
            RelegateOrg.infisCode AS relegateOrgCode,
            AttachOrg.infisCode AS attachOrgCode,
            Diagnosis.MKB,
            MKB.DiagName,
            AccDiagnosis.MKB AS AccMKB,
            IF(rbDiagnosticResult.code IS NULL,
                EventResult.code,
                rbDiagnosticResult.code) AS resultCode,
            IF(rbDiagnosticResult.regionalCode IS NULL,
                EventResult.regionalCode,
                rbDiagnosticResult.regionalCode) AS resultRegionalCode,
            rbDiagnosisType.code AS diagnosisCode,
            rbTraumaType.code AS traumaTypeCode,
            IF(rbDiseaseCharacter.code IN ('1','2'),1,0) AS firstTime,
            Person.regionalCode AS personCode,
            rbSpeciality.regionalCode AS specialityCode,
            CreatePerson.regionalCode AS createPersonCode,
            Benefit.regionalCode AS benefitCode,
            SocStatus.regionalCode AS socStatusCode,
            IF(multipleBirthAction.id IS NOT NULL,1,0) AS isMultipleBirth,
            rbEventTypePurpose.regionalCode AS eventPurposeTypeRegionalCode,
            rbScene.code AS sceneCode,
            TempInvalid.closed AS tempInvalidClosed,
            TempInvalid.id AS tempInvalidId,
            TempInvalid.begDate AS tempInvalidBegDate,
            TempInvalid.endDate AS tempInvalidEndDate,
            EventType.code AS eventTypeCode,
            rbTempInvalidReason.code AS tempInvalidReasonCode,
            Account.number AS accountNumber,
            Account.settleDate,
            Account_Item.amount,
            Account_Item.sum,
            Payer.infisCode AS payerCode,
            Event.setDate AS begDate,
            Event.execDate AS endDate
            FROM Event
            LEFT JOIN Client ON Client.id = Event.client_id
            LEFT JOIN ClientPolicy ON ClientPolicy.client_id = Client.id AND
                      ClientPolicy.id = (SELECT MAX(CP.id)
                                         FROM   ClientPolicy AS CP
                                         LEFT JOIN rbPolicyType AS CPT ON CPT.id = CP.policyType_id
                                         WHERE  CP.client_id = Client.id AND CP.deleted=0 AND CPT.code IN ('1','2'))
            LEFT JOIN Organisation AS Insurer ON ClientPolicy.insurer_id = Insurer.id
            LEFT JOIN rbPolicyKind ON ClientPolicy.policyKind_id = rbPolicyKind.id
            LEFT JOIN ClientDocument ON ClientDocument.client_id = Client.id AND
                      ClientDocument.id = (SELECT MAX(CD.id)
                                         FROM   ClientDocument AS CD
                                         LEFT JOIN rbDocumentType AS rbDT ON rbDT.id = CD.documentType_id
                                         LEFT JOIN rbDocumentTypeGroup AS rbDTG ON rbDTG.id = rbDT.group_id
                                         WHERE  rbDTG.code = '1' AND CD.client_id = Client.id AND CD.deleted=0)
            LEFT JOIN rbDocumentType ON rbDocumentType.id = ClientDocument.documentType_id
            LEFT JOIN ClientAddress ClientRegAddress ON ClientRegAddress.client_id = Client.id AND
                      ClientRegAddress.id = (SELECT MAX(CRA.id)
                                         FROM   ClientAddress AS CRA
                                         WHERE  CRA.type = 0 AND CRA.client_id = Client.id AND CRA.deleted=0)
            LEFT JOIN ClientAddress ClientLocAddress ON ClientLocAddress.client_id = Client.id AND
                      ClientLocAddress.id = (SELECT MAX(CLA.id)
                                         FROM   ClientAddress AS CLA
                                         WHERE  CLA.type = 1 AND CLA.client_id = Client.id AND CLA.deleted=0)
            LEFT JOIN Address   RegAddress ON RegAddress.id = ClientRegAddress.address_id
            LEFT JOIN AddressHouse RegAddressHouse ON RegAddressHouse.id = RegAddress.house_id
            LEFT JOIN Address   LocAddress ON LocAddress.id = ClientLocAddress.address_id
            LEFT JOIN AddressHouse LocAddressHouse ON LocAddressHouse.id = LocAddress.house_id
            LEFT JOIN kladr.KLADR RegKLADR ON RegKLADR.CODE = RegAddressHouse.KLADRCode
            LEFT JOIN kladr.STREET RegStreet ON RegStreet.CODE = RegAddressHouse.KLADRStreetCode
            LEFT JOIN kladr.KLADR LocKLADR ON LocKLADR.CODE = LocAddressHouse.KLADRCode
            LEFT JOIN kladr.STREET LocStreet ON LocStreet.CODE = LocAddressHouse.KLADRStreetCode
            LEFT JOIN Organisation AS RelegateOrg ON Event.relegateOrg_id = RelegateOrg.id
            LEFT JOIN ClientAttach ON ClientAttach.client_id=Event.client_id AND
                        ClientAttach.id = (SELECT max(CA.id)
                                         FROM ClientAttach AS CA
                                         WHERE CA.client_id=Client.id AND CA.deleted=0)
            LEFT JOIN Organisation AS AttachOrg ON ClientAttach.LPU_id = AttachOrg.id
            LEFT JOIN Diagnostic ON ( Diagnostic.event_id = Event.id
                                      AND Diagnostic.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code IN ('1', '2'))
                                      AND Diagnostic.person_id = Event.execPerson_id AND Diagnostic.deleted = 0)
            LEFT JOIN Diagnostic AS AccDiagnostic ON AccDiagnostic.id = (
             SELECT id FROM Diagnostic AS AD
             WHERE AD.event_id = Event.id AND
                AD.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code='9') AND
                AD.person_id = Event.execPerson_id AND
                AD.deleted = 0 LIMIT 1)
            LEFT JOIN Diagnosis AS AccDiagnosis ON
                AccDiagnosis.id = AccDiagnostic.diagnosis_id AND
                AccDiagnosis.deleted = 0
            LEFT JOIN ClientWork ON ClientWork.client_id=Event.client_id AND
                        ClientWork.id = (SELECT max(CW.id)
                                         FROM ClientWork AS CW
                                         WHERE CW.client_id=Client.id AND CW.deleted=0)
            LEFT JOIN Diagnosis ON Diagnosis.id = Diagnostic.diagnosis_id AND Diagnosis.deleted = 0
            LEFT JOIN MKB ON Diagnosis.MKB = MKB.DiagId
            LEFT JOIN rbDiagnosticResult ON rbDiagnosticResult.id=Diagnostic.result_id
            LEFT JOIN rbResult AS EventResult ON EventResult.id=Event.result_id
            LEFT JOIN rbDiagnosisType ON Diagnostic.diagnosisType_id = rbDiagnosisType.id
            LEFT JOIN rbTraumaType ON rbTraumaType.id = Diagnostic.traumaType_id
            LEFT JOIN rbDiseaseCharacter ON rbDiseaseCharacter.id = Diagnostic.character_id
            LEFT JOIN Person ON Person.id = Event.execPerson_id
            LEFT JOIN rbSpeciality ON Person.speciality_id = rbSpeciality.id
            LEFT JOIN Person AS CreatePerson ON CreatePerson.id = Event.createPerson_id
            LEFT JOIN Action AS HospitalAction ON
                HospitalAction.id = (
                    SELECT MAX(A.id)
                    FROM Action A
                    WHERE A.event_id = Event.id AND
                              A.deleted = 0 AND
                              A.actionType_id IN (
                                    SELECT AT.id
                                    FROM ActionType AT
                                    WHERE AT.flatCode ='moving'
                              )
                )
            LEFT JOIN ClientSocStatus ClientBenefit ON
                ClientBenefit.id = (
                    SELECT MAX(CSS.id)
                    FROM ClientSocStatus CSS
                    WHERE CSS.client_id = Client.id AND
                              CSS.deleted = 0 AND
                              (CSS.begDate IS NULL OR CSS.begDate >= Event.setDate) AND
                              (CSS.endDate IS NULL OR CSS.endDate <= Event.execDate) AND
                              CSS.socStatusClass_id = (
                                    SELECT S.id
                                    FROM rbSocStatusClass S
                                    WHERE S.code = '1'
                                    LIMIT 1
                              )
                    LIMIT 1
                )
            LEFT JOIN ClientSocStatus CSocStatus ON
                CSocStatus.id = (
                    SELECT MAX(CSS.id)
                    FROM ClientSocStatus CSS
                    WHERE CSS.client_id = Client.id AND
                              CSS.deleted = 0 AND
                              (CSS.begDate IS NULL OR CSS.begDate >= Event.setDate) AND
                              (CSS.endDate IS NULL OR CSS.endDate <= Event.execDate) AND
                              CSS.socStatusClass_id = (
                                    SELECT S.id
                                    FROM rbSocStatusClass S
                                    WHERE S.code = '9'
                                    LIMIT 1
                              )
                    LIMIT 1
                )
            LEFT JOIN rbSocStatusType SocStatus ON CSocStatus.socStatusType_id = SocStatus.id
            LEFT JOIN rbSocStatusType Benefit ON ClientBenefit.socStatusType_id = Benefit.id
            LEFT JOIN Action AS multipleBirthAction ON
                multipleBirthAction.id = (
                    SELECT MAX(A.id)
                    FROM Action A
                    WHERE A.event_id = Event.id AND
                              A.deleted = 0 AND
                              A.actionType_id IN (
                                    SELECT AT.id
                                    FROM ActionType AT
                                    WHERE AT.flatCode ='MultipleBirth'
                              )
                )
            LEFT JOIN EventType ON Event.eventType_id = EventType.id
            LEFT JOIN rbEventTypePurpose ON EventType.purpose_id = rbEventTypePurpose.id
            LEFT JOIN Visit AS LastVisit ON LastVisit.id = (
                SELECT max(LV.id)
                FROM Visit AS LV
                WHERE LV.event_id=Event.id AND LV.deleted=0
            )
            LEFT JOIN rbScene ON LastVisit.scene_id = rbScene.id
            LEFT JOIN TempInvalid ON
               TempInvalid.id = (
                   SELECT  TI.id
                   FROM    TempInvalid TI
                   WHERE   TI.client_id = Client.id AND
                               TI.deleted = 0 AND
                               (TI.begDate IS NULL OR TI.begDate >= Event.setDate) AND
                               (TI.endDate IS NULL OR TI.endDate <= Event.execDate)
                   LIMIT 1
               )
            LEFT JOIN rbTempInvalidReason ON rbTempInvalidReason.id = TempInvalid.tempInvalidReason_id
            LEFT JOIN Account_Item ON Event.id = Account_Item.event_id AND
                Account_Item.deleted = 0
            LEFT JOIN Account ON Account_Item.master_id = Account.id AND
                Account.deleted = 0
            LEFT JOIN Organisation AS Payer ON Payer.id = Account.payer_id
            WHERE %s
                AND HospitalAction.id IS %s NULL
            """ % (tableEvent['id'].inlist(idList),
                        ('NOT' if self.isHospital else ''))
        return db.query(stmt)

# *****************************************************************************************

    def prepareToExport(self):
        self._done = False
        self.aborted = False
        dbf = self.createDbf()
        self.prbEvents.setText(u'Запрос в БД...')
        QtGui.qApp.processEvents()
        query = self.createQuery()
        querySize = query.size() if query else 0
        self.setEventsCount(max(querySize, 1))
        return dbf, query


    def export(self):
        self.start()

        for proc in (self.exportInt,  self.exportPack):
            (result, rc) = QtGui.qApp.call(self, proc)

            if self.aborted or not result:
                self.prbEvents.setText(u'прервано')
                break
            else:
                self.prbEvents.setText(u'готово')
                self._done = True

        self.stop()


    def exportInt(self):
        dbf, query = self.prepareToExport()

        self.codeLpu = forceString(QtGui.qApp.db.translate('Organisation',
                    'id',  QtGui.qApp.currentOrgId(), 'infisCode'))

        (dbfDet, dbfPat, dbfEvent, dbfDiag,
            dbfDisp, dbfDiscert, dbfLpu, dbfInv) = dbf

        row = dbfDet.newRecord()
        row['FILENAME'] = self.getZipFileName()
        row['ENCODING'] = 'cp866'
        row['NUM'] = self.packageNum
        row['TYPE'] = 1 if self.isHospital else 2
        row.store()

        row = dbfLpu.newRecord()
        #Код ЛПУ
        row['LPU_CODE'] = self.codeLpu
        #Тип ЛПУ
        row['TIP_LPU'] = 0
        # Количество мест в дневных стационарах
        row['DNSTACMEST'] = 0
        # Количество смен в дневных стационарах
        row['DNSTACSMEN']  = 0
        # Количество мест в дневных стационарах при АПУ
        row['DNSAPUMEST'] = 0
        # Количество смен в дневных стационарах при АПУ
        row['DNSAPUSMEN'] = 0
        # Количество мест в дневных стационарах на дому
        row['DNSDOMMEST'] = 0
        #Количество мест в дневных стационарахдля детей(0-17 лет)
        row['STAC_MDET'] = 0
        #Количество смен в дневных стационарах для детей(0-17 лет)
        row['STAC_SDET'] = 0
        #Количество мест в дневных стационарахпри АПУ  для детей(0-17 лет)
        row['APU_MDET'] = 0
        #Количество смен в дневных стационарах при АПУ для детей(0-17 лет)
        row['APU_SDET'] = 0
        #Количество мест в дневных стационарах на дому для детей(0-17 лет)
        row['DOM_MDET'] = 0
        row.store()

        if query:
            while query.next():
                QtGui.qApp.processEvents()
                if self.aborted:
                    break
                self.prbEvents.step()
                self.process(dbf, query.record())

        for  x in dbf:
            x.close()


    def exportPack(self):
        success = False

        zipFileName = self.getZipFileName()
        zipFilePath = os.path.join(forceStringEx(self.getTmpDir()),
                                                zipFileName)
        zf = ZipFile(zipFilePath, 'w', allowZip64=True)

        for src in ('DET.DBF', 'PAT.DBF',
                        'HOSP.DBF' if self.isHospital else 'TAP.DBF',
                        'DIAG.DBF',  'DISP.DBF',  'DISCERT.DBF',
                        'LPU.DBF', 'INV.DBF'):
            filePath = os.path.join(forceStringEx(self.getTmpDir()),
                                                    os.path.basename(src))
            zf.write(filePath, src, ZIP_DEFLATED)

        zf.close()

        dst = os.path.join(forceStringEx(self.destDir), zipFileName)
        success, result = QtGui.qApp.call(self, shutil.move, (zipFilePath, dst))
        return success


    def process(self, dbf, record):
        clientId = forceRef(record.value('clientId'))
        clientAge = forceInt(record.value('clientAge'))
        eventId = forceRef(record.value('eventId'))
        createPersonCode = forceString(record.value('createPersonCode'))
        personCode = forceString(record.value('personCode'))
        tempInvalidId = forceRef(record.value('tempInvalidId'))

        (dbfDet, dbfPat, dbfEvent, dbfDiag, dbfDisp, dbfDiscert, dbfLpu, dbfInv) = dbf

        if not (clientId in self.exportedClientIdSet):
            specialCase = []
            row = dbfPat.newRecord()
            row['INS'] = self.codeLpu
            row['SERPOL'] = forceString(record.value('policySerial'))
            row['NMBPOL'] = forceString(record.value('policyNumber'))

            if not row['SERPOL'] and not row['NMBPOL']:
                 # «1» –  отсутствие у пациента полиса обязательного медицинского страхования;
                specialCase.append('1')

            # Hомер договора
            #('DOGOVOR', 'C', 100),
             # Социальный статус
            row['SOC_STATUS'] = forceInt(record.value('socStatusCode')) % 10
            row['FAM'] = forceString(record.value('lastName')) # Фамилия пациента
            row['IMM'] = forceString(record.value('firstName')) # Имя пациента
            row['OTC'] = forceString(record.value('patrName')) # Отчество пациента

            if not row['OTC']:
                #«4» – отсутствие отчества в документе, удостоверяющем личность пациента (представителя пациента);
                specialCase.append('4')

            row['SER_PASP'] = forceString(record.value('documentSerial')) # Серия документа, удостоверяющего личность
            row['NUM_PASP'] = forceString(record.value('documentNumber')) # Номер документа, удостоверяющего личность
            row['TYP_DOC'] = forceInt(record.value('documentRegionalCode')) % 100 # Тип документа, удостоверяющего личность (приложение «Типы документов»)
            row['SS'] = forceString(record.value('SNILS')) # Страховой номер индивидуального лицевого счета (СНИЛС)
            # Персональный  индивидуальный номер застрахованного по ОМС
            row['PIN'] = '' # 'C', 16
            # Дата рождения пациента    Для новорожденных передавать время
            birthDate = forceDate(record.value('birthDate'))
            row['BIRTHDAY'] = pyDate(birthDate)
            # Пол пациента   («М», «Ж»)
            sex = u'М' if forceInt(record.value('sex')) == 1 else u'Ж'
            row['SEX'] = sex
            # Код населенного пункта проживания пациента по справочнику фонда
            row['TAUN'] = forceString(record.value('regPlaceCode'))
            # Номер мед.карты
            row['NUMMEDKART'] = forceString(clientId)

            placeName = forceString(record.value('regPlaceName'))
            streetName = forceString(record.value('regStreetName'))
            number = forceString(record.value('regNumber'))
            corpus = forceString(record.value('regCorpus'))
            flat = forceString(record.value('regFlat'))

            # Адрес регистрации по месту жительства
            row['ADRES'] = self.formatAddress(placeName, streetName, number, corpus, flat)

            #Житель (город/село)    Возможные значения:
            regKLADRCode = forceString(record.value('regKLADRCode'))
            locKLADRCode = forceString(record.value('locKLADRCode'))
            kladrCode = regKLADRCode if regKLADRCode else locKLADRCode
            row['GITEL_TAUN'] = 1 if kladrCode[8:11] == '000' else 0

            #Категория льготы
            row['KAT_LGOTA'] = forceInt(record.value('benefitCode')) % 10

            #На какой неделе беременности родился   Для детей до 1 года
            if clientAge < 1:
                row['WEEK_PREG'] = forceInt(record.value('pregnancyWeek')) % 10

#            ('PAT_INDUCT', 'N',1), #Призывник
            # Тип ДУЛ
            row['VIDDOKOMS'] = forceString(record.value('documentRegionalCode'))
            #Населенный пункт   Код населенного пункта проживания по справочнику ТФОМС
            row['NASP_OMS'] = forceString(record.value('regPlaceCode'))
             #Тип полиса    Код типа полиса по справочнику ТФОМС
            row['POLITYPOMS'] = forceString(record.value('policyKindCode'))
            # Дата выдачи полиса
            row['POLDATEB'] = pyDate(forceDate(record.value('policyBegDate')))
#            ('POLDATEC', 'D'), # Дата аннулирования полиса Заполняется, если полис аннулирован
            # Дата окончания полиса Заполняется при наличии (временный полис)
            row['POLDATETO'] = pyDate(forceDate(record.value('policyEndDate')))
            # Код ОМС ЛПУ лечения
            row['LPU_LECH'] = self.codeLpu
            # Код ОМС ЛПУ прикрепления  В соответствии со справочниками ТФОМС
            row['LPU_PRIKR'] = forceString(record.value('attachOrgCode'))
            # Код ОМС направившего ЛПУ  В соответствии со справочниками ТФОМС
            row['LPU_DIR']  = forceString(record.value('relegateOrgCode'))
             # Место рождения   В соответствии с записью в ДУЛ
            row['ORTOFBIRTH'] = forceString(record.value('birthPlace'))
            info = self.getClientRepresentativeInfo(clientId)

            if info != {}:
                if clientAge < 15:
                    #«3» – при оказании медицинской помощи ребенку до 14 лет был
                    # предъявлен документ, удостоверяющий личность, и полис
                    # ОМС одного из его родителей или  представителей;
                    specialCase.append('3')

                row['P_FAM'] = info.get('lastName', '') # Фамилия представителя
                row['P_IM'] = info.get('firstName', '') # Имя представителя
                row['P_OT'] = info.get('patrName', '') # Отчество представителя
                row['P_DR'] = pyDate(info.get('birthDate', QDate())) # Дата рождения представителя
                row['P_POL'] = u'М' if info.get('sex', 0) else u'Ж' #   Пол представителя   «М» или «Ж»
                # Тип ДУЛ представителя Код в соответствии со справочником ТФОМС
                row['P_VIDDOK'] = info.get('documentTypeRegionalCode', '')
                row['P_SERDOK'] = info.get('serial', '') # Серия ДУЛ представителя
                row['P_NUMDOK'] = info.get('number', '') # Номер ДУЛ представителя

            # Признак новорожденного
            # ПДДММГГН, где П – пол ребенка в соответствии с классификатором V005 ФОМС;
            # ДД – день рождения; ММ – месяц рождения;
            # ГГ – последние две цифры года рождения;
            # Н – порядковый номер ребенка (до двух знаков).
            if clientAge < 1:
                # «2»  - медицинская помощь оказана новорожденному;
                specialCase.append('2')
                row['NOVOR'] = '%s%s1'% (sex[:1],  birthDate.toString('ddMMyy'))

            #«5» -  медицинская помощь, оказанная новорожденному при  многоплодных родах.
            isMultipleBirth = forceBool(record.value('isMultipleBirth'))

            if isMultipleBirth:
                specialCase.append('5')

            row['Q_G'] = ''.join(specialCase)
            #   Территория страхования Код по ОКАТО
            row['KOD_TER'] = forceString(record.value('insurerOKATO'))

            row.store()
            self.exportedClientIdSet.add(clientId)

        row = dbfEvent.newRecord()
        # Номер мед.карты (ЭМК)
        row['NUMMEDKART'] = forceString(clientId)
        # Номер истории болезни
        row['CARD'] = forceString(eventId)
        #Код медицинского сотрудника, создавшего запись
        row['MED_CR'] = createPersonCode
        #Код медицинского сотрудника (автор)
        row['MED_AUTH'] = personCode

        if self.isHospital:
            row['VID_OPLA']  = 0 # Вид оплаты
#            ('KEM_NAPR', 'C', 100),  #Направление на госпитализацию: Кем направлен
#            ('NUM_NAPR', 'C', 100),  #Направление на госпитализацию: № направления
#            ('DATE_NAPR', 'D'),  # Направление на госпитализацию: Дата направления
#            ('DIAG_NAPR', 'C', 100),  #    Диагноз направившего учреждения
#            ('DIAG_PRIEM', 'C', 100),  # Диагноз приемного отделения
#            ('SOST_OP', 'N'),  # Доставлен в состоянии опьянения
#            ('H_CURRYEAR', 'N'),  # Госпитализирован по поводу данного заболевания в текущем году
#            ('AFTER_HOUR', 'N'),  # Доставлен в стационар от начала заболевания (получения травмы)
#            ('KANAL_HOSP', 'N'),  # Канал госпитализации
            #Вид травмы
            row['TRAVMA'] = forceInt(record.value('traumaTypeCode')) % 10
#            ('DATEPOSTUP', 'D'),  #Дата и время поступления в приемное отделение
#            ('OTD_CODE', 'C', 100),  #Отделение госпитализации
#            ('DATE_HOSP', 'D'),  #Дата и время поступления в отделение
#            ('VID_STAC', 'N'),  #Вид стационара
#            ('DATE_VYPIS', 'D'),  #Дата и время выбытия (выписки или смерти)
#            ('KOJKO_DNEJ', 'N'),  #Продолжительность госпитализации (койко-дней)
#            ('ISHOD_HOSP', 'N'),  #Исход госпитализации
#            ('REZ_HOSP', 'N'),  # Результат госпитализации
#            ('LN_DATE_FR', 'D'),  # Дата начала случая ВН
#            ('LN_DATE_TO', 'D'),  # Дата закрытия случая ВН
#            ('DEF_DOHOSP', 'N'),  # Дефекты догоспитального этапа
#            ('HOSP_GOAL', 'N'),  # Цель госпитализации
#            ('PROF_POST', 'C', 20),  #Профиль поступления
#            ('PROF_VYP', 'C', 20),  #Профиль выписки
#            ('SIF_EXAM', 'N',1),  #Обследован серологически с целью выявления сифилиса
#            ('ZAB_FROM', 'D'),  #Дата и время начала заболевания
#            ('TROMB_THER', 'N',1),  #Проведена тромбологическая терапия
#            ('STENTIR', 'N',1),  #Проведено стентирование
#            ('PREG_WEEKS   ', 'N'),  #Срок беременности, недель
#            ('BORN_W', 'N'),  #Масса при рождении, грамм
#            ('KRIM', 'C', 1),  #Признак противоправных действий
#            ('ISH_ZABOMS', 'C', 3),  #Исход заболевания
#            ('PLAN_PREDV', 'C', 20),  #Предварительный МЭС
#            ('CHILD', 'C', 100),  #Номер истории развития новорожденного
#            ('OSOB_SL_CH   ', 'C', 2),  #Признак особенности
#            ('PLAN_ZAKL', 'C', 20),  #Заключительный МЭС
        else:
            #Дата приема пациента
            row['DATE'] = pyDate(forceDate(record.value('endDate')))
            #Цель посещения
            # 0 - не заполнено 1 – Заболевание 2 – Профосмотр
            # 3 – Патронаж 4 – Другое
            row['CEL_POSEH'] = forceInt(record.value('eventPurposeTypeRegionalCode')) % 10
            #Случай посещения
            # 0 - не заполнено 1 – Первичный 2 – Повторный
            eventPrimary = forceInt(record.value('eventPrimary')) % 10
            row['SLUH_POS'] = eventPrimary if eventPrimary in (1, 2) else 0
            #Результат посещения
            row['REZUL_POS'] = forceInt(record.value('resultRegionalCode')) % 10
            #Место обслуживания
            row['MESTO_OBSL'] = forceInt(record.value('sceneCode')) % 10
            #Вид оплаты
            #row['VID_OPLAT'] = 0
            #Повод обращения
            #row['POVOD_OBR'] = 0
            #Лист нетрудоспособности
            # Возможные значения: 1) 0 - не заполнено 2) 1 – открыт 3) 2 – закрыт
            tempInvalidClosed = forceInt(record.value('tempInvalidClosed')) % 10 + 1 if tempInvalidId else 0
            row['LN_NUM'] =  tempInvalidClosed if tempInvalidClosed in (0, 1, 2) else 0
            # Код ЛПУ пребывания по справочнику фонда
            row['CODE_HOSP'] = self.codeLpu
            # Специальность врача
            row['MED_SPEC'] = forceString(record.value('specialityCode'))
            # Код врача
            row['MED_CODE'] = personCode
            #Врач/средний медперсонал
            row['MED_TYPE'] = 2 if personCode[:1] in ('0', '1', '2', '3') else 1
            #Отказ в госпита лизации
            #row['HOSP_CANC'] = 0
            #Дата закрытия талона
            row['DATE_ZAKR'] = pyDate(forceDate(record.value('endDate')))
            #Дата открытия талона
            row['DATE_OPEN'] = pyDate(forceDate(record.value('begDate')))
#            ('ISHOD', 'C', 3), # Исход заболевания

        row.store()

        row = dbfDiag.newRecord()
        # Номер мед.карты (ЭМК)
        row['NUMMEDKART'] = forceString(clientId)
        # Номер истории болезни
        row['CARD'] = forceString(eventId)
        row['VID_ZABOL'] = forceInt(record.value('diagnosisCode')) #Вид диагноза
#        row['DIAG_STAC'] = , 'N', 1), #Диагноз стационара
        row['DIAG_STR'] = forceString(record.value('DiagName')) # Диагноз (текст)
        row['DIAG_CODE'] = forceString(record.value('MKB'))  #Диагноз заболевания
#        ('DIAGCODEXT', 'C', 10), #Расширение диагноза
        # Вид травмы
        row['DIAGINJURY'] = forceInt(record.value('traumaTypeCode')) % 10
        row['FIRST_UST'] = forceInt(record.value('firstTime'))
        #Привел к смерти
        row['VYZV_SMERT'] = 1 if forceString(record.value('resultCode')) in ('13', '99') else 0
        #Первичный диагноз
        row['DIAG_PERV'] = forceString(record.value('MKB'))
        #Код медицинского сотрудника, создавшего запись
        row['MED_CR'] = createPersonCode
        #Код медицинского сотрудника (автор)
        row['MED_AUTH'] = personCode
        row.store()

        if forceString(record.value('eventTypeCode')) == '03':
            row = dbfDisp.newRecord()
            # Номер мед.карты (ЭМК)
            row['NUMMEDKART'] = forceString(clientId)
            # Дата постановки на Д-учет
            row['DISP_FROM'] = pyDate(forceDate(record.value('begDate')))
            # Диагноз заболевания
            row['DISP_DIAG'] = forceString(record.value('DiagName'))
            # Специальность врача
            row['MED_SPEC'] = forceString(record.value('specialityCode'))
            # Врач/средний медперсонал
            row['MED_TYPE'] = 2 if personCode[:1] in ('0', '1', '2', '3') else 1
            # Дата снятия с наблюдения
            row['DATE_TO'] = pyDate(forceDate(record.value('endDate')))
            #Код медицинского сотрудника, создавшего запись
            row['MED_CR'] = createPersonCode
            #Код медицинского сотрудника (автор)
            row['MED_AUTH'] = personCode
            row.store()

        if tempInvalidId:
            row = dbfDiscert.newRecord()
            # Номер мед.карты (ЭМК)
            row['NUMMEDKART'] = forceString(clientId)
            # Номер истории болезни или стат. талона
            row['CARD'] = forceString(eventId)
            # Дата начала случая
            row['DATE_BEGIN'] = pyDate(forceDate(record.value('tempInvalidBegDate')))
            # Дата окончания случая
            row['DATE_END'] = pyDate(forceDate(record.value('tempInvalidEndDate')))
            # Основной диагноз в рамках случая ВН
            row['CIM10'] = forceString(record.value('DiagName'))
            tempInvalidReasonCode = forceString(record.value('tempInvalidReasonCode'))
            # По уходу
            row['PO_UHODU'] = 1 if tempInvalidReasonCode == '09' else 0
            # Отпуск в связи с санаторно-курортным лечением (без туберкулеза и долечивания инфаркта миокарда)
            row['OTPUSK_SK'] = 1 if tempInvalidReasonCode == '08' else 0
            # Освобождение от работы в  связи с карантином и бактерионосительством
            row['KARANTIN'] = 1 if tempInvalidReasonCode == '03' else 0
            #Отпуск по беременности и родам (неосложненным)
            row['OTPUSKRODY'] = 1 if tempInvalidReasonCode == '05' else 0
            #Код медицинского сотрудника, создавшего запись
            row['MED_CR'] = createPersonCode
            #Код медицинского сотрудника (автор)
            row['MED_AUTH'] = personCode
            row.store()

        row = dbfInv.newRecord()
        # Номер счета
        row['INV_NUM'] = forceString(record.value('accountNumber'))
        # Дата выставления счета
        row['INV_DATE'] = pyDate(forceDate(record.value('settleDate')))
        # Номер мед.карты(ЭМК)
        row['NUMMEDKART'] = forceString(clientId)
        # Номер истории болезни или номер статталона
        row['CARD'] = forceString(eventId)
        # ЛПУ, выставляющее счет
        row['ORGPAY_OMS'] = self.codeLpu
        # СМО для выставления счета
        row['ORG_OMS'] = forceString(record.value('payerCode'))
        # Оказанная услуга
        row['SERV_OMS'] = forceString(record.value('serviceCode'))
        # Количество оказаний
        row['CNT'] =  forceInt(record.value('amount'))
        # Дата и время начала оказания услуги
        row['DATE_CREAT'] = pyDate(forceDate(record.value('begDate')))
        # Дата и время окончания оказания услуги
        row['DATE_END'] = pyDate(forceDate(record.value('endDate')))
        # Исход оказания
        row['ISHOD_SERV'] = forceString(record.value('resultCode'))
        # Стоимость оказания
        row['PRICETOPAY'] = forceDouble(record.value('sum'))
        # Сумма из средств Федеральных субвенций
        row['STOIM_S'] = 0.0
        # Сумма из средств Субъекта РФ (статья 6 – иные расходы)
        row['STOIM_R']= 0.0
        # Сумма доплаты из средств Федерального фонда ОМС
        row['STOIM_F'] = 0.0
        # Код медицинского сотрудника, оказавшего услугу
        row['MED_KOD'] =  personCode
        # Специальность врача, оказавшего услугу/прием
        row['MEDSPECOMS'] = forceString(record.value('specialityCode'))
         # Уровень оказанной помощи
        row['LEVEL'] = ''
        row.store()
