# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""Импорт организаций из XML"""

import csv
import os
import re

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import (Qt, pyqtSignature, SIGNAL, QString, QVariant, QFile,
                          QTextStream)

from library.Utils import forceString, forceInt, getPrefString, setPref

from XmlReader import CXmlStreamReader
from Ui_Control import Ui_ProgressDialog


class CControlDialog(QtGui.QDialog, Ui_ProgressDialog):
    __pyqtSignals__ = ('startWork()', )

    requiredTags = {
        'ZGLV': ('VERSION', 'DATA', 'FILENAME', 'SD_Z'),
        'SCHET': ('CODE', 'CODE_MO', 'YEAR', 'MONTH', 'NSCHET', 'DSCHET',
            'SUMMAV'),
        'ZAP': ('N_ZAP', 'PR_NOV', 'PACIENT', 'SLUCH'),
        'PACIENT': ('ID_PAC', 'VPOLIS', 'NPOLIS', 'NOVOR'),
        'SLUCH': ('IDCASE', 'USL_OK', 'VIDPOM', 'FOR_POM', 'PODR', 'PROFIL',
            'DET', 'NHISTORY', 'DATE_1', 'DATE_2', 'DS1', 'RSLT', 'ISHOD',
            'PRVS', 'IDDOKT', 'IDSP', 'TARIF', 'SUMV'),
        'USL': ('IDSERV', 'LPU', 'PODR', 'PROFIL', 'DET', 'DATE_IN', 'DATE_OUT',
            'DS', 'CODE_USL', 'KOL_USL', 'TARIF', 'SUMV_USL', 'PRVS', 'CODE_MD')
    }

    requiredTagsHealthExamination = {
        'ZGLV': ('VERSION', 'DATA', 'FILENAME', 'SD_Z'),
        'SCHET': ('CODE', 'CODE_MO', 'YEAR', 'MONTH', 'NSCHET', 'DSCHET',
            'SUMMAV'),
        'ZAP': ('N_ZAP', 'PR_NOV', 'PACIENT'),
        'PACIENT': ('ID_PAC', 'VPOLIS', 'NPOLIS'),
        'SLUCH': ('IDCASE', 'VIDPOM', 'LPU', 'VBR', 'NHISTORY', 'P_OTK',
            'DATE_1', 'DATE_2', 'DS1', 'RSLT_D', 'IDSP', 'TARIF', 'SUMV'),
        'DS2_N': ('DS2', ),
        'USL': ('IDSERV', 'LPU', 'DATE_IN', 'DATE_OUT', 'P_OTK', 'CODE_USL',
            'TARIF', 'SUMV_USL', 'PRVS', 'CODE_MD')
    }

    medicalAidTypeFields = ('PODR', 'PROFIL', 'PRVS', 'VIDPOM')
    visitCountFields = ('code_usl', 'minAmount', 'maxAmount')
    serviceSpecialityFields = ('CODE_USL', 'PRVS')
    resultFields = ('TYPE_DISP','RLST_D')
    mkbZ02xx = ('Z02.3', 'Z02.7', 'Z04.8')

    modeNormal = 0
    modeHealthExamination = 1

    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self._parent = parent
        self.setupUi(self)

        self.start()
        self.connect(self, SIGNAL('startWork()'), self.startWork,
                Qt.QueuedConnection)

        self.nAdded = 0
        self.nProcessed = 0
        self.nUpdated = 0

        self._setCount(self.prbControl, 0)
        self._setCount(self.prbVisitCount, 0)
        self._setCount(self.prbMedicalAidType, 0)

        self.checkRequiredTags = False
        self.checkEventTariff = False
        self.checkMedicalAidType = False
        self.checkResults = False
        self.checkVisitCount = False
        self.checkHealthExaminationRequiredTags = False
        self.checkMkbDots = False
        self.checkMkbZ = False
        self.checkMkbByFilter = False
        self.checkMkbNoDots = False
        self.checkMkbZ02xx = False
        self.checkPolicy = False

        self._mode = self.modeNormal
        self._mkbPattern = re.compile('.*\.\w{2}.*')

        self._reMkbNoDots = re.compile('^\D\d\d$')
        self._rePolicySerial = re.compile(u'^[а-яА-Я]+$')
        self._rePolicyNumberAny = re.compile('\d+')
        self._rePolicyNumber9 = re.compile('^\d{9}$')
        self._rePolicyNumber16 = re.compile('^\d{16}$')


    def _setCount(self, progressBar, cnt):
        if cnt:
            progressBar.setValue(0)
            progressBar.setMaximum(cnt)
            progressBar.setEnabled(True)
            progressBar.setTextVisible(True)
        else:
            progressBar.setValue(0)
            progressBar.setMaximum(1)
            progressBar.setEnabled(False)
            progressBar.setTextVisible(False)


    def start(self):
        self.processDone = False
        self.aborted = False
        self.setExportMode(True)


    def stop(self):
        self.processDone = True
        self.setExportMode(False)


    def done(self, r):
        self.stop()
        QtGui.QDialog.done(self, r)


    def exec_(self):
        self.emit(SIGNAL('startWork()'))

        return QtGui.QDialog.exec_(self)


    def setCheckMode(self, mode):
        self._mode = mode


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        if self.processDone:
            self.accept()
        else:
            self.aborted = True


    @pyqtSignature('')
    def on_btnLogSave_clicked(self):
        prefs =  QtGui.qApp.preferences.appPrefs

        fileName = QtGui.QFileDialog.getSaveFileName(
            self, u'Укажите файл с данными',
            getPrefString(prefs, 'logFileName', ''),
            u'Текстовые файлы (*.txt)')

        if fileName != '':
            outFile = QFile(fileName)

            if not outFile.open(QFile.WriteOnly | QFile.Text):
                QtGui.QMessageBox.warning(self, u'Экспорт жунала',
                    u'Не могу открыть файл для записи {fName}:\n'
                    u'{err}.'.format(fName=fileName, err=outFile.errorString()))
            txtStream = QTextStream(outFile)
            txtStream.setCodec('utf-8')
            txtStream << self.logBrowser.toPlainText()
            outFile.close()
            setPref(prefs, 'logFileName', QVariant(fileName))


    def startWork(self):
        self.start()
        QtGui.qApp.call(self, self.work)
        self.stop()


    def work(self):
        u"""Запускает рабочую функцию диалога"""
        self.processXml(self.prbControl)


    def setExportMode(self, enabled=True):
        u'Переключает состояние кнопок'
        self.btnCancel.setText(u'Отмена' if enabled else u'Закрыть')
        self.btnLogSave.setEnabled(not enabled)

# ******************************************************************************

    def processXml(self, progressBar):
        u"""Разбирает указанный XML файл, отправляет данные в БД"""
        fileName = forceString(self._parent.edtFileName.text())

        if not fileName:
            return False

        progressBar.setFormat(u'%v байт')
        inFile = QtCore.QFile(fileName)

        if not inFile.open(QtCore.QFile.ReadOnly | QtCore.QFile.Text):
            QtGui.QMessageBox.warning(self, u'Импорт страховых компаний',
                  QString(u'Не могу открыть файл для чтения %1:\n%2.')
                  .arg(fileName)
                  .arg(inFile.errorString()))
            return False

        progressBar.setMaximum(max(inFile.size(), 1))
        reader = CXmlStreamReader(self, self.logBrowser, True)
        reader.setDevice(inFile)
        params = {}

        if self._mode == self.modeNormal:
            if self.checkMedicalAidType:
                params['medicalAidTypeRb'] = self.loadMedicalAidTypeRb()

            if self.checkVisitCount:
                params['visitCountRb'] = self.loadVisitCountRb()

            if self.checkMkbByFilter:
                params['mkbFilterRb'] = self.loadMkbFilterRb()

            if self.checkMkbNoDots:
                params['mkbExceptionsRb'] = self.loadMkbExceptionsRb()
        else:
            if self.checkHealthExaminationServiceSpeciality:
                params['serviceSpecialityRb'] = self.loadServiceSpecialityRb()

            if self.checkHealthExaminationResult:
                params['resultRb'] = self.loadResultRb()

        if reader.readHeader():
            for name, row in reader.readData():
                progressBar.setValue(inFile.pos())
                QtGui.qApp.processEvents()

                if row:
                    self.processXmlRow(row, name, params)

                if self.aborted:
                    break

        self.log(u'контроль элементов: %d обработано' % self.nProcessed)

        if not (reader.hasError() or self.aborted):
            self.log(u'Готово')
            return True
        else:
            self.log(u'Прервано')

            if self.aborted:
                self.log(u'! Прервано пользователем.')
            else:
                self.log(u'! Ошибка: файл %s, %s' % (fileName,
                    reader.errorString()))

            return False


    def log(self, msg):
        self.logBrowser.append(msg)
        self.logBrowser.update()


    def processXmlRow(self, row, name, params):
        self.nProcessed += 1

        if self._mode == self.modeNormal:
            if self.checkRequiredTags:
                for err, tags, vals in self.processRequiredTags(row, name,
                        self.requiredTags):
                    if tags:
                        self.log(u'Пропущен тег %s: %s' % (
                                 ','.join(tags), err))
                    if vals:
                        self.log(u'Отсутствуют данные в обязательном '
                                 u'теге %s: %s' % (','.join(vals), err))

            if self.checkResults:
                for err in self.processResults(row, name):
                    self.log(err)

            if self.checkMedicalAidType:
                for err in self.processMedicalAidType(
                        row, name, params['medicalAidTypeRb']):
                    self.log(err)

            if self.checkEventTariff:
                for err in self.processEventTariff(row, name):
                    self.log(err)

            if self.checkVisitCount:
                for err in self.processVisitCount(
                        row, name, params['visitCountRb']):
                    self.log(err)

            if self.checkMkbDots or self.checkMkbZ or self.checkMkbZ02xx:
                for err in self.processMkb(row, name,
                        self.checkMkbDots, self.checkMkbZ, self.checkMkbZ02xx):
                    self.log(err)

            if self.checkMkbByFilter:
                for err in self.processMkbByFilter(row, name,
                        params['mkbFilterRb']):
                    self.log(err)

            if self.checkMkbNoDots:
                for err in self.processMkbNoDots(row, name,
                                                 params['mkbExceptionsRb']):
                    self.log(err)

            if self.checkPolicy:
                for err in self.processPolicy(row, name):
                    self.log(err)
        else:
            if self.checkHealthExaminationRequiredTags:
                for err, tags, vals in self.processRequiredTags(row, name,
                        self.requiredTagsHealthExamination):
                    if tags:
                        self.log(u'Пропущен тег %s: %s' % (','.join(tags), err))
                    if vals:
                        self.log(u'Пропущено значение %s: %s' % (','.join(vals),
                                                                 err))

            if self.checkHealthExaminationResult:
                for err in self.processHealthExaminationResult(row, name,
                        params['resultRb'], params):
                    self.log(err)

            if self.checkHealthExaminationServiceSpeciality:
                for err in self.processHealthExaminationServiceSpeciality(row,
                        name, params['serviceSpecialityRb']):
                    self.log(err)


    def processRequiredTags(self, row, name, tagList, errPrefix=''):
        u"""Возвращает множество найденных обязательных тэгов в записи"""

        def describeError():
            err = u'-'

            if name == 'ZAP':
                err = u'N_ZAP %s' % row.get('N_ZAP', '?')
            elif name == u'PACIENT':
                err = u'ID_PAC %s' % row.get('ID_PAC', '?')
            elif name == u'SLUCH':
                err = u'SLUCH %s' % row.get('IDCASE', '?')
            elif name == u'USL':
                err = u'IDSERV %s' % row.get('IDSERV', '?')

            return (u'%s %s' % (errPrefix, err)).strip()

        presentTags = set()
        absentVals = set()
        subresult = []
        requiredTags = set(tagList.get(name, tuple()))

        for key, val in row.iteritems():
            if key in requiredTags:
                presentTags.add(key)

                if val == '':
                    absentVals.add(key)

            if isinstance(val, dict):
                subresult.extend(
                    self.processRequiredTags(val, key, tagList, describeError()))
            elif isinstance(val, list):
                for item in val:
                    subresult.extend(
                        self.processRequiredTags(item, key, tagList, describeError()))

        absentTags = requiredTags.difference(presentTags)
        result = ([(describeError(), absentTags, absentVals), ]
                  if absentTags or absentVals else [])
        result.extend(subresult)
        return result


    def processResults(self, row, name):
        u"""Проверяет значения тегов sluch.RSLT и sluch.ISHOD"""

        def checkResults(e):
            if e.get('USL_OK') in ('1', '2', '3', '4'):
                uslOk = e.get('USL_OK')
                rslt = e.get('RSLT', '')
                ishod = e.get('ISHOD', '')

                if (len(rslt) != 3 or rslt[0] != uslOk or
                        len(ishod) != 3 or ishod[0] != uslOk):
                    result.append(u'IDCASE %s: в теге RSLT (или ISHOD) '
                        u'некорректное значение при указанном USL_OK' % (
                        e.get('IDCASE', '?')))

        result = []

        if name == 'ZAP':
            event = row.get('SLUCH', {})

            if isinstance(event, list):
                for i in event:
                    checkResults(i)
            else:
                checkResults(event)

        return result


    def processEventTariff(self, row, name):
        u"""Проверка тарификации более 1 услуги в событии"""

        def checkEventTariff(e):
            codes = set()

            if e.get('IDSP') == '9':
                return

            if e.get('USL_OK') == '3' and isinstance(e.get('USL'), list):
                for service in e['USL']:
                    codes.add(service.get('CODE_USL'))

            if len(codes) > 1:
                result.append(u'IDCASE %s: Тарификация более 1 услуги в '
                    u'событии' % e.get('IDCASE', '?'))

        result = []

        if name == 'ZAP':
            event = row.get('SLUCH', {})

            if isinstance(event, list):
                for i in event:
                    checkEventTariff(i)
            else:
                checkEventTariff(event)
        return result


    def processMedicalAidType(self, row, name, rb):
        result = []
        eventId = ''

        def checkMedicalAidType(e, idCase):
            orgStruct = e.get('PODR')

            if rb.has_key(orgStruct):
                isOk = False

                for profile, code, aidType in rb[orgStruct]:
                    for _list, key in ((profile, 'PROFIL'), (code, 'PRVS'),
                            (aidType, 'VIDPOM')):
                        if _list and e.get(key, '') not in _list:
                            break
                    else:
                        isOk = True # нет break, значит есть совпадение

                    if isOk: #нашли совпадение, все хорошо, дальше не ищем
                        break

                else:
                    result.append(u'IDCASE %s: Несоответствие значений '
                        u'Подр.-Проф.-Спец.-Вид МП' % idCase)

            else:
                result.append(
                    u'PODR=`%s` отсутствует в справочнике' % orgStruct)

        if name == 'ZAP':
            event = row.get('SLUCH', {})

            if not isinstance(event, list):
                event = [event]

            for e in event:
                eventId = e.get('IDCASE', '?')
                checkMedicalAidType(e, eventId)
                service = e.get('USL', {})

                if isinstance(service, list):
                    for s in service:
                        s['VIDPOM'] = e.get('VIDPOM')
                        checkMedicalAidType(s, eventId)
                else:
                    service['VIDPOM'] = e.get('VIDPOM')
                    checkMedicalAidType(service, eventId)

        return result


    def processVisitCount(self, row, name, rb):
        result = []

        if name == 'ZAP':
            event = row.get('SLUCH', {})
            if not isinstance(event, list):
                event = [event]

            for e in event:
                service = e.get('USL', {})

                if e.get('USL_OK') != '3' or e.get('IDSP') == '9':
                    # выкидываем стоматологию (IDSP = 9)
                    continue

                if not isinstance(service, list):
                    service = [service]

                code = service[0].get('CODE_USL', '')

                if code:
                    serviceCount = len(service)
                    _min, _max = rb.get(code, (0, 999))

                    if _min > serviceCount or serviceCount > _max:
                        result.append(
                            u'IDCASE %s: Некорректное количество визитов '
                            u'в событии' % e.get('IDCASE', '?'))
                        result.append(
                            u'CODE_USL=`%s`: %d шт, min=%d, max=%d' % (
                            code, serviceCount, _min, _max))

        return result


    def processHealthExaminationResult(self, row, name, rb, params):
        result = []

        if name == 'SCHET':
            params['DISP'] = row.get('DISP')

        if name == 'ZAP':
            event = row.get('SLUCH', {})
            code = params.get('DISP')

            if not isinstance(event, list):
                event = [event]

            for e in event:
                val = e.get('RSLT_D')

                if rb.has_key(code) and val not in rb[code]:
                    result.append(u'IDCASE {idcase} некорректный '
                        u'результат диспансеризации'.format(
                        idcase=e.get('IDCASE', '?')))

        return result


    def processHealthExaminationServiceSpeciality(self, row, name, rb):
        result = []

        if name == 'ZAP':
            event = row.get('SLUCH', {})
            if not isinstance(event, list):
                event = [event]

            for e in event:
                service = e.get('USL', {})

                if not isinstance(service, list):
                    service = [service]

                for srv in service:
                    code = srv.get('CODE_USL')
                    val = srv.get('PRVS')

                    if rb.has_key(code) and val not in rb[code]:
                        result.append(u'IDCASE {idcase} IDSERV {idserv} '
                            u'cпециальность не соответствует услуге'.format(
                            idcase=e.get('IDCASE', '?'),
                            idserv=srv.get('IDSERV', '?')))

        return result


    def processMkb(self, row, name, checkMkbDots, checkMkbZ, checkMkbZ02xx):
        u""" Проверяет:
            1. применение диагнозов с 2-я знаками после точки (кроме диагнозов
            категории S)" При установленном флаге в данном пункте необходимо
            проверять реестр на наличие случаев лечения, у которых в тегах
            sluch.DS1, sluch.DS2, sluch.DS3, sluch.DS0, usl.DS стоят значения,
            у которых после символа "." (точка) стоят 2 символа (исключением
            являются диагнозы, начинающиеся на символ "S"). Если Диагноз не
            начинается на S и после точки 2 символа - это считается ошибкой и
            необходимо вывести сообщение об ошибке: IDCASE [значение IDCASE] -
            указан диагноз с 2 символами после точки.

            2. "События с диагнозами Z" Необходимо пройтись по всем sluch и если
            в теге sluch.DS1 стоит значение, начинающееся на символ 'Z' - считай
            его ошибкой и выводи сообщение об ошибке: IDCASE [значение IDCASE] -
            диагноз [значение тега DS1] """

        result = []

        if name == 'ZAP':
            event = row.get('SLUCH', {})
            if not isinstance(event, list):
                event = [event]

            for e in event:
                eventId = e.get('IDCASE', '?')

                if checkMkbDots:
                    for key in ('DS0', 'DS1', 'DS2', 'DS3', 'DS'):
                        if e.has_key(key) and self._mkbPattern.match(e[key]):
                            result.append(u'IDCASE {eventId} - указан диагноз с'
                                u' 2 символами после точки {mkb}'.format(
                                    eventId=eventId, mkb=e[key]))

                if ((checkMkbZ and e.get('DS1', '')[:1] == 'Z')
                    or (checkMkbZ02xx and e.get('DS1', '') in self.mkbZ02xx)):
                    result.append(u'IDCASE {eventId} - диагноз {mkb}'.format(
                        eventId=eventId, mkb=e['DS1']
                    ))

        return result


    def processMkbByFilter(self, row, name, mkbMask):
        u"""При проставленном флаге утилита будет смотреть в реестре в тегах
            sluch.DS0, sluch.DS1, sluch.DS2, sluch.DS3, usl.DS значения. Если
            значение какого-либо из данных тегов будет совпадать хотя бы с 1
            строкой справочника, выводит сообщение об ошибке."""
        result = []

        if name == 'ZAP':
            event = row.get('SLUCH', {})
            if not isinstance(event, list):
                event = [event]

            for e in event:
                eventId = e.get('IDCASE', '?')

                for key in ('DS0', 'DS1', 'DS2', 'DS3', 'DS'):
                    if e.has_key(key) and mkbMask.match(e[key]):
                        result.append(u'IDCASE {eventId} применен диагноз'
                            u' {mkb}'.format(eventId=eventId, mkb=e[key]))

        return result


    def processMkbNoDots(self, row, name, exceptionList):
        u'Сообщает об МКБ из трех символов с учетом исключений.'
        result = []

        if name == 'ZAP':
            event = row.get('SLUCH', {})
            if not isinstance(event, list):
                event = [event]

            for e in event:
                eventId = e.get('IDCASE', '?')

                for key in ('DS0', 'DS1', 'DS2', 'DS3', 'DS'):
                    if (e.has_key(key) and self._reMkbNoDots.match(e[key])
                            and e[key] not in exceptionList):
                        result.append(u'IDCASE {eventId} применен диагноз'
                            u' {mkb}'.format(eventId=eventId, mkb=e[key]))

        return result

    def processPolicy(self, row, name):
        u"""Если тег <VPOLIS> содержит 1, тогда тег <SPOLIS> должен
        содержать только буквы (могут быть и прописные и заглавные,
        но только кириллица).
        тег <NPOLIS> должен содержать только цифры.

        Если тег <VPOLIS> содержит 2, тогда
        тег <SPOLIS> должен быть пустым
        тег <NPOLIS> должен содержать только цифры. Цифр должно быть 9.

        Если тег <VPOLIS> содержит 3, тогда
        тег <SPOLIS> должен быть пустым
        тег <NPOLIS> должен содержать только цифры. Цифр должно быть 16."""
        result = []

        if name == 'ZAP':
            clientInfo = row.get('PACIENT', {})
            policyKind = forceInt(clientInfo.get('VPOLIS', 0))
            clientId = clientInfo.get('ID_PAC', '?')
            serial = clientInfo.get('SPOLIS', '')
            number = clientInfo.get('NPOLIS', '')
            badPolicy = ((policyKind == 1 and
                          not (self._rePolicySerial.match(serial) and
                               self._rePolicyNumberAny.match(number))) or
                         (policyKind == 2 and
                          not (not serial and
                               self._rePolicyNumber9.match(number))) or
                         (policyKind == 3 and
                          not (not serial and
                               self._rePolicyNumber16.match(number))))

            if badPolicy:
                result.append(u'ID_PAC {clientId} ошибка в реквизитах'
                              u'полиса'.format(clientId=clientId))

        return result

# ******************************************************************************

    def loadMedicalAidTypeRb(self):
        u"""Разбирает указанный CSV файл, отправляет данные в БД"""
        fileName = forceString(self._parent.edtMedicalAidTypeRbFileName.text())
        progressBar = self.prbMedicalAidType
        progressBar.setMaximum(os.path.getsize(fileName))
        progressBar.setFormat('%p%')
        progressBar.setTextVisible(True)
        progressBar.setEnabled(True)

        self.log(u'Импорт данных из `%s`' % fileName)
        result = {}

        with open(fileName) as csvfile:
            reader = UnicodeDictReader(csvfile, self.medicalAidTypeFields,
                restkey='unused', restval='')
            reader.next() # пропускаем заголовок

            for row in reader:
                progressBar.setValue(csvfile.tell())
                orgStruct = row.get('PODR', '').strip()
                data = (row.get('PROFIL', '').strip().split('/'),
                    row.get('PRVS', '').strip().split('/'),
                    row.get('VIDPOM', '').strip().split('/'))
                result.setdefault(orgStruct, []).append(data)
                QtGui.qApp.processEvents()

        self.log(u'Импорт завершен')
        progressBar.setMaximum(1)
        progressBar.setValue(1)
        return result

# ******************************************************************************

    def loadVisitCountRb(self):
        u"""Разбирает указанный CSV файл, отправляет данные в БД"""
        fileName = forceString(self._parent.edtVisitCountRbFileName.text())
        progressBar = self.prbVisitCount
        progressBar.setMaximum(os.path.getsize(fileName))
        progressBar.setFormat('%p%')
        progressBar.setTextVisible(True)
        progressBar.setEnabled(True)

        self.log(u'Импорт данных из `%s`' % fileName)
        result = {}

        with open(fileName) as csvfile:
            reader = UnicodeDictReader(csvfile, self.visitCountFields,
                restkey='unused', restval='')
            reader.next() # пропускаем заголовок

            for row in reader:
                progressBar.setValue(csvfile.tell())
                _min = row.get('minAmount')
                if not _min:
                    _min = 0

                _max = row.get('maxAmount')
                if not _max:
                    _max = 0

                result[row.get('code_usl', '').strip()] = (
                    forceInt(_min), forceInt(_max))
                QtGui.qApp.processEvents()

        self.log(u'Импорт завершен')
        progressBar.setMaximum(1)
        progressBar.setValue(1)
        return result

# ******************************************************************************

    def loadSimpleRb(self, edt, fields, code, name):
        u"""Разбирает указанный CSV файл, отправляет данные в БД"""
        fileName = forceString(edt.text())
        progressBar = self.prbVisitCount
        progressBar.setMaximum(os.path.getsize(fileName))
        progressBar.setFormat('%p%')
        progressBar.setTextVisible(True)
        progressBar.setEnabled(True)

        self.log(u'Импорт данных из `%s`' % fileName)
        result = {}

        with open(fileName) as csvfile:
            reader = UnicodeDictReader(csvfile, fields, restkey='unused',
                restval='')
            reader.next() # пропускаем заголовок

            for row in reader:
                progressBar.setValue(csvfile.tell())
                _code = row.get(code, '').strip()
                _val = row.get(name, '').strip()
                result.setdefault(_code, set()).add(_val)
                QtGui.qApp.processEvents()

        self.log(u'Импорт завершен')
        progressBar.setMaximum(1)
        progressBar.setValue(1)
        return result

# ******************************************************************************

    def loadServiceSpecialityRb(self):
        u"""Разбирает указанный CSV файл, отправляет данные в БД"""
        return self.loadSimpleRb(self._parent.edtServiceSpecialityRbFileName,
            self.serviceSpecialityFields, 'CODE_USL', 'PRVS')

# ******************************************************************************

    def loadResultRb(self):
        u"""Разбирает указанный CSV файл, отправляет данные в БД"""
        return self.loadSimpleRb(
            self._parent.edtHealthExaminationResultRbFileName,
            self.resultFields, 'TYPE_DISP', 'RLST_D')

# ******************************************************************************

    def loadMkbFilterRb(self):
        u"""Разбирает указанный CSV файл, отправляет данные в БД"""
        fileName = forceString(self._parent.edtMkbFilterRbFileName.text())
        progressBar = self.prbVisitCount
        progressBar.setMaximum(os.path.getsize(fileName))
        progressBar.setFormat('%p%')
        progressBar.setTextVisible(True)
        progressBar.setEnabled(True)

        self.log(u'Импорт данных из `%s`' % fileName)
        result = set()

        with open(fileName) as csvfile:
            reader = UnicodeDictReader(csvfile, ('diagnosis', ), restkey='unused',
                restval='')
            reader.next() # пропускаем заголовок

            for row in reader:
                progressBar.setValue(csvfile.tell())
                result.add(row.get('diagnosis', '').strip())
                QtGui.qApp.processEvents()

        self.log(u'Импорт завершен')
        progressBar.setMaximum(1)
        progressBar.setValue(1)
        mask = '^({0}).*'.format('|'.join(re.escape(x) for x in result))
        result = re.compile(mask)
        return result

# ******************************************************************************

    def loadMkbExceptionsRb(self):
        u"""Разбирает указанный CSV файл, отправляет данные в БД"""
        fileName = forceString(self._parent.edtMkbExceptionsFileName.text())
        progressBar = self.prbVisitCount
        progressBar.setMaximum(os.path.getsize(fileName))
        progressBar.setFormat('%p%')
        progressBar.setTextVisible(True)
        progressBar.setEnabled(True)

        self.log(u'Импорт данных из `%s`' % fileName)
        result = set()

        with open(fileName) as csvfile:
            reader = UnicodeDictReader(csvfile, ('diagnosis', ), restkey='unused',
                restval='')
            reader.next() # пропускаем заголовок

            for row in reader:
                progressBar.setValue(csvfile.tell())
                result.add(row.get('diagnosis', '').strip())
                QtGui.qApp.processEvents()

        self.log(u'Импорт завершен')
        progressBar.setMaximum(1)
        progressBar.setValue(1)
        return result

# ******************************************************************************

def UnicodeDictReader(csvfile, fieldnames=None, restkey=None, restval=None, dialect='excel', *args, **kwds):
    csv_reader = csv.DictReader(csvfile, fieldnames, restkey, restval, dialect, *args, **kwds)
    for row in csv_reader:
        yield dict((unicode(key, 'utf-8'), unicode(value, 'utf-8')) for key, value in row.iteritems())

# ******************************************************************************
