#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


import codecs
import gc
import locale
import os
import os.path
import shutil
import sys
import tempfile
import traceback
from optparse import OptionParser

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import (pyqtSignature, QDir, qInstallMsgHandler, QVariant, Qt,
    QDateTime)


from Users.UserInfo      import CUserInfo, CDemoUserInfo
from Users.Tables        import (tblUser, usrLogin, usrPassword, usrRetired,
    demoUserName)
from library             import database
from library.Preferences import CPreferences
from library.Utils       import (getPref, getPrefString, getPrefBool,
    getPrefInt, setPref, forceString, anyToUnicode, forceBool, forceRef)

from Control              import CControlDialog
from Ui_logicalControl     import Ui_MainWindow

gTitle = u'САМСОН'
gTitleLat = 'SAMSON'
gVersion = '2.5'
gAbout = u'Комплекс Программных Средств \n' \
        u'"Система Автоматизации Медико-Страхового Обслуживания Населения"\n' \
        u'КПС «%s»\n'   \
        u'Версия %s (ревизия %s от %s)\n' \
        u'Copyright © 2015 ООО "САМСОН Групп"\n' \
        u'распространяется под лицензией GNU GPL v.3 или выше\n' \
        u'Основано на КПС «САМСОН-ВИСТА» версии 2.0\n' \
        u'телефон тех.поддержки: (812) 418-39-70'


class CMyApp(QtGui.QApplication):
    __pyqtSignals__ = ('dbConnectionChanged(bool)',
                       'currentUserIdChanged()',
                      )

    def __init__(self, args, demoModeRequest):
        QtGui.QApplication.__init__(self, args)
        self.logDir = os.path.join(unicode(QDir.toNativeSeparators(QDir.homePath())), '.samson-vista')
        self.oldMsgHandler = qInstallMsgHandler(self.msgHandler)
        self.oldEexceptHook = sys.excepthook
        sys.excepthook = self.logException
        self.traceActive = True
        self.homeDir= None
        self.saveDir= None
        self.userId = None
        self.userSpecialityId = None
        self.userOrgStructureId = None
        self.db     = None
        self.mainWindow = None
        self.preferences = CPreferences('r60control.ini')
        self.preferences.load()
        self.preferencesDir = self.preferences.getDir()
        self.registerDocumentTables()
        self._currentClientId = None
        self.userInfo = None
        self.demoModeRequested = True


    def doneTrace(self):
        self.traceActive = False
        qInstallMsgHandler(self.oldMsgHandler)
        sys.excepthook = self.oldEexceptHook


    def applyDecorPreferences(self):
        self.setStyle(self.preferences.decorStyle)
        if self.preferences.decorStandardPalette:
            self.setPalette(self.style().standardPalette())
        if self.mainWindow:
            state = Qt.WindowNoState
            if self.preferences.decorMaximizeMainWindow:
                state |= Qt.WindowMaximized
            if self.preferences.decorFullScreenMainWindow:
                state |= Qt.WindowFullScreen
            self.mainWindow.setWindowState(state)


    def registerDocumentTables(self):
        database.registerDocumentTable('Account')
        database.registerDocumentTable('Action')
        database.registerDocumentTable('ActionProperty')
        database.registerDocumentTable('ActionPropertyTemplate')
        database.registerDocumentTable('ActionTemplate')
        database.registerDocumentTable('ActionType')
        database.registerDocumentTable('Address')
#        database.registerDocumentTable('AddressAreaItem')
        database.registerDocumentTable('AddressHouse')
        database.registerDocumentTable('Bank')
        database.registerDocumentTable('CalendarExceptions')
        database.registerDocumentTable('Client')
        database.registerDocumentTable('ClientAddress')
        database.registerDocumentTable('ClientAllergy')
        database.registerDocumentTable('ClientAttach')
        database.registerDocumentTable('ClientContact')
        database.registerDocumentTable('ClientIdentification')
        database.registerDocumentTable('ClientIntoleranceMedicament')
        database.registerDocumentTable('ClientDocument')
        database.registerDocumentTable('ClientPolicy')
        database.registerDocumentTable('ClientSocStatus')
        database.registerDocumentTable('ClientWork')
        database.registerDocumentTable('Contract')
        database.registerDocumentTable('Diagnosis')
        database.registerDocumentTable('Diagnostic')
        database.registerDocumentTable('Event')
        database.registerDocumentTable('EventType')
        database.registerDocumentTable('InformerMessage')
        database.registerDocumentTable('Licence')
        database.registerDocumentTable('Organisation')
        database.registerDocumentTable('OrgStructure')
        database.registerDocumentTable('Person')
        database.registerDocumentTable('TempInvalid')
        database.registerDocumentTable('Visit')


    def getHomeDir(self):
        if not self.homeDir:
            homeDir = os.path.expanduser('~')
            if homeDir == '~':
                homeDir = unicode(QDir.homePath())
            if isinstance(homeDir, str):
                homeDir = unicode(homeDir, locale.getpreferredencoding())
            self.homeDir = homeDir
        return self.homeDir


    def getSaveDir(self):
        self.saveDir = forceString(self.preferences.appPrefs.get('saveDir', QVariant()))
        if not self.saveDir:
            self.saveDir = self.getHomeDir()
        return self.saveDir


    def setSaveDir(self, path):
#        mode = os.stat(path)[ST_MODE]
#        if if S_ISDIR(mode):
        saveDir = os.path.dirname(unicode(path))
        if saveDir and self.saveDir != saveDir:
            self.preferences.appPrefs['saveDir'] = QVariant(saveDir)


    def getTemplateDir(self):
        result = forceString(self.preferences.appPrefs.get('templateDir', None))
        if not result:
            result = os.path.join(self.logDir, 'templates')
        return result


    def getTmpDir(self, suffix=''):
        return unicode(tempfile.mkdtemp('','samson_%s_'%suffix), locale.getpreferredencoding())


    def removeTmpDir(self, dir):
        try:
            shutil.rmtree(dir, False)
        except:
            self.logCurrentException()


    def log(self, title, message, stack=None):
        try:
            if not os.path.exists(self.logDir):
                os.makedirs(self.logDir)
            logFile = os.path.join(self.logDir, 'error.log')
            timeString = unicode(QDateTime.currentDateTime().toString(Qt.SystemLocaleDate))
            logString = u'%s\n%s: %s(%s)\n' % ('='*72, timeString, title, message)
            if stack:
                try:
                    logString += ''.join(traceback.format_list(stack)).decode('utf-8') + '\n'
                except:
                    logString += 'stack lost\n'
            file = codecs.open(logFile, mode='a', encoding=locale.getpreferredencoding())
            file.write(logString)
            file.close()
        except:
            pass


    def logException(self, exceptionType, exceptionValue, exceptionTraceback):
        title = repr(exceptionType)
        message = unicode(exceptionValue)
        self.log(title, message, traceback.extract_tb(exceptionTraceback))
        sys.__excepthook__(exceptionType, exceptionValue, exceptionTraceback)


    def logCurrentException(self):
        self.logException(*sys.exc_info())


    def msgHandler(self, type, msg):
        if type == 0: # QtMsgType.QtDebugMsg:
            typeName = 'QtDebugMsg'
        elif type == 1: # QtMsgType.QtWarningMsg:
            typeName = 'QtWarningMsg'
        elif type == 2: # QtMsgType.QtCriticalMsg:
            typeName = 'QtCriticalMsg'
        elif type == 3: # QtFatalMsg
            typeName = 'QtFatalMsg'
        else:
            typeName = 'QtUnknownMsg'

        self.log( typeName, msg, traceback.extract_stack()[:-1])
#        print typeName, msg


    def openDatabase(self):
        self.db = None
        self.db = database.connectDataBase(self.preferences.dbDriverName,
                                     self.preferences.dbServerName,
                                     self.preferences.dbServerPort,
                                     self.preferences.dbDatabaseName,
                                     self.preferences.dbUserName,
                                     self.preferences.dbPassword)
        self.emit(QtCore.SIGNAL('dbConnectionChanged(bool)'), True)


    def closeDatabase(self):
        if self.db:
            self.db.close()
            self.db = None
            self.emit(QtCore.SIGNAL('dbConnectionChanged(bool)'), False)


    def notify(self, receiver, event):
        try:
#            report = ["notify:: receiver.type=",unicode(type(receiver))," receiver.name=", unicode(receiver.objectName())," event.type=",unicode(type(event)), "event.type()=", unicode(event.type())]
#            print >>sys.stderr, ("".join(report)).encode('cp866')
            return QtGui.QApplication.notify(self, receiver, event)
        except Exception, e:
            if self.traceActive:
                self.logCurrentException()
                widget = self.activeModalWidget()
                if widget is None:
                    widget = self.mainWindow
                QtGui.QMessageBox.critical( widget,
                                            u'Произошла ошибка',
                                            unicode(e),
                                            QtGui.QMessageBox.Close)
            return False
        except:
            return False


    def call(self, widget, func, params = ()):
        try:
            return True, func(*params)
        except IOError, e:
            msg=''
            if hasattr(e, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno, anyToUnicode(e.strerror))
            else:
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            QtGui.QMessageBox.critical(widget,
                u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
        except Exception, e:
            self.logCurrentException()
            widget = widget or self.activeModalWidget() or self.mainWindow
            QtGui.QMessageBox.critical( widget,
                                        u'Произошла ошибка',
                                        unicode(e),
                                        QtGui.QMessageBox.Close)
        return False, None


    def callWithWaitCursor(self, widget, func, *params, **kwparams):
        self.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
        try:
            return func(*params, **kwparams)
        finally:
            self.restoreOverrideCursor()


    def demoModePosible(self):
        try:
            tableUser = self.db.table(tblUser)
            record = self.db.getRecordEx(tableUser, [usrPassword, usrRetired], tableUser[usrLogin].eq(demoUserName))
            return not record or (not forceString(record.value(usrPassword)) and not forceBool(record.value(usrRetired)))
        except:
            pass
        return False


    def setUserId(self, userId, demoMode = False):
        self.userId = userId
        record = self.db.getRecord('Person', ['speciality_id', 'orgStructure_id'], userId) if userId else None
        if record:
            self.userSpecialityId = forceRef(record.value('speciality_id'))
            self.userOrgStructureId = forceRef(record.value('orgStructure_id'))
        else:
            self.userSpecialityId = None
            self.userOrgStructureId = None
        self.demoMode = demoMode
        if demoMode:
            self.userInfo = CDemoUserInfo(userId)
        else:
            if userId:
                self.userInfo = CUserInfo(userId)
            else:
                self.userInfo = None
        self.emit(QtCore.SIGNAL('currentUserIdChanged()'))


    def userName(self):
        if self.userInfo:
#            orgId = self.currentOrgId()
#            orgInfo = getOrganisationInfo(orgId)
#            if not orgInfo:
#                QtGui.qApp.preferences.appPrefs['orgId'] = QVariant()
#            shortName = orgInfo.get('shortName', u'ЛПУ в настройках не указано!')
            return u'%s (%s)' % (self.userInfo.name(), self.userInfo.login())
        else:
            return ''


    def userHasRight(self, right):
        return self.userInfo is not None and self.userInfo.hasRight(right)


    def userHasAnyRight(self, rights):
        return self.userInfo is not None and self.userInfo.hasAnyRight(rights)



    def defaultKLADR(self):
        # возвращает код по умолчанию
        result = forceString(self.preferences.appPrefs.get('defaultKLADR', ''))
        if not result:
            result = '7800000000000'
        return result



# #############################################################


class CMainWindow(QtGui.QMainWindow, Ui_MainWindow):

    maskCsv = u'Файлы CSV (*.csv)'

    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)

        self.loadPreferences()


    def loadPreferences(self):
        preferences = getPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), {})
        geometry = getPref(preferences, 'geometry', None)
        if type(geometry) == QVariant and geometry.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreGeometry(geometry.toByteArray())
        state = getPref(preferences, 'state', None)
        if type(state) == QVariant and state.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreState(state.toByteArray())

        appPreferences = QtGui.qApp.preferences.appPrefs
        self.edtFileName.setText(
                getPrefString(appPreferences, 'fileName', ''))
        self.chkEventTariff.setChecked(
                getPrefBool(appPreferences, 'eventTariff', False))
        self.chkMedicalAidType.setChecked(
                getPrefBool(appPreferences, 'medicalAidType', False))
        self.chkRequiredTags.setChecked(
                getPrefBool(appPreferences, 'requiredTags', False))
        self.chkResults.setChecked(
                getPrefBool(appPreferences, 'results', False))
        self.chkVisitCount.setChecked(
                getPrefBool(appPreferences, 'visitCount', False))
        self.edtVisitCountRbFileName.setText(
                getPrefString(appPreferences, 'visitCountRbFileName', ''))
        self.edtMedicalAidTypeRbFileName.setText(
                getPrefString(appPreferences, 'medicalAidTypeRbFileName', ''))
        self.edtServiceSpecialityRbFileName.setText(
            getPrefString(appPreferences, 'serviceSpecialityRbFileName', ''))
        self.edtHealthExaminationResultRbFileName.setText(getPrefString(
            appPreferences, 'healthExaminationResultRbFileName', ''))
        self.chkHealthExaminationRequiredTags.setChecked(
            getPrefBool(appPreferences, 'healthExaminationRequiredTags', False))
        self.chkHealthExaminationServiceSpeciality.setChecked(getPrefBool(
            appPreferences, 'healthExaminationServiceSpeciality', False))
        self.chkHealthExaminationResult.setChecked(getPrefBool(appPreferences,
            'healthExaminationResult', False))
        self.tabControlType.setCurrentIndex(getPrefInt(
            appPreferences, 'controlTypeIndex', 0))
        self.chkMkbDots.setChecked(getPrefBool(
            appPreferences, 'mkbDots', False))
        self.chkMkbZ.setChecked(getPrefBool(appPreferences, 'mkbZ', False))
        self.chkMkbFilter.setChecked(getPrefBool(
            appPreferences, 'mkbFilter', False))
        self.edtMkbFilterRbFileName.setText(
            getPrefString(appPreferences, 'mkbFilterRbFileName', ''))
        self.chkMkbNoDots.setChecked(getPrefBool(appPreferences, 'mkbNoDots',
                                                 False))
        self.edtMkbExceptionsFileName.setText(
            getPrefString(appPreferences, 'mkbMkbExceptionsFileName', ''))
        self.chkPolicy.setChecked(getPrefBool(appPreferences, 'policy', False))
        self.chkMkbZ02xx.setChecked(getPrefBool(appPreferences, 'mkbZ02xx',
                                                False))
        self.checkFileNames()


    def savePreferences(self):
        preferences = {}
        setPref(preferences,'geometry',QVariant(self.saveGeometry()))
        setPref(preferences,'state', QVariant(self.saveState()))
        setPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), preferences)

        appPreferences = QtGui.qApp.preferences.appPrefs
        setPref(appPreferences, 'fileName',
            QVariant(self.edtFileName.text()))
        setPref(appPreferences, 'eventTariff',
            QVariant(self.chkEventTariff.isChecked()))
        setPref(appPreferences, 'medicalAidType',
            QVariant(self.chkMedicalAidType.isChecked()))
        setPref(appPreferences, 'requiredTags',
            QVariant(self.chkRequiredTags.isChecked()))
        setPref(appPreferences, 'results',
            QVariant(self.chkResults.isChecked()))
        setPref(appPreferences, 'visitCount',
            QVariant(self.chkVisitCount.isChecked()))
        setPref(appPreferences, 'visitCountRbFileName',
            QVariant(self.edtVisitCountRbFileName.text()))
        setPref(appPreferences, 'medicalAidTypeRbFileName',
            QVariant(self.edtMedicalAidTypeRbFileName.text()))
        setPref(appPreferences, 'serviceSpecialityRbFileName',
            QVariant(self.edtServiceSpecialityRbFileName.text()))
        setPref(appPreferences, 'healthExaminationResultRbFileName',
            QVariant(self.edtHealthExaminationResultRbFileName.text()))
        setPref(appPreferences, 'healthExaminationRequiredTags',
            QVariant(self.chkHealthExaminationRequiredTags.isChecked()))
        setPref(appPreferences, 'healthExaminationServiceSpeciality',
            QVariant(self.chkHealthExaminationServiceSpeciality.isChecked()))
        setPref(appPreferences, 'healthExaminationResult',
            QVariant(self.chkHealthExaminationResult.isChecked()))
        setPref(appPreferences, 'controlTypeIndex',
            QVariant(self.tabControlType.currentIndex()))
        setPref(appPreferences, 'mkbDots',
            QVariant(self.chkMkbDots.isChecked()))
        setPref(appPreferences, 'mkbZ', QVariant(self.chkMkbZ.isChecked()))
        setPref(appPreferences, 'mkbFilter',
            QVariant(self.chkMkbFilter.isChecked()))
        setPref(appPreferences, 'mkbFilterRbFileName',
                QVariant(self.edtMkbFilterRbFileName.text()))
        setPref(appPreferences, 'mkbNoDots',
                QVariant(self.chkMkbNoDots.isChecked()))
        setPref(appPreferences, 'mkbMkbExceptionsFileName',
                QVariant(self.edtMkbExceptionsFileName.text()))
        setPref(appPreferences, 'policy', QVariant(self.chkPolicy.isChecked()))
        setPref(appPreferences, 'mkbZ02xx',
                QVariant(self.chkMkbZ02xx.isChecked()))


    def setUserName(self, userName):
        title = u'Форматно-логический контроль для Псковской области'
        if userName:
            self.setWindowTitle(title + u': '+ userName)
        else:
            self.setWindowTitle(title)


    def updateStatus(self):
        pass


    @pyqtSignature('')
    def on_btnControl_clicked(self):
        process = CControlDialog(self)
        process.checkRequiredTags = self.chkRequiredTags.isChecked()
        process.checkEventTariff = self.chkEventTariff.isChecked()
        process.checkMedicalAidType = (self.chkMedicalAidType.isChecked()
            and self.chkMedicalAidType.isEnabled())
        process.checkResults = self.chkResults.isChecked()
        process.checkVisitCount = (self.chkVisitCount.isChecked()
            and self.chkVisitCount.isEnabled())
        process.checkHealthExaminationRequiredTags = (
            self.chkHealthExaminationRequiredTags.isChecked())
        process.checkHealthExaminationServiceSpeciality = (
            self.chkHealthExaminationServiceSpeciality.isChecked())
        process.checkHealthExaminationResult = (
            self.chkHealthExaminationResult.isChecked())
        process.checkMkbDots = self.chkMkbDots.isChecked()
        process.checkMkbZ = self.chkMkbZ.isChecked()
        process.checkMkbByFilter = self.chkMkbFilter.isChecked()
        process.checkMkbNoDots = self.chkMkbNoDots.isChecked()
        process.checkMkbZ02xx = self.chkMkbZ02xx.isChecked()
        process.checkPolicy = self.chkPolicy.isChecked()
        mode = (process.modeNormal if self.tabControlType.currentIndex() == 0
            else process.modeHealthExamination)
        process.setCheckMode(mode)
        QtGui.qApp.call(self, process.exec_)


    def checkFileNames(self):
        self.btnControl.setEnabled(
            ((self.tabControlType.currentIndex() == 0 and
                (self.chkEventTariff.isChecked() or
                self.chkMedicalAidType.isChecked() or
                self.chkRequiredTags.isChecked() or
                self.chkResults.isChecked() or
                self.chkVisitCount.isChecked() or
                self.chkMkbDots.isChecked() or
                self.chkMkbZ.isChecked() or
                self.chkMkbFilter.isChecked() or
                self.chkMkbNoDots.isChecked() or
                self.chkPolicy.isChecked() or
                self.chkMkbZ02xx.isChecked()))
            or (self.tabControlType.currentIndex() == 1 and
                (self.chkHealthExaminationRequiredTags.isChecked() or
                self.chkHealthExaminationResult.isChecked() or
                self.chkHealthExaminationServiceSpeciality.isChecked())))
            and not self.edtFileName.text().isEmpty())

        self.chkMedicalAidType.setEnabled(
            not self.edtMedicalAidTypeRbFileName.text().isEmpty())
        self.chkVisitCount.setEnabled(
            not self.edtVisitCountRbFileName.text().isEmpty())
        self.chkHealthExaminationResult.setEnabled(
            not self.edtHealthExaminationResultRbFileName.text().isEmpty())
        self.chkHealthExaminationServiceSpeciality.setEnabled(
            not self.edtServiceSpecialityRbFileName.text().isEmpty())
        self.chkMkbFilter.setEnabled(
            not self.edtMkbFilterRbFileName.text().isEmpty())
        self.chkMkbNoDots.setEnabled(
            not self.edtMkbExceptionsFileName.text().isEmpty())


    def getOpenFileName(self, editor, mask):
        fileName = QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл с данными', editor.text(), mask)
        if fileName != '':
            editor.setText(QDir.toNativeSeparators(fileName))
            self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkEventTariff_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkMedicalAidType_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkRequiredTags_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkResults_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkVisitCount_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkMkbDots_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkMkbZ_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkHealthExaminationRequiredTags_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkHealthExaminationResult_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkHealthExaminationServiceSpeciality_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkMkbFilter_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkMkbNoDots_toggled(self, checked):
        self.checkFileNames()

    @pyqtSignature('bool')
    def on_chkPolicy_toggled(self, checked):
        self.checkFileNames()

    @pyqtSignature('bool')
    def on_chkMkbZ02xx_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtVisitCountRbFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtMedicalAidTypeRbFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtHealthExaminationResultRbFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtServiceSpecialityRbFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtMkbFilterRbFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtMkbExceptionsFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('')
    def on_btnSelectFileName_clicked(self):
        self.getOpenFileName(self.edtFileName, u'Файлы XML (*.xml)')


    @pyqtSignature('')
    def on_btnSelectVisitCountRbFileName_clicked(self):
        self.getOpenFileName(self.edtVisitCountRbFileName, self.maskCsv)


    @pyqtSignature('')
    def on_btnSelectMedicalAidTypeRbFileName_clicked(self):
        self.getOpenFileName(self.edtMedicalAidTypeRbFileName, self.maskCsv)


    @pyqtSignature('')
    def on_btnSelectServiceSpecialityRbFileName_clicked(self):
        self.getOpenFileName(self.edtServiceSpecialityRbFileName, self.maskCsv)


    @pyqtSignature('')
    def on_btnSelectHealthExaminationResultRbFileName_clicked(self):
        self.getOpenFileName(self.edtHealthExaminationResultRbFileName,
            self.maskCsv)


    @pyqtSignature('')
    def on_btnSelectMkbFilterRbFileName_clicked(self):
        self.getOpenFileName(self.edtMkbFilterRbFileName, self.maskCsv)


    @pyqtSignature('')
    def on_btnSelectMkbExceptionsFileName_clicked(self):
        self.getOpenFileName(self.edtMkbExceptionsFileName, self.maskCsv)


    @pyqtSignature('int')
    def on_tabControlType_currentChanged(self, index):
        self.checkFileNames()


    @pyqtSignature('')
    def on_actQuit_triggered(self):
        self.close()


    @QtCore.pyqtSignature('')
    def on_actAbout_triggered(self):
        global lastChangedRev
        global lastChangedDate
        try:
            from buildInfo import lastChangedRev, lastChangedDate
        except:
            lastChangedRev  = 'unknown'
            lastChangedDate = 'unknown'

        QtGui.QMessageBox.about(
            self, u'О программе', gAbout % ( gTitle, gVersion, lastChangedRev,
                                            lastChangedDate))


    @QtCore.pyqtSignature('')
    def on_actAboutQt_triggered(self):
        QtGui.QMessageBox.aboutQt(self, u'О Qt')


def parseGCDebug(val):
    result = 0
    for char in val.upper():
        if char == 'S':
            result |= gc.DEBUG_STATS
        elif char == 'C':
            result |= gc.DEBUG_COLLECTABLE
        elif char == 'U':
            result |= gc.DEBUG_UNCOLLECTABLE
        elif char == 'I':
            result |= gc.DEBUG_INSTANCES
        elif char == 'O':
            result |= gc.DEBUG_OBJECTS
    return result

def parseGCThreshold(val):
    result = []
    if val:
        for s in val.split(','):
            try:
                v = int(s)
            except:
                v = 0
            result.append(v)
        l = len(result)
        if l > 3:
            result = result[0:3]
    return result


if __name__ == '__main__':
##    gc.set_threshold(*[10*k for k in gc.get_threshold()])
    parser = OptionParser(usage = "usage: %prog [options]")
    parser.add_option('-d', '--demo',
                      dest='demo',
                      help='Login as demo',
                      action='store_true',
                      default=False)
    parser.add_option('--gc-debug',
                      dest='gcDebug',
                      help='gc debug flags, chars S(stat), C(collectable), U(uncollectable), I(instances), O(objects), for details see gc.set_debug',
                      metavar='[S][C][U][I][O]',
                      default=''
                      )
    parser.add_option('--gc-threshold',
                      dest='gcThreshold',
                      help='gc thresholds list, see gc.set_threshold ',
                      metavar='threshold0[,threshold1[,threshold2]]',
                      default=''
                     )
    parser.add_option('--version',
                      dest='version',
                      help='print version and exit',
                      action='store_true',
                      default=False)
#    parser.add_option("-q", "--quiet",   action="store_false", dest="verbose", default=True,  help="don't print status messages to stdout")
    (options, args) = parser.parse_args()

    if options.version:
        print '%s v.0.1' % (sys.argv[0] if sys.argv else '')
    else:
        gcDebug = parseGCDebug(options.gcDebug)
        gcThreshold = parseGCThreshold(options.gcThreshold)
        if gcDebug:
            gc.set_debug(gcDebug)
            print 'debug set to ', gcDebug
        if gcThreshold:
            gc.set_threshold(*gcThreshold)
            print 'threshold set to ', gcThreshold

        app = CMyApp(sys.argv, options.demo)
        stdTtranslator = QtCore.QTranslator()
        stdTtranslator.load('i18n/std_ru.qm')
        app.installTranslator(stdTtranslator)

        QtGui.qApp = app
        app.applyDecorPreferences() # надеюсь, что это поможет немного сэкономить при создании гл.окна
        mainWindow = CMainWindow()
        app.mainWindow = mainWindow
        app.applyDecorPreferences() # применение максимизации/полноэкранного режима к главному окну
        mainWindow.show()
        res = app.exec_()
        mainWindow.savePreferences()
        app.preferences.save()
        app.doneTrace()
        QtGui.qApp = None

#    sys.exit(res)
