#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import codecs
import gc
import locale
import os
import os.path
import shutil
import sys
import tempfile
import traceback
import StringIO
import glob
import re
from optparse import OptionParser

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *
from PyQt4.QtGui  import *


from library.Preferences import CPreferences
from library.Utils       import *
from library.dbfpy.dbf import Dbf
from library.AmountToWords import amountToWords

from Ui_assembleRegistryPage1 import Ui_Page_1
from Ui_assembleRegistryPage2 import Ui_Page_2


class CMyApp(QtGui.QApplication):

    def __init__(self, args):
        QtGui.QApplication.__init__(self, args)
        self.logDir = os.path.join(unicode(QDir.toNativeSeparators(QDir.homePath())), '.samson-vista')
        self.oldMsgHandler = qInstallMsgHandler(self.msgHandler)
        self.oldEexceptHook = sys.excepthook
        sys.excepthook = self.logException
        self.traceActive = True
        self.mainWindow = None
        self.preferences = CPreferences('r67AssembleReestr.ini')
        self.preferences.load()
        self.preferencesDir = self.preferences.getDir()


    def doneTrace(self):
        self.traceActive = False
        qInstallMsgHandler(self.oldMsgHandler)
        sys.excepthook = self.oldEexceptHook


    def getTmpDir(self, suffix=''):
        return unicode(tempfile.mkdtemp('','samson_%s_'%suffix), locale.getpreferredencoding())


    def removeTmpDir(self, dir):
        try:
            shutil.rmtree(dir, False)
        except:
            self.logCurrentException()


    def log(self, title, message, stack=None):
        try:
            if not os.path.exists(self.logDir):
                os.makedirs(self.logDir)
            logFile = os.path.join(self.logDir, 'error.log')
            timeString = unicode(QDateTime.currentDateTime().toString(Qt.SystemLocaleDate))
            logString = u'%s\n%s: %s(%s)\n' % ('='*72, timeString, title, message)
            if stack:
                try:
                    logString += ''.join(traceback.format_list(stack)).decode('utf-8') + '\n'
                except:
                    logString += 'stack lost\n'
            file = codecs.open(logFile, mode='a', encoding=locale.getpreferredencoding())
            file.write(logString)
            file.close()
        except:
            pass


    def logException(self, exceptionType, exceptionValue, exceptionTraceback):
        title = repr(exceptionType)
        message = unicode(exceptionValue)
        self.log(title, message, traceback.extract_tb(exceptionTraceback))
        sys.__excepthook__(exceptionType, exceptionValue, exceptionTraceback)


    def logCurrentException(self):
        self.logException(*sys.exc_info())


    def msgHandler(self, type, msg):
        if type == 0: # QtMsgType.QtDebugMsg:
            typeName = 'QtDebugMsg'
        elif type == 1: # QtMsgType.QtWarningMsg:
            typeName = 'QtWarningMsg'
        elif type == 2: # QtMsgType.QtCriticalMsg:
            typeName = 'QtCriticalMsg'
        elif type == 3: # QtFatalMsg
            typeName = 'QtFatalMsg'
        else:
            typeName = 'QtUnknownMsg'

        self.log( typeName, msg, traceback.extract_stack()[:-1])


    def notify(self, receiver, event):
        try:
            return QtGui.QApplication.notify(self, receiver, event)
        except Exception, e:
            if self.traceActive:
                self.logCurrentException()
                widget = self.activeModalWidget()
                if widget is None:
                    widget = self.mainWindow
                QtGui.QMessageBox.critical( widget,
                                            u'Произошла ошибка',
                                            unicode(e),
                                            QtGui.QMessageBox.Close)
            return False
        except:
            return False


    def call(self, widget, func, params = ()):
        try:
            return True, func(*params)
        except IOError, e:
            msg=''
            if hasattr(e, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno, anyToUnicode(e.strerror))
            else:
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            QtGui.QMessageBox.critical(widget,
                u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
        except Exception, e:
            self.logCurrentException()
            widget = widget or self.activeModalWidget() or self.mainWindow
            QtGui.QMessageBox.critical( widget,
                                        u'Произошла ошибка',
                                        unicode(e),
                                        QtGui.QMessageBox.Close)
        return False, None


    def callWithWaitCursor(self, widget, func, *params, **kwparams):
        self.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
        try:
            return func(*params, **kwparams)
        finally:
            self.restoreOverrideCursor()

# #############################################################


class CMainWindow(QtGui.QWizard):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setWizardStyle(QtGui.QWizard.ModernStyle)
        self.setWindowTitle(u'Компиляция реестров счетов')
        self.page1 = CAssembleRegistryPage1(self)
        self.page2 = CAssembleRegistryPage2(self)
        self.addPage(self.page1)
        self.addPage(self.page2)
        self.loadPreferences()


    def loadPreferences(self):
        preferences = getPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), {})
        geometry = getPref(preferences, 'geometry', None)
        if type(geometry) == QVariant and geometry.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreGeometry(geometry.toByteArray())
        state = getPref(preferences, 'state', None)
#        if type(state) == QVariant and state.type() == QVariant.ByteArray and not geometry.isNull():
#            self.restoreState(state.toByteArray())

        appPreferences = QtGui.qApp.preferences.appPrefs
        self.page1.edtSrcDir.setText(getPrefString(appPreferences, 'SrcDir', ''))
        self.page1.edtOutputDir.setText(getPrefString(appPreferences, 'OutputDir', ''))
        self.page1.edtAccNumber.setText(getPrefString(appPreferences, 'AccNumber', ''))


    def savePreferences(self):
        preferences = {}
        setPref(preferences,'geometry',QVariant(self.saveGeometry()))
        #setPref(preferences,'state', QVariant(self.saveState()))
        setPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), preferences)

        appPreferences = QtGui.qApp.preferences.appPrefs
        setPref(appPreferences, 'SrcDir', QVariant(self.page1.edtSrcDir.text()))
        setPref(appPreferences, 'OutputDir', QVariant(self.page1.edtOutputDir.text()))
        setPref(appPreferences, 'AccNumber', QVariant(self.page1.edtAccNumber.text()))

# #############################################################

class CAssembleRegistryPage1(QtGui.QWizardPage, Ui_Page_1):
    def __init__(self,  parent):
        QtGui.QWizardPage.__init__(self, parent)
        self.setupUi(self)
        self.parent=parent #wtf? скрывать Qt-шный parent - это плохая затея


    def isComplete(self):
        return self.edtSrcDir.text() !='' and self.edtOutputDir.text()!= ''


    @pyqtSignature('')
    def on_btnSelectSrcDir_clicked(self):
        dirName = forceString(QtGui.QFileDialog.getExistingDirectory(
            self, u'Укажите каталог источник', self.edtSrcDir.text()))
        if dirName != '':
            self.edtSrcDir.setText(QDir.toNativeSeparators(dirName))


    @pyqtSignature('')
    def on_btnSelectOutputDir_clicked(self):
        dirName = forceString(QtGui.QFileDialog.getExistingDirectory(
            self, u'Укажите каталог приёмник', self.edtOutputDir.text()))
        if dirName != '':
            self.edtOutputDir.setText(QDir.toNativeSeparators(dirName))


class CAssembleRegistryPage2(QtGui.QWizardPage, Ui_Page_2):
    def __init__(self,  parent):
        QtGui.QWizardPage.__init__(self, parent)
        self.setupUi(self)
        self.parent=parent #wtf? скрывать Qt-шный parent - это плохая затея
        self.done = False
        self._setCount(self.prbFiles, 0)
        self._setCount(self.prbRows, 0)
        self.totalClients = 0
        self.totalMesSum = 0
        self.tableHeader = ''


    def isComplete(self):
        return self.done


    @pyqtSignature('')
    def on_btnCompile_clicked(self):
        QtGui.qApp.call(self, self.compileRegistry)

    def _setCount(self, progressBar, cnt):
        if cnt:
            progressBar.setValue(0)
            progressBar.setMaximum(cnt)
            progressBar.setEnabled(True)
            progressBar.setTextVisible(True)
        else:
            progressBar.setValue(0)
            progressBar.setMaximum(1)
            progressBar.setEnabled(False)
            progressBar.setTextVisible(False)


    def compileRegistry(self):
        self.btnCompile.setEnabled(False)
        inputDir = forceString(self.parent.page1.edtSrcDir.text())
        outputDir = forceString(self.parent.page1.edtOutputDir.text())
        postfix = forceString(self.parent.page1.edtAccNumber.text())

        files = 0

        for prefix in ('C', 'I', 'P', 'S'):
            files += len(glob.glob(os.path.join(inputDir, '%s*.dbf' % prefix)))

        textList = glob.glob(os.path.join(inputDir, '*.txt'))
        files += len(textList)
        self._setCount(self.prbFiles, max(1, files))

        for prefix in ('C', 'I', 'P', 'S'):
            dbfList = glob.glob(os.path.join(inputDir, '%s*.dbf' % prefix))

            if dbfList:
                self.mergeDbf(dbfList, os.path.join(outputDir, '%s%s.dbf' % (prefix, postfix)))
            self.prbFiles.setValue(self.prbFiles.value()+1)

        self.totalMesSum = 0
        self.tableHeader = ''
        self.mergeTxt(textList, os.path.join(outputDir, '%s.txt' %  postfix))
        self.done = True
        self.btnCompile.setEnabled(True)
        self.emit(QtCore.SIGNAL('completeChanged()'))


    def mergeDbf(self, list, outputFileName):
        outDbf = Dbf(outputFileName, new=True, encoding='cp866')
        outHasStruct = False
        filterClients = os.path.basename(outputFileName)[0] in ('I', 'P')

        if filterClients:
            exportedClients = set()
            field = 'D_H' if os.path.basename(outputFileName)[0] == 'I' else 'RECID'

        rows = 0

        for x in list:
            rows += len(Dbf(x, readOnly=True, encoding='cp866'))

        self._setCount(self.prbRows, max(1, rows))

        for x in list:
            inDbf = Dbf(x, readOnly=True, encoding='cp866')

            if not outHasStruct:
                for f in inDbf.header.fields: # копируем структуру dbf-файла
                    outDbf.addField(f)
                outHasStruct = True

            for row in inDbf:
                if filterClients:
                   if row[field] in exportedClients:
                       continue
                   else:
                       exportedClients.add(row[field])

                QtGui.qApp.processEvents()
                record = outDbf.newRecord()

                for name in inDbf.fieldNames:
                    record[name] = row[name]

                record.store()
                self.prbRows.setValue(self.prbRows.value()+1)

            inDbf.close()
            self.prbFiles.setValue(self.prbFiles.value()+1)

        if filterClients:
            self.totalClients = len(exportedClients)

        outDbf.close()


    def mergeTxt(self, list, outputFileName):
        outTxt = QFile(outputFileName)
        outTxt.open(QIODevice.WriteOnly | QIODevice.Text)
        out = QTextStream(outTxt)
        out.setCodec('CP866')
        summary = {}

        for fileName in list:
            r = self.processTxt(fileName, out, fileName == list[0])

            for k in r.keys():
                if summary.has_key(k):
                    if k == 'clients':
                        summary[k] += r[k]
                    else:
                        for i in range(len(r[k])):
                            summary[k][i] += r[k][i]
                else:
                    summary[k] = r[k]

        self.writeSummary(out, summary)
        outTxt.close()

    mesStr = re.compile(ur'Выполнено услуг по МЭС\s*(\d+)\s*на сумму\s*(\d+\.\d+)\s*')

    def processTxt(self, fileName, out, copyHeader):
        inFile = QFile(fileName)
        inFile.open(QIODevice.ReadOnly | QIODevice.Text)
        inTxt = QTextStream(inFile)
        inTxt.setCodec('CP866')

        while not inTxt.atEnd():
            line = forceString(inTxt.readLine())
            if copyHeader:
                out << line << '\n'

            if line == u'1. РАБОТА ПРОФИЛЬНЫХ ОТДЕЛЕНИЙ':
                break

        while not inTxt.atEnd():
            line = forceString(inTxt.readLine())

            if line[:23] == u'Медицинская организация':
                self.tableHeader = ''
                for i in range(0, 6):
                    line = forceString(inTxt.readLine())
                    self.tableHeader += line + '\n'

                self.skipLines(inTxt, 6)
                line = forceString(inTxt.readLine())
                m = self.mesStr.match(line)
                self.totalMesSum += forceDouble(m.group(2))
                break

            out << line << '\n'

        return self.processFooter(inTxt)


    tableStr = re.compile(ur'\|\s*([^\|]+)\s*\|\s*(\d+)\s*\|\s*(\d+)\s*\|\s*(\d+)\s*\|\s*(\d+)\s*\|\s*(\d+)\s*\|'\
                                ur'\s*(\d+\.\d+)\s*\|\s*(\d+\.\d+)\s*\|\s*(\d+\.\d+)\s*\|\s*(\d+)\s*\|'\
                                ur'\s*(\d+\.\d+)\s*\|\s*(\d+\.\d+)\s*\|\s*(\d+)\s*\|\s*(\d+\.\d+)\s*\|')


    def processFooter(self, inTxt):
        result = {}
        self.skipLines(inTxt, 11)

        while not inTxt.atEnd():
            line = forceString(inTxt.readLine())
            m = self.tableStr.match(line)
            if not m:
                break

            name = m.group(1).strip()
            result[name]= [
                forceInt(m.group(2)), forceInt(m.group(3)), forceInt(m.group(4)), forceInt(m.group(5)),
                forceInt(m.group(6)),
                forceDouble(m.group(7)), forceDouble(m.group(8)), forceDouble(m.group(9)),
                forceInt(m.group(10)),
                forceDouble(m.group(11)), forceDouble(m.group(12)), forceInt(m.group(13)), forceDouble(m.group(14))]

            if name == u'Итого':
                break

            inTxt.readLine()

        return result


    def skipLines(self, inTxt, n):
        for i in range(0, n):
            if inTxt.atEnd():
                break

            inTxt.readLine()


    def writeSummary(self, out, summary):
        delimeter = u'+-------------------+------+------+------+------+------+----------+-----------+-----+------+----------+-----------+------+-----------+\n'
        format = u'|%19.19s|%6d|%6d|%6d|%6d|%6d|%10.2f|%11.2f|%5.2f|%6d|%10.2f|%11.2f|%6d|%11.2f|\n'
        totalClientCount = 0
        totalAmbClientCount = 0
        totalHospClientCount = 0
        totalDayHospClientCount = 0
        totalAmount = 0
        totalAmbFedSum = 0
        totalAmbTerSum = 0
        totalUet = 0
        totalBedDays = 0
        totalHospTerSum = 0
        totalHospFedSum = 0
        totalSum = 0
        totalMES= 0
        totalDayHospSum = 0

        orgChief = ''
        orgAccountant = ''

        if summary.has_key(u'Итого'):
            (totalClientCount, totalAmbClientCount, totalHospClientCount, totalDayHospClientCount,
            totalAmount, totalAmbTerSum, totalAmbFedSum,  totalUet, totalMES,
            totalHospTerSum, totalHospFedSum, totalBedDays, totalSum) = summary.pop(u'Итого')

        out << self.tableHeader
        out <<u"""\nКоличество пролеченных пациентов %7d на сумму %11.2f
             из них амбулаторных %7d на сумму %11.2f
                    стационарных %7d на сумму %11.2f
             дневного стационара %7d на сумму %11.2f
Выполнено услуг                  %7d на сумму %11.2f
Выполнено услуг по МЭС           %7d на сумму %11.2f
Выполнено к/д                %7d на сумму %11.2f
+-------------------+---------------------------+-----------------------------------+-----------------------------+------------------+
|Наименование       | Пролечено пациентов       | Выполнено услуг                   |  Реализовано МЭС, к/д       |      |Всего      |
|отделения,         |                           |                                   |   по учету к/д              |      |cтоимость  |
|МЭС                +------+------+------+------+------+----------------------+-----+------+-----------+----------+      |           |
|                   |Всего | Амб. | Стац.| Дн.  |Кол-во| Стоимость            | УЕТ |Кол-во| Стоимость            |Кол-во|           |
|                   |      |      |      |      |      +----------------------+     | МЭС  |----------------------+ к/д  |           |
|                   |      |      |      | Стац.|      | Тариф    | Тариф доп.|     |      | Тариф    | Тариф доп.|      |           |
|                   |      |      |      |      |      |(территори|(федера    |     |      |(территори|(федера    |      |           |
|                   |      |      |      |      |      | альный)  | льный)    |     |      | альный)  | льный)    |      |           |
+-------------------+------+------+------+------+------+----------+-----------+-----+------+----------+-----------+------+-----------+
""" % (self.totalClients,  totalSum,
        totalAmbClientCount,  totalAmbTerSum+totalAmbFedSum,
        totalHospClientCount, totalHospTerSum+totalHospFedSum,
        totalDayHospClientCount,  totalDayHospSum,
        totalAmount, totalAmbTerSum+totalAmbFedSum,
        totalMES,  self.totalMesSum,
        totalBedDays, totalHospTerSum+totalHospFedSum)

        for (key, val) in summary.iteritems():
            (clientCount, ambClientCount, hospClientCount, dayHospClientCount,
            amount, ambTerSum, ambFedSum,  uet, MESCount, hospFedSum, hospTerSum, bedDays, sum) = val
            out << format % (key.ljust(19),  clientCount, ambClientCount, hospClientCount, dayHospClientCount,
            amount, ambTerSum, ambFedSum,  uet, MESCount, hospFedSum, hospTerSum, bedDays, sum)
            out << delimeter

        out << format % (u'Итого'.ljust(19), self.totalClients,
                totalAmbClientCount, totalHospClientCount, totalDayHospClientCount,
                totalAmount, totalAmbTerSum, totalAmbFedSum, totalUet, totalMES,
                totalHospTerSum, totalHospFedSum, totalBedDays, totalSum)
        out << delimeter
        out << u"""
ВСЕГО ПРЕДСТАВЛЕНО К ОПЛАТЕ: %11.2f
      (%s)

По территориальному тарифу на сумму %11.2f (%s)
По дополнительному тарифу на сумму %11.2f (%s)

Руководитель мед.учреждения ____________________ %s

Гл.бухгалтер мед.учреждения ____________________ %s
""" % (totalSum, amountToWords(totalSum),
            totalAmbTerSum+totalHospTerSum, amountToWords(totalAmbTerSum+totalHospTerSum),
            totalAmbFedSum+totalHospFedSum, amountToWords(totalAmbFedSum+totalHospFedSum),
            orgChief, orgAccountant)


# #############################################################

def parseGCDebug(val):
    result = 0
    for char in val.upper():
        if char == 'S':
            result |= gc.DEBUG_STATS
        elif char == 'C':
            result |= gc.DEBUG_COLLECTABLE
        elif char == 'U':
            result |= gc.DEBUG_UNCOLLECTABLE
        elif char == 'I':
            result |= gc.DEBUG_INSTANCES
        elif char == 'O':
            result |= gc.DEBUG_OBJECTS
    return result

def parseGCThreshold(val):
    result = []
    if val:
        for s in val.split(','):
            try:
                v = int(s)
            except:
                v = 0
            result.append(v)
        l = len(result)
        if l > 3:
            result = result[0:3]
    return result


if __name__ == '__main__':
##    gc.set_threshold(*[10*k for k in gc.get_threshold()])
    parser = OptionParser(usage = "usage: %prog [options]")
    parser.add_option('-d', '--demo',
                      dest='demo',
                      help='Login as demo',
                      action='store_true',
                      default=False)
    parser.add_option('--gc-debug',
                      dest='gcDebug',
                      help='gc debug flags, chars S(stat), C(collectable), U(uncollectable), I(instances), O(objects), for details see gc.set_debug',
                      metavar='[S][C][U][I][O]',
                      default=''
                      )
    parser.add_option('--gc-threshold',
                      dest='gcThreshold',
                      help='gc thresholds list, see gc.set_threshold ',
                      metavar='threshold0[,threshold1[,threshold2]]',
                      default=''
                     )
    parser.add_option('--version',
                      dest='version',
                      help='print version and exit',
                      action='store_true',
                      default=False)
#    parser.add_option("-q", "--quiet",   action="store_false", dest="verbose", default=True,  help="don't print status messages to stdout")
    (options, args) = parser.parse_args()

    if options.version:
        print '%s v.0.1' % (sys.argv[0] if sys.argv else '')
    else:
        gcDebug = parseGCDebug(options.gcDebug)
        gcThreshold = parseGCThreshold(options.gcThreshold)
        if gcDebug:
            gc.set_debug(gcDebug)
            print 'debug set to ', gcDebug
        if gcThreshold:
            gc.set_threshold(*gcThreshold)
            print 'threshold set to ', gcThreshold

        app = CMyApp(sys.argv)
        stdTtranslator = QtCore.QTranslator()
        stdTtranslator.load('i18n/std_ru.qm')
        app.installTranslator(stdTtranslator)
        app.setStyle(QStyleFactory.create('plastique'))

        QtGui.qApp = app
        mainWindow = CMainWindow()
        app.mainWindow = mainWindow
        mainWindow.show()
        res = app.exec_()
        mainWindow.savePreferences()
        app.preferences.save()
        app.doneTrace()
        QtGui.qApp = None

#    sys.exit(res)
