#!/usr/bin/env python
# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import codecs
import gc
import locale
import os
import os.path
import shutil
import sys
import tempfile
import traceback
import StringIO
from optparse import OptionParser

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *
from PyQt4.QtGui  import *

import library.patches
from Orgs.Orgs           import selectOrganisation
from Users.Login         import CLoginDialog
from Users.RightList     import CUserRightListDialog,  CUserRightProfileListDialog
from Users.UserInfo      import CUserInfo, CDemoUserInfo
from Users.Rights        import *
from Users.Tables        import *
from library             import database
from library.Preferences import CPreferences
from library.DialogBase  import CDialogBase
from library.Utils       import *

from preferences.connection import CConnectionDialog
from preferences.decor import CDecorDialog

from EISConnection       import CEISConnectionDialog
from Preferences         import CPreferencesDialog
from Progress            import CProgressDialog
from Ui_exportToEIS      import Ui_MainWindow


class CMyApp(QtGui.QApplication):
    __pyqtSignals__ = ('dbConnectionChanged(bool)',
                       'currentUserIdChanged()',
                      )

    def __init__(self, args, demoModeRequest):
        QtGui.QApplication.__init__(self, args)
        self.logDir = os.path.join(unicode(QDir.toNativeSeparators(QDir.homePath())), '.samson-vista')
        self.oldMsgHandler = qInstallMsgHandler(self.msgHandler)
        self.oldEexceptHook = sys.excepthook
        sys.excepthook = self.logException
        self.traceActive = True
        self.homeDir= None
        self.saveDir= None
        self.userId = None
        self.userSpecialityId = None
        self.userOrgStructureId = None
        self.onlyWithHistory = True
        self.db     = None
        self.mainWindow = None
        self.preferences = CPreferences('r78ExportToEIS.ini')
        self.preferences.load()
        self.preferencesDir = self.preferences.getDir()
        self.registerDocumentTables()
        self._currentClientId = None
        self._orgId = None
        self._orgStructureId = None
        self._onlyWithHistory = True
        self._doctorCodeSource = 1
        self.userInfo = None
        self.demoModeRequested = True


    def doneTrace(self):
        self.traceActive = False
        qInstallMsgHandler(self.oldMsgHandler)
        sys.excepthook = self.oldEexceptHook


    def applyDecorPreferences(self):
        self.setStyle(self.preferences.decorStyle)
        if self.preferences.decorStandardPalette:
            self.setPalette(self.style().standardPalette())
        if self.mainWindow:
            state = Qt.WindowNoState
            if self.preferences.decorMaximizeMainWindow:
                state |= Qt.WindowMaximized
            if self.preferences.decorFullScreenMainWindow:
                state |= Qt.WindowFullScreen
            self.mainWindow.setWindowState(state)


    def registerDocumentTables(self):
        pass


    def getHomeDir(self):
        if not self.homeDir:
            homeDir = os.path.expanduser('~')
            if homeDir == '~':
                homeDir = unicode(QDir.homePath())
            if isinstance(homeDir, str):
                homeDir = unicode(homeDir, locale.getpreferredencoding())
            self.homeDir = homeDir
        return self.homeDir


    def getSaveDir(self):
        self.saveDir = forceString(self.preferences.appPrefs.get('saveDir', QVariant()))
        if not self.saveDir:
            self.saveDir = self.getHomeDir()
        return self.saveDir


    def setSaveDir(self, path):
#        mode = os.stat(path)[ST_MODE]
#        if if S_ISDIR(mode):
        saveDir = os.path.dirname(unicode(path))
        if saveDir and self.saveDir != saveDir:
            self.preferences.appPrefs['saveDir'] = QVariant(saveDir)


    def getTemplateDir(self):
        result = forceString(self.preferences.appPrefs.get('templateDir', None))
        if not result:
            result = os.path.join(self.logDir, 'templates')
        return result


    def getTmpDir(self, suffix=''):
        return unicode(tempfile.mkdtemp('','samson_%s_'%suffix), locale.getpreferredencoding())


    def removeTmpDir(self, dir):
        try:
            shutil.rmtree(dir, False)
        except:
            self.logCurrentException()


    def currentOrgId(self):
        return self._orgId


    def currentOrgStructureId(self):
        return self._orgStructureId


    def onlyWithHistory(self):
        return self._onlyWithHistory


    def doctorCodeSource(self):
        return self._doctorCodeSource

#    def setTemplateDir(self, path):
#        self.preferences.appPrefs.setdefault('templateDir', QVariant(path))


    def log(self, title, message, stack=None):
        try:
            if not os.path.exists(self.logDir):
                os.makedirs(self.logDir)
            logFile = os.path.join(self.logDir, 'error.log')
            timeString = unicode(QDateTime.currentDateTime().toString(Qt.SystemLocaleDate))
            logString = u'%s\n%s: %s(%s)\n' % ('='*72, timeString, title, message)
            if stack:
                try:
                    logString += ''.join(traceback.format_list(stack)).decode('utf-8') + '\n'
                except:
                    logString += 'stack lost\n'
            file = codecs.open(logFile, mode='a', encoding=locale.getpreferredencoding())
            file.write(logString)
            file.close()
        except:
            pass


    def logException(self, exceptionType, exceptionValue, exceptionTraceback):
        title = repr(exceptionType)
        message = unicode(exceptionValue)
        self.log(title, message, traceback.extract_tb(exceptionTraceback))
        sys.__excepthook__(exceptionType, exceptionValue, exceptionTraceback)


    def logCurrentException(self):
        self.logException(*sys.exc_info())


    def msgHandler(self, type, msg):
        if type == 0: # QtMsgType.QtDebugMsg:
            typeName = 'QtDebugMsg'
        elif type == 1: # QtMsgType.QtWarningMsg:
            typeName = 'QtWarningMsg'
        elif type == 2: # QtMsgType.QtCriticalMsg:
            typeName = 'QtCriticalMsg'
        elif type == 3: # QtFatalMsg
            typeName = 'QtFatalMsg'
        else:
            typeName = 'QtUnknownMsg'

        self.log( typeName, msg, traceback.extract_stack()[:-1])
#        print typeName, msg


    def openDatabase(self):
        self.db = None
        self.db = database.connectDataBase(self.preferences.dbDriverName,
                                     self.preferences.dbServerName,
                                     self.preferences.dbServerPort,
                                     self.preferences.dbDatabaseName,
                                     self.preferences.dbUserName,
                                     self.preferences.dbPassword)
        self.emit(QtCore.SIGNAL('dbConnectionChanged(bool)'), True)


    def closeDatabase(self):
        if self.db:
            self.db.close()
            self.db = None
            self.emit(QtCore.SIGNAL('dbConnectionChanged(bool)'), False)


    def notify(self, receiver, event):
        try:
#            report = ["notify:: receiver.type=",unicode(type(receiver))," receiver.name=", unicode(receiver.objectName())," event.type=",unicode(type(event)), "event.type()=", unicode(event.type())]
#            print >>sys.stderr, ("".join(report)).encode('cp866')
            return QtGui.QApplication.notify(self, receiver, event)
        except Exception, e:
            if self.traceActive:
                self.logCurrentException()
                widget = self.activeModalWidget()
                if widget is None:
                    widget = self.mainWindow
                QtGui.QMessageBox.critical( widget,
                                            u'Произошла ошибка',
                                            unicode(e),
                                            QtGui.QMessageBox.Close)
            return False
        except:
            return False


    def call(self, widget, func, params = ()):
        try:
            return True, func(*params)
        except IOError, e:
            msg=''
            if hasattr(e, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno, anyToUnicode(e.strerror))
            else:
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            QtGui.QMessageBox.critical(widget,
                u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
        except Exception, e:
            self.logCurrentException()
            widget = widget or self.activeModalWidget() or self.mainWindow
            QtGui.QMessageBox.critical( widget,
                                        u'Произошла ошибка',
                                        unicode(e),
                                        QtGui.QMessageBox.Close)
        return False, None


    def callWithWaitCursor(self, widget, func, *params, **kwparams):
        self.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
        try:
            return func(*params, **kwparams)
        finally:
            self.restoreOverrideCursor()


    def demoModePosible(self):
        try:
            tableUser = self.db.table(tblUser)
            record = self.db.getRecordEx(tableUser, [usrPassword, usrRetired], tableUser[usrLogin].eq(demoUserName))
            return not record or (not forceString(record.value(usrPassword)) and not forceBool(record.value(usrRetired)))
        except:
            pass
        return False


    def getEISPreferences(self):
        appPreferences = QtGui.qApp.preferences.appPrefs
        eisPreferences = appPreferences.get('eis', {})
        result = { 'server'  : forceString(eisPreferences.get('server', 'localhost')),
                   'port'    : forceInt(eisPreferences.get('port', 0)),
                   'database': forceString(eisPreferences.get('database', 'C:\\EISOMS\\DBMU.GDB')),
                   'user'    : forceString(eisPreferences.get('user', 'SYSDBA')),
                   'password': forceString(eisPreferences.get('password', 'masterkey')),
                 }
        return result

    def setEISPreferences(self, eisPreferences):
        appPreferences = QtGui.qApp.preferences.appPrefs
        appPreferences['eis'] = eisPreferences
        QtGui.qApp.preferences.save()


# #############################################################


class CMainWindow(QtGui.QMainWindow, Ui_MainWindow):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.connect(QtGui.qApp, QtCore.SIGNAL('dbConnectionChanged(bool)'), self.onConnectionChanged)
        self.loadPreferences()


    def loadPreferences(self):
        preferences = getPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), {})
        geometry = getPref(preferences, 'geometry', None)
        if type(geometry) == QVariant and geometry.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreGeometry(geometry.toByteArray())
        state = getPref(preferences, 'state', None)
        if type(state) == QVariant and state.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreState(state.toByteArray())

        appPreferences = QtGui.qApp.preferences.appPrefs
        QtGui.qApp._orgId = getPrefRef(appPreferences, 'orgId', None)
        QtGui.qApp._orgStructureId = getPrefRef(appPreferences, 'orgStructureId', None)
        QtGui.qApp._onlyWithHistory = getPrefBool(appPreferences, 'onlyWithHistory', True)
        QtGui.qApp._doctorCodeSource = getPrefInt(appPreferences, 'doctorCodeSource', 1)
#        self.updateImportPosibility()


    def savePreferences(self):
        preferences = {}
        setPref(preferences,'geometry',QVariant(self.saveGeometry()))
        setPref(preferences,'state', QVariant(self.saveState()))
        setPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), preferences)

        appPreferences = QtGui.qApp.preferences.appPrefs
        setPref(appPreferences, 'orgId', QVariant(QtGui.qApp._orgId))
        setPref(appPreferences, 'orgStructureId', QVariant(QtGui.qApp._orgStructureId))
        setPref(appPreferences, 'onlyWithHistory', QVariant(QtGui.qApp._onlyWithHistory))
        setPref(appPreferences, 'doctorCodeSource', QVariant(QtGui.qApp._doctorCodeSource))


#    def updateImportPosibility(self):
#        importPosible = self.chkImportFromKES.isChecked() or self.chkImportFromEIS.isChecked()
#        self.btnImport.setEnabled(importPosible)

    def onConnectionChanged(self, connected):
        self.actPreferences.setEnabled(connected)
        self.btnExport.setEnabled(connected)


    @pyqtSignature('')
    def on_btnExport_clicked(self):
        def doExport():
            eisPreferences = QtGui.qApp.getEISPreferences()
            process = CProgressDialog(self)
            db = QtGui.qApp.db
            process.orgId = QtGui.qApp._orgId
            process.orgStructureId = QtGui.qApp._orgStructureId
            process.onlyWithHistory = QtGui.qApp._onlyWithHistory
            process.doctorCodeSource = QtGui.qApp._doctorCodeSource

            process.EISDb   = database.connectDataBase('FIREBIRD', #driverName
                                                     eisPreferences['server'],   #serverName
                                                     eisPreferences['port'],     #serverPort
                                                     eisPreferences['database'], # databaseName
                                                     eisPreferences['user'],     # userName
                                                     eisPreferences['password'], # password
                                                     'EISDb')    # connectionName
            process.exec_()
        QtGui.qApp.call(self, doExport)


    @pyqtSignature('')
    def on_actLogin_triggered(self):
        try:
            QtGui.qApp.openDatabase()
            return
        except database.CDatabaseException, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                unicode(e),
                QtGui.QMessageBox.Close)
        except Exception, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                u'Невозможно установить соедиенение с базой данных\n' + unicode(e),
                QtGui.QMessageBox.Close)
            self.logCurrentException()
        QtGui.qApp.closeDatabase()


    @pyqtSignature('')
    def on_actLogout_triggered(self):
        QtGui.qApp.closeDatabase()
#        self.setUserName('')


    @pyqtSignature('')
    def on_actQuit_triggered(self):
        self.close()


    @QtCore.pyqtSignature('')
    def on_actConnection_triggered(self):
        dlg   = CConnectionDialog(self)
        preferences = QtGui.qApp.preferences
        dlg.setDriverName(preferences.dbDriverName)
        dlg.setServerName(preferences.dbServerName)
        dlg.setServerPort(preferences.dbServerPort)
        dlg.setDatabaseName(preferences.dbDatabaseName)
        dlg.setUserName(preferences.dbUserName)
        dlg.setPassword(preferences.dbPassword)
        if dlg.exec_():
            preferences.dbDriverName = dlg.driverName()
            preferences.dbServerName = dlg.serverName()
            preferences.dbServerPort = dlg.serverPort()
            preferences.dbDatabaseName = dlg.databaseName()
            preferences.dbUserName = dlg.userName()
            preferences.dbPassword = dlg.password()
            preferences.save()


    @QtCore.pyqtSignature('')
    def on_actPreferences_triggered(self):
        dlg = CPreferencesDialog(self)
        dlg.setOrgId(QtGui.qApp._orgId)
        dlg.setOrgStructureId(QtGui.qApp._orgStructureId)
        dlg.setOnlyWithHistory(QtGui.qApp._onlyWithHistory)
        dlg.setDoctorCodeSource(QtGui.qApp._doctorCodeSource)
        if dlg.exec_():
            QtGui.qApp._orgId = dlg.orgId()
            QtGui.qApp._orgStructureId = dlg.orgStructureId()
            QtGui.qApp._onlyWithHistory = dlg.onlyWithHistory()
            QtGui.qApp._doctorCodeSource = dlg.doctorCodeSource()


    @QtCore.pyqtSignature('')
    def on_actEISConnection_triggered(self):
        eisPreferences = QtGui.qApp.getEISPreferences()
        dlg = CEISConnectionDialog(self)
        dlg.setServerName(eisPreferences['server'])
        dlg.setServerPort(eisPreferences['port'])
        dlg.setDatabaseName(eisPreferences['database'])
        dlg.setUserName(eisPreferences['user'])
        dlg.setPassword(eisPreferences['password'])
        if dlg.exec_():
            eisPreferences['server']   = dlg.serverName()
            eisPreferences['port']     = dlg.serverPort()
            eisPreferences['database'] = dlg.databaseName()
            eisPreferences['user']     = dlg.userName()
            eisPreferences['password'] = dlg.password()
            QtGui.qApp.setEISPreferences(eisPreferences)


    @QtCore.pyqtSignature('')
    def on_actDecor_triggered(self):
        dlg = CDecorDialog(self)
        preferences = QtGui.qApp.preferences
        dlg.setStyle(preferences.decorStyle)
        dlg.setStandardPalette(preferences.decorStandardPalette)
        dlg.setMaximizeMainWindow(preferences.decorMaximizeMainWindow)
        dlg.setFullScreenMainWindow(preferences.decorFullScreenMainWindow)
        if dlg.exec_():
            preferences.decorStyle = dlg.style()
            preferences.decorStandardPalette = dlg.standardPalette()
            preferences.decorMaximizeMainWindow = dlg.maximizeMainWindow()
            preferences.decorFullScreenMainWindow = dlg.fullScreenMainWindow()
            preferences.save()
            QtGui.qApp.applyDecorPreferences()


#    @QtCore.pyqtSignature('')
#    def on_actAbout_triggered(self):
#        global lastChangedRev
#        global lastChangedDate
#        try:
#            from buildInfo import lastChangedRev, lastChangedDate
#        except:
#            lastChangedRev  = 'unknown'
#            lastChangedDate = 'unknown'
#
#        QtGui.QMessageBox.about(
#            self, u'О программе', about % ( title, lastChangedRev, lastChangedDate)
#            )


    @QtCore.pyqtSignature('')
    def on_actAboutQt_triggered(self):
        QtGui.QMessageBox.aboutQt(self, u'О Qt')


def parseGCDebug(val):
    result = 0
    for char in val.upper():
        if char == 'S':
            result |= gc.DEBUG_STATS
        elif char == 'C':
            result |= gc.DEBUG_COLLECTABLE
        elif char == 'U':
            result |= gc.DEBUG_UNCOLLECTABLE
        elif char == 'I':
            result |= gc.DEBUG_INSTANCES
        elif char == 'O':
            result |= gc.DEBUG_OBJECTS
    return result

def parseGCThreshold(val):
    result = []
    if val:
        for s in val.split(','):
            try:
                v = int(s)
            except:
                v = 0
            result.append(v)
        l = len(result)
        if l > 3:
            result = result[0:3]
    return result


if __name__ == '__main__':
##    gc.set_threshold(*[10*k for k in gc.get_threshold()])
    parser = OptionParser(usage = "usage: %prog [options]")
    parser.add_option('-d', '--demo',
                      dest='demo',
                      help='Login as demo',
                      action='store_true',
                      default=False)
    parser.add_option('--gc-debug',
                      dest='gcDebug',
                      help='gc debug flags, chars S(stat), C(collectable), U(uncollectable), I(instances), O(objects), for details see gc.set_debug',
                      metavar='[S][C][U][I][O]',
                      default=''
                      )
    parser.add_option('--gc-threshold',
                      dest='gcThreshold',
                      help='gc thresholds list, see gc.set_threshold ',
                      metavar='threshold0[,threshold1[,threshold2]]',
                      default=''
                     )
    parser.add_option('--version',
                      dest='version',
                      help='print version and exit',
                      action='store_true',
                      default=False)
#    parser.add_option("-q", "--quiet",   action="store_false", dest="verbose", default=True,  help="don't print status messages to stdout")
    (options, args) = parser.parse_args()

    if options.version:
        print '%s v.0.1' % (sys.argv[0] if sys.argv else '')
    else:
        gcDebug = parseGCDebug(options.gcDebug)
        gcThreshold = parseGCThreshold(options.gcThreshold)
        if gcDebug:
            gc.set_debug(gcDebug)
            print 'debug set to ', gcDebug
        if gcThreshold:
            gc.set_threshold(*gcThreshold)
            print 'threshold set to ', gcThreshold

        app = CMyApp(sys.argv, options.demo)
        stdTtranslator = QtCore.QTranslator()
        stdTtranslator.load('i18n/std_ru.qm')
        app.installTranslator(stdTtranslator)

        QtGui.qApp = app
        app.applyDecorPreferences() # надеюсь, что это поможет немного сэкономить при создании гл.окна
        mainWindow = CMainWindow()
        app.mainWindow = mainWindow
        app.applyDecorPreferences() # применение максимизации/полноэкранного режима к главному окну

        if app.preferences.dbAutoLogin:
            mainWindow.actLogin.activate(QtGui.QAction.Trigger)
        mainWindow.show()
        res = app.exec_()
        mainWindow.savePreferences()
        app.preferences.save()
        app.closeDatabase()
        app.doneTrace()
        QtGui.qApp = None

#    sys.exit(res)
