# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
from PyQt4 import QtGui
from library.TreeModel        import *
from library.TreeComboBox import CTreeComboBox
from library.AgeSelector import composeAgeSelector, parseAgeSelector
from Accounting.Utils import CContractTreeItem
from Events.Utils import *

from RefBooks.Ui_GroupActionTypeEditor import Ui_GroupActionTypeEditorDialog


class CTableWidget(QtGui.QTableWidget):
    def __init__(self, parent = None):
        QtGui.QTableWidget.__init__(self, parent)
        self._popupMenu = QtGui.QMenu(self)
        self._actAddRow = QtGui.QAction(u'Добавить строку', self)
        self._actAddRow.triggered.connect(self.addRow)
        self._popupMenu.addAction(self._actAddRow)
        self._actDelRow = QtGui.QAction(u'Удалить строку', self)
        self._actDelRow.triggered.connect(self.delRow)
        self._popupMenu.addAction(self._actDelRow)
        self.connect(self, SIGNAL('customContextMenuRequested(QPoint)'), self.showMenu)

    def showMenu(self, pos):
        if self._popupMenu:
            self._popupMenu.exec_(self.mapToGlobal(pos))

    def addRow(self):
        self.insertRow(self.rowCount())

    def delRow(self):
        self.removeRow(self.currentRow())


class CContractRootTreeItem(CContractTreeItem):
    def __init__(self, contracts, financeTypeCodeList=None):
        CContractTreeItem.__init__(self, u'Все договоры', None)
        self._contractIdToPath = {}
        self._pathToContractId = {}
        db = QtGui.qApp.db
        table = db.table('Contract')
        cond = []
        if financeTypeCodeList:
            cond.append(table['finance_id'].inlist([CFinanceType.getId(code) for code in financeTypeCodeList]))
        records = db.getRecordList(table,
                                   'id, finance_id, grouping, resolution, date, number',
                                   where=db.joinAnd(cond),
                                   order='finance_id, grouping, resolution, date, number, id')
        for record in records:
            contractId = forceRef(record.value('id'))
            contractName = forceString(record.value('number')) + u' от ' +forceString(record.value('date'))
            path = [ self.getFinanceName(record.value('finance_id')),
                     forceString(record.value('grouping')),
                     forceString(record.value('resolution')),
                     contractName
                   ]
            self.addItem(path, 0, contractId)
            contracts[contractId] = contractName
            self._contractIdToPath[contractId] = path
            self._pathToContractId['\\'.join(path)] = contractId


    def getPathById(self, contractId):
        return self._contractIdToPath.get(contractId, '')

    def getIdByPath(self, path):
        return self._pathToContractId.get(path, None)

    def getFinanceName(self, val):
        financeId = forceRef(val)
        financeRecord = QtGui.qApp.db.getRecord('rbFinance', 'name', financeId)
        if financeRecord:
            return forceString(financeRecord.value(0))
        else:
            return '{%s}' % financeId

class CContractTreeModel(QAbstractItemModel):
    def __init__(self, parent=None, financeTypeCodeList=None):
        QAbstractItemModel.__init__(self, parent)
        self.contracts = {}
        self._rootItem = CContractRootTreeItem(self.contracts, financeTypeCodeList)


    def items(self):
        return self.getRootItem().descendants()


    def getPathById(self, contractId):
        return self.getRootItem().getPathById(contractId)

    def getIdByPath(self, path):
        return self.getRootItem().getIdByPath(path)

    def getRootItem(self):
        return self._rootItem


    def columnCount(self, parent=None):
        return 1


    def data(self, index, role):
        if not index.isValid():
            return QVariant()

        if role != Qt.DisplayRole:
            return QVariant()

        item = index.internalPointer()
        return QVariant(item.data(index.column()))


    def flags(self, index):
        if not index.isValid():
            return 0
        item = index.internalPointer()
        return item.flags()

#    def headerData(self, section, orientation, role):
#        return QVariant()

    def index(self, row, column, parent = QModelIndex()):
        if parent and parent.isValid():
            parentItem = parent.internalPointer()
            childItem = parentItem.child(row)
            return self.createIndex(row, column, childItem)
        else:
            return self.createIndex(0, 0, self.getRootItem())


    def parent(self, index):
        if not index.isValid():
            return QModelIndex()
        childItem = index.internalPointer()
        parentItem = childItem.parent
        if not parentItem:
            return QModelIndex()
        return self.createIndex(parentItem.row(), 0, parentItem)


    def rowCount(self, parent):
        if parent and parent.isValid():
            parentItem = parent.internalPointer()
            return parentItem.childCount()
        else:
            return 1

class CContractComboBox(CTreeComboBox):
    def __init__(self, parent):
        CTreeComboBox.__init__(self, parent)
        self.SizeAdjustPolicy(QtGui.QComboBox.AdjustToMinimumContentsLength)

    def init(self):
        if QtGui.qApp.db:
            self._model = CContractTreeModel(self)
            self.setModel(self._model)


    def getPath(self):
        index = self.currentModelIndex()
        if index and index.isValid():
            item = index.internalPointer()
            parts = []
            while item:
                parts.append(email.utils.quote(item.name))
                item = item.parent
            return '\\'.join(parts[::-1])


    def setPath(self, path):
        names = [email.utils.unquote(part) for part in path.split('\\')][1:]
        item = self._model.getRootItem()
        for name in names:
            nextItem = item.mapNameToItem.get(name, None)
            if nextItem:
                item = nextItem
            else:
                break
        index = self._model.createIndex(item.row(), 0, item)
        self.setCurrentModelIndex(index)


    def getPathById(self, contractId):
        path = self._model.getPathById(contractId)
        return '\\'.join(path)


    def getIdByPath(self, path):
        return self._model.getIdByPath(path)


    def getIdList(self):
        index = self.currentModelIndex()
        item = index.internalPointer()
        return item.idList if item else []


class CGroupActionTypeEditor(QtGui.QDialog, Ui_GroupActionTypeEditorDialog):
    def __init__(self, parent = None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.confirmed = False

    def done(self, result):
        if result:
            self.confirmed = True
        QtGui.QDialog.done(self, result)

    def loadSettings(self, fields):
        sex = fields.get('sex', None)
        if sex:
            self.chkSex.setChecked(True)
            self.cmbSex.setCurrentIndex(sex)
        age = fields.get('age', None)
        if age:
            self.chkAge.setChecked(True)
            (begUnit, begCount, endUnit, endCount) = parseAgeSelector(age)
            self.cmbBegAgeUnit.setCurrentIndex(begUnit)
            self.edtBegAgeCount.setText(str(begCount))
            self.cmbEndAgeUnit.setCurrentIndex(endUnit)
            self.edtEndAgeCount.setText(str(endCount))
        if 'showTime' in fields:
            self.chkChkShowTime.setChecked(True)
            self.chkShowTime.setChecked(fields['showTime'])
        if 'isRequiredCoordination' in fields:
            self.chkChkRequiredCoordination.setChecked(True)
            self.chkRequiredCoordination.setChecked(fields['isRequiredCoordination'])
        if 'showInForm' in fields:
            self.chkChkShowInForm.setChecked(True)
            self.chkShowInForm.setChecked(fields['showInForm'])
        if 'hasAssistant' in fields:
            self.chkAssistant.setChecked(True)
            self.chkHasAssistant.setChecked(fields['hasAssistant'])
        if 'context' in fields:
            self.chkContext.setChecked(True)
            self.edtContext.text(fields['context'])
        if 'isPreferable' in fields:
            self.chkPreferable.setChecked(True)
            self.chkIsPreferable.setChecked(fields['isPreferable'])
        if 'maxOccursInEvent' in fields:
            self.chkMaxOccursInEvent.setChecked(True)
            self.edtMaxOccursInEvent.setValue(fields['maxOccursInEvent'])
        if 'serviceType' in fields:
            self.chkServiceType.setChecked(True)
            self.cmbServiceType.setValue(fields['serviceType'])
        if 'exposeDateSelector' in fields:
            self.chkExposeDateSelector.setChecked(True)
            self.cmbExposeDateSelector.setCurrentIndex(fields['exposeDateSelector'])
        if 'defaultStatus' in fields:
            self.chkDefaultStatus.setChecked(True)
            self.cmbDefaultStatus.setCurrentIndex(fields['defaultStatus'])
        if 'defaultPlannedEndDate' in fields:
            self.chkDefaultPlannedEndDate.setChecked(True)
            self.cmbDefaultPlannedEndDate.setCurrentIndex(fields['defaultPlannedEndDate'])
        if 'defaultEndDate' in fields:
            self.chkDefaultEndDate.setChecked(True)
            self.cmbDefaultEndDate.setCurrentIndex(fields['defaultEndDate'])
        if 'defaultDirectionDate' in fields:
            self.chkDefaultDirectionDate.setChecked(True)
            self.cmbDefaultDirectionDate.setCurrentIndex(fields['defaultDirectionDate'])
        if 'defaultExecPerson_id' in fields:
            self.chkDefaultExecPerson.setChecked(True)
            self.cmbDefaultExecPerson.setValue(fields['defaultExecPerson_id'])
        if 'defaultPersonInEvent' in fields:
            self.chkDefaultPersonInEvent.setChecked(True)
            self.cmbDefaultPersonInEvent.setCurrentIndex(fields['defaultPersonInEvent'])
        if 'defaultPersonInEditor' in fields:
            self.chkDefaultPersonInEditor.setChecked(True)
            self.cmbDefaultPersonInEditor.setCurrentIndex(fields['defaultPersonInEditor'])
        if 'defaultMKB' in fields:
            self.chkDefaultMKB.setChecked(True)
            self.cmbDefaultMKB.setCurrentIndex(fields['defaultMKB'])
        if 'isMorphologyRequired' in fields:
            self.chkIsMorphologyRequired.setChecked(True)
            self.cmbIsMorphologyRequired.setCurrentIndex(fields['isMorphologyRequired'])
        if 'defaultOrg_id' in fields:
            self.chkDefaultOrg.setChecked(True)
            self.cmbDefaultOrg.setValue(fields['defaultOrg_id'])
        if 'amount' in fields:
            self.chkAmount.setChecked(True)
            self.edtAmount.setValue(fields['amount'])
        if 'office' in fields:
            self.chkOffice.setChecked(True)
            self.edtOffice.setText(fields['office'])
        if 'propertyAssignedVisible' in fields:
            self.grpPropertiesFields.setChecked(True)
            self.chkPropertyAssignedVisible.setChecked(fields['propertyAssignedVisible'])
            self.chkPropertyUnitVisible.setChecked(fields['propertyUnitVisible'])
            self.chkPropertyNormVisible.setChecked(fields['propertyNormVisible'])
            self.chkPropertyEvaluationVisible.setChecked(fields['propertyEvaluationVisible'])

    def getSettings(self):
        if not self.confirmed:
            return {}
        fields = {'id':None}
        if self.chkSex.isChecked():
            fields['sex'] = self.cmbSex.currentIndex()
        if self.chkAge.isChecked():
            fields['age'] = composeAgeSelector(self.cmbBegAgeUnit.currentIndex(),  forceStringEx(self.edtBegAgeCount.text()),
                                               self.cmbEndAgeUnit.currentIndex(),  forceStringEx(self.edtEndAgeCount.text()))
        if self.chkChkShowTime.isChecked():
            fields['showTime'] = self.chkShowTime.isChecked()
        if self.chkChkRequiredCoordination.isChecked():
            fields['isRequiredCoordination'] = self.chkRequiredCoordination.isChecked()
        if self.chkChkShowInForm.isChecked():
            fields['showInForm'] = self.chkShowInForm.isChecked()
        if self.chkAssistant.isChecked():
            fields['hasAssistant'] = self.chkHasAssistant.isChecked()
        if self.chkContext.isChecked():
            fields['context'] = forceStringEx(self.edtContext.text())
        if self.chkPreferable.isChecked():
            fields['isPreferable'] = self.chkIsPreferable.isChecked()
        if self.chkMaxOccursInEvent.isChecked():
            fields['maxOccursInEvent'] = self.edtMaxOccursInEvent.value()
        if self.chkServiceType.isChecked():
            fields['serviceType'] = self.cmbServiceType.value()
        if self.chkExposeDateSelector.isChecked():
            fields['exposeDateSelector'] = self.cmbExposeDateSelector.currentIndex()
        if self.chkDefaultStatus.isChecked():
            fields['defaultStatus'] = self.cmbDefaultStatus.currentIndex()
        if self.chkDefaultPlannedEndDate.isChecked():
            fields['defaultPlannedEndDate'] = self.cmbDefaultPlannedEndDate.currentIndex()
        if self.chkDefaultEndDate.isChecked():
            fields['defaultEndDate'] = self.cmbDefaultEndDate.currentIndex()
        if self.chkDefaultDirectionDate.isChecked():
            fields['defaultDirectionDate'] = self.cmbDefaultDirectionDate.currentIndex()
        if self.chkDefaultExecPerson.isChecked():
            fields['defaultExecPerson_id'] = self.cmbDefaultExecPerson.value()
        if self.chkDefaultPersonInEvent.isChecked():
            fields['defaultPersonInEvent'] = self.cmbDefaultPersonInEvent.currentIndex()
        if self.chkDefaultPersonInEditor.isChecked():
            fields['defaultPersonInEditor'] = self.cmbDefaultPersonInEditor.currentIndex()
        if self.chkDefaultMKB.isChecked():
            fields['defaultMKB'] = self.cmbDefaultMKB.currentIndex()
        if self.chkIsMorphologyRequired.isChecked():
            fields['isMorphologyRequired'] = self.cmbIsMorphologyRequired.currentIndex()
        if self.chkDefaultOrg.isChecked():
            fields['defaultOrg_id'] = self.cmbDefaultOrg.value()
        if self.chkAmount.isChecked():
            fields['amount'] = self.edtAmount.value()
        if self.chkOffice.isChecked():
            fields['office'] = forceStringEx(self.edtOffice.text())
        if self.grpPropertiesFields.isChecked():
            fields['propertyAssignedVisible'] = self.chkPropertyAssignedVisible.isChecked()
            fields['propertyUnitVisible'] = self.chkPropertyUnitVisible.isChecked()
            fields['propertyNormVisible'] = self.chkPropertyNormVisible.isChecked()
            fields['propertyEvaluationVisible'] = self.chkPropertyEvaluationVisible.isChecked()
        return fields


def insertIgnoreRecord(db, table, record):
    db.checkdb()
    table = db.forceTable(table)
    table.beforeInsert(record)
    fieldsCount = record.count()
    fields = []
    values = []
    for i in range(fieldsCount):
        if not record.value(i).isNull():
            fields.append(db.escapeFieldName(record.fieldName(i)))
            values.append(db.formatValue(record.field(i)))
    stmt = ('INSERT IGNORE INTO ' +  table.name() +
            '(' + (', '.join(fields)) + ') '+
            'VALUES (' + (', '.join(values)) + ')')
###        print unicode(stmt).encode('cp866')
    id = db.query(stmt).lastInsertId().toInt()[0]
    idFieldName = table.idFieldName()
    record.setValue(idFieldName, QVariant(id))
    return id
