# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2016-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
##
## Описание присоединённого файла, который хранится где-то
## и модель списка таких файлов.
##
#############################################################################

import posixpath
import locale

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, SIGNAL, QAbstractTableModel, QModelIndex, QVariant

from library.Utils import forceRef, forceString
from library.naturalSort import convertKeyForNaturalSort

class CAttachedFile:
    u'Описание присоединённого файла, который хранится где-то (в WebDAV)'

    def __init__(self):
        self.persistentDir = None
        self.tmpDir        = None
        self.oldName       = None
        self.newName       = None

        self.size          = None
        self.lastModified  = None
        self.authorId      = None
        self.author        = None
        self.isLost        = False


    def _setRemoteFile(self, filePath, size, lastModified):
        self.persistentDir, self.oldName = posixpath.split(filePath)
        self.size = size
        self.lastModified = lastModified
        self.newName = self.oldName
        self.tmpDir  = None


    def _setLostFile(self, filePath):
        self.persistentDir, self.oldName = posixpath.split(filePath)
        self.newName = self.oldName
        self.tmpDir  = None
        self.isLost = True


    def _setTmpFile(self, filePath, size, lastModified):
        self.tmpDir, self.oldName = posixpath.split(filePath)
        self.size = size
        self.lastModified = lastModified
        self.newName = self.oldName
        self.persistentDir = None
        self.isLost = False


    def setAuthorId(self, authorId):
        self.authorId = authorId
        db = QtGui.qApp.db
        self.author = forceString(db.translate('vrbPersonWithSpeciality', 'id', authorId, 'name'))


    @classmethod
    def remoteFile(cls, path, size, lastModified):
        result = cls()
        result._setRemoteFile(path, size, lastModified)
        return result


    @classmethod
    def lostFile(cls, path):
        result = cls()
        result._setLostFile(path)
        return result


    @classmethod
    def tmpFile(cls, path, size, lastModified):
        result = cls()
        result._setTmpFile(path, size, lastModified)
        result.setAuthorId(QtGui.qApp.userId)
        return result


    def rename(self, newName):
        self.newName = newName


    def getPath(self):
        return posixpath.join(self.persistentDir if self.persistentDir else self.tmpDir,
                              self.oldName
                             )


class CAttachedFilesLoader:

    @staticmethod
    def itemsPresent(interface, tableName, masterId):
        if interface:
            db = QtGui.qApp.db
            table = db.table(tableName)
            cond = db.joinAnd([table['deleted'].eq(0), table['master_id'].eq(masterId)])
            return db.getCount(table, countCol='1', where=cond)
        else:
            return 0


    @staticmethod
    def loadItems(interface, tableName, masterId):
        if interface:
            db = QtGui.qApp.db
            table = db.table(tableName)
            cond = db.joinAnd([table['deleted'].eq(0), table['master_id'].eq(masterId)])
            records = db.getRecordList(table, ['path', 'createPerson_id'], cond)
            result = []
            for record in records:
                path = forceString(record.value('path'))
                item = interface.createAttachedFileItem(path)
                item.setAuthorId(forceRef(record.value('createPerson_id')))
                result.append(item)
            return result
        else:
            return []


    @staticmethod
    def saveItems(interface, tableName, masterId, items):
        if interface:
            attachedFileSet = set(interface.saveFiles(items))
            db = QtGui.qApp.db
            table = db.table(tableName)
            cond = db.joinAnd([table['deleted'].eq(0),
                               table['master_id'].eq(masterId),
                               table['path'].notInlist(attachedFileSet)
                              ])
            db.deleteRecord(table, cond)
            cond = db.joinAnd([table['deleted'].eq(0),
                               table['master_id'].eq(masterId),
                              ])
            records = db.getRecordList(table, 'path', cond)
            for record in records:
                path = forceString(record.value('path'))
                attachedFileSet.discard(path)
            for path in attachedFileSet:
                record = table.newRecord()
                record.setValue('master_id', masterId)
                record.setValue('deleted', 0)
                record.setValue('path', path)
                db.insertRecord(table, record)


class CAttachedFilesModel(QAbstractTableModel):
    u'Список прикреплённых файлов'

    def __init__(self, parent):
        QAbstractTableModel.__init__(self)
        self.interface = None
        self.tableName = None
        self.items = []


    def setInterface(self, interface):
        self.interface = interface


    def setTable(self, tableName):
        self.tableName = tableName


    def loadItems(self, masterId):
        self.items = CAttachedFilesLoader.loadItems(self.interface, self.tableName, masterId)
        self.reset()


    def saveItems(self, masterId):
        CAttachedFilesLoader.saveItems(self.interface, self.tableName, masterId, self.items)


    def columnCount(self, index=None):
        return 4


    def rowCount(self, index=None):
        return len(self.items)


    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return QVariant((u'Имя файла', u'Размер', u'Дата', u'Автор')[section])
        return QVariant()


#    def flags(self, index):
#        row = index.row()
#        item = self.items[row]
#        if item.isLost:
#            return Qt.ItemIsSelectable
#        else:
#            return Qt.ItemIsSelectable | Qt.ItemIsEnabled


    def data(self, index, role=Qt.DisplayRole):
        if role == Qt.DisplayRole:
            row = index.row()
            column = index.column()
            item = self.items[row]
            if column == 0:
                return QVariant(item.newName)
            if column == 1:
                return QVariant(item.size)
            if column == 2:
                return QVariant(item.lastModified)
            if column == 3:
                return QVariant(item.author)
        elif role == Qt.TextAlignmentRole:
            column = index.column()
            if column == 0:
                return QVariant(Qt.AlignLeft|Qt.AlignVCenter)
            if column == 1:
                return QVariant(Qt.AlignRight|Qt.AlignVCenter)
            if column == 2:
                return QVariant(Qt.AlignCenter|Qt.AlignVCenter)
            return QVariant(Qt.AlignLeft|Qt.AlignVCenter)

        elif role == Qt.DecorationRole:
            row = index.row()
            column = index.column()
            item = self.items[row]
            if column == 0:
                if item.isLost:
                    return QVariant(QtGui.qApp.style().standardIcon(QtGui.QStyle.SP_MessageBoxWarning))
                else:
#                    return QVariant(QtGui.QIcon(QtGui.QPixmap(row*10,row*10)))
                    return QVariant(QtGui.QColor(0, 0, 0, 0))
#        elif role == Qt.DecorationRole:

        return QVariant()


    def sort(self, column, order = Qt.AscendingOrder):
#        def prepKey(s):
#            return ( convertKeyForNaturalSort(s.upper().replace(u'Ё', u'Е'))
#                     + unichr(0xFFFF)
#                     + s
#                   )
        def prepKey(s):
            return locale.strxfrm(convertKeyForNaturalSort(s))

        keys = { 0: lambda item: prepKey(item.newName),
                 1: lambda item: item.size,
                 2: lambda item: item.lastModified,
                 3: lambda item: prepKey(item.author)
               }
        self.items.sort( key=keys[column],
                         reverse = order != Qt.AscendingOrder
                       )
        self.reset()


    def removeRows(self, row,  count, parent = QModelIndex()):
        self.beginRemoveRows(parent, row, row+count-1)
        try:
            del self.items[row: row+count]
            return True
        except:
            return False
        finally:
            self.endRemoveRows()


    def isNotEmpty(self):
        return bool(self.items)


#    def append(self, item):
#        self.items.append(item)


    def setAttachedFileItemList(self, attachedFileItemList):
        self.items = attachedFileItemList
        self.reset()


    def uploadFiles(self, localFileList):
        for localFile in localFileList:
            fileItem = self.interface.uploadFile(localFile)
            if fileItem:
                self.beginInsertRows(QModelIndex(), len(self.items), len(self.items)+1)
                self.items.append( fileItem )
                self.endInsertRows()

#?
    def saveFiles(self):
        return self.interface.saveFiles(self.items)


    def indexOfItem(self, item):
        return self.items.index(item)


    def renameFile(self, row, newName):
        self.items[row].rename(newName)
        self.emit(SIGNAL('dataChanged()'),
                  self.index(row, 0),
                  self.index(row, self.columnCount()-1)
                 )

