#! /usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
##
## CSmartCardWatcher - простой класс для наблюдения за смарт-картами.
## Мне не понравился подход pyscard (типа CardMonitor или ReaderMonitor )
## так как они используют потоки (которые в python умеренно полезны),
## а в нашем приложении всегда найдётся время для idle
##
#############################################################################

#http://www.cardwerk.com/smartcards/smartcard_standard_ISO7816-4_5_basic_organizations.aspx
#http://www.musclecard.com
#http://sourceforge.net/projects/pyscard

import weakref
from sys import exc_info
from smartcard.CardRequest import CardRequest


__all__ = ( 'CSmartCardWatcher',
          )


class CSmartCardWatcher:
    def __init__(self):
        self.watchedCards = []
        self.subscribers  = []
        self.cardRequest = CardRequest(timeout=0)

# Я бы с большим удовольствием сделал регистрацию call-back-а,
# мне одновременно кажется привлекательной идея weakref
# (для исключения циркулярных ссылок) и использование callable.
# но использование weakref с bound method и тем более - с lambda
# встречает серьёзные затруднения.
# Так что будем обходиться экземплярами классов.
#
# subscriber должен предоставлять метод addSmartCardNotice
# подписчики извещаются в порядке, обратном добавлению
# если подписчик возврашает True, по другие подписчики
# более не извещаются.
# Это сделано для модальных диалогов.

    def addSubscriber(self, subscriber):
        self.subscribers = [ref for ref in self.subscribers if ref]
        self.subscribers.append(weakref.ref(subscriber))


    def removeSubscriber(self, subscriber):
        self.subscribers = [ref
                            for ref in self.subscribers
                            if ref() and ref() is not subscriber
                           ]

    def watch(self):
        try:
            currentCards = self.cardRequest.waitforcardevent()
        except:
            currentCards = []

        addedCards = [ card for card in currentCards if card not in self.watchedCards ]
#        removedCards = [ card for card in self.watchedCards if card not in currentCards ]
        self.watchedCards = currentCards
        self.processAddedCards(addedCards)


    def processAddedCards(self, addedCards):
        for card in addedCards:
            connection = card.createConnection()
            try:
                connection.connect()
                try:
                    for ref in reversed(self.subscribers):
                        subscriber = ref()
                        if subscriber and subscriber.addSmartCardNotice(connection):
                            break
                except:
                    pass
                    print exc_info()[0], ':', exc_info()[1]
                finally:
                    connection.disconnect()
            except:
                pass
                print exc_info()[0], ':', exc_info()[1]


if __name__ == '__main__':
    from time import sleep, clock
    from PolicySmartCard import CPolicySmartCardConnection

    class CPolicySmartCardSubscriber:
        def addSmartCardNotice(self, connection):
            print connection
            c = CPolicySmartCardConnection(connection)
            try:
                t0 = clock()
                oi = c.getOwnerInformation()
                t1 = clock()

                print 'lastName'.ljust(30,'.'),     oi.get('lastName')
                print 'firstName'.ljust(30,'.'),    oi.get('firstName')
                print 'patrName'.ljust(30,'.'),     oi.get('patrName')
                print 'birthDate'.ljust(30,'.'),    oi.get('birthDate')
                print 'sex'.ljust(30,'.'),          oi.get('sex')
                print 'birthPlace'.ljust(30,'.'),   oi.get('birthPlace')

                print 'SNILS'.ljust(30,'.'),        oi.get('SNILS')

                print 'policyNumber'.ljust(30,'.'), oi.get('policyNumber')
                print 'issueDate'.ljust(30,'.'),    oi.get('issueDate')
                print 'expireDate'.ljust(30,'.'),   oi.get('expireDate')

                citizenship = oi.get('citizenship')
                if citizenship:
                    print 'citizenship.counryCode'.ljust(30,'.'), citizenship.get('counryCode')
                    print 'citizenship.coutryCyrillicName'.ljust(30,'.'), citizenship.get('coutryCyrillicName')

                print ''.ljust(30,'-')
                print 'access time =', t1-t0

                print '='*80
                t0 = clock()
                pi = c.getCurrentSMOInformation()
                t1 = clock()
                print 'OGRN'.ljust(30,'.'),    pi.get('OGRN')
                print 'OKATO'.ljust(30,'.'),   pi.get('OKATO')
                print 'begDate'.ljust(30,'.'), pi.get('begDate')
                print 'endDate'.ljust(30,'.'), pi.get('endDate')
                print ''.ljust(30,'-')
                print 'access time =', t1-t0

                return True
            except:
                pass


    cw = CSmartCardWatcher()
    ps = CPolicySmartCardSubscriber()

    cw.addSubscriber(ps)
    for i in xrange(30):
        print 'try#',i
        cw.watch()
        sleep(3)
    cw.removeSubscriber(ps)
