#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import library.patches

import codecs
import gc
import locale
import os.path
import re
import serial
import sip
import shutil
import sys
import tempfile
import time
import traceback
from optparse import OptionParser

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, pyqtSignature, SIGNAL, QByteArray, QDateTime, QDir, QEvent, qInstallMsgHandler, QObject, QProcess, QThread, QTime, QTimer, QTranslator, QVariant

from library                            import database
from library.Preferences                import CPreferences
from library.SerialSettings             import CSerialSettings
from library.Calendar                   import CCalendarInfo
from library.SendMailDialog             import sendMail
from library.getHostName                import getHostName
from library.DbfViewDialog              import viewDbf
from library.Attach.WebDAVInterface     import CWebDAVInterface
from library.Utils                      import anyToUnicode, exceptionToUnicode, forceBool, forceInt, forceRef, forceString, forceStringEx, getPref, quote, setPref, toVariant, withWaitCursor

try:
    from library.SmartCard.SmartCardWatcher import CSmartCardWatcher
    from library.SmartCard.PolicySmartCard  import CPolicySmartCardConnection
    gSmartCardAvailable = True
except:
    gSmartCardAvailable = False

from KLADR.Utils                        import getProvinceKLADRCode

from preferences.appPreferencesDialog   import CAppPreferencesDialog, DefaultAverageDuration
from preferences.CalendarExceptionList  import CCalendarExceptionList
from preferences.connection             import CConnectionDialog
from preferences.decor                  import CDecorDialog

from Accounting.AccountingDialog        import CAccountingDialog
from Accounting.CashBookDialog          import CCashBookDialog

from Blank.BlanksDialog                 import CBlanksDialog

from DataCheck.CheckClients             import ClientsCheck
from DataCheck.CheckEvents              import CEventsCheck
from DataCheck.CheckTempInvalid         import TempInvalidCheck
from DataCheck.LogicalControlDiagnosis  import CControlDiagnosis
from DataCheck.LogicalControlDoubles    import CControlDoubles
from DataCheck.LogicalControlMes        import CLogicalControlMes

from Exchange.Export131                 import Export131
from Exchange.ExportActionResult        import ExportActionResult
from Exchange.ExportActions             import ExportActionType
from Exchange.ExportEvents              import ExportEventType
from Exchange.ExportFeedDataCsv         import exportFeedDataCsv
from Exchange.ExportHL7v2_5             import ExportHL7v2_5
from Exchange.ExportPrimaryDocInXml     import ExportPrimaryDocInXml
from Exchange.ExportRbComplain          import ExportRbComplain
from Exchange.ExportRbDiagnosticResult  import ExportRbDiagnosticResult
from Exchange.ExportRbResult            import ExportRbResult
from Exchange.ExportRbThesaurus         import ExportRbThesaurus
from Exchange.ExportRbUnit              import ExportRbUnit
from Exchange.ExportXmlEmc              import ExportXmlEmc
from Exchange.Import131Errors           import Import131Errors
from Exchange.Import131                 import Import131
from Exchange.Import131XML              import Import131XML
from Exchange.ImportActions             import ImportActionType
from Exchange.ImportDD                  import ImportDD
from Exchange.ImportEISOMS_Clients      import importClientsFromEisOms
from Exchange.ImportEISOMS_LPU          import ImportEISOMS_LPU
from Exchange.ImportEISOMS_SMO          import ImportEISOMS_SMO
from Exchange.ImportEvents              import ImportEventType
from Exchange.ImportFromSail            import ImportFromSail
from Exchange.ImportFromSailXML         import ImportFromSailXML
from Exchange.ImportLgot                import ImportLgot
from Exchange.ImportOrgsINFIS           import ImportOrgsINFIS
from Exchange.ImportPolicySerialDBF     import ImportPolicySerialDBF
from Exchange.ImportPrimaryDocFromXml   import ImportPrimaryDocFromXml
from Exchange.ImportProfiles            import ImportProfiles
from Exchange.ImportProfilesINFIS       import ImportProfilesINFIS
from Exchange.ImportQuotaFromVTMP       import ImportQuotaFromVTMP
from Exchange.ImportRbComplain          import ImportRbComplain
from Exchange.ImportRbDiagnosticResult  import ImportRbDiagnosticResult
from Exchange.ImportRbResult            import ImportRbResult
from Exchange.ImportRbThesaurus         import ImportRbThesaurus
from Exchange.ImportRbUnit              import ImportRbUnit
from Exchange.ImportServiceMes          import CImportServiceMes
from Exchange.ImportServiceNomen        import CImportServiceNomen
from Exchange.TFUnifiedIdent.Service    import CTFUnifiedIdentService

from Surveillance.SurveillanceDialog    import CSurveillanceDialog
from HospitalBeds.HospitalBedsDialog    import CHospitalBedsDialog

from Orgs.Banks                         import CBanksList
from Orgs.Contracts                     import CContractsList
from Orgs.CreateAttachClientsForArea    import CCreateAttachClientsForAreaDialog
from Orgs.Orgs                          import COrgsList
from Orgs.Utils                         import getOrganisationInfo

from Quoting.QuotingDialog              import CQuotingDialog

from RefBooks.ActionPropertyTemplate    import CActionPropertyTemplateList
from RefBooks.ActionTemplate            import CActionTemplateList
from RefBooks.ActionType                import CActionTypeList
from RefBooks.EventType                 import CEventTypeList
from RefBooks.KLADRMilitaryUnits        import CKLADRMilitaryUnitsList
from RefBooks.MKB                       import CMKBList
from RefBooks.MKBSubclass               import CMKBSubclass
from RefBooks.OrgStructure              import COrgStructureList
from RefBooks.Person                    import CPersonList
from RefBooks.QuotaType                 import CQuotaTypeList
from RefBooks.RBAccountExportFormat     import CRBAccountExportFormatList
from RefBooks.RBAccountingSystem        import CRBAccountingSystemList
from RefBooks.RBActionShedule           import CRBActionSheduleList
from RefBooks.RBActivity                import CRBActivityList
from RefBooks.RBAgreementType           import CRBAgreementTypeList
from RefBooks.RBAppointmentPurpose      import CRBAppointmentPurposeList
from RefBooks.RBMedicalBoardExpertiseCharacter import CRBMedicalBoardExpertiseCharacter
from RefBooks.RBMedicalBoardExpertiseKind      import CRBMedicalBoardExpertiseKind
from RefBooks.RBMedicalBoardExpertiseObject    import CRBMedicalBoardExpertiseObject
from RefBooks.RBMedicalBoardExpertiseArgument  import CRBMedicalBoardExpertiseArgument
from RefBooks.RBAttachType              import CRBAttachTypeList
from RefBooks.RBBlanksDialog            import CBlanksList
from RefBooks.RBBloodType               import CRBBloodTypeList
from RefBooks.RBCashOperation           import CRBCashOperationList
from RefBooks.RBClientConsentType       import CRBClientConsentTypeList
from RefBooks.RBComplain                import CRBComplainList
from RefBooks.RBContactType             import CRBContactTypeList
from RefBooks.RBContainerType           import CRBContainerTypeList
from RefBooks.RBContractAttributeType   import CRBContractAttributeTypeList
from RefBooks.RBContractCoefficientType import CRBContractCoefficientTypeList
from RefBooks.RBCounter                 import CRBCounterList
from RefBooks.RBCureMethod              import CRBCureMethod
from RefBooks.RBCureType                import CRBCureType
from RefBooks.RBDiagnosisType           import CRBDiagnosisTypeList
from RefBooks.RBDiagnosticResult        import CRBDiagnosticResultList
from RefBooks.RBDiet                    import CRBDiet
from RefBooks.RBDiseaseCharacter        import CRBDiseaseCharacterList
from RefBooks.RBDiseasePhases           import CRBDiseasePhasesList
from RefBooks.RBDiseaseStage            import CRBDiseaseStageList
from RefBooks.RBDispanser               import CRBDispanserList
from RefBooks.RBDocumentTypeForTracking import CRBDocumentTypeForTrackingList
from RefBooks.RBDocumentTypeGroup       import CRBDocumentTypeGroupList
from RefBooks.RBDocumentType            import CRBDocumentTypeList
from RefBooks.RBEmergencyAccident       import CRBEmergencyAccidentList
from RefBooks.RBEmergencyBrigade        import CRBEmergencyBrigadeList
from RefBooks.RBEmergencyCauseCall      import CRBEmergencyCauseCallList
from RefBooks.RBEmergencyDeath          import CRBEmergencyDeathList
from RefBooks.RBEmergencyDiseased       import CRBEmergencyDiseasedList
from RefBooks.RBEmergencyEbriety        import CRBEmergencyEbrietyList
from RefBooks.RBEmergencyMethodTransportation      import CRBEmergencyMethodTransportationList
from RefBooks.RBEmergencyPlaceCall                 import CRBEmergencyPlaceCallList
from RefBooks.RBEmergencyPlaceReceptionCall        import CRBEmergencyPlaceReceptionCallList
from RefBooks.RBEmergencyReasondDelays             import CRBEmergencyReasondDelaysList
from RefBooks.RBEmergencyReceivedCall              import CRBEmergencyReceivedCallList
from RefBooks.RBEmergencyResult                    import CRBEmergencyResultList
from RefBooks.RBEmergencyTransferredTransportation import CRBEmergencyTransferredTransportationList
from RefBooks.RBEmergencyTypeAsset                 import CRBEmergencyTypeAssetList
from RefBooks.RBEquipment               import CRBEquipmentList
from RefBooks.RBEquipmentType           import CRBEquipmentTypeList
from RefBooks.RBEventProfile            import CRBEventProfileList
from RefBooks.RBEventTypePurpose        import CRBEventTypePurposeList
from RefBooks.RBExpenseServiceItem      import CRBExpenseServiceItemList
from RefBooks.RBFinance                 import CRBFinanceList
from RefBooks.RBHealthGroup             import CRBHealthGroupList
from RefBooks.RBHospitalBedProfile      import CRBHospitalBedProfileList
from RefBooks.RBHurtFactorType          import CRBHurtFactorTypeList
from RefBooks.RBHurtType                import CRBHurtTypeList
from RefBooks.RBImageMap                import CRBImageMapList
from RefBooks.RBInfection               import CRBInfectionList
from RefBooks.RBIngredient              import CRBIngredientList
from RefBooks.RBJobPurpose              import CRBJobPurposeList
from RefBooks.RBJobType                 import CRBJobTypeList
from RefBooks.RBLaboratory              import CRBLaboratoryList
from RefBooks.RBLfForm                  import CRBLfFormList
from RefBooks.RBLocationCardType        import CRBLocationCardTypeList
from RefBooks.RBMealTime                import CRBMealTime
from RefBooks.RBMedicalAidKind          import CRBMedicalAidKindList
from RefBooks.RBMedicalAidProfile       import CRBMedicalAidProfileList
from RefBooks.RBMedicalAidType          import CRBMedicalAidTypeList
from RefBooks.RBMedicalAidUnit          import CRBMedicalAidUnitList
from RefBooks.RBMedicalExemptionType    import CRBMedicalExemptionTypeList
from RefBooks.RBMenu                    import CRBMenu
from RefBooks.RBMesSpecification        import CRBMesSpecificationList
from RefBooks.RBMKBMorphology           import CRBMKBMorphologyList
from RefBooks.RBNet                     import CRBNetList
from RefBooks.RBNomenclatureClass       import CRBNomenclatureClassList
from RefBooks.RBNomenclature            import CRBNomenclatureList
from RefBooks.RBNomenclatureKind        import CRBNomenclatureKindList
from RefBooks.RBNomenclatureType        import CRBNomenclatureTypeList
from RefBooks.RBOKFS                    import CRBOKFSList
from RefBooks.RBOKPF                    import CRBOKPFList
from RefBooks.RBPatientModel            import CRBPatientModel
from RefBooks.RBPayRefuseType           import CRBPayRefuseTypeList
from RefBooks.RBPlanningHospitalActivity import CRBPlanningHospitalActivityList
from RefBooks.RBPolicyKind              import CRBPolicyKindList
from RefBooks.RBPolicyType              import CRBPolicyTypeList
from RefBooks.RBPost                    import CRBPostList
from RefBooks.RBPrintTemplate           import CRBPrintTemplate
from RefBooks.RBReaction                import CRBReactionList
from RefBooks.RBReasonOfAbsence         import CRBReasonOfAbsenceList
from RefBooks.RBRelativeEditor          import CRBRelativeList
from RefBooks.RBResult                  import CRBResultList
from RefBooks.RBScene                   import CRBSceneList
from RefBooks.RBServiceGroup            import CRBServiceGroupList
from RefBooks.RBService                 import CRBServiceList
from RefBooks.RBServiceType             import CRBServiceTypeList
from RefBooks.RBSocStatusClass          import CRBSocStatusClassList
from RefBooks.RBSocStatusType           import CRBSocStatusTypeList
from RefBooks.RBSpeciality              import CRBSpecialityList
from RefBooks.RBSpecimenType            import CRBSpecimenTypeList
from RefBooks.RBStatusObservationClientType import CRBStatusObservationClientTypeList
from RefBooks.RBStockRecipe             import CRBStockRecipeList
from RefBooks.RBStockMotionNumber             import CRBStockMotionNumberList
from RefBooks.RBSuiteReagent            import CRBSuiteReagentList
from RefBooks.RBTariffCategory          import CRBTariffCategoryList
from RefBooks.RBTempInvalidBreak        import CRBTempInvalidBreakList
from RefBooks.RBTempInvalidDocument     import CRBTempInvalidDocumentList
from RefBooks.RBTempInvalidDuplicateReason     import CRBTempInvalidDuplicateReasonList
from RefBooks.RBTempInvalidExtraReason  import CRBTempInvalidExtraReasonList
from RefBooks.RBTempInvalidReason       import CRBTempInvalidReasonList
from RefBooks.RBTempInvalidRegime       import CRBTempInvalidRegimeList
from RefBooks.RBTempInvalidResult       import CRBTempInvalidResultList
from RefBooks.RBTestGroup               import CRBTestGroupList
from RefBooks.RBTest                    import CRBTestList
from RefBooks.RBThesaurus               import CRBThesaurus
from RefBooks.RBTissueType              import CRBTissueTypeList
from RefBooks.RBTraumaType              import CRBTraumaTypeList
from RefBooks.RBUnit                    import CRBUnitList
from RefBooks.RBVaccinationCalendar     import CRBVaccinationCalendarList
from RefBooks.RBVaccinationProbe        import CRBVaccinationProbeList
from RefBooks.RBVaccine                 import CRBVaccineList
from RefBooks.RBVaccineSchemaTransitionType import CRBVaccineSchemaTransitionTypeList
from RefBooks.RBVisitType               import CRBVisitTypeList
from RefBooks.RBActionTypeGroup import CRBActionTypeGroupAppointment

from Registry.DiagnosisDock             import CDiagnosisDockWidget
from Registry.FreeQueueDock             import CFreeQueueDockWidget
from Registry.RegistryWindow            import CRegistryWindow
from Registry.ResourcesDock             import CResourcesDockWidget
from Registry.SuspendedAppointment      import CSuspenedAppointmentWindow

from Reports.ClientNomenclatureInvoiceReport import CClientNomenclatureInvoiceReport
from Reports.ClientNomenclatureActionReport import CClientNomenclatureActionReport
from Reports.NomenclatureBook import CNomenclatureBook
from Reports.ReportNomenclaureMotions import CReportNomenclaureMotions
from Reports.MonthlyNomenclatureReport import CMonthlyNomenclatureReport
from Reports.PlannedClientInvoiceNomenclaturesReport import CPlannedClientInvoiceNomenclaturesReport
from Reports.ActionPropertiesTestsReport import CActionPropertiesTestsReport
from Reports.AnaliticReportsAdditionalSurgery   import CAnaliticReportsAdditionalSurgery
from Reports.AnaliticReportsChildrenLeaved import CAnaliticReportsChildrenLeaved
from Reports.AnaliticReportsDeathStationary     import CAnaliticReportsDeathStationary
from Reports.AnaliticReportsGeneralInfoSurgery  import CAnaliticReportsGeneralInfoSurgery
from Reports.AnaliticReportsStationary          import CAnaliticReportsStationary
from Reports.AnaliticReportsSurgery             import CAnaliticReportsSurgery
from Reports.AnaliticReportsSurgeryStationary   import CAnaliticReportsSurgeryStationary
from Reports.AnalyticsExecutionMes      import showCheckMesDescription
from Reports.AnalyticsReportHospitalizedClients import CAnalyticsReportHospitalizedClients
from Reports.AnalyticsReportIncomeAndLeavedClients import CAnalyticsReportIncomeAndLeavedClients
from Reports.AttachedContingent         import CAttachedContingent
from Reports.BySMOContingent            import CBySMOContingent
from Reports.DailyJournalBeforeRecord   import CDailyJournalBeforeRecord, CDailyJournalBeforeRecord2
from Reports.DeathList                  import CDeathList
from Reports.DeathReportByZones         import CDeathReportByZones, CDetailedDeathReportByZones
from Reports.DeathReport                import CDeathReport
from Reports.DeathSurvey                import CDeathSurvey
from Reports.DispObservationList        import CDispObservationList
from Reports.DispObservationSurvey      import CDispObservationSurvey
from Reports.StatReportF12Clients       import CStatReportF12Clients
from Reports.EventResultList            import CReportEventResultList
from Reports.EventResultSurvey          import CEventResultSurvey
from Reports.FactorRateSurvey           import CFactorRateSurvey
from Reports.FinanceSumByServicesExpenses import CFinanceSumByServicesExpensesEx
from Reports.FinanceSummaryByDoctors    import CFinanceSummaryByDoctorsEx
from Reports.FinanceSummaryByDoctorsOld import CFinanceSummaryByDoctorsExOld
from Reports.FinanceSummaryByRejections import CFinanceSummaryByRejectionsEx
from Reports.FinanceSummaryByServices   import CFinanceSummaryByServicesEx
from Reports.JournalBeforeRecordDialog  import CJournalBeforeRecordEx, CJournalBeforeRecordAV
from Reports.MUOMSOFCity                import CMUOMSOFCity
from Reports.MUOMSOFForeign             import CMUOMSOFForeign
from Reports.OutgoingDirectionsReport   import COutgoingDirectionsReport
from Reports.PersonVisits               import CPersonVisits
from Reports.PopulationStructure        import CPopulationStructure
from Reports.PreRecordClientsCard       import CPreRecordClientsCard
from Reports.PreRecordDoctors           import CPreRecordDoctorsEx
from Reports.PreRecordPlanExecutionByDoctors           import CPreRecordPlanExecutionByDoctors
from Reports.PreRecordSpeciality        import CPreRecordSpecialityEx
from Reports.PreRecordUsers             import CPreRecordUsersEx
from Reports.ReportAccountDoneActions   import CReportAccountDoneActions
from Reports.ReportActionsByOrgStructure import CReportActionsByOrgStructure
from Reports.ReportActionsByServiceType import CReportActionsByServiceType
from Reports.ReportActions              import CReportActions
from Reports.ReportActionsServiceCutaway import CReportActionsServiceCutaway
from Reports.ReportAcuteInfections      import CReportAcuteInfections
from Reports.ReportAttachingMotion      import CReportAttachingMotion
from Reports.ReportBreachSA             import CReportBreachSA
from Reports.ReportWorkloadOfMedicalStaff import CReportWorkloadOfMedicalStaff
from Reports.ReportClientActions        import CReportClientActions
from Reports.ReportClients              import CReportClients
from Reports.ReportClientSummary        import CReportClientSummary
from Reports.ReportClientTreatmentsStructure import CClientTreatmentsStructureReport
from Reports.ReportNumberResidentsAddress    import CReportNumberResidentsAddress
from Reports.ReportDailyAcuteInfections import CReportDailyAcuteInfections
from Reports.ReportDailyCash            import CReportDailyCash
from Reports.ReportDirectionActions     import CReportDirectionActions
from Reports.ReportBIRADS               import CReportBIRADS
from Reports.ReportRoadMap            import CReportRoadMap
from Reports.ReportDispansMO_1_1        import CReportDispansMO_1_1
from Reports.ReportDoctor               import CReportDoctor
from Reports.ReportDoctorPreCalc        import CReportDoctorPreCalc
from Reports.ReportDoctorSummary        import CReportDoctorSummary
from Reports.ReportDoneActions          import CReportApproxDoneActions
from Reports.ReportEmergencyCallList    import CReportEmergencyCallList
from Reports.ReportEmergencyF40         import CReportEmergencyAdditionally, CReportEmergencyF402000, CReportEmergencyF302120, CReportEmergencyF402001, CReportEmergencyF402100, CReportEmergencyF402500, CReportEmergencyF40TimeIndicators, CReportEmergencyTalonSignal
from Reports.ReportEmergencyF30         import CReportEmergencyF302350
from Reports.ReportEmergencySize        import CEmergencySizeReport
from Reports.ReportExternalDirections   import CExternalOutgoingDirectionsReport
from Reports.ReportExternalIncomePaidServices import CExternalIncomePaidServices
from Reports.ReportF1children           import CReportF1_1000, CReportF1_2000
from Reports.ReportF1RB                 import CReportF1RB_2000, CReportF1RB_AmbulatoryCare, CReportF1RB_DentalTreatment, CReportF1RB_AmbulanceCall
from Reports.ReportF30_2100             import CReportF30_2100
from Reports.ReportF30_2110             import CReportF30_2110
from Reports.ReportF30                  import CReportF30
from Reports.ReportF30_SMP              import CReportF30_SMP
from Reports.ReportF39                  import CReportF39
from Reports.ReportF62_4000             import CReportF62_4000
#from Reports.ReportF62_7000             import CReportF62_7000
from Reports.ReportForm131_o_1000_2014  import CReportForm131_o_1000_2014
from Reports.ReportForm131_o_1000_2015  import CReportForm131_o_1000_2015
from Reports.ReportForm131_o_1000_2016  import CReportForm131_o_1000_2016
from Reports.ReportForm131_o_2000_2014  import CReportForm131_o_2000_2014
from Reports.ReportForm131_o_2000_2015  import CReportForm131_o_2000_2015
from Reports.ReportForm131_o_2000_2016  import CReportForm131_o_2000_2016
from Reports.ReportForm131_o_3000_2014  import CReportForm131_o_3000_2014
from Reports.ReportForm131_o_3000_2015  import CReportForm131_o_3000_2015
from Reports.ReportForm131_o_3000_2016  import CReportForm131_o_3000_2016
from Reports.ReportForm131_o_4000_2014  import CReportForm131_o_4000_2014
from Reports.ReportForm131_o_4000_2015  import CReportForm131_o_4000_2015
from Reports.ReportForm131_o_4000_2016  import CReportForm131_o_4000_2016
from Reports.ReportForm131_o_5000_2014  import CReportForm131_o_5000_2014, CReportForm131_o_6000_2014
from Reports.ReportForm131_o_5000_2015  import CReportForm131_o_5000_2015, CReportForm131_o_5001_2015, CReportForm131_o_6000_2015
from Reports.ReportForm131_o_5000_2016  import CReportForm131_o_5000_2016, CReportForm131_o_5001_2016, CReportForm131_o_6000_2016
from Reports.ReportForm131_o_7000_2014  import CReportForm131_o_7000_2014
from Reports.ReportForm131_o_7000_2015  import CReportForm131_o_7000_2015
from Reports.ReportForm131_o_7000_2016  import CReportForm131_o_7000_2016
from Reports.ReportIncomingExternalDirections import CExternalIncomingDirectionsReport
from Reports.ReportInsuredMedicalCare   import CReportInsuredMedicalCare
from Reports.ReportInternalDirections   import CReportInternalDirections
from Reports.ReportLengthOfStayInHospital import CReportLengthOfStayInHospital
from Reports.ReportMonthActions         import CReportMonthActions
from Reports.ReportMovingAndBeds        import CReportMovingAndBeds
from Reports.ReportOnPerson             import CReportOnPerson
from Reports.ReportOnServiceType                import CReportOnServiceType
from Reports.ReportOrgStructureSummary  import CReportOrgStructureSummary
from Reports.ReportPayers               import CReportPayers
from Reports.ReportPayersWithFinance    import CReportPayersWithFinance
from Reports.ReportPersonSickList       import CReportPersonSickList
from Reports.ReportPersonSickListStationary import CReportPersonSickListStationary
from Reports.ReportSurgeryPlanDay       import CReportSurgeryPlanDay
from Reports.ReportPGG                  import CReportPGG
from Reports.ReportPolyclinicsOthersPersonList import CReportPolyclinicsOthersPersonList
from Reports.ReportsBeingInStationary    import CReportsBeingInStationary
from Reports.ReportServicesMonitoredContingent import CReportServicesMonitoredContingent
from Reports.ReportSMOClients           import CReportSMOClients
from Reports.ReportStomatF30_2700_2015  import CReportStomatF30_2700_2015
from Reports.ReportStomatF30_2700       import CReportStomatF30_2700
from Reports.ReportStomatF39_3          import CReportStomatF39_3
from Reports.ReportSummaryOnAccounts    import CReportSummaryOnAccounts
from Reports.ReportSummaryOnServices    import CReportSummaryOnServices
from Reports.ReportTreatments           import CReportTreatments
from Reports.ReportUniversalEventList   import CReportUniversalEventList
from Reports.ReportUseContainers        import CReportUseContainers
from Reports.ReportVisitByPurposeEvents import CReportVisitByPurposeEvents
from Reports.ActReconciliationMutualSettlements import CActReconciliationMutualSettlements
from Reports.ReportVisitByQueue         import CReportVisitBySchedule, CReportVisitByNextEventDate
from Reports.ReportVisitsServiceCutaway import CReportVisitsServiceCutaway
from Reports.ReportYearActions          import CReportYearActions
from Reports.ServiceAverageDurationAcuteDisease import CAverageDurationAcuteDiseaseBase
from Reports.SickRateAbort_1000         import CSickRateAbort_1000
from Reports.SickRateAbort_2000         import CSickRateAbort_2000
from Reports.SickRateAbort_3000         import CSickRateAbort_3000
from Reports.SickRateAbort              import CSickRateAbort
from Reports.SickRateSurvey             import CSickRateSurvey
from Reports.SocStatus                  import CSocStatus
from Reports.SpendingToClients          import CSpendingToClients
from Reports.StationaryDeliveryChannels import CDeliveryChannelsReport
from Reports.StationaryDiseasesAnalysis import CDiseasesAnalysisReport
from Reports.StationaryF007DS           import CStationaryF007DSClientList, CStationaryF007DSMoving
from Reports.StationaryF007             import CStationaryF007ClientList, CStationaryF007Moving
from Reports.StationaryF014             import CStationaryF14_4300_4301_4302, CStationaryAdultF142000, CStationaryAdultNoSeniorF142000, CStationaryChildrenF142000, CStationaryF142100, CStationaryF143000, CStationaryF144000, CStationaryF144001, CStationaryF144001A, CStationaryF144002, CStationaryF144100, CStationaryF144200, CStationaryF144201, CStationaryF144202, CStationaryF144400, CStationarySeniorF142000
from Reports.StationaryF016_02          import CStationaryF016_02
from Reports.StationaryF016ForPeriod    import CStationaryF016ForPeriod
from Reports.StationaryF016             import CStationaryF016
from Reports.StationaryF14DC            import CStationaryOne_2015F14DC, CStationaryOneF14DC, CStationaryOneHospitalF14DC, CStationaryOneHouseF14DC, CStationaryOnePolyclinicF14DC, CStationaryTwoAdult_2015F14DC, CStationaryTwoAdultF14DC, CStationaryTwoAdultHospitalF14DC, CStationaryTwoAdultHouseF14DC, CStationaryTwoAdultPoliclinicF14DC, CStationaryTwoChildren_2015F14DC, CStationaryTwoChildrenF14DC, CStationaryTwoChildrenHospitalF14DC, CStationaryTwoChildrenHouseF14DC, CStationaryTwoChildrenPoliclinicF14DC, CStationaryTypePaymentF14DC
from Reports.StationaryF30              import CStationaryF30_3101, CStationaryF30Moving
from Reports.StationaryLengthOfStay     import CLengthOfStayReport
from Reports.StationaryMESOneF14DC      import CStationaryMESHospitalF14DC, CStationaryMESHouseF14DC, CStationaryMESOneF14DC, CStationaryMESPolyclinicF14DC
from Reports.StationaryPatientsCompositionByRegion import CStationaryPatientsCompositionByRegion
from Reports.StationaryPatientsComposition import CStationaryPatientsComposition
from Reports.StationaryPlanning         import CStationaryPlanMoving
from Reports.StationaryReportForMIACHard import CStationaryReportForMIACHard
from Reports.StationaryReportForMIAC    import CStationaryReportForMIAC
from Reports.StationaryTallySheetMoving import CStationaryTallySheetMoving
from Reports.StatReport1DD1000          import CStatReport1DD1000
from Reports.StatReport1DD2000          import CStatReport1DD2000
from Reports.StatReport1DD3000          import CStatReport1DD3000
from Reports.StatReport1DDAll           import CStatReport1DDAll
from Reports.StatReport1NP2000          import CStatReport1NP2000
from Reports.StatReport1NP3000          import CStatReport1NP3000
from Reports.StatReport1NP4000          import CStatReport1NP4000
from Reports.StatReport1NP5000          import CStatReport1NP5000
from Reports.StatReport1NP7000          import CStatReport1NP7000
from Reports.StatReportDiseaseInPermille import CStatReportDiseaseInPermille
from Reports.StatReportEED              import CStatReportEEDMonth, CStatReportEEDMonthSummary, CStatReportEEDPeriod, CStatReportEEDPeriodSummary, CStatReportEEDYear, CStatReportEEDYearSummary
from Reports.StatReportF12Able_2015     import CStatReportF12Able_2015
from Reports.StatReportF12Able_2016     import CStatReportF12Able_2016
from Reports.StatReportF12Able          import CStatReportF12Able
from Reports.StatReportF12Adults_2015   import CStatReportF12Adults_2015
from Reports.StatReportF12Adults_2016   import CStatReportF12Adults_2016
from Reports.StatReportF12Adults        import CStatReportF12Adults
from Reports.StatReportF12Children_2015 import CStatReportF12Children_2015
from Reports.StatReportF12Children_2016 import CStatReportF12Children0_14_2016, CStatReportF12Children0_1_2016
from Reports.StatReportF12Children      import CStatReportF12Children
from Reports.StatReportF12_D_1_07       import CStatReportF12_D_1_07
from Reports.StatReportF12_D_1_08       import CStatReportF12_D_1_08
from Reports.StatReportF12_D_1_10       import CStatReportF12_D_1_10
from Reports.StatReportF12_D_2_07       import CStatReportF12_D_2_07
from Reports.StatReportF12_D_2_10       import CStatReportF12_D_2_10
from Reports.StatReportF12_D_3_M        import CStatReportF12_D_3_M
from Reports.StatReportF12Inset2008     import CStatReportF12Inset2008
from Reports.StatReportF12Seniors_2015  import CStatReportF12Seniors_2015
from Reports.StatReportF12Seniors_2016  import CStatReportF12Seniors_2016
from Reports.StatReportF12Seniors       import CStatReportF12Seniors
from Reports.StatReportF12Teenagers_2015 import CStatReportF12Teenagers_2015
from Reports.StatReportF12Teenagers_2016 import CStatReportF12Teenagers_2016
from Reports.StatReportF12Teenagers     import CStatReportF12Teenagers
from Reports.StatReportF131ByDD         import CStatReportF131ByDD
from Reports.StatReportF131ByDoctors    import CStatReportF131ByDoctors
from Reports.StatReportF131ByEmployer   import CStatReportF131ByEmployer
from Reports.StatReportF131ByRaion      import CStatReportF131ByRaion
from Reports.StatReportF131Raion        import CStatReportF131Raion
from Reports.StatReportF4_D_For_Teenager import CStatReportF4_D_For_Teenager
from Reports.StatReportF57              import CStatReportF57, CStatReportF57_1000, CStatReportF57_2000, CStatReportF57_3000, CStatReportF57_3500
from Reports.StatReportF5_D_For_Teenager import CStatReportF5_D_For_Teenager
from Reports.StatReportF63              import CStatReportF63
from Reports.StatReportF71              import CStatReportF71
from Reports.StatReportF9_2000          import CStatReportF9_2000
from Reports.StatReportF9_2001          import CStatReportF9_2001
from Reports.StatReportF9_2003          import CStatReportF9_2003
from Reports.StatReportF9_2005          import CStatReportF9_2005
from Reports.StatReportF9_2006          import CStatReportF9_2006
from Reports.StatReportStomatCompositeList import CStomatReportCompositeList, CStomatReportToSpecialityList
from Reports.StatReportStomatDay_2015   import CStomatDayReport_2015
from Reports.StatReportStomatNew           import CStomatReport, CStomatDayReport
from Reports.SummaryOnServiceType               import CSummaryOnServiceType
from Reports.TempInvalidBookF036        import CTempInvalidBookF036
from Reports.TempInvalidExpert          import CTempInvalidExpert
from Reports.TempInvalidF16             import CTempInvalidF16
from Reports.TempInvalidList            import CTempInvalidList
from Reports.TempInvalidSurvey          import CTempInvalidSurvey
from Reports.TimelineForOffices         import CTimelineForOfficesEx
from Reports.DailyReportPreRecord       import CDailyReportPreRecord
from Reports.TimelineForPerson          import CTimelineForPersonEx
from Reports.UnfinishedEventsByDoctor   import CUnfinishedEventsByDoctor
from Reports.UnfinishedEventsByMes      import CUnfinishedEventsByMes
from Reports.Workload                   import CWorkload

from Resources.JobTicketReserve         import CJobTicketReserveHolder

from Timeline.TimelineDialog            import CTimelineDialog
from TissueJournal.TissueJournalDialog  import CTissueJournalDialog

from Users.Informer                     import CInformerList, showInformer
from Users.Login                        import CLoginDialog
from Users.RightList                    import CUserRightListDialog,  CUserRightProfileListDialog
from Users.Rights                       import *
from Users.Tables                       import demoUserName, tblUser, usrLogin, usrRetired
from Users.tryKerberosAuth              import tryKerberosAuth
from Users.UserInfo                     import CUserInfo, CDemoUserInfo

try:
    from buildInfo import lastChangedRev  as gLastChangedRev, \
                          lastChangedDate as gLastChangedDate
except:
    gLastChangedRev  = 'unknown'
    gLastChangedDate = 'unknown'


from Ui_s11main     import Ui_MainWindow

gTitle = u'САМСОН'
gTitleLat = 'SAMSON'
gVersion = '2.5'
gAbout = u'Комплекс Программных Средств \n' \
        u'"Система Автоматизации Медико-Страхового Обслуживания Населения"\n' \
        u'КПС «%s»\n'   \
        u'Версия %s (ревизия %s от %s)\n' \
        u'Copyright © 2015 ООО "САМСОН Групп"\n' \
        u'распространяется под лицензией GNU GPL v.3 или выше\n' \
        u'Основано на КПС «САМСОН-ВИСТА» версии 2.0\n' \
        u'телефон тех.поддержки: (812) 418-39-70'


def getLatVersion():
    return u'%s, v.%s (rev. %s from %s)' % (gTitleLat, gVersion, gLastChangedRev, gLastChangedDate)


class CS11mainApp(QtGui.QApplication):
    __pyqtSignals__ = ('dbConnectionChanged(bool)',
                       'currentOrgIdChanged()',
                       'currentOrgStructureIdChanged()',
                       'currentClientIdChanged()',
                       'currentClientInfoChanged()',
                       'currentUserIdChanged()',
                       'scannedReceived(QByteArray)',
                       'policySmartCardReceived(QVariant)'
                      )
    inputCalculatorMimeDataType  = 'samson/inputLaboratoryCalculator'
    outputCalculatorMimeDataType = 'samson/outputLaboratoryCalculator'

    def __init__(self, args, demoModeRequest, iniFileName, disableLock, appLogMark):
        QtGui.QApplication.__init__(self, args)
        self.logDir = os.path.join(unicode(QDir.toNativeSeparators(QDir.homePath())), '.samson-vista')
        self.oldMsgHandler = qInstallMsgHandler(self.msgHandler)
        self.oldEexceptHook = sys.excepthook
        sys.excepthook = self.logException
        self.eventLogFile = None
        self.eventLogTime = 0
        self.traceActive = True
        self.homeDir= None
        self.saveDir= None
        self.userId = None
        self.hostName = getHostName()
        self.userSpecialityId = None
        self.userOrgStructureId = None
        self.demoMode = False
        self.demoModeRequested = demoModeRequest
        self.disableLockRequested = disableLock
        self.disableLock = disableLock
        self.db = None
        self.EIS_db = None
        self.mainWindow = None
        self.preferences = CPreferences(iniFileName if iniFileName else 'S11App.ini')
        self.preferences.load()
        self.preferencesDir = self.preferences.getDir()
        self.registerDocumentTables()
        self.clearPreferencesCache()
        self._currentClientId = None
        self.userInfo = None
        self.calendarInfo = CCalendarInfo(self)
        self.maxLifeDuration = 130
        self.jobTicketReserveHolder = CJobTicketReserveHolder(self)
        self._counterController = None
        self._globalPreferences = {}
        self._oldClipboardSlot = None
        self.accountingSystem = None
        self._gpSpecialityId = None   # специальность врача общей практики
        self.morphologyMKBIsVisible = None
        self.isManualSwitchDiagnosis = None
        self.isHospitalBedProfileByMoving = None
        self.isHospitalBedFinanceByMoving = None
        self.isNeedPreCreateEventPerson = None
        self._isGlobalAppLog = None
        self.scanner = None
        self.scannerTimer = None
        self.smartCardWatcher = None
        self.smartCardTimer = None
        self.appLogMark = appLogMark

        self.defaultFont = self.font()
        self.webDAVInterface = CWebDAVInterface()


    def writeAppLog(self, log, mark=u''):
        try:
            self.db.query('CALL writeAppLog(\'%s\',\'%s\')'%(log, mark or self.appLogMark))
        except:
            pass


    def connectClipboard(self, slotFunction):
        self.disconnectClipboard()
        self._oldClipboardSlot = slotFunction
        self.connect(self.clipboard(), SIGNAL('dataChanged()'), slotFunction)


    def disconnectClipboard(self):
        if self._oldClipboardSlot:
            self.disconnect(self.clipboard(), SIGNAL('dataChanged()'), self._oldClipboardSlot)
            self._oldClipboardSlot = None


    def doneTrace(self):
        self.traceActive = False
        qInstallMsgHandler(self.oldMsgHandler)
        sys.excepthook = self.oldEexceptHook


    def applyDecorPreferences(self):
        self.setStyle(self.preferences.decorStyle)
        if self.preferences.decorStandardPalette:
            self.setPalette(self.style().standardPalette())
        if self.mainWindow:
            state = Qt.WindowNoState
            if self.preferences.decorMaximizeMainWindow:
                state |= Qt.WindowMaximized
            if self.preferences.decorFullScreenMainWindow:
                state |= Qt.WindowFullScreen
            self.mainWindow.setWindowState(state)
            if self.preferences.useCustomFont:
                self.setFont(self.preferences.font)
            else:
                self.setFont(self.defaultFont)


    def registerDocumentTables(self):
        database.registerDocumentTable('Account')
        database.registerDocumentTable('Action')
        database.registerDocumentTable('Action_FileAttach')
        database.registerDocumentTable('Action_ExecutionPlan')
        database.registerDocumentTable('ActionProperty')
        database.registerDocumentTable('ActionPropertyTemplate')
        database.registerDocumentTable('ActionTemplate')
        database.registerDocumentTable('ActionType')
        database.registerDocumentTable('Address')
        database.registerDocumentTable('AddressHouse')
        database.registerDocumentTable('Bank')
        database.registerDocumentTable('BlankActions_Moving')
        database.registerDocumentTable('BlankActions_Party')
        database.registerDocumentTable('BlankTempInvalid_Moving')
        database.registerDocumentTable('BlankTempInvalid_Party')
        database.registerDocumentTable('CalendarException')
        database.registerDocumentTable('Client')
        database.registerDocumentTable('Client_FileAttach')
        database.registerDocumentTable('ClientAddress')
        database.registerDocumentTable('ClientAllergy')
        database.registerDocumentTable('ClientAttach')
        database.registerDocumentTable('ClientConsent')
        database.registerDocumentTable('ClientContact')
        database.registerDocumentTable('ClientDocument')
        database.registerDocumentTable('ClientIdentification')
        database.registerDocumentTable('ClientIntoleranceMedicament')
        database.registerDocumentTable('ClientPolicy')
        database.registerDocumentTable('Client_Quoting')
        database.registerDocumentTable('ClientRelation')
        database.registerDocumentTable('ClientSocStatus')
        database.registerDocumentTable('ClientWork')
        database.registerDocumentTable('Contract')
        database.registerDocumentTable('Diagnosis')
        database.registerDocumentTable('Diagnostic')
        database.registerDocumentTable('EmergencyCall')
        database.registerDocumentTable('Event')
        database.registerDocumentTable('Event_Feed')
        database.registerDocumentTable('Event_FileAttach')
        database.registerDocumentTable('Event_JournalOfPerson')
        database.registerDocumentTable('Event_LocalContract')
        database.registerDocumentTable('Event_Payment')
        database.registerDocumentTable('EventType')
        database.registerDocumentTable('InformerMessage')
        database.registerDocumentTable('Job')
        database.registerDocumentTable('Licence')
        database.registerDocumentTable('Notification_Rule')
        database.registerDocumentTable('Organisation')
        database.registerDocumentTable('OrgStructure')
        database.registerDocumentTable('Person')
        database.registerDocumentTable('Person_Activity')
        database.registerDocumentTable('Person_Contact')
        database.registerDocumentTable('Person_TimeTemplate')
        database.registerDocumentTable('Probe')
        database.registerDocumentTable('QuotaType')
        database.registerDocumentTable('Quoting')
        database.registerDocumentTable('Quoting_Region')
        database.registerDocumentTable('Schedule')
        database.registerDocumentTable('Schedule_Item')
        database.registerDocumentTable('StockMotion')
        database.registerDocumentTable('SuspendedAppointment')
        database.registerDocumentTable('TakenTissueJournal')
        database.registerDocumentTable('TempInvalid')
        database.registerDocumentTable('TempInvalidDocument')
        database.registerDocumentTable('Visit')
        database.registerDocumentTable('Event_CardLocation')
        database.registerDocumentTable('rbUserProfile') # это против правил :(
        database.registerDocumentTable('Client_DocumentTracking')
        database.registerDocumentTable('Client_DocumentTrackingItem')
        database.registerDocumentTable('rbService')
        database.registerDocumentTable('ActionTypeGroup')
        database.registerDocumentTable('ActionTypeGroup_Item')
        database.registerDocumentTable('Person_ActionProperty')
        database.registerDocumentTable('rbStockMotionItemReason')


    def isAdmin(self):
        return self.userHasRight(urAdmin)


    def getHomeDir(self):
        if not self.homeDir:
            homeDir = os.path.expanduser('~')
            if homeDir == '~':
                homeDir = unicode(QDir.homePath())
            if isinstance(homeDir, str):
                homeDir = unicode(homeDir, locale.getpreferredencoding())
            self.homeDir = homeDir
        return self.homeDir


    def getDocumentsDir(self):
        return unicode(QtGui.QDesktopServices.storageLocation(QtGui.QDesktopServices.DocumentsLocation))


    def getSaveDir(self):
        self.saveDir = forceString(self.preferences.appPrefs.get('saveDir', QVariant()))
        if not self.saveDir:
            self.saveDir = self.getDocumentsDir() or self.getHomeDir()
        return self.saveDir


    def setSaveDir(self, path):
#        mode = os.stat(path)[ST_MODE]
#        if if S_ISDIR(mode):
        saveDir = os.path.dirname(unicode(path))
        if saveDir and self.saveDir != saveDir:
            self.preferences.appPrefs['saveDir'] = QVariant(saveDir)


    def getTemplateDir(self):
        result = forceString(self.preferences.appPrefs.get('templateDir', None))
        if not result:
            result = os.path.join(self.logDir, 'templates')
        return result


    def getTmpDir(self, suffix=''):
        return unicode(tempfile.mkdtemp('','samson_%s_'%suffix), locale.getpreferredencoding())


    def removeTmpDir(self, dir):
        try:
            shutil.rmtree(dir, False)
        except:
            self.logCurrentException()


#    def setTemplateDir(self, path):
#        self.preferences.appPrefs.setdefault('templateDir', QVariant(path))


    def getLogFileName(self):
        return os.path.join(self.logDir, 'error.log')


    def removeBigOrOldLogFile(self):
        try:
            if os.path.exists(self.logDir):
                logFile = self.getLogFileName()
                size = os.path.getsize(logFile)
                cdt = QDateTime.fromTime_t( int(os.path.getctime(logFile)) )
                if size > 1024**2 or cdt < QDateTime.currentDateTime().addMonths(-1):
                    os.unlink(logFile)
        except:
            pass


    def log(self, title, message, stack=None):
        try:
            if not os.path.exists(self.logDir):
                os.makedirs(self.logDir)
            logFile = self.getLogFileName()
            timeString = unicode(QDateTime.currentDateTime().toString(Qt.SystemLocaleDate))
            logString = u'%s\n%s: %s(%s)\n' % ('='*72, timeString, title, message)
            if stack:
                try:
                    logString += ''.join(traceback.format_list(stack)).decode('utf-8') + '\n'
                except:
                    logString += 'stack lost\n'
            file = codecs.open(logFile, mode='a', encoding=locale.getpreferredencoding())
            file.write(logString)
            file.close()
        except:
            pass


    def logException(self, exceptionType, exceptionValue, exceptionTraceback):
        title = repr(exceptionType)
        message = exceptionToUnicode(exceptionValue)
        self.log(title, message, traceback.extract_tb(exceptionTraceback))
        sys.__excepthook__(exceptionType, exceptionValue, exceptionTraceback)


    def logCurrentException(self):
        self.logException(*sys.exc_info())


    def msgHandler(self, type, msg):
        if type == 0: # QtMsgType.QtDebugMsg:
            typeName = 'QtDebugMsg'
        elif type == 1: # QtMsgType.QtWarningMsg:
            typeName = 'QtWarningMsg'
        elif type == 2: # QtMsgType.QtCriticalMsg:
            typeName = 'QtCriticalMsg'
        elif type == 3: # QtFatalMsg
            typeName = 'QtFatalMsg'
        else:
            typeName = 'QtUnknownMsg'

        self.log( typeName, msg, traceback.extract_stack()[:-1])
#        print typeName, msg


    def openDatabase(self):
        self.db = None
        self.db = database.connectDataBase(self.preferences.dbDriverName,
                                     self.preferences.dbServerName,
                                     self.preferences.dbServerPort,
                                     self.preferences.dbDatabaseName,
                                     self.preferences.dbUserName,
                                     self.preferences.dbPassword,
                                     compressData = self.preferences.dbCompressData)
        self.loadGlobalPreferences()
        self.emit(SIGNAL('dbConnectionChanged(bool)'), True)
        self.setCurrentClientId(None)

        #FIXME надо ли загружать календарную информацию сразу в момент открытия БД????
        self.calendarInfo.load()
        self.disableLock = self.disableLockRequested or self.db.db.record('AppLock').count() == 0
        if not self.disableLock:
            try:
                if self.db.name == 'postgres':
                    self.db.query('select getAppLock_prepare()')
                else:
                    self.db.query('CALL getAppLock_prepare()')
            except:
                self.disableLock = True

        self.webDAVInterface.setWebDAVUrl( self.getWebDAVUrl() )


    def closeNewDBConnection(self, name):
        if hasattr(self, name):
            db = getattr(self, name)
            db.close()
            self.__delattr__(name)


    def loadGlobalPreferences(self):
        if self.db:
            try:
                recordList = self.db.getRecordList('GlobalPreferences')
            except:
                recordList = []
            for record in recordList:
                code  = forceString(record.value('code'))
                value = forceString(record.value('value'))
                self._globalPreferences[code] = value


    def checkGlobalPreference(self, code, chkValue, default=None):
        value = self._globalPreferences.get(code, default)
        if value:
            return unicode(value).lower() == unicode(chkValue).lower()
        return False


    def isGlobalAppLog(self):
        if self._isGlobalAppLog is None:
            self._isGlobalAppLog = self.checkGlobalPreference(u'appLog', u'да')
        return self._isGlobalAppLog


    def getWebDAVUrl(self):
        value = self._globalPreferences.get('WebDAV', None)
        return unicode(value) if value else None


    def getGlobalPreference(self, code):
        value = self._globalPreferences.get(code, None)
        return unicode(value).lower() if value else None


    def tryOpenScanner(self):
        if self.scannerEnabled():
            self.scanner = None
            try:
                settings = self.scannerPortSettings()
                self.scanner = serial.Serial(settings.port,
                                             baudrate=settings.baudrate,
                                             bytesize=8,
                                             parity=settings.parity,
                                             stopbits=settings.stopbits,
                                             xonxoff=settings.xonxoff,
                                             rtscts=settings.rtscts,
                                             timeout=0.1)
#            except IOError, e:
            except UnicodeError, e:
                if self.scannerConnectionReport():
                    try:
                        QtGui.QMessageBox.information(self.mainWindow, u'Ошибка подключения к сканеру', anyToUnicode(e.object), QtGui.QMessageBox.Close, QtGui.QMessageBox.Close)
                    except:
                        pass
                #self.logCurrentException()
                return
            except Exception, e:
                if self.scannerConnectionReport():
                    try:
                        QtGui.QMessageBox.information(self.mainWindow, u'Ошибка подключения к сканеру', anyToUnicode(str(e)), QtGui.QMessageBox.Close, QtGui.QMessageBox.Close)
                    except:
                        pass
                #self.logCurrentException()
                return
            if self.scannerTimer is None:
                self.scannerTimer = QTimer(self)
                self.connect(self.scannerTimer, SIGNAL('timeout()'), self.checkScanner)
            self.scannerTimer.start(250)


    def checkScanner(self):
        if self.scanner is not None:
            data = None
            try:
                if self.scanner.inWaiting():
                    data = self.scanner.readall()
#            except IOError, e:
            except UnicodeError, e:
                if self.scannerConnectionReport():
                    QtGui.QMessageBox.information(self.mainWindow, u'Ошибка считывания со сканера',  anyToUnicode(e.object), QtGui.QMessageBox.Close, QtGui.QMessageBox.Close)
                    self.scannerTimer.stop()
            except Exception, e:
                if self.scannerConnectionReport():
                    QtGui.QMessageBox.information(self.mainWindow, u'Ошибка считывания со сканера',  anyToUnicode(str(e)), QtGui.QMessageBox.Close, QtGui.QMessageBox.Close)
                    self.scannerTimer.stop()
                #self.logCurrentException()
                return
            if data is not None:
#                if re.match('[ 0-9a-zA-Z:+-_]+(\n|\r|\t)*$', data):
                simpleBarCode = re.match('^\x02?([ 0-9]+)[\n\r\t\x00]*$', data)
                if simpleBarCode:
                    self.emulateKeyboardInput('\x02'+simpleBarCode.group(1)+'\n')
                else:
                    if self.scannerCorrectBackslashes() and data.count('\\') == data.count('\\\\')*2:
                        data = data.replace('\\\\',  '\\')
                    self.emit(SIGNAL('scannedReceived(QByteArray)'), QByteArray(data))


    def emulateKeyboardInput(self, data):
        specialKeys = { '\r'  : (Qt.Key_Return, Qt.NoModifier),
                        '\n'  : (Qt.Key_Enter,  Qt.NoModifier),
                        '\t'  : (Qt.Key_Tab,    Qt.NoModifier),
                        '\x02': (Qt.Key_B,      Qt.ControlModifier)
                      }
        for c in data:
            key = specialKeys.get(c)
            if key is None:
                key = ord(c.upper()), Qt.ShiftModifier if c.isupper() else Qt.NoModifier
            keyEvent = QtGui.QKeyEvent(QtGui.QKeyEvent.KeyPress, key[0], key[1],  c)
            if self.focusWidget():
                self.sendEvent(self.focusWidget(), keyEvent)
            keyEvent = QtGui.QKeyEvent(QtGui.QKeyEvent.KeyRelease, key[0],  key[1],  c)
            if self.focusWidget():
                self.sendEvent(self.focusWidget(), keyEvent)


    def closeScanner(self):
        if self.scannerTimer is not None:
            self.scannerTimer.stop()
        if self.scanner is not None:
            self.scanner.close()
            self.scanner = None


    def tryOpenSmartCardReader(self):
        global gSmartCardAvailable
        if gSmartCardAvailable:
            try:
                self.smartCardWatcher = CSmartCardWatcher()
            except:
                # если у нас будет настройка "использовать смарт-карты",
                # то нужно будет выдавать явное сообщение о проблеме
                gSmartCardAvailable = False
                #self.logCurrentException()
                return
            self.smartCardWatcher.addSubscriber(self)
            if self.smartCardTimer is None:
                self.smartCardTimer = QTimer(self)
                self.connect(self.smartCardTimer, SIGNAL('timeout()'), self.checkSmartCard)
            self.smartCardTimer.start(250)


    def checkSmartCard(self):
        self.smartCardWatcher.watch()


    def addSmartCardNotice(self, connection):
        pscc = CPolicySmartCardConnection(connection)
        try:
            clientInformation = pscc.getOwnerInformation()
            smoInformation = pscc.getCurrentSMOInformation()
            clientInformation['smo'] = { 'OGRN'   : smoInformation.get('OGRN'),
                                         'OKATO'  : smoInformation.get('OKATO'),
                                         'begDate': smoInformation.get('begDate'),
                                         'endDate': smoInformation.get('endDate'),
                                       }
            self.emit(SIGNAL('policySmartCardReceived(QVariant)'), QVariant(clientInformation))
            return True
        except:
            pass


    def closeSmartCardReader(self):
        if self.smartCardTimer is not None:
            self.smartCardTimer.stop()
        self.smartCardWatcher = None


    def closeDatabase(self):
        if self.db:
            # Подчистка блокировок при выходе из приложения
            if not self.disableLock:
                try:
                    if self.db.name == 'postgres':
                        self.db.query('select cleanup_locks()')
                    else:
                        self.db.query('CALL CleanupLocks()')
                except:
                    pass
            self.db.close()
            self.setCurrentClientId(None)
            self.db = None
            self.emit(SIGNAL('dbConnectionChanged(bool)'), False)
            self.calendarInfo.clear()
            self._globalPreferences.clear()
            self.accountingSystem = None
            self._gpSpecialityId = None


    def recordEvent(self, receiver, event):
        def getPath(obj):
            names = []
            while obj:
                parent = obj.parent()
                name = obj.objectName()
                if parent:
                    children = parent.findChildren(QObject, name)
                    if not children:
                        self.eventLogFile.write('Warning child %s type %s not found\n' %(
                            repr(unicode(name)),
                            repr(obj.metaObject().className()),
                            ))
                    elif len(children)>1:
                        try:
                            idx = children.index(obj)
                        except:
                            idx = -1
                        name = name + ':' + str(idx)
                names.append(unicode(name))
                obj = obj.parent()
            names.append('')
            return '/'.join(names[::-1])

        def recordKeyEvent(typeName):
            t0, self.eventLogTime = self.eventLogTime, time.time()
            self.eventLogFile.write('Time %f\n' % (self.eventLogTime-t0))
            self.eventLogFile.write('%s %s %X %X %s\n' % (typeName,
                                                     getPath(receiver),
                                                     event.key(),
                                                     int(event.modifiers()),
                                                     repr(unicode(event.text()))
                                      ))

        def recordMouseEvent(typeName):
            t0, self.eventLogTime = self.eventLogTime, time.time()
            self.eventLogFile.write('Time %f\n' % (self.eventLogTime-t0))
            self.eventLogFile.write('%s %s %X %X %X %d %d\n' % (typeName,
                                                     getPath(receiver),
                                                     int(event.button()),
                                                     int(event.buttons()),
                                                     int(event.modifiers()),
                                                     event.x(),
                                                     event.y()
                                      ))

        t = event.type()
        if t == QEvent.KeyPress:
            recordKeyEvent('KeyPress')
        elif t == QEvent.KeyRelease:
            recordKeyEvent('KeyRelease')
        elif t == QEvent.MouseButtonPress:
            recordMouseEvent('MouseButtonPress')
        elif t == QEvent.MouseButtonRelease:
            recordMouseEvent('MouseButtonRelease')
        elif t == QEvent.MouseButtonDblClick:
            recordMouseEvent('MouseButtonDblClick')
        elif t == QEvent.MouseMove:
            recordMouseEvent('MouseMove')


    def notify(self, receiver, event):
        try:
            if self.eventLogFile and isinstance(event, QtGui.QInputEvent):
                self.recordEvent(receiver, event)
#            report = ['notify:: receiver.type=',unicode(type(receiver)),' receiver.name=', unicode(receiver.objectName()),' event.type=',unicode(type(event)), 'event.type()=', unicode(event.type())]
#            print >>sys.stderr, ("".join(report)).encode('cp866')
            return QtGui.QApplication.notify(self, receiver, event)
        except Exception, e:
            if self.traceActive:
                self.logCurrentException()
                widget = self.activeModalWidget()
                if widget is None:
                    widget = self.mainWindow
                QtGui.QMessageBox.critical( widget,
                                            u'Произошла ошибка',
                                            exceptionToUnicode(e),
                                            QtGui.QMessageBox.Close)
            return False
        except:
            return False


    def call(self, widget, func, params = ()):
        try:
            return True, func(*params)
        except IOError, e:
            self.logCurrentException()
            msg=''
            if getattr(e, 'filename', None):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno, anyToUnicode(e.strerror))
            elif getattr(e, 'strerror', None):
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            # эти хлопоты с вложенными exception связаны со своеобразной обработкой
            # ошибок в requests
#            elif isinstance(e.message, Exception):
#                if len(e.message.args) >= 2 and isinstance(e.message.args[1], IOError):
#                    msg = u'%s: [Errno %s] %s' % ( e.message.args[0], e.message.args[1].errno, anyToUnicode(e.message.args[1].strerror))
#                else:
#                    msg = unicode(e.message)
            else:
                msg = exceptionToUnicode(e)
            QtGui.QMessageBox.critical(widget,
                u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
        except Exception, e:
            self.logCurrentException()
            widget = widget or self.activeModalWidget() or self.mainWindow
            QtGui.QMessageBox.critical( widget,
                                        u'Произошла ошибка',
                                        exceptionToUnicode(e),
                                        QtGui.QMessageBox.Close)
        return False, None


    def setWaitCursor(self):
        self.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))


    def callWithWaitCursor(self, widget, func, *params, **kwparams):
        self.setWaitCursor()
        try:
            return func(*params, **kwparams)
        finally:
            self.restoreOverrideCursor()


    def execProgram(self, cmd, args=[], env={}):
        self.setWaitCursor()
        try:
            process = QProcess()
            if env:
                outEnv = QProcess.systemEnvironment()
                for key, value in env.iteritems():
                    outEnv.append("%s=%s"%(key, value))
                process.setEnvironment(outEnv)
            if args:
                outArgs = [arg if isinstance(arg, basestring) else unicode(arg) for arg in args]
                process.start(cmd, outArgs)
            else:
                process.start(cmd)
            stepTimeOut = 200 # ms
            for i in xrange(100):
                started=process.waitForStarted(stepTimeOut)
                self.processEvents()
                if started:
                    break;
            if started:
                # Обнаружено, что некоторые программы (winword.exe, например)
                # сообщают об успешном выполнении до того, как реально начнут работать
                # Возможно это связано с тем, что второй экремпляр программы
                # просто передаёт первому имя файла и завершается.
                # Из-за этого мы успеваем удалить исходные файлы раньше, чем
                # программа их прочитает.
                # дадим 2 секунды на раскачку.
                t = QTime()
                t.start()
                while process.state() == QProcess.Running and not process.waitForFinished(stepTimeOut):
                    self.processEvents()
                while 0<=t.elapsed()<=2000:
                    QThread.msleep(stepTimeOut)
                    self.processEvents()
            return started, process.error(), process.exitCode()
        finally:
            self.restoreOverrideCursor()


    def editDocument(self,  textDocument):
        tmpDir = QtGui.qApp.getTmpDir('edit')
        try:
            fileName = os.path.join(tmpDir, 'document.html')
            txt = textDocument.toHtml(QByteArray('utf-8'))
            file = codecs.open(unicode(fileName), mode='w', encoding='utf-8')
            file.write(unicode(txt))
            file.close()
            cmdLine = u'"%s" "%s"' % (self.documentEditor(), fileName)
            started, error, exitCode = self.execProgram(cmdLine)
            if started:
                for enc in ('utf-8', 'cp1251'):
                    try:
                        file = codecs.open(fileName, mode='r', encoding=enc)
                        txt = file.read()
                        file.close()
                        textDocument.setHtml(txt)
                        return True
                    except:
                        pass
                QtGui.QMessageBox.critical(None,
                                       u'Внимание!',
                                       u'Не удалось загрузить "%s" после' % fileName,
                                       QtGui.QMessageBox.Close)
            else:
                QtGui.QMessageBox.critical(None,
                                       u'Внимание!',
                                       u'Не удалось запустить "%s"' % self.documentEditor(),
                                       QtGui.QMessageBox.Close)
        finally:
            QtGui.qApp.removeTmpDir(tmpDir)
        return False


    def startProgressBar(self, steps, format=u'%v из %m'):
        progressBar = self.mainWindow.getProgressBar()
        progressBar.setFormat(format)
        progressBar.setMinimum(0)
        progressBar.setMaximum(steps)
        progressBar.setValue(0)


    def stepProgressBar(self, step=1):
        progressBar = self.mainWindow.getProgressBar()
        progressBar.setValue(progressBar.value()+step)


    def stopProgressBar(self):
        self.mainWindow.hideProgressBar()


    def kerberosAuthEnabled(self):
        return True


    def passwordAuthEnabled(self):
        return True


    def demoModePosible(self):
        try:
            tableUser = self.db.table(tblUser)
            record = self.db.getRecordEx(tableUser,
                                         ['password = \'\' AS passwordIsEmpty',
                                          usrRetired
                                         ],
                                         [tableUser[usrLogin].eq(demoUserName),
                                          tableUser['deleted'].eq(0)
                                         ]
                                        )
            return record is None or (forceBool(record.value('passwordIsEmpty')) and not forceBool(record.value(usrRetired)))
        except:
            pass
        return False


    def setUserId(self, userId, demoMode = False):
        assert(userId or demoMode)

        self.demoMode = demoMode
        self.userId = userId
        self.userSpecialityId = None
        self.userOrgStructureId = None

        db = self.db
        if db.isProcedureExists('registerUserLoginEx'):
            db.query(u'CALL registerUserLoginEx(%s, %s, %s)' % (str(userId) if userId else 'NULL',
                                                                quote(self.hostName),
                                                                quote(getLatVersion()),
                                                              )
                    )
        elif db.isProcedureExists('registerUserLogin'):
            db.query(u'CALL registerUserLogin(%s, %s)' % (str(userId) if userId else 'NULL',
                                                          quote(self.hostName))
                                                         )

        if userId:
            record = db.getRecord('Person', ['speciality_id', 'orgStructure_id'], userId)
            if record:
                self.userSpecialityId = forceRef(record.value('speciality_id'))
                self.userOrgStructureId = forceRef(record.value('orgStructure_id'))
        if demoMode:
            self.userInfo = CDemoUserInfo(userId)
        else:
            self.userInfo = CUserInfo(userId)
        self.emit(SIGNAL('currentUserIdChanged()'))


    def clearUserId(self, emitUserIdChanged=True):
        self.demoMode = False
        self.userId = None
        self.userSpecialityId = None
        self.userOrgStructureId = None
        self.userInfo = None

        db = self.db
        if db.isProcedureExists('registerUserLogout'):
            db.query('CALL registerUserLogout()')
        if emitUserIdChanged:
            self.emit(SIGNAL('currentUserIdChanged()'))


    def isUserAvailableJobTypeId(self, jobTypeId):
        return self.userInfo.isAvailableJobTypeId(jobTypeId)


    def isUserAvailableJobType(self, jobTypeCode):
        return self.userInfo.isAvailableJobType(jobTypeCode)


    def userAvailableJobTypeIdList(self):
        return self.userInfo.availableJobTypeIdList()


    def userName(self):
        if self.userInfo:
            orgId = self.currentOrgId()
            orgInfo = getOrganisationInfo(orgId)
            if not orgInfo:
                QtGui.qApp.preferences.appPrefs['orgId'] = QVariant()
            shortName = orgInfo.get('shortName', u'ЛПУ в настройках не указано!')
            return u'%s (%s) %s' % (self.userInfo.name(), self.userInfo.login(), shortName)
        else:
            return u''


    def userHasRight(self, right):
        return self.userInfo is not None and self.userInfo.hasRight(right)


    def userHasAnyRight(self, rights):
        return self.userInfo is not None and self.userInfo.hasAnyRight(rights)


    def currentOrgId(self):
        return forceRef(QtGui.qApp.preferences.appPrefs.get('orgId', QVariant()))


#    def currentOrgStructureId(self):
#        return forceRef(QtGui.qApp.preferences.appPrefs.get('orgStructureId', QVariant()))


    def currentOrgStructureId(self):
        return forceRef(QtGui.qApp.preferences.appPrefs.get('orgStructureId', QVariant())) or self.userOrgStructureId


    def filterPaymentByOrgStructure(self):
        return forceBool(QtGui.qApp.preferences.appPrefs.get('filterPaymentByOrgStructure', QVariant()))


    def emitCurrentClientInfoJLWChanged(self, scheduleItemId):
        self.emit(SIGNAL('currentClientInfoJLWChanged(int)'), scheduleItemId)


    def emitCurrentClientInfoJLWDeleted(self, scheduleItemId):
        self.emit(SIGNAL('currentClientInfoJLWDeleted(int)'), scheduleItemId)


    def setCurrentClientId(self, id):
        if self._currentClientId != id:
            self._currentClientId = id
            self.emit(SIGNAL('currentClientIdChanged()'))


    def emitCurrentClientInfoChanged(self):
        self.emit(SIGNAL('currentClientInfoChanged()'))


    def currentClientId(self):
        return self._currentClientId


    def canFindClient(self):
        return bool(self.mainWindow.registry)


    def findClient(self, clientId):
        if self.mainWindow.registry:
            self.mainWindow.registry.findClient(clientId)


    def identServiceEnabled(self):
        if self._identService is None:
            return forceBool(self.preferences.appPrefs.get('TFCheckPolicy', False))
        else:
            return True


    def identService(self, firstName, lastName, patrName, sex, birthDate, snils='', policySerial='', policyNumber='', docSerial='', docNumber=''):
        if self._identService is None:
            url = forceString(self.preferences.appPrefs.get('TFCPUrl', ''))
            self._identService = CTFUnifiedIdentService(url)
        try:
            policy, attach, message = self.callWithWaitCursor(self, self._identService.getPolicyAndAttach, firstName, lastName, patrName, sex, birthDate, snils, policySerial, policyNumber, docSerial, docNumber)
            return policy, attach, message
        except Exception, e:
            self.logCurrentException()
            return None, None, exceptionToUnicode(e)


    def TFAccountingSystemId(self):
        return forceRef(self.preferences.appPrefs.get('TFAccountingSystemId', None))


    def scannerEnabled(self):
        return forceBool(self.preferences.appPrefs.get('ScannerEnable', False))


    def scannerPortSettings(self):
        return CSerialSettings(forceString(self.preferences.appPrefs.get('ScannerPortSettings', '')))


    def scannerCorrectBackslashes(self):
        return forceBool(self.preferences.appPrefs.get('ScannerCorrectBackslashes', False))


    def scannerConnectionReport(self):
        return forceBool(self.preferences.appPrefs.get('ScannerConnectionReport', True))


    def scannerParseReport(self):
        return forceBool(self.preferences.appPrefs.get('ScannerParseReport', True))


    def getGPSpecialityId(self):
        if self._gpSpecialityId is None:
            self._gpSpecialityId = ( forceRef(self.db.translate('rbSpeciality', 'federalCode', '2019', 'id'))
                                     or forceRef(self.db.translate('rbSpeciality', 'OKSOCode',    '040819', 'id'))
                                   )
        return self._gpSpecialityId


    def requestNewEvent(self, clientId, typeQueue=-1):
        if self.mainWindow.registry:
            self.mainWindow.registry.findClient(clientId)
            self.mainWindow.registry.requestNewEventQueue(typeQueue)


    def canFindEvent(self):
        return bool(self.mainWindow.registry)


    def findEvent(self, id):
        if self.mainWindow.registry:
            self.mainWindow.registry.findEvent(id)


    def setEventList(self, idList):
        if self.mainWindow.registry:
            self.mainWindow.registry.setEventList(idList)


    def addMruEvent(self, eventId, descr):
        self.mainWindow.addMruEvent(eventId, descr)


    def clearPreferencesCache(self):
        self._defaultKLADR = None
        self._provinceKLADR = None
        self._averageDuration = None
        self._highlightRedDate = None
        self._highlightInvalidDate = None
        self._identService = None
        self.accountingSystem = None
        self.loadGlobalPreferences()


    def defaultKLADR(self):
        # код КЛАДР по умолчанию
        if self._defaultKLADR is None:
            self._defaultKLADR = forceString(self.preferences.appPrefs.get('defaultKLADR', ''))
            if not self._defaultKLADR:
                self._defaultKLADR = '7800000000000'
        return self._defaultKLADR


    def provinceKLADR(self):
        # областной код КЛАДР по умолчанию
        if self._provinceKLADR is None:
            self._provinceKLADR = forceString(self.preferences.appPrefs.get('provinceKLADR', ''))
            if not self._provinceKLADR:
                self._provinceKLADR = getProvinceKLADRCode(self.defaultKLADR())
        return self._provinceKLADR


    def averageDuration(self):
        # средняя длительность заболевания
        if self._averageDuration is None:
            self._averageDuration = forceInt(self.preferences.appPrefs.get('averageDuration', QVariant(DefaultAverageDuration)))
        return self._averageDuration


    def tempInvalidDoctype(self):
        # возвращает код документа временной нетрудоспособности
        return forceString(self.preferences.appPrefs.get('tempInvalidDoctype', ''))


    def tempInvalidReason(self):
        # возвращает код причины временной нетрудоспособности
        return forceString(self.preferences.appPrefs.get('tempInvalidReason', ''))


    def enableFastPrint(self):
        return forceBool(self.preferences.appPrefs.get('enableFastPrint', False))


    def labelPrinter(self):
        printerName = forceString(self.preferences.appPrefs.get('labelPrinter', ''))
        if 'QPrinterInfo' in QtGui.__dict__: # work-around for current version of pyqt in ubuntu 8.04
            printerInfoList = [ pi for pi in  QtGui.QPrinterInfo.availablePrinters() if pi.printerName() == printerName ]
        else:
            printerInfoList = []
        if printerInfoList:
            return QtGui.QPrinter(printerInfoList[0], QtGui.QPrinter.HighResolution)
        else:
            return None


    def showingInInfoBlockSocStatus(self):
        # отображение в информационном блоке данных по Соц.статусу в возможных вариантах:
        # 0: код
        # 1: наименование
        # 2: код и наименование
        return forceInt(self.preferences.appPrefs.get('showingInInfoBlockSocStatus', 0))


    def doubleClickQueuePerson(self):
        # изменение двойного щелчка в листе предварительной записи врача
        # 0: изменить жалобы/примечания
        # 1: перейти в картотеку
        # 2: новое обращение
        return forceInt(self.preferences.appPrefs.get('doubleClickQueuePerson', 0))


    def doubleClickQueueClient(self):
        # изменение двойного щелчка в листе предварительной записи пациента
        # 0: Нет действия
        # 1: новое обращение
        # 2: изменить жалобы/примечания
        # 3: напечатать приглашение
        # 4: перейти в график
        return forceInt(self.preferences.appPrefs.get('doubleClickQueueClient', 0))


    def doubleClickQueueFreeOrder(self):
        # изменение двойного щелчка в списке свободных номерков
        # 0: Нет действия
        # 1: Поставить в очередь
        # 2: Выполнить бронирование
        return forceInt(self.preferences.appPrefs.get('doubleClickQueueFreeOrder', 0))


    def diagnosisTypeAfterFinalForm043(self):
        # Предпочитаемый тип диагноза в форме ввода 043 после закл:
        # 0: Сопутствующий
        # 1: Основной
        return forceInt(self.preferences.appPrefs.get('diagnosisTypeAfterFinalForm043', 0))


    def ambulanceUserCheckable(self):
        return forceBool(QtGui.qApp.preferences.appPrefs.get('ambulanceUserCheckable', False))


    def highlightRedDate(self):
        if self._highlightRedDate is None:
            self._highlightRedDate = forceBool(QtGui.qApp.preferences.appPrefs.get('highlightRedDate', False))
        return self._highlightRedDate


    def highlightInvalidDate(self):
        if self._highlightInvalidDate is None:
            self._highlightInvalidDate = forceBool(QtGui.qApp.preferences.appPrefs.get('highlightInvalidDate', False))
        return self._highlightInvalidDate


    def documentEditor(self):
        return forceString(QtGui.qApp.preferences.appPrefs.get('documentEditor', ''))


    def cashBox(self):
        # номер кассы
        return forceString(self.preferences.appPrefs.get('cashBox', ''))


    def defaultAccountingSystem(self):
        if self.accountingSystem is None:
            id = None
            name = None
            code = self.getGlobalPreference('3')
            if code is not None:
                table = self.db.table('rbAccountingSystem')
                record = self.db.getRecordEx(table, ['id', 'name'], table['code'].eq(code))
                if record:
                    id = forceRef(record.value('id'))
                    name = forceString(record.value('name'))
            self.accountingSystem = id, name
        return self.accountingSystem


    def defaultAccountingSystemId(self):
        return self.defaultAccountingSystem()[0]


    def defaultAccountingSystemName(self):
        return self.defaultAccountingSystem()[1]


    def defaultMorphologyMKBIsVisible(self):
        if self.morphologyMKBIsVisible is None:
            self.morphologyMKBIsVisible = self.checkGlobalPreference('2', u'да')
        return self.morphologyMKBIsVisible


    def defaultHospitalBedProfileByMoving(self):
        # ЭТА ГЛОБАЛЬНАЯ НАСТРОЙКА ВРЕМЕННО ЗАМОРОЖЕНА.
#        # Учитывать профиль койки по движению:
#        # нет: по койке
#        # да:  по движению
#        #if self.isHospitalBedProfileByMoving is None:
#        #self.isHospitalBedProfileByMoving = self.checkGlobalPreference('5', u'да')
        self.isHospitalBedProfileByMoving = False
        return self.isHospitalBedProfileByMoving


    def defaultNeedPreCreateEventPerson(self):
        # необходимость контроля ответственного врача в диалоге 'новое обращение'
        if self.isNeedPreCreateEventPerson is None:
            self.isNeedPreCreateEventPerson = self.checkGlobalPreference('6', u'да')
        return self.isNeedPreCreateEventPerson


    def defaultIsManualSwitchDiagnosis(self):
        if self.isManualSwitchDiagnosis is None:
            self.isManualSwitchDiagnosis = self.checkGlobalPreference('1', u'в ручную')
        return self.isManualSwitchDiagnosis


    def isTNMSVisible(self):
        return self.checkGlobalPreference('7', u'да')


#    def getLevelFromGlobalPreference(self, preferenceCode):
#        checkGlobalPreferenceList = {u'нет':0, u'да':1, u'не выполнять':2}
#        value = self._globalPreferences.get(code, None)
#        return checkGlobalPreferenceList.get(unicode(value), None)


    def isStrictPolicyCheckOnEventCreation(self):
        return self.isStrictCheckOn('4')


    def isStrictAttachCheckOnEventCreation(self):
        return self.isStrictCheckOn('8')


    def isStrictCheckPolicyOnEndAction(self):
        return self.isStrictCheckOn('16')


    def isStrictCheckAttachOnEndAction(self):
        return self.isStrictCheckOn('17')


    def isStrictCheckOn(self, code):
        checkGlobalPreferenceList = {u'нет':0, u'да':1, u'не выполнять':2}
        value = self._globalPreferences.get(code, None)
        return checkGlobalPreferenceList.get(unicode(value), None)


    def isStrictCheckControlAddress(self):
        return self.isCheckOnControlAddress('25')


    def isCheckOnControlAddress(self, code):
        checkGlobalPreferenceList = {u'не выполнять':0, u'выполнять':1}
        value = self._globalPreferences.get(code, None)
        return checkGlobalPreferenceList.get(unicode(value), None)


    def getStrictCheckSocStatus(self):
        checkGlobalPreferenceList = {u'нет':0, u'мягко':1, u'жестко':2}
        value = self._globalPreferences.get('27', None)
        value = value.replace(u';', u';')
        value = value.replace(u',', u',')
        value = value.replace(u':', u':')
        resList = value.split(u';')
        if len(resList) > 1:
            checkVal = resList[0]
            socStatusVal = resList[1]
        else:
            return 0, []
        isCheckSocStatus = checkGlobalPreferenceList.get(checkVal, None)
        socStatusRes = []
        if isCheckSocStatus:
            socStatusList = socStatusVal.split(u',')
            db = QtGui.qApp.db
            table = db.table('rbSocStatusClass')
            for socStatus in socStatusList:
                socStatusCodeList = socStatus.split(u':')
                if len(socStatusCodeList) > 1:
                    groupCode = socStatusCodeList[0]
                    code = socStatusCodeList[1]
                    cond = [table['code'].eq(code),
                            table['group_id'].isNotNull()
                            ]
                    cond.append(u'''rbSocStatusClass.group_id IN (SELECT SSC.id
                    FROM rbSocStatusClass AS SSC
                    WHERE SSC.code = %s AND SSC.id != rbSocStatusClass.id)'''%(groupCode))
                    record = db.getRecordEx(table, [table['id']], cond)
                    if record:
                        socStatusRes.append(forceRef(record.value('id')))
                else:
                    code = socStatusCodeList[0]
                    cond = [table['code'].eq(code),
                            table['group_id'].isNull()
                            ]
                    record = db.getRecordEx(table, [table['id']], cond)
                    if record:
                        socStatusRes.append(forceRef(record.value('id')))
        return isCheckSocStatus, socStatusRes


    def defaultHospitalBedFinanceByMoving(self):
        # Глобальная настройка для определения правила учета источника финансирования в стац. мониторе:
        # по событию
        # по движению
        # по движению или событию
        if self.isHospitalBedFinanceByMoving is None:
            self.isHospitalBedFinanceByMoving = self.checkGlobalPreferenceHospitalBedFinance('9')
        return self.isHospitalBedFinanceByMoving


    def checkGlobalPreferenceHospitalBedFinance(self, code):
        checkGlobalPreferenceList = {u'по событию':0, u'по движению':1, u'по движению или событию':2}
        value = self._globalPreferences.get(code, None)
        if value:
            return checkGlobalPreferenceList.get(unicode(value).lower(), None)
        return None


    def isRestrictedEventCreationByContractPresence(self):
        return self.checkGlobalPreference('10', u'да')


    def isNextEventCreationFromAction(self):
        checkGlobalPreferenceList = {u'никогда':0, u'иногда':1, u'всегда':2}
        value = self._globalPreferences.get('11', None)
        if value:
            return checkGlobalPreferenceList.get(unicode(value).lower(), None)
        return None


    def isTimelineAccessibilityDays(self):
        code = '12'
        checkGlobalPreferenceList = {u'Внешняя ИС':0, u'Вся ИС':1}
        value = self._globalPreferences.get(code, None)
        if value:
            return checkGlobalPreferenceList.get(unicode(value), None)
        return None


    def isCheckedEventForPersonOrSpeciality(self):
        return self.checkGlobalPreference('13', u'по врачу')


    def isExistsActionControlledInAddActionF9(self):
        return self.explainGlobalPreference('14', {u'не выполнять':0, u'мягкий':1, u'строгий':2})


    def isGetPersonStationary(self):
        return self.checkGlobalPreference('15', u'да')


    def isReStagingInQueue(self):
        # Повторная постановка в очередь(да, нет)
        # Когда пациент уже записан к врачу этой специальности, может возникнуть необходимость подтвердить повторную запись.
        return self.checkGlobalPreference('18', u'да')


    def isCheckClientHouses(self):
        # Выполнять контроль номера дома по КЛАДР, по умолчанию - нет
        return self.checkGlobalPreference('21', u'да')


    def isCheckEventJournalOfPerson(self):
        # Использовать Журнал назначения лечащего врача, по умолчанию - нет
        return self.checkGlobalPreference('26', u'да')

    def needSaveTissueOnSaveJT(self):
        # Необходимость сохранения забора биоматериала при нажатии кнопки ОК в редакторе работы
        return self.checkGlobalPreference('saveTissueOnSaveJT', u'да', default=u'да')

    def getStockMotionNumberGlobalCounterCode(self):
        return self.getGlobalPreference('SMCounterCode')

    def askCloseJTAfterIBMBarCodeScanning(self):
        return self.checkGlobalPreference('askCloseJTAfterIBMBarCodeScanning', u'да')

    def explainGlobalPreference(self, code, checkGlobalPreferenceList):
        value = self.getGlobalPreference(code)
        if value:
            return checkGlobalPreferenceList.get(unicode(value).lower(), None)
        return None


    def isUniqueNumberCardCall(self):
        checkGlobalPreferenceList = {u'не выполнять':0, u'мягкий':1, u'жёсткий':2}
        value = self._globalPreferences.get(u'uniqueNumberCard', None)
        if value:
            return checkGlobalPreferenceList.get(unicode(value).lower(), None)
        return None


    def controlHBPatronChanges(self):
        checkGlobalPreferenceList = {u'мягкий':0, u'жёсткий':1}
        value = self._globalPreferences.get(u'patronChanges', None)
        if value:
            return checkGlobalPreferenceList.get(unicode(value).lower(), None)
        return None


    def controlSMFinance(self):
        #контроль типа финансирования ЛСиИМН при назначении и списании в соответствии с типом финансирования события
        checkGlobalPreferenceList = {u'не выполнять':0, u'мягкий':1, u'жёсткий':2}
        value = self._globalPreferences.get(u'SMFinanceControl', None)
        if value:
            return checkGlobalPreferenceList.get(unicode(value).lower(), None)
        return None


    def isLockUpdateEventHospitalBeds(self):
        # Блокировать возможность перевода и выписки в Стационарном мониторе при открытии формы ввода, по умолчанию - нет
        return self.checkGlobalPreference('lockUpdEventHB', u'да')


    def isCheckDoubleVisitsEnteredByService(self):
        #Контроль регистрации визита по услуге
        checkGlobalPreferenceList = {u'не выполнять':0, u'мягкий':1, u'жёсткий':2}
        value = self._globalPreferences.get(u'28', None)
        if value:
            return checkGlobalPreferenceList.get(unicode(value).lower(), None)
        return None


    def isCheckDoubleVisitsEnteredBySpeciality(self):
        #Контроль регистрации визита по специальности
        checkGlobalPreferenceList = {u'не выполнять':0, u'мягкий':1, u'жёсткий':2}
        value = self._globalPreferences.get(u'29', None)
        if value:
            return checkGlobalPreferenceList.get(unicode(value).lower(), None)
        return None


    def controlMKBExForTraumaType(self):
        #Контроль заполнения столбца "Доп.МКБ" для учета травм
        checkGlobalPreferenceList = {u'мягкий':0, u'жёсткий':1}
        value = self._globalPreferences.get(u'30', None)
        if value:
            return checkGlobalPreferenceList.get(unicode(value).lower(), None)
        return None


    def isCitizenshipControl(self):
        #Контроль Гражданства пациента
        checkGlobalPreferenceList = {u'не выполнять':0, u'мягкий':1, u'жёсткий':2}
        value = self._globalPreferences.get(u'Citizenship', None)
        if value:
            return checkGlobalPreferenceList.get(unicode(value).lower(), None)
        return None


    ######
    def setCounterController(self, counterController):
        if counterController and self._counterController:
            self._counterController.resetAllCounterValueIdReservation()
        self._counterController = counterController


    def getDocumentNumber(self, clientId, counteId, date = None):
        if self._counterController:
            return self._counterController.getDocumentNumber(clientId, counteId, date)


    def delAllCounterValueIdReservation(self):
        if self._counterController:
            self._counterController.delAllCounterValueIdReservation()


    def resetAllCounterValueIdReservation(self):
        if self._counterController:
            self._counterController.resetAllCounterValueIdReservation()

    def resetCounterValueIdReservation(self, reservationId):
        if self._counterController:
            self._counterController.resetCounterValueCacheReservation(reservationId)


    def counterController(self):
        return self._counterController

    ######

    def setJTR(self, jtr):
        self.jobTicketReserveHolder.addLevel(jtr)

    def unsetJTR(self, jtr):
        self.jobTicketReserveHolder.delLevel(jtr)

#    def addJobTicketReservation(self, jobTicketId, chainLength=1):
    def addJobTicketReservation(self, jobTicketId):
        return self.jobTicketReserveHolder.addJobTicketReservation(jobTicketId)


    def getReservedJobTickets(self):
        return self.jobTicketReserveHolder.getReservedJobTickets()


#    def makeJobTicketIdQueue(self, jobTicketId):
#        if self.currentJTR:
#            self.currentJTR.makeJobTicketIdQueue(jobTicketId)


#    def delAllJobTicketReservationsWithExcluding(self, excludingJTIdList):
#        if self.currentJTR:
#            self.currentJTR.delAllJobTicketReservationsWithExcluding(excludingJTIdList)

    def setupDocFreeQueue(self, orgStructureId, specialityId, personId, begDate):
        if self.userHasRight(urAccessGraph):
            dockFreeQueue = self.mainWindow.dockFreeQueue
            if not dockFreeQueue.isVisible():
                dockFreeQueue.show()
            dockFreeQueue.raise_()
            dockFreeQueue.setFilter(orgStructureId, specialityId, personId, begDate)



# #############################################################



class CS11MainWindow(QtGui.QMainWindow, Ui_MainWindow):
    def __init__(self, bgParams=None, parent = None):
        QtGui.QMainWindow.__init__(self, parent)
        self.initDockResources()
        self.initDockFreeQueue()
        self.initDockDiagnosis()
        self.setupUi(self)
        self.setCentralWidget(self.centralWidget)
        self.setCorner(Qt.TopLeftCorner,    Qt.LeftDockWidgetArea)
        self.setCorner(Qt.BottomLeftCorner, Qt.LeftDockWidgetArea)
        self.setCorner(Qt.TopRightCorner, Qt.RightDockWidgetArea)
        self.setCorner(Qt.BottomRightCorner, Qt.RightDockWidgetArea)
        for dock in (self.dockResources, self.dockFreeQueue, self.dockDiagnosis):
            self.menuPreferences.addAction(dock.toggleViewAction())
#            dock.loadDialogPreferences()

        if bgParams:
            self.centralWidget.setBackground(bgParams)

        self.prepareStatusBar()
        self.registry = None
        self.suspendedAppointment = None
        self.updateActionsState()
        self.setUserName('')
        self.mruEventList = []
        self.mruEventListChanged = False
        self.loadPreferences()


    def initDockResources(self):
        self.dockResources = CResourcesDockWidget(self)
        self.dockResources.setObjectName('dockResources')
        self.addDockWidget(Qt.LeftDockWidgetArea, self.dockResources)


    def initDockFreeQueue(self):
        self.dockFreeQueue = CFreeQueueDockWidget(self)
        self.dockFreeQueue.setObjectName('dockFreeQueue')
        self.addDockWidget(Qt.LeftDockWidgetArea, self.dockFreeQueue)


    def initDockDiagnosis(self):
        self.dockDiagnosis = CDiagnosisDockWidget(self)
        self.dockDiagnosis.setObjectName('dockDiagnosis')
        self.addDockWidget(Qt.LeftDockWidgetArea, self.dockDiagnosis)


    def prepareStatusBar(self):
        self.progressBar = QtGui.QProgressBar()
        self.progressBar.setMaximumWidth(200)
        self.progressBar.setMaximumHeight(self.statusBar.height()/2)
        self.progressBarVisible = False


    def closeEvent(self, event):
        self.dockResources.saveDialogPreferences()
        self.dockFreeQueue.saveDialogPreferences()
        self.dockDiagnosis.saveDialogPreferences()
        self.savePreferences()
        self.centralWidget.closeAllSubWindows()
        QtGui.QMainWindow.closeEvent(self, event)


    def loadPreferences(self):
        preferences = getPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), {})
        geometry = getPref(preferences, 'geometry', None)
        if type(geometry) == QVariant and geometry.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreGeometry(geometry.toByteArray())
        state = getPref(preferences, 'state', None)
        if type(state) == QVariant and state.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreState(state.toByteArray())


    def savePreferences(self):
        preferences = {}
        setPref(preferences,'geometry',QVariant(self.saveGeometry()))
        setPref(preferences,'state', QVariant(self.saveState()))
        setPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), preferences)


    def updateActionsState(self):
#        notImplemented = False
        app = QtGui.qApp
        loggedIn = bool(app.db) and (app.demoMode or app.userId is not None)
        orgIdSet = loggedIn and bool(app.currentOrgId())
#        orgStructureIdSet = loggedIn and bool(app.currentOrgStructureId())
        isAdmin      = loggedIn and app.userHasRight(urAdmin)
        isAccountant = orgIdSet and (app.userHasRight(urAccessAccountInfo) or isAdmin)
        isTimekeeper = orgIdSet and app.userHasAnyRight([urAccessReadTimeLine, urAccessEditTimeLine, isAdmin])
        isBlankskeeper = orgIdSet and (app.userHasRight(urAccessBlanks) or isAdmin)
        isRefAdmin   = loggedIn and (app.userHasRight(urAccessRefBooks) or isAdmin)
        isEmergencyRefAdmin = loggedIn and (isRefAdmin or app.userHasRight(urAccessRefEmergency))
        isFeedRefAdmin = loggedIn and (isRefAdmin or app.userHasRight(urAccessRefFeed))
        isMedicalRefAdmin = loggedIn and (isRefAdmin or app.userHasRight(urAccessRefMedical))
        isClassificatorRefAdmin = loggedIn and (isRefAdmin or app.userHasRight(urAccessRefClassificators))
        exchangeEnabled = orgIdSet and (app.userHasRight(urAccessExchange) or isAdmin)

        fullAnalysisEnabled = orgIdSet and (app.userHasRight(urAccessAnalysis) or isAdmin)
        personalAnalysisEnabled = orgIdSet and (app.userHasRight(urAccessAnalysisPersonal)
                                                            or app.userHasRight(urAccessAnalysis) or isAdmin)
        orgStructAnalysisEnabled = orgIdSet and (app.userHasRight(urAccessAnalysisOrgStruct)
                                                            or app.userHasRight(urAccessAnalysis) or isAdmin)
        analysisTimelineEnabled = orgIdSet and (app.userHasRight(urAccessAnalysisTimeline)
                                                            or app.userHasRight(urAccessAnalysis) or isAdmin)
        analysisTimelinePreRecordEnabled = orgIdSet and (app.userHasRight(urAccessAnalysisTimelinePreRecord)
                                                            or app.userHasRight(urAccessAnalysis) or isAdmin)
        analysisAnaliticReportsEnabled = orgIdSet and (app.userHasRight(urAccessAnalysisAnaliticReports)
                                                            or app.userHasRight(urAccessAnalysis) or isAdmin)

        # Меню Сессия
        self.actLogin.setEnabled(not loggedIn)
        self.actLogout.setEnabled(loggedIn)
        self.actQuit.setEnabled(True)

        # Меню Работа
        self.actRegistry.setEnabled(orgIdSet and (app.userHasRight(urAccessRegistry) or isAdmin))
        self.actSuspenedAppointment.setEnabled(app.userHasRight(urAccessSuspenedAppointment))
        self.actTimeline.setEnabled(isTimekeeper)
        self.actFormRegistration.setEnabled(isBlankskeeper)
        self.actHospitalBeds.setEnabled(orgIdSet and (app.userHasRight(urAccessHospitalBeds) or isAdmin))
        self.actSurveillance.setEnabled(orgIdSet and (app.userHasRight(urAccessSurveillance) or isAdmin))
        self.actJobsPlanning.setEnabled(orgIdSet and (app.userHasRight(urAccessJobsPlanning) or isAdmin))
        self.actJobsOperating.setEnabled(orgIdSet and (app.userHasRight(urAccessJobsOperating) or isAdmin))
        self.actQuoting.setEnabled(app.userHasRight(urAccessQuoting) or
                                   app.userHasRight(urAccessQuotingWatch) or
                                   isAdmin)
        self.actTissueJournal.setEnabled(app.userHasRight(urAccessTissueJournal))
        self.actStockControl.setEnabled(orgIdSet and (app.userHasRight(urAccessStockControl) or isAdmin))
        # Меню Расчет
        self.mnuAccounting.setEnabled(orgIdSet and (   isAccountant
                                                    or app.userHasAnyRight((urAccessAccounting,
                                                                            urAccessAccountingBudget,
                                                                            urAccessAccountingCMI,
                                                                            urAccessAccountingVMI,
                                                                            urAccessAccountingCash,
                                                                            urAccessAccountingTargeted,
                                                                            urAccessContract,
                                                                            urAccessCashBook
                                                                           )
                                                                          )
                                                   )
                                     )
        self.actAccounting.setEnabled(orgIdSet and (   isAccountant
                                                    or app.userHasAnyRight((urAccessAccounting,
                                                                            urAccessAccountingBudget,
                                                                            urAccessAccountingCMI,
                                                                            urAccessAccountingVMI,
                                                                            urAccessAccountingCash,
                                                                            urAccessAccountingTargeted
                                                                           )
                                                                          )
                                                   )
                                     )
        self.actContract.setEnabled(orgIdSet and (isAccountant or app.userHasRight(urAccessContract)))
        self.actCashBook.setEnabled(orgIdSet and (isAccountant or app.userHasRight(urAccessCashBook)))

        # Меню Обмен
        self.mnuExchange.setEnabled(exchangeEnabled)

        # Меню Анализ
        self.mnuStatReports.setEnabled(orgStructAnalysisEnabled)
        self.mnuReportF131.setEnabled(orgStructAnalysisEnabled)
        self.actSpendingToClients.setEnabled(personalAnalysisEnabled)
        self.actGenRep.setEnabled(orgStructAnalysisEnabled)
        self.mnuReportsByVisits.setEnabled(personalAnalysisEnabled)
        self.mnuReportsBuMorbidity.setEnabled(personalAnalysisEnabled)
        self.mnuDispObservation.setEnabled(personalAnalysisEnabled)
        self.mnuTempInvalid.setEnabled(personalAnalysisEnabled)
        self.mnuDeath.setEnabled(personalAnalysisEnabled)
        self.mnuContingent.setEnabled(personalAnalysisEnabled)
        self.mnuStationary.setEnabled(personalAnalysisEnabled)
        self.mnuTimelineReports.setEnabled(analysisTimelineEnabled)
        self.mnuPreRecord.setEnabled(analysisTimelinePreRecordEnabled)
        self.mnuAnaliticReports.setEnabled(analysisAnaliticReportsEnabled)
        self.mnuDD2007.setEnabled(fullAnalysisEnabled)
        self.mnuAnalysisService.setEnabled(fullAnalysisEnabled)
        self.actWorkload.setEnabled(fullAnalysisEnabled)
        self.mnuAccountingAnalysis.setEnabled(fullAnalysisEnabled)
        self.mnuEmergencyCall.setEnabled(fullAnalysisEnabled)
        self.mnuReportImunoprophylaxis.setEnabled(fullAnalysisEnabled)
        self.mnuReportLaboratory.setEnabled(fullAnalysisEnabled)

        self.actReportDispansMO_1_2.setEnabled(False)
        self.actReportDispansMO_1_3.setEnabled(False)

        arhRegion = QtGui.qApp.defaultKLADR().startswith('29')
        self.actMUOMSOFCity.setEnabled(orgStructAnalysisEnabled and arhRegion)
        self.actMUOMSOFForeign.setEnabled(orgStructAnalysisEnabled and arhRegion)
        self.actMUOMSOFCity.setVisible(arhRegion)
        self.actMUOMSOFForeign.setVisible(arhRegion)
        self.menuArhRegion.menuAction().setVisible(arhRegion)


        # Меню отчеты иммунопрофилактики
        self.actReportImunoprophylaxisForm6.setEnabled(False)

        # Меню Справочники
        # Подменю Адреса
#        self.menu.setEnabled(notImplemented)
        self.menu.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAddress,
                                 urAccessRefAddressKLADR,
                                 urAccessRefAddressAreas
                                ]))
        self.actMilitaryUnits.setEnabled(isRefAdmin or app.userHasAnyRight([urAccessRefAddressKLADR]))
#        self.actKLADR.setEnabled(isRefAdmin or app.userHasAnyRight([urAccessRefAddress, urAccessRefAddressKLADR]))
#        self.actAreas.setEnabled(isRefAdmin or app.userHasAnyRight([urAccessRefAddress, urAccessRefAddressAreas]))
#        self.actKLADR.setEnabled(notImplemented)
#        self.actAreas.setEnabled(notImplemented)

        # Подменю Персонал
        self.menu_2.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefPerson,
                                 urAccessRefPersonOrgStructure,
                                 urAccessRefPersonRBSpeciality,
                                 urAccessRefPersonRBPost,
                                 urAccessRefPersonRBActivity,
                                 urAccessRefPersonRBAppointmentPurpose,
                                 urAccessRefPersonPersonal
                                ]))
        self.actOrgStructure.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefPerson, urAccessRefPersonOrgStructure]))
        self.actRBSpeciality.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefPerson, urAccessRefPersonRBSpeciality]))
        self.actRBPost.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefPerson, urAccessRefPersonRBPost]))
        self.actRBActivity.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefPerson, urAccessRefPersonRBActivity]))
        self.actRBAppointmentPurpose.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefPerson, urAccessRefPersonRBAppointmentPurpose]))
        self.actPersonal.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefPerson, urAccessRefPersonPersonal]))

        # Подменю Скорая помощь
        self.menu_9.setEnabled(isEmergencyRefAdmin or
            app.userHasAnyRight([urAccessRefEmergency]))

        # Подменю Питание
        self.menu_16.setEnabled(isFeedRefAdmin or
            app.userHasAnyRight([urAccessRefFeed]))

        # Подменю Медицинские
        self.menu_3.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical,
                                 urAccessRefMedMKB,
                                 urAccessRefMedMKBSubClass,
                                 urAccessRefMedDiseaseCharacter,
                                 urAccessRefMedDiseaseStage,
                                 urAccessRefMedDiseasePhases,
                                 urAccessRefMedDiagnosisType,
                                 urAccessRefMedTraumaType,
                                 urAccessRefMedHealthGroup,
                                 urAccessRefMedDispanser,
                                 urAccessRefMedResult,
                                 urAccessRefMedTempInvalidReason,
                                 urAccessRefMedTempInvalidDocument,
                                 urAccessRefMedTempInvalidBreak,
                                 urAccessRefMedTempInvalidRegime,
                                 urAccessRefMedTempInvalidResult,
                                 urAccessRefMedComplain,
                                 urAccessRefMedThesaurus,
                                 urAccessRefMedBloodType
                                ]))
        self.actMKB.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedMKB]))
        self.actMKBSubclass.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedMKBSubClass]))
        self.actMKBMorphology.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical]))
        self.actRBDiseaseCharacter.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedDiseaseCharacter]))
        self.actRBDiseaseStage.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedDiseaseStage]))
        self.actRBDiseasePhases.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedDiseasePhases]))
        self.actRBDiagnosisType.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedDiagnosisType]))
        self.actRBTraumaType.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedTraumaType]))
        self.actRBHealthGroup.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedHealthGroup]))
        self.actRBDispanser.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedDispanser]))
        self.actRBResult.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedResult]))
        self.actRBTempInvalidReason.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedTempInvalidReason]))
        self.actRBTempInvalidDocument.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedTempInvalidDocument]))
        self.actRBTempInvalidBreak.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedTempInvalidBreak]))
        self.actRBTempInvalidRegime.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedTempInvalidRegime]))
        self.actRBTempInvalidResult.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedTempInvalidResult]))
        self.actRBTempInvalidDuplicateReason.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedTempInvalidResult]))

        self.actRBComplain.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedComplain]))
        self.actRBThesaurus.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedThesaurus]))
        self.actRBBloodType.setEnabled(isMedicalRefAdmin or
            app.userHasAnyRight([urAccessRefMedical, urAccessRefMedBloodType]))
        self.actRBImageMap.setEnabled(isMedicalRefAdmin or app.userHasAnyRight([urAccessRefMedical,]))

        # Подменю Классификаторы
        self.menu_4.setEnabled(isClassificatorRefAdmin
            or app.userHasAnyRight([urAccessRefClOKPF,
                                    urAccessRefClOKFS,
                                    urAccessRefClHurtType,
                                    urAccessRefClHurtFactorType,
                                    urAccessRefClUnit,
                                   ]))
        self.actRBOKPF.setEnabled(isClassificatorRefAdmin or app.userHasRight(urAccessRefClOKPF))
        self.actRBOKFS.setEnabled(isClassificatorRefAdmin or app.userHasRight(urAccessRefClOKFS))
        self.actRBHurtType.setEnabled(isClassificatorRefAdmin or app.userHasRight(urAccessRefClHurtType))
        self.actRBHurtFactorType.setEnabled(isClassificatorRefAdmin or app.userHasRight(urAccessRefClHurtFactorType))
        self.actRBUnit.setEnabled(isClassificatorRefAdmin or app.userHasRight(urAccessRefClUnit))

        # Подменю Учет
        self.menu_6.setEnabled(isRefAdmin or
           app.userHasAnyRight([urAccessRefAccount,
                                urAccessRefAccountRBVisitType,
                                urAccessRefAccountEventType,
                                urAccessRefAccountRBEventTypePurpose,
                                urAccessRefAccountRBScene,
                                urAccessRefAccountRBAttachType,
                                urAccessRefAccountRBMedicalAidUnit,
                                urAccessRefAccountActionPropertyTemplate,
                                urAccessRefAccountActionType,
                                urAccessRefAccountActionTemplate,
                                urAccessRefAccountRBReasonOfAbsence,
                                urAccessRefAccountRBHospitalBedProfile,
                               ]))
        self.actRBVisitType.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAccount, urAccessRefAccountRBVisitType]))
        self.actRBTest.setEnabled(isRefAdmin or app.userHasAnyRight([urAccessRefAccount]))
        self.actRBTestGroup.setEnabled(isRefAdmin or app.userHasAnyRight([urAccessRefAccount]))
        self.actRBLaboratory.setEnabled(isRefAdmin or app.userHasAnyRight([urAccessRefAccount]))
        self.actRBSuiteReagent.setEnabled(isRefAdmin or app.userHasAnyRight([urAccessRefAccount]))
        self.actEventType.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAccount, urAccessRefAccountEventType]))
        self.actRBEventTypePurpose.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAccount, urAccessRefAccountRBEventTypePurpose]))
        self.actRBEventProfile.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAccount, urAccessRefAccountRBEventProfile]))
        self.actRBScene.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAccount, urAccessRefAccountRBScene]))
        self.actRBAttachType.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAccount, urAccessRefAccountRBAttachType]))
        self.actRBMedicalAidUnit.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAccount, urAccessRefAccountRBMedicalAidUnit]))
        self.actRBMedicalAidKind.setEnabled(isRefAdmin or app.userHasRight(urAccessRefAccount))
        self.actRBMedicalAidType.setEnabled(isRefAdmin or app.userHasRight(urAccessRefAccount))
        self.actRBMedicalAidProfile.setEnabled(isRefAdmin or app.userHasRight(urAccessRefAccount))
        self.actRBMesSpecification.setEnabled(isRefAdmin or app.userHasRight(urAccessRefAccount))

        self.actActionPropertyTemplate.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAccount, urAccessRefAccountActionPropertyTemplate]))
        self.actRBActionShedule.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAccount, urAccessRefAccountRBActionShedule]))
        self.actActionType.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAccount, urAccessRefAccountActionType]))
        self.actActionTemplate.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAccount, urAccessRefAccountActionTemplate]))
        self.actRBReasonOfAbsence.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAccount, urAccessRefAccountRBReasonOfAbsence]))
        self.actRBHospitalBedProfile.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefAccount, urAccessRefAccountRBHospitalBedProfile]))
        self.actRBJobType.setEnabled(isRefAdmin or app.userHasAnyRight([urAccessRefAccount]))
        self.actRBJobPurpose.setEnabled(isRefAdmin or app.userHasAnyRight([urAccessRefAccountJobPurpose]))
        self.actBlankType.setEnabled(isRefAdmin or app.userHasAnyRight([urAccessRefAccountBlankType]))
        self.actRBAgreementType.setEnabled(isRefAdmin or app.userHasAnyRight([urAccessRefAgreementType]))
        self.actQuotaType.setEnabled(isRefAdmin or app.userHasAnyRight([urAccessRefAccountQuotaType]))

        # Подменю Организации
        self.menu_11.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefOrganization, urAccessRefOrgRBNet, urAccessRefOrgBank, urAccessRefOrgOrganisation]))
        self.actRBNet.setEnabled(isRefAdmin or \
            app.userHasAnyRight([urAccessRefOrganization, urAccessRefOrgRBNet]))
        self.actBank.setEnabled(isRefAdmin or \
            app.userHasAnyRight([urAccessRefOrganization, urAccessRefOrgBank]))
        self.actOrganisation.setEnabled(isRefAdmin or \
            app.userHasAnyRight([urAccessRefOrganization, urAccessRefOrgOrganisation]))

        # Подменю Финансовые
        self.menu_5.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefFinancial, urAccessRefFinRBFinance,
                                 urAccessRefFinRBService, urAccessRefFinRBTariffCategory,
                                 urAccessRefFinRBPayRefuseType, urAccessRefFinRBCashOperation]))
        self.actRBFinance.setEnabled(isRefAdmin or \
            app.userHasAnyRight([urAccessRefFinancial, urAccessRefFinRBFinance]))
        self.actRBServiceGroup.setEnabled(isRefAdmin or \
            app.userHasAnyRight([urAccessRefFinancial, urAccessRefFinRBService]))
        self.actRBService.setEnabled(isRefAdmin or \
            app.userHasAnyRight([urAccessRefFinancial, urAccessRefFinRBService]))
        self.actRBTariffCategory.setEnabled(isRefAdmin or \
            app.userHasAnyRight([urAccessRefFinancial, urAccessRefFinRBTariffCategory]))
        self.actRBPayRefuseType.setEnabled(isRefAdmin or \
            app.userHasAnyRight([urAccessRefFinancial, urAccessRefFinRBPayRefuseType]))
        self.actRBCashOperation.setEnabled(isRefAdmin or \
            app.userHasAnyRight([urAccessRefFinancial, urAccessRefFinRBCashOperation]))

        # Подменю Социальный Статус
        self.menu_12.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefSocialStatus, urAccessRefSocialStatusType, urAccessRefSocialStatusClass]))
        self.actRBSocStatusType.setEnabled(isRefAdmin or \
            app.userHasAnyRight([urAccessRefSocialStatus, urAccessRefSocialStatusType]))
        self.actRBSocStatusClass.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefSocialStatus, urAccessRefSocialStatusClass]))

        # Подменю Персонификация
        self.menu_13.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefPersonfication,
                                 urAccessRefPersnftnPolicyKind,
                                 urAccessRefPersnftnPolicyType,
                                 urAccessRefPersnftnDocumentTypeGroup,
                                 urAccessRefPersnftnDocumentType,
                                 urAccessRefPersnftnContactType
                                ]))
        self.actRBPolicyType.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefPersonfication, urAccessRefPersnftnPolicyType]))
        self.actRBPolicyKind.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefPersonfication, urAccessRefPersnftnPolicyKind]))
        self.actRBDocumentTypeGroup.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefPersonfication, urAccessRefPersnftnDocumentTypeGroup]))
        self.actRBDocumentType.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefPersonfication, urAccessRefPersnftnDocumentType]))
        self.actRBContactType.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefPersonfication, urAccessRefPersnftnContactType]))
        self.actRBLocationCardType.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefEditLocationCard]))
        self.actRBDocumentTypeForTracking.setEnabled(isRefAdmin or
            app.userHasAnyRight([urAccessRefEditCardType]))

        # Подменю Номенклатура
        self.mnuNomenclature.setEnabled(isRefAdmin
                                        or app.userHasRight(urAccessNomenclature)
                                        or app.userHasRight(urAccessStockRecipe)
                                       )
        self.actRBNomenclatureClass.setEnabled(isRefAdmin or app.userHasRight(urAccessNomenclature))
        self.actRBNomenclatureKind.setEnabled(isRefAdmin or app.userHasRight(urAccessNomenclature))
        self.actRBNomenclatureType.setEnabled(isRefAdmin or app.userHasRight(urAccessNomenclature))
        self.actRBLfForm.setEnabled(isRefAdmin or app.userHasRight(urAccessNomenclature))
        self.actRBIngredient.setEnabled(isRefAdmin or app.userHasRight(urAccessNomenclature))
        self.actRBNomenclature.setEnabled(isRefAdmin or app.userHasRight(urAccessNomenclature))
        self.actRBStockRecipe.setEnabled(isRefAdmin or app.userHasRight(urAccessStockRecipe))
        self.actRBStockMotionItemUtilizationReason.setEnabled(isRefAdmin or app.userHasRight(urAccessNomenclature))

        # Подменю Лаборатория
        self.mnuLaboratory.setEnabled(isRefAdmin or app.userHasRight(urAccessLaboratory))

        # Подменю Оборудование
        self.mnuEquipment.setEnabled(isRefAdmin or app.userHasRight(urAccessEquipment))

        # Подменю Иммунопрофилактика
        self.mnuImunoprophylaxis.setEnabled(app.userHasRight(urAccessImunoprophylaxis))

        # Меню Сервис
        self.actCreateAttachClientsForArea.setEnabled(isAdmin or app.userHasRight(urAccessAttachClientsForArea))
        self.actTestSendMail.setEnabled(loggedIn)
        self.actTestMKB.setEnabled(loggedIn)
        self.actAverageDurationAcuteDisease.setEnabled(loggedIn)
        self.actTestRLS.setEnabled(loggedIn)
        self.actTestMES.setEnabled(loggedIn)
        self.actTestTNMS.setEnabled(loggedIn)
        self.actTestCSG.setEnabled(loggedIn)
        # Подменю Логический контроль
        self.mnuCheck.setEnabled(isAdmin or app.userHasRight(urAccessLogicalControl))
        self.actControlDiagnosis.setEnabled(isAdmin or app.userHasRight(urAccessLogicalControlDiagnosis))
        self.actControlDoubles.setEnabled(isAdmin or app.userHasRight(urAccessLogicalControlDoubles))
        self.actControlMes.setEnabled(isAdmin or app.userHasRight(urAccessLogicalControlMES))
        # Подменю оповещения
        self.mnuNotifications.setEnabled(isAdmin or app.userHasRight(urAccessNotifications))
        self.actNotificationLog.setEnabled(isAdmin or app.userHasRight(urAccessNotificationLog))
        self.actRBNotificationKind.setEnabled(isAdmin or app.userHasRight(urAccessNotificationKind))
        self.actNotificationRules.setEnabled(isAdmin or app.userHasRight(urAccessNotificationRule))
        # Неприкаянный экспорт, который непонятно даже как назвать
        self.actExportLocalLabResultsToUSISH.setEnabled(exchangeEnabled)

        # Меню Настройки
        self.actConnection.setEnabled(True)
        self.actAppPreferences.setEnabled(isAdmin or app.userHasRight(urAccessSetupDefault))

        self.actRBPrintTemplate.setEnabled(isAdmin or app.userHasRight(urAccessSetupTemplate))
        self.actRBAccountExportFormat.setEnabled(isAdmin or app.userHasRight(urAccessSetupAccountSystems))
        self.actRBAccountingSystem.setEnabled(isAdmin or app.userHasRight(urAccessSetupExport))

        self.actUserRightListDialog.setEnabled(isAdmin or app.userHasRight(urAccessSetupUserRights))
        self.actUserRightProfileListDialog.setEnabled(isAdmin or app.userHasRight(urAccessSetupProfilesUserRights))
        self.actCalendar.setEnabled(isAdmin or app.userHasRight(urAccessCalendar))
        self.actRBCounter.setEnabled(isAdmin or app.userHasRight(urAccessSetupCounter))
        self.actInformerMessages.setEnabled(loggedIn)

        for dock, right in ((self.dockResources, urAccessGraph),
                            (self.dockFreeQueue, urAccessGraph),
                            (self.dockDiagnosis, urAccessLUD)):
            action = dock.toggleViewAction()

            if loggedIn and app.userHasRight(right):
                action.setEnabled(True)
                dock.loadDialogPreferences() # ради загрузки видимости и пр.
            else:
                dock.setVisible(False)
                action.setEnabled(False)

        # Меню помощь
        self.actAbout.setEnabled(True)
        self.actAboutQt.setEnabled(True)


    def getProgressBar(self):
        if not self.progressBarVisible:
            self.progressBarVisible = True
            self.statusBar.addWidget(self.progressBar)
            self.progressBar.show()
        return self.progressBar


    def hideProgressBar(self):
        self.statusBar.removeWidget(self.progressBar)
        self.progressBarVisible = False


    def setUserName(self, userName):
        if userName:
            orgStructureName = u''
            orgStructureId = QtGui.qApp.currentOrgStructureId()
            if orgStructureId:
                db = QtGui.qApp.db
                table = db.table('OrgStructure')
                record = db.getRecordEx(table, [table['name']], [table['id'].eq(orgStructureId), table['deleted'].eq(0)])
                orgStructureName = forceStringEx(record.value('name')) if record else u''
            self.setWindowTitle(u'%s: %s%s' % (gTitle, userName, (u': %s'%(orgStructureName)) if orgStructureName else u''))
        else:
            self.setWindowTitle(gTitle)


    def findSubwindow(self, widget):
        for subwindow in self.centralWidget.subWindowList():
            if subwindow.widget() == widget:
                return subwindow
        return None


    @withWaitCursor
    def openRegistryWindow(self):
        self.registry = CRegistryWindow(self)
        self.centralWidget.addSubWindow(self.registry, Qt.Window)


    def closeRegistryWindow(self):
        if self.registry:
            subwindow = self.findSubwindow(self.registry)
            if subwindow:
                subwindow.close()
#                self.registry = None


    @withWaitCursor
    def openSuspenedAppointmentWidnow(self):
        self.suspendedAppointment = CSuspenedAppointmentWindow(self)
        self.centralWidget.addSubWindow(self.suspendedAppointment, Qt.Window)


    def closeSuspendedAppointmentWindow(self):
        if self.suspendedAppointment:
            subwindow = self.findSubwindow(self.suspendedAppointment)
            if subwindow:
                subwindow.close()
#            self.suspendedAppointment = None


    def addMruItem(self, mruList, id, descr):
        mruList = filter(lambda item: item[0] != id, mruList)
        mruList.insert(0, (id, descr))
        return mruList[:20]


    def prepareMruMenu(self, menu, clearAction, mruList, method):
        for action in menu.actions():
            if action.isSeparator():
                break
            menu.removeAction(action)
            sip.delete(action)
        actions = menu.actions()
        topAction = actions[0] if actions else None
        actions = []
        for id, descr in mruList:
            action = QtGui.QAction(descr, self)
            action.setData(toVariant(id))
            self.connect(action, SIGNAL('triggered()'), method)
            actions.append(action)
        menu.insertActions(topAction, actions)
        clearAction.setEnabled(bool(actions))


    def addMruEvent(self, eventId, descr):
        newList = self.addMruItem(self.mruEventList, eventId, descr)
        self.mruEventListChanged = newList != self.mruEventList
        self.mruEventList = newList


    @pyqtSignature('')
    def on_actLogin_triggered(self):
        try:
            QtGui.qApp.openDatabase()
            if QtGui.qApp.demoModePosible() and QtGui.qApp.demoModeRequested:
                ok, userId, demoMode, login = True, None, True, 'demo'
            else:
                ok = False
                if QtGui.qApp.kerberosAuthEnabled():
                    userId = tryKerberosAuth()
                    if userId:
                        ok, demoMode = True, False
                        login = forceString(QtGui.qApp.db.translate('Person', 'id', userId, 'login'))
                if not ok and QtGui.qApp.passwordAuthEnabled():
                    dialog = CLoginDialog(self)
                    dialog.setLoginName(QtGui.qApp.preferences.appUserName)
                    if dialog.exec_():
                        ok, userId, demoMode, login = True, dialog.userId(), dialog.demoMode(), dialog.loginName()
                    else:
                        ok, userId, demoMode, login = False, None, False, ''
            if ok:
                QtGui.qApp.setUserId(userId, demoMode)
                QtGui.qApp.preferences.appUserName = login
                self.setUserName(QtGui.qApp.userName())
                self.updateActionsState()
                try:
                    showInformer(QtGui.qApp.mainWindow, True)
                except:
                    pass
                return
        except database.CDatabaseException, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                unicode(e),
                QtGui.QMessageBox.Close)
        except Exception, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                u'Невозможно установить соедиенение с базой данных\n' + unicode(e),
                QtGui.QMessageBox.Close)
            QtGui.qApp.logCurrentException()
        QtGui.qApp.closeDatabase()


    @pyqtSignature('')
    def on_actLogout_triggered(self):
        self.closeRegistryWindow()
        self.closeSuspendedAppointmentWindow()
        if QtGui.qApp.db:
            QtGui.qApp.clearUserId(True)
            QtGui.qApp.closeDatabase()
        self.updateActionsState()
        self.setUserName('')
        self.mruEventListChanged = True
        self.mruEventList = []


    @pyqtSignature('')
    def on_mnuMruEvents_aboutToShow(self):
        if self.mruEventListChanged:
            self.prepareMruMenu(self.mnuMruEvents, self.actClearMruEvents, self.mruEventList, self.onEventEdit)
            self.mruEventListChanged = False


    def onEventEdit(self):
        from Events.CreateEvent import editEvent
        eventId = forceRef(self.sender().data())
        if eventId:
            if not forceBool(QtGui.qApp.db.translate('Event', 'id', eventId, 'deleted')):
                editEvent(self, eventId)
            else:
                QtGui.QMessageBox.critical(self,
                                       u'Внимание!',
                                       u'Эту запись нельзя редактировать так как она удалена',
                                       QtGui.QMessageBox.Close)


    @pyqtSignature('')
    def on_actClearMruEvents_triggered(self):
        self.mruEventList = []
        self.mruEventListChanged = True


    @pyqtSignature('')
    def on_actQuit_triggered(self):
        self.close()


    @pyqtSignature('')
    def on_actRegistry_triggered(self):
        if self.registry is None:
            self.openRegistryWindow()
        else:
            subwindow = self.findSubwindow(self.registry)
            if subwindow:
                subwindow.show()
        if self.registry:
            self.registry.setWindowState(Qt.WindowMaximized)
            self.closeSuspendedAppointmentWindow()


    @pyqtSignature('')
    def on_actSuspenedAppointment_triggered(self):
        if self.suspendedAppointment is None:
            self.openSuspenedAppointmentWidnow()
        else:
            subwindow = self.findSubwindow(self.suspendedAppointment)
            if subwindow:
                subwindow.show()
        if self.suspendedAppointment:
            self.suspendedAppointment.setWindowState(Qt.WindowMaximized)
            self.closeRegistryWindow()


    @pyqtSignature('')
    def on_actTimeline_triggered(self):
        CTimelineDialog(self).exec_()


    @pyqtSignature('')
    def on_actFormRegistration_triggered(self):
        QtGui.qApp.setWaitCursor()
        try:
            blanksDialog = CBlanksDialog(self)
        finally:
            QtGui.qApp.restoreOverrideCursor()
        if blanksDialog:
            blanksDialog.exec_()


    @pyqtSignature('')
    def on_actQuoting_triggered(self):
        dialog = CQuotingDialog(self)
        if not (QtGui.qApp.userHasRight(urAccessQuoting) or QtGui.qApp.userHasRight(urAdmin)):
            dialog.setEditable(False)
        dialog.exec_()


    @pyqtSignature('')
    def on_actTissueJournal_triggered(self):
        CTissueJournalDialog(self).exec_()


    @pyqtSignature('')
    def on_actHospitalBeds_triggered(self):
        QtGui.qApp.setWaitCursor()
        try:
            hospitalBedsDialog = CHospitalBedsDialog(self)
        finally:
            QtGui.qApp.restoreOverrideCursor()
        if hospitalBedsDialog:
            hospitalBedsDialog.exec_()


    @pyqtSignature('')
    def on_actSurveillance_triggered(self):
        QtGui.qApp.setWaitCursor()
        try:
            surveillanceDialog = CSurveillanceDialog(self)
        finally:
            QtGui.qApp.restoreOverrideCursor()
        if surveillanceDialog:
            surveillanceDialog.exec_()


    @pyqtSignature('')
    def on_actJobsPlanning_triggered(self):
        from Resources.JobPlanner import CJobPlanner
        CJobPlanner(self).exec_()


    @pyqtSignature('')
    def on_actJobsOperating_triggered(self):
        from Resources.JobsOperatingDialog import CJobsOperatingDialog
        CJobsOperatingDialog(self).exec_()


    @pyqtSignature('')
    def on_actStockControl_triggered(self):
        from Stock.StockDialog import CStockDialog
        CStockDialog(self).exec_()


    @pyqtSignature('')
    def on_actAccounting_triggered(self):
        CAccountingDialog(self).exec_()


    @pyqtSignature('')
    def on_actCashBook_triggered(self):
        CCashBookDialog(self).exec_()


    @pyqtSignature('')
    def on_actImportActionType_triggered(self):
        ImportActionType()


    @pyqtSignature('')
    def on_actImportEventType_triggered(self):
        ImportEventType()


    @pyqtSignature('')
    def on_actImportActionTemplate_triggered(self):
        from Exchange.ImportActionTemplate import ImportActionTemplate
        ImportActionTemplate()


    @pyqtSignature('')
    def on_actImportDD_triggered(self):
        ImportDD()


    @pyqtSignature('')
    def on_actImport131DBF_triggered(self):
        Import131(self)


    @pyqtSignature('')
    def on_actImport131XML_triggered(self):
        Import131XML(self)


    @pyqtSignature('')
    def on_actImport131Errors_triggered(self):
        Import131Errors(self)


    @pyqtSignature('')
    def on_actImportLgot_triggered(self):
        ImportLgot(self)


    @pyqtSignature('')
    def on_actImportQuotaFromVTMP_triggered(self):
        ImportQuotaFromVTMP(self)


    @pyqtSignature('')
    def on_actImportProfilesEIS_triggered(self):
        ImportProfiles(self)


    @pyqtSignature('')
    def on_actImportProfilesINFIS_triggered(self):
        ImportProfilesINFIS(self)


    @pyqtSignature('')
    def on_actImportEisOmsClients_triggered(self):
        importClientsFromEisOms(self)


    @pyqtSignature('')
    def on_actImportEisOmsLpu_triggered(self):
        ImportEISOMS_LPU(self)


    @pyqtSignature('')
    def on_actImportEisOmsSmo_triggered(self):
        ImportEISOMS_SMO(self)


    @pyqtSignature('')
    def on_actImportServiceNomen_triggered(self):
        CImportServiceNomen(self).exec_()

    def on_actImportServiceNomen2_triggered(self):
        CImportServiceNomen(self, 2).exec_()


    @pyqtSignature('')
    def on_actImportServiceMes_triggered(self):
        CImportServiceMes(self).exec_()


    @pyqtSignature('')
    def on_actImportRbResult_triggered(self):
        ImportRbResult(self)


    @pyqtSignature('')
    def on_actImportRbDiagnosticResult_triggered(self):
        ImportRbDiagnosticResult(self)


    @pyqtSignature('')
    def on_actImportRbUnit_triggered(self):
        ImportRbUnit(self)


    @pyqtSignature('')
    def on_actImportRbThesaurus_triggered(self):
        ImportRbThesaurus(self)


    @pyqtSignature('')
    def on_actImportRbService_triggered(self):
        from Exchange.ImportRbService import ImportRbService
        ImportRbService(self)


    @pyqtSignature('')
    def on_actImportRbComplain_triggered(self):
        ImportRbComplain(self)


    @pyqtSignature('')
    def on_actImportPolicySerialDBF_triggered(self):
        ImportPolicySerialDBF(self)


    @pyqtSignature('')
    def on_actImportOrgsINFIS_triggered(self):
        ImportOrgsINFIS(self)


    @pyqtSignature('')
    def on_actImpotrFromSail_triggered(self):
        ImportFromSail()


    @pyqtSignature('')
    def on_actImportFromSailXML_triggered(self):
        ImportFromSailXML()


    @pyqtSignature('')
    def on_actImportPrimaryDocFromXml_triggered(self):
        ImportPrimaryDocFromXml(self)


    @pyqtSignature('')
    def on_actExport131_triggered(self):
        Export131(self).exec_()


    @pyqtSignature('')
    def on_actExportActionType_triggered(self):
        ExportActionType()


    @pyqtSignature('')
    def on_actExportEventType_triggered(self):
        ExportEventType()


    @pyqtSignature('')
    def on_actExportActionTemplate_triggered(self):
        from Exchange.ExportActionTemplate import ExportActionTemplate
        ExportActionTemplate()


    @pyqtSignature('')
    def on_actExportPrimaryDocInXml_triggered(self):
        ExportPrimaryDocInXml(self)


    @pyqtSignature('')
    def on_actExportXmlEmc_triggered(self):
        ExportXmlEmc(self)


    @pyqtSignature('')
    def on_actExportActionResult_triggered(self):
        ExportActionResult(self)

    @pyqtSignature('')
    def on_actExportFeedDataCsv_triggered(self):
        exportFeedDataCsv(self)

    @pyqtSignature('')
    def on_actExportRbResult_triggered(self):
        ExportRbResult(self)


    @pyqtSignature('')
    def on_actExportRbDiagnosticResult_triggered(self):
        ExportRbDiagnosticResult(self)


    @pyqtSignature('')
    def on_actExportRbUnit_triggered(self):
        ExportRbUnit(self)


    @pyqtSignature('')
    def on_actExportRbThesaurus_triggered(self):
        ExportRbThesaurus(self)


    @pyqtSignature('')
    def on_actExportRbComplain_triggered(self):
        ExportRbComplain(self)


    @pyqtSignature('')
    def on_actExportRbService_triggered(self):
        from Exchange.ExportRbService import ExportRbService
        ExportRbService(self)


    @pyqtSignature('')
    def on_actExportHL7v2_5_triggered(self):
        ExportHL7v2_5(self)


    @pyqtSignature('')
    def on_actStatReportF12_D_1_07_triggered(self):
        CStatReportF12_D_1_07(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12_D_2_07_triggered(self):
        CStatReportF12_D_2_07(self, mode='07').exec_()


    @pyqtSignature('')
    def on_actStatReportF12_D_1_08_triggered(self):
        CStatReportF12_D_1_08(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12_D_2_08_triggered(self):
        CStatReportF12_D_2_07(self, mode='08').exec_()


    @pyqtSignature('')
    def on_actStatReportF12_D_3_M_triggered(self):
        CStatReportF12_D_3_M(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12_D_2_09_triggered(self):
        CStatReportF12_D_2_07(self, mode='09').exec_()


    @pyqtSignature('')
    def on_actStatReportF12_D_1_10_triggered(self):
        CStatReportF12_D_1_10(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12_D_2_10_triggered(self):
        CStatReportF12_D_2_10(self).exec_()


    @pyqtSignature('')
    def on_actStatReport1NP2000_triggered(self):
        CStatReport1NP2000(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF5_D_For_Teenager_triggered(self):
        CStatReportF5_D_For_Teenager(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF4_D_For_Teenager_triggered(self):
        CStatReportF4_D_For_Teenager(self).exec_()


    @pyqtSignature('')
    def on_actStatReport1NP3000_triggered(self):
        CStatReport1NP3000(self).exec_()


    @pyqtSignature('')
    def on_actStatReport1NP4000_triggered(self):
        CStatReport1NP4000(self).exec_()


    @pyqtSignature('')
    def on_actStatReport1NP5000_triggered(self):
        CStatReport1NP5000(self).exec_()


    @pyqtSignature('')
    def on_actStatReport1NP7000_triggered(self):
        CStatReport1NP7000(self).exec_()


    @pyqtSignature('')
    def on_actStatReport1DD1000_triggered(self):
        CStatReport1DD1000(self).exec_()


    @pyqtSignature('')
    def on_actStatReport1DD2000_triggered(self):
        CStatReport1DD2000(self).exec_()


    @pyqtSignature('')
    def on_actStatReport1DD3000_triggered(self):
        CStatReport1DD3000(self).exec_()


    @pyqtSignature('')
    def on_actMUOMSOFTable1_triggered(self):
        from Reports.MUOMSOFTable1 import CMUOMSOFTable1
        CMUOMSOFTable1(self).exec_()


    @pyqtSignature('')
    def on_actMUOMSOFTable3_triggered(self):
        from Reports.MUOMSOFTable3 import CMUOMSOFTable3
        CMUOMSOFTable3(self).exec_()


    @pyqtSignature('')
    def on_actMUOMSOFCity_triggered(self):
        CMUOMSOFCity(self).exec_()


    @pyqtSignature('')
    def on_actMUOMSOFForeign_triggered(self):
        CMUOMSOFForeign(self).exec_()


    @pyqtSignature('')
    def on_actReportF14App3ksg_triggered(self):
        from Reports.ReportF14App3ksg import CReportF14App3ksg
        CReportF14App3ksg(self).exec_()


    @pyqtSignature('')
    def on_actStatReport1DDAll_triggered(self):
        CStatReport1DDAll(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF131ByDoctors_triggered(self):
        CStatReportF131ByDoctors(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF131ByEmployer_triggered(self):
        CStatReportF131ByEmployer(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF131ByRaion_triggered(self):
        CStatReportF131ByRaion(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF131Raion_triggered(self):
        CStatReportF131Raion(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF131ByDD_triggered(self):
        CStatReportF131ByDD(self).exec_()


    @pyqtSignature('')
    def on_actStatReportEEDMonth_triggered(self):
        CStatReportEEDMonth(self).exec_()


    @pyqtSignature('')
    def on_actStatReportEEDYear_triggered(self):
        CStatReportEEDYear(self).exec_()


    @pyqtSignature('')
    def on_actStatReportEEDPeriod_triggered(self):
        CStatReportEEDPeriod(self).exec_()


    @pyqtSignature('')
    def on_actStatReportEEDMonthSummary_triggered(self):
        CStatReportEEDMonthSummary(self).exec_()


    @pyqtSignature('')
    def on_actStatReportEEDYearSummary_triggered(self):
        CStatReportEEDYearSummary(self).exec_()


    @pyqtSignature('')
    def on_actStatReportEEDPeriodSummary_triggered(self):
        CStatReportEEDPeriodSummary(self).exec_()


    @pyqtSignature('')
    def on_actStomatReport039_triggered(self):
        CStomatReport(self).exec_()


    @pyqtSignature('')
    def on_actStomatReportDay_triggered(self):
        CStomatDayReport(self).exec_()


    @pyqtSignature('')
    def on_actFinanceStomatReportDay_2015_triggered(self):
        CStomatDayReport_2015(self).exec_()


    @pyqtSignature('')
    def on_actFinanceStomatReportCompositeList_triggered(self):
        CStomatReportCompositeList(self).exec_()


    @pyqtSignature('')
    def on_actFinanceStomatReportToSpecialityList_triggered(self):
        CStomatReportToSpecialityList(self).exec_()


    @pyqtSignature('')
    def on_actTimelineForPerson_triggered(self):
        CTimelineForPersonEx(self).exec_()


    @pyqtSignature('')
    def on_actReportVisitBySchedule_triggered(self):
        CReportVisitBySchedule(self).exec_()


    @pyqtSignature('')
    def on_actReportVisitByNextEventDate_triggered(self):
        CReportVisitByNextEventDate(self).exec_()


    @pyqtSignature('')
    def on_actReportOutgoingDirections_triggered(self):
        CExternalOutgoingDirectionsReport(self).exec_()


    @pyqtSignature('')
    def on_actOutgoingDirectionsReport_triggered(self):
        COutgoingDirectionsReport(self).exec_()


    @pyqtSignature('')
    def on_actReportInternalDirections_triggered(self):
        CReportInternalDirections(self).exec_()


    @pyqtSignature('')
    def on_actReportIncomingDirections_triggered(self):
        CExternalIncomingDirectionsReport(self).exec_()


    @pyqtSignature('')
    def on_actTimelineForOffices_triggered(self):
        CTimelineForOfficesEx(self).exec_()


    @pyqtSignature('')
    def on_actDailyReportPreRecord_triggered(self):
        CDailyReportPreRecord(self).exec_()


    @pyqtSignature('')
    def on_actPreRecordDoctors_triggered(self):
        CPreRecordDoctorsEx(self).exec_()


    @pyqtSignature('')
    def on_actPreRecordPlanExecutionByDoctors_triggered(self):
        CPreRecordPlanExecutionByDoctors(self).exec_()


    @pyqtSignature('')
    def on_actPreRecordUsers_triggered(self):
        CPreRecordUsersEx(self).exec_()


    @pyqtSignature('')
    def on_actPreRecordSpeciality_triggered(self):
        CPreRecordSpecialityEx(self).exec_()


    @pyqtSignature('')
    def on_actJournalBeforeRecord_triggered(self):
        CJournalBeforeRecordEx(self).exec_()


    @pyqtSignature('')
    def on_actJournalBeforeRecordAV_triggered(self):
        CJournalBeforeRecordAV(self).exec_()


    @pyqtSignature('')
    def on_actDailyJournalBeforeRecord_triggered(self):
        CDailyJournalBeforeRecord(self).exec_()

    @pyqtSignature('')
    def on_actActionPropertiesTestsReport_triggered(self):
        CActionPropertiesTestsReport(self).exec_()

    @pyqtSignature('')
    def on_actNomenclatureReportForm2M3_triggered(self):
        CMonthlyNomenclatureReport(self).exec_()

    @pyqtSignature('')
    def on_actPlannedClientInvoiceNomenclaturesReport_triggered(self):
        CPlannedClientInvoiceNomenclaturesReport(self).exec_()

    @pyqtSignature('')
    def on_actClientNomenclatureInvoiceReport_triggered(self):
        CClientNomenclatureInvoiceReport(self).exec_()

    @pyqtSignature('')
    def on_actReportNomenclaureMotions_triggered(self):
        CReportNomenclaureMotions(self).exec_()

    @pyqtSignature('')
    def on_actClientNomenclatureActionReport_triggered(self):
        CClientNomenclatureActionReport(self).exec_()


    @pyqtSignature('')
    def on_actNomenclatureBook_triggered(self):
        CNomenclatureBook(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_1000_2014_triggered(self):
        CReportForm131_o_1000_2014(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_2000_2014_triggered(self):
        CReportForm131_o_2000_2014(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_3000_2014_triggered(self):
        CReportForm131_o_3000_2014(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_4000_2014_triggered(self):
        CReportForm131_o_4000_2014(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_5000_2014_triggered(self):
        CReportForm131_o_5000_2014(self).exec_()


    @pyqtSignature('')
    def on_actReportForm1_1000_triggered(self):
        CReportF1_1000(self).exec_()


    @pyqtSignature('')
    def on_actReportForm1_2000_triggered(self):
        CReportF1_2000(self).exec_()


    @pyqtSignature('')
    def on_actReportForm1RB_2000_triggered(self):
        CReportF1RB_2000(self).exec_()


    @pyqtSignature('')
    def on_actReportF1RB_AmbulatoryCare_triggered(self):
        CReportF1RB_AmbulatoryCare(self).exec_()


    @pyqtSignature('')
    def on_actReportF1RB_DentalTreatment_triggered(self):
        CReportF1RB_DentalTreatment(self).exec_()


    @pyqtSignature('')
    def on_actReportF1RB_AmbulanceCall_triggered(self):
        CReportF1RB_AmbulanceCall(self).exec_()


    @pyqtSignature('')
    def on_actReportUniversalEventList_triggered(self):
        CReportUniversalEventList(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_6000_2014_triggered(self):
        CReportForm131_o_6000_2014(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_7000_2014_triggered(self):
        CReportForm131_o_7000_2014(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_1000_2015_triggered(self):
        CReportForm131_o_1000_2015(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_2000_2015_triggered(self):
        CReportForm131_o_2000_2015(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_3000_2015_triggered(self):
        CReportForm131_o_3000_2015(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_4000_2015_triggered(self):
        CReportForm131_o_4000_2015(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_5000_2015_triggered(self):
        CReportForm131_o_5000_2015(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_5001_2015_triggered(self):
        CReportForm131_o_5001_2015(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_6000_2015_triggered(self):
        CReportForm131_o_6000_2015(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_7000_2015_triggered(self):
        CReportForm131_o_7000_2015(self).exec_()


# ############################################################################
    @pyqtSignature('')
    def on_actReportForm131_o_1000_2016_triggered(self):
        CReportForm131_o_1000_2016(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_2000_2016_triggered(self):
        CReportForm131_o_2000_2016(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_3000_2016_triggered(self):
        CReportForm131_o_3000_2016(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_4000_2016_triggered(self):
        CReportForm131_o_4000_2016(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_5000_2016_triggered(self):
        CReportForm131_o_5000_2016(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_5001_2016_triggered(self):
        CReportForm131_o_5001_2016(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_6000_2016_triggered(self):
        CReportForm131_o_6000_2016(self).exec_()


    @pyqtSignature('')
    def on_actReportForm131_o_7000_2016_triggered(self):
        CReportForm131_o_7000_2016(self).exec_()
# ############################################################################


    @pyqtSignature('')
    def on_actReportImunoprophylaxisForm5_triggered(self):
        from Reports.ReportImunoprophylaxisForm5 import CReportImunoprophylaxisForm5
        CReportImunoprophylaxisForm5(self).exec_()


    @pyqtSignature('')
    def on_actReportLaboratoryProbeExportImport_triggered(self):
        from Reports.ReportLaboratoryProbeExportImport import CReportLaboratoryProbeExportImport
        CReportLaboratoryProbeExportImport(self).exec_()


    @pyqtSignature('')
    def on_actReportUseContainers_triggered(self):
        CReportUseContainers(self).exec_()


    @pyqtSignature('')
    def on_actReportDispansMO_1_1_triggered(self):
        CReportDispansMO_1_1(self).exec_()


    @pyqtSignature('')
    def on_actDailyJournalBeforeRecord2_triggered(self):
        CDailyJournalBeforeRecord2(self).exec_()


    @pyqtSignature('')
    def on_actUnfinishedEventsByDoctor_triggered(self):
        CUnfinishedEventsByDoctor(self).exec_()


    @pyqtSignature('')
    def on_actPreRecordClientsCard_triggered(self):
        CPreRecordClientsCard(self).exec_()


    @pyqtSignature('')
    def on_actClientTreatmentsStructureReport_triggered(self):
        CClientTreatmentsStructureReport(self).exec_()


    @pyqtSignature('')
    def on_actNumberResidentsAddress_triggered(self):
        CReportNumberResidentsAddress(self).exec_()


    @pyqtSignature('')
    def on_actPatientsCompositionReport_triggered(self):
        CStationaryPatientsComposition(self).exec_()


    @pyqtSignature('')
    def on_actPatientsCompositionByRegionReport_triggered(self):
        CStationaryPatientsCompositionByRegion(self).exec_()


    @pyqtSignature('')
    def on_actUnfinishedEventsByMes_triggered(self):
        CUnfinishedEventsByMes(self).exec_()


    @pyqtSignature('')
    def on_actExecutionMes_triggered(self):
        showCheckMesDescription(self)


    @pyqtSignature('')
    def on_actAnalyticsReportHospitalizedClients_triggered(self):
        CAnalyticsReportHospitalizedClients(self).exec_()


    @pyqtSignature('')
    def on_actAnaliticReportsPerson_triggered(self):
        CAnaliticReportsStationary(self).exec_()


    @pyqtSignature('')
    def on_actAnaliticReportsDeathStationary_triggered(self):
        CAnaliticReportsDeathStationary(self).exec_()


    @pyqtSignature('')
    def on_actAnaliticReportsAdditionalSurgery_triggered(self):
        CAnaliticReportsAdditionalSurgery(self).exec_()


    @pyqtSignature('')
    def on_actAnaliticReportsGeneralInfoSurgery_triggered(self):
        CAnaliticReportsGeneralInfoSurgery(self).exec_()


    @pyqtSignature('')
    def on_actAnaliticReportsSurgeryStationary_triggered(self):
        CAnaliticReportsSurgeryStationary(self).exec_()


    @pyqtSignature('')
    def on_actAnaliticReportsSurgery_triggered(self):
        CAnaliticReportsSurgery(self).exec_()


    @pyqtSignature('')
    def on_actSummaryOnServiceType_triggered(self):
        CSummaryOnServiceType(self).exec_()


    @pyqtSignature('')
    def on_actReportOnServiceType_triggered(self):
        CReportOnServiceType(self).exec_()


    @pyqtSignature('')
    def on_actAnalyticsReportIncomeAndLeavedClients_triggered(self):
        CAnalyticsReportIncomeAndLeavedClients(self).exec_()


    @pyqtSignature('')
    def on_actAnaliticReportsChildrenLeaved_triggered(self):
        CAnaliticReportsChildrenLeaved(self).exec_()


    @pyqtSignature('')
    def on_actStationaryTallySheetMoving_triggered(self):
        CStationaryTallySheetMoving(self).exec_()


    @pyqtSignature('')
    def on_actAnalyticsReporStationaryPlanning_triggered(self):
        CStationaryPlanMoving(self).exec_()


    @pyqtSignature('')
    def on_actReportF30_OLD_triggered(self):
        CReportF30(self).exec_()


    @pyqtSignature('')
    def on_actReportF30_SMP_triggered(self):
        CReportF30_SMP(self).exec_()


    @pyqtSignature('')
    def on_actReportF30_2100_triggered(self):
        CReportF30_2100(self).exec_()


    @pyqtSignature('')
    def on_actReportF30_2110_triggered(self):
        CReportF30_2110(self).exec_()


    @pyqtSignature('')
    def on_actReportStomatF30_2700_triggered(self):
        CReportStomatF30_2700(self).exec_()


    @pyqtSignature('')
    def on_actReportStomatF30_2700_2015_triggered(self):
        CReportStomatF30_2700_2015(self).exec_()


    @pyqtSignature('')
    def on_actReportF30M_triggered(self):
        CReportF30(self, True).exec_()


    @pyqtSignature('')
    def on_actReportF30M_2100_triggered(self):
        CReportF30_2100(self, True).exec_()


    @pyqtSignature('')
    def on_actReportF30M_2110_triggered(self):
        CReportF30_2110(self, True).exec_()


    @pyqtSignature('')
    def on_actReportF39_triggered(self):
        CReportF39(self).exec_()


    @pyqtSignature('')
    def on_actReportF39_3y_triggered(self):
        CReportStomatF39_3(self).exec_()


    @pyqtSignature('')
    def on_actReportF39M_triggered(self):
        CReportF39(self, True).exec_()


    @pyqtSignature('')
    def on_actReportPGG_Ambulance_triggered(self):
        CReportPGG(self, type=1).exec_()


    @pyqtSignature('')
    def on_actReportPGG_Stomotology_triggered(self):
        CReportPGG(self, type=2).exec_()


    @pyqtSignature('')
    def on_actReportPGG_DHospital_triggered(self):
        CReportPGG(self, type=3).exec_()


    @pyqtSignature('')
    def on_actReportPGG_Hospital_triggered(self):
        CReportPGG(self, type=4).exec_()


    @pyqtSignature('')
    def on_actReportPGG_Emergency_triggered(self):
        CReportPGG(self, type=5).exec_()


    @pyqtSignature('')
    def on_actPersonVisits_triggered(self):
        CPersonVisits(self).exec_()


    @pyqtSignature('')
    def on_actReportPersonSickList_triggered(self):
        CReportPersonSickList(self).exec_()


    @pyqtSignature('')
    def on_actReportPolyclinicsOthersPersonList_triggered(self):
        CReportPolyclinicsOthersPersonList(self).exec_()


    @pyqtSignature('')
    def on_actReportBreachSA_triggered(self):
        CReportBreachSA(self).exec_()


    @pyqtSignature('')
    def on_actReportWorkloadOfMedicalStaff_triggered(self):
        CReportWorkloadOfMedicalStaff(self).exec_()


    @pyqtSignature('')
    def on_actStatReportDiseaseInPermille_triggered(self):
        CStatReportDiseaseInPermille(self).exec_()


    @pyqtSignature('')
    def on_actReportPersonSickListStationary_triggered(self):
        CReportPersonSickListStationary(self).exec_()


    @pyqtSignature('')
    def on_actReportSurgeryPlanDay_triggered(self):
        CReportSurgeryPlanDay(self).exec_()


    @pyqtSignature('')
    def on_actReportAcuteInfections_triggered(self):
        CReportAcuteInfections(self).exec_()


    @pyqtSignature('')
    def on_actReportDailyAcuteInfections_triggered(self):
        CReportDailyAcuteInfections(self).exec_()


    @pyqtSignature('')
    def on_actReportMonthActions_triggered(self):
        CReportMonthActions(self).exec_()


    @pyqtSignature('')
    def on_actReportToYearActions_triggered(self):
        CReportYearActions(self).exec_()


    @pyqtSignature('')
    def on_actReportDirectionActions_triggered(self):
        CReportDirectionActions(self).exec_()


    @pyqtSignature('')
    def on_actReportBIRADS_triggered(self):
        CReportBIRADS(self).exec_()


    @pyqtSignature('')
    def on_actReportRoadMap_triggered(self):
        CReportRoadMap(self).exec_()


    @pyqtSignature('')
    def on_actReportActionsByOrgStructure_triggered(self):
        CReportActionsByOrgStructure(self).exec_()


    @pyqtSignature('')
    def on_actReportActionsByServiceType_triggered(self):
        CReportActionsByServiceType(self).exec_()


    @pyqtSignature('')
    def on_actReportClientSummary_triggered(self):
        CReportClientSummary(self).exec_()


    @pyqtSignature('')
    def on_actReportDoctorSummary_triggered(self):
        CReportDoctorSummary(self).exec_()


    @pyqtSignature('')
    def on_actReportDoctor_triggered(self):
        CReportDoctor(self).exec_()


    def on_actReportDoctorPreCalc_triggered(self):
        CReportDoctorPreCalc(self).exec_()


    def on_actReportMonitoredContingent_triggered(self):
        CReportServicesMonitoredContingent(self).exec_()


    @pyqtSignature('')
    def on_actReportOrgStructureSummary_triggered(self):
        CReportOrgStructureSummary(self).exec_()


    @pyqtSignature('')
    def on_actReportApproxDoneActions_triggered(self):
        CReportApproxDoneActions(self).exec_()


    @pyqtSignature('')
    def on_actReportAccountDoneActions_triggered(self):
        CReportAccountDoneActions(self).exec_()


    @pyqtSignature('')
    def on_actExternalIncomePaidServices_triggered(self):
        CExternalIncomePaidServices(self).exec_()


    @pyqtSignature('')
    def on_actReportSummaryOnAccounts_triggered(self):
        CReportSummaryOnAccounts(self).exec_()


    @pyqtSignature('')
    def on_actReportSummaryOnServices_triggered(self):
        CReportSummaryOnServices(self).exec_()


    @pyqtSignature('')
    def on_actReportPayers_triggered(self):
        CReportPayers(self).exec_()


    @pyqtSignature('')
    def on_actReportPayersWithFinance_triggered(self):
        CReportPayersWithFinance(self).exec_()


    @pyqtSignature('')
    def on_actReportClientActions_triggered(self):
        CReportClientActions(self).exec_()


    @pyqtSignature('')
    def on_actReportF62_4000_triggered(self):
        CReportF62_4000(self).exec_()


#    @pyqtSignature('')
#    def on_actReportF62_7000_triggered(self):
#        CReportF62_7000(self).exec_()


    @pyqtSignature('')
    def on_actReportActions_triggered(self):
        CReportActions(self).exec_()


    @pyqtSignature('')
    def on_actUETService_triggered(self):
        CReportActionsServiceCutaway(self).exec_()


    @pyqtSignature('')
    def on_actReportOnPerson_triggered(self):
        CReportOnPerson(self).exec_()


    @pyqtSignature('')
    def on_actVisitsServiceUET_triggered(self):
        CReportVisitsServiceCutaway(self).exec_()


    @pyqtSignature('')
    def on_actFinanceSummaryByDoctorsOld_triggered(self):
        CFinanceSummaryByDoctorsExOld(self).exec_()


    @pyqtSignature('')
    def on_actFinanceSummaryByDoctors_triggered(self):
        CFinanceSummaryByDoctorsEx(self).exec_()


    @pyqtSignature('')
    def on_actVisitByPurposeEvents_triggered(self):
        CReportVisitByPurposeEvents(self).exec_()


    @pyqtSignature('')
    def on_actReconciliationMutualSettlements_triggered(self):
        CActReconciliationMutualSettlements(self).exec_()


    @pyqtSignature('')
    def on_actFinanceSummaryByServices_triggered(self):
        CFinanceSummaryByServicesEx(self).exec_()


    @pyqtSignature('')
    def on_actFinanceSumByServicesExpenses_triggered(self):
        CFinanceSumByServicesExpensesEx(self).exec_()


    @pyqtSignature('')
    def on_actFinanceSummaryByRejections_triggered(self):
        CFinanceSummaryByRejectionsEx(self).exec_()


    @pyqtSignature('')
    def on_actReportDailyCash_triggered(self):
        CReportDailyCash(self).exec_()


    @pyqtSignature('')
    def on_actReportInsuredMedicalCare_triggered(self):
        CReportInsuredMedicalCare(self).exec_()


    @pyqtSignature('')
    def on_actReportClients_triggered(self):
        CReportClients(self).exec_()


    @pyqtSignature('')
    def on_actReportTreatments_triggered(self):
        CReportTreatments(self).exec_()


    @pyqtSignature('')
    def on_actEventResultSurvey_triggered(self):
        CEventResultSurvey(self).exec_()


    @pyqtSignature('')
    def on_actEventResultList_triggered(self):
        CReportEventResultList(self).exec_()


    @pyqtSignature('')
    def on_actReportSocStatus_triggered(self):
        CSocStatus(self).exec_()

    @pyqtSignature('')
    def on_actSickRateSurvey_triggered(self):
        CSickRateSurvey(self).exec_()


    @pyqtSignature('')
    def on_actSickRateAbort_triggered(self):
        CSickRateAbort(self).exec_()


    @pyqtSignature('')
    def on_actSickRateAbort_1000_triggered(self):
        CSickRateAbort_1000(self).exec_()


    @pyqtSignature('')
    def on_actSickRateAbort_2000_triggered(self):
        CSickRateAbort_2000(self).exec_()


    @pyqtSignature('')
    def on_actSickRateAbort_3000_triggered(self):
        CSickRateAbort_3000(self).exec_()


    @pyqtSignature('')
    def on_actFactorRateSurvey_triggered(self):
        CFactorRateSurvey(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Children_triggered(self):
        CStatReportF12Children(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Children_2015_triggered(self):
        CStatReportF12Children_2015(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Children0_14_2016_triggered(self):
        CStatReportF12Children0_14_2016(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Children0_1_2016_triggered(self):
        CStatReportF12Children0_1_2016(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Teenagers_triggered(self):
        CStatReportF12Teenagers(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Teenagers_2015_triggered(self):
        CStatReportF12Teenagers_2015(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Teenagers_2016_triggered(self):
        CStatReportF12Teenagers_2016(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Adults_triggered(self):
        CStatReportF12Adults(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Adults_2015_triggered(self):
        CStatReportF12Adults_2015(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Adults_2016_triggered(self):
        CStatReportF12Adults_2016(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Seniors_triggered(self):
        CStatReportF12Seniors(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Seniors_2015_triggered(self):
        CStatReportF12Seniors_2015(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Seniors_2016_triggered(self):
        CStatReportF12Seniors_2016(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Able_triggered(self):
        CStatReportF12Able(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Able_2015_triggered(self):
        CStatReportF12Able_2015(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Able_2016_triggered(self):
        CStatReportF12Able_2016(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Inset2008_triggered(self):
        CStatReportF12Inset2008(self).exec_()


    @pyqtSignature('')
    def on_actStatReport_F57_triggered(self):
        CStatReportF57(self).exec_()


    @pyqtSignature('')
    def on_actStatReport_F57_1000_triggered(self):
        CStatReportF57_1000(self).exec_()


    @pyqtSignature('')
    def on_actStatReport_F57_2000_triggered(self):
        CStatReportF57_2000(self).exec_()


    @pyqtSignature('')
    def on_actStatReport_F57_3000_triggered(self):
        CStatReportF57_3000(self).exec_()


    @pyqtSignature('')
    def on_actStatReport_F57_3500_triggered(self):
        CStatReportF57_3500(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF63_triggered(self):
        CStatReportF63(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF71_triggered(self):
        CStatReportF71(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF9_2000_triggered(self):
        CStatReportF9_2000(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF9_2001_triggered(self):
        CStatReportF9_2001(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF9_2003_triggered(self):
        CStatReportF9_2003(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF9_2005_triggered(self):
        CStatReportF9_2005(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF9_2006_triggered(self):
        CStatReportF9_2006(self).exec_()


    @pyqtSignature('')
    def on_actDispObservationList_triggered(self):
        CDispObservationList(self).exec_()


    @pyqtSignature('')
    def on_actDispObservationSurvey_triggered(self):
        CDispObservationSurvey(self).exec_()


    @pyqtSignature('')
    def on_actStatReportF12Clients_triggered(self):
        CStatReportF12Clients(self).exec_()


    @pyqtSignature('')
    def on_actTempInvalidList_triggered(self):
        CTempInvalidList(self).exec_()


    @pyqtSignature('')
    def on_actTempInvalidSurvey_triggered(self):
        CTempInvalidSurvey(self).exec_()


    @pyqtSignature('')
    def on_actTempInvalidF16_triggered(self):
        CTempInvalidF16(self).exec_()


    @pyqtSignature('')
    def on_actTempInvalidBookF036_triggered(self):
        CTempInvalidBookF036(self).exec_()


    @pyqtSignature('')
    def on_actTempInvalidExpert_triggered(self):
        CTempInvalidExpert(self).exec_()


    @pyqtSignature('')
    def on_actDeathList_triggered(self):
        CDeathList(self).exec_()


    @pyqtSignature('')
    def on_actDeathReport_triggered(self):
        CDeathReport(self).exec_()


    @pyqtSignature('')
    def on_actDeathReportByZones_triggered(self):
        CDeathReportByZones(self).exec_()


    @pyqtSignature('')
    def on_actDetailedDeathReportByZones_triggered(self):
        CDetailedDeathReportByZones(self).exec_()


    @pyqtSignature('')
    def on_actDeathSurvey_triggered(self):
        CDeathSurvey(self).exec_()


    @pyqtSignature('')
    def on_actWorkload_triggered(self):
        CWorkload(self).exec_()


    @pyqtSignature('')
    def on_actAttachedContingent_triggered(self):
        CAttachedContingent(self).exec_()


    @pyqtSignature('')
    def on_actBySMOContingent_triggered(self):
        CBySMOContingent(self).exec_()


    @pyqtSignature('')
    def on_actAttachingMotion_triggered(self):
        CReportAttachingMotion(self).exec_()


    @pyqtSignature('')
    def on_actReportSMOClients_triggered(self):
        CReportSMOClients(self).exec_()


    @pyqtSignature('')
    def on_actForma007Moving_triggered(self):
        CStationaryF007Moving(self).exec_()


    @pyqtSignature('')
    def on_actMovingAndBedsReport_triggered(self):
        CReportMovingAndBeds(self).exec_()


    @pyqtSignature('')
    def on_actStationaryReportForMIAC_triggered(self):
        CStationaryReportForMIAC(self).exec_()


    @pyqtSignature('')
    def on_actStationaryReportForMIACInHard_triggered(self):
        CStationaryReportForMIACHard(self).exec_()


    @pyqtSignature('')
    def on_actReportsBeingInStationary_triggered(self):
        CReportsBeingInStationary(self).exec_()


    @pyqtSignature('')
    def on_actReportLengthOfStayInHospital_triggered(self):
        CReportLengthOfStayInHospital(self).exec_()


    @pyqtSignature('')
    def on_actDeliveryChannelsReport_triggered(self):
        CDeliveryChannelsReport(self).exec_()


    @pyqtSignature('')
    def on_actDiseasesAnalysisReport_triggered(self):
        CDiseasesAnalysisReport(self).exec_()


    @pyqtSignature('')
    def on_actLengthOfStayReport_triggered(self):
        CLengthOfStayReport(self).exec_()


    @pyqtSignature('')
    def on_actForma007ClientList_triggered(self):
        CStationaryF007ClientList(self).exec_()


    @pyqtSignature('')
    def on_actForma007DSMoving_triggered(self):
        CStationaryF007DSMoving(self).exec_()


    @pyqtSignature('')
    def on_actForma007DSClientList_triggered(self):
        CStationaryF007DSClientList(self).exec_()


    @pyqtSignature('')
    def on_actFormaF016_02_triggered(self):
        CStationaryF016_02(self).exec_()


    @pyqtSignature('')
    def on_actFormaF016_triggered(self):
        CStationaryF016(self).exec_()


    @pyqtSignature('')
    def on_actFormaF016ForPeriod_triggered(self):
        CStationaryF016ForPeriod(self).exec_()


    @pyqtSignature('')
    def on_actStationaryF30Moving_triggered(self):
        CStationaryF30Moving(self).exec_()


    @pyqtSignature('')
    def on_actStationaryF30_3101_triggered(self):
        CStationaryF30_3101(self).exec_()


    @pyqtSignature('')
    def on_actFormaAdultF142000_triggered(self):
        CStationaryAdultF142000(self).exec_()


    @pyqtSignature('')
    def on_actFormaAdultNoSeniorF142000_triggered(self):
        CStationaryAdultNoSeniorF142000(self).exec_()


    @pyqtSignature('')
    def on_actFormaChildrenF142000_triggered(self):
        CStationaryChildrenF142000(self).exec_()


    @pyqtSignature('')
    def on_actFormaSeniorF142000_triggered(self):
        CStationarySeniorF142000(self).exec_()


    @pyqtSignature('')
    def on_actStationaryF142100_triggered(self):
        CStationaryF142100(self).exec_()


    @pyqtSignature('')
    def on_actStationaryF14_4300_4301_4302_triggered(self):
        CStationaryF14_4300_4301_4302(self).exec_()


    @pyqtSignature('')
    def on_actFormaF0143000_triggered(self):
        CStationaryF143000(self).exec_()


    @pyqtSignature('')
    def on_actFormaF0144000_triggered(self):
        CStationaryF144000(self).exec_()


    @pyqtSignature('')
    def on_actFormaF0144001_triggered(self):
        CStationaryF144001(self).exec_()


    @pyqtSignature('')
    def on_actFormaF0144001A_triggered(self):
        CStationaryF144001A(self).exec_()


    @pyqtSignature('')
    def on_actFormaF0144002_triggered(self):
        CStationaryF144002(self).exec_()


    @pyqtSignature('')
    def on_actFormaF0144100_triggered(self):
        CStationaryF144100(self).exec_()


    @pyqtSignature('')
    def on_actFormaF0144200_triggered(self):
        CStationaryF144200(self).exec_()


    @pyqtSignature('')
    def on_actFormaF0144201_triggered(self):
        CStationaryF144201(self).exec_()


    @pyqtSignature('')
    def on_actFormaF0144202_triggered(self):
        CStationaryF144202(self).exec_()


    @pyqtSignature('')
    def on_actFormaF0144400_triggered(self):
        CStationaryF144400(self).exec_()


    @pyqtSignature('')
    def on_actOne2015Forma14DC_triggered(self):
        CStationaryOne_2015F14DC(self).exec_()


    @pyqtSignature('')
    def on_actTwoAdult_2015Forma14DC_triggered(self):
        CStationaryTwoAdult_2015F14DC(self).exec_()


    @pyqtSignature('')
    def on_actTwoChildren_2015Forma14DC_triggered(self):
        CStationaryTwoChildren_2015F14DC(self).exec_()


    @pyqtSignature('')
    def on_actOneForma14DC_triggered(self):
        CStationaryOneF14DC(self).exec_()


    @pyqtSignature('')
    def on_actOneHospitalF14DC_triggered(self):
        CStationaryOneHospitalF14DC(self).exec_()


    @pyqtSignature('')
    def on_actMESHospitalF14DC_triggered(self):
        CStationaryMESHospitalF14DC(self).exec_()


    @pyqtSignature('')
    def on_actMESHouseF14DC_triggered(self):
        CStationaryMESHouseF14DC(self).exec_()


    @pyqtSignature('')
    def on_actMESPolyclinicF14DC_triggered(self):
        CStationaryMESPolyclinicF14DC(self).exec_()


    @pyqtSignature('')
    def on_actMESOneF14DC_triggered(self):
        CStationaryMESOneF14DC(self).exec_()


    @pyqtSignature('')
    def on_actOnePolyclinicF14DC_triggered(self):
        CStationaryOnePolyclinicF14DC(self).exec_()


    @pyqtSignature('')
    def on_actOneHouseF14DC_triggered(self):
        CStationaryOneHouseF14DC(self).exec_()


    @pyqtSignature('')
    def on_actEmergency2000_triggered(self):
        CReportEmergencyF402000(self).exec_()


    @pyqtSignature('')
    def on_actEmergency2120_triggered(self):
        CReportEmergencyF302120(self).exec_()


    @pyqtSignature('')
    def on_actEmergency2001_triggered(self):
        CReportEmergencyF402001(self).exec_()


    @pyqtSignature('')
    def on_actEmergency2350_triggered(self):
        CReportEmergencyF302350(self).exec_()


    @pyqtSignature('')
    def on_actEmergency2100_triggered(self):
        CReportEmergencyF402100(self).exec_()


    @pyqtSignature('')
    def on_actEmergency2500_triggered(self):
        CReportEmergencyF402500(self).exec_()


    @pyqtSignature('')
    def on_actEmergencyF40TimeIndicators_triggered(self):
        CReportEmergencyF40TimeIndicators(self).exec_()


    @pyqtSignature('')
    def on_actEmergencyAdditionally_triggered(self):
        CReportEmergencyAdditionally(self).exec_()


    @pyqtSignature('')
    def on_actEmergencyTalonSignal_triggered(self):
        CReportEmergencyTalonSignal(self).exec_()


    @pyqtSignature('')
    def on_actEmergencySize_triggered(self):
        CEmergencySizeReport(self).exec_()


    @pyqtSignature('')
    def on_actReportEmergencyCallList_triggered(self):
        CReportEmergencyCallList(self).exec_()


    @pyqtSignature('')
    def on_actTwoAdultForma14DC_triggered(self):
        CStationaryTwoAdultF14DC(self).exec_()


    @pyqtSignature('')
    def on_actTwoAdultHospitalF14DC_triggered(self):
        CStationaryTwoAdultHospitalF14DC(self).exec_()


    @pyqtSignature('')
    def on_actTwoAdultPoliclinicF14DC_triggered(self):
        CStationaryTwoAdultPoliclinicF14DC(self).exec_()


    @pyqtSignature('')
    def on_actTwoAdultHouseF14DC_triggered(self):
        CStationaryTwoAdultHouseF14DC(self).exec_()


    @pyqtSignature('')
    def on_actTwoChildrenForma14DC_triggered(self):
        CStationaryTwoChildrenF14DC(self).exec_()


    @pyqtSignature('')
    def on_actTwoChildrenHospitalF14DC_triggered(self):
        CStationaryTwoChildrenHospitalF14DC(self).exec_()


    @pyqtSignature('')
    def on_actTwoChildrenPoliclinicF14DC_triggered(self):
        CStationaryTwoChildrenPoliclinicF14DC(self).exec_()


    @pyqtSignature('')
    def on_actTwoChildrenHouseF14DC_triggered(self):
        CStationaryTwoChildrenHouseF14DC(self).exec_()


    @pyqtSignature('')
    def on_actStationaryTypePaymentF14DC_triggered(self):
        CStationaryTypePaymentF14DC(self).exec_()


    @pyqtSignature('')
    def on_actPopulationStructure_triggered(self):
        CPopulationStructure(self).exec_()


    @pyqtSignature('')
    def on_actSpendingToClients_triggered(self):
        CSpendingToClients(self).exec_()


    @pyqtSignature('')
    def on_actReportClientNomenclaturePlan_triggered(self):
        from Reports.ReportClientNomenclaturePlan import CReportClientNomenclaturePlan
        CReportClientNomenclaturePlan(self).exec_()


    @pyqtSignature('')
    def on_actGenRep_triggered(self):
        genRep = forceString(QtGui.qApp.preferences.appPrefs['extGenRep'])
        if genRep:
            prg=QtGui.qApp.execProgram(u'"%s"' % genRep)
            if not prg[0]:
                QtGui.QMessageBox.critical(self,
                                       u'Внимание!',
                                       u'Не удалось запустить "%s"' % genRep,
                                       QtGui.QMessageBox.Close)
            if prg[2]:
                QtGui.QMessageBox.critical(self,
                                       u'Внимание!',
                                       u'Ошибка при выполнении "%s"' % genRep,
                                       QtGui.QMessageBox.Close)
        else:
            QtGui.QMessageBox.critical(self,
                                    u'Ошибка!',
                                    u'Не указан исполнимый файл генератора отчетов\n'+
                                    u'Смотрите пункт меню "Настройки.Умолчания", закладка "Прочие настройки",\n'+
                                    u'строка "Внешний генератор отчетов".',
                                    QtGui.QMessageBox.Close)


    @pyqtSignature('')
    def on_actAreas_triggered(self):
        pass


    @pyqtSignature('')
    def on_actKLADR_triggered(self):
        pass


    @pyqtSignature('')
    def on_actMKB_triggered(self):
        CMKBList(self).exec_()

    @pyqtSignature('')
    def on_actMKBSubclass_triggered(self):
        CMKBSubclass(self).exec_()


    @pyqtSignature('')
    def on_actMKBMorphology_triggered(self):
        CRBMKBMorphologyList(self).exec_()


    @pyqtSignature('')
    def on_actBank_triggered(self):
        CBanksList(self).exec_()


    @pyqtSignature('')
    def on_actContract_triggered(self):
        CContractsList(self).exec_()


    @pyqtSignature('')
    def on_actOrganisation_triggered(self):
        COrgsList(self).exec_()


    @pyqtSignature('')
    def on_actActionPropertyTemplate_triggered(self):
        CActionPropertyTemplateList(self).exec_()


    @pyqtSignature('')
    def on_actRBActionShedule_triggered(self):
        CRBActionSheduleList(self).exec_()


    @pyqtSignature('')
    def on_actActionType_triggered(self):
        CActionTypeList(self).exec_()


    @pyqtSignature('')
    def on_actQuotaType_triggered(self):
        CQuotaTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBAgreementType_triggered(self):
        CRBAgreementTypeList(self).exec_()


    @pyqtSignature('')
    def on_actBlankType_triggered(self):
        CBlanksList(self).exec_()


    @pyqtSignature('')
    def on_actRBMedicalBoardExpertiseCharacter_triggered(self):
        CRBMedicalBoardExpertiseCharacter(self).exec_()


    @pyqtSignature('')
    def on_actRBMedicalBoardExpertiseKind_triggered(self):
        CRBMedicalBoardExpertiseKind(self).exec_()


    @pyqtSignature('')
    def on_actRBMedicalBoardExpertiseObject_triggered(self):
        CRBMedicalBoardExpertiseObject(self).exec_()


    @pyqtSignature('')
    def on_actRBMedicalBoardExpertiseArgument_triggered(self):
        CRBMedicalBoardExpertiseArgument(self).exec_()


    @pyqtSignature('')
    def on_actCureMethod_triggered(self):
        CRBCureMethod(self).exec_()


    @pyqtSignature('')
    def on_actCureType_triggered(self):
        CRBCureType(self).exec_()


    @pyqtSignature('')
    def on_actPatientModel_triggered(self):
        CRBPatientModel(self).exec_()


    @pyqtSignature('')
    def on_actActionTemplate_triggered(self):
        CActionTemplateList(self).exec_()


    @pyqtSignature('')
    def on_actEventType_triggered(self):
        CEventTypeList(self).exec_()


    @pyqtSignature('')
    def on_actPersonal_triggered(self):
        CPersonList(self).exec_()


    @pyqtSignature('')
    def on_actPlanningHospitalActivity_triggered(self):
        CRBPlanningHospitalActivityList(self).exec_()


    @pyqtSignature('')
    def on_actOrgStructure_triggered(self):
        COrgStructureList(self).exec_()


    @pyqtSignature('')
    def on_actRBAccountExportFormat_triggered(self):
        CRBAccountExportFormatList(self).exec_()


    @pyqtSignature('')
    def on_actRBAccountingSystem_triggered(self):
        CRBAccountingSystemList(self).exec_()


    @pyqtSignature('')
    def on_actRBCounter_triggered(self):
        CRBCounterList(self).exec_()

    @pyqtSignature('')
    def on_actRBAttachType_triggered(self):
        CRBAttachTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBContactType_triggered(self):
        CRBContactTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBContractAttributeType_triggered(self):
        CRBContractAttributeTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBContractCoefficientType_triggered(self):
        CRBContractCoefficientTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBDiagnosisType_triggered(self):
        CRBDiagnosisTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBDiseaseCharacter_triggered(self):
        CRBDiseaseCharacterList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyBrigade_triggered(self):
        CRBEmergencyBrigadeList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyCauseCall_triggered(self):
        CRBEmergencyCauseCallList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyTransferredTransportation_triggered(self):
        CRBEmergencyTransferredTransportationList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyPlaceReceptionCall_triggered(self):
        CRBEmergencyPlaceReceptionCallList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyReceivedCall_triggered(self):
        CRBEmergencyReceivedCallList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyReasondDelays_triggered(self):
        CRBEmergencyReasondDelaysList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyResult_triggered(self):
        CRBEmergencyResultList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyAccident_triggered(self):
        CRBEmergencyAccidentList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyDeath_triggered(self):
        CRBEmergencyDeathList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyEbriety_triggered(self):
        CRBEmergencyEbrietyList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyDiseased_triggered(self):
        CRBEmergencyDiseasedList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyPlaceCall_triggered(self):
        CRBEmergencyPlaceCallList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyMethodTransportation_triggered(self):
        CRBEmergencyMethodTransportationList(self).exec_()


    @pyqtSignature('')
    def on_actRBEmergencyTypeAsset_triggered(self):
        CRBEmergencyTypeAssetList(self).exec_()


    @pyqtSignature('')
    def on_actRBMealTime_triggered(self):
        CRBMealTime(self).exec_()


    @pyqtSignature('')
    def on_actRBDiet_triggered(self):
        CRBDiet(self).exec_()


    @pyqtSignature('')
    def on_actRBMenu_triggered(self):
        CRBMenu(self).exec_()


    @pyqtSignature('')
    def on_actRecativeClient_triggered(self):
        CRBRelativeList(self).exec_()


    @pyqtSignature('')
    def on_actRBDiseaseStage_triggered(self):
        CRBDiseaseStageList(self).exec_()


    @pyqtSignature('')
    def on_actRBDiseasePhases_triggered(self):
        CRBDiseasePhasesList(self).exec_()


    @pyqtSignature('')
    def on_actRBDispanser_triggered(self):
        CRBDispanserList(self).exec_()


    @pyqtSignature('')
    def on_actRBDocumentType_triggered(self):
        CRBDocumentTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBDocumentTypeGroup_triggered(self):
        CRBDocumentTypeGroupList(self).exec_()


    @pyqtSignature('')
    def on_actRBEventTypePurpose_triggered(self):
        CRBEventTypePurposeList(self).exec_()


    @pyqtSignature('')
    def on_actRBEventProfile_triggered(self):
        CRBEventProfileList(self).exec_()


    @pyqtSignature('')
    def on_actRBFinance_triggered(self):
        CRBFinanceList(self).exec_()


    @pyqtSignature('')
    def on_actRBExpenseServiceItem_triggered(self):
        CRBExpenseServiceItemList(self).exec_()


    @pyqtSignature('')
    def on_actRBHealthGroup_triggered(self):
        CRBHealthGroupList(self).exec_()


    @pyqtSignature('')
    def on_actRBHurtFactorType_triggered(self):
        CRBHurtFactorTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBHurtType_triggered(self):
        CRBHurtTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBMedicalAidKind_triggered(self):
        CRBMedicalAidKindList(self).exec_()


    @pyqtSignature('')
    def on_actRBMedicalAidType_triggered(self):
        CRBMedicalAidTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBMedicalAidProfile_triggered(self):
        CRBMedicalAidProfileList(self).exec_()


    @pyqtSignature('')
    def on_actRBMesSpecification_triggered(self):
        CRBMesSpecificationList(self).exec_()


    @pyqtSignature('')
    def on_actRBMedicalAidUnit_triggered(self):
        CRBMedicalAidUnitList(self).exec_()


    @pyqtSignature('')
    def on_actRBNet_triggered(self):
        CRBNetList(self).exec_()


    @pyqtSignature('')
    def on_actRBNomenclatureClass_triggered(self):
        CRBNomenclatureClassList(self).exec_()


    @pyqtSignature('')
    def on_actRBNomenclatureKind_triggered(self):
        CRBNomenclatureKindList(self).exec_()


    @pyqtSignature('')
    def on_actRBNomenclatureType_triggered(self):
        CRBNomenclatureTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBLfForm_triggered(self):
        CRBLfFormList(self).exec_()


    @pyqtSignature('')
    def on_actRBIngredient_triggered(self):
        CRBIngredientList(self).exec_()


    @pyqtSignature('')
    def on_actRBNomenclature_triggered(self):
        CRBNomenclatureList(self).exec_()


    @pyqtSignature('')
    def on_actRBStockRecipe_triggered(self):
        CRBStockRecipeList(self).exec_()

    @pyqtSignature('')
    def on_actRBStockMotionItemUtilizationReason_triggered(self):
        from RefBooks.RBStockMotionItemReason import getStockMotionItemReasonDialog
        from Stock.StockModel import CStockMotionType

        getStockMotionItemReasonDialog(CStockMotionType.utilization, self).exec_()


    @pyqtSignature('')
    def on_actRBStockMotionNumber_triggered(self):
        CRBStockMotionNumberList(self).exec_()


    @pyqtSignature('')
    def on_actRBTissueType_triggered(self):
        CRBTissueTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBOKFS_triggered(self):
        CRBOKFSList(self).exec_()


    @pyqtSignature('')
    def on_actRBOKPF_triggered(self):
        CRBOKPFList(self).exec_()


    @pyqtSignature('')
    def on_actRBPayRefuseType_triggered(self):
        CRBPayRefuseTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBCashOperation_triggered(self):
        CRBCashOperationList(self).exec_()


    @pyqtSignature('')
    def on_actRBPolicyKind_triggered(self):
        CRBPolicyKindList(self).exec_()


    @pyqtSignature('')
    def on_actRBPolicyType_triggered(self):
        CRBPolicyTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBPost_triggered(self):
        CRBPostList(self).exec_()


    @pyqtSignature('')
    def on_actRBActivity_triggered(self):
        CRBActivityList(self).exec_()


    @pyqtSignature('')
    def on_actRBAppointmentPurpose_triggered(self):
        CRBAppointmentPurposeList(self).exec_()


    @pyqtSignature('')
    def on_actRBPrintTemplate_triggered(self):
        CRBPrintTemplate(self).exec_()


    @pyqtSignature('')
    def on_actRBReasonOfAbsence_triggered(self):
        CRBReasonOfAbsenceList(self).exec_()


    @pyqtSignature('')
    def on_actRBHospitalBedProfile_triggered(self):
        CRBHospitalBedProfileList(self).exec_()


    @pyqtSignature('')
    def on_actRBJobType_triggered(self):
        CRBJobTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBJobPurpose_triggered(self):
        CRBJobPurposeList(self).exec_()


    @pyqtSignature('')
    def on_actRBResult_triggered(self):
        CRBResultList(self).exec_()


    @pyqtSignature('')
    def on_actRBDiagnosticResult_triggered(self):
        CRBDiagnosticResultList(self).exec_()


    @pyqtSignature('')
    def on_actRBScene_triggered(self):
        CRBSceneList(self).exec_()


    @pyqtSignature('')
    def on_actRBService_triggered(self):
        CRBServiceList(self).exec_()


    @pyqtSignature('')
    def on_actRBServiceGroup_triggered(self):
        CRBServiceGroupList(self).exec_()


    @pyqtSignature('')
    def on_actRBServiceType_triggered(self):
        CRBServiceTypeList(self).exec_()


    @pyqtSignature('')
    def on_actMilitaryUnits_triggered(self):
        CKLADRMilitaryUnitsList(self).exec_()


    @pyqtSignature('')
    def on_actRBSocStatusClass_triggered(self):
        CRBSocStatusClassList(self).exec_()


    @pyqtSignature('')
    def on_actRBSocStatusType_triggered(self):
        CRBSocStatusTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBSpeciality_triggered(self):
        CRBSpecialityList(self).exec_()


    @pyqtSignature('')
    def on_actRBTariffCategory_triggered(self):
        CRBTariffCategoryList(self).exec_()


    @pyqtSignature('')
    def on_actRBTempInvalidDocument_triggered(self):
        CRBTempInvalidDocumentList(self).exec_()


    @pyqtSignature('')
    def on_actRBTempInvalidBreak_triggered(self):
        CRBTempInvalidBreakList(self).exec_()


    @pyqtSignature('')
    def on_actRBBloodType_triggered(self):
        CRBBloodTypeList(self).exec_()

    @pyqtSignature('')
    def on_actRBImageMap_triggered(self):
        CRBImageMapList(self).exec_()


    @pyqtSignature('')
    def on_actRBTempInvalidReason_triggered(self):
        CRBTempInvalidReasonList(self).exec_()


    @pyqtSignature('')
    def on_actRBTempInvalidExtraReason_triggered(self):
        CRBTempInvalidExtraReasonList(self).exec_()


    @pyqtSignature('')
    def on_actRBTempInvalidRegime_triggered(self):
        CRBTempInvalidRegimeList(self).exec_()


    @pyqtSignature('')
    def on_actRBTempInvalidResult_triggered(self):
        CRBTempInvalidResultList(self).exec_()


    @pyqtSignature('')
    def on_actRBTempInvalidDuplicateReason_triggered(self):
        CRBTempInvalidDuplicateReasonList(self).exec_()


    @pyqtSignature('')
    def on_actRBComplain_triggered(self):
        CRBComplainList(self).exec_()


    @pyqtSignature('')
    def on_actRBThesaurus_triggered(self):
        CRBThesaurus(self).exec_()


    @pyqtSignature('')
    def on_actRBTraumaType_triggered(self):
        CRBTraumaTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBUnit_triggered(self):
        CRBUnitList(self).exec_()


    @pyqtSignature('')
    def on_actRBVisitType_triggered(self):
        CRBVisitTypeList(self).exec_()

    @pyqtSignature('')
    def on_actRBActionTypeAppointment_triggered(self):
        CRBActionTypeGroupAppointment(self).exec_()

    @pyqtSignature('')
    def on_actRBInfection_triggered(self):
        CRBInfectionList(self).exec_()


    @pyqtSignature('')
    def on_actRBVaccine_triggered(self):
        CRBVaccineList(self).exec_()


    @pyqtSignature('')
    def on_actRBVaccineSchemaTransitionType_triggered(self):
        CRBVaccineSchemaTransitionTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBVaccinationCalendar_triggered(self):
        CRBVaccinationCalendarList(self).exec_()


    @pyqtSignature('')
    def on_actRBVaccinationProbe_triggered(self):
        CRBVaccinationProbeList(self).exec_()


    @pyqtSignature('')
    def on_actRBReaction_triggered(self):
        CRBReactionList(self).exec_()


    @pyqtSignature('')
    def on_actRBMedicalExemptionType_triggered(self):
        CRBMedicalExemptionTypeList(self).exec_()



    @pyqtSignature('')
    def on_actRBTest_triggered(self):
        CRBTestList(self).exec_()


    @pyqtSignature('')
    def on_actRBTestGroup_triggered(self):
        CRBTestGroupList(self).exec_()


    @pyqtSignature('')
    def on_actRBLaboratory_triggered(self):
        CRBLaboratoryList(self).exec_()


    @pyqtSignature('')
    def on_actRBSuiteReagent_triggered(self):
        CRBSuiteReagentList(self).exec_()


    @pyqtSignature('')
    def on_actRBSpecimenType_triggered(self):
        CRBSpecimenTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBEquipmentType_triggered(self):
        CRBEquipmentTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBEquipment_triggered(self):
        dlg = CRBEquipmentList(self)
        dlg.setEditableJournal(QtGui.qApp.userHasAnyRight([urAccessEquipmentMaintenanceJournal, urAccessRefBooks]))
        dlg.exec_()


    @pyqtSignature('')
    def on_actRBContainerType_triggered(self):
        CRBContainerTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBLocationCardType_triggered(self):
        CRBLocationCardTypeList(self).exec_()

    @pyqtSignature('')
    def on_actRBDocumentTypeForTracking_triggered(self):
        CRBDocumentTypeForTrackingList(self).exec_()


    @pyqtSignature('')
    def on_actRBStatusObservationClientType_triggered(self):
        CRBStatusObservationClientTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBClientConsentType_triggered(self):
        CRBClientConsentTypeList(self).exec_()


    @pyqtSignature('')
    def on_actRBContingentType_triggered(self):
        from RefBooks.RBContingentType import CRBContingentTypeList
        CRBContingentTypeList(self).exec_()


    @pyqtSignature('')
    def on_actEventsCheck_triggered(self):
        CEventsCheck(self).exec_()


    @pyqtSignature('')
    def on_actControlDiagnosis_triggered(self):
        CControlDiagnosis(self).exec_()


    @pyqtSignature('')
    def on_actControlDoubles_triggered(self):
        CControlDoubles(self).exec_()


    @pyqtSignature('')
    def on_actControlMes_triggered(self):
        CLogicalControlMes(self).exec_()


    @pyqtSignature('')
    def on_actClientsCheck_triggered(self):
        ClientsCheck(self).exec_()


    @pyqtSignature('')
    def on_actTempInvalidCheck_triggered(self):
        TempInvalidCheck(self).exec_()


    @pyqtSignature('')
    def on_actCreateAttachClientsForArea_triggered(self):
        dlg = CCreateAttachClientsForAreaDialog(self)
        dlg.exec_()
        dlg.stop()


    @pyqtSignature('')
    def on_actDbfView_triggered(self):
        viewDbf(self)


    @pyqtSignature('')
    def on_actTestSendMail_triggered(self):
        sendMail(self, '', u'проверка связи', u'проверка связи\n1\n2\n3', [])


    @pyqtSignature('')
    def on_actTestMKB_triggered(self):
        from library.ICDTreeTest import testICDTree
        testICDTree()


    @pyqtSignature('')
    def on_actAverageDurationAcuteDisease_triggered(self):
        CAverageDurationAcuteDiseaseBase(self).exec_()


    @pyqtSignature('')
    def on_actTestRLS_triggered(self):
        from library.RLS.RLSComboBoxTest import testRLSComboBox
        testRLSComboBox()


    @pyqtSignature('')
    def on_actTestMES_triggered(self):
        from library.MES.MESComboBoxTest import testMESComboBox
        testMESComboBox()


    @pyqtSignature('')
    def on_actTestCSG_triggered(self):
        from library.CSG.CSGComboBoxTest import testCSGComboBox
        testCSGComboBox()


    @pyqtSignature('')
    def on_actTestTNMS_triggered(self):
        from library.TNMS.TNMSComboBoxTest import testTNMSComboBox
        testTNMSComboBox()


    @pyqtSignature('')
    def on_actNotificationLog_triggered(self):
        from Notifications.NotificationLog import CNotificationLogList
        CNotificationLogList(self).exec_()


    @pyqtSignature('')
    def on_actNotificationRules_triggered(self):
        from Notifications.NotificationRule import CNotificationRuleList
        CNotificationRuleList(self).exec_()


    @pyqtSignature('')
    def on_actRBNotificationKind_triggered(self):
        from RefBooks.RBNotificationKind import CRBNotificationKindList
        CRBNotificationKindList(self).exec_()


    @pyqtSignature('')
    def on_actExportLocalLabResultsToUSISH_triggered(self):
        from TissueJournal import ExportLocalLabResultsToUsish
        try:
            ExportLocalLabResultsToUsish.exportLocalLabResultsToUsish()
        except:
            QtGui.qApp.logCurrentException()


    @pyqtSignature('')
    def on_actAppPreferences_triggered(self):
        qApp = QtGui.qApp
        try:
            qApp.closeScanner()
            qApp.closeSmartCardReader()
            dialog = CAppPreferencesDialog(self)
            dialog.setTabGlobalEnabled(qApp.userHasAnyRight([urAdmin,
                                                             urAccessSetupGlobalPreferencesWatching,
                                                             urAccessSetupGlobalPreferencesEdit]))
            dialog.setProps(qApp.preferences.appPrefs)
#            globalPreferencesId = forceRef(QtGui.qApp.db.translate('GlobalPreferences', 'code', '5', 'id'))
#            if globalPreferencesId:
#                row = dialog.tblGlobal.model()._idList.index(globalPreferencesId)
#                dialog.tblGlobal.model()._idList.pop(row)
            if dialog.exec_():
                prevOrgId = qApp.currentOrgId()
                prevOrgStructureId = qApp.currentOrgStructureId()
                qApp.preferences.appPrefs.update(dialog.props())
                qApp.preferences.save()
                orgId = qApp.currentOrgId()
                if not orgId:
                    self.closeRegistryWindow()
                    self.closeSuspendedAppointmentWindow()
                self.setUserName(qApp.userName())
                self.updateActionsState()
                if orgId != prevOrgId:
                    qApp.emit(SIGNAL('currentOrgIdChanged()'))
                if QtGui.qApp.currentOrgStructureId() != prevOrgStructureId:
                    qApp.emit(SIGNAL('currentOrgStructureIdChanged()'))
                qApp.clearPreferencesCache()
                qApp.webDAVInterface.setWebDAVUrl(qApp.getWebDAVUrl())
        finally:
            qApp.tryOpenScanner()
            qApp.tryOpenSmartCardReader()


    @pyqtSignature('')
    def on_actConnection_triggered(self):
        dlg   = CConnectionDialog(self)
        preferences = QtGui.qApp.preferences
        dlg.setDriverName(preferences.dbDriverName)
        dlg.setServerName(preferences.dbServerName)
        dlg.setServerPort(preferences.dbServerPort)
        dlg.setDatabaseName(preferences.dbDatabaseName)
        dlg.setCompressData(preferences.dbCompressData)
        dlg.setUserName(preferences.dbUserName)
        dlg.setPassword(preferences.dbPassword)
        if dlg.exec_():
            preferences.dbDriverName = dlg.driverName()
            preferences.dbServerName = dlg.serverName()
            preferences.dbServerPort = dlg.serverPort()
            preferences.dbDatabaseName = dlg.databaseName()
            preferences.dbCompressData = dlg.compressData()
            preferences.dbUserName = dlg.userName()
            preferences.dbPassword = dlg.password()
            preferences.save()


    @pyqtSignature('')
    def on_actDecor_triggered(self):
        dlg = CDecorDialog(self)
        preferences = QtGui.qApp.preferences
        dlg.setStyle(preferences.decorStyle)
        dlg.setStandardPalette(preferences.decorStandardPalette)
        dlg.setMaximizeMainWindow(preferences.decorMaximizeMainWindow)
        dlg.setFullScreenMainWindow(preferences.decorFullScreenMainWindow)
        dlg.setUseCustomFont(preferences.useCustomFont)
        dlg.setFont(preferences.font)
        if dlg.exec_():
            preferences.decorStyle = dlg.style()
            preferences.decorStandardPalette = dlg.standardPalette()
            preferences.decorMaximizeMainWindow = dlg.maximizeMainWindow()
            preferences.decorFullScreenMainWindow = dlg.fullScreenMainWindow()
            preferences.useCustomFont = dlg.useCustomFont()
            preferences.font = dlg.font()
            preferences.save()
            QtGui.qApp.applyDecorPreferences()


    @pyqtSignature('')
    def on_actCalendar_triggered(self):
        CCalendarExceptionList(self).exec_()
        calendarInfo = QtGui.qApp.calendarInfo
        calendarInfo.clear()
        calendarInfo.load()


    @pyqtSignature('')
    def on_actInformerMessages_triggered(self):
        CInformerList(self).exec_()


    @pyqtSignature('')
    def on_actInformer_triggered(self):
        showInformer(self, False)


    @pyqtSignature('')
    def on_actUserRightListDialog_triggered(self):
        CUserRightListDialog(self).exec_()


    @pyqtSignature('')
    def on_actUserRightProfileListDialog_triggered(self):
        CUserRightProfileListDialog(self).exec_()


    @pyqtSignature('')
    def on_actAbout_triggered(self):
        QtGui.QMessageBox.about(
            self, u'О программе', gAbout % ( gTitle, gVersion, gLastChangedRev, gLastChangedDate)
            )


    @pyqtSignature('')
    def on_actAboutQt_triggered(self):
        QtGui.QMessageBox.aboutQt(self, u'О Qt')


    @pyqtSignature('QWidget*')
    def on_centralWidget_mdiSubwindowClose(self, widget):
        if widget == self.registry:
            self.registry = None
        elif widget == self.suspendedAppointment:
            self.suspendedAppointment = None



def parseGCDebug(val):
    result = 0
    for char in val.upper():
        if char == 'S':
            result |= gc.DEBUG_STATS
        elif char == 'C':
            result |= gc.DEBUG_COLLECTABLE
        elif char == 'U':
            result |= gc.DEBUG_UNCOLLECTABLE
        elif char == 'I':
            result |= gc.DEBUG_INSTANCES
        elif char == 'O':
            result |= gc.DEBUG_OBJECTS
    return result

def parseGCThreshold(val):
    result = []
    if val:
        for s in val.split(','):
            try:
                v = int(s)
            except:
                v = 0
            result.append(v)
        l = len(result)
        if l > 3:
            result = result[0:3]
    return result


def parseBgParams(image, size, position):
    bgParams = None
    if image:
        bgParams = {'image': os.path.expanduser(image)}
        if size:
            bgParams['size'] = size
        if position:
            bgParams['position'] = position
    return bgParams



def main():
##    gc.set_threshold(*[10*k for k in gc.get_threshold()])
    parser = OptionParser(usage = "usage: %prog [options]")
    parser.add_option('-c', '--config',
                      dest='iniFile',
                      help='custom .ini file name',
                      metavar='iniFile',
                      default='S11App.ini'
                     )
    parser.add_option('-r', '--record',
                      dest='recordEventLogFile',
                      help='file name for record events',
                      metavar='eventLogFile',
                      default=None,
                     )
    parser.add_option('-p', '--play',
                      dest='playEventLogFile',
                      help='file name for playback events',
                      metavar='eventLogFile',
                      default=None
                     )
    parser.add_option('-d', '--demo',
                      dest='demo',
                      help='Login as demo',
                      action='store_true',
                      default=False
                     )
    parser.add_option('-l', '--nolock',
                      dest='nolock',
                      help='Disable record lock',
                      action='store_true',
                      default=False
                     )
    parser.add_option('--gc-debug',
                      dest='gcDebug',
                      help='gc debug flags, chars S(stat), C(collectable), U(uncollectable), I(instances), O(objects), for details see gc.set_debug',
                      metavar='[S][C][U][I][O]',
                      default=''
                     )
    parser.add_option('--gc-threshold',
                      dest='gcThreshold',
                      help='gc thresholds list, see gc.set_threshold ',
                      metavar='threshold0[,threshold1[,threshold2]]',
                      default=''
                     )
    parser.add_option('--version',
                      dest='version',
                      help='print version and exit',
                      action='store_true',
                      default=False
                     )
    parser.add_option('--applog',
                      dest='appLogMark',
                      help='set main app log mark',
                      metavar='appLogMark',
                      default='')
    parser.add_option('--bgImage',
                      dest='bgImage',
                      help='set backgroung image',
                      metavar='path to image',
                      default=None)
    parser.add_option('--bgSize',
                      dest='bgSize',
                      help='set backgroung Size. x for percent scaling or x,y for scaling to x.y size or x,y,1 for keeping aspect ratio scale',
                      metavar = '200,200,1',
                      default=None)
    parser.add_option('--bgPos',
                      dest='bgPosition',
                      help='set backgroung position. center, topRight, bottomRight, bottomLeft',
                      metavar = 'center',
                      default=None)
#    parser.add_option("-q", "--quiet",   action="store_false", dest="verbose", default=True,  help="don't print status messages to stdout")
    (options, args) = parser.parse_args()
    parser.destroy()

    if options.version:
#        print '%s, v.2.5' % titleLat
        print getLatVersion()
        return

    gcDebug = parseGCDebug(options.gcDebug)
    gcThreshold = parseGCThreshold(options.gcThreshold)
    if gcDebug:
        gc.set_debug(gcDebug)
        print 'debug set to ', gcDebug
    if gcThreshold:
        gc.set_threshold(*gcThreshold)
        print 'threshold set to ', gcThreshold

    bgParams = parseBgParams(options.bgImage, options.bgSize, options.bgPosition)

    app = CS11mainApp(sys.argv, options.demo, options.iniFile, options.nolock, options.appLogMark)
    app.removeBigOrOldLogFile()
    if options.recordEventLogFile:
        app.eventLogFile = open(options.recordEventLogFile, 'w+', 0)
        app.eventLogTime = time.time()
    stdTtranslator = QTranslator()
    stdTtranslator.load(os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), 'i18n', 'std_ru.qm'))
    app.installTranslator(stdTtranslator)
    #Перевод стандартных диалогов Qt
    stdTtranslatorLib = QTranslator()
    stdTtranslatorLib.load(os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), 'i18n', 'qt_ru_lib.qm'))
    app.installTranslator(stdTtranslatorLib)

    QtGui.qApp = app

    app.applyDecorPreferences() # надеюсь, что это поможет немного сэкономить при создании гл.окна
    MainWindow = CS11MainWindow(bgParams)
    app.mainWindow = MainWindow
    app.applyDecorPreferences() # применение максимизации/полноэкранного режима к главному окну
    MainWindow.show()
    app.tryOpenScanner()
    app.tryOpenSmartCardReader()
    if app.preferences.dbAutoLogin:
        MainWindow.actLogin.activate(QtGui.QAction.Trigger)
#        from Accounting.InstantAccountDialog import createInstantAccount
#        createInstantAccount(2298206)
    res = app.exec_()
    app.preferences.save()
    if app.db:
        app.clearUserId(False)
        app.closeDatabase()
    app.doneTrace()
    QtGui.qApp = None
    return res


if __name__ == '__main__':
    locale.setlocale(locale.LC_ALL, '')
    sys.stdout = codecs.getwriter(locale.getpreferredencoding())(sys.stdout)
    sys.stderr = codecs.getwriter(locale.getpreferredencoding())(sys.stderr)
    main()
#    sys.exit(res)

